/**
 █▒▓▒░ The FlexPaper Project

 This file is part of FlexPaper.

 FlexPaper is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, version 3 of the License.

 FlexPaper is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with FlexPaper.  If not, see <http://www.gnu.org/licenses/>.

 For more information on FlexPaper please see the FlexPaper project
 home page: http://flexpaper.devaldi.com
 */
window.isTouchScreen = (function() {try {return 'ontouchstart' in document.documentElement;} catch (e) {return false;} })();
window.dispatchJQueryEvent = function (id, eventName, args) {jQuery("#" + id).trigger(eventName, args);};

function translateUrlByFormat(url, format) {
    if (url.indexOf("{") == 0 && format != "swf") { // loading in split file mode
        url = url.substring(1, url.lastIndexOf(","));
        url = url.replace("[*,0]", "{page}")
    }
    return (url != null && url.indexOf('{format}') > 0 ? url.replace("{format}", format) : null);
}

window.$FlexPaper = window.getDocViewer = window["$FlexPaper"] = function(id) {
    var ref = "FlexPaperViewer_Instance" + (id || "");
    var mode = window["ViewerMode"];
    if ("flash" == mode) {
        return window[ref].getApi();
    } else if ("html" == mode) {
        return window[ref];
    } else {
        throw "Unable to locate FlexPaper instance";
    }
};
/** HTML mode viewer */
window.FlexPaperViewerHtmlMode = function(root, config) {
    window['ViewerMode'] = 'html';

    var userAgent = navigator.userAgent.toLowerCase();
    var browser = {
        version: (userAgent.match(/.+(?:rv|it|ra|ie)[\/: ]([\d.]+)/) || [])[1],
        safari: /webkit/.test(userAgent),
        opera: /opera/.test(userAgent),
        msie: /msie/.test(userAgent) && !/opera/.test(userAgent),
        mozilla: /mozilla/.test(userAgent) && !/(compatible|webkit)/.test(userAgent),
        chrome: /chrome/.test(userAgent)
    };

    var platform = {
        win:/win/.test(userAgent),
        mac:/mac/.test(userAgent),
        ios : ((userAgent.match(/iphone/i)) || (userAgent.match(/ipod/i)) || (userAgent.match(/ipad/i)))
    };

    var viewerId = jQuery(root).attr('id');
    var supportsPDFJS = (browser.mozilla && browser.version.split(".")[0] >= 4)
            || (browser.chrome && browser.version.split(".") >= 535)
            || (browser.msie && browser.version.split(".")[0] >= 9)
            || platform.ios
            || (browser.safari && browser.version.split(".")[0] >= 534);

    var renderingOrder = config.RenderingOrder.split(",");
    var pageRenderer = null;

    if (config.PdfFile != null
            && config.PdfFile.length > 0
            && config.RenderingOrder.split(",").length >= 1
            && supportsPDFJS
            && (renderingOrder[0] == "html5"
            || (renderingOrder.length > 1 && renderingOrder[0] == "flash" && renderingOrder[1] == "html5"))
            && !(window.isTouchScreen && renderingOrder.length > 1 && renderingOrder[1] == "html")) {

        pageRenderer = new CanvasPageRenderer(viewerId, config.PdfFile, config.jsDirectory);
    } else {
        pageRenderer = new ImagePageRenderer(viewerId,
                                             {
                                                 jsonfile : config.JSONFile,
                                                 pageImagePattern : config.IMGFiles,
                                                 pageThumbImagePattern : config.ThumbIMGFiles,
                                                 compressedJSONFormat : !config.useCustomJSONFormat,
                                                 JSONPageDataFormat : config.JSONPageDataFormat,
                                                 JSONDataType : config.JSONDataType
                                             },
                                             config.jsDirectory);
    }

    var instance = "FlexPaperViewer_Instance" + ((viewerId === "undefined") ? "" : viewerId);
    window[instance] = new FlexPaperViewer_HTML({
        rootid: viewerId,
        Toolbar: ((config.Toolbar != null) ? config.Toolbar : null),
        BottomToolbar: ((config.BottomToolbar != null) ? config.BottomToolbar : null),
        instanceid: instance,

        document: {
            SWFFile: config.SwfFile,
            IMGFiles: config.IMGFiles,
            ThumbIMGFiles: config.ThumbIMGFiles,
            JSONFile: config.JSONFile,
            PDFFile: config.PdfFile,
            Scale: config.Scale,
            FitPageOnLoad: config.FitPageOnLoad,
            FitWidthOnLoad: config.FitWidthOnLoad,
            MinZoomSize: config.MinZoomSize,
            MaxZoomSize: config.MaxZoomSize,
            SearchMatchAll: config.SearchMatchAll,
            InitViewMode: config.InitViewMode,
            StartAtPage: config.StartAtPage,
            RenderingOrder: config.RenderingOrder,
            useCustomJSONFormat: config.useCustomJSONFormat,
            JSONPageDataFormat: config.JSONPageDataFormat,
            JSONDataType: config.JSONDataType,
            ZoomInterval: config.ZoomInterval,
            ViewModeToolsVisible: config.ViewModeToolsVisible,
            ZoomToolsVisible: config.ZoomToolsVisible,
            NavToolsVisible: config.NavToolsVisible,
            CursorToolsVisible: config.CursorToolsVisible,
            SearchToolsVisible: config.SearchToolsVisible,
            AnnotationToolsVisible: config.AnnotationToolsVisible,
            StickyTools: config.StickyTools,
            PrintPaperAsBitmap: config.PrintPaperAsBitmap,
            AutoAdjustPrintSize: config.AutoAdjustPrintSize,
            EnableCornerDragging: config.EnableCornerDragging,
            UIConfig: config.UIConfig,
            BackgroundColor: config.BackgroundColor, // FlexPaper Zine parameter
            PanelColor: config.PanelColor, // FlexPaper Zine parameter
            localeChain: config.localeChain
        },
        renderer: pageRenderer,
        key: config.key,
        jsDirectory: config.jsDirectory,
        localeDirectory: config.localeDirectory,
        cssDirectory: config.cssDirectory,
        docSizeQueryService: config.DocSizeQueryService
    });

    window[instance].initialize();
    window[instance].bindEvents();

    window[instance]['load'] = window[instance].loadFromUrl;
    window[instance]['loadDoc'] = window[instance].loadDoc;
    window[instance]['fitWidth'] = window[instance].fitwidth;
    window[instance]['fitHeight'] = window[instance].fitheight;
    window[instance]['gotoPage'] = window[instance].gotoPage;
    window[instance]['getCurrPage'] = window[instance].getCurrPage;
    window[instance]['getTotalPages'] = window[instance].getTotalPages;
    window[instance]['nextPage'] = window[instance].next;
    window[instance]['prevPage'] = window[instance].previous;
    window[instance]['setZoom'] = window[instance].Zoom;
    window[instance]['Zoom'] = window[instance].Zoom;
    window[instance]['openFullScreen'] = window[instance].openFullScreen;
    window[instance]['sliderChange'] = window[instance].sliderChange;
    window[instance]['searchText'] = window[instance].searchText;
    window[instance]['resize'] = window[instance].resize;
    window[instance]['rotate'] = window[instance].rotate;
    //window[instance]['nextSearchMatch'] = window[instance].nextSearchMatch; //TBD
    //window[instance]['prevSearchMatch'] = window[instance].nextSearchMatch; //TBD
    window[instance]['switchMode'] = window[instance].switchMode;
    window[instance]['printPaper'] = window[instance].printPaper;
    //window[instance]['highlight'] = window[instance].highlight; //TBD
    //window[instance]['postSnapshot'] = window[instance].postSnapshot; //TBD
    window[instance]['setCurrentCursor'] = window[instance].setCurrentCursor;

    pageRenderer.initialize(function() {
        //console.profile('FlexPaper init-postinit');
        window[instance].document.numPages = pageRenderer.getNumPages();
        window[instance].document.dimensions = pageRenderer.getDimensions();
        window[instance].show();

        jQuery(root).trigger('onDocumentLoaded', pageRenderer.getNumPages());
    });
};
/** FLASH mode viewer */
window.FlexPaperViewerFlashMode = function(root, config, errorHandler) {
    window['ViewerMode'] = "flash";

    if (config.Toolbar) {
        // create toolbar
        var wrapper = jQuery(root).wrap("<div style='" + jQuery(root).attr("style") + "'></div>").parent();
        jQuery(root).css({
            position: "relative", top: "0px", left: "0px",
            width: "100%", height: "100%"
        }).addClass('flexpaper_viewer');

        wrapper.prepend(jQuery(config.Toolbar));

        window.FlexPaperViewerFlashMode_BindEventHandlers(root.attr("id"));
    }

    var id = root.attr("id");
    window["FlexPaperViewer_Instance" + id] = flashembed(id,
                                                         {
                                                             src: config.swfDirectory + "/FlexPaperViewer.swf",
                                                             version: [10, 0],
                                                             expressInstall: config.swfDirectory + "/expressinstall.swf",
                                                             wmode: config.WMode || "direct",
                                                             onFail: errorHandler
                                                         },
                                                         window.FlexPaperViewerConfiguration(id, config));
};

window.FlexPaperViewerConfiguration = function(id, config) {
    var _SWFFile,_PDFFile,_IMGFiles,_IMGFiles_thumbs="",_JSONFile = "",_jsDirectory = "",_cssDirectory = "",_localeDirectory = "",_swfDirectory;
    var _uDoc = (config.DOC != null ? unescape(config.DOC) : null);
    var _JSONDataType = config.JSONDataType || "json";

    if (_uDoc != null) {
        _SWFFile = translateUrlByFormat(_uDoc, "swf");
        _PDFFile = translateUrlByFormat(_uDoc, "pdf");
        _JSONFile = translateUrlByFormat(_uDoc, _JSONDataType);
        _IMGFiles = translateUrlByFormat(_uDoc, "png");
    }

    _SWFFile = (config.SWFFile != null ? config.SWFFile : _SWFFile);
    _PDFFile = (config.PDFFile != null ? config.PDFFile : _PDFFile);
    _IMGFiles = (config.IMGFiles != null ? config.IMGFiles : _IMGFiles);
    _IMGFiles = (config.PageImagePattern != null ? config.PageImagePattern : _IMGFiles);
    _IMGFiles_thumbs = (config.ThumbIMGFiles!=null?config.ThumbIMGFiles:_IMGFiles_thumbs);
    _JSONFile = (config.JSONFile != null ? config.JSONFile : _JSONFile);
    _jsDirectory = (config.jsDirectory != null ? config.jsDirectory : "js/");
    _cssDirectory = (config.cssDirectory != null ? config.cssDirectory : "css/");
    _localeDirectory = (config.localeDirectory != null ? config.localeDirectory : "locale/");
    _swfDirectory = _jsDirectory + "../";
    if (_SWFFile != null && _SWFFile.indexOf("{") == 0 && _SWFFile.indexOf("[*,") > 0 && _SWFFile.indexOf("]") > 0) {_SWFFile = escape(_SWFFile);} // split file fix

    return {
        ElementId: id,
        SwfFile: _SWFFile,
        PdfFile: _PDFFile,
        IMGFiles: _IMGFiles,
        ThumbIMGFiles: _IMGFiles_thumbs,
        JSONFile: _JSONFile,
        useCustomJSONFormat: config.useCustomJSONFormat,
        JSONPageDataFormat: config.JSONPageDataFormat,
        JSONDataType: _JSONDataType,
        Scale: config.Scale || 0.8,
        ZoomTransition: config.ZoomTransition || "easeOut",
        ZoomTime: config.ZoomTime || 0.5,
        ZoomInterval: config.ZoomInterval || 0.2,
        FitPageOnLoad: config.FitPageOnLoad != null ? config.FitPageOnLoad : false,
        FitWidthOnLoad: config.FitWidthOnLoad != null ? config.FitWidthOnLoad : false,
        FullScreenAsMaxWindow: config.FullScreenAsMaxWindow != null ? config.FullScreenAsMaxWindow : false,
        ProgressiveLoading: config.ProgressiveLoading != null ? config.ProgressiveLoading : false,
        MinZoomSize: config.MinZoomSize || 0.2,
        MaxZoomSize: config.MaxZoomSize || 5,
        SearchMatchAll: (config.SearchMatchAll != null) ? config.SearchMatchAll : false,
        SearchServiceUrl: config.SearchServiceUrl,
        InitViewMode: config.InitViewMode,
        BitmapBasedRendering: (config.BitmapBasedRendering != null) ? config.BitmapBasedRendering : false,
        StartAtPage: config.StartAtPage,
        PrintPaperAsBitmap: (config.PrintPaperAsBitmap != null) ? config.PrintPaperAsBitmap : false,
        AutoAdjustPrintSize: (config.AutoAdjustPrintSize != null) ? config.AutoAdjustPrintSize : false,

        ViewModeToolsVisible: (config.ViewModeToolsVisible != null) ? config.ViewModeToolsVisible : true,
        ZoomToolsVisible: (config.ZoomToolsVisible != null) ? config.ZoomToolsVisible : true,
        NavToolsVisible: (config.NavToolsVisible != null) ? config.NavToolsVisible : true,
        CursorToolsVisible: (config.SearchToolsVisible != null) ? config.CursorToolsVisible : true,
        SearchToolsVisible: (config.SearchToolsVisible != null) ? config.SearchToolsVisible : true,

        // Annotations viewer parameter
        AnnotationToolsVisible : (config.AnnotationToolsVisible != null) ? config.AnnotationToolsVisible : true,

        StickyTools: config.StickyTools,
        Toolbar: config.Toolbar,
        BottomToolbar: config.BottomToolbar,
        DocSizeQueryService: config.DocSizeQueryService,

        RenderingOrder: config.RenderingOrder,

        localeChain: config.localeChain || "en_US",
        jsDirectory: _jsDirectory,
        cssDirectory: _cssDirectory,
        localeDirectory: _localeDirectory,
        key: config.key
    };
};

var DocumentViewerFactory = function(caller) {
    if (DocumentViewerFactory.getInstance != caller) {
        throw "There is no public constructor for DocumentViewerFactory";
    }
};
DocumentViewerFactory.getInstance = function(type) {
    var instance = new DocumentViewerFactory(arguments.callee);
    instance.type = type || "flexpaper";
    return instance;
};
DocumentViewerFactory.prototype.install = function(id, config, mimeType) {
    var container = jQuery("#" + id);

    switch (this.type.toLowerCase()) {
        case "native":
            if (/image\/(gif|jpeg|png)/.test(mimeType)) {
                return nativeImageViewer(container, config);
            } else {
                if ("application/x-shockwave-flash" == mimeType) {
                    return nativeFlashViewer(container, config);
                } else {
                    throw "Unsupported navite viewer type: " + mimeType;
                }
            }
            break;

        case "flexpaper":
            return flexPaperViewer(container, config);

        default:
            throw "Unsupported type: " + this.type;
    }

    function nativeImageViewer(container, url) {
        return container.html("<img src=\"" + encodeURI(url) + "\" alt=\"\" />").css({textAlign: "center"});
    }

    function nativeFlashViewer(container, url) {
        return container.html(flashembed.getHTML({
            src: url, width: "100%", height: "100%", id: "_" + (new Date().getTime()).toString(36)
        }));
    }

    function flexPaperViewer(container, config) {
        if (!config.RenderingOrder) {
            var order = [];
            if (config.PdfFile != null) { order.push("html5"); }
            if (config.SwfFile != null) { order.push("flash"); }
            if (config.JSONFile != null) { order.push("html"); }
            config.RenderingOrder = order.join(",");
        }

        var userAgent = navigator.userAgent.toLowerCase();
        var browser = {
            version: (userAgent.match(/.+(?:rv|it|ra|ie)[\/: ]([\d.]+)/) || [])[1],
            safari: /webkit/.test(userAgent),
            opera: /opera/.test(userAgent),
            msie: /msie/.test(userAgent) && !/opera/.test(userAgent),
            mozilla: /mozilla/.test(userAgent) && !/(compatible|webkit)/.test(userAgent),
            chrome: /chrome/.test(userAgent)
        };
        var supportsHTML4 = (browser.mozilla && browser.version.split(".")[0] >= 3)
                || browser.chrome
                || (browser.msie && browser.version.split(".")[0] >= 8)
                || browser.safari
                || browser.opera;


        var renderingOrder = config.RenderingOrder.split(",");
        var supported = false;

        for (var i = 0; i < renderingOrder.length && !supported; ++i) {
            var type = renderingOrder[i];

            switch (type) {
                case "flash": {
                    var fallback2Html = (i == 0 || (i > 0 && !supportsHTML4));
                    installFlashViewer(container, config, fallback2Html);
                    supported = true;
                    break;
                }
                case "html5":
                case "html":
                default: {
                    installHtmlViewer(container, config);
                    supported = true;
                    break;
                }
            }
        }

        if (config.Toolbar) {
            bindToolbarHandlers(container.attr("id"));
        }
    }

    function installFlashViewer(container, config, fallback2Html) {
        window.FlexPaperViewerFlashMode(container, config, function() {
            if (fallback2Html) {
                $(".flexpaper_toolbar").remove();
                container.empty();
                installHtmlViewer(container, config);
            }
        });
    }

    function installHtmlViewer(container, config) {
        jQuery.getScript(config.jsDirectory + "/FlexPaperViewer.js", function() {
            window.FlexPaperViewerHtmlMode(container, config);
        });
    }

    function bindToolbarHandlers(id) {
        function instance() { return window.$FlexPaper(id); }
        jQuery("#toolbar1 .flexpaper_bttnSinglePage").click(function() {instance().switchMode("Portrait");});
        jQuery("#toolbar1 .flexpaper_bttnTwoPage").click(function() {instance().switchMode("TwoPage");});
        jQuery("#toolbar1 .flexpaper_bttnThumbView").click(function() {instance().switchMode("Tile");});
        jQuery("#toolbar1 .flexpaper_bttnFitWidth").click(function() {instance().fitWidth();});
        jQuery("#toolbar1 .flexpaper_bttnFitHeight").click(function() {instance().fitHeight();});
        jQuery("#toolbar1 .flexpaper_bttnRotate").click(function() {instance().rotate();});
        jQuery("#toolbar1 .flexpaper_bttnPrevPage").click(function() {instance().prevPage();});
        jQuery("#toolbar1 .flexpaper_bttnPrevNext").click(function() {instance().nextPage();});
        jQuery("#toolbar1 .flexpaper_bttnTextSelect").click(function() {instance().setCurrentCursor("TextSelectorCursor");});
        jQuery("#toolbar1 .flexpaper_bttnHandSelect").click(function() {instance().setCurrentCursor("ArrowCursor");});
        jQuery("#toolbar1 .flexpaper_bttnTextSearch").click(function() {instance().searchText(jQuery(this).parent().find(".flexpaper_txtSearch").val());});
    }
};

window.FlexPaperViewerFlashMode_BindEventHandlers = function(id) {
    var viewer = jQuery("#"+id);
    function instance(){return window.$FlexPaper(id); }
    function toolbarButton(button){return viewer.parent().find(button); }
    function toolbarButtonState(button,cssClass,state) {var btn=toolbarButton(button);if(state){btn.addClass(cssClass);}else{btn.removeClass(cssClass);}}
    function toolbarButtonPressed(button,pressed){toolbarButtonState(button,"flexpaper_tbbutton_pressed",pressed);}
    function toolbarButtonDisabled(button,disabled){toolbarButtonState(button,"flexpaper_tbbutton_disabled",disabled);}
    function showProgress(){jQuery(".flexpaper_progress").show();}
    function hideProgress(){jQuery(".flexpaper_progress").hide();}
    function setTotalPages(value){toolbarButton(".flexpaper_lblTotalPages").html(" / "+value);}
    function zoomSlider(){var ref=instance();if(ref["slider"]==undefined){var el=toolbarButton(".flexpaper_zoomSlider").get(0);ref["slider"]=new Slider(el,{callback:function(value){instance().sliderChange(5 * value);},animation_callback:function(value){if(value>0){instance().sliderChange(5*value);}}});}return ref["slider"];}
    function bindKeystrokes() {
        jQuery(".flexpaper_txtZoomFactor").bind("keypress",function(e){if(e.keyCode==13){var zoom=jQuery(this).val().replace("%","")/100;try {instance().sliderChange(zoom);}catch(e){}}});
        jQuery(".flexpaper_txtSearch").bind("keypress",function(e){if(e.keyCode==13){try{instance().searchText(jQuery(this).val());}catch(e){}}});
        jQuery(".flexpaper_txtPageNumber").bind("keypress",function(e){if(e.keyCode==13){try{instance().gotoPage(jQuery(this).val());}catch(e){}}});
    }
    viewer.bind("onDocumentLoading",function(e) { showProgress(); });
    viewer.bind("onDocumentLoaded",function(e, totalPages) { hideProgress();setTotalPages(totalPages); bindKeystrokes(); });
    viewer.bind("onDocumentLoadedError",function(e, errMessage) { hideProgress(); alert(errMessage || "Error loading document.");});
    viewer.bind("onPageLoaded",function(e,pageNumber) { hideProgress(); });
    viewer.bind("onProgress",function(e,loaded,total) {});
    viewer.bind("onFitModeChanged",function(e, mode) { toolbarButtonPressed(".flexpaper_tbbutton_fitmode", false); "Fit Height" == mode && toolbarButtonPressed(".flexpaper_bttnFitHeight", true); "Fit Width" == mode && toolbarButtonPressed(".flexpaper_bttnFitWidth", true); });
    viewer.bind("onViewModeChanged",function(e,mode) { toolbarButtonPressed(".flexpaper_tbbutton_viewmode", false); ("Tile" == mode || "ThumbView" == mode) && toolbarButtonPressed(".flexpaper_bttnThumbView", true); "TwoPage" == mode && toolbarButtonPressed(".flexpaper_bttnTwoPage", true); "Portrait" == mode && toolbarButtonPressed(".flexpaper_bttnSinglePage", true);var disabled = (mode !== "Portrait");toolbarButtonDisabled(".flexpaper_zoomSlider",disabled);toolbarButtonDisabled(".flexpaper_txtZoomFactor",disabled);toolbarButtonDisabled(".flexpaper_bttnFitWidth",disabled);toolbarButtonDisabled(".flexpaper_bttnFitHeight",disabled);toolbarButtonDisabled(".flexpaper_bttnRotate",disabled);if(disabled){zoomSlider().disable();}else{zoomSlider().enable();}});
    viewer.bind("onCurrentPageChanged",function(e,page){ toolbarButton(".flexpaper_txtPageNumber").val(page); });
    viewer.bind("onCursorModeChanged",function(e,mode){ "TextSelectorCursor" == mode && (toolbarButtonPressed(".flexpaper_bttnTextSelect", true) || toolbarButtonPressed(".flexpaper_bttnHandSelect", false)); "ArrowCursor" == mode && (toolbarButtonPressed(".flexpaper_bttnTextSelect", false) || toolbarButtonPressed(".flexpaper_bttnHandSelect", true));});
    viewer.bind("onExternalLinkClicked",function(e,link) { /*window.location.href = link;*/ });
    viewer.bind("onScaleChanged",function(e,value) { toolbarButton(".flexpaper_txtZoomFactor").val(Math.round((value*100))+"%");if(!window.isTouchScreen){var sl=zoomSlider();sl.setValue(value/5,true);sl.slide();sl.show();}});
};