function InactivityService(actionForm) {
    this._constructor(actionForm);
}
(function() { // local scope

    var MS_PER_SECOND = 1000;
    var REMINDER_INTERVAL = MS_PER_SECOND * 1;

    InactivityService.prototype = {

        actionForm: null,
        panel: null,
        timeout: null,
        gracePeriod: null,
        remainTime:null,
        remainTimerContainer: null,
        timerId: null,
        reminderIntervalId: null,

        _constructor: function(actionForm) {
            this.messages = [];
            this.actionForm = actionForm;
            this.setInactiveTime(0, 0);
        },

        setInactiveTime: function(timeout, gracePeriod) {
            this.timeout = timeout > 0 ? timeout : 0;
            this.gracePeriod = gracePeriod > 0 ? gracePeriod : 0;
        },

        start: function() {
            if (this.timeout <= 0) {
                return;
            }

            this._clearTimer();
            var thisObj = this;
            this.timerId = setTimeout(function() {
                thisObj._doStart();
            }, this.timeout);
        },

        stop: function() {
            this._clearTimer();
            if (this.panel != null) {
                this.panel.setVisible(false);
            }
        },

        restart: function() {
            this.stop();
            this.start();
        },

        logout: function() {
            this.stop();
            new Action("Logout", this.actionForm).perform();
        },

        _doStart: function() {
            this._clearTimer();

            if (this.panel == null) {
                this.panel = new InactivityPane(this);
            }

            this.panel.setVisible(true);
            this._startRemainder(this.gracePeriod);
        },

        _doRemind: function() {
            if (this.remainTime > 0) {
                this.panel.setRemainSecond(Math.round(this.remainTime/MS_PER_SECOND));
                this.remainTime -= REMINDER_INTERVAL;
            } else {
                this.logout();
            }
        },

        _startRemainder: function(remainingTime) {
            if (remainingTime <= 0) {
                return;
            }

            this.remainTime = remainingTime;
            this._doRemind();

            var thisObj = this;
            this.reminderIntervalId = setInterval(function() {
                thisObj._doRemind();
            }, REMINDER_INTERVAL);
        },

        _clearTimer: function() {
            clearInterval(this.reminderIntervalId);
            this.reminderIntervalId = null;

            clearTimeout(this.timerId);
            this.timerId = null;
        }
    };
})();