function KeyboardManager(caller) {
    if (KeyboardManager.getInstance != caller) {
        throw new Error("There is no public constructor for KeyboardManager class.");
    }
}(function() {

    KeyboardManager._soleInstance = null;

    KeyboardManager.getInstance = function() {
        if (this._soleInstance == null) {
            this._soleInstance = new KeyboardManager(arguments.callee);
        }
        return this._soleInstance;
    };

    KeyboardManager.prototype = {

        handlers: [],

        dispatchKeyEvent: function(e) {
            var ke = new KeyEvent(e);
            var keyHandlers = this._getKeyHandlers(ke.getKeyCode(), false);
            if (ke.isModifiersDown() || keyHandlers == null) {
                return;
            }

            for (var i = 0, size = keyHandlers.length; !ke.isCanceled() && i < size; ++i) {
                var handler = keyHandlers[i];

                if (handler && Object.isFunction(handler.processKeyEvent)) {
                    if (Object.isElement(handler) && handler.disabled) {
                        ke.cancel();
                    } else {
                        handler.processKeyEvent(ke);
                    }
                } else {
                    handler(ke);
                }
            }
        },

        registerHandler: function(keyCode, handler) {
            if (!(Object.isFunction(handler) || Object.isFunction(handler.processKeyEvent))) {
                throw new Error("handler must be a function or an object with processKeyEvent(KeyEvent ke) method.");
            }

            var keyHandlers = this._getKeyHandlers(keyCode, true);

            if (keyHandlers.indexOf(handler) == -1) {
                keyHandlers.push(handler);
            }
        },

        unregisterHandler: function(key, handler) {
            var keyHandlers = this._getKeyHandlers(key);

            if (keyHandlers != null) {
                var index = keyHandlers.indexOf(handler);
                if (index != -1) {
                    delete keyHandlers[index];
                    keyHandlers.splice(index, 1);

                    if (keyHandlers.length == 0) {
                        this._removeKeyHandlers(key);
                    }
                }
            }
        },

        _getKeyHandlers: function(key, create) {
            var result = this.handlers[key.toString()];

            if (result == null && create) {
                result = new Array();
                this.handlers[key.toString()] = result;
            }

            return result;
        },

        _removeKeyHandlers: function(key) {
            delete this.handlers[key.toString()];
        }

    };
})();

document.onkeydown = function(e) { KeyboardManager.getInstance().dispatchKeyEvent(e); };