var List = Class.create({

    listElement: null,
    listeners: null,
    model: null,

    _constructor: function(listElement) {
        this.listElement = listElement;
        this.listeners = [];
        this.model = [];

        Event.observe(this.listElement, "change", this._handleSelectionChanged.bind(this));
    },

    addPropertyChangeListener: function(listener) {
        if (listener && Object.isFunction(listener.propertyChange)) {
            this.listeners.push(listener);
        }
    },

    removePropertyChangeListener: function(listener) {
        for (var i = 0; i < this.listeners.length; ++i) {
            if (this.listeners[i] === listener) {
                this.listeners[i] = null;
                this.listeners.splice(i, 1);
                break;
            }
        }
    },

    addItem: function(item) {
        var text = item.toString();
        var option = document.createElement("option");
        option.appendChild(document.createTextNode(text));
        option.value = text;
        option.title = text;

        this.listElement.appendChild(option);
        this.model.push(item);

        this._firePropertyChange(List.PCS_MODEL, null, item);
    },

    removeItem: function(item) {
        var index = this.model.indexOf(item);
        if (index == -1) {
            return;
        }

        var option = this.listElement.options[index];
        if (!Object.isElement(option)) {
            return;
        }

        var shouldUpdateSelection = this.getSelectedIndex() == index;
        this.listElement.removeChild(option);
        this.model.splice(index, 1);

        this._firePropertyChange(List.PCS_MODEL, item, null);

        if (shouldUpdateSelection) {
            var count = this.getItemCount();
            this.setSelectedIndex(index < count ? index : count - 1);
        }
    },

    clear: function() {
        while (this.model.length > 0) {
            this.removeItem(this.model[0]);
        }
    },

    getItemCount: function() {
        return this.listElement.options.length;
    },

    getSelectedIndex: function() {
        return this.listElement.selectedIndex;
    },

    setSelectedIndex: function(index) {
        if (index > this.getItemCount()) {
            return;
        }

        try {
            var oldSelectedIndex = this.getSelectedIndex();
            this.listElement.selectedIndex = index;
            this._firePropertyChange(List.PCS_SELECTED_INDEX, oldSelectedIndex, index);
        } catch(e) {
            // fails in IE6
            (function(self, oldValue, newValue) {
                self.listElement.selectedIndex = newValue;
                self._firePropertyChange(List.PCS_SELECTED_INDEX, oldValue, newValue);
            }.delay(10, this, oldSelectedIndex, index));
        }
    },

    getSelectedItems: function() {
        var result = new Array();
        var options = this.listElement.options;

        for (var i = 0; i < options.length; ++i) {
            var option = options[i];
            if (option.selected) {
                result.push(this._asModelItem(option));
            }
        }

        return result;
    },

    _handleSelectionChanged: function() {
        this._firePropertyChange(List.PCS_SELECTED_INDEX, null, this.getSelectedIndex());
    },

    _firePropertyChange: function(propertyName, oldValue, newValue) {
        if (oldValue != null && newValue != null && oldValue === newValue) {
            return;
        }

        var event = {propertyName: propertyName, oldValue: oldValue, newValue: newValue};

        for (var i = 0; i < this.listeners.length; ++i) {
            this.listeners[i].propertyChange(event);
        }
    },

    _asModelItem: function(option) {
        try {
            this._checkBounds(option.index);
            return this.model[option.index];
        } catch(e) {
            throw e;
            //return null;
        }
    },

    _checkBounds: function(index) {
        if (isNaN(index) || index < 0 || index > this.model.length - 1) {
            throw "Index out of bounds: " + index;
        }
    }
});

List.PCS_SELECTED_INDEX = "List.selectedIndex";
List.PCS_MODEL = "List.model";