var EmailFileController = Class.create({

    config: null,
    contacts: null,
    selectedContactIds: null,

    composeDialog: null,
    contactListDialog: null,
    addContactDialog: null,

    _constructor: function(config) {
        this.config = Object.extend(Object.extend({}, EmailFileController.DEFAULT_CONFIG), config);
        this.contacts = [];
        this.selectedContactIds = [];
    },

    setContacts: function(data) {
        this.contacts = [];
        var values = $A(data);

        for (var i = 0, size = values.length; i < size; i = i + 2) {
            var name = values[i];
            var email = i + 1 < size ? values[i + 1] : "";
            this.contacts.push({id: name + email, name: name, email: email});
        }

        this._updateAddressBookState();
    },

    compose: function(filenames) {
        this._hideComposeDialog();
        this._showComposeDialog(filenames);
    },

    _onAddContact: function(dialog) {
        this._hideAddContactDialog();
        this._showAddContactDialog();
    },

    _onGetAddressBook: function(dialog) {
        this._hideContactListDialog();
        this._showContactListDialog();
    },

    _onCompose$Cancel: function() {
        this._hideComposeDialog();
    },

    _onCompose$LoadError: function(dialog) {
        dialog.hide();
        this._showLoadError();
    },

    _onCompose$Done: function(dialog) {
        var message = dialog.getDoneMessage();
        this._hideComposeDialog();
        this._showMessage(message);
    },

    _onContactList$Submit: function(dialog) {
        var contacts = dialog.getSelectedContacts();
        this._hideContactListDialog();
        this._updateSelectedContacts(contacts);
    },

    _onContactList$Cancel: function() {
        this._hideContactListDialog();
    },

    _onAddContact$Cancel: function(dialog) {
        this._hideAddContactDialog();
    },

    _onAddContact$Done: function(dialog) {
        this._hideAddContactDialog();
    },

    _showComposeDialog: function(filenames) {
        var viewport = document.viewport.getDimensions();

        var dialog = new ComposeEmailDialog(this.config.cefu, this.config.metadata);
        dialog.setSize(viewport.width * 0.6, Math.min(550, viewport.height * 0.9));
        dialog.show(filenames, Object.extend(new ComposeEmailDialog.Listener(), {
            onAddContact: this._onAddContact.bind(this),
            onGetAddressBook: this._onGetAddressBook.bind(this),
            onLoadError: this._onCompose$LoadError.bind(this),
            onCancel: this._onCompose$Cancel.bind(this),
            onDone: this._onCompose$Done.bind(this)
        }));

        this.composeDialog = dialog;
    },

    _hideComposeDialog: function() {
        this._hideAddContactDialog();
        this._hideContactListDialog();

        this._disposeDialog(this.composeDialog);
        this.composeDialog = null;
        this.selectedContactIds = [];
    },

    _showContactListDialog: function() {
        var dialog = new ContactListDialog(this.contacts.clone());
        dialog.selectContacts(this._findSelectedContacts());
        dialog.show({
            onSubmit: this._onContactList$Submit.bind(this),
            onCancel: this._onContactList$Cancel.bind(this)
        });

        this.contactListDialog = dialog;
    },

    _hideContactListDialog: function() {
        this._disposeDialog(this.contactListDialog);
        this.contactListDialog = null;
    },

    _showAddContactDialog: function() {
        var dialog = new FormDialog("AddContactDialog", this.config.aciu);
        dialog.setSize(430, 235);
        dialog.show(Object.extend(new FormDialog.Listener(), {
            onCancel: this._onAddContact$Cancel.bind(this),
            onDone: this._onAddContact$Done.bind(this)
        }), this.config.metadata);

        this.addContactDialog = dialog;
    },

    _hideAddContactDialog: function() {
        this._disposeDialog(this.addContactDialog);
        this.addContactDialog = null;
    },

    _disposeDialog: function(dialog) {
        if (dialog) {
            dialog.dispose();
        }
    },

    _updateSelectedContacts: function(contacts) {
        var ids = this.selectedContactIds.clone();
        var selection = contacts.clone();

        for (var i = 0; i < ids.length; ) {
            var contact = this._findContactById(ids[i]);
            var index = selection.indexOf(contact);

            if (index != -1) {
                selection.splice(index, 1);
                ++i;
            } else {
                this.composeDialog.removeRecipient(contact);
                ids.splice(i, 1);
            }
        }

        while (selection.length != 0) {
            var newContact = selection.shift();
            this.composeDialog.addRecipient(newContact);
            ids.push(newContact.id);
        }

        this.selectedContactIds = ids;
    },

    _updateAddressBookState: function() {
        if (this.composeDialog) {
            this.composeDialog.setAddressBookEnabled(this.contacts.length != 0);
        }
    },

    _findSelectedContacts: function() {
        var result = [];
        for (var i = 0; i < this.selectedContactIds.length; ++i) {
            var id = this.selectedContactIds[i];
            for (var j = 0; j < this.contacts.length; ++j) {
                var candidate = this.contacts[j];
                if (candidate.id == id) {
                    result.push(candidate);
                }
            }
        }
        return result;
    },

    _findContactById: function(id) {
        for (var i = 0; i < this.contacts.length; ++i) {
            var candidate = this.contacts[i];
            if (candidate.id == id) {
                return candidate;
            }
        }
        return null;
    },

    _showLoadError: function() {
        ModalDialog.alert(ComposeEmailDialog.i18n.LOAD_ERROR_MESSAGE, ComposeEmailDialog.i18n.LOAD_ERROR_TITLE);
    },

    _showMessage: function(message) {
        if (message) {
            ModalDialog.alert(message);
        }
    }
});
EmailFileController.DEFAULT_CONFIG = {cefu: "", aciu: "", metadata: {}};

var EmailFileView = Class.create({

    listener: null,

    _constructor: function(doc) {
        function byId(id, d) { return (d || document).getElementById(id); }
        this.toField = byId("to", doc);
        this.ccField = byId("cc", doc);
        this.bccField = byId("bcc", doc);
        this.replyToField = byId("replyto", doc);

        this.addCcButton = byId("adhoc-addcc", doc);
        this.addBccButton = byId("adhoc-addbcc", doc);
        this.addReplyToButton = byId("adhoc-addreplyto", doc);

        this.contactListButton = byId("clbtn", doc);
        this.contactListAddButton = byId("claddbtn", doc);
        this.enableExpiresBox = byId("enexpd", doc);
        this.expiresField = byId("expires", doc);
        this.enableMaxDownloadsBox = byId("emaxdl", doc);
        this.maxDownloadsField = byId("maxdownloads", doc);
        this.deleteAfterMaxDownloadsBox = byId("deleteaftermaxdownloads", doc);
        this.protectWithComboBox = byId("protectwith", doc);
        this.customPasswordField = byId("custompwd", doc);

        new NumberEditor(this.maxDownloadsField).setMinValue(1);
        Utils.setStyle(this.contactListButton, "cursor:pointer");
        Utils.setStyle(this.contactListAddButton, "cursor:pointer");
    },

    addRecipient: function(contact) {
        var contacts = this._parseContacts();
        contacts.push(this._formatContact(contact));
        this._setContacts(contacts);
    },

    removeRecipient: function(contact) {
        var contacts = this._parseContacts();
        var index = contacts.indexOf(this._formatContact(contact));
        if (index != -1) {
            contacts.splice(index, 1);
            this._setContacts(contacts);
        }
    },

    setAddressBookEnabled: function(value) {
        // Utils.setEnabled(this.contactListButton, value);
        Utils.setVisible(this.contactListButton, value);
    },

    show: function(listener) {
        this.listener = Object.extend(new EmailFileView.Listener(), listener || {});
        this._bindEvents();
        this._update();
    },

    hide: function() {
        this._unbindEvents();
        this.listener = null;
    },

    _update: function() {
        var maxDownloadsEnabled = this.enableMaxDownloadsBox.checked;
        Utils.setEnabled(this.expiresField, this.enableExpiresBox.checked);
        Utils.setEnabled(this.maxDownloadsField, maxDownloadsEnabled);
        Utils.setEnabled(this.deleteAfterMaxDownloadsBox, maxDownloadsEnabled);

        if (Object.isElement(this.customPasswordField) && Object.isElement(this.protectWithComboBox)) {
            var customSelected = $F(this.protectWithComboBox) == 'protectwithcustom';
            Utils.setEnabled(this.customPasswordField, customSelected);
        }
    },

    _onClick$AddCc: function(e) {
        Event.stop(e);
        Utils.setVisible(Utils.findAncestor(this.ccField, "tr"), true);
        Utils.setVisible(this.addCcButton, false);
        if (!Utils.isVisible(this.addBccButton)) {
            Utils.setVisible(Utils.findAncestor(this.addCcButton, "tr"), false);
        }
        this.ccField.focus();
    },

    _onClick$AddBcc: function(e) {
        Event.stop(e);
        Utils.setVisible(Utils.findAncestor(this.bccField, "tr"), true);
        Utils.setVisible(this.addBccButton, false);
        if (!Utils.isVisible(this.addCcButton)) {
            Utils.setVisible(Utils.findAncestor(this.addBccButton, "tr"), false);
        }
        this.bccField.focus();
    },

    _onClick$AddReplyTo: function(e) {
        Event.stop(e);
        Utils.setVisible(Utils.findAncestor(this.replyToField, "tr"), true);
        Utils.setVisible(this.addReplyToButton, false);
        Utils.setVisible(Utils.findAncestor(this.addReplyToButton, "tr"), false);
        this.replyToField.focus();
    },

    _onClick$AddContact: function(e) {
        Event.stop(e);
        this.listener.onAddContact(this);
    },

    _onClick$GetAddressBook: function(e) {
        Event.stop(e);
        this.listener.onGetAddressBook(this);
    },

    _bindEvents: function() {
        this.addCcHandler = this._onClick$AddCc.bindAsEventListener(this);
        this.addBccHandler = this._onClick$AddBcc.bindAsEventListener(this);
        this.addReplyToHandler = this._onClick$AddReplyTo.bindAsEventListener(this);
        this.contactListAddHandler = this._onClick$AddContact.bindAsEventListener(this);
        this.contactListSelectHandler = this._onClick$GetAddressBook.bindAsEventListener(this);
        this.viewStateHandler = this._update.bind(this);

        Event.observe(this.addCcButton, "click", this.addCcHandler);
        Event.observe(this.addBccButton, "click", this.addBccHandler);
        Event.observe(this.addReplyToButton, "click", this.addReplyToHandler);
        Event.observe(this.contactListButton, "click", this.contactListSelectHandler);
        Event.observe(this.contactListAddButton, "click", this.contactListAddHandler);
        Event.observe(this.enableExpiresBox, "click", this.viewStateHandler);
        Event.observe(this.enableMaxDownloadsBox, "click", this.viewStateHandler);

        if (Object.isElement(this.protectWithComboBox)) {
            Event.observe(this.protectWithComboBox, "change", this.viewStateHandler);
        }
    },

    _unbindEvents: function() {
        Event.unobserve(this.addCcButton, "click", this.addCcHandler);
        Event.unobserve(this.addBccButton, "click", this.addBccHandler);
        Event.unobserve(this.addReplyToButton, "click", this.addReplyToHandler);
        Event.unobserve(this.contactListAddButton, "click", this.contactListAddHandler);
        Event.unobserve(this.enableExpiresBox, "click", this.viewStateHandler);
        Event.unobserve(this.enableMaxDownloadsBox, "click", this.viewStateHandler);

        if (Object.isElement(this.protectWithComboBox)) {
            Event.unobserve(this.protectWithComboBox, "change", this.viewStateHandler);
        }

        this.addCcHandler = null;
        this.addBccHandler = null;
        this.addReplyToHandler = null;
        this.contactListAddHandler = null;
        this.contactListSelectHandler = null;
        this.viewStateHandler = null;
    },

    _formatContact: function(contact) {
        return "\"" + contact.name + "\" <" + contact.email + ">";
    },

    _setContacts: function(contacts) {
        this.toField.value = contacts.join(", ");
    },

    _parseContacts: function() {
        var value = new String(this.toField.value || "").trim();
        return (value.length != 0) ? value.split(/\s*,\s*/) : [];
    }

});
EmailFileView.Listener = Class.create({
    onAddContact: function(view) {},
    onGetAddressBook: function(view) {}
});

var ComposeEmailDialog = Class.create(ModalDialog, {

    form: null,
    metadata: null,
    view: null,
    channel: null,
    addressBookEnabled: false,
    done: false,
    doneMessage: null,

    listener: null,

    _constructor: function(url, metadata) {
        this.channel = new IFrameChannel({preventCaching: true});
        this.channel.open(url, "post", Object.extend(new Channel.Listener(), {
            onComplete: this._handleFormLoaded.bind(this)
        }));
        this.metadata = metadata || {};

        this.setTitle(ComposeEmailDialog.TITLE);
        this.addButton(ModalDialog.BUTTON_OK, ComposeEmailDialog.Buttons.OK, this._handleSubmitPressed.bind(this));
        this.addButton(ModalDialog.BUTTON_CANCEL, ComposeEmailDialog.Buttons.CANCEL, this._handleCancelPressed.bind(this));
    },

    getId: function() {
        return "ComposeEmailDialog";
    },

    getForm: function() {
        return this.form;
    },

    setAddressBookEnabled: function(value) {
        this.addressBookEnabled = value;
        if (this.view) {
            this.view.setAddressBookEnabled(this.addressBookEnabled);
        }
    },

    addRecipient: function(contact) {
        if (this.view) {
            this.view.addRecipient(contact);
        }
    },

    removeRecipient: function(contact) {
        if (this.view) {
            this.view.removeRecipient(contact);
        }
    },

    show: function(filenames, listener) {
        this.form = null;
        this.listener = listener || new ComposeEmailDialog.Listener();
        this.channel.send(Object.extend({filename: filenames}, this.metadata));

        this.setButtonEnabled(ModalDialog.BUTTON_OK, false);
        this.setVisible(true);
    },

    dispose: function() {
        this.form = null;
        this.channel.close();
        this._super.dispose.call(this);
    },

    getDoneMessage: function() {
        return this.doneMessage || "";
    },

    isDone: function() {
        return this.done;
    },

    /** EmailFileView.Listener implementation */
    onAddContact: function() {
        this.listener.onAddContact(this);
    },

    /** EmailFileView.Listener implementation */
    onGetAddressBook: function() {
        this.listener.onGetAddressBook(this);
    },

    _handleSubmitPressed: function() {
        this.setButtonEnabled(ModalDialog.BUTTON_OK, false);

        if (this.form != null) {
            this.channel.submitForm(this.form);
        }

        this.listener.onSubmit(this);
    },

    _handleCancelPressed: function() {
        this.setVisible(false);
        this.channel.close();
        this.listener.onCancel(this);
    },

    _handleFormLoaded: function() {
        var metadata = this.channel.getResponseMetadata();
        this.done = (metadata["x-dialog-control"] || "").toLocaleLowerCase() == "done";
        this.doneMessage = metadata["x-dialog-reason"] || "";

        if (this.isDone()) {
            this.listener.onDone(this);
        } else {
            if (this._isRequestedFormLoaded()) {
                this._initDialog();
                this.listener.onLoaded(this);
            } else {
                this.channel.close();
                this.listener.onLoadError(this);
            }
        }
    },

    _isRequestedFormLoaded: function() {
        var doc = this.channel.getResponseDocument();
        var channelUrl = this.channel.getUrl();
        channelUrl = channelUrl.replace(/\?.*/, "");
        var documentUrl = channelUrl.match(/^http/i) ? doc.location.href : doc.location.pathname;
        return doc.forms.length != 0 && documentUrl.indexOf(channelUrl) == 0;
    },

    _initDialog: function() {
        this.getContent().innerHTML = this.channel.getResponseHtml();
        var forms = this.getContent().getElementsByTagName("form");

        if (forms.length != 0) {
            this.form = forms[0];
            Event.observe(this.form, "submit", this._handleSubmitPressed.bind(this));

            this.view = new EmailFileView();
            this.view.setAddressBookEnabled(this.addressBookEnabled);
            this.view.show(this);
        } else {
            this.form = null;

            if (this.view != null) {
                this.view.hide();
            }
            this.view = null;
        }

        this.setButtonEnabled(ModalDialog.BUTTON_OK, true);
    }
});
ComposeEmailDialog.TITLE = "Email File(s)";
ComposeEmailDialog.Buttons = {OK: 'Send', CANCEL: 'Cancel'};
ComposeEmailDialog.i18n = {LOAD_ERROR_TITLE: "Error", LOAD_ERROR_MESSAGE: "The requested content cannot be loaded.<br/>Please try again later."};
ComposeEmailDialog.Listener = Class.create(ModalDialog.Listener, {
    onAddContact: function(d) {},
    onGetAddressBook: function(d) {},
    onLoaded: function(d) {},
    onLoadError: function(d) {},
    onDone: function(d) {}
});

var ContactListDialog = Class.create(ModalDialog, {

    inputs: null,
    contacts: null,

    _constructor: function(contacts) {
        this.inputs = [];
        this.contacts = contacts;

        this._init();
    },

    getId: function() {
        return "ContactListDialog";
    },

    getSelectedContacts: function() {
        var result = [];

        for (var i = 0, s1 = this.inputs.length; i < s1; ++i) {
            var id = $F(this.inputs[i]);
            for (var j = 0, s2 = this.contacts.length; j < s2; ++j) {
                var contact = this.contacts[j];
                if (contact.id == id) {
                    result.push(contact);
                    break;
                }
            }
        }
        return result;
    },

    selectContacts: function(contacts) {
        for (var i = 0; i < this.inputs.length; ++i) {
            var input = this.inputs[i];
            var found = false;
            for (var j = 0; j < contacts.length; ++j) {
                if (input.value == contacts[j].id) {
                    found = true;
                    break;
                }
            }
            input.checked = found;
        }
    },

    _handleSubmitPressed: function() {
        this.listener.onSubmit(this);
    },

    _handleCancelPressed: function() {
        this.listener.onCancel(this);
    },

    _init: function() {
        this.setTitle(ContactListDialog.TITLE);
        this.setContent(this._createContentPane());
        this.addButton(ModalDialog.BUTTON_OK, ContactListDialog.Buttons.OK, this._handleSubmitPressed.bind(this));
        this.addButton(ModalDialog.BUTTON_CANCEL, ContactListDialog.Buttons.CANCEL, this._handleCancelPressed.bind(this));
    },

    _createContentPane: function() {
        var panel = document.createElement(Tag.DIV);
        for (var i = 0, size = this.contacts.length; i < size; ++i) {
            var row = this._createContactInputRow("contact" + i, this.contacts[i]);
            panel.appendChild(row);
        }
        return panel;
    },

    _createContactInputRow: function(id, contact) {
        var input = Utils.createInput(Tag.INPUT_TYPE_CHECKBOX, id);
        input.id = contact.id;
        input.value = contact.id;
        this.inputs.push(input);

        var label = document.createElement(Tag.LABEL);
        label.htmlFor = input.id;

        var email = document.createElement(Tag.SPAN);
        email.appendChild(document.createTextNode(" <" + contact.email + ">"));
        Utils.setStyle(email, "color:#777;");
        label.appendChild(document.createTextNode(" \"" + contact.name +"\""));
        label.appendChild(email);

        var panel = document.createElement(Tag.PARAGRAPH);
        panel.appendChild(input);
        panel.appendChild(label);

        return panel;
    }

});
ContactListDialog.TITLE = "Contact List";
ContactListDialog.Buttons = {OK: 'OK', CANCEL: 'Cancel'};