var AdHocTransferController = Class.create({

    view: null,

    _constructor: function(view) {
        this.view = view;
    },

    start: function() {
        this.view.show(this);
    },

    stop: function() {
        this.view.hide();
    },

    onRevoke: function(view, items) {
        if (items.length == 0) {
            return;
        }

        this.view.showRevokeDialog(items, this._handleConfirmRevoke.bind(this));
    },

    onExtend: function(view, items) {
        if (items.length == 0) {
            return;
        }

        var date = new Date();
        date.setDate(new Date().getDate() + 7);

        this.view.showExtendDialog(items, date, this._handleConfirmExtend.bind(this));
    },

    _handleConfirmRevoke: function(dialog, items) {
        dialog.dispose();

        this._sendForm(this.view.revokeButton);
    },

    _handleConfirmExtend: function(dialog, items, date) {
        dialog.dispose();

        this.view.expirationDateField.value = date;
        this._sendForm(this.view.extendButton);
    },

    _sendForm: function(button) {
        var form = this.view.form;
        var formAction = button && button.getAttribute("formaction");
        if (formAction) {
            form.action = formAction;
        }
        form.submit();
    }
});
var AdHocTransferView = Class.create({

    form: null,
    table: null,
    revokeButton: null,
    extendButton: null,
    selectAllBox: null,
    selectItemBoxes: null,
    expirationDateField: null,
    dateFormat: "MM/dd/yyyy",

    _constructor: function() {
        this.form = $("itemsform");
        this.table = $("itemslist");
        this.revokeButton = $("revokeitem");
        this.extendButton = $("extenditem");
        this.selectAllBox = $("selectall");
        this.selectItemBoxes = [];
        this.expirationDateField = $("newexpires");
    },

    show: function(listener) {
        this.listener = listener || new AdHocTransferView.Listener();
        this._update();
        this._bindEvents();
    },

    hide: function() {
        this.listener = null;
        this._unbindEvents();
    },

    showRevokeDialog: function(items, handler) {
        var buttons = {};
        buttons[AdHocTransferView.Buttons.OK] = function(dialog) {handler.call(handler, dialog, items);};
        buttons[AdHocTransferView.Buttons.CANCEL] = function(dialog) {dialog.dispose();};
        ModalDialog.confirm(document.title, AdHocTransferView.Messages.CONFIRM_REVOKE, buttons);
    },

    showExtendDialog: function(items, defaultDate, handler) {
        var message = document.createElement("P");
        message.appendChild(document.createTextNode(AdHocTransferView.Messages.CONFIRM_EXTEND));
        var input = Utils.createTextInput("expirationdate");
        var label = Utils.createLabel(AdHocTransferView.Labels.DATE, input.id);
        var format = document.createElement("SPAN");
        format.appendChild(document.createTextNode(this.dateFormat));

        Utils.setStyle(message, "font-weight:bold;");
        Utils.addCssClass(format, "desc");
        Utils.setStyle(format, "margin-left:6px");

        var dialog = new ModalDialog("AdHocTransferExtendDialog");
        var content = dialog.getContent();
        content.appendChild(message);
        content.appendChild(label);
        content.appendChild(input);
        content.appendChild(format);

        var dateEditor = new DateEditor(input, this.dateFormat);
        dateEditor.setMinValue(new Date().getTime());
        dateEditor.setDefaultValue(defaultDate);
        dateEditor.setValue(defaultDate);

        dialog.addButton(ModalDialog.BUTTON_OK, AdHocTransferView.Buttons.OK);
        dialog.addButton(ModalDialog.BUTTON_CANCEL, AdHocTransferView.Buttons.CANCEL);
        dialog.setTitle(document.title);
        dialog.show({
            onSubmit: function(dialog) {
                var date = dateEditor.getValue();
                handler.call(handler, dialog, items, date);
            },
            onCancel: function(dialog) {
                dialog.dispose();
            }
        });
    },

    _getSelectedItems: function() {
        var items = [];
        for (var i = 0, size = this.selectItemBoxes.length; i < size; ++i) {
            var value = $F(this.selectItemBoxes[i]);
            if (value != null) {
                items.push(value);
            }
        }
        return items;
    },

    _update: function() {
        var count = this._getSelectedItems().length;
        this.selectAllBox.checked = (this.selectItemBoxes.length == count);
        Utils.setEnabled(this.revokeButton, count != 0);
        Utils.setEnabled(this.extendButton, count != 0);
    },

    _bindEvents: function () {
        Event.observe(this.revokeButton, "click", this._handleRevokePerformed.bindAsEventListener(this));
        Event.observe(this.extendButton, "click", this._handleExtendPerformed.bindAsEventListener(this));
        Event.observe(this.selectAllBox, "click", this._handleSelectAllPerformed.bind(this));

        var selectionHandler = this._update.bind(this);
        for (var i = 0; i < this.selectItemBoxes.length; ++i) {
            Event.observe(this.selectItemBoxes[i], "click", selectionHandler);
        }
    },

    _unbindEvents: function() {
        // todo:
    },

    _handleSelectAllPerformed: function() {
        var selected = Boolean.toBoolean($F(this.selectAllBox));
        for (var i = 0, size = this.selectItemBoxes.length; i < size; ++i) {
            this.selectItemBoxes[i].checked = selected;
        }
        this._update();
    },

    _handleRevokePerformed: function(e) {
        Event.stop(e);
        this.listener.onRevoke(this, this._getSelectedItems());
    },

    _handleExtendPerformed: function(e) {
        Event.stop(e);
        this.listener.onExtend(this, this._getSelectedItems());
    }
});
AdHocTransferView.Labels = {DATE: "Date:"};
AdHocTransferView.Buttons = {OK: "OK", CANCEL: "Cancel"};
AdHocTransferView.Messages = {CONFIRM_REVOKE: "Revoke selected item(s)?", CONFIRM_EXTEND: "Extend selected item(s)?"};
AdHocTransferView.Listener = Class.create({
    onRevoke: function(view, items) {},
    onExtend: function(view, items) {}
});

var ManageTransferView = Class.create(AdHocTransferView, {

    _constructor: function() {
        this.selectItemBoxes = Utils.getFormInputs(this.form, Tag.INPUT_TYPE_CHECKBOX, "h");

        var sorter = new TableSorter(this.table);
        sorter.setSortedColumns([1, 2, 3, 4, 5, 6]);
        sorter.setErasableHolderClass("erasable");
        sorter.setColumnType(2, TableSorter.TYPE_ERASABLE);
        sorter.setColumnType(4, TableSorter.TYPE_NUMBER);
        sorter.setColumnType(5, TableSorter.TYPE_NUMBER);
        sorter.setSortedBy(1, false);
        sorter.sortByColumn(1, false);
    }
});
var EditTransferView = Class.create(AdHocTransferView, {

    _constructor: function() {
        this.selectItemBoxes = Utils.getFormInputs(this.form, Tag.INPUT_TYPE_CHECKBOX, "file");

        var sorter = new TableSorter(this.table);
        sorter.setSortedColumns([1, 2, 3, 4, 5]);
        sorter.setErasableHolderClass("erasable");
        sorter.setColumnType(3, TableSorter.TYPE_NUMBER);
        sorter.setColumnType(4, TableSorter.TYPE_ERASABLE);
        sorter.setSortedBy(1, false);
        sorter.sortByColumn(1, false);
    }
});