var BrandingController = Class.create({

    view: null,

    _constructor: function() {
        this.view = new BrandingView();
    },

    start: function() {
        this.view.show(this);
    },

    stop: function() {
        this.view.hide();
    },

    onAddItem: function(view) {
        this._sendForm(this.view.addButton);
    },

    onDeleteItem: function(view, items) {
        if (items.length == 0) {
            return;
        }
        this.view.showConfirmDeleteDialog(items, this._handleConfirmDelete.bind(this));
    },

    _handleConfirmDelete: function(dialog) {
        dialog.dispose();

        this._sendForm(this.view.deleteButton);
    },

    _sendForm: function(button) {
        var form = this.view.form;
        var formAction = button && button.getAttribute("formaction");
        if (formAction) {
            form.action = formAction;
        }
        form.submit();
    }
});
var BrandingView = Class.create({

    form: null,
    table: null,
    addButton: null,
    deleteButton: null,
    selectAllBox: null,
    selectItemBoxes: null,

    addButtonHandler: null,
    deleteButtonHandler: null,
    selectAllBoxHandler: null,
    selectItemBoxHandler: null,

    _constructor: function() {
        this.form = $("itemsform");
        this.table = $("itemslist");
        this.addButton = $("additem");
        this.deleteButton = $("deleteitem");
        this.selectAllBox = $("selectall");
        this.selectItemBoxes = Utils.getFormInputs(this.form, Tag.INPUT_TYPE_CHECKBOX, "h");

        var sorter = new TableSorter(this.table);
        sorter.setSortedColumns([1, 2, 3]);
        sorter.setSortedBy(1, true);
        sorter.sortByColumn(1, true);
    },

    show: function(listener) {
        this.listener = listener || new BrandingView.Listener();
        this._update();
        this._bindEvents();
    },

    hide: function() {
        this.listener = null;
        this._unbindEvents();
    },

    showConfirmDeleteDialog: function(items, handler) {
        var buttons = {};
        buttons[BrandingView.Buttons.OK] = function(dialog) {handler.call(handler, dialog, items);};
        buttons[BrandingView.Buttons.CANCEL] = function(dialog) {dialog.dispose();};
        ModalDialog.confirm(document.title, BrandingView.Messages.CONFIRM_DELETE, buttons);
    },

    _getSelectedItems: function() {
        var items = [];
        for (var i = 0, size = this.selectItemBoxes.length; i < size; ++i) {
            var value = $F(this.selectItemBoxes[i]);
            if (value != null) {
                items.push(value);
            }
        }
        return items;
    },

    _update: function() {
        var count = this._getSelectedItems().length;
        this.selectAllBox.checked = (count != 0 && this.selectItemBoxes.length == count);
        Utils.setEnabled(this.deleteButton, count != 0);
    },

    _bindEvents: function () {
        this.addButtonHandler = this._handleAddItemPerformed.bindAsEventListener(this);
        this.deleteButtonHandler = this._handleDeleteItemPerformed.bindAsEventListener(this);
        this.selectAllBoxHandler = this._handleSelectAllPerformed.bind(this);
        Event.observe(this.addButton, "click", this.addButtonHandler);
        Event.observe(this.deleteButton, "click", this.deleteButtonHandler);
        Event.observe(this.selectAllBox, "click", this.selectAllBoxHandler);

        this.selectItemBoxHandler = this._update.bind(this);
        for (var i = 0; i < this.selectItemBoxes.length; ++i) {
            Event.observe(this.selectItemBoxes[i], "click", this.selectItemBoxHandler);
        }
    },

    _unbindEvents: function() {
        Event.unobserve(this.addButton, "click", this.addButtonHandler);
        Event.unobserve(this.deleteButton, "click", this.deleteButtonHandler);
        Event.unobserve(this.selectAllBox, "click", this.selectAllBoxHandler);
        for (var i = 0; i < this.selectItemBoxes.length; ++i) {
            Event.unobserve(this.selectItemBoxes[i], "click", this.selectItemBoxHandler);
        }
        this.addButtonHandler = null;
        this.deleteButtonHandler = null;
        this.selectAllBoxHandler = null;
        this.selectItemBoxHandler = null;
    },

    _handleSelectAllPerformed: function() {
        var selected = Boolean.toBoolean($F(this.selectAllBox));
        for (var i = 0, size = this.selectItemBoxes.length; i < size; ++i) {
            this.selectItemBoxes[i].checked = selected;
        }
        this._update();
    },

    _handleAddItemPerformed: function(e) {
        Event.stop(e);
        this.listener.onAddItem(this);
    },

    _handleDeleteItemPerformed: function(e) {
        Event.stop(e);
        this.listener.onDeleteItem(this, this._getSelectedItems());
    }
});
BrandingView.Buttons = {OK: "OK", CANCEL: "Cancel"};
BrandingView.Messages = {CONFIRM_DELETE: "Delete selected item(s)?"};
BrandingView.Listener = Class.create({
    onAddItem: function(view) {},
    onDeleteItem: function(view, items) {}
});