var DocViewerController = Class.create({

    url: null,
    view: null,

    _constructor: function(url) {
        this.url = url;
        this.view = new DocViewerView();
    },

    start: function(filename) {
        this.view.showDocument(this.url, filename);
    }
});

var DocViewerView = Class.create({

    showDocument: function(url, filename) {
        var dialog = new DocumentDialog();
        var viewport = document.viewport.getDimensions();
        dialog.setSize(viewport.width * 0.98, viewport.height * 0.93);

        dialog.show(url, {filename: filename}, {
            onClose: this._handleClose.bind(this),
            onError: this._handleError.bind(this)
        });
    },

    showError: function(message) {
        var html = "<div class=\"error-modal\">" + message + "</div>";
        ModalDialog.alert(html, DocViewerView.i18n.TITLE_ERROR);
    },

    _handleClose: function(dialog) {
        dialog.hide();
    },

    _handleError: function(dialog, message) {
        dialog.hide();
        this.showError(message || DocViewerView.i18n.LOAD_ERROR);
    }

});
DocViewerView.i18n = {TITLE_ERROR: "Error", TIMEOUT_ERROR: "Connection timed out.", LOAD_ERROR: "Error loading document."};

var DocumentDialog = Class.create(ModalDialog, {

    channel: null,
    closeButton: null,
    closeHandler: null,

    _constructor: function() {
        this.channel = new IFrameChannel();
        /** override */
        this.channel._handleLoadComplete = function() {
            this._stopTimeoutTimer();
            try {
                this._parseResponse(this._getContentDocument());
                this.listener.onComplete(this);
            } catch(e) {
                this.listener.onError(this);
            }
            this._disposeForm();
        };
        /** override */
        this.channel._appendIframe = function() {
            this.getContent().appendChild(this.channel.iframe);
        }.bind(this);
    },

    show: function(url, data, listener) {
        this.channel.open(url, "post", {
            onComplete: this._handleLoaded.bind(this),
            onTimeout: this._handleTimeout.bind(this),
            onError: this._handleError.bind(this)
        });

        this.listener = listener || new DocumentDialog.Listener();
        this._bindEvents();
        this._setCoverVisible(true);
        this.channel.send(data);
    },

    hide: function() {
        this.channel.close();
        this._super.hide.call(this);
        this.dispose();
    },

    onSubmit: function() {
        this.listener.onClose(this);
    },

    onCancel: function() {
        this.listener.onClose(this);
    },

    /** override */
    _compose: function() {
        this._super._compose.call(this);

        this.closeButton = document.createElement("DIV");
        Utils.setStyle(this.closeButton, {
            width:"24px",height:"24px",position:"absolute",top:"4px",right:0,
            background:"url(/img/close_16.png) no-repeat",
            border:"1px solid transparent",outline:0,opacity:0.5,cursor:"pointer"
        });
        this.titlePane.appendChild(this.closeButton);
        Utils.setVisible(this.buttonsBar, false);
    },

    /** override */
    _bindEvents: function() {
        this._super._bindEvents.call(this);

        this.closeHandler = this.onCancel.bind(this);
        Event.observe(this.closeButton, "click", this.closeHandler);
    },

    /** override */
    _unbindEvents: function() {
        Event.unobserve(this.closeButton, "click", this.closeHandler);
        this.closeHandler = null;

        this._super._unbindEvents.call(this);
    },

    _setCoverVisible: function(visible) {
        var displayValue = visible ? "block" : "none";

        if (this.shim != null) {
            this.shim.style.display = displayValue;
            if (this.isShimOverflowNavigator) {
                this._setShimOverflowTagsVisible(!visible);
            }
        }

        if (visible || ModalDialog._VISIBLE_DIALOGS.length == 0) {
            this.cover.style.display = displayValue;
        }

        if (this.isIENavigator) {
            document.getElementsByTagName("html")[0].style.overflow = visible ? "hidden" : "";
            this._setTabbedTagsEnabled(!visible);
        }
    },

    _handleLoaded: function() {
        var metadata = this.channel.getResponseMetadata();
        if ("done" == metadata["x-dialog-control"]) {
            Utils.setStyle(this.getContent(), {margin:0,padding:0});
            Utils.setStyle(this.channel.iframe, "width:100%;height:100%;visibility:visible;vertical-align:top;");
            this.setVisible(true);
        } else if ("error" == metadata["x-dialog-control"]) {
            this.listener.onError(this, metadata["x-dialog-reason"]);
        } else {
            this.listener.onError(this);
        }
    },

    _handleTimeout: function() {
        this.listener.onError(this, DocViewerView.i18n.TIMEOUT_ERROR);
    },

    _handleError: function() {
        this.listener.onError(this, DocViewerView.i18n.LOAD_ERROR);
    }

});
DocumentDialog.Listener = Class.create(ModalDialog, {
    onClose: function(d) {},
    onError: function(d, message) {}
});