var DateEditor = Class.create(NumberEditor, {

    format: null,
    lastValue: null,
    time: null,

    _constructor: function(control, format) {
        this.format = new DateFormat(format);
        this.control.size = Math.max(0, Math.min(this.control.size, this.format.getPattern().length + 1));

        var date;
        try {
            date = this.format.parse(this.control.value);
        } catch(e) {
            date = new Date();
        }

        this.setValue(date.getTime());
        this.setDefaultValue(date.getTime());

        this._updateAllowedChars();
    },

    getValue: function() {
        return this.time;
    },

    setValue: function(time) {
        this.time = Math.max(this.minValue, Math.min(this.maxValue, time));
        this.control.value = this.format.format(new Date(this.time));
        this.lastValue = this.control.value;
    },

    getNextValue: function() {
        var date = new Date(this.time);
        return date.setDate(date.getDate() + 1);
    },

    getPreviousValue: function() {
        var date = new Date(this.time);
        return date.setDate(date.getDate() - 1);
    },

    /* override */
    _handleKeyUp: function(e) {
        this._super._handleKeyUp.call(this, e);
        if (!e.cancelBubble && this.control.value != this.lastValue) {
            try {
                this.time = this.format.parse(this.control.value).getTime();
            } catch(e) {
                this.time = -1;
            }
            this.lastValue = this.control.value;
        }
    },

    _updateAllowedChars: function() {
        var chars = this.format.getCharSequence();

        for (var i = 0; i < chars.length; ++i) {
            var c = chars.charAt(i);
            this.addAllowedChar(c.toLowerCase());
            this.addAllowedChar(c.toUpperCase());
        }
    }

});

