var DateFormat = Class.create({

    pattern: null,

    _constructor: function(pattern) {
        this.pattern = pattern || "MMM dd yyyy HH:mm:ss";
    },

    getPattern: function() {
        return this.pattern;
    },

    getCharSequence: function() {
        var text = this.format(new Date());

        if (/M{4,}/.test(this.pattern)) {
            text += DateFormat.MONTH_NAMES.join("");
        } else if (/M{3}/.test(this.pattern)) {
            text += DateFormat.MONTH_ABBRS.join("");
        }

        return text;
    },

    parse: function(str) {
        if (!Object.isString(this.pattern) || !Object.isString(str) || str.trim().length == 0) {
            throw "Error parsing date.";
        }

        var patternLength = function(pattern, data) {
            var result = new RegExp(pattern + "{1,}", "g").exec(data);
            return result != null ? result[0].length : 0;
        };

        var curDate = new Date();
        var year = curDate.getFullYear();
        var month = curDate.getMonth() + 1;
        var day = curDate.getDate();
        var hour = 0, minute = 0, seconds = 0;

        for (var i = 0, position = {offset: 0, length: 0}; i < this.pattern.length; i += position.length) {
            switch (this.pattern.charAt(i)) {
                case 'y': {
                    position.length = patternLength('y', this.pattern);
                    year = this._parseYear(str, position);
                    break;
                }
                case 'M':
                    position.length = patternLength('M', this.pattern);
                    month = this._parseMonth(str, position);
                    break;
                case 'd':
                    position.length = patternLength('d', this.pattern);
                    day = this._parseNumber(str, position);
                    break;
                case 'H':
                    position.length = patternLength('H', this.pattern);
                    hour = this._parseNumber(str, position);
                    break;
                case 'm':
                    position.length = patternLength('m', this.pattern);
                    minute = this._parseNumber(str, position);
                    break;
                case 's':
                    position.length = patternLength('s', this.pattern);
                    seconds = this._parseNumber(str, position);
                    break;
                default:
                    position.offset++;
                    position.length = 1;
                    break;
            }
        }

        var date = new Date(year, month - 1, day, hour, minute, seconds);
        var isRecalculated = (date.getFullYear() != year || date.getMonth() != month - 1 || date.getDate() != day);

        if (!isRecalculated) {
            return date;
        }

        throw "Error parsing date: invalid date."
    },

    format: function(date) {
        var str = this._formatYear(date, this._formatMonth(date, this.pattern));
        return str.replace(/d{1,}/, this._zeroPaddedNumber(date.getDate()))
                .replace(/H{1,}/, this._zeroPaddedNumber(date.getHours()))
                .replace(/m{1,}/, this._zeroPaddedNumber(date.getMinutes()))
                .replace(/s{1,}/, this._zeroPaddedNumber(date.getSeconds()));
    },

    _parseNumber: function(data, position) {
        var match;
        if (position.length == 1) {
            match = /^\d\d?/.exec(data.substring(position.offset));
        } else {
            var re = new RegExp("^\\d{1," + position.length + "}");
            match = re.exec(data.substr(position.offset, position.length));
        }

        if (match != null) {
            position.offset += match[0].length;
            return parseInt(match[0], 10);
        }

        throw "Illegal argument";
    },

    _parseMonth: function(data, position) {
        if (position.length <= 2) {
            return this._parseNumber(data, position);
        }

        var monthName = data.substr(position.offset, position.length);
        return (position.length == 3) ? DateFormat.MONTH_ABBRS.indexOf(monthName) : DateFormat.MONTH_NAMES.indexOf(monthName);
    },

    _parseYear: function(data, position) {
        var year = this._parseNumber(data, position);
        // is ambiguous year
        if (position.length <= 2 || year < 100) {
            year += new Date().getFullYear() - (new Date().getFullYear() % 100);
        }
        return year;
    },

    _formatYear: function(date, str) {
        var re = /y{1,}/;
        if (re.test(str)) {
            var count = RegExp.lastMatch.length;
            var text = (count >= 4) ? date.getFullYear() : String(date.getFullYear()).substr(2);
            str = str.replace(re, text);
        }
        return str;
    },

    _formatMonth: function(date, str) {
        var re = /M{1,}/;
        if (re.test(str)) {
            var count = RegExp.lastMatch.length;
            var value = date.getMonth();
            var text = (count >= 4)
                    ? DateFormat.MONTH_NAMES[value]
                    : (count == 3 ? DateFormat.MONTH_ABBRS[value] : this._zeroPaddedNumber(value + 1));
            str = str.replace(re, text);
        }
        return str;
    },

    _zeroPaddedNumber: function(value) {
        var str = '0' + value;
        return str.substring(str.length - 2);
    }
});

DateFormat.MONTH_NAMES = "January February March April May June July August September October November December".split(/\s+/);
DateFormat.MONTH_ABBRS = "Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec".split(/\s+/);