var NumberEditor = Class.create({

    control: null,
    minValue: null,
    maxValue: null,
    defaultValue: null,
    allowedChars: null,

    _constructor: function(control) {
        this.control = control;
        this.control.setAttribute("autocomplete", "off");
        this.allowedChars = [''];
        this.minValue = Number.MIN_VALUE;
        this.maxValue = Number.MAX_VALUE;
        this.defaultValue = this.minValue;

        Event.observe(this.control, "keydown", this._handleKeyDown.bindAsEventListener(this));
        Event.observe(this.control, "keypress", this._handleKeyPress.bindAsEventListener(this));
        Event.observe(this.control, "keyup", this._handleKeyUp.bindAsEventListener(this));
    },

    addAllowedChar: function(c) {
        if (this.allowedChars.indexOf(c) == -1 && !this._isDigit(c.charCodeAt(0))) {
            this.allowedChars.push(c);
        }
    },

    setMinValue: function(value) {
        this.minValue = parseInt(value, 10);
    },

    setMaxValue: function(value) {
        this.maxValue = parseInt(value, 10);
    },

    getValue: function() {
        var result = parseInt(this.control.value, 10);
        return isNaN(result) ? 0 : result;
    },

    setValue: function(value) {
        this.control.value = Math.max(this.minValue, Math.min(this.maxValue, parseInt(value, 10)));
    },

    getNextValue: function() {
        return this.getValue() + 1;
    },

    getPreviousValue: function() {
        return this.getValue() - 1;
    },

    getDefaultValue: function() {
        return this.defaultValue;
    },

    setDefaultValue: function(value) {
        this.defaultValue = value;
    },

    toString: function() {
        return this.control.value;
    },

    _handleKeyDown: function(e) {
        var ke = new KeyEvent(e);
        if (ke.isModifiersDown()) {
            return;
        }

        var code = ke.getKeyCode();
        if (code == KeyEvent.VK_DOWN || code == KeyEvent.VK_PAGE_DOWN) {
            this.setValue(this.getPreviousValue());
            Event.stop(e);
        } else if (code == KeyEvent.VK_UP || code == KeyEvent.VK_PAGE_UP) {
            this.setValue(this.getNextValue());
            Event.stop(e);
        } else if (code == KeyEvent.VK_ESCAPE) {
            this.setValue(this.getDefaultValue());
        }
    },

    _handleKeyUp: function(e) {
    },

    _handleKeyPress: function(e) {
        var ke = new KeyEvent(e);
        var code = ke.getKeyCode();

        if (ke.isModifiersDown() || !(this._isAllowedKey(code) || this._isAllowedChar(ke.getChar()) || this._isDigit(code))) {
            Event.stop(e);
        }
    },

    _isDigit: function(code) {
        return code >= 48 && code <= 57;
    },

    _isAllowedChar: function(c) {
        for (var i = 0; i < this.allowedChars.length; ++i) {
            if (c == this.allowedChars[i]) {
                return true;
            }
        }

        return false;
    },

    _isAllowedKey: function(code) {
        return NumberEditor.ALLOWED_KEY_CODES.indexOf(code) != -1;
    }

});

NumberEditor.ALLOWED_KEY_CODES = [
    KeyEvent.VK_ENTER,
    KeyEvent.VK_ESCAPE,
    KeyEvent.VK_LEFT,
    KeyEvent.VK_RIGHT,
    KeyEvent.VK_UP,
    KeyEvent.VK_DOWN,
    KeyEvent.VK_PAGE_UP,
    KeyEvent.VK_PAGE_DOWN,
    KeyEvent.VK_BACK_SPACE,
    KeyEvent.VK_DELETE,
    KeyEvent.VK_TAB
];