var RenameFileDialog = Class.create({

    textFiled: null,
    container: null,
    keyboardHandler: null,
    focusHandler: null,
    listener: null,

    _constructor: function() {
        this.textField = Utils.createTextInput("renameFileInputField");
        Utils.setStyle(this.textField, {border: "1px solid #000", font: "92% Arial, Helvetica, sans-serif"});
        this.container = document.createElement("div");
    },

    getOldFilename: function() {
        return this.textField.defaultValue;
    },

    getNewFilename: function() {
        return this.textField.value;
    },

    show: function(filename, fileNode, listener) {
        this.listener = listener || new RenameFileDialog.Listener();
        this._attachEvents();
        this._position(fileNode);
        this._initInputText(filename);
    },

    hide: function() {
        this._detachEvents();
        this.dispose();
    },

    dispose: function() {
        if (this.textField.parentNode) {
            this.textField.parentNode.removeChild(this.textField);
        }
        if (this.container.parentNode) {
            this.container.parentNode.removeChild(this.container);
        }
    },

    _handleKeyBinding: function(e) {
        var ke = new KeyEvent(e);
        switch (ke.getKeyCode()) {
            case KeyEvent.VK_ENTER:
                this.listener.onSubmit(this);
                break;
            case KeyEvent.VK_UP: // fall through, todo:
            case KeyEvent.VK_DOWN: // fall through, todo:
            case KeyEvent.VK_ESCAPE:
                this.listener.onCancel(this);
                break;
        }
    },

    _handleFocusLost: function() {
        this.listener.onCancel(this);
    },

    _initInputText: function(text) {
        this.textField.defaultValue = text;
        this.textField.value = text;
        this.textField.select();
        this.textField.focus();
    },

    _position: function(fileNode) {
        this.container.appendChild(this.textField);
        Utils.setStyle(this.container, {width: Utils.getDimension(fileNode).width+"px", display: "block", position: "absolute", zIndex: 2147483392});
        Utils.setStyle(this.textField, "width: 100%; padding-left: 2px; margin: -1px 0 0 20px; cursor: text !important;");

        fileNode.parentNode.insertBefore(this.container, fileNode);
    },

    _attachEvents: function() {
        this.keyboardHandler = this._handleKeyBinding.bindAsEventListener(this);
        this.focusHandler = this._handleFocusLost.bind(this);
        Event.observe(this.textField, "keydown", this.keyboardHandler);
        Event.observe(this.textField, "blur", this.focusHandler);
    },

    _detachEvents: function() {
        Event.unobserve(this.textField, "keydown", this.keyboardHandler);
        Event.unobserve(this.textField, "blur", this.focusHandler);
        this.keyboardHandler = null;
        this.focusHandler = null;
    }

});

RenameFileDialog.Listener = Class.create({
    onSubmit: function(view) {},
    onCancel: function(view) {}
});