var StorageController = Class.create({

    config: null,
    view: null,
    uploader: null,
    docViewer: null,

    _constructor: function(config) {
        this.config = Object.extend(Object.extend({}, StorageController.DEFAULT_CONFIG), config);
        this.view = new StorageView(this.config);
        if (config.uploadAllowed) {
            this.uploader = new Uploader(config);
        }

        var form = $("actionform");
        this.downloadFileAction = new Action("dload", form);
        this.downloadZipAction = new Action("dloadzip", form);
        this.renameFileAction = new Action("rename", form);
        this.refreshAction = new Action("refresh", form);
        this.changeDirUpAction = new Action("cdup", form);
        this.changeWorkingDirAction = new Action("cwd", form);
        this.createDirAction = new Action("mkdir", form);
        this.deleteFileAction = new Action("delete", form);
        this.moveFileAction = new Action("move", form);
        this.tagFileAction = new Action("ManageTags", form);
        this.cancelUploadAction = new Action("", form);
        this.switchUiAction = new Action("switchui", form);
    },

    start: function() {
        if (this.uploader) {
            this.uploader.start({
                onStart: this.onStartUpload.bind(this),
                onComplete: this.onCompleteUpload.bind(this),
                onTimeout: this.onTimeoutUpload.bind(this)
            });
        }
        this.view.show(this);
    },

    onStartUpload: function() {
        this.view.hideMessage();
        this._stopInactivityService();
    },

    onCompleteUpload: function() {
        var error = this.uploader.getTransferError();
        if (error) {
            //todo: this.view.showTransferError(error);
        }
        // delay to complete all active work
        (function(action, parameters) {
            action.perform(parameters);
        }.delay(300, this.refreshAction, this._actionParameters()));
    },

    onTimeoutUpload: function() {
        //todo: this.view.showConnectionTimeoutMessage();
        this.refreshAction.perform(this._actionParameters());
    },

    onRenameFile: function(view) {
        var items = view.getSelectedItems();

        if (items.length == 0) {
            view.showEmptySelectionMessage();
        } else if (items.length == 1) {
            var dialog = new RenameFileDialog();
            dialog.show(items.first().path, items.first().node, {
                onSubmit: this._onSubmitRenameFileDialog.bind(this),
                onCancel: function(d) { d.hide(); }
            });
        }
    },

    onMoveFile: function(view, data) {
        this.moveFileAction.perform(Object.extend(this._actionParameters(), {
            filename: data.files,
            destination: data.destination
        }));
    },

    onNewDir: function() {
        var dialog = new NewDirectoryDialog();
        dialog.show({onCreate: this._onCreateNewDirectory.bind(this)});
    },

    onChangeDir: function() {
        var dialog = new ChangeDirectoryDialog();
        dialog.show({onChange: this._onChangeDirectory.bind(this)});
    },

    onChangeWorkingDir: function(view, path) {
        if (path == "..") {
            this.changeDirUpAction.perform(this._actionParameters());
        } else {
            this.changeWorkingDirAction.perform(this._actionParameters(path));
        }
    },

    onDownloadFile: function(view, path) {
        this.view.hideMessage();
        this.downloadFileAction.perform(this._actionParameters(path));
    },

    onDownloadZip: function(view, items) {
        if (items.length == 0) {
            view.showEmptySelectionMessage();
        } else {
            this.downloadZipAction.perform(this._actionParameters(this._itemsAsPaths(items)));
        }
    },

    onDeleteFile: function(view) {
        var items = view.getSelectedItems();
        if (items.length == 0) {
            view.showEmptySelectionMessage();
        } else {
            var files = this._itemsAsPaths(items);
            this.view.showConfirmDeleteFile(files, this._onConfirmDeleteFile.bind(this, files));
        }
    },

    onEmailFile: function(view) {
        var items = view.getSelectedItems();
        if (items.length == 0) {
            view.showEmptySelectionMessage();
        } else {
            // todo:
            var paths = this._itemsAsPaths(items);
            if (window.adHocController) {
                window.adHocController.compose(paths);
            }
        }
    },

    onTagFile: function(view) {
        var items = view.getSelectedItems();
        if (!this.config.tagFileAllowed) {
            return;
        }
        if (items.length == 0) {
            view.showEmptySelectionMessage();
            return;
        }

        this.view.hideMessage();
        var dialog = new FormDialog("tag-file-dialog", this.config.mtu);
        dialog.setSize(470, 225);

        var formData = Object.extend({filename: this._itemsAsPaths(items)}, this.config.metadata);
        dialog.show(Object.extend(new FormDialog.Listener(), {
            onDone: this._onDoneFileTagsDialog.bind(this),
            onCancel: function(d) { d.dispose(); }
        }), formData);
    },

    onViewFile: function(view) {
        if (!this.config.viewFileAllowed) {
            return;
        }
        var items = view.getSelectedItems();
        if (items.length == 0) {
            view.showEmptySelectionMessage();
            return;
        }
        if (this.docViewer == null) {
            this.docViewer = new DocViewerController(this.config.dvu);
        }

        var file = this._itemsAsPaths(items).first();
        this.docViewer.start(file);
    },

    onRefreshView: function(view) {
        this.refreshAction.perform(this._actionParameters());
    },

    onSwitchView: function(view) {
        this.switchUiAction.perform();
    },

    _onChangeFileSelection: function() {
        this.view.update();
    },

    _onCreateNewDirectory: function(dialog, path) {
        dialog.hide();
        if (path && path.trim().length != 0) {
            this.createDirAction.perform(this._actionParameters(path));
        }
    },

    _onChangeDirectory: function(dialog, path) {
        dialog.dispose();
        if (path && path.trim().length != 0) {
            this.changeWorkingDirAction.perform(this._actionParameters(path.replace("\\", "/")));
        }
    },

    _onConfirmDeleteFile: function(paths) {
        this.deleteFileAction.perform(this._actionParameters(paths));
    },

    _onSubmitRenameFileDialog: function(dialog) {
        var oldFilename = dialog.getOldFilename();
        var newFilename = dialog.getNewFilename();
        dialog.hide();

        if (oldFilename !== newFilename) {
            this.renameFileAction.perform(Object.extend(this._actionParameters(), {
                oldname: oldFilename,
                newname: newFilename
            }));
        }
    },

    _onDoneFileTagsDialog: function(dialog) {
        dialog.dispose();
        //todo:
        this.refreshAction.perform(this._actionParameters());
    },

    _itemsAsPaths: function(items) {
        var result = [];
        for (var i = 0; i < items.length; ++i) {
            result.push(items[i].path);
        }
        return result;
    },

    //todo:
    _actionParameters: function(filenames) { return window.createParameters ? window.createParameters(filenames) : {}; },
    _stopInactivityService: function() { window.stopInactivityService && window.stopInactivityService(); }
});

StorageController.DEFAULT_CONFIG = {
    mtu: "", dvu: "", usu: "", ufu: "", upu: "", uploadForms: [], uploadFormRequired: false, uploadAllowed: true,
    registerKeystroke: false, adHocTransferEnabled: false, adHocTransferAllowed: false, viewFileAllowed: false, tagFileAllowed: false,
    connectionTimeout: 60000
};