var StorageView = Class.create({

    fileListView: null,
    messageBar: null,
    adHocTransferEnabled: false,
    disabledCssClass: null,

    listener: null,

    _constructor: function(config) {
        this.renameFileButton = this._createButton($("renamebtn"), this._handleRenameFile, KeyEvent.VK_F2, config.registerKeystroke);
        this.refreshButton = this._createButton($("refreshbtn"), this._handleRefreshView, KeyEvent.VK_F5, config.registerKeystroke);
        this.newDirectoryButton = this._createButton($("newdirbtn"), this._handleNewDirectory, KeyEvent.VK_F7, config.registerKeystroke);
        this.dloadZipButton = this._createButton($("dloadzipbtn"), this._handleDownloadZip, null, config.registerKeystroke);
        this.changeDirectoryButton = this._createButton($("changedirbtn"), this._handleChangeDir, null, config.registerKeystroke);
        this.deleteFileButton = this._createButton($("deletebtn"), this._handleDeleteFile, KeyEvent.VK_F8, config.registerKeystroke);
        this.switchUiButton = this._createButton($("switchuibtn"), this._handleSwitchView);

        if (config.adHocTransferAllowed) {
            this.emailFileButton = this._createButton($("emailbtn"), this._handleEmailFile);
        }
        if (config.viewFileAllowed) {
            this.viewFileButton = this._createButton($("viewdocbtn"), this._handleViewFile);
        }
        if (config.tagFileAllowed) {
            this.tagFileButton = this._createButton($("managetagsbtn"), this._handleTagFile);
        }
        this.adHocTransferEnabled = config.adHocTransferEnabled || false;
        this.disabledCssClass = config.disabledCssClass || "disabled";

        this.fileListView = new FileListView($("list"), $("selectall"));
        this.messageBar = $("messagebar");
    },

    getSelectedItems: function() {
        return this.fileListView.getSelectedItems();
    },

    getSelectedFile: function() {
        // todo:
        return window.getSelectedFile();
    },

    show: function(listener) {
        this.listener = listener || new StorageView.Listener();
        this.fileListView.show({
            onChangeDir: this._handleChangeWorkingDir.bind(this),
            onDloadFile: this._handleDownloadFile.bind(this),
            onMoveFile: this._handleMoveFile.bind(this),
            onSelectionChanged: this.update.bind(this)
        });

        this.update();
    },

    hide: function() {
        this.fileListView.hide();
        this.listener = null;
    },

    showMessage: function(text) {
        ModalDialog.alert(text);
    },

    hideMessage: function() {
        if (Object.isElement(this.messageBar)) {
            Utils.setVisible(this.messageBar, false);
        }
    },

    showEmptySelectionMessage: function() {
        ModalDialog.alert(Messages.NO_FILES_SELECTED);
    },

    showConfirmDeleteFile: function(files, handler) {
        ModalDialog.confirm(document.title, Messages.CONFIRM_DELETE, {
            'OK': function(dialog) {
                dialog.dispose();
                handler.call(handler);
            },
            'Cancel': function(dialog) {
                dialog.dispose();
            }
        });
    },

    _createButton: function(element, handler, keyCode, registerKeystroke) {
        if (element != null) {
            if (keyCode && registerKeystroke) {
                element.processKeyEvent = handler.bindAsEventListener(this);
                Event.observe(element, "click", element.processKeyEvent);
                KeyboardManager.getInstance().registerHandler(keyCode, element);
            } else {
                Event.observe(element, "click", handler.bindAsEventListener(this));
            }
        }

        return element;
    },

    update: function() {
        var items = this.getSelectedItems();
        var containsParentDir = this._containsParentDirItem(items);
        var singleSelection = items.length == 1 && !containsParentDir;
        var multipleSelection = items.length > 0 && !containsParentDir;

        Utils.setEnabled(this.viewFileButton, singleSelection && items[0].access.view, this.disabledCssClass);
        Utils.setEnabled(this.renameFileButton, singleSelection, this.disabledCssClass);
        Utils.setEnabled(this.dloadZipButton, multipleSelection, this.disabledCssClass);
        Utils.setEnabled(this.deleteFileButton, multipleSelection, this.disabledCssClass);
        Utils.setEnabled(this.emailFileButton, this.adHocTransferEnabled && multipleSelection, this.disabledCssClass);
        Utils.setEnabled(this.tagFileButton, singleSelection || multipleSelection, this.disabledCssClass);
    },

    _handleRenameFile: function(e) {
        this._stopEventPropagation(e);
        this.listener.onRenameFile(this);
    },

    _handleMoveFile: function(files, destination) {
        this.listener.onMoveFile(this, {files: files, destination: destination});
    },

    _handleNewDirectory: function(e) {
        this._stopEventPropagation(e);
        this.listener.onNewDir(this);
    },

    _handleChangeDir: function(e) {
        this._stopEventPropagation(e);
        this.listener.onChangeDir(this);
    },

    _handleChangeWorkingDir: function(path) {
        this.listener.onChangeWorkingDir(this, path);
    },

    _handleDownloadFile: function(path) {
        this.listener.onDownloadFile(this, path);
    },

    _handleDownloadZip: function(e) {
        this._stopEventPropagation(e);
        this.listener.onDownloadZip(this, this.getSelectedItems());
    },

    _handleDeleteFile: function(e) {
        this._stopEventPropagation(e);
        this.listener.onDeleteFile(this);
    },

    _handleEmailFile: function(e) {
        this._stopEventPropagation(e);
        this.listener.onEmailFile(this);
    },

    _handleTagFile: function(e) {
        this._stopEventPropagation(e);
        this.listener.onTagFile(this);
    },

    _handleViewFile: function(e) {
        this._stopEventPropagation(e);
        this.listener.onViewFile(this);
    },

    _handleRefreshView: function(e) {
        this._stopEventPropagation(e);
        this.listener.onRefreshView(this);
    },

    _handleSwitchView: function(e) {
        this._stopEventPropagation(e);
        this.listener.onSwitchView(this);
    },

    _containsParentDirItem:function (items) {
        for (var i = 0; i < items.length; ++i) {
            if (items[i].path == "..") {
                return true;
            }
        }
        return false;
    },

    _bindEvents: function() {

    },

    _unbindEvents: function() {

    },

    _stopEventPropagation: function(e) {
        if (e instanceof KeyEvent) {
            e.cancel();
        } else {
            Event.stop(e);
        }
    }

});

StorageView.Listener = Class.create({
    onRenameFile: function(view) {},
    onMoveFile: function(view, data) {},
    onNewDir: function(view) {},
    onChangeDir: function(view) {},
    onChangeWorkingDir: function(view, path) {},
    onDownloadFile: function(view, path) {},
    onDownloadZip: function(view, items) {},
    onDeleteFile: function(view) {},
    onEmailFile: function(view) {},
    onTagFile: function(view) {},
    onViewFile: function(view) {},
    onRefreshView: function(view) {},
    onSwitchView: function(view) {}
});