var FormDialog = Class.create(ModalDialog, {

    form: null,
    formSubmitHandler: null,
    channel: null,
    listener: null,

    responseMetadata: null,

    _constructor: function(id, formUrl) {
        this.id = id;
        this.channel = new IFrameChannel({preventCaching: true});
        this.channel.open(formUrl, "post", Object.extend(new Channel.Listener(), {
            onComplete: this._handleFormLoaded.bind(this)
        }));
        this.responseMetadata = {};

        this.addButton(ModalDialog.BUTTON_OK, FormDialog.Buttons.OK, this._handleSubmitPressed.bind(this));
        this.addButton(ModalDialog.BUTTON_CANCEL, FormDialog.Buttons.CANCEL, this._handleCancelPressed.bind(this));
        this.setTitle(FormDialog.i18n.LOADING_TITLE);
    },

    getForm: function() {
        return this.form;
    },

    show: function(listener, formData) {
        this.form = null;
        this.listener = listener || new FormDialog.Listener();

        this.setButtonEnabled(ModalDialog.BUTTON_OK, false);
        this._bindEvents();
        this.setVisible(true);
        this._sendFormData(formData);
    },

    dispose: function() {
        this.form = null;
        this.channel.close();
        this._super.dispose.call(this);
    },

    getDoneMessage: function() {
        return this.getMetaValue("x-dialog-reason") || "";
    },

    isDone: function() {
        return "done" == this.getMetaValue("x-dialog-control");
    },

    getMetaValue: function(key) {
        return key in this.responseMetadata ? this.responseMetadata[key] : null;
    },

    _sendFormData: function(data) {
        this.channel.send(data || {});
    },

    _handleSubmitPressed: function() {
        this.setButtonEnabled(ModalDialog.BUTTON_OK, false);

        if (this.listener.onBeforeSubmit(this) && this.form != null) {
            this.channel.submitForm(this.form);
            this.listener.onSubmit(this);
        } else {
            this.setButtonEnabled(ModalDialog.BUTTON_OK, true);
        }
    },

    _handleCancelPressed: function() {
        this.setVisible(false);
        this.channel.close();

        this.listener.onCancel(this);
    },

    _handleFormLoaded: function() {
        this.responseMetadata = this.channel.getResponseMetadata();

        if (this.isDone()) {
            this.listener.onDone(this);
        } else {
            if (this._isRequestedFormLoaded()) {
                this._initForm();
                this.listener.onLoaded(this);
            } else {
                this._showLoadErrorMessage();
                this.listener.onLoadError(this);
            }
        }
    },

    _isRequestedFormLoaded: function() {
        var doc = this.channel.getResponseDocument();
        var channelUrl = this.channel.getUrl();
        channelUrl = channelUrl.replace(/\?.*/, "");
        var documentUrl = channelUrl.match(/^http/i) ? doc.location.href : doc.location.pathname;
        return doc.forms.length != 0 && documentUrl.indexOf(channelUrl) == 0;
    },

    _initForm: function() {
        this._updateTitle(this.channel.getResponseDocument().title);
        this.getContent().innerHTML = this.channel.getResponseHtml();
        var forms = this.getContent().getElementsByTagName("form");

        if (forms.length != 0) {
            this.form = forms[0];

            this.setButtonEnabled(ModalDialog.BUTTON_OK, true);
            this._bindEvents();
        } else {
            this.form = null;
        }
    },

    _showLoadErrorMessage: function() {
        this.channel.close();

        var el = document.createElement("DIV");
        el.innerHTML = FormDialog.i18n.LOAD_ERROR_MESSAGE;
        Utils.setStyle(el, "text-align:center;padding:1em");
        this.setContent(el);
        this.setTitle(FormDialog.i18n.LOAD_ERROR_TITLE);

        this.listener.onLoadError(this);
    },

    _bindEvents: function() {
        this._super._bindEvents.call(this);

        if (this.form) {
            this.formSubmitHandler = this._handleSubmitPressed.bind(this);
            Event.observe(this.form, "submit", this.formSubmitHandler);
        }
    },

    _unbindEvents: function() {
        if (this.form) {
            Event.unobserve(this.form, "submit", this.formSubmitHandler);
            this.formSubmitHandler = null;
        }

        this._super._unbindEvents.call(this);
    },

    _updateTitle: function(title) {
        title = title || document.title;
        var curTitle = Utils.getInnerText(this.titlePane);

        if (title && title !== curTitle) {
            this.setTitle(title);
        }
    }

});

FormDialog.i18n = {LOADING_TITLE: "Loading...", LOAD_ERROR_TITLE: "Error", LOAD_ERROR_MESSAGE: "The requested content cannot be loaded.<br/>Please try again later."};
FormDialog.Buttons = {OK: "OK", CANCEL: "Cancel"};
FormDialog.Listener = Class.create(ModalDialog.Listener, {
    onLoaded: function(dialog) {},
    onLoadError: function(dialog) {},
    onBeforeSubmit: function(dialog) { return true; },
    onDone: function(dialog) {}
});