var InlineDialog = Class.create({

    id: null,
    dialog: null,
    contentPane: null,
    titlePane: null,
    closeBtn: null,
    closeHandler: null,

    listener: null,

    _constructor: function(id, parent) {
        this.id = id || "InlineDialog" + new Date().getTime();
        this._init(parent || document.body);
    },

    show: function(listener) {
        this.listener = listener || new InlineDialog.Listener();
        this._bindEvents();

        Utils.setStyle(this.dialog, "visibility: visible");
    },

    hide: function() {
        this.listener = null;
        this._unbindEvents();
        this.dispose();
    },

    dispose: function() {
        if (this.dialog && this.dialog.parentNode) {
            this.dialog.parentNode.removeChild(this.dialog);
        }
    },

    setTitle: function(value) {
        var newNode = document.createTextNode(value || "");
        var currentNode = Utils.getTextNode(this.titlePane);

        if (currentNode) {
            this.titlePane.replaceChild(newNode, currentNode);
        } else {
            this.titlePane.appendChild(newNode);
        }
    },

    append: function() {
        var fragment = document.createDocumentFragment();
        var content = document.createElement("DIV");
        content.innerHTML = $A(arguments).join("");

        if (content.hasChildNodes()) {
            var children = content.childNodes;

            for (var i = 0; i < children.length; ++i) {
                fragment.appendChild(children[i].cloneNode(true));
            }
            this.contentPane.appendChild(fragment);
        }
    },

    _init: function(parent) {
        this._initDialog(parent);
    },

    _initDialog: function(parent) {
        this.dialog = document.createElement("DIV");
        this.titlePane = this.dialog.appendChild(document.createElement("DIV"));
        this.closeBtn = this.titlePane.appendChild(document.createElement("SPAN"));
        this.contentPane = this.dialog.appendChild(document.createElement("DIV"));

        this.dialog.id = this.id;
        Utils.setStyle(this.dialog, "visibility: hidden");
        Utils.addCssClass(this.dialog, "dialog");
        Utils.addCssClass(this.dialog, "inline");
        Utils.addCssClass(this.dialog, "shadow");
        Utils.addCssClass(this.titlePane, "title-bar");
        Utils.addCssClass(this.closeBtn, "closebtn");
        Utils.addCssClass(this.contentPane, "content-pane");

        parent.appendChild(this.dialog);
    },

    _handleClose: function(e) {
        Event.stop(e);
        this.listener.onCancel(this);
    },

    _bindEvents: function() {
        this.closeHandler = this._handleClose.bindAsEventListener(this);
        Event.observe(this.closeBtn, "click", this.closeHandler);
    },

    _unbindEvents: function() {
        Event.unobserve(this.closeBtn, "click", this.closeHandler);
        this.closeHandler = null;
    }

});

InlineDialog.Listener = Class.create({
    onSubmit: function (dialog) {},
    onCancel: function (dialog) { dialog.hide(); },
    onDone: function (dialog) { dialog.hide(); }
});