var ManagePathsPane = Class.create({

    itemBoxes: null,
    itemOrderInput: null,
    itemOrderSeparator: null,
    selectAllBox: null,
    addButton: null,
    editButton: null,
    deleteButton: null,
    moveUpButton: null,
    moveDownButton: null,

    _constructor: function(config) {
        this.itemBoxes = config.pathBoxes;
        this.itemOrderInput= config.pathOrderInput;
        this.itemOrderSeparator = config.pathOrderSeparator;
        this.selectAllBox = config.selectAllBox;
        this.addButton = config.addButton;
        this.editButton = config.editButton;
        this.deleteButton = config.deleteButton;
        this.moveUpButton = config.upButton;
        this.moveDownButton = config.downButton;

        for (var i = 0, size = this.itemBoxes.length; i < size; ++i) {
            Event.observe(this.itemBoxes[i], "click", this._update.bind(this));
        }

        Event.observe(this.selectAllBox, "click", this._handleSelectAll.bind(this));
        Event.observe(this.addButton, "click", this._handleAdd.bind(this));
        Event.observe(this.editButton, "click", this._handleEdit.bindAsEventListener(this));
        Event.observe(this.deleteButton, "click", this._handleDelete.bindAsEventListener(this));
        Event.observe(this.moveUpButton, "click", this._handleMoveUp.bindAsEventListener(this));
        Event.observe(this.moveDownButton, "click", this._handleMoveDown.bindAsEventListener(this));
        Event.observe(this.itemOrderInput.form, "submit", this._updateItemOrder.bind(this));

        this._update();
        this._updateItemOrder();
    },

    _handleSelectAll: function() {
        var selected = this.selectAllBox.checked;
        for (var i = 0, size = this.itemBoxes.length; i < size; ++i) {
            this.itemBoxes[i].checked = selected;
        }
        this._update();
    },

    _handleAdd: function() {
        // do nothing
    },

    _handleEdit: function(e) {
        if (this._getSelectedIndex() == -1) {
            Event.stop(e);
        }
    },

    _handleDelete: function(e) {
        if (this._getSelectedCount() == 0) {
            Event.stop(e);
        } else {
            this._doDeleteLocal.bind(this).delay(100);
        }
    },

    _handleMoveUp: function(e) {
        var index = this._getSelectedIndex();
        if (this._getSelectedCount() != 1 || index == 0) {
            Event.stop(e);
        } else {
            this._swap(this.itemBoxes, index - 1, index);
            this._doOrderLocal.bind(this, index - 1, index).delay(50);
        }
    },

    _handleMoveDown: function(e) {
        var index = this._getSelectedIndex();
        if (this._getSelectedCount() != 1 && index == this.itemBoxes.length - 1) {
            Event.stop(e);
        } else {
            this._swap(this.itemBoxes, index, index + 1);
            this._doOrderLocal.bind(this, index, index + 1).delay(50);
        }
    },

    _getPathList: function() {
        var result = [];
        for (var i = 0; i < this.itemBoxes.length; ++i) {
            result.push(this.itemBoxes[i].value);
        }
        return result;
    },

    _getSelectedCount: function() {
        var count = 0;
        for (var i = 0, size = this.itemBoxes.length; i < size; ++i) {
            if (this.itemBoxes[i].checked) { ++count; }
        }
        return count;
    },

    _getSelectedIndex: function() {
        for (var i = 0, size = this.itemBoxes.length; i < size; ++i) {
            if (this.itemBoxes[i].checked) {
                return i;
            }
        }
        return -1;
    },

    _getParentRow: function(elem) {
        for (var el = elem.parentNode; el != null; el = el.parentNode) {
            if (Utils.isRow(el)) {
                return el;
            }
        }
        return null;
    },

    _doDeleteLocal: function() {
        var newBoxes = [];

        for (var i = 0, size = this.itemBoxes.length; i < size; ++i) {
            var candidate = this.itemBoxes[i];
            if (candidate.checked) {
                var row = this._getParentRow(candidate);
                row.parentNode.removeChild(row);
            } else {
                newBoxes.push(candidate);
            }
        }

        this.itemBoxes = newBoxes;
        this._update();
    },

    _doOrderLocal: function(index1, index2) {
        var row1 = this._getParentRow(this.itemBoxes[index1]);
        var row2 = this._getParentRow(this.itemBoxes[index2]);
        row1.parentNode.insertBefore(row1, row2);

        this._update();
    },

    _swap: function(items, index1, index2) {
        var tmp = items[index1];
        items[index1] = items[index2];
        items[index2] = tmp;
    },

    _update: function() {
        var selectedCount = this._getSelectedCount();
        var selectedIndex = this._getSelectedIndex();
        var itemCount = this.itemBoxes.length;
        var canMove = (selectedCount == 1 && itemCount > 1);

        this.selectAllBox.checked = (itemCount == selectedCount && itemCount != 0);
        Utils.setEnabled(this.selectAllBox, itemCount != 0);
        Utils.setEnabled(this.editButton, selectedCount == 1);
        Utils.setEnabled(this.deleteButton, selectedCount != 0 && itemCount > 1);
        Utils.setEnabled(this.moveUpButton, canMove && selectedIndex != 0);
        Utils.setEnabled(this.moveDownButton, canMove && selectedIndex != itemCount - 1);
    },

    _updateItemOrder: function() {
        this.itemOrderInput.value = this._getPathList().join(this.itemOrderSeparator);
    }
});
