function ModalDialog(id, useShim) {
    this._constructor(id, useShim);
}
(function() { // create local scope;

    ModalDialog.BUTTON_OK = "OK";
    ModalDialog.BUTTON_CANCEL = "Cancel";
    ModalDialog.BUTTON_CLOSE = "Close";
    ModalDialog._VISIBLE_DIALOGS = [];

    var COVER_ID = "modalDialogCover";
    var Z_INDEX = 0x7FFFFE00;
    var DEFAULT_OPACITY = 0.35;
    var TABBED_TAGS = ["A", "AREA", "BUTTON", "INPUT", "OBJECT", "SELECT", "TEXTAREA"];
    var SHIM_OVERFLOW_TAGS = ["APPLET", "OBJECT", "EMBED"];
    var NON_BREAK_SPACE = String.fromCharCode(160);

    ModalDialog.prototype = {

        id: null,
        shim: null,
        cover: null,
        dialog: null,
        titlePane: null,
        titleButtonsBar: null,
        contentPane: null,
        buttonsBar: null,
        buttons: null,
        isIENavigator: null,
        isShimOverflowNavigator: null,
        useInactivityService: null,
        windowResizeMessageHandler: null,

        listener: null,

        _constructor: function(id, useShim) {
            this.isIENavigator = /MSIE\s*(5\.5|6\.)/.test(navigator.userAgent);
            this.isShimOverflowNavigator = Utils.Browser.Opera || Utils.Browser.Safari;
            this.id = id || ("ModalDialog" + new Date().getTime());
            this.buttons = new Array();
            this.useInactivityService = true;

            this._compose(useShim || /MSIE/.test(navigator.userAgent));
            //this.setVisible(false);

            this.setTitle(null);
            this.setOpacity(DEFAULT_OPACITY);
            this.setZIndex(Z_INDEX);

            this.windowResizeMessageHandler = this._position.bind(this);
            Event.observe(window, "resize", this.windowResizeMessageHandler);
            this._position();
        },

        toString: function() {
            return this.id;
        },

        getId: function() {
            return this.id;
        },

        addButton: function(type, text, handler) {
            var button = Utils.createInput(Tag.INPUT_TYPE_BUTTON, type);
            button.value = text;

            this.buttonsBar.appendChild(document.createTextNode(NON_BREAK_SPACE));
            this.buttonsBar.appendChild(button);
            this.buttons.push(button);

            var thisObj = this;
            button.onclick = function() {
                if (Object.isFunction(handler)) {
                    handler.call(handler, thisObj);
                } else {
                    thisObj._handleButtonPressed(this);
                }
            };
        },

        getButton: function(name) {
            for (var i = 0; i < this.buttons.length; ++i) {
                var candidate = this.buttons[i];
                if (candidate.name == name) {
                    return candidate;
                }
            }
            return null;
        },

        setButtonEnabled: function(name, enable) {
            var button = this.getButton(name);
            if (button != null) {
                Utils.setEnabled(button, enable);
            }
        },

        show: function(listener) {
            this.listener = listener || new ModalDialog.Listener();
            this._bindEvents();
            this.setVisible(true);
        },

        hide: function() {
            this.listener = null;
            this._unbindEvents();
            this.setVisible(false);
        },

        onSubmit: function() {
            this.listener.onSubmit(this);
        },

        onCancel: function() {
            this.listener.onCancel(this);
            //this.setVisible(false);
        },

        getContent: function() {
            return this.contentPane;
        },

        setContent: function(content) {
            if (this.contentPane.hasChildNodes()) {
                while (this.contentPane.hasChildNodes()) {
                    this.contentPane.removeChild(this.contentPane.lastChild);
                }
            }

            if (Object.isElement(content)) {
                this.contentPane.appendChild(content);
            }
        },

        setTitle: function(value) {
            var title = Utils.getTextNode(this.titlePane);
            var newTitle = document.createTextNode(value || document.title);

            if (title != null) {
                this.titlePane.replaceChild(newTitle, title);
            } else {
                this.titlePane.appendChild(newTitle);
            }
        },

        setOpacity: function(value) {
            Utils.setOpacity(this.cover, value || DEFAULT_OPACITY);
            if (this.shim != null) {
                Utils.setOpacity(this.shim, value || DEFAULT_OPACITY);
            }
        },

        setCssClass: function(value) {
            Utils.addCssClass(this.dialog, value || '');
        },

        setZIndex: function(value) {
            value = parseInt(value);
            Utils.setStyle(this.cover, {zIndex: value});
            Utils.setStyle(this.dialog, {zIndex: value + 1});
            if (this.shim != null) {
                Utils.setStyle(this.shim, {zIndex: value - 1});
            }
        },

        isVisible: function() {
            return Utils.isVisible(this.dialog);
        },

        setVisible: function(visible) {
            var displayValue = visible ? "block" : "none";
            this.dialog.style.display = displayValue;

            if (this.shim != null) {
                this.shim.style.display = displayValue;
                if (this.isShimOverflowNavigator) {
                    this._setShimOverflowTagsVisible(!visible);
                }
            }

            // setups cover visibility
            var index = ModalDialog._VISIBLE_DIALOGS.indexOf(this);
            if (visible && index == -1) {
                ModalDialog._VISIBLE_DIALOGS.push(this);
            } else if (!visible && index != -1) {
                ModalDialog._VISIBLE_DIALOGS.splice(index, 1);
            }
            if (visible || ModalDialog._VISIBLE_DIALOGS.length == 0) {
                this.cover.style.display = displayValue;
            }

            if (this.isIENavigator) {
                document.getElementsByTagName("html")[0].style.overflow = visible ? "hidden" : "";
                this._setTabbedTagsEnabled(!visible);
            }

            this._bringToTop();
            this._requestFocus();
            this._registerKeystrokes();
            this._restartInactivityService();
        },

        setSize: function(width, height) {
            var w = parseInt(width);
            var h = parseInt(height);
            Utils.setStyle(this.dialog, {width: isNaN(w) ? width : w + "px", height: isNaN(h) ? height : h + "px"});
            Utils.setStyle(this.contentPane, {
                height: !isNaN(h) ? (h - this._calcContentDimension().height - 10) + "px" : "",
                padding: !isNaN(h) ? "0 1.5em" : "",
                overflow: "auto"
            });
            this._position();
        },

        pack: function() {
            this.setSize(null, null);
        },

        dispose: function() {
            this.listener = null;
            this._unbindEvents();
            this.setVisible(false);

            Event.unobserve(window, "resize", this.windowResizeMessageHandler);
            this.windowResizeMessageHandler = null;

            this.dialog.parentNode.removeChild(this.dialog);
            this.dialog = null;

            if (this.shim != null) {
                this.shim.parentNode.removeChild(this.shim);
                this.shim = null;
            }
            this.cover = null;
            this.titlePane = null;
            this.titleButtonsBar = null;
            this.contentPane = null;
            this.buttonsBar = null;
            this.buttons = null;
        },

        _calcContentDimension: function() {
            var dimension = {width: 0, height: 0};
            var children = [this.titlePane, this.buttonsBar];
            var hidden = !this.isVisible();
            var dialogCss, origLeft, origDisplay;

            if (hidden) {
                dialogCss = this.dialog.style;
                origLeft = dialogCss.left;
                origDisplay = dialogCss.display;
                dialogCss.left = -100000 + "px";
                dialogCss.display = "block";
            }
            for (var i = 0; i < children.length; ++i) {
                if (Utils.isVisible(children[i])) {
                    dimension.height += Utils.getDimension(children[i]).height;
                }
            }
            if (hidden) {
                dialogCss.display = origDisplay;
                dialogCss.left = origLeft;
            }

            return dimension;
        },

        _bringToTop: function() {
            for (var i = 0; i < ModalDialog._VISIBLE_DIALOGS.length; ++i) {
                var dialog = ModalDialog._VISIBLE_DIALOGS[i];
                dialog.setZIndex(dialog == this ? Z_INDEX : (Z_INDEX - 1));
            }
        },

        _restartInactivityService: function() {
            if (this.useInactivityService && window.restartInactivityService) {
                window.restartInactivityService();
            }
        },

        _createShim: function() {
            var holder = document.body;
            var html = "<iframe src='about:blank' frameborder='0' scrolling='no' "
                       + "style='position:absolute;display:none;width:0;height:0;border:none;'>"
                       + "</iframe>";
            if (holder.insertAdjacentHTML) {
                holder.insertAdjacentHTML("AfterBegin", html);
                return holder.firstChild;
             } else if (document.createRange) {
                var range = document.createRange();
                range.setStartBefore(holder);
                holder.insertBefore(range.createContextualFragment(html), holder.firstChild);
                return holder.firstChild;
            }

            return null;
        },

        _createCover: function() {
            var result = $(COVER_ID);

            if (result == null) {
                result = document.body.appendChild(document.createElement(Tag.DIV));
                result.id = COVER_ID;
                Utils.setStyle(result, this.isIENavigator
                                       ? {position: "absolute", display: "none"}
                                       : {position: "fixed", top: 0, left: 0, width: "100%", height: "100%", display: "none"});
            }

            return result;
        },

        _compose: function(isUseShim) {
            this.shim = isUseShim ? this._createShim() : null;
            this.cover = this._createCover();

            this.dialog = document.createElement(Tag.DIV);
            this.titlePane = this.dialog.appendChild(document.createElement("DIV"));
            this.contentPane = this.dialog.appendChild(document.createElement("DIV"));
            this.buttonsBar = this.dialog.appendChild(document.createElement("DIV"));

            Utils.setStyle(this.dialog, "display: none");
            Utils.addCssClass(this.dialog, "dialog");
            Utils.addCssClass(this.dialog, "modal");
            Utils.addCssClass(this.dialog, "shadow");
            Utils.addCssClass(this.titlePane, "title-bar");
            Utils.addCssClass(this.contentPane, "content-pane");
            Utils.addCssClass(this.buttonsBar, "action-bar");

            this.dialog.id = this.getId();
            document.body.appendChild(this.dialog);
            Utils.setStyle(this.dialog, this.isIENavigator ? "position:absolute" : {position: "fixed", top: 0, left: 0});
        },

        _position: function() {
            var viewport = document.viewport.getDimensions();
            var scrollOffset = document.viewport.getScrollOffsets();
            var dialogDimension = Utils.getDimension(this.dialog);
            var viewportPopulating = Math.floor(viewport.height / dialogDimension.height);
            var dialogTop = scrollOffset.top + ((viewport.height - dialogDimension.height) / 2) - (viewportPopulating > 1 ? viewport.height * 0.15: 0);
            var dialogLeft = scrollOffset.left + (viewport.width - dialogDimension.width) / 2;
            var coverStyle = {top: scrollOffset.top + "px", left: scrollOffset.left + "px", width: viewport.width + "px", height: viewport.height + "px"};
            var dialogStyle = {top: Math.max(0, dialogTop) + "px", left: Math.max(0, dialogLeft) + "px"};

            if (this.shim != null) {
                Utils.setStyle(this.shim, coverStyle);
            }
            if (this.isIENavigator) {
                Utils.setStyle(this.cover, coverStyle);
            }
            Utils.setStyle(this.dialog, dialogStyle);
        },

        _setTabbedTagsEnabled: function(enabled) {
            var tags = this._getSuitableTags(TABBED_TAGS);
            if (!enabled) {
                this._setProperty(tags, "disabled", true, true);
            } else {
                this._restoreProperty(tags, "disabled");
            }
        },

        _setShimOverflowTagsVisible: function(visible) {
            var styles = {visibility: visible ? "" : "hidden"};
            var tags = this._getSuitableTags(SHIM_OVERFLOW_TAGS);
            for (var i = 0; i < tags.length; ++i) {
                Utils.setStyle(tags[i], styles);
            }
        },

        _requestFocus: function(field) {
            if (this.isVisible()) {
                if (!Object.isElement(field)) {
                    var buttons = this.buttonsBar.getElementsByTagName(Tag.INPUT);
                    field = (buttons.length > 0) ? buttons[0] : this.getContent();
                }
                try {
                    field.focus();
                } catch (ex) {
                    this._log("Unable to request focus: " + ex);
                }
            }
        },

        _bindEvents: function() {
        },

        _unbindEvents: function() {
        },

        _registerKeystrokes: function() {
            var km = KeyboardManager.getInstance();
            if (this.isVisible()) {
                km.registerHandler(KeyEvent.VK_ENTER, this);
                km.registerHandler(KeyEvent.VK_ESCAPE, this);
            } else {
                km.unregisterHandler(KeyEvent.VK_ENTER, this);
                km.unregisterHandler(KeyEvent.VK_ESCAPE, this);
            }
        },

        /** @see KeyboardManage#registerHandler */
        processKeyEvent: function(ke) {
            switch (ke.getKeyCode()) {
                case KeyEvent.VK_ENTER:
                    var input = Utils.findAncestor(ke.getSource(), "input");
                    if (input && input.type == "text" && Utils.successorOf(this.contentPane, input)) {
                        ke.cancel();
                        var okButton = this.getButton(ModalDialog.BUTTON_OK);
                        if (okButton) {
                            okButton.click();
                        } else {
                            this.onSubmit();
                        }
                    }
                    break;

                case KeyEvent.VK_ESCAPE:
                    ke.cancel();
                    var cancelButton = this.getButton(ModalDialog.BUTTON_CANCEL);
                    if (cancelButton) {
                        cancelButton.click();
                    } else {
                        this.onCancel();
                    }
                    break;
            }
        },

        _handleButtonPressed: function(button) {
            if (button.name == ModalDialog.BUTTON_OK && !button.disabled) {
                this.onSubmit();
            } else if (button.name == ModalDialog.BUTTON_CANCEL) {
                this.onCancel();
            } else if (button.name == ModalDialog.BUTTON_CLOSE) {
                this.setVisible(false);
            }
        },

        _setProperty: function(objects, propertyName, propertyValue, storeOriginalValue) {
            var originalPropertyName = this._getPropertyBackupName(propertyName);
            for (var i = 0; i < objects.length; ++i) {
                var obj = objects[i];
                if (storeOriginalValue) {
                    obj[originalPropertyName] = obj[propertyName];
                }
                obj[propertyName] = propertyValue;
            }
        },

        _restoreProperty: function(objects, propertyName) {
            var originalPropertyName = this._getPropertyBackupName(propertyName);
            for (var i = 0; i < objects.length; ++i) {
                var obj = objects[i];
                if (!Object.isUndefined(obj[originalPropertyName])) {
                    obj[propertyName] = obj[originalPropertyName];
                }
            }
        },

        _getPropertyBackupName: function(propertyName) {
            return "original" + propertyName.charAt(0).toUpperCase() + propertyName.substring(1);
        },

        _getSuitableTags: function(tagNames) {
            var result = new Array();
            var documentElements = this._getNamedTags(document, tagNames);
            var dialogElements = this._getNamedTags(this.dialog, tagNames);

            for (var i = 0; i < documentElements.length; ++i) {
                var candidate = documentElements[i];
                if (dialogElements.indexOf(candidate) == -1) {
                    result.push(candidate);
                }
            }
            return result;
        },

        _getNamedTags: function(parent, tagNames) {
            var result = new Array();
            for (var i = 0; i < tagNames.length; ++i) {
                var tags = parent.getElementsByTagName(tagNames[i]);
                for (var j = 0; j < tags.length; ++j) {
                    result.push(tags[j]);
                }
            }
            return result;
        },

        _log: function(message) {
            if (window.console) {
                window.console.log(message);
            }
        }
    };
})();
ModalDialog.Listener = Class.create({
    onSubmit: function(dialog) {},
    onCancel: function(dialog) {}
});

ModalDialog.alert = function(htmlMessage, title, okButton) {
    var dialog = new ModalDialog();
    var content = document.createElement("P");
    content.innerHTML = htmlMessage;
    dialog.addButton(ModalDialog.BUTTON_OK, okButton || "OK");
    dialog.setTitle(title || document.title);
    dialog.setContent(content);
    dialog.show({
        onSubmit: function(d) {d.dispose();},
        onCancel: function(d) {d.dispose();}
    });
};

ModalDialog.confirm = function(title, htmlMessage, buttons) {
    if (Object.isUndefined(buttons)) {
        buttons = {};
        buttons[ModalDialog.BUTTON_OK] = function(d) {d.dispose();};
        buttons[ModalDialog.BUTTON_CANCEL] = function(d) {d.dispose();};
    }
    var dialog = new ModalDialog();
    for (var prop in buttons) {
        if (buttons.hasOwnProperty(prop)) {
            dialog.addButton(prop, String(prop), buttons[prop]);
        }
    }
    var content = document.createElement("P");
    content.innerHTML = htmlMessage;
    dialog.setTitle(title || document.title);
    dialog.setContent(content);
    dialog.show();
};