var AccountQuotaController = Class.create({

    view: null,

    _constructor: function() {
        this.view = new AccountQuotaView();
    },

    start: function() {
        this.view.show({
            onUploadStateChanged: this._handleUploadStateChanged.bind(this),
            onDownloadStateChanged: this._handleDownloadStateChanged.bind(this),
            onTransferStateChanged: this._handleTransferStateChanged.bind(this)
        })
    },

    _handleUploadStateChanged: function(view) {
        if (view.isUploadQuotaEnabled()) {
            view.setTransferQuotaEnabled(false);
        }
    },

    _handleDownloadStateChanged: function(view) {
        if (view.isDownloadQuotaEnabled()) {
            view.setTransferQuotaEnabled(false);
        }
    },

    _handleTransferStateChanged: function(view) {
        if (view.isTransferQuotaEnabled()) {
            view.setUploadQuotaEnabled(false);
            view.setDownloadQuotaEnabled(false);
        }
    }

});

var AccountQuotaView = Class.create({

    uploadQuotaEntry: null,
    downloadQuotaEntry: null,
    transferQuotaEntry: null,
    maxUploadsPerSessionBox: null,
    maxUploadsPerSessionField: null,
    maxDownloadsPerSessionBox: null,
    maxDownloadsPerSessionField: null,

    clickHandler: null,
    listener: null,

    _constructor: function() {
        this.uploadQuotaEntry = new AccountQuotaView.Entry($("uploadsquotaenabled"), $("uploadsquotasize"), $("uploadsquotaperiod"));
        this.downloadQuotaEntry = new AccountQuotaView.Entry($("dloadsquotaenabled"), $("dloadsquotasize"), $("dloadsquotaperiod"));
        this.transferQuotaEntry = new AccountQuotaView.Entry($("transfersquotaenabled"), $("transfersquotasize"), $("transfersquotaperiod"));
        this.maxUploadsPerSessionBox = $("maxuploadspersessionenabled");
        this.maxUploadsPerSessionField = $("maxuploadspersessioncount");
        this.maxDownloadsPerSessionBox = $("maxdownloadspersessionenabled");
        this.maxDownloadsPerSessionField = $("maxdownloadspersessioncount");

        new NumberEditor(this.maxUploadsPerSessionField).setMinValue(0);
        new NumberEditor(this.maxDownloadsPerSessionField).setMinValue(0);
    },

    isUploadQuotaEnabled: function() {
        return this.uploadQuotaEntry.isEnabled();
    },

    setUploadQuotaEnabled: function(value) {
        this.uploadQuotaEntry.setEnabled(value);
    },

    isDownloadQuotaEnabled: function() {
        return this.downloadQuotaEntry.isEnabled();
    },

    setDownloadQuotaEnabled: function(value) {
        this.downloadQuotaEntry.setEnabled(value);
    },

    isTransferQuotaEnabled: function() {
        return this.transferQuotaEntry.isEnabled();
    },

    setTransferQuotaEnabled: function(value) {
        this.transferQuotaEntry.setEnabled(value);
    },

    show: function(listener) {
        this.listener = listener;
        this._bindEvents();

        var handler = {
            onStateChanged: this._handleStateChanged.bind(this)
        };
        this.uploadQuotaEntry.show(handler);
        this.downloadQuotaEntry.show(handler);
        this.transferQuotaEntry.show(handler);
    },

    hide: function() {
        this._unbindEvents();
        this.uploadQuotaEntry.hide();
        this.downloadQuotaEntry.hide();
        this.transferQuotaEntry.hide();

        this.listener = null;
    },

    _handleStateChanged: function(entry) {
        if (entry == this.uploadQuotaEntry) {
            this.listener.onUploadStateChanged(this);
        } else if (entry == this.downloadQuotaEntry) {
            this.listener.onDownloadStateChanged(this);
        } else if (entry == this.transferQuotaEntry) {
            this.listener.onTransferStateChanged(this);
        }
    },

    _handleClick: function() {
        Utils.setEnabled(this.maxUploadsPerSessionField, this.maxUploadsPerSessionBox.checked);
        Utils.setEnabled(this.maxDownloadsPerSessionField, this.maxDownloadsPerSessionBox.checked);
    },

    _bindEvents: function() {
        this.stateHandler = this._handleStateChanged.bind(this);

        this.clickHandler = this._handleClick.bind(this);
        Event.observe(this.maxUploadsPerSessionBox, "click", this.clickHandler);
        Event.observe(this.maxDownloadsPerSessionBox, "click", this.clickHandler);
    },

    _unbindEvents: function() {
        Event.unobserve(this.maxUploadsPerSessionBox, "click", this.clickHandler);
        Event.unobserve(this.maxDownloadsPerSessionBox, "click", this.clickHandler);

        this.clickHandler = null;
    }
});
AccountQuotaView.Listener = Class.create({
    onUploadStateChanged: function(view) {},
    onDownloadStateChanged: function(view) {},
    onTransferStateChanged: function(view) {}
});
AccountQuotaView.Entry = Class.create({
    enableBox: null,
    valueField: null,
    periodField: null,
    stateHandler: null,

    _constructor: function(enableBox, valueField, periodField) {
        this.enableBox = enableBox;
        this.valueField = valueField;
        this.periodField = periodField;

        new NumberEditor(this.valueField).setMinValue(0);
        new NumberEditor(this.periodField).setMinValue(0);
    },

    isEnabled: function() {
        return this.enableBox.checked;
    },

    setEnabled: function(value) {
        this.enableBox.checked = value;
        this._update();
    },

    show: function(listener) {
        this.listener = listener;
        this._bindEvents();
        this._update();
    },

    hide: function() {
        this._unbindEvents();
        this.listener = null;
    },

    _update: function() {
        Utils.setEnabled(this.valueField, this.isEnabled());
        Utils.setEnabled(this.periodField, this.isEnabled());
    },

    _bindEvents: function() {
        this.stateHandler = this._handleStateChanged.bind(this);
        Event.observe(this.enableBox, "click", this.stateHandler);
    },

    _unbindEvents: function() {
        Event.unobserve(this.enableBox, "click", this.stateHandler);
        this.stateHandler = null;
    },

    _handleStateChanged: function() {
        this._update();
        this.listener.onStateChanged(this);
    }

});