var AccountController = Class.create({

    view: null,
    quotaController: null,

    _constructor: function(config) {
        if (config && config.changeQuotas) {
            this.quotaController = new AccountQuotaController();
        }
    },

    start: function() {
        this.view.show(this);
        this.quotaController && this.quotaController.start();
    }
});


var AddAccountController = Class.create(AccountController, {
    _constructor: function(config) {
        this.view = new AddAccountView(config);
    },
    onTemplateChanged: function(view) {
        var uri = view.form.action;
        var f = view.templateField;
        if (f != null && f.name) {
            uri += uri.indexOf('?') == -1 ? "?" : "&";
            uri += f.name + "=" + encodeURIComponent($F(f));
        }
        document.location.href = uri;
    }
});

var EditAccountController = Class.create(AccountController, {
    pathsController: null,
    ipAccessController: null,
    _constructor: function(config) {
        this.view = new AccountView(config);
        this.pathsController = new AccountPathsController();
        this.ipAccessController = new AccountIpAccessController();
    },
    start: function() {
        this._super.start.call(this);
        this.pathsController.start();
        this.ipAccessController.start();
    }
});

var AccountView = Class.create({

    expiresOnBox: null,
    expirationField: null,
    assignToGroupBox: null,
    groupsField: null,

    mouseHandler: null,

    _constructor: function(config) {
        this.expiresOnBox = $("expiresondate");
        this.expirationField = $("expirationdate");
        this.assignToGroupBox = $("assigntogroup");
        this.groupsField = $("group");

        this._init(config);
    },

    show: function(listener) {
        this.listener = listener || new AccountView.Listener();
        this._bindEvents();
        this._update();
    },

    hide: function() {
        this.listener = null;
        this._unbindEvents();
    },

    _init: function(config) {
        if (this._groupAssignmentAllowed()) {
            this.assignToGroupBox.disabled = this.groupsField.length == 0;
        }

        var de = new DateEditor(this.expirationField, config.datePattern || "MM/dd/yyyy");
        de.setMinValue(new Date().getTime());
    },

    _update: function() {
        this._updateExpirationDate();
        this._updateGroups();
    },

    _updateExpirationDate: function () {
        if (this.expirationField && this.expiresOnBox) {
            this.expirationField.disabled = !this.expiresOnBox.checked;
        }
    },

    _updateGroups: function() {
        if (this._groupAssignmentAllowed()) {
            this.groupsField.disabled = !this.assignToGroupBox.checked;
        }
    },

    _handleMouseEvent: function() {
        this._update();
    },

    _bindEvents: function() {
        this.mouseHandler = this._handleMouseEvent.bind(this);

        Event.observe(this.expiresOnBox, "click", this.mouseHandler);
        if (this._groupAssignmentAllowed()) {
            Event.observe(this.assignToGroupBox, "click", this.mouseHandler);
        }
    },

    _unbindEvents: function() {
        Event.unobserve(this.expiresOnBox, "click", this.mouseHandler);
        if (this._groupAssignmentAllowed()) {
            Event.unobserve(this.assignToGroupBox, "click", this.mouseHandler);
        }

        this.mouseHandler = null;
    },

    _groupAssignmentAllowed: function() {
        return Object.isElement(this.assignToGroupBox) && Object.isElement(this.groupsField);
    }

});

AccountView.Listener = Class.create({});

var AddAccountView = Class.create(AccountView, {

    form: null,
    templateField: null,

    _constructor: function() {
        this.templateField = $("template");
        this.form = this.templateField.form;
    },

    show: function(listener) {
        this._super.show.call(this, listener || new AddAccountView.Listener());
    },

    _handleTemplateChanged: function() {
        this.listener.onTemplateChanged(this);
    },

    _bindEvents: function() {
        this._super._bindEvents.apply(this, arguments);

        this.templateHandler = this._handleTemplateChanged.bind(this);
        Event.observe(this.templateField, "change", this.templateHandler);
    },

    _unbindEvents: function() {
        Event.unobserve(this.templateField, "change", this.templateHandler);
        this.templateHandler = null;

        this._super._unbindEvents.apply(this, arguments);
    }

});

AddAccountView.Listener = Class.create(AccountView.Listener, {
    onTemplateChanged: function(view) {}
});