var UserListController = Class.create({

    view: null,
    sorter: null,

    _constructor: function() {
        this.view = new UserListView();
        this.sorter = new TableSorter(this.view.table);
        this.sorter.setSortedColumns([1, 2, 3, 4, 5]);
        this.sorter.setSortedBy(1, true);
        this.sorter.sortByColumn(1, true);
    },

    start: function() {
        this.view.show({
            onAdd: this._handleAdd.bind(this),
            onDelete: this._handleDelete.bind(this)
        });
    },

    _handleAdd: function() {
        this._sendForm(this.view.addButton);
    },

    _handleDelete: function(view, deleteOwned) {
        this.view.deleteOwnedAccountsBox.value = deleteOwned;
        this._sendForm(this.view.deleteButton);
    },

    _sendForm: function(button) {
        var form = this.view.form;
        var formAction = button && button.getAttribute("formaction");
        if (formAction) {
            form.action = formAction;
        }
        form.submit();
    }
});

var UserListView = Class.create({

    form: null,
    table: null,
    addButton: null,
    deleteButton: null,
    selectAllBox: null,
    selectionBoxes: null,
    deleteOwnedAccountsBox: null,
    deleteConfirmed: false,

    listener: null,

    _constructor: function() {
        this.form = $("usersform");
        this.table = $("userlist");
        this.addButton = $("adduser");
        this.deleteButton = $("deleteuser");
        this.selectAllBox = $("selectall");
        this.selectionBoxes = Utils.getFormInputs(this.form, Tag.INPUT_TYPE_CHECKBOX, "id");
        this.deleteOwnedAccountsBox = $("deleteownedaccounts");
        this.deleteConfirmed = false;
    },

    show: function(listener) {
        this.listener = listener || new UserListView.Listener();
        this._bindEvents();
        this._update();
    },

    _getSelectedCount: function() {
        var count = 0;
        for (var i = 0, size = this.selectionBoxes.length; i < size; ++i) {
            if (this.selectionBoxes[i].checked) {
                count++;
            }
        }
        return count;
    },

    _bindEvents: function() {
        Event.observe(this.selectAllBox, "click", this._handleSelectAllBox.bind(this));
        Event.observe(this.addButton, "click", this._handleAddButton.bindAsEventListener(this));
        Event.observe(this.deleteButton, "click", this._handleDeleteButton.bindAsEventListener(this));
        var selectionHandler = this._update.bind(this);

        for (var i = 0; i < this.selectionBoxes.length; ++i) {
            Event.observe(this.selectionBoxes[i], "click", selectionHandler);
        }
    },

    _handleSelectAllBox: function() {
        var selected = Boolean.toBoolean($F(this.selectAllBox));
        for (var i = 0, size = this.selectionBoxes.length; i < size; ++i) {
            this.selectionBoxes[i].checked = selected;
        }
        this._update();
    },

    _handleAddButton: function(event) {
        Event.stop(event);
        this.listener.onAdd(this);
    },

    _handleDeleteButton: function(event) {
        Event.stop(event);
        this._showConfirmDeleteDialog();
    },

    _handleConfirmDelete: function(dialog, deleteOwned) {
        dialog.hide();

        this.listener.onDelete(this, deleteOwned);
    },

    _showConfirmDeleteDialog: function() {
        var dialog = new UsersConfirmDeleteDialog();
        dialog.show({
            onConfirm: this._handleConfirmDelete.bind(this)
        });
    },

    _update: function() {
        var count = this._getSelectedCount();
        this.selectAllBox.checked = (count != 0 && this.selectionBoxes.length == count);
        Utils.setEnabled(this.deleteButton, count != 0);
    }
});
UserListView.Listener = Class.create({
    onAdd: function(view) {},
    onDelete: function(view, deleteOwned) {}
});

var UsersConfirmDeleteDialog = Class.create(ModalDialog, {

    deleteOwnedAccountsBox: null,

    _constructor: function() {
        this.deleteOwnedAccountsBox = Utils.createInput(Tag.INPUT_TYPE_CHECKBOX, "ownedAccountsBox", "true");
        this.setContent(this._createContent());

        this.addButton(ModalDialog.BUTTON_OK, UsersConfirmDeleteDialog.Buttons.OK);
        this.addButton(ModalDialog.BUTTON_CANCEL, UsersConfirmDeleteDialog.Buttons.CANCEL);
    },

    getId: function() {
        return "ConfirmDeleteUserDialog";
    },

    show: function(listener) {
        this._super.show.call(this, Object.extend(new UsersConfirmDeleteDialog.Listener(), listener || {}));
    },

    hide: function() {
        this._super.hide.call(this);
        this.dispose();
    },

    onSubmit: function() {
        var deleteOwned = this.deleteOwnedAccountsBox.checked;
        this.listener.onConfirm(this, deleteOwned);
    },

    onCancel: function() {
        this.hide();
    },

    _createContent: function() {
        var messagePanel = document.createElement("P");
        messagePanel.appendChild(document.createTextNode(UsersConfirmDeleteDialog.Messages.CONFIRM_DELETE));

        this.deleteOwnedAccountsBox.id = "deleteOwnedAccountsBox";
        var deleteOwnedAccountsLabel = Utils.createLabel(UsersConfirmDeleteDialog.Labels.DELETE_OWNED_ACCOUNTS, this.deleteOwnedAccountsBox.id);

        var optionsPanel = document.createElement("DIV");
        optionsPanel.appendChild(this.deleteOwnedAccountsBox);
        optionsPanel.appendChild(deleteOwnedAccountsLabel);

        Utils.setStyle(messagePanel, "font-weight:bold;");
        Utils.setStyle(optionsPanel, "font-size:0.9em;margin-bottom:.7em;padding-left:1em");
        Utils.setStyle(this.deleteOwnedAccountsBox, "vertical-align:sub");

        var content = document.createElement("DIV");
        content.appendChild(messagePanel);
        content.appendChild(optionsPanel);

        return content;
    }
});
UsersConfirmDeleteDialog.Listener = Class.create(ModalDialog.Listener, {
    onConfirm: function(dialog, deleteOwned) {}
});
UsersConfirmDeleteDialog.Buttons = {OK: "OK", CANCEL: "Cancel"};
UsersConfirmDeleteDialog.Labels = {DELETE_OWNED_ACCOUNTS: "Delete owned accounts"};
UsersConfirmDeleteDialog.Messages = {CONFIRM_DELETE: "Delete selected user(s)?"};
