var AccountPathsController = Class.create({

    view: null,

    _constructor: function() {
        this.view = new AccountPathsView();
    },

    start: function() {
        this.view.show({
            onAdd: this._handleAdd.bind(this),
            onEdit: this._handleEdit.bind(this),
            onDelete: this._handleDelete.bind(this),
            onMoveUp: this._handleMoveUp.bind(this),
            onMoveDown: this._handleMoveDown.bind(this)
        });
    },

    _handleAdd: function(view) {
        this._sendForm(this.view.addButton);
    },

    _handleEdit: function(view) {
        this._sendForm(this.view.editButton);
    },

    _handleDelete: function(view) {
        this._sendForm(this.view.deleteButton);

        var items = this.view.getSelectedItems();
        (function (items, view) {
            for (var i = 0; i < items.length; ++i) {
                view.removeItem(items[i]);
            }
        }).delay(100, items, view);
    },

    _handleMoveUp: function(view) {
        var index = view.getSelectedIndex();
        view.swapItems(index - 1, index);

        this._sendForm(this.view.moveUpButton);
    },

    _handleMoveDown: function(view) {
        var index = view.getSelectedIndex();
        view.swapItems(index, index + 1);

        this._sendForm(this.view.moveDownButton);
    },

    _sendForm: function(button) {
        var form = this.view.form;
        var formAction = button && button.getAttribute("formaction");
        if (formAction) {
            form.action = formAction;
        }
        form.submit();
    }
});

var AccountPathsView = Class.create({

    form: null,
    items: null,
    markedItems: null,

    addButton: null,
    editButton: null,
    deleteButton: null,
    moveUpButton: null,
    moveDownButton: null,
    pathOrderField: null,
    buttonHandler: null,

    selectAllHandler: null,
    selectAllBox: null,

    listener: null,

    _constructor: function() {
        this.items = [];
        this.markedItems = [];
        this.form = $("usersform");
        this.selectAllBox = $("selectallpath");
        this.addButton = $("addpathbtn");
        this.editButton = $("editpathbtn");
        this.deleteButton = $("deletepathbtn");
        this.moveUpButton = $("moveuppathbtn");
        this.moveDownButton = $("movedownpathbtn");
        this.pathOrderField = $("pathlistorder");

        this._init($("accountpaths"));
    },

    show: function(listener) {
        this.listener = listener || new AccountPathsView.Listener();
        this._bindEvents();
        this.update();
    },

    hide: function() {
        this._unbindEvents();
        this.listener = null;
    },

    update: function() {
        var singleSelection = this.markedItems.length == 1;
        var deleteAllowed = this.markedItems.length != 0 && this.items.length > 1;
        var moveUpAllowed = singleSelection && this.items.indexOf(this.markedItems[0]) > 0;
        var moveDownAllowed = singleSelection && this.items.indexOf(this.markedItems[0]) < this.items.length - 1;

        this.selectAllBox.checked = this.items.length != 0 && this.items.length == this.markedItems.length;
        Utils.setEnabled(this.editButton, singleSelection);
        Utils.setEnabled(this.deleteButton, deleteAllowed);
        Utils.setEnabled(this.moveUpButton, moveUpAllowed);
        Utils.setEnabled(this.moveDownButton, moveDownAllowed);
    },

    getSelectedIndex: function() {
        return this.markedItems.length != 0 ? this.items.indexOf(this.markedItems[0]) : -1;
    },

    getSelectedItems: function() {
        return this.markedItems.clone();
    },

    removeItem: function(item) {
        var index = this.items.indexOf(item);
        if (index != -1) {
            this.items.splice(index, 1);
        }
        var markedIndex = this.markedItems.indexOf(item);
        if (markedIndex != -1) {
            this.markedItems.splice(markedIndex, 1);
        }
        if (item.row && item.row.parentNode) {
            item.row.parentNode.removeChild(item.row);
        }

        this.update();
    },

    swapItems: function(index1, index2) {
        var tmp = this.items[index1];
        this.items[index1] = this.items[index2];
        this.items[index2] = tmp;
        this._adjustItemOrder();

        var row1 = this.items[index1].row;
        var row2 = this.items[index2].row;
        row1.parentNode.insertBefore(row1, row2);

        this.update();
    },

    _init: function(table) {
        var listener  = {
            onMark: this._handleMarkup.bind(this)
        };

        var rows = table.tBodies[0].rows;
        for (var i = 0, size = rows.length; i < size; ++i) {
            var item = AccountPathsView.Item.fromRow(rows[i], listener);
            this.items.push(item);
        }

        Utils.setEnabled(this.selectAllBox, this.items.length != 0);
    },

    _adjustItemOrder: function() {
        var order = "";
        for (var i = 0, size = this.items.length; i < size; ++i) {
            order += this.items[i].id;
            if (i < size - 1) {
                order += "|";
            }
        }
        this.pathOrderField.value = order;
    },

    _bindEvents: function() {
        this.selectAllHandler = this._handleSelectAll.bind(this);
        this.buttonHandler = this._handleButtonPressed.bindAsEventListener(this);

        Event.observe(this.selectAllBox, "click", this.selectAllHandler);
        Event.observe(this.addButton, "click", this.buttonHandler);
        Event.observe(this.editButton, "click", this.buttonHandler);
        Event.observe(this.deleteButton, "click", this.buttonHandler);
        Event.observe(this.moveUpButton, "click", this.buttonHandler);
        Event.observe(this.moveDownButton, "click", this.buttonHandler);

        for (var i = 0; i < this.items.length; ++i) {
            this.items[i]._bindEvents();
        }
    },

    _unbindEvents: function() {
        for (var i = 0; i < this.items.length; ++i) {
            this.items[i]._unbindEvents();
        }

        Event.unobserve(this.selectAllBox, "click", this.selectAllHandler);
        Event.unobserve(this.addButton, "click", this.buttonHandler);
        Event.unobserve(this.editButton, "click", this.buttonHandler);
        Event.unobserve(this.deleteButton, "click", this.buttonHandler);
        Event.unobserve(this.moveUpButton, "click", this.buttonHandler);
        Event.unobserve(this.moveDownButton, "click", this.buttonHandler);

        this.buttonHandler = null;
        this.selectAllHandler = null;
    },

    _handleSelectAll: function() {
        var selected = this.selectAllBox.checked;
        this.markedItems = [];

        for (var i = 0; i < this.items.length; ++i) {
            var item = this.items[i];
            if (selected) {
                item.setMarked(true);
                this.markedItems.push(item);
            } else {
                item.setMarked(false);
            }
        }

        this.update();
    },

    _handleButtonPressed: function(e) {
        Event.stop(e);
        var button = Event.findAncestor(e, "INPUT");

        if (button == this.addButton) {
            this.listener.onAdd(this);
        } else if (button == this.editButton) {
            this.listener.onEdit(this);
        } else if (button == this.deleteButton) {
            this.listener.onDelete(this);
        } else if (button == this.moveUpButton) {
            this.listener.onMoveUp(this);
        } else if (button == this.moveDownButton) {
            this.listener.onMoveDown(this);
        }
    },

    _handleMarkup: function(item) {
        var index = this.markedItems.indexOf(item);

        if (index == -1 && item.isMarked()) {
            this.markedItems.push(item);
        } else if (index != -1 && !item.isMarked()) {
            this.markedItems.splice(index, 1);
        }

        this.update();
    }

});

AccountPathsView.Item = Class.create({
    id: null,
    row: null,
    markBox: null,
    mouseHandler: null,
    listener: null,

    _constructor: function(row, markBox, listener) {
        this.row = row;
        this.markBox = markBox;
        this.id = (markBox && markBox.value) || "";
        this.listener = listener;
    },
    isMarked: function() {
        return this.markBox && this.markBox.checked;
    },
    setMarked: function(value) {
        if (this.markBox) {
            this.markBox.checked = value;
        }
    },
    _bindEvents: function() {
        this.mouseHandler = this._handleMouseEvent.bindAsEventListener(this);
        Event.observe(this.markBox, "click", this.mouseHandler);
    },
    _unbindEvents: function() {
        Event.unobserve(this.markBox, "click", this.mouseHandler);
        this.mouseHandler = null;
    },
    _handleMouseEvent: function(e) {
        var target = e.target || e.srcElement || null;
        if (target && target == this.markBox) {
            this.listener.onMark(this);
        }
    }
});
AccountPathsView.Item.fromRow = function(row, listener) {
    var inputs = row.getElementsByTagName("INPUT");
    var markBox = inputs.length > 0 && Utils.isCheckboxInput(inputs[0]) ? inputs[0] : null;
    return new AccountPathsView.Item(row, markBox, listener);
};

AccountPathsView.Listener = Class.create({
    onAdd: function(v) {},
    onEdit: function(v) {},
    onDelete: function(v) {},
    onMoveUp: function(v) {},
    onMoveDown: function(v) {}
});
