var DlpSettingsController = Class.create({

    config: null,

    addEntryDialog: null,
    editEntryDialog: null,
    settingsDialog: null,

    _constructor: function(config) {
        this.config = config || Object.extend({}, DlpSettingsController.DEFAULT_CONFIG);
    },

    start: function() {
        this._showSettingsDialog();
    },

    _showSettingsDialog: function() {
        this.settingsDialog = new DlpSettingsDialog(this.config.dlpsu, this.config.metadata);
        this.settingsDialog.show(Object.extend(new FormDialog.Listener(), {
            onAdd: this._handleAddEntry.bind(this),
            onEdit: this._handleEditEntry.bind(this),
            onDelete: this._handleDeleteEntry.bind(this),
            onMoveUp: this._handleMoveEntryUp.bind(this),
            onMoveDown: this._handleMoveEntryDown.bind(this),
            onSubmit: function(d) { d.hide(); },
            onCancel: function(d) { d.hide(); }
        }));
    },

    _reloadSettingsDialog: function() {
        if (this.settingsDialog) {
            this.settingsDialog.hide();
        }
        this._showSettingsDialog();
    },

    _handleAddEntry: function() {
        var dialog = new FormDialog("AddDlpEntryDilaog", this.config.dlpau);
        dialog.setSize(430, 250);
        dialog.show(Object.extend(new FormDialog.Listener(), {
            onLoaded: this._handleLoadedAddEntry.bind(this),
            onBeforeSubmit: this._handleBeforeSubmitAddEntry.bind(this),
            onDone: this._handleDoneAddEntry.bind(this),
            onCancel: function(d) { d.dispose(); }
        }), this.config.metadata);
    },

    _handleEditEntry: function(id) {
        var dialog = new FormDialog("EditDlpEntryDilaog", this.config.dlpeu);
        dialog.setSize(450, 280);
        dialog.show(Object.extend(new FormDialog.Listener(), {
            onDone: this._handleDoneEditEntry.bind(this),
            onCancel: function(d) { d.dispose(); }
        }), Object.extend({dlpid: id}, this.config.metadata));
    },

    _handleDeleteEntry: function(dialog) {
        var ids = dialog.getSelectedItems();
        dialog.hide();

        var channel = new IFrameChannel();
        channel.open(this.config.dlpdu, "post", {
            onComplete: this._handleDeleteComplete.bind(this),
            onError: this._handleError.bind(this)
        });
        channel.send(Object.extend({dlpid: ids}, this.config.metadata));
    },

    _handleDeleteComplete: function(channel) {
        channel.close();
        //this._reloadSettingsDialog();
    },

    _handleLoadedAddEntry: function(dialog) {
        var rules = $("rule");
        dialog.setButtonEnabled(ModalDialog.BUTTON_OK, Utils.isSelect(rules) && rules.options.length != 0);
    },

    _handleBeforeSubmitAddEntry: function(dialog) {
        return !!$F($("rule"));
    },

    _handleDoneAddEntry: function(dialog) {
        dialog.dispose();
        this._reloadSettingsDialog();
    },

    _handleDoneEditEntry: function(dialog) {
        dialog.dispose();
        this._reloadSettingsDialog();
    },

    _handleMoveEntryUp: function(dialog) {
        var channel = new IFrameChannel();
        channel.open(this.config.dlpou, "post", {
            onComplete: function() {},
            onError: this._handleError.bind(this)
        });
        channel.send(Object.extend({dlplistorder: dialog.getListOrder()}, this.config.metadata));
    },

    _handleMoveEntryDown: function(dialog) {
        var channel = new IFrameChannel();
        channel.open(this.config.dlpou, "post", {
            onComplete: function() {},
            onError: this._handleError.bind(this)
        });
        channel.send(Object.extend({dlplistorder: dialog.getListOrder()}, this.config.metadata));
    },

    _handleError: function(channel) {
        try {
            ModalDialog.alert(channel.getResponseMetadata()['x-dialog-reason'] || "Unknown.", "Error");
        } finally {
            channel.close();
        }
    }
});

DlpSettingsController.DEFAULT_CONFIG = { dlpau: "", dlpeu: "", dlpdu: "", dlpou: "", metadata: {}};

var DlpSettingsDialog = Class.create({

    dialog: null,
    metadata: null,

    idBoxes: null,
    selectAllBox: null,
    addButton: null,
    editButton: null,
    deleteButton: null,
    moveUpButton: null,
    moveDownButton: null,
    buttonHandler: null,
    updateHandler: null,
    selectAllHandler: null,
    listOrderField: null,

    listener: null,

    _constructor: function(formUrl, metadata) {
        this.dialog = new FormDialog("DlpSettingsDialog", formUrl);
        this.metadata = metadata || {};
    },

    show: function(listener) {
        this.listener = listener || new DlpSettingsDialog.Listener();
        this.dialog.setSize(650, 390);
        this.dialog.show(Object.extend(new FormDialog.Listener(), {
            onLoaded: this._handleLoaded.bind(this),
            onSubmit: this._handleSubmit.bind(this)
        }), this.metadata);
    },

    hide: function() {
        this.listener = null;
        this._unbindEvents();
        this.dialog.dispose();
    },

    getSelectedIndex: function() {
        for (var i = 0, size = this.idBoxes.length; i < size; ++i) {
            if (this.idBoxes[i].checked) {
                return i;
            }
        }
        return -1;
    },

    getSelectedItems: function() {
        var items = [];
        for (var i = 0, size = this.idBoxes.length; i < size; ++i) {
            var value = $F(this.idBoxes[i]);
            if (value) {
                items.push(value);
            }
        }
        return items;
    },

    getListOrder: function() {
        return this.listOrderField.value;
    },

    moveUp: function(index) {
        this.idBoxes.splice(index - 1, 2, this.idBoxes[index], this.idBoxes[index - 1]);

        this._adjustItemOrder();
        this._moveLocal.bind(this).delay(50, index - 1, index);
    },

    moveDown: function(index) {
        this.idBoxes.splice(index, 2, this.idBoxes[index + 1], this.idBoxes[index]);

        this._adjustItemOrder();
        this._moveLocal.bind(this).delay(50, index, index + 1);
    },

    _adjustItemOrder: function() {
        var result = "";
        for (var i = 0, size = this.idBoxes.length; i < size; ++i) {
            result += this.idBoxes[i].value;
            if (i < size - 1) {
                result += "|";
            }
        }
        this.listOrderField.value = result;
    },

    _moveLocal: function(index1, index2) {
        var row1 = Utils.findAncestor(this.idBoxes[index1], "TR");
        var row2 = Utils.findAncestor(this.idBoxes[index2], "TR");
        row1.parentNode.insertBefore(row1, row2);
        this._update();
    },

    _update: function() {
        var selectedCount = this.getSelectedItems().length;
        var selectedIndex = this.getSelectedIndex();
        var itemCount = this.idBoxes.length;
        var canMove = (selectedCount == 1 && itemCount > 1);

        this.selectAllBox.checked = (itemCount == selectedCount && itemCount != 0);
        Utils.setEnabled(this.selectAllBox, itemCount != 0);
        Utils.setEnabled(this.editButton, selectedCount == 1);
        Utils.setEnabled(this.deleteButton, selectedCount != 0 && itemCount > 0);
        Utils.setEnabled(this.moveUpButton, canMove && selectedIndex != 0);
        Utils.setEnabled(this.moveDownButton, canMove && selectedIndex != itemCount - 1);
    },

    _handleLoaded: function() {
        this.idBoxes = Utils.getFormInputs($("dlpform"), Tag.INPUT_TYPE_CHECKBOX, "dlpid");
        this.selectAllBox = $("selectall");
        this.addButton = $("additembtn");
        this.editButton = $("edititembtn");
        this.deleteButton = $("deleteitembtn");
        this.moveUpButton = $("moveupbtn");
        this.moveDownButton = $("movedownbtn");
        this.listOrderField = $("dlplistorder");

        this._bindEvents();
        this._adjustItemOrder();
        this._update();
    },

    _handleSubmit: function() {
        this.listener.onSubmit(this);
    },

    _handleSelectAll: function() {
        var selected = this.selectAllBox.checked;
        for (var i = 0, size = this.idBoxes.length; i < size; ++i) {
            this.idBoxes[i].checked = selected;
        }
        this._update();
    },

    _handleButtonPressed: function(e) {
        var button = Event.findAncestor(e, "INPUT");
        Event.stop(e);

        if (button == this.addButton) {
            this.listener.onAdd(this);
        } else if (button == this.editButton) {
            var id = this.idBoxes[this.getSelectedIndex()].value;
            this.listener.onEdit(id);
        } else if (button == this.deleteButton) {
            this.listener.onDelete(this)
        } else if (button == this.moveUpButton) {
            this.moveUp(this.getSelectedIndex());
            this.listener.onMoveUp(this);
        } else if (button == this.moveDownButton) {
            this.moveDown(this.getSelectedIndex());
            this.listener.onMoveDown(this);
        }
    },

    _bindEvents: function() {
        this.buttonHandler = this._handleButtonPressed.bindAsEventListener(this);
        this.updateHandler = this._update.bind(this);
        this.selectAllHandler = this._handleSelectAll.bind(this);

        for (var i = 0, size = this.idBoxes.length; i < size; ++i) {
            Event.observe(this.idBoxes[i], "click", this.updateHandler);
        }
        Event.observe(this.selectAllBox, "click", this.selectAllHandler);
        Event.observe(this.addButton, "click", this.buttonHandler);
        Event.observe(this.editButton, "click", this.buttonHandler);
        Event.observe(this.deleteButton, "click", this.buttonHandler);
        Event.observe(this.moveUpButton, "click", this.buttonHandler);
        Event.observe(this.moveDownButton, "click", this.buttonHandler);
    },

    _unbindEvents: function() {
        for (var i = 0, size = this.idBoxes.length; i < size; ++i) {
            Event.unobserve(this.idBoxes[i], "click", this.updateHandler);
        }
        Event.unobserve(this.selectAllBox, "click", this.selectAllHandler);
        Event.unobserve(this.addButton, "click", this.buttonHandler);
        Event.unobserve(this.editButton, "click", this.buttonHandler);
        Event.unobserve(this.deleteButton, "click", this.buttonHandler);
        Event.unobserve(this.moveUpButton, "click", this.buttonHandler);
        Event.unobserve(this.moveDownButton, "click", this.buttonHandler);

        this.buttonHandler = null;
        this.updateHandler = null;
        this.selectAllHandler = null
    }
});

DlpSettingsDialog.Listener = Class.create({
    onAdd: function(d) {},
    onEdit: function(d) {},
    onDelete: function(d) {},
    onMoveUp: function(d) {},
    onMoveDown: function(d) {},
    onSubmit: function(d) {},
    onCancel: function(d) {}
});