var VirtualFileController = Class.create({

    config: null,
    dlpController: null,
    view: null,

    _constructor: function(config) {
        this.config = Object.extend(Object.extend({}, VirtualFileController.DEFAULT_CONFIG), config);
        this.dlpController = new DlpSettingsController(config);
    },

    start: function() {
        this.view = new VirtualFileView();
        this.view.show(Object.extend(new VirtualFileView.Listener(), {
            onPgpEncryptionSettings: this._handleShowPgpEncryptionSettings.bind(this),
            onDlpSettings: this._handleShowDlpSettings.bind(this)
        }));
    },

    _handleShowPgpEncryptionSettings: function() {
        var dialog = new FormDialog("PgpEncryptionSettingsDialog", this.config.pgpsu);
        dialog.setSize(550, 280);
        dialog.show(Object.extend(new FormDialog.Listener(), {
            onDone: function(d) { d.dispose(); },
            onCancel: function(d) { d.dispose(); }
        }), this.config.metadata);
    },

    _handleShowDlpSettings: function() {
        this.dlpController.start();
    }

});

VirtualFileController.DEFAULT_CONFIG = { pgpsu: "", dlpsu: "", metadata: {}};

var VirtualFileView = Class.create({

    updateViewHandler: null,
    pgpEncryptionSettingsHandler: null,
    dlpSettingsHandler: null,
    listener: null,

    _constructor: function() {
        this.realPathType = $("ptreal");
        this.realPathField = $("realpath");
        this.resourcePathType = $("ptresource");
        this.resourcesField = $("resource");
        this.createIfNotFound = $("createifnotexists");
        this.includeInSearchIndex = $("searchindex");
        this.enablePgpEncryption = $("encryptuploads");
        this.pgpEncryptionSettings = $("encryptuploadssettings");
        this.enableDlp = $("dlp");
        this.dlpSettings = $("dlpsettings");
    },

    show: function(listener) {
        this.listener = listener || new VirtualFileView.Listener();
        this._bindEvents();
        this._update();
    },

    hide: function() {
        this._unbindEvents();
        this.listener = null;
    },

    _update: function() {
        var typeResource = this.resourcePathType.checked;
        var enable = !typeResource && this.realPathType.checked;

        Utils.setEnabled(this.realPathField, enable);
        Utils.setEnabled(this.createIfNotFound, enable);
        Utils.setEnabled(this.includeInSearchIndex, enable);
        Utils.setEnabled(this.enablePgpEncryption, enable);
        Utils.setEnabled(this.pgpEncryptionSettings, this.enablePgpEncryption.checked && enable);
        Utils.setEnabled(this.enableDlp, enable);
        Utils.setEnabled(this.dlpSettings, this.enableDlp.checked && enable);
        Utils.setEnabled(this.resourcesField, typeResource);
    },

    _handlePgpEncryptionSettings: function(e) {
        Event.stop(e);
        this.listener.onPgpEncryptionSettings(this);
    },

    _handleDlpSettings: function(e) {
        Event.stop(e);
        this.listener.onDlpSettings(this);
    },

    _bindEvents: function() {
        this.updateViewHandler = this._update.bind(this);
        this.pgpEncryptionSettingsHandler = this._handlePgpEncryptionSettings.bindAsEventListener(this);
        this.dlpSettingsHandler = this._handleDlpSettings.bindAsEventListener(this);

        Event.observe(this.realPathType, "click", this.updateViewHandler);
        Event.observe(this.resourcePathType, "click", this.updateViewHandler);
        Event.observe(this.pgpEncryptionSettings, "click", this.pgpEncryptionSettingsHandler);
        Event.observe(this.dlpSettings, "click", this.dlpSettingsHandler);
        Event.observe(this.enablePgpEncryption, "change", this.updateViewHandler);
        Event.observe(this.enableDlp, "change", this.updateViewHandler);
    },

    _unbindEvents: function() {
        Event.unobserve(this.realPathType, "click", this.updateViewHandler);
        Event.unobserve(this.resourcePathType, "click", this.updateViewHandler);
        Event.unobserve(this.pgpEncryptionSettings, "click", this.pgpEncryptionSettingsHandler);
        Event.unobserve(this.dlpSettings, "click", this.dlpSettingsHandler);
        Event.unobserve(this.enablePgpEncryption, "change", this.updateViewHandler);
        Event.unobserve(this.enableDlp, "change", this.updateViewHandler);

        this.updateViewHandler = null;
        this.pgpEncryptionSettingsHandler = null;
        this.dlpSettingsHandler = null;
    }

});

VirtualFileView.Listener = Class.create({
    onPgpEncryptionSettings: function(v) {},
    onDlpSettings: function(v) {},
    onSubmit: function(v) {},
    onCancel: function(v) {}
});