var FileChooser = Class.create({

    chooserElement: null,
    overflowElement: null,
    repositionHandler: null,
    listener: null,
    debug: false,

    createUploadQueue: function() {
        if (this.uploadQueue == null) {
            this.uploadQueue = this._doCreateUploadQueue();
        }
        return this.uploadQueue;
    },

    show: function(overflowElement, listener) {
        this.overflowElement = overflowElement;
        this.listener = listener;

        Utils.setEnabled(this.overflowElement, false);
        this._createChooserElement();
        this._bindEvents();
        this._position();
    },

    hide: function() {
        this._unbindEvents();
        Utils.setStyle(this.chooserElement, "visibility: hidden");
        Utils.setEnabled(this.overflowElement, true);
    },

    destroy: function() {
        this._unbindEvents();
        this._destroyChooserElement();

        this.overflowElement = null;
        this.listener = null;
    },

    asElement: function() {
        return this.chooserElement;
    },

    _doCreateUploadQueue: function() {
        throw "Not implemented";
    },

    _handleStateChanged: function(state) {
        if (/loaded|complete/.test(state)) {
            Utils.setEnabled(this.overflowElement, true);
            Utils.setStyle(this.chooserElement, "visibility: visible");

            this.listener.onReady(this);
        }
    },

    _handleFilesSelected: function(files) {
        var items = [];
        for (var i = 0; i < files.length; ++i) {
            items.push(new FileChooser.FileItem(files[i]));
        }

        this.listener.onFileSelect(items);
    },

    _handleError: function(name, message) {
        this._log("Error: " + name + ", " + message);
    },

    _bindEvents: function() {
        this.repositionHandler = this._position.bind(this);

        this.overflowElement.onclick = function(e) { Event.stop(e || window.event); };
        Event.observe(window, "load", this.repositionHandler);
        Event.observe(window, "resize", this.repositionHandler);
    },

    _unbindEvents: function() {
        this.overflowElement.onclick = null;
        Event.unobserve(window, "load", this.repositionHandler);
        Event.unobserve(window, "resize", this.repositionHandler);

        this.repositionHandler = null;
    },

    _createChooserElement: function() {
        throw "Not implemented";
    },

    _destroyChooserElement: function() {
        if (Object.isElement(this.chooserElement) && this.chooserElement.parentNode) {
            this.chooserElement.parentNode.removeChild(this.chooserElement);
        }
        this.chooserElement = null;
    },

    _position: function() {
        if (!Object.isElement(this.chooserElement)) {
            return;
        }

        var dimension = Utils.getDimension(this.overflowElement);
        var formOffset = document.viewport.getOffsetFor(this.overflowElement.form);
        var overflowElementOffset = document.viewport.getOffsetFor(this.overflowElement);
        //var scrollOffset = document.viewport.getScrollOffsets();

        Utils.setStyle(this.overflowElement.form, "position:relative");
        Utils.setStyle(this.chooserElement, {
            width: dimension.width + "px", height: dimension.height + "px",
            top: ((overflowElementOffset.top - formOffset.top) /*+ scrollOffset.top*/) + "px",
            left: ((overflowElementOffset.left - formOffset.left) /*+ scrollOffset.left*/) + "px",
            position: "absolute", zIndex: 1000, overflow: "hidden"
        });
    },

    _log: function(message) {
        if (this.debug) {
            if (window.console) {
                window.console.log(message);
            } else {
                alert(message);
            }
        }
    }
});

FileChooser.Listener = Class.create({
    onReady: function(fileChooser) {},
    onFileSelect: function(fileItems) {}
});

FileChooser.UploadQueue = Class.create({

    items: null,
    client: null,

    _constructor: function() {
        this.items = [];
    },

    add: function(fileItem) {
        this.items.push(fileItem);
        return true;
    },

    remove: function(fileItem) {
        var modified = false;
        var index = this.items.indexOf(fileItem);

        if (index != -1) {
            this.items[index] = null;
            this.items.splice(index, 1);
            modified = true;
        }
        return modified;
    },

    clear: function() {
        var copy = this.fileItems();
        for (var i = 0; i < copy.length; ++i) {
            this.remove(copy[i]);
        }
    },

    fileItems: function() {
        return this.items.clone()
    },

    isEmpty: function() {
        return this.items.length == 0;
    },

    send: function(url, listener) {
        throw "Not implemented";
    },

    abort: function() {
    }
});
FileChooser.UploadQueue.Listener = Class.create({
    onComplete: function() {},
    onError: function(message) {}
});


FileChooser.FileItem = Class.create({

    id: null,
    path: null,
    filename: null,
    size: null,

    _constructor: function(path, id) {
        this.id = id || FileChooser.FileItem.nextId();
        this.path = path || "";
        this.filename = this._resolveFilename(this.path);
        this.size = -1;
    },

    _resolveFilename: function(path) {
        var separator = path.indexOf("/") != -1 ? "/" : "\\";
        return path.substr(path.lastIndexOf(separator) + 1);
    },

    toString: function() {
        return this.filename;
    }
});
(function() {
    this._sequence = 0;
    this.nextId = function() { return ++this.sequence + new Date().getTime().toString(36);}
}).call(FileChooser.FileItem);
