var FormFileChooser = Class.create(FileChooser, {

    sequence: 0,
    uploadQueue: null,

    /** override */
    hide: function() {
        this._super.hide.apply(this, arguments);
        this._destroyChooserElement();
    },

    _doCreateUploadQueue: function() {
        return new FormFileChooser.UploadQueue(this);
    },

    _createChooserElement: function() {
        var chooser = Utils.createFileInput("filename" + this.sequence++);
        chooser.multiple = true;
        chooser.title = FormFileChooser.Input.TITLE;
        chooser.size = 1;
        chooser.tabIndex = -1;

        chooser.onchange = this._handleFilesSelected.bind(this);
        chooser.onkeypress = function(e) { Event.stop(e); this.blur(); };

        this.chooserElement = this.overflowElement.form.insertBefore(chooser, this.overflowElement);
        this._handleStateChanged("complete");
    },

    _position: function() {
        if (Object.isElement(this.chooserElement)) {
            Utils.addCssClass(this.chooserElement, "filechooser");
        }
    },

    _destroyChooserElement: function() {
        this.chooserElement.onchange = null;
        this.chooserElement.onkeypress = null;
        this._super._destroyChooserElement.apply(this, arguments);
    },

    /** override */
    _handleStateChanged: function(state) {
        if (this.sequence == 1) {
            this._super._handleStateChanged.apply(this, arguments);
        }
    },

    _handleFilesSelected: function() {
        var selectedChooser = this.chooserElement;
        Utils.setStyle(selectedChooser, "position:absolute; left:-10000px; visibility:hidden");
        this._createChooserElement();
        this._position();

        var fileItems = [];
        if (this._isMultipleChooser(selectedChooser)) {
            var fileList = selectedChooser.files;
            for (var i = 0; i < fileList.length; ++i) {
                var item = new FormFileChooser.FileItem(fileList[i].name);
                item.fileInput = selectedChooser;
                fileItems.push(item)
            }
        } else {
            fileItems.push(FormFileChooser.FileItem.fromChooser(selectedChooser));
        }

        this.listener.onFileSelect(fileItems);
    },

    _isMultipleChooser: function(chooser) {
        return window.FileList && chooser.files instanceof window.FileList;
    }
});
FormFileChooser.Css = {FILE_INPUT: "filechooser"};
FormFileChooser.Input = {TITLE: ""};


FormFileChooser.FileItem = Class.create(FileChooser.FileItem, {
    fileInput: null,
    destroy: function() {
        if (Object.isElement(this.fileInput)) {
            this.fileInput.onclick = this.fileInput.onchange = this.fileInput.onkeypress = null;
            if (this.fileInput.parentNode) {
                this.fileInput.parentNode.removeChild(this.fileInput);
            }
        }
        this.fileInput = null;
    }
});
FormFileChooser.FileItem.fromChooser = function(chooser) {
    var fileItem = new FormFileChooser.FileItem(chooser.value);
    fileItem.fileInput = chooser;
    return fileItem;
};
FormFileChooser.FileItem.multipleChooser = function(chooser) {
    var fileItem = new FormFileChooser.FileItem("");
    fileItem.fileInput = chooser;
    return fileItem;
};


FormFileChooser.UploadQueue = Class.create(FileChooser.UploadQueue, {

    form: null,
    dataChannel: null,
    listener: null,

    _constructor: function(fileChooser) {
        if (!Object.isElement(fileChooser.asElement().form)) {
            throw "Invalid form element";
        }

        this.client = "html_form";
        this.form = fileChooser.asElement().form;
        this.form.acceptCharset = "utf-8";
        this.form.enctype = "multipart/form-data";
        this.form.onsubmit = this._onBeforeSubmit.bind(this);

        this.dataChannel = new IFrameChannel({timeout: 0});
        this.dataChannel.open(null, null, {
            onComplete: this._handleComplete.bind(this),
            onError: this._handleError.bind(this)
        });
    },

    add: function(item) {
        if (!(item instanceof FormFileChooser.FileItem)) {
            throw "Invalid file item.";
        }
        if (item.fileInput.form !== this.form) {
            throw "Invalid file input owner form.";
        }
        item.fileInput.disabled = true;

        return this._super.add.call(this, item);
    },

    remove: function(item) {
        if (this._super.remove.apply(this, arguments)) {
            item.destroy();
            return true;
        }
        return false;
    },

    send: function(url, listener) {
        this.form.action = url.toString();
        this.listener = listener || new FileChooser.UploadQueue.Listener();

        this._onBeforeSubmit();
        this.dataChannel.submitForm(this.form);
    },

    abort: function() {
        if (this.dataChannel) {
            this.dataChannel.close();
        }
        this._super.abort.call(this);
    },

    _onBeforeSubmit: function() {
        for (var i = 0; i < this.items.length; ++i) {
            this.items[i].fileInput.disabled = false;
        }
    },

    _handleComplete: function() {
        this.listener.onComplete();
    },

    _handleError: function() {
        this.listener.onError();
    }
});