var jscape = jscape || {};

/** @class */
jscape.UploadForm = Class.create(
    /** @lends {jscape.UploadForm.prototype} */
{
    listener: null,

    /** @constructor */
    _constructor: function (url, metadata, required) {
        this.url = url;
        this.metadata = metadata || {};
        this.required = !!required;
        this.forms = [];
        this.files = [];
    },

    setForms: function (forms) {
        for (var i = 0; i < forms.length; ++i) {
            var form = forms[i];
            this.forms.push({name: form.name, description: form.desc, batch: form.batch});
        }
    },

    start: function (files, listener) {
        this.listener = listener || jscape.UploadForm.LISTENER;
        this.files = files.clone();

        if (this.forms.length) {
            if (this.required && this.forms.length == 1) {
                this._fillFormData(this.forms[0].name);
            } else {
                this._showFormSelectionDialog();
            }
        } else {
            this.listener.onStartUpload();
        }
    },

    _showFormSelectionDialog: function () {
        var dialog = new jscape.UploadFormDialog(this.forms, this.required);
        dialog.show({
            onSubmit: this._onSubmit$FormSelection.bind(this),
            onCancel: this._onCancel$FormSelection.bind(this),
            onCancelUpload: this._onCancelUpload$FormSelection.bind(this)
        });
    },

    _showUploadFormDialog: function (formName, filename, callback) {
        var listener = Object.extend(new FormDialog.Listener(), {
            onBeforeSubmit: this._onBeforeSubmit$UploadForm.bind(this),
            onCancel: this._onCancel$UploadForm.bind(this),
            onDone: function(d) { d.dispose(); callback(); }
        });

        var dialog = new FormDialog("upload-form-dialog", this.url);
        var viewport = document.viewport.getDimensions();
        dialog.setSize(viewport.width * 0.5, viewport.height * 0.6);
        dialog.show(listener, Object.extend({
            form: formName,
            filename: filename
        }, this.metadata));
    },

    _onBeforeSubmit$UploadForm: function (dialog) {
        if (!this._isValidForm(dialog.getForm())) {
            ModalDialog.alert(jscape.UploadForm.Messages.INVALID_REQUIRED, jscape.UploadForm.Messages.TITLE);
            return false;
        }
        return true;
    },

    _onCancel$UploadForm: function (dialog) {
        dialog.dispose();
        this._showFormSelectionDialog();
    },

    _onSubmit$FormSelection: function (dialog, formName) {
        dialog.hide();
        this._fillFormData(formName, this.files);
    },

    _onCancel$FormSelection: function (dialog) {
        dialog.hide();
        this.listener.onStartUpload();
    },

    _onCancelUpload$FormSelection: function (dialog) {
        dialog.hide();
        this.listener.onCancelUpload();
    },

    _fillFormData: function(formName) {
        var form = this._findFormNamed(formName);
        if (form.batch) {
            this._batchUploadFormData(form.name, this.files);
        } else {
            this._perFileUploadFormData(form.name, this.files.clone());
        }
    },

    _batchUploadFormData: function(formName, files) {
        this._showUploadFormDialog(formName, files, function() {
            this.listener.onStartUpload();
        }.bind(this));
    },

    _perFileUploadFormData: function(formName, files) {
        this._showUploadFormDialog(formName, files.pop(), function() {
            if (files.length) {
                this._perFileUploadFormData(formName, files);
            } else {
                this.listener.onStartUpload();
            }
        }.bind(this));
    },

    _findFormNamed: function(name) {
        for (var i = 0; i < this.forms.length; i++) {
            var form = this.forms[i];
            if (form.name == name) {
                return form;
            }
        }
        return null;
    },

    _isValidForm: function (form) {
        for (var i = 0; i < form.elements.length; ++i) {
            var input = form.elements[i];
            var requiredNode = input.getAttributeNode("required");
            var isRequiredField = Tag.INPUT_TYPE_HIDDEN != input.type && (requiredNode && requiredNode.specified);

            if (!isRequiredField) {
                continue;
            }

            if (Utils.isRadioInput(input) && form.elements[input.name].length > 1) {
                var radioGroup = form.elements[input.name];
                if (!this._hasSelectedItem(radioGroup)) {
                    return false;
                }
            } else {
                var value = $F(input) || "";
                if (value.trim().length == 0) {
                    return false;
                }
            }
        }
        return true;
    },

    _hasSelectedItem: function (items) {
        for (var i = 0; i < items.length; ++i) {
            if (items[i].checked) {
                return true;
            }
        }
        return false;
    }
});
jscape.UploadForm.Messages = {TITLE: "Form Data", INVALID_REQUIRED: "Please provide all required fields."};
jscape.UploadForm.LISTENER = {
    onStartUpload: function () {},
    onCancelUpload: function () {}
};

/** @class */
jscape.UploadFormDialog = Class.create(ModalDialog,
    /** @lends {jscape.UploadFormDialog.prototype} */
{
    /** @constructor */
    _constructor: function(forms, required) {
        this.formsBox = document.createElement("SELECT");
        this.formsBox.name = "formname";

        this.formsBox.id = this.formsBox.name;
        this.formsBox.onchange = this._update.bind(this);
        this.formDescElement = document.createElement(Tag.SPAN);
        this.formDescElement.id = "formdescription";
        this.setContent(this._createContent(forms));
        this.addButton(ModalDialog.BUTTON_OK, jscape.UploadFormDialog.Buttons.OK);
        if (!required) {
            this.addButton(ModalDialog.BUTTON_CANCEL, jscape.UploadFormDialog.Buttons.CANCEL);
        }
        this.addButton(ModalDialog.BUTTON_CANCEL, jscape.UploadFormDialog.Buttons.CANCEL_UPLOAD, this.onCancelUpload.bind(this));
        this.setTitle(jscape.UploadForm.Messages.TITLE);
    },
    getId: function() {
        return "UploadFormSelectionDialog";
    },
    show: function(listener) {
        var viewport = document.viewport.getDimensions();
        var width = Math.max(400, viewport.width * 0.4);
        var height = Math.max(245, viewport.height * 0.37);
        this._super.setSize.call(this, width, height);
        this._update();
        this._super.show.call(this, Object.extend(Object.extend({}, jscape.UploadFormDialog.LISTENER), listener || {}));
    },
    hide: function() {
        this._super.hide.call(this);
        this.dispose();
    },
    onSubmit: function() {
        this.listener.onSubmit(this, $F(this.formsBox));
    },
    onCancel: function() {
        this.listener.onCancel(this);
    },
    onCancelUpload: function() {
        this.listener.onCancelUpload(this);
    },
    _createContent: function(forms) {
        for (var i = 0; i < forms.length; ++i) {
            var form = forms[i];
            var item = document.createElement(Tag.OPTION);
            item.appendChild(document.createTextNode(form.name));
            item.value = form.name;
            item.title = form.description;
            this.formsBox.appendChild(item);
        }
        var panel = document.createElement(Tag.DIV);
        var infoPanel = document.createElement(Tag.DIV);
        infoPanel.appendChild(document.createTextNode(jscape.UploadFormDialog.Messages.INFO));
        Utils.setStyle(infoPanel, "padding: 2em 1.5em 2em 4em;");
        panel.appendChild(infoPanel);
        var formPanel = document.createElement(Tag.DIV);
        formPanel.appendChild(Utils.createLabel(jscape.UploadFormDialog.Labels.FORM, this.formsBox.id));
        formPanel.appendChild(this.formsBox);
        formPanel.appendChild(document.createElement("BR"));
        formPanel.appendChild(Utils.createLabel(jscape.UploadFormDialog.Labels.DESCRIPTION, this.formDescElement.id));
        formPanel.appendChild(this.formDescElement);
        Utils.setStyle(formPanel, "padding-left: 5em; line-height: 2");
        panel.appendChild(formPanel);
        return panel;
    },
    _update: function() {
        if (Object.isElement(this.formsBox)) {
            var description = this._descriptionFor($F(this.formsBox));
            this._setDescription(description || "");
        }
    },
    _descriptionFor: function(name) {
        for (var i = 0; i < this.formsBox.options.length; ++i) {
            var item = this.formsBox.options[i];
            if (name == item.value) {
                return item.title;
            }
        }
        return null;
    },
    _setDescription: function(description) {
        var descElement = document.createTextNode(description);
        if (this.formDescElement.firstChild) {
            this.formDescElement.replaceChild(descElement, this.formDescElement.firstChild);
        } else {
            this.formDescElement.appendChild(descElement);
        }
    }
});
jscape.UploadFormDialog.Buttons = {OK: "OK", CANCEL: "Cancel", CANCEL_UPLOAD: "Cancel Upload"};
jscape.UploadFormDialog.Labels = {FORM: "Form", DESCRIPTION: "Description"};
jscape.UploadFormDialog.Messages = {INFO: "Select form to use for file upload and click OK or click Cancel to upload files without form data."};
jscape.UploadFormDialog.LISTENER = {
    onSubmit: function(dialog) {},
    onCancel: function(dialog) {},
    onCancelUpload: function (dialog) {}
};