var jscape = jscape || {};

/** @class */
jscape.UploadView = Class.create(
    /** @lends {jscape.UploadView.prototype} */
{
    /** @const */
    CSS_BUTTON_DISABLED: "disabled",

    listener: null,

    /** @constructor */
    _constructor: function (fileChooser) {
        this.addButton = $("addfile");
        this.clearButton = $("clearfilelist");
        this.fileCounter = $("filecounter");
        this.uploadButton = $("startupload");
        this.container = $("uploadcontainer");
        this.fileList = new List($("filelist"));
        this.fileChooser = fileChooser;
    },

    show: function (listener) {
        this.listener = listener || jscape.UploadView.LISTENER;

        Utils.setStyle(this.container, "visibility:visible;height:auto");
        this.fileChooser.show(this.addButton, {
            onReady: this._handleChooserLoaded.bind(this),
            onFileSelect: this._handleFilesChoosed.bind(this)
        });

        this._bindEvents();
        this.update();
    },

    hide: function () {
        this._unbindEvents();
        Utils.setStyle(this.container, "visibility:hidden;height:0");
        this.fileChooser.hide();
        this.fileChooser = null;
        this.listener = null;
    },

    update: function () {
        var count = this.fileList.getItemCount();
        this.fileCounter.innerHTML = count;
        this._setButtonEnabled(this.uploadButton, count != 0);
        this._setButtonEnabled(this.clearButton, this.fileList.getItemCount() != 0);
    },

    getForm: function () {
        return this.uploadButton.form;
    },

    addFile: function (fileItem) {
        this.fileList.addItem(fileItem.filename);
    },

    removeFile: function (fileItem) {
        this.fileList.removeItem(fileItem.filename);
    },

    clear: function () {
        this.fileList.clear();
    },

    /** property change listener */
    propertyChange: function (event) {
        var property = event.propertyName;
        if (List.PCS_SELECTED_INDEX == property || List.PCS_MODEL == property) {
            this.update();
        }
    },

    _setButtonEnabled: function (button, enable) {
        Utils.setEnabled(button, enable, this.CSS_BUTTON_DISABLED);
    },

    _handleChooserLoaded: function () {
        this.listener.onReady(this.fileChooser);
    },

    _handleFilesChoosed: function (files) {
        this.listener.onChooseFiles(files);
    },

    _showNoFilesSelectedMessage: function () {
        ModalDialog.alert(Uploader.Messages.NO_FILES_SELECTED);
    },

    _showConnectionTimeoutMessage: function () {
        ModalDialog.alert(Uploader.Messages.CONNECTION_TIMEOUT);
    },

    _showConnectionErrorMessage: function (message) {
        ModalDialog.alert(message || Uploader.Messages.TRANSFER_ERROR);
    },

    _showConfirmOverwriteMessage: function (files, handler) {
        var html = [];
        html.push(Uploader.Messages.CONFIRM_OVERWRITE);
        html.push("<b>");
        for (var i = 0; i < files.length; ++i) {
            html.push(files[i].filename); //bug in IE #toString()
        }
        html.push("</b>");
        var message = html.join("<br/>");

        ModalDialog.confirm(document.title, message, {
            'Yes': function (dialog) {
                dialog.dispose();
                handler.apply(handler, [true, files]);
            },
            'No': function (dialog) {
                dialog.dispose();
                handler.apply(handler, [false, files]);
            }
        });
    },

    _handleClearButtonPressed: function (e) {
        Event.stop(e);
        this.listener.onRemoveAll();
    },

    _handleUploadButtonPressed: function (e) {
        Event.stop(e);
        // this.fileChooser.hide();
        this.listener.onStartUpload();
    },

    _bindEvents: function () {
        this.clearButtonHandler = this._handleClearButtonPressed.bindAsEventListener(this);
        this.uploadButtonHandler = this._handleUploadButtonPressed.bindAsEventListener(this);

        Event.observe(this.clearButton, "click", this.clearButtonHandler);
        Event.observe(this.uploadButton, "click", this.uploadButtonHandler);
        this.fileList.addPropertyChangeListener(this);
    },

    _unbindEvents: function () {
        Event.unobserve(this.clearButton, "click", this.clearButtonHandler);
        Event.unobserve(this.uploadButton, "click", this.uploadButtonHandler);
        this.fileList.removePropertyChangeListener(this);

        this.clearButtonHandler = null;
        this.uploadButtonHandler = null;
    }
});
jscape.UploadView.LISTENER = {
    onReady: function(chooser) {},
    onChooseFiles: function(fileItems) {},
    onRemoveAll: function() {},
    onStartUpload: function() {}
};

/** @class */
jscape.ProgressMeter = Class.create(
    /** @lends {jscape.ProgressMeter.prototype} */
{
    /** @const */
    STATUS: {Queued: 'QUEUED', InProgress: 'IN_PROGRESS', Complete: 'COMPLETE', Canceled: 'CANCELED', Error: 'ERROR'},

    callback: null,
    listener: null,
    sessionId: null,

    /** @constructor */
    _constructor: function(config) {
        this.url = config.upu;
        this.metadata = config.metadata || {};

        this.channel = new ScriptChannel({timeout: config.connectionTimeout || 0, preventCaching: true});
        this.timer = new Timer(this._poll.bind(this));
        this.timer.setRepeats(true);
        this.timer.setDelay(1000);
    },

    start: function(listener, sessionId) {
        this.listener = listener || jscape.ProgressMeter.LISTENER;
        this.sessionId = sessionId;
        this.callback = this._handleResponse.bind(this);

        this.channel.open(this.url, Object.extend(new Channel.Listener(), {
            onTimeout: this._handleTimeout.bind(this)
        }));

        this.timer.start();
        this._poll(); // initial delay 0ms
    },

    interrupt: function() {
        this.timer.stop();
        if (this.channel) {
            this.channel.close();
        }
    },

    finish: function() {
        this.timer.stop();
        if (this.channel) {
            this.channel.close();
        }
        this.timer = null;
        this.channel = null;
        this.callback = null;
        this.sessionId = null;
    },

    _poll: function() {
        this.channel.send(Object.extend({id: this.sessionId || ""}, this.metadata), this.callback);
    },

    _handleResponse: function(data) {
        var copy = Object.extend({}, data);
        var state = (copy.state || "").toUpperCase();

        switch (state) {
            case this.STATUS.Queued:
            case this.STATUS.InProgress:
                this._handleProgress(copy);
                break;
            case this.STATUS.Complete:
                this._handleComplete(copy);
                break;
            case this.STATUS.Canceled:
                this._handleCancel();
                break;
            case this.STATUS.Error:
                this._handleError(copy.statusMessage);
                break;
            default:
                this._handleError("Unable to obtain file upload progress.");
                break;
        }
    },

    _handleProgress: function(data) {
        var percent = parseInt(data.percent) || -1;
        var statusMessage = data.statusMessage || "";
        var timeMessage = data.timeMessage || "";

        this.listener.onProgress(percent, statusMessage, timeMessage);
    },

    _handleComplete: function(data) {
        this.timer.stop();
        if (data && data.statusMessage && data.timeMessage) {
            this.listener.onProgress(100, data.statusMessage, data.timeMessage);
        }
        this.listener.onComplete();
    },

    _handleCancel: function() {
        this.timer.stop();
        this.listener.onCancel();
    },

    _handleError: function(message) {
        this.timer.stop();
        this.listener.onError(message)
    },

    _handleTimeout: function() {
        this.timer.stop();
        this.listener.onTimeout();
    }
});
jscape.ProgressMeter.LISTENER = {
    onProgress: function(percent, statusMessage, timeMessage) {},
    onComplete: function() {},
    onCancel: function() {},
    onTimeout: function() {},
    onError: function(message) {}
};

/** @class */
jscape.ProgressBar = Class.create(
    /** @lends {jscape.ProgressBar.prototype} */
{
    /** @const */
    CSS: {CONTAINER: "progressbar", PROGRESS_TEXT: "progress", BOX_RECTANGLE: "bar"},
    /** @const */
    REPAINT_INTERVAL: 55,
    MOVING_BOX_WIDTH: 5,
    MOVING_STEP: 3,

    container: null,
    boxRectangle: null,
    progressText: null,
    progress: null,
    indeterminate: null,
    boxWidth: null,
    boxPosition: null,
    boxDirection: null,
    timer: null,

    /** @constructor */
    _constructor: function (container) {
        this._compose(container);

        this.progress = 0;
        this.boxPosition = 0;
        this.boxWidth = this.MOVING_BOX_WIDTH;
        this.indeterminate = false;
        this.setVisible(false);
    },

    isVisible: function () {
        return Utils.isVisible(this.container);
    },

    setVisible: function (value) {
        Utils.setVisible(this.container, value);
    },

    setEnabled: function (value) {
        if (!value && this.indeterminate) {
            this.setProgress(0);
        }
    },

    isIndeterminate: function () {
        return this.indeterminate;
    },

    setIndeterminate: function (value) {
        if (this.indeterminate !== value) {
            this.indeterminate = value;
            if (value) {
                this._startAnimation();
            } else {
                this._stopAnimation();
            }
        }
    },

    setProgress: function (value) {
        if (value == -1) {
            this.setIndeterminate(true);
        } else if (this.indeterminate) {
            this.setIndeterminate(false);
        }

        this.progress = Math.max(0, Math.min(100, value));
        this._repaint();
    },

    _compose: function (parent) {
        this.container = document.createElement("DIV");
        Utils.addCssClass(this.container, this.CSS.CONTAINER);

        this.progressText = document.createElement("SPAN");
        Utils.addCssClass(this.progressText, this.CSS.PROGRESS_TEXT);
        this.container.appendChild(this.progressText);

        this.boxRectangle = document.createElement("SPAN");
        Utils.addCssClass(this.boxRectangle, this.CSS.BOX_RECTANGLE);
        this.container.appendChild(this.boxRectangle);

        if (parent.firstChild) {
            parent.insertBefore(this.container, parent.firstChild);
        } else {
            parent.appendChild(this.container);
        }
    },

    _repaint: function () {
        if (!this.isVisible()) {
            return;
        }

        if (!this.indeterminate) {
            var percent = this.progress + "%";
            this.progressText.innerHTML = percent;
            Utils.setStyle(this.boxRectangle, {width: percent});
        } else {
            if ((this.boxPosition) >= (100 - this.boxWidth)) {
                this.boxDirection = -1;
            } else if (this.boxPosition <= 0) {
                this.boxDirection = 1;
            }

            this.boxPosition += (this.boxDirection * this.MOVING_STEP);
            var posPercent = Math.max(0, Math.min(100 - this.boxWidth, this.boxPosition)) + "%";
            Utils.setStyle(this.boxRectangle, {left: posPercent});
        }
    },

    _startAnimation: function () {
        Utils.setStyle(this.progressText, {visibility: "hidden"});
        Utils.setStyle(this.boxRectangle, {width: this.boxWidth + "%", position: "relative", left: this.boxPosition
                + "%"});

        this.timer = window.setInterval(function () {
            this._repaint();
        }.bind(this), this.REPAINT_INTERVAL);
    },

    _stopAnimation: function () {
        window.clearInterval(this.timer);
        this.timer = null;

        Utils.setStyle(this.progressText, {visibility: "visible"});
        Utils.setStyle(this.boxRectangle, "position:inherit;left:auto"); // bug in IE7
        this._repaint();
    }
});

/** @class */
jscape.ProgressView = Class.create(
    /** @lends {jscape.ProgressView.prototype} */
{
    /** @constructor */
    _constructor: function (listener) {
        this.container = $("uploadprogress");
        this.statusText = $("uploadstatus");
        this.timeText = $("uploadtime");
        this.progressBar = new jscape.ProgressBar($("pbcontainer"));
        this.cancelButton = $("cancelupload");
        this.cancelButton.onclick = listener.onCancelUpload.bind(listener);
    },

    setVisible: function (value) {
        this.container.style.display = value ? "block" : "none";
        this.progressBar.setVisible(value);
        this.setEnabled(value);

        Utils.setVisible($("logininfo"), !value);
        Utils.setVisible($("messagebar"), !value);
        Utils.setVisible($("buttonsbar"), !value);
        Utils.setVisible($("search"), !value);
        Utils.setVisible($("list"), !value);
        Utils.setVisible($("tagcloud"), !value);
    },

    setEnabled: function (value) {
        this.progressBar.setEnabled(value);
        Utils.setEnabled(this.cancelButton, value);
    },

    setProgress: function (value) {
        this.progressBar.setProgress(value);
    },

    setStatus: function (text) {
        this._setMessage(this.statusText, text);
    },

    setTime: function (text) {
        this._setMessage(this.timeText, text);
    },

    _setMessage: function (holder, text) {
        var node = document.createTextNode(text);
        if (holder.firstChild) {
            holder.replaceChild(node, holder.firstChild);
        } else {
            holder.appendChild(node);
        }
    }
});