function $(id) {
    return document.getElementById(id);
}
function $F(elem) {
    return Utils.isSelect(elem)
            ? Utils.getSelectedValue(elem)
            : ((Utils.isRadioInput(elem) || Utils.isCheckboxInput(elem))
                ? (elem.checked ? elem.value : null)
                : (Utils.isInputTag(elem, Tag.INPUT_TYPE_TEXT) || Utils.isTag(elem, Tag.TEXTAREA) ? elem.value : null));
}
function $A(iterable) {
    var result = [];
    if (iterable) {
        for (var i = 0, len = iterable.length || 0; i < len; ++i) {
            result[i] = iterable[i];
        }
    }
    return result;
}
(function() {
    function extend(target, source) {
        for (var property in source) {
            target[property] = source[property];
        }
        return target;
    }
    function isArray(obj) {
        return obj != null && typeof obj === "object" && 'splice' in obj && 'join' in obj;
    }
    function isFunction(obj) {
        return typeof obj === "function";
    }
    function isString(obj) {
        return typeof obj === "string";
    }
    function isUndefined(obj) {
        return typeof obj === "undefined";
    }
    function isElement(obj) {
        return !!(obj && obj.nodeType == 1);
    }

    extend(Object, {
        extend: extend,
        isArray: isArray,
        isElement: isElement,
        isFunction: isFunction,
        isString: isString,
        isUndefined: isUndefined
    })
})();

function Class() {
    var args = $A(arguments);
    function instance() {
        if (this._super && this._super._constructor) {
            this._super._constructor.apply(this, arguments);
        }
        this._constructor.apply(this, arguments);
    }
    if (Object.isFunction(args.first())) {
        var parent = args.shift();
        function _super(){};
        _super.prototype = parent.prototype;
        instance.prototype = new _super();
        instance.prototype._super = parent.prototype;
    }
    for (var i = 0, size = args.length; i < size; ++i) {
        var properties = args[i];
        Object.extend(instance.prototype, properties);
    }
    if (!instance.prototype._constructor) {
        instance.prototype._constructor = function() {};
    }
    instance.prototype.constructor = instance;
    return instance;
}
Class.create = function() {
    return Class.apply(this, arguments);
};

Object.extend(Function.prototype, {
    bind: function(context) {
        if (arguments.length < 2 && Object.isUndefined(arguments[0])) return this;
        var self = this, args = Array.prototype.slice.call(arguments, 1);
        return function() {
            return self.apply(context, args.concat($A(arguments)));
        };
    },
    bindAsEventListener: function(context) {
        var self = this, args = Array.prototype.slice.call(arguments, 1);
        return function(event) {
            return self.apply(context, [event || window.event].concat(args));
        };
    },
    delay: function(timeout) {
        var self = this, args = Array.prototype.slice.call(arguments, 1);
        return window.setTimeout(function() {
            return self.apply(self, args);
        }, timeout);
    }
});

Object.extend(Array.prototype, {
    first: function() { return this[0] },
    last: function() { return this[this.length - 1]; },
    clone: function() { return [].concat(this); }
});
if (!Array.prototype.indexOf) {
    Array.prototype.indexOf = function(needle, offset) {
        for (var i = offset || 0, size = this.length; i < size; ++i) {
            if (this[i] === needle) {
                return i;
            }
        }
        return -1;
    };
}
Array.prototype.contains = function(value) {
    return this.indexOf(value, 0) != -1;
};
String.prototype.trim = function () {
    return this.replace(/^\s+|\s+$/g, "");
};
Boolean.toBoolean = function(value) {
    return (/^true$/i).test(value);
};

document.viewport = {
    getDimensions: function() {
        return Utils.Browser.Safari && !document.evaluate
                ? {width: window.innerWidth, height: window.innerHeight}
                : Utils.Browser.Opera
                ? {width: document.body.clientWidth, height: document.body.clientHeight}
                : {width: document.documentElement.clientWidth, height: document.documentElement.clientHeight};
    },

    getScrollOffsets: function() {
        var top = window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop;
        var left = window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft;
        return {top: top, left: left};
    },

    getOffsetFor: function(elem) {
        var result = {top: 0, left: 0};

        for (var el = elem; el != null; el = el.offsetParent) {
            result.top += el.offsetTop || 0;
            result.left += el.offsetLeft || 0;
            // Safari bug
            if (el.offsetParent == document.body && el.style.position == 'absolute') {
                break;
            }
        }

        for (var el = elem; el != null; el = el.parentNode) {
            if (!Utils.Browser.Opera || el.tagName == Tag.BODY) {
                result.top -= el.scrollTop || 0;
                result.left -= el.scrollLeft || 0;
            }
        }

        return result;
    }
};

function Utils() {
    this._constructor();
}(function() { // create local scope;
    Utils.Browser = {
        IE: !+"\v1",
        IEMac: (navigator.userAgent.indexOf("Mac") != -1) && (navigator.userAgent.indexOf("MSIE") != -1),
        Opera: !!window.opera,
        Gecko: (navigator.product == "Gecko"),
        Safari: (navigator.userAgent.indexOf("Safari") != -1)
    };

    // prior IE9 name and type properties are read-only
    var createElementSupportHtml = false;
    (function() {
        try {
            var name = "fakeinput" + new Date().getTime();
            var el = document.createElement("<input name=\"" + name + "\"");
            createElementSupportHtml = el.name == name;
            el = name = null;
        } catch (ignored) {}
    })();

    Utils.getInnerText = function(elem) {
        if (Object.isString(elem) || Object.isUndefined(elem)) {
            return elem;
        }
        if (elem.nodeType == Node.TEXT) {
            return Utils.getInnerText(elem.parentNode);
        }

        var result = "";
        for (var el = elem.firstChild; el != null; el = el.nextSibling) {
            switch (el.nodeType) {
                case Node.ELEMENT:
                    result += Utils.getInnerText(el);
                    break;
                case Node.TEXT:
                    result += el.nodeValue;
                    break;
            }
        }
        var whitespaceRegexp = /\u00A0/g;
        return result.replace(whitespaceRegexp, " ");
    };

    Utils.getTextNode = function(elem) {
        if (Object.isUndefined(elem)) {
            return null;
        }

        for (var el = elem.firstChild; el != null; el = el.nextSibling) {
            if (el.nodeType == Node.TEXT) {
                return el;
            }
        }

        return null;
    };

    Utils.compareIgnoreCase = function(str1, str2) {
        if (str1 != null && str2 != null) {
            var s1 = str1.toString().toLowerCase();
            var s2 = str2.toString().toLowerCase();

            return (s1 == s2 ? 0 : (s1 < s2 ? -1 : 1));
        }

        return 0;
    };

    Utils.equalsIgnoreCase = function(str1, str2) {
        return (Utils.compareIgnoreCase(str1, str2) == 0);
    };

    Utils.createLabel = function(title, htmlFor) {
        var result = document.createElement(Tag.LABEL);
        result.appendChild(document.createTextNode(title));
        result.htmlFor = Object.isString(htmlFor) ? htmlFor : "";

        return result;
    };

    Utils.createFileInput = function(name) {
        return Utils.createInput(Tag.INPUT_TYPE_FILE, name);
    };

    Utils.createRadioInput = function(name) {
        return Utils.createInput(Tag.INPUT_TYPE_RADIO, name);
    };

    Utils.createTextInput = function(name) {
        return Utils.createInput(Tag.INPUT_TYPE_TEXT, name);
    };

    Utils.createInput = function(type, name, value) {
        var result = null;

        if (createElementSupportHtml && !Utils.Browser.Opera) {
            result = document.createElement("<" + Tag.INPUT + " type=\"" + type + "\" name=\"" + name + "\" />");
        } else if (document.createElement) {
            result = document.createElement(Tag.INPUT);
            result.type = type;
            result.name = name;
        }
        if (!Object.isUndefined(value)) {
            result.value = value;
        }

        return result;
    };

    Utils.createOption = function(text, value) {
        var option = document.createElement(Tag.OPTION);
        option.appendChild(document.createTextNode(text));
        option.value = value || text;
        option.title = text;
        return option;
    };

    Utils.isTag = function(elem, tagName) {
        return (elem != null && elem.nodeType == Node.ELEMENT && elem.tagName == tagName);
    };

    Utils.isInputTag = function(elem, type) {
        return (Utils.isTag(elem, Tag.INPUT) && elem.type == type);
    };

    Utils.isCheckboxInput = function(elem) {
        return Utils.isInputTag(elem, Tag.INPUT_TYPE_CHECKBOX);
    };

    Utils.isRadioInput = function(elem) {
        return Utils.isInputTag(elem, Tag.INPUT_TYPE_RADIO);
    };

    Utils.isFileInput = function(elem) {
        return Utils.isInputTag(elem, Tag.INPUT_TYPE_FILE);
    };

    Utils.getFormInputs = function(form, type, name) {
        var inputs = form.getElementsByTagName(Tag.INPUT);
        if (!type && !name) { return inputs; }

        var result = new Array();
        for (var i = 0; i < inputs.length; ++i) {
            var victim = inputs[i];
            if ((type && victim.type != type) || (name && victim.name != name)) {
                continue;
            }
            result.push(victim);
        }
        return result;
    };

    Utils.isSelect = function(elem) {
        return Utils.isTag(elem, Tag.SELECT);
    };

    Utils.clearSelect = function(elem) {
        if (Utils.isSelect(elem)) {
            elem.options.length = 0;
        }
    };

    Utils.getSelectedValue = function(select) {
        return (Utils.isSelect(select) && select.selectedIndex != -1)
                ? select.options[select.selectedIndex].value
                : null;
    };

    Utils.isBody = function(elem) {
        return Utils.isTag(elem, Tag.BODY);
    };

    Utils.isLink = function(elem) {
        return Utils.isTag(elem, Tag.LINK);
    };

    Utils.isCell = function(elem) {
        return Utils.isTag(elem, Tag.TABLE_CELL);
    };

    Utils.isHeader = function(elem) {
        return Utils.isTag(elem, Tag.TABLE_HEAD_CELL)
                || (Utils.isCell(elem) && Utils.isTag(elem.parentNode.parentNode, Tag.TABLE_HEAD));
    };

    Utils.isRow = function(elem) {
        return Utils.isTag(elem, Tag.TABLE_ROW);
    };

    Utils.isHeaderRow = function(elem) {
        if (!Utils.isRow(elem) || elem.cells.length == 0) {
            return false;
        }

        for (var i = 0; i < elem.cells.length; ++i) {
            if (!Utils.isHeader(elem.cells[i])) {
                return false;
            }
        }

        return true;
    };

    Utils.setEnabled = function(elem, enabled, disabledCssClass) {
        if (elem != null) {
            if (Utils.isTag(elem, Tag.INPUT) || Utils.isSelect(elem) || Utils.isTag(elem, Tag.BUTTON)) {
                elem.disabled = !enabled;
                elem.blur();
            }

            if (disabledCssClass) {
                if (enabled) {
                    Utils.removeCssClass(elem, disabledCssClass);
                } else {
                    Utils.addCssClass(elem, disabledCssClass);
                }
            }

            for (var el = elem.firstChild; el != null; el = el.nextSibling) {
                Utils.setEnabled(el, enabled, disabledCssClass);
            }
        }
    };

    Utils.isVisible = function(el) {
        return (el && el.style.display != "none");
    };

    Utils.setVisible = function(el, visible) {
        if (Object.isElement(el)) {
            el.style.display = visible ? "" : "none";
        }
    };

    Utils.setOpacity = function(el, opacity) {
        if ((el.currentStyle && !el.currentStyle.hasLayout)
             || (!el.currentStyle && el.style.zoom == 'normal')) {
            el.style.zoom = 1;
        }

        var value = (opacity < 0.00001) ? 0 : Math.min(Math.max(opacity, 0), 1);
        var cssValue = (value == 1 ? '' : value);
        var css = el.style;

        if (!Object.isUndefined(css.opacity)) {
            css.opacity = cssValue;
        } else if (!Object.isUndefined(css.MozOpacity)) {
            css.MozOpacity = cssValue;
        } else if (!Object.isUndefined(css.KHTMLOpacity)) {
            css.KHTMLOpacity = cssValue;
        } else if (!Object.isUndefined(css.KhtmlOpacity)) {
            css.KhtmlOpacity = cssValue;
        } else if (!Object.isUndefined(css.filter)) {
            css.filter = (value < 1) ? "alpha(opacity=" + (value * 100) + ")" : "";
        }
    };

    Utils.setStyle = function(elem, styles) {
        if (Object.isString(styles)) {
            elem.style.cssText += ";" + styles;
            var opacityStyleReg = /opacity:\s*(\d?\.?\d*)/;
            if (opacityStyleReg.test(styles)) {
                Utils.setOpacity(elem, RegExp.$1);
            }
        } else {
            var css = elem.style;
            for (var property in styles) {
                if (property == 'opacity') {
                    Utils.setOpacity(elem, styles[property]);
                } else if (property == 'float' || property == 'cssFloat') {
                    var key = Object.isUndefined(css.styleFloat) ? 'cssFloat' : 'styleFloat';
                    css[key] = styles[property];
                } else {
                    css[property] = styles[property];
                }
            }
        }
    };

    Utils.getDimension = function(elem) {
        if (Utils.isVisible(elem)) {
            return {width: elem.offsetWidth, height: elem.offsetHeight};
        }

        var elemStyle = elem.style;
        var origVisibility = elemStyle.visibility;
        var origPosition = elemStyle.position;
        var origDisplay = elemStyle.display;
        elemStyle.visibility = 'hidden';
        elemStyle.position = 'absolute';
        elemStyle.display = 'block';
        var origWidth = elem.clientWidth;
        var origHeight = elem.clientHeight;
        elemStyle.display = origDisplay;
        elemStyle.position = origPosition;
        elemStyle.visibility = origVisibility;

        return {width: origWidth, height: origHeight};
    };

    Utils.addCssClass = function(elem, cssClass) {
        var classNames = Utils._getCssClasses(elem);

        if (!Utils.containsCssClass(elem, cssClass)) {
            classNames[classNames.length] = cssClass;
            Utils._setCssClasses(elem, classNames);
        }
    };

    Utils.removeCssClass = function(elem, cssClass) {
        var classNames = Utils._getCssClasses(elem);

        for (var i = 0; i < classNames.length; ++i) {
            if (classNames[i] == cssClass) {
                delete classNames[i];
            }
        }

        Utils._setCssClasses(elem, classNames);
    };

    Utils.containsCssClass = function(elem, cssClass) {
        var classNames = Utils._getCssClasses(elem);

        for (var i = 0; i < classNames.length; ++i) {
            if (classNames[i] == cssClass) {
                return true;
            }
        }

        return false;
    };

    Utils._getCssClasses = function(elem) {
        if (elem != null && elem.nodeType == Node.ELEMENT) {
            var elemClassName = elem.className.trim();
            if (elemClassName.length != 0) {
                return elemClassName.split(/\s+/);
            }
        }

        return  new Array();
    };

    Utils._setCssClasses = function(elem, cssClasses) {
        if (elem != null && elem.nodeType == Node.ELEMENT) {
            //todo: check performance in IE
            elem.className = cssClasses.join(" ");
        }
    };

    Utils.findAncestor = function(element, tagName) {
        tagName = tagName.toUpperCase();
        for (var el = element; el != null; el = el.parentNode) {
            if (el.tagName === tagName) {
                return el;
            }
        }
        return null;
    };

    Utils.successorOf = function(parent, element) {
        for (var el = element; el != null; el = el.parentNode) {
            if (el === parent) {
                return true;
            }
        }
        return false;
    };

    Utils.getCookieValue = function(name, defaultValue) {
        var needle = name + "=";
        var pairs = (document.cookie || "").split(/;\s*/);
        for (var i = 0; i < pairs.length; ++i) {
            var pair = pairs[i];
            if (pair.indexOf(needle) == 0) {
                return pair.substr(needle.length);
            }
        }
        return defaultValue;
    };

    Utils.setCookieValue = function(name, value, expireAfterSeconds, path, domain, secure) {
        var data = [];
        data.push(name + "=" + value);

        if (expireAfterSeconds > 0) {
            var expires = new Date(new Date);
            expires.setTime(expires.getTime() + expireAfterSeconds * 1000);
            data.push("expires=" + expires.toUTCString());
        } else if (expireAfterSeconds == 0) {
            data.push("expires=" + new Date(1970, 1, 1).toUTCString());
        }

        path && data.push("path=" + path);
        domain && data.push("domain=" + domain);
        secure && data.push("secure");

        document.cookie = data.join(";");
    };

    Utils.prototype = {
        _constructor: function() {
        }
    };

})();

function Tag() {}
(function() {

    Tag.BODY = "BODY";
    Tag.DIV = "DIV";
    Tag.PARAGRAPH = "P";
    Tag.FORM = "FORM";
    Tag.SPAN = "SPAN";
    Tag.LINK = "A";
    Tag.TABLE = "TABLE";
    Tag.TABLE_CAPTION = "CAPTION";
    Tag.TABLE_HEAD = "THEAD";
    Tag.TABLE_BODY = "TBODY";
    Tag.TABLE_FOOT = "TFOOT";
    Tag.TABLE_ROW = "TR";
    Tag.TABLE_CELL = "TD";
    Tag.TABLE_HEAD_CELL = "TH";
    Tag.TEXTAREA = "TEXTAREA";
    Tag.LABEL = "LABEL";
    Tag.SELECT = "SELECT";
    Tag.BUTTON = "BUTTON";
    Tag.IMAGE = "IMG";
    Tag.INPUT = "INPUT";
    Tag.INPUT_TYPE_HIDDEN = "hidden";
    Tag.INPUT_TYPE_BUTTON = "button";
    Tag.INPUT_TYPE_CHECKBOX = "checkbox";
    Tag.INPUT_TYPE_RADIO = "radio";
    Tag.INPUT_TYPE_FILE = "file";
    Tag.INPUT_TYPE_TEXT = "text";
    Tag.OPTION = "option";

})();


function Node() {}
(function() {

    Node.ELEMENT = 1;
    Node.ATTRIBUTE = 2;
    Node.TEXT = 3;
    Node.CDATA_SECTION = 4;
    Node.ENTITY_REFERENCE = 5;
    Node.ENTITY = 6;
    Node.PROCESSING_INSTRUCTION = 7;
    Node.COMMENT = 8;
    Node.DOCUMENT = 9;
    Node.DOCUMENT_TYPE = 10;
    Node.DOCUMENT_FRAGMENT = 11;
    Node.NOTATION = 12;

})();

if (!window.Event) var Event = {};
Event.observe = function(element, name, handler) {
    if (element.addEventListener) {
        element.addEventListener(name, handler, false);
    } else if (element.attachEvent) {
        element.attachEvent("on" + name, handler);
    } else {
        element["on" + name] = handler;
    }
};
Event.unobserve = function(element, name, handler) {
    if (element.removeEventListener) {
        element.removeEventListener(name, handler, false);
    } else if (element.detachEvent) {
        element.detachEvent("on" + name, handler);
    } else {
        element["on" + name] = null;
    }
};
Event.stop = function(event) {
    if (event.preventDefault && event.stopPropagation) {
        if (event.cancelable) {
            event.preventDefault();
        }
        event.stopPropagation();
    } else {
        event.cancelBubble = true;
        event.returnValue = false;
    }
};
Event.findAncestor = function(event, tagName) {
    var target = event.target || event.srcElement || document;
    if (target.nodeType === Node.TEXT) {
        target = target.parentNode;
    }
    tagName = tagName.toUpperCase();
    for (var el = target; el != null; el = el.parentNode) {
        if (el.tagName === tagName) {
            return el;
        }
    }
    return null;
};
Event.bindReady = function(handler) {
    Event.observe(document, "dataavailable", handler);
};

(function() { // support for the DOMContentLoaded event
    var timer;

    function fireContentLoadedEvent() {
        if (document.loaded) { return; }
        if (timer) { window.clearInterval(timer); }
        if (document.createEvent) {
            var e = document.createEvent("HTMLEvents");
            e.initEvent("dataavailable", true, true);
            (!document.dispatchEvent ? document.documentElement : document).dispatchEvent(e);
        } else {
            document.fireEvent("ondataavailable", document.createEventObject());
        }
        document.loaded = true;
    }

    if (document.addEventListener) {
        if (/WebKit/i.test(navigator.userAgent)) {
            timer = window.setInterval(function() {
                if (/loaded|complete/.test(document.readyState))
                    fireContentLoadedEvent();
            }, 10);
            Event.observe(window, "load", fireContentLoadedEvent);
        } else {
            document.addEventListener("DOMContentLoaded", fireContentLoadedEvent, false);
        }
    } else {
        var scriptId = "__onDOMContentLoaded";
        document.write("<script id=" + scriptId + " defer src=//:><\/script>");
        $(scriptId).onreadystatechange = function() {
            if (this.readyState == "complete") {
                this.onreadystatechange = null;
                this.parentNode.removeChild(this);
                fireContentLoadedEvent();
            }
        };
    }
})();

var Timer = Class.create({
    callback: null,
    delay: null,
    repeats: null,
    executing: false,
    intervalId: null,

    _constructor: function(callback, delay) {
        this.callback = callback;
        this.setDelay(delay);
        this.setRepeats(true);
    },
    setDelay: function(delay) {
        this.delay = delay || 0;
    },
    setRepeats: function(repeats) {
        this.repeats = repeats;
    },
    start: function() {
        if (this.intervalId != null) {
            throw new Error("Illegal state: already started.");
        }
        if (this.repeats) {
            this.intervalId = window.setInterval(this._handleRepeatedly.bind(this), this.delay);
        } else {
            this.intervalId = window.setTimeout(this._handleOnce.bind(this), this.delay);
        }
    },
    stop: function() {
        if (this.repeats) {
            window.clearInterval(this.intervalId);
        } else {
            window.clearTimeout(this.intervalId);
        }
        this.intervalId = null;
    },
    _handleOnce: function() {
        this.stop();
        this.callback(this);
    },
    _handleRepeatedly: function() {
        if (!this.executing) {
            try {
                this.executing = true;
                this.callback(this);
            } finally {
                this.executing = false;
            }
        }
    }
});