var ACTION_FORM_NAME = "actionform";
var LIST_TABLE_ID = "list";
var ROW_SELECTED_CLASS = "sel";
var DIALOG_CHANGE_PWD = "changePasswordDialog";

function getListTable() {
    return document.getElementById(LIST_TABLE_ID);
}

function getUploadForm() {
    return document.getElementById("uploadform");
}

function getActionForm() {
    return document.getElementById(ACTION_FORM_NAME);
}
function fireSelectionChanged() {
    window.storageController._onChangeFileSelection();
}

function findParentRow(elem) {
    for (var parent = elem.parentNode; parent != null; parent = parent.parentNode) {
        if (isRow(parent)) {
            return parent;
        }
    }

    return null;
}

function setRowSelected(row, selected) {
    if (isRow(row)) {
        row.className = selected ? ROW_SELECTED_CLASS : "";
    }
}

function setDefaultRowSelected() {
    var table = getListTable();

    if (table != null) {
        for (var i = 0; i < table.rows.length; ++i) {
            var candidate = table.rows[i];
            if (isRow(candidate)) {
                setRowSelected(candidate, true);
                break;
            }
        }
    }
}

function markSelectedFile() {
    var row = getSelectedRow();

    if (row != null) {
        var checkbox = getFirstCheckbox(row);

        if (checkbox != null) {
            checkbox.checked = !checkbox.checked;
        }
    }
}

function listFiles() {
    var result = [];
    var rows = getListTable().rows;

    for (var i = 0, size = rows.length; i < size; ++i) {
        var links = rows[i].getElementsByTagName(Tag.LINK);
        var filename = (links.length !=0) ? getFilename(links[0]) : null;
        if (filename != null) {
            result[result.length] = filename;
        }
    }

    return result;
}

function rowDown() {
    var selectedRow = getSelectedRow();

    if (isRow(selectedRow)) {
        var nextRow = findNextRow(selectedRow);

        if (isRow(nextRow)) {
            setRowSelected(nextRow, true);
            setRowSelected(selectedRow, false);
        }
    } else {
        setDefaultRowSelected();
    }
}

function rowUp() {
    var selectedRow = getSelectedRow();

    if (isRow(selectedRow)) {
        var previousRow = findPreviousRow(selectedRow);

        if (isRow(previousRow)) {
            setRowSelected(previousRow, true);
            setRowSelected(selectedRow, false);
        }
    } else {
        setDefaultRowSelected();
    }
}

function findNextRow(row) {
    return findRowSibling(row, false);
}

function findPreviousRow(row) {
    return findRowSibling(row, true);
}

function findRowSibling(row, previous) {
    var table = getListTable();

    if (table != null && isRow(row)) {
        for (var i = 0; i < table.rows.length; ++i) {
            if (table.rows[i] == row) {
                if (previous && isRow(table.rows[i - 1])) {
                    return table.rows[i - 1];
                } else if (!previous && isRow(table.rows[i + 1])) {
                    return table.rows[i + 1];
                } else {
                    return null;
                }
            }
        }
    }

    return null;
}

function getSelectedRow() {
    var rows = getListTable().rows;

    for (var i = 0, size = rows.length; i < size; ++i) {
        var row = rows[i];

        if (isSelectedRow(row)) {
            return row;
        }
    }

    return null;
}


function isSelectedRow(row) {
    return (isRow(row) && row.className == ROW_SELECTED_CLASS);
}

function isRow(elem) {
    return ((elem != null) && (elem.tagName == "TR") && (elem.cells.length > 0) && (elem.cells[0].tagName == "TD"));
}

function getFirstCheckbox(elem) {
    if (elem != null) {
        var inputs = elem.getElementsByTagName(Tag.INPUT);
        return (inputs.length > 0 && Utils.isCheckboxInput(inputs[0])) ? inputs[0] : null;
    }

    return null;
}

function createParameters(filenames) {
    var sortedColumnIndex = (window.sorter) ? sorter.getSortedColumnIndex() : null;
    var sortedAscending = (window.sorter) ? sorter.isSortedAscending() : null;

    if (arguments.length == 1) {
        return {
            filename: filenames,
            sortedColumnIndex: sortedColumnIndex,
            sortedAscending: sortedAscending};
    } else {
        return {
            sortedColumnIndex: sortedColumnIndex,
            sortedAscending: sortedAscending};
    }
}

function getFilename(el) {
    if (el == null) {
        return null;
    }
    var path = el.getAttribute("data-path");
    return path != null ? path : Utils.getInnerText(el);
}

function getSelectedFile() {
    return getFilename(getSelectedLink(true));
}

function getSelectedLink(excludeParentDir) {
    var selected = getSelectedRow();

    if (isRow(selected)) {
        var links = selected.getElementsByTagName(Tag.LINK);
        if (links.length > 0) {
            if (excludeParentDir) {
                return (links[0].id != "cdup") ? links[0] : null;
            } else {
                return links[0];
            }
        }
    }

    return null;
}

function startInactivityService() {
    if (window.inactivityService) {
        inactivityService.start();
    }
}

function stopInactivityService() {
    if (window.inactivityService) {
        inactivityService.stop();
    }
}

function restartInactivityService() {
    if (window.inactivityService) {
        inactivityService.restart();
    }
}