<#ftl strip_whitespace=true>
<#-- @ftlvariable name="report" type="com.jscape.inet.mft.subsystems.management.web.rest.PageManagementServer.UserReportModel" -->
<#import "/WEB-INF/tags/html.ftl" as html/>
<#import "/WEB-INF/tags/form.ftl" as form/>
<#import "/WEB-INF/tags/message.ftl" as i18n/>

<#macro format_date date="">
    <#local date_pattern>${i18n.text("Users.ReportDialog.date.pattern")}</#local>
    <#if date?is_date>${date?string(date_pattern)}<#else>${i18n.text("Users.ReportDialog.date.null")}</#if>
</#macro>
<#macro format_quota quota="">
    <#local quota_factor=1024 * 1024/>
    <#local quota_reset_factor=24 * 60 * 60 * 1000/>
    <#if quota?has_content>${i18n.text("Users.ReportDialog.template.quota", [quota.bytes / quota_factor, quota.resetPeriod / quota_reset_factor])}</#if>
</#macro>
<#macro format_login_redirect value="">
    <#if value?has_content && value.class.simpleName == "DirectoryRedirection">
        ${i18n.text("Users.ReportDialog.template.redirection.directory")}
    <#else>
        ${i18n.text("Users.ReportDialog.template.redirection.management")}
    </#if>
</#macro>
<#macro format_administration value="">
    <#if value?has_content>
        <#local manage_non_owned><@format_allowed value.manageNonOwned/></#local>
        <#local group_assignment><@format_allowed value.allowGroupAssignment/></#local>
        <#local resource_mappingt><@format_allowed value.allowResourceMapping/></#local>
        <#local phone_auth><@format_allowed value.allowSetPhoneAuth/></#local>
        <#local share_quotas><@format_enabled value.shareQuotas/></#local>
        <#local restrict_path><#if value.restrictDir??>${i18n.text("Users.ReportDialog.administration.restrictPath", value.restrictDir)}</#if></#local>
        <#local manage_public_contact><@format_allowed value.allowManagePublicContact/></#local>
        <#local administration=[
            '${i18n.text("Users.ReportDialog.administration.managedAccounts", [value.maxOwnedCount!0])}',
            '${i18n.text("Users.ReportDialog.administration.manageNonOwned", [manage_non_owned])}',
            '${i18n.text("Users.ReportDialog.administration.groupAssignment", [group_assignment])}',
            '${i18n.text("Users.ReportDialog.administration.resourceMapping", [resource_mappingt])}',
            '${i18n.text("Users.ReportDialog.administration.phoneAuthentication", [phone_auth])}',
            '${i18n.text("Users.ReportDialog.administration.shareQuotas", [share_quotas])}',
            restrict_path,
            '${i18n.text("Users.ReportDialog.administration.managePublicContacts", [manage_public_contact])}'
        ] />
        <#list administration as entry>
            <#if entry?has_content>${entry}<#if entry_has_next>,<br/></#if></#if>
        </#list>
    </#if>
</#macro>
<#macro format_enabled value><#if value>${i18n.text("Users.ReportDialog.template.enabled")}<#else>${i18n.text("Users.ReportDialog.template.disabled")}</#if></#macro>
<#macro format_allowed value><#if value>${i18n.text("Users.ReportDialog.template.allowed")}<#else>${i18n.text("Users.ReportDialog.template.disabled")}</#if></#macro>
<#macro format_required value><#if value>${i18n.text("Users.ReportDialog.template.required")}<#else>${i18n.text("Users.ReportDialog.template.disabled")}</#if></#macro>
<#macro format_ignored value><#if value>${i18n.text("Users.ReportDialog.template.ignored")}<#else>${i18n.text("Users.ReportDialog.template.followed")}</#if></#macro>


<@html.page title=i18n.text("Users.ReportDialog.title")>
<style type="text/css">html, body { font-size: 90% }</style>

<h1 style="text-align: center"><@i18n.message key="Users.ReportDialog.title" /></h1>

<#list report.users as user>
    <h2>${user.username!user.login}</h2>
<fieldset class="large">
    <dl class="group">
        <dt>Login:</dt><dd>${user.login}</dd>
        <dt>Status:</dt><dd><#if user.enabled><@i18n.message key="Users.ReportDialog.template.enabled" /><#else><@i18n.message key="Users.ReportDialog.template.disabled" /></#if></dd>
        <dt>Expires at:</dt><dd><@format_date user.expirationDate/></dd>
        <dt>Email address:</dt><dd>${user.emailAddress!""}</dd>
        <dt>Phone:</dt><dd><#if user.phone??>${i18n.text("Users.ReportDialog.template.phone", [user.phone.code!"", user.phone.number!"", user.phone.extension!""])}<#else>&nbsp;</#if></dd>
        <dt>Group name:</dt><dd><#if user.groups??><#list user.groups as group>${group}<#if group_has_next>, </#if></#list></#if></dd>
        <dt>Last login date:</dt><dd><@format_date user.lastLoginDate/></dd>

<#if report.detailed>
    <dt>Upload quota:</dt><dd><@format_quota user.uploadsQuota/></dd>
    <dt>Download quota:</dt><dd><@format_quota user.downloadsQuota/></dd>
    <dt>Transfer quota:</dt><dd><@format_quota user.transfersQuota/></dd>
    <dt>Secured connection:</dt><dd><@format_required user.secured/></dd>
    <dt>Password changing:</dt><dd><@format_allowed user.passwordChangingAllowed/></dd>
    <dt>Email file transfer:</dt><dd><@format_allowed user.emailFileTransferAllowed/></dd>
    <dt>Phone authentication:</dt><dd><@format_required user.usePhoneAuthentication/></dd>
    <dt>Password creation date:</dt><dd><@format_date user.passwordDate/></dd>
    <dt>Password aging rules:</dt><dd><@format_ignored user.ignorePasswordAgingRules /></dd>
    <dt>Owned by:</dt><dd>${user.owner!""}</dd>
    <dt>Domain administration:</dt><dd><@format_administration user.administration /></dd>
    <dt>Password reminder sent:</dt><dd><@format_date report.passwordNotificationDateOf(user) /></dd>
    <dt>Web login default view:</dt><dd><@format_login_redirect user.loginRedirection /></dd>
    <dt>Bound keys:</dt><dd><#if user.bindedKeys??><#list user.bindedKeys as key>${key}<#if key_has_next>, </#if></#list></#if></dd>
    <dt>Virtual paths:</dt>
    <dd>
        <table class="easyui-datagrid" fitcolumns="true">
            <thead>
            <tr>
                <th data-options="field:'path'">Path</th>
                <th data-options="field:'mapping'" width="400">Real Path / Reverse Proxy</th>
                <th data-options="field:'permissions'">Access Permissions</th>
            </tr>
            </thead>
            <tbody>
                <#list user.resources as resource>
                    <tr>
                        <td><#if resource.path?has_content>${resource.path}<#else>/</#if></td>
                        <td>${resource.realPath!resource.name}</td>
                        <td>${report.formatAccessPermissions(resource.accessPermissions)}</td>
                    </tr>
                </#list>
            </tbody>
        </table>
    </dd>
</#if>
    </dl>
</fieldset>
<br/>
</#list>

</@html.page>