/** @class */
jscape.Application = Class.extend(
    /** @lends {jscape.Application.prototype} */
{
    modules: null,

    /** @constructor */
    initialize: function(modules) {
        this.modules = modules || [];
        this.menu = new jscape.Menu();
        this.menu.bind(this, "application");
    },

    start: function() {
        try {
            this._createModules.apply(this, arguments);
            this._startModules();
            this.onReady();
        } catch (e) {
            throw "Error starting modules: " + e;
        }
    },

    onReady: function() {
        $(document).on("broadcast", $.proxy(this.onBroadcast, this));
        $(window).on("beforeunload", $.proxy(this.onBeforeUnload, this))
                .on("unload", $.proxy(this.onUnload, this));
    },

    onMenuItemSelected: function(item) {
        for (var i = 0; i < this.modules.length; i++) {
            if (this.modules[i].onMenuItemSelected(item)) {
                return true;
            }
        }
        return false;
    },

    onViewStateChanged: function (index, visible) {
        if (index >= 0 && index < this.modules.length) {
            var m = this.modules[index];
            if (visible) {
                m.onResume();
            } else {
                m.onPause();
            }
        }
    },

    onBroadcast: function (e, message) {
        if (message && message.trim().length) {
            var style = {top: document.body.scrollTop + document.documentElement.scrollTop + 20, right: "", bottom: ""};
            $.messager.show({msg: message, noheader: true, height: "auto", showType: "fade", showSpeed: 200, style: style});
        }
    },

    onBeforeUnload: function(e) {
        for (var i = 0; i < this.modules.length; i++) {
            this.modules[i].onPause();
        }
        return e.returnValue || undefined;
    },

    onUnload: function() {
        while (this.modules.length) {
            this.modules.pop().onStop();
        }
    },

    _createModules: function() {
        for (var i = 0; i < this.modules.length; i++) {
            var m = this.modules[i];
            m.onCreate.apply(m, arguments);
        }
    },

    _startModules: function() {
        for (var i = 0; i < this.modules.length; i++) {
            var m = this.modules[i];
            m.onStart();
        }
    }
});

/** @class */
jscape.ApplicationModule = Class.extend(
    /** @lends {jscape.ApplicationModule.prototype} */
{
    managementService: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    onCreate: function(managementService) {
        this.managementService = managementService;
    },

    onStart: function() {
    },

    onStop: function() {
    },

    onPause: function() {
    },

    onResume: function() {
    },

    onMenuItemSelected: function(item) {
        return false;
    },

    onRedirect: function(url) {
        window.location = url;
    },

    _confirmApply: function() {
        var e = this._eventQueueOf(arguments.callee, "beforeunload");
        if (e) {
            e.returnValue = this.resources.getString("APPLICATION_CONFIRM_LEAVE");
            return $.Deferred().reject().promise();
        }
        return jscape.ConfirmDialog.confirm(this.resources.getString("APPLICATION_TITLE"),
                                            this.resources.getString("APPLICATION_CONFIRM_APPLY"));
    },

    _eventQueueOf: function(fn, type) {
        if (fn.caller == null) {
            return null;
        }
        var e = fn.caller.arguments.length ? fn.caller.arguments[0] : null;
        return e instanceof jQuery.Event && e.type === type ? e : this._eventQueueOf(fn.caller, type);
    }
});

/** @class */
jscape.Menu = Class.extend(
    /** @lends {jscape.Menu.prototype} */
{
    /** @constructor */
    initialize: function () {
        var handler = $.proxy(this._onClick$MenuItem, this);

        $("#menu-home").data("id", jscape.Menu.ITEM.Home).linkbutton({plain: true, onClick: function() { handler(new jQuery.Event("click", {delegateTarget: this})); }});

        $("#menu-server").menubutton({plain: true, menu: "#menu-server-items"});
        $("#menu-server-newdomain").data("id", jscape.Menu.ITEM.NewDomain).click(handler);
        $("#menu-server-domains-start").data("id", jscape.Menu.ITEM.StartDomains).click(handler);
        $("#menu-server-domains-stop").data("id", jscape.Menu.ITEM.StopDomains).click(handler);
        $("#menu-server-domains-pause").data("id", jscape.Menu.ITEM.PauseDomains).click(handler);
        $("#menu-server-domains-resume").data("id", jscape.Menu.ITEM.ResumeDomains).click(handler);
        $("#menu-server-keystore").data("id", jscape.Menu.ITEM.KeyManager).click(handler);
        $("#menu-server-settings").data("id", jscape.Menu.ITEM.ServerSettings).click(handler);

        $("#menu-help").menubutton({plain: true, menu: "#menu-help-items"});
        $("#menu-logout").data("id", jscape.Menu.ITEM.Logout).linkbutton({plain: true, onClick: function() { handler(new jQuery.Event("click", {delegateTarget: this})); }});
    },

    bind: function(context, namespace) {
        var ns = (namespace ? "." + namespace : "");
        $("body").on("menu.application" + ns, $.proxy(function(e, item) {
            if (this.onMenuItemSelected(item)) {
                e.stopImmediatePropagation();
            }
        }, context));
    },

    unbind: function(namespace) {
        $("body").off("menu.application" + (namespace ? "." + namespace : ""));
    },

    _onClick$MenuItem: function (e) {
        var target = $(e.delegateTarget);
        $("body").triggerHandler("menu.application", {
            id: target.data("id"),
            url: target.find("a").prop("href") || ""
        });
        return false;
    }
});
jscape.Menu.ITEM = {
    Home: "home",
    NewDomain: "domains-newdomain",
    StartDomains: "domains-start",
    StopDomains: "domains-stop",
    PauseDomains: "domains-pause",
    ResumeDomains: "domains-resume",
    KeyManager: "server-keymanager",
    ServerSettings: "server-settings",
    Logout: "logout"
};