jscape.domain = jscape.domain || {};
jscape.domain.wizard = jscape.domain.wizard || {};


/** @class */
jscape.domain.NewDomainController = Class.extend(
    /** @lends {jscape.domain.NewDomainController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService) {
        this.managementService = managementService;
        this.deferred = $.Deferred();

        this._setupDialog().show(this);

        return this.deferred.promise();
    },

    onPageSelected: function(page) {
    },

    onSubmit: function(dialog) {
        try {
            var descriptor = this._createDescriptor();
            this.managementService.storeDomain(descriptor)
                    .done($.proxy(function() {
                        dialog.hide();
                        this.managementService.startDomain(descriptor.name)
                                .always($.proxy(function() { this.deferred.resolve(descriptor); }, this));
                    }, this))
                    .fail($.proxy(function() { this.deferred.reject(); }, this));
        } catch (e) {
            this.deferred.reject();
        }
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _setupDialog: function () {
        var dialog = new jscape.domain.AddDomainDialog(this.resources);

        this.nameController = new jscape.domain.wizard.DomainNameController(this.resources, dialog.container());
        dialog.addItem(this.nameController);

        this.serviceController = new jscape.domain.wizard.TransportServiceController(this.resources, dialog.container());
        dialog.addItem(this.serviceController);

        this.datastoreServiceController = new jscape.domain.wizard.DatastoreController(this.resources, dialog.container());
        dialog.addItem(this.datastoreServiceController);

        this.logServiceController = new jscape.domain.wizard.LogServiceController(this.resources, dialog.container());
        dialog.addItem(this.logServiceController);

        this.nameController.start(this.managementService);
        this.serviceController.start(this.managementService);
        this.datastoreServiceController.start(this.managementService);
        this.logServiceController.start(this.managementService);

        return dialog;
    },

    _createDescriptor: function() {
        var transportService = this.serviceController.applyChanges();
        var datastoreService = this.datastoreServiceController.applyChanges();
        var loggingService = this.logServiceController.applyChanges();

        return new jscape.DomainDescriptor(this.nameController.getDomainName(),
                                           this.nameController.getDomainDescription(),
                                           [transportService, datastoreService, loggingService]);
    }
});

/** @class */
jscape.domain.AddDomainDialog = jscape.view.WizardDialog.extend(
    /** @lends {jscape.domain.AddDomainDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#add-domain-wizard"), resources);
        this.resources = resources;
    }
});

/** @class */
jscape.domain.wizard.BaseDomainServiceController = Class.extend(
    /** @lends {jscape.domain.wizard.BaseDomainServiceController.prototype} */
{
    view: null,

    resume: function () {
        this.view.show(this);
    },

    pause: function () {
        this.view.hide();
    },

    valid: function () {
        return this.view.valid();
    },

    applyChanges: function () {
        return this.view.valid() ? this._createDescriptor() : null;
    },

    _createDescriptor: function() {
        throw "Abstract method.";
    }
});

/** @class */
jscape.domain.wizard.BaseDomainServiceView = Class.extend(
    /** @lends {jscape.domain.wizard.BaseDomainServiceView.prototype} */
{
    fieldset: null,

    /** @constructor */
    initialize: function (fieldset) {
        this.fieldset = fieldset.hide();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    show: function() {
        this.fieldset.show().form("validate");
        this.fieldset.find(":input:visible:not(:disabled)").first().focus();
    },

    hide: function() {
        this.fieldset.hide();
    }
});

/** @class */
jscape.domain.wizard.DomainNameController = jscape.view.WizardDialog.Page.extend(
    /** @lends {jscape.domain.wizard.DomainNameController.prototype} */
{
    domainNames: null,

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.wizard.DomainNameView(resources, container);

        this._initValidators(resources);
    },

    /** @override */
    start: function(managementService) {
        managementService.getDomainNames()
                .done($.proxy(function(names) { this.domainNames = names || []; }, this))
                .fail($.proxy(function() { this.domainNames = []; }, this));
    },

    pause: function() {
        this.view.hide();
    },

    resume: function() {
        this.view.show();
    },

    valid: function() {
        return this.view.valid();
    },

    getDomainName: function() {
        return this.view.getName();
    },

    getDomainDescription: function() {
        return this.view.getDescription();
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("domain_name_nonempty", resources.getString("DOMAIN_ERROR_BAD_NAME"));
        v.rule("domain_unique", $.proxy(this._domainNotExists, this), resources.getString("DOMAIN_ERROR_NAME_EXISTS"));
    },

    _domainNotExists: function(value) {
        return $.inArray(value, this.domainNames) == -1;
    }
});

/** @class */
jscape.domain.wizard.DomainNameView = Class.extend(
    /** @lends {jscape.domain.wizard.DomainNameView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=domainparametersfields]", container).hide();

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["domain_name_nonempty", "domain_unique"], missingMessage: resources.getString("DOMAIN_ERROR_BAD_NAME")});

        this.description = $("textarea[name=description]", this.fieldset).css({width: "30em", height: "10em"});
    },

    getName: function() {
        return this.name.val();
    },

    getDescription: function() {
        return this.description.val();
    },

    show: function() {
        this.fieldset.show();
        this.name.focus();
    },

    hide: function() {
        this.fieldset.hide();
    },

    valid: function() {
        return this.fieldset.form("validate");
    }
});

/** @class */
jscape.domain.wizard.TransportServiceController = jscape.view.ListView.extend(
    /** @lends {jscape.domain.wizard.TransportServiceController.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super();
        this.resources = resources;
        this.view = new jscape.domain.wizard.TransportServiceView(this.resources, container);

        this.addItem(jscape.Service.Protocol.AFTP, jscape.domain.wizard.AftpServiceController);
        this.addItem(jscape.Service.Protocol.AS2, jscape.domain.wizard.As2ServiceController);
        this.addItem(jscape.Service.Protocol.FTP, jscape.domain.wizard.FtpServiceController);
        this.addItem(jscape.Service.Protocol.HTTP, jscape.domain.wizard.HttpServiceController);
        this.addItem(jscape.Service.Protocol.OFTP, jscape.domain.wizard.OftpServiceController);
        this.addItem(jscape.Service.Protocol.SFTP, jscape.domain.wizard.SftpServiceController);
        this.addItem(jscape.Service.Protocol.WEBDAV, jscape.domain.wizard.WebDavServiceController);
    },

    start: function(managementService) {
        this.managementService = managementService;

        this.view.show(this);

        this._loadData()
                .always($.proxy(function() {
                    this.view.setProtocol(jscape.Service.Protocol.FTP);
                }, this));
    },

    pause: function() {
        this.view.hide();
    },

    resume: function() {
        this.view.show(this);
    },

    valid: function() {
        var item = this.getItem(this.getCurrentItem());
        return $.type(item) === "object" ? item.valid() : true;
    },

    applyChanges: function() {
        var item = this.getItem(this.getCurrentItem());
        return $.type(item) === "object" ? item._createDescriptor() : null;
    },

    onChangeProtocol: function () {
        this.setCurrentItem(this.view.getProtocol().toUpperCase());
    },

    _loadData: function () {
        return $.when(this.managementService.getServerKeys(),
                      this.managementService.getServerParameters())
                .done($.proxy(function (keys, parameters) {
                    this.privateKeys = $.map(keys || [], function (key) { return key.alias; });
                    this.hosts = parameters && parameters.getInetAddresses();
                }, this));
    },

    /** @override */
    _createItem: function(item) {
        var controller = new item(this.resources, this.view.container());
        controller.start(this.hosts, this.privateKeys);
        return controller;
    }
});

/** @class */
jscape.domain.wizard.TransportServiceView = Class.extend(
    /** @lends {jscape.domain.wizard.TransportServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=domainservicefields]", container).hide();

        this.protocol = $("select[name=protocol]", this.fieldset);
        this.protocol.combobox({required: true, editable: false, panelHeight: "auto", onChange: $.proxy(this._onChange$Protocol, this)});
    },

    getProtocol: function() {
        return this.protocol.combobox("getValue");
    },

    setProtocol: function (value) {
        var select = this.protocol.combobox("getValue");
        this.protocol.combobox("setValue", value);
        if (select === value) {
            this._onChange$Protocol();
        }
    },

    container: function() {
        return this.fieldset;
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.wizard.TransportServiceView.LISTENER;
        this.fieldset.show();
    },

    hide: function() {
        this.fieldset.hide();
    },

    _onChange$Protocol: function() {
        this.listener.onChangeProtocol(this);
    }
});
jscape.domain.wizard.TransportServiceView.LISTENER = {
    onChangeProtocol: $.noop
};

/** @class */
jscape.domain.wizard.FtpServiceController = jscape.domain.wizard.BaseDomainServiceController.extend(
    /** @lends {jscape.domain.wizard.FtpServiceController.prototype} */
{
    DEFAULTS: {
        "regular": {port: 21, keyUsed: false},
        "explicit SSL": {port: 21, keyUsed: true},
        "forced explicit SSL": {port: 21, keyUsed: true},
        "implicit SSL": {port: 990, keyUsed: true}
    },

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.wizard.FtpServiceView(resources, container);
    },

    start: function(hosts, keys) {
        this.view.setHosts(hosts);
        this.view.setKeys(keys);
        this.view.show(this);

        this.view.setType(jscape.Service.FTP_TYPE.EXPLICIT_SSL);
    },

    onChangeType: function () {
        var type = this.view.getType();
        if (this.DEFAULTS.hasOwnProperty(type)) {
            this.view.setPort(this.DEFAULTS[type].port);
            this.view.setKeyUsed(this.DEFAULTS[type].keyUsed);
        }
    },

    _createDescriptor: function () {
        return jscape.Service.ftp(this.view.getHost(),
                                  this.view.getPort(),
                                  this.view.getType(),
                                  this.view.getKeyAlias());
    }
});

/** @class */
jscape.domain.wizard.FtpServiceView = jscape.domain.wizard.BaseDomainServiceView.extend(
    /** @lends {jscape.domain.wizard.FtpServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=ftpservicefields]", container));

        this.host = $("select[name=host]", this.fieldset);
        this.host.combobox({required: true, width: 200,  panelHeight: "auto", validType: "length[1,500]", missingMessage: resources.getString("DOMAIN_ERROR_BAD_HOST"), invalidMessage: resources.getString("DOMAIN_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 80});

        this.type = $("select[name=type]", this.fieldset);
        this.type.combobox({required: true, editable: false, panelHeight: "auto", onChange: $.proxy(this._onChange$Type, this)});

        this.keyAlias = $("select[name=privatekey]", this.fieldset);
        this.keyAlias.combobox({required: true, editable: false, panelHeight: "auto", width: 170});
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHosts: function(values) {
        var data = $.map(values || [], function(host) { return {text: host, value: host}; });
        this.host.combobox("loadData", data);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", parseInt(value)).numberspinner("isValid");
    },

    getType: function() {
        return this.type.combobox("getValue");
    },

    setType: function (value) {
        var select = this.type.combobox("getValue");
        this.type.combobox("setValue", value);
        if (select === value) {
            this._onChange$Type();
        }
    },

    getKeyAlias: function() {
        return !this.keyAlias.combobox("options").disabled ? this.keyAlias.combobox("getValue") : null;
    },

    setKeys: function(values) {
        var data = $.map(values || [], function(key) { return {text: key, value: key}; });
        this.keyAlias.combobox("loadData", data);
    },

    setKeyUsed: function(value) {
        this.keyAlias.combobox(!!value ? "enable" : "disable").combobox(!!value ? "enableValidation" : "disableValidation");
    },

    /** @override */
    show: function(listener) {
        this.listener = listener || jscape.domain.wizard.FtpServiceView.LISTENER;
        this._super();
    },

    _onChange$Type: function() {
        if (this.listener) {
            this.listener.onChangeType(this);
        }
    }
});
jscape.domain.wizard.FtpServiceView.LISTENER = {
    onChangeType: $.noop
};

/** @class */
jscape.domain.wizard.OftpServiceController = jscape.domain.wizard.BaseDomainServiceController.extend(
    /** @lends {jscape.domain.wizard.OftpServiceController.prototype} */
{
    DEFAULT_PORT: 3305,

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.wizard.OftpServiceView(resources, container);
    },

    start: function(hosts, keys) {
        this.view.setHosts(hosts);
        this.view.setKeys(keys);
        this.view.setPort(this.DEFAULT_PORT);
        this.view.setUseSsl(true);
        this.view.setFileEncryptionRequired(true);
        this.view.setFileSignatureRequired(true);
        this.view.show();
    },

    _createDescriptor: function () {
        return jscape.Service.oftp(this.view.getHost(),
                                   this.view.getPort(),
                                   this.view.getIdCode(),
                                   this.view.getKeyAlias(),
                                   this.view.isUseSsl(),
                                   this.view.isFileEncryptionRequired(),
                                   this.view.isFileSignatureRequired());
    }
});

/** @class */
jscape.domain.wizard.OftpServiceView = jscape.domain.wizard.BaseDomainServiceView.extend(
    /** @lends {jscape.domain.wizard.OftpServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=oftpservicefields]", container));

        this.host = $("select[name=host]", this.fieldset);
        this.host.combobox({required: true, width: 200,  panelHeight: "auto", validType: "length[1,500]", missingMessage: resources.getString("DOMAIN_ERROR_BAD_HOST"), invalidMessage: resources.getString("DOMAIN_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 80});

        this.idCode = $("input[name=idcode]", this.fieldset);
        this.idCode.validatebox({required: true, validType: "length[1,500]", missingMessage: resources.getString("DOMAIN_ERROR_BAD_ID_CODE"), invalidMessage: resources.getString("DOMAIN_ERROR_BAD_ID_CODE")});

        this.keyAlias = $("select[name=privatekey]", this.fieldset);
        this.keyAlias.combobox({required: true, editable: false, width: 170,  panelHeight: "auto"});

        this.useSsl = $("input[name=usessl]:checkbox", this.fieldset);
        this.fileEncryption = $("input[name=fileencryption]:checkbox", this.fieldset);
        this.fileSignature = $("input[name=filesignature]:checkbox", this.fieldset);
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHosts: function(values) {
        var data = $.map(values || [], function(host) { return {text: host, value: host}; });
        this.host.combobox("loadData", data);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value);
    },

    getIdCode: function() {
        return this.idCode.val();
    },

    getKeyAlias: function() {
        return !this.keyAlias.combobox("options").disabled ? this.keyAlias.combobox("getValue") : null;
    },

    setKeys: function(values) {
        var data = $.map(values || [], function(key) { return {text: key, value: key}; });
        this.keyAlias.combobox("loadData", data);
    },

    isUseSsl: function() {
        return this.useSsl.is(":checked");
    },

    setUseSsl: function(value) {
        this.useSsl.prop("checked", !!value);
    },

    isFileEncryptionRequired: function() {
        return this.fileEncryption.is(":checked");
    },

    setFileEncryptionRequired: function(value) {
        this.fileEncryption.prop("checked", !!value);
    },

    isFileSignatureRequired: function() {
        return this.fileSignature.is(":checked");
    },

    setFileSignatureRequired: function(value) {
        this.fileSignature.prop("checked", !!value);
    }
});

/** @class */
jscape.domain.wizard.SftpServiceController = jscape.domain.wizard.BaseDomainServiceController.extend(
    /** @lends {jscape.domain.wizard.SftpServiceController.prototype} */
{
    DEFAULT_PORT: 22,

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.wizard.SftpServiceView(resources, container);
    },

    start: function(hosts, keys) {
        this.view.setPort(this.DEFAULT_PORT);
        this.view.setHosts(hosts);
        this.view.setKeys(keys);
        this.view.show();
    },

    _createDescriptor: function () {
        return jscape.Service.sftp(this.view.getHost(),
                                   this.view.getPort(),
                                   this.view.getAuthentication(),
                                   this.view.getKeyAlias());
    }
});

/** @class */
jscape.domain.wizard.SftpServiceView = jscape.domain.wizard.BaseDomainServiceView.extend(
    /** @lends {jscape.domain.wizard.SftpServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=sftpservicefields]", container));

        this.host = $("select[name=host]", this.fieldset);
        this.host.combobox({required: true, width: 200,  panelHeight: "auto", validType: "length[1,500]", missingMessage: resources.getString("DOMAIN_ERROR_BAD_HOST"), invalidMessage: resources.getString("DOMAIN_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 80});

        this.keyAlias = $("select[name=privatekey]", this.fieldset);
        this.keyAlias.combobox({required: true, editable: false, width: 170,  panelHeight: "auto"});

        this.auth = $("select[name=auth]", this.fieldset);
        this.auth.combobox({required: true, editable: false, panelHeight: "auto"});
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHosts: function(values) {
        var data = $.map(values || [], function(host) { return {text: host, value: host}; });
        this.host.combobox("loadData", data);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", parseInt(value)).numberspinner("validate");
    },

    getAuthentication: function() {
        return this.auth.combobox("getValue");
    },

    getKeyAlias: function() {
        return !this.keyAlias.combobox("options").disabled ? this.keyAlias.combobox("getValue") : null;
    },

    setKeys: function(values) {
        var data = $.map(values || [], function(key) { return {text: key, value: key}; });
        this.keyAlias.combobox("loadData", data);
    }
});

/** @class */
jscape.domain.wizard.AftpServiceController = jscape.domain.wizard.BaseDomainServiceController.extend(
    /** @lends {jscape.domain.wizard.AftpServiceController.prototype} */
{
    DEFAULT_PORT: 3000,

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.wizard.AftpServiceView(resources, container);
    },

    start: function(hosts, keys) {
        this.view.setPort(this.DEFAULT_PORT);
        this.view.setHosts(hosts);
        this.view.setKeys(keys);
        this.view.show();
    },

    _createDescriptor: function () {
        return jscape.Service.aftp(this.view.getHost(),
                                   this.view.getPort(),
                                   this.view.getKeyAlias(),
                                   this.view.isProtectionRequired());
    }
});

/** @class */
jscape.domain.wizard.AftpServiceView = jscape.domain.wizard.BaseDomainServiceView.extend(
    /** @lends {jscape.domain.wizard.AftpServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=aftpservicefields]", container));

        this.host = $("select[name=host]", this.fieldset);
        this.host.combobox({required: true, width: 200,  panelHeight: "auto", validType: "length[1,500]", missingMessage: resources.getString("DOMAIN_ERROR_BAD_HOST"), invalidMessage: resources.getString("DOMAIN_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 80});

        this.keyAlias = $("select[name=privatekey]", this.fieldset);
        this.keyAlias.combobox({required: true, editable: false, width: 170,  panelHeight: "auto"});

        this.protectionRequired = $("input[name=requiresecure]:checkbox", this.fieldset);
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHosts: function(values) {
        var data = $.map(values || [], function(host) { return {text: host, value: host}; });
        this.host.combobox("loadData", data);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", parseInt(value)).numberspinner("validate");
    },

    getKeyAlias: function() {
        return !this.keyAlias.combobox("options").disabled ? this.keyAlias.combobox("getValue") : null;
    },

    setKeys: function(values) {
        var data = $.map(values || [], function(key) { return {text: key, value: key}; });
        this.keyAlias.combobox("loadData", data);
    },

    isProtectionRequired: function() {
        return this.protectionRequired.is(":checked");
    }
});

/** @class */
jscape.domain.wizard.HttpServiceController = jscape.domain.wizard.BaseDomainServiceController.extend(
    /** @lends {jscape.domain.wizard.HttpServiceController.prototype} */
{
    /** @constructor */
    initialize: function (resources, container) {
        this.view = new jscape.domain.wizard.HttpServiceView(resources, container);
    },

    start: function() {
        this.view.setPlain(true);
        this.view.setSecure(true);
        this.view.show();
    },

    _createDescriptor: function () {
        return jscape.Service.http(this.view.isPlain(), this.view.isSecure());
    }
});

/** @class */
jscape.domain.wizard.BaseWebServiceView = jscape.domain.wizard.BaseDomainServiceView.extend(
    /** @lends {jscape.domain.wizard.BaseWebServiceView.prototype} */
{
    /** @constructor */
    initialize: function(fieldset) {
        this._super(fieldset);

        this.plain = $("input[name=plain]:checkbox", fieldset);
        this.secure = $("input[name=secure]:checkbox", fieldset);
    },

    isPlain: function() {
        return this.plain.is(":checked");
    },

    setPlain: function(value) {
        this.plain.prop("checked", !!value);
    },

    isSecure: function() {
        return this.secure.is(":checked");
    },

    setSecure: function(value) {
        this.secure.prop("checked", !!value);
    }
});

/** @class */
jscape.domain.wizard.HttpServiceView = jscape.domain.wizard.BaseWebServiceView.extend(
    /** @lends {jscape.domain.wizard.HttpServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=httpservicefields]", container));
    }
});

/** @class */
jscape.domain.wizard.WebDavServiceController = jscape.domain.wizard.BaseDomainServiceController.extend(
    /** @lends {jscape.domain.wizard.WebDavServiceController.prototype} */
{
    /** @constructor */
    initialize: function (resources, container) {
        this.view = new jscape.domain.wizard.WebDavServiceView(resources, container);
    },

    start: function() {
        this.view.setPlain(true);
        this.view.setSecure(true);
        this.view.show();
    },

    _createDescriptor: function () {
        return jscape.Service.webdav(this.view.isPlain(), this.view.isSecure());
    }
});

/** @class */
jscape.domain.wizard.WebDavServiceView = jscape.domain.wizard.BaseWebServiceView.extend(
    /** @lends {jscape.domain.wizard.WebDavServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=webdavservicefields]", container));
    }
});

/** @class */
jscape.domain.wizard.As2ServiceController = jscape.domain.wizard.BaseDomainServiceController.extend(
    /** @lends {jscape.domain.wizard.As2ServiceController.prototype} */
{
    /** @constructor */
    initialize: function (resources, container) {
        this.view = new jscape.domain.wizard.As2ServiceView(resources, container);
    },

    start: function() {
        this.view.setPlain(true);
        this.view.setSecure(true);
        this.view.show();
    },

    _createDescriptor: function () {
        return jscape.Service.as2(this.view.isPlain(), this.view.isSecure());
    }
});

/** @class */
jscape.domain.wizard.As2ServiceView = jscape.domain.wizard.BaseWebServiceView.extend(
    /** @lends {jscape.domain.wizard.As2ServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=as2servicefields]", container));
    }
});

/** @class */
jscape.domain.wizard.DatastoreController = jscape.view.ListView.extend(
    /** @lends {jscape.domain.wizard.DatastoreController.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super();
        this.resources = resources;
        this.view = new jscape.domain.wizard.DatastoreView(resources, container);

        this.addItem(jscape.DatastoreService.Type.FILE, jscape.domain.FileDatastoreServiceController);
        this.addItem(jscape.DatastoreService.Type.FILE_RECORD, jscape.domain.FileRecordDatastoreServiceController);
        this.addItem(jscape.DatastoreService.Type.DB, jscape.domain.DbDatastoreServiceController);
    },

    start: function(managementService) {
        this.managementService= managementService;
        this.view.show(this);
        this.view.setType(jscape.DatastoreService.Type.FILE_RECORD);
    },

    pause: function() {
        this.view.hide();
    },

    resume: function() {
        this.view.show(this);
    },

    valid: function() {
        var item = this.getItem(this.getCurrentItem());
        return $.type(item) === "object" ? item.valid() : true;
    },

    applyChanges: function() {
        var item = this.getItem(this.getCurrentItem());
        return $.type(item) === "object" ? item.applyChanges() : null;
    },

    onChangeType: function () {
        this.setCurrentItem(this.view.getType().toUpperCase());
    },

    /** @override */
    _createItem: function(item) {
        var controller = new item(this.resources, this.view.container());
        controller.start(this.managementService, null, null);
        return controller;
    }
});

/** @class */
jscape.domain.wizard.DatastoreView = Class.extend(
    /** @lends {jscape.domain.wizard.DatastoreView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=datastoreservicefields]", container).hide();

        this.type = $("select[name=type]", this.fieldset);
        this.type.combobox({required: true, editable: false, width: 120, panelHeight: "auto", onChange: $.proxy(this._onChange$Type, this)});
    },

    getType: function() {
        return this.type.combobox("getValue");
    },

    setType: function(value) {
        var select = this.type.combobox("getValue");
        this.type.combobox("setValue", value);
        if (select === value) {
            this._onChange$Type();
        }
    },

    container: function() {
        return this.fieldset;
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.wizard.DatastoreView.LISTENER;
        this.fieldset.show();
    },

    hide: function() {
        this.fieldset.hide();
    },

    _onChange$Type: function() {
        this.listener.onChangeType(this);
    }
});
jscape.domain.wizard.DatastoreView.LISTENER = {
    onChangeType: $.noop
};


/** @class */
jscape.domain.wizard.LogServiceController = jscape.view.ListView.extend(
    /** @lends {jscape.domain.wizard.LogServiceController.prototype} */
{
    /** @constructor */
    initialize: function (resources, container) {
        this._super();
        this.resources = resources;
        this.view = new jscape.domain.wizard.LogServiceView(resources, container);

        this.addItem(jscape.LogService.Type.FILE, jscape.domain.FileLogServiceController);
        this.addItem(jscape.LogService.Type.DB, jscape.domain.DbLogServiceController);
    },

    start: function (managementService) {
        this.managementService = managementService;
        this.view.show(this);
        this.view.setType(jscape.LogService.Type.FILE);
    },

    pause: function() {
        this.view.hide();
    },

    resume: function() {
        this.view.show(this);
    },

    valid: function() {
        var item = this.getItem(this.getCurrentItem());
        return $.type(item) === "object" ? item.valid() : true;
    },

    applyChanges: function() {
        var item = this.getItem(this.getCurrentItem());
        return $.type(item) === "object" ? item.applyChanges() : null;
    },

    onChangeType: function () {
        this.setCurrentItem(this.view.getType().toUpperCase());
    },

    /** @override */
    _createItem: function(item) {
        var controller = new item(this.resources, this.view.container());
        controller.start(this.managementService, null, null);
        return controller;
    }
});

/** @class */
jscape.domain.wizard.LogServiceView = Class.extend(
    /** @lends {jscape.domain.wizard.LogServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=logservicefields]", container).hide();

        this.type = $("select[name=type]", this.fieldset);
        this.type.combobox({required: true, editable: false, width: 120, panelHeight: "auto", onChange: $.proxy(this._onChange$Type, this)});
    },

    getType: function() {
        return this.type.combobox("getValue");
    },

    setType: function (value) {
        var select = this.type.combobox("getValue");
        this.type.combobox("setValue", value);
        if (select === value) {
            this._onChange$Type();
        }
    },

    container: function() {
        return this.fieldset;
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.wizard.LogServiceView.LISTENER;
        this.fieldset.show();
    },

    hide: function() {
        this.fieldset.hide();
    },

    _onChange$Type: function() {
        this.listener.onChangeType(this);
    }
});
jscape.domain.wizard.LogServiceView.LISTENER = {
    onChangeType: $.noop
};