jscape.domain = jscape.domain || {};

/** @class */
jscape.As2Message = Class.extend(
    /** @lends {jscape.As2Message.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e, f, g, h, i, j) {
        this.messageId = a || "";
        this.date = parseInt(b) || 0;
        this.type = c || jscape.As2Message.REQUEST;
        this.sender = d || "";
        this.recipient = e || "";
        this.username = f || "";
        this.tradingPartner = g || "";
        this.filename = h || "";
        this.receiptType = i || jscape.As2Message.ReceipType.NONE;
        this.status = j || jscape.As2Message.Status.PENDING;
    }
});
jscape.As2Message.fromJSON = function(data) {
    return $.extend(new jscape.As2Message(), data);
};
jscape.As2Message.Type = {REQUEST: "REQUEST", RECEIPT: "RECEIPT"};
jscape.As2Message.ReceipType = {SYNC: "SYNC", ASYNC: "ASYNC", NONE: "NONE"};
jscape.As2Message.Status = {SUCCESS: "SUCCESS", FAILURE: "FAILURE", PENDING: "PENDING"};

/** @class */
jscape.domain.AS2Module = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.AS2Module.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.As2MessagesController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onResume: function () {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.As2MessagesController = Class.extend(
    /** @lends {jscape.domain.As2MessagesController.prototype} */
{
    messages: null,

    /** @constructor */
    initialize: function(resources) {
        this.messages = [];
        this.resources = resources;

        this.view = new jscape.domain.As2MessagesView(resources);
        this.sendFileController = new jscape.domain.As2SendFileController(resources);
        this.viewController = new jscape.domain.As2ViewMessageController(resources);
        this.deleteController = new jscape.domain.As2DeleteMessageController(resources)
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onRefresh: function() {
        this._loadData();
    },

    onSendFile: function() {
        this.sendFileController.start(this.managementService, this.domainName)
                .done($.proxy(function() {
                    window.setTimeout($.proxy(this._loadData, this), 1000);
                }, this));
    },

    onViewMessage: function(message) {
        this.viewController.start(this.managementService, this.domainName, message);
    },

    onResendMessage: function(message) {
        var id = message.messageId;
        this._confirmResend(id).done($.proxy(function() {
            this.managementService.resendAs2Message(this.domainName, id)
                    .done($.proxy(function() {
                        window.setTimeout($.proxy(this._loadData, this), 1000);
                    }, this));
        }, this));
    },

    onDeleteMessage: function(messages) {
        this.deleteController.start(this.managementService, this.domainName, messages).done($.proxy(this._messageDeleted, this));
    },

    _confirmResend: function(messageId) {
        var deferred = $.Deferred();
        var win = $.messager.confirm(this.resources.getString("AS2_MESSAGES_CONFIRM_RESEND_TITLE"),
                                     this.resources.getString("AS2_MESSAGES_CONFIRM_RESEND_MESSAGE", {0: messageId}),
                                     function(confirmed) { if (confirmed) { deferred.resolve(); } else { deferred.reject(); }});
        win.window("resize", {width: 620}).window("center");
        return deferred.promise();
    },

    _loadData: function() {
        this.managementService.getAs2Messages(this.domainName)
                .done($.proxy(function(messages) {
                    this.messages = messages || [];
                    this.view.setAs2Messages(this.messages);
                }, this));
    },

    _messageDeleted: function (items) {
        var index = -1;
        while (items.length) {
            index = $.inArray(items.pop(), this.messages);
            this.messages.splice(index, 1);
        }
        this.view.setAs2Messages(this.messages);

        if (this.messages.length) {
            index = Math.max(0, Math.min(index, this.messages.length - 1));
            this.view.selectItem(this.messages[index]);
        }
    }
});


/** @class */
jscape.domain.As2MessagesView = Class.extend(
    /** @lends {jscape.domain.As2MessagesView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.selection = [];
        this.resources = resources;

        this.fieldset = $("fieldset[name=as2messagesfields]");

        this.data = $("table[role=grid][aria-label=as2messages]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            ctrlSelect: true,
            remoteSort: false,
            pagination: true,
            search: true,
            sortName: "date",
            columns: [[
                {field: "date", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 200, align: "right", formatter: function(value) { return new Date(value).toLocaleString(); }},
                {field: "username", title: $("thead th:nth-child(2)", this.data).text(), sortable: true, width: 90},
                {field: "tradingPartner", title: $("thead th:nth-child(3)", this.data).text(), sortable: true, width: 120},
                {field: "type", title: $("thead th:nth-child(4)", this.data).text(), sortable: true, width: 130, formatter: $.proxy(this._formatType, this)},
                {field: "messageId", title: $("thead th:nth-child(5)", this.data).text(), sortable: true, width: 130},
                {field: "filename", title: $("thead th:nth-child(6)", this.data).text(), sortable: true, width: 180},
                {field: "sender", title: $("thead th:nth-child(7)", this.data).text(), sortable: true, width: 120},
                {field: "recipient", title: $("thead th:nth-child(8)", this.data).text(), sortable: true, width: 150},
                {field: "receiptType", title: $("thead th:nth-child(9)", this.data).text(), sortable: true, width: 90, formatter: $.proxy(this._formatReceiptType, this)},
                {field: "status", title: $("thead th:nth-child(10)", this.data).text(), sortable: true, width: 100, formatter: $.proxy(this._formatStatus, this), styler: $.proxy(this._formatStatusStyle, this)}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$ViewButton, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this)
        });

        this.refreshButton = $("a[role=button][name=refresh]", this.fieldset);
        this.refreshButton.linkbutton({onClick: $.proxy(this._onClick$RefreshButton, this)});

        this.sendFileButton = $("a[role=button][name=sendfile]", this.fieldset);
        this.sendFileButton.linkbutton({onClick: $.proxy(this._onClick$SendFileButton, this)});

        this.viewButton = $("a[role=button][name=view]", this.fieldset);
        this.viewButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ViewButton, this)});

        this.resendButton = $("a[role=button][name=resend]", this.fieldset);
        this.resendButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ResendButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.As2MessagesView.LISTENER;
        this.data.datagrid("resize");
        this._adjustButtonsState();
    },

    setAs2Messages: function(values) {
        this.selection = [];
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectItem: function (value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    _formatType: function(value) {
        return this.resources.getString("AS2_MESSAGES_TYPE_" + value.toUpperCase());
    },

    _formatReceiptType: function(value) {
        return this.resources.getString("AS2_MESSAGES_RECEIPT_TYPE_" + value.toUpperCase());
    },

    _formatStatus: function(value) {
        return this.resources.getString("AS2_MESSAGES_STATUS_" + value.toUpperCase());
    },

    _formatStatusStyle: function(value) {
        switch (value) {
            case jscape.As2Message.Status.PENDING:
                return "color:grey";
            case jscape.As2Message.Status.SUCCESS:
                return "color:green";
            case jscape.As2Message.Status.FAILURE:
                return "color:red";
            default:
                return "";
        }
    },

    _onClick$RefreshButton: function() {
        this.listener.onRefresh();
    },

    _onClick$SendFileButton: function() {
        this.listener.onSendFile();
    },

    _onClick$ViewButton: function() {
        this.listener.onViewMessage(this.selection[0]);
    },

    _onClick$ResendButton: function() {
        this.listener.onResendMessage(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteMessage(this.selection);
    },

    _adjustRowSelection: function () {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var message = this.selection.length == 1 ? this.selection[0] : null;
        var resendAllowed = message != null && (message.type == jscape.As2Message.Type.REQUEST || message.receiptType == jscape.As2Message.ReceipType.ASYNC);

        this.viewButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.resendButton.linkbutton(resendAllowed ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }});
jscape.domain.As2MessagesView.LISTENER = {
    onRefresh: $.noop,
    onSendFile: $.noop,
    onViewMessage: $.noop,
    onResendMessage: $.noop,
    onDeleteMessage: $.noop
};

/** @class */
jscape.domain.As2SendFileController = Class.extend(
    /** @lends {jscape.domain.As2SendFileController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._loadPartners();

        return this.deferred.promise();
    },

    onSelectFile: function(dialog) {
        var loadFn = $.proxy(this.managementService.listFileSystem, this.managementService);
        var createFn = $.proxy(this.managementService.createFileSystemDir, this.managementService);
        new SelectPathController(loadFn, createFn).selectFile(function(entry) { dialog.setFile(entry.path); });
    },

    onSubmit: function (dialog) {
        var partnerName = dialog.getPartner();
        var file = dialog.getFile();
        dialog.destroy();

        this.managementService.sendTradingPartnerFile(this.domainName, partnerName, file)
                .done(this.deferred.resolve)
                .fail(this.deferred.reject);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _loadPartners: function() {
        this.managementService.getTradingPartners(this.domainName)
                .done($.proxy(this._setupDialog, this))
                .fail(this.deferred.reject);
    },

    _setupDialog: function(partners) {
        partners = $.grep(partners, function(partner) { return jscape.TradingPartner.Protocol.AS2 == partner.protocol});
        if (partners.length == 0) {
            $.messager.alert(this.resources.getString("APPLICATION_ERROR_TITLE"),
                             this.resources.getString("AS2_MESSAGES_ERROR_NO_PARTNERS"),
                             "warning");
        } else {
            var dialog = new jscape.domain.As2SendFileDialog(this.resources);
            dialog.setPartners($.map(partners, function(partner) { return partner.name; }));
            dialog.show(this, partners);
        }
    }
});

/** @class */
jscape.domain.As2SendFileDialog = Class.extend(
    /** @lends {jscape.domain.As2SendFileDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-as2-sendfile").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 600, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=as2sendfilefields]", this.dialog);

        this.partner = $("select[name=partner]", this.fieldset);
        this.partner.combobox({required: true, editable: false, width: 170, panelHeight: "auto", onLoadSuccess: function() {
            var data = $(this).combobox("getData");
            data.length && $(this).combobox("select", data[0].value);
        }});

        this.file = $("input[name=file]", this.fieldset);
        this.file.validatebox({required: true, tipPosition: "left", missingMessage: resources.getString("AS2_MESSAGES_ERROR_BAD_FILE")});
        this.browseFileButton = $("a[role=button][name=browsefile]", this.fieldset);
        this.browseFileButton.linkbutton({onClick: $.proxy(this._onClick$BrowseFileButton, this)});
    },

    getPartner: function() {
        return this.partner.combobox("getValue");
    },

    setPartners: function(values) {
        values = values.sort(function(a, b) { return a.toUpperCase() > b.toUpperCase(); });
        values = $.map(values, function(value) {return {text: value, value: value}; });
        this.partner.combobox("loadData", values);
    },

    getFile: function() {
        return this.file.val();
    },

    setFile: function(value) {
        this.file.val(value).validatebox("validate");
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.As2SendFileDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$BrowseFileButton: function() {
        this.listener.onSelectFile(this);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.As2SendFileDialog.LISTENER = {
    onSelectFile: $.noop
};

/** @class */
jscape.domain.As2ViewMessageController = Class.extend(
    /** @lends {jscape.domain.As2ViewMessageController.prototype} */
{
    recordData: null,
    recordRawData: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, message) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.messageId = message.messageId;
        this.deferred = $.Deferred();
        this.recordData = null;
        this.recordRawData = null;

        var dialog = new jscape.domain.ViewAs2MessageDialog(this.resources);
        dialog.setTitle(message.type == jscape.As2Message.Type.RECEIPT
                        ? this.resources.getString("AS2_MESSAGES_VIEW_DIALOG_TITLE_RECEIPT")
                        : this.resources.getString("AS2_MESSAGES_VIEW_DIALOG_TITLE_REQUEST"));
        dialog.show(this);

        this.managementService.getAs2MessageRecordView(this.domainName, this.messageId)
                .done($.proxy(function(data) {
                    this.recordData = data || null;
                    dialog.setRecordView(data);
                }, this))
                .fail(this.deferred.reject);

        return this.deferred.promise();
    },

    onRecordView: function(dialog) {
        dialog.setRecordView(this.recordData);
    },

    onRawView: function(dialog) {
        if (this.recordRawData == null) {
            this.managementService.getAs2MessageRecordRawView(this.domainName, this.messageId)
                    .done($.proxy(function(data) {
                        this.recordRawData = data || null;
                        dialog.setRecordRawView(data);
                    }, this));
        } else {
            dialog.setRecordRawView(this.recordRawData);
        }
    },

    onSubmit: function(dialog) {
        dialog.destroy();
        this.deferred.resolve();
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.resolve();
    }
});

/** @class */
jscape.domain.ViewAs2MessageDialog = Class.extend(
    /** @lends {jscape.domain.ViewAs2MessageDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.dialog = $("#as2-viewrecord").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 700, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_CLOSE") || "Close", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.data = $("table[role=grid][aria-label=as2messagerecord]", this.dialog);
        this.data.propertygrid({
            height: 500, sortName: "name",
            toolbar: "#{0} div[role=toolbar]".supplant({0: this.dialog.attr("id")})
        });

        this.rawViewButton = $("a[role=button][name=row]", this.dialog);
        this.rawViewButton.linkbutton({toggle: true, onClick: $.proxy(this._onClick$RawViewButton, this)});
    },

    setTitle: function(value) {
        this.dialog.dialog("setTitle", value || "");
    },

    setRecordView: function(record) {
        var data = $.map(record || {}, function(value, key) { return {name: key, value: value}; });
        this.data.propertygrid("getPanel").find(".datagrid-view").show();
        this.data.propertygrid("loading");
        this.data.propertygrid("loadData", data);
        this.data.propertygrid("loaded");
    },

    setRecordRawView: function(value) {
        var view = this.data.propertygrid("getPanel").find(".datagrid-view");
        if (this.rawData == null) {
            this.rawData = $("<div></div>").appendTo(view.parent());
            this.rawData.width(view.width()).height(view.height()).css("overflow", "auto");
        }

        this.rawData.html("<pre>" + value + "</pre>");
        view.hide();
        this.rawData.show();
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.ViewAs2MessageDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$RawViewButton: function() {
        if (this.rawViewButton.linkbutton("options").selected) {
            this.listener.onRawView(this);
        } else {
            this.listener.onRecordView(this);
        }
    },

    _onSubmit: function () {
        this.hide();
        this.listener.onSubmit(this);
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.ViewAs2MessageDialog.LISTENER = {
    onRecordView: $.noop,
    onRawView: $.noop
};


/** @class */
jscape.domain.As2DeleteMessageController = Class.extend(
    /** @lends {jscape.domain.As2DeleteMessageController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function (managementService, domainName, items) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._confirmDelete(items)
                .done($.proxy(this._deleteConfirmed, this, items))
                .fail(this.deferred.reject);

        return this.deferred.promise();
    },

    _confirmDelete: function(items) {
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("AS2_MESSAGES_CONFIRM_DELETE_TITLE"),
                this.resources.getString("AS2_MESSAGES_CONFIRM_DELETE_MESSAGE"));
    },

    _deleteConfirmed: function(items) {
        var promises = new Array(items.length);
        for (var i = 0; i < items.length; i++) {
            promises[i] = this.managementService.deleteAs2Message(this.domainName, items[i].messageId);
        }

        $.when.apply(jQuery, promises)
                .done($.proxy(this.deferred.resolve, this.deferred, items))
                .fail(this.deferred.reject);
    }
});