jscape.domain = jscape.domain || {};

/** @class */
jscape.AuthService = Class.extend(
    /** @lends {jscape.AuthService.prototype} */
{
    /** @constructor */
    initialize: function(type) {
        this.type = type || null;
    }
});
jscape.AuthService.fromJSON = function(data) {
    if (data && data.hasOwnProperty("usernameConversion")) {
        data.usernameConversion = jscape.UsernameConversion.valueOf(data.usernameConversion);
    }
    return data ? $.extend(new jscape.AuthService(), data) : jscape.AuthService.NONE;
};
jscape.AuthService.TYPE = {NONE: 'NONE', DOMAIN: 'DOMAIN', DB: 'DB', DB_QUERY: 'DB_QUERY', LDAP: 'LDAP', LDAP_QUERY: 'LDAP_QUERY', NTLM: 'NTLM', PAM: 'PAM', CUSTOM: 'CUSTOM', PHONE: 'PHONE'};
jscape.UsernameConversion = {
    NONE: 'NONE', UPPERCASE: 'UPPERCASE', LOWERCASE: 'LOWERCASE',
    values: function() { return [this.NONE, this.UPPERCASE, this.LOWERCASE]; },
    valueOf: function(value) { return $.inArray(value, this.values()) != -1 ? value : this.NONE; }
};
jscape.AuthService.NONE = $.extend(new jscape.AuthService(jscape.AuthService.TYPE.NONE), {toJSON: function() { return null; }});
jscape.AuthService.domain = function() {
    return new jscape.AuthService(jscape.AuthService.TYPE.DOMAIN);
};
jscape.AuthService.db = function(url, template, usernameConversion) {
    return $.extend(new jscape.AuthService(jscape.AuthService.TYPE.DB), {url: url, accountTemplate: template, usernameConversion: usernameConversion});
};
jscape.AuthService.dbQuery = function(url, username, password, query, hasher, template, usernameConversion) {
    return $.extend(new jscape.AuthService(jscape.AuthService.TYPE.DB_QUERY), {url: url, username: username, password: password, sql: query, hasher: hasher, accountTemplate: template, usernameConversion: usernameConversion});
};
jscape.AuthService.ldap = function(host, port, timeout, userDN, useSsl, anonymousAllowed, template, usernameConversion, failover) {
    return $.extend(new jscape.AuthService(jscape.AuthService.TYPE.LDAP), {host: host, port: port, timeout: timeout, userDN: userDN, useSsl: !!useSsl, anonymousAllowed: !!anonymousAllowed, accountTemplate: template, usernameConversion: usernameConversion, failoverServers: failover ? [failover] : []});
};
jscape.AuthService.ldapQuery = function(host, port, timeout, userDN, searchUserDN, baseDN, password, filter, useSsl, anonymousAllowed, template, usernameConversion, failover) {
    return $.extend(new jscape.AuthService(jscape.AuthService.TYPE.LDAP_QUERY), {host: host, port: port, timeout: timeout, userDN: userDN, searchUserDN: searchUserDN, baseDN: baseDN, password: password, filter: filter, useSsl: !!useSsl, anonymousAllowed: !!anonymousAllowed, accountTemplate: template, usernameConversion: usernameConversion, failoverServers: failover ? [failover] : []});
};
jscape.AuthService.ntlm = function(host, domain, template, usernameConversion) {
    return $.extend(new jscape.AuthService(jscape.AuthService.TYPE.NTLM), {host: host, windowsDomain: domain, accountTemplate: template, usernameConversion: usernameConversion});
};
jscape.AuthService.pam = function(debug, template, usernameConversion) {
    return $.extend(new jscape.AuthService(jscape.AuthService.TYPE.PAM), {debugEnabled: !!debug, accountTemplate: template, usernameConversion: usernameConversion});
};
jscape.AuthService.custom = function(className, template, usernameConversion) {
    return $.extend(new jscape.AuthService(jscape.AuthService.TYPE.CUSTOM), {className: className, accountTemplate: template, usernameConversion: usernameConversion});
};

/** @class */
jscape.PhoneFactorService = Class.extend(
    /** @lends {jscape.PhoneFactorService.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.licenseDirectory = a || "";
        this.password = b || "";
        this.internationalCallsAllowed = !!c;
    }
});
jscape.PhoneFactorService.fromJSON = function(data) {
    return data ? $.extend(new jscape.PhoneFactorService(), data) : jscape.AuthService.NONE;
};

/** @class */
jscape.WebSsoService = Class.extend(
    /** @lends {jscape.WebSsoService.prototype} */
{
    /** @constructor */
    initialize: function(type) {
        this.type = type || null;
    }
});
jscape.WebSsoService.fromJSON = function(data) {
    return data ? $.extend(new jscape.WebSsoService(), data) : jscape.AuthService.NONE;
};
jscape.WebSsoService.TYPE = {OPEN_ID: 'OPEN_ID', SAML: 'SAML'};
jscape.WebSsoService.openId = function(signInUrl, signOutUrl, nonSsoLoginAllowed, template, usernameConversion) {
    return $.extend(new jscape.WebSsoService(jscape.WebSsoService.TYPE.OPEN_ID), {signInUrl: signInUrl, signOutUrl: signOutUrl, nonSsoLoginAllowed: !!nonSsoLoginAllowed, accountTemplate: template, usernameConversion: usernameConversion});
};
jscape.WebSsoService.saml = function(signInUrl, signOutUrl, changePasswordUrl, certificate, nonSsoLoginAllowed, template, usernameConversion) {
    return $.extend(new jscape.WebSsoService(jscape.WebSsoService.TYPE.SAML), {signInUrl: signInUrl, signOutUrl: signOutUrl, changePasswordUrl: changePasswordUrl, verificationHostKey: certificate, nonSsoLoginAllowed: !!nonSsoLoginAllowed, accountTemplate: template, usernameConversion: usernameConversion});
};

/** @class */
jscape.domain.AuthenticationModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.AuthenticationModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.authServiceController = new jscape.domain.AuthenticationServiceController(this.resources);
        this.twoFactorController = new jscape.domain.TwoFactorAuthServiceController(this.resources);
        this.webSsoController = new jscape.domain.WebSsoAuthServiceController(this.resources);
    },

    onStart: function () {
        this.authServiceController.start(this.managementService, this.domainName);
        this.twoFactorController.start(this.managementService, this.domainName);
        this.webSsoController.start(this.managementService, this.domainName);
    },

    onPause: function() {
        var authServiceChanged = this.authServiceController.hasChanged();
        var twoFactorChanged = this.twoFactorController.hasChanged();
        var webSsoChanged = this.webSsoController.hasChanged();

        if (authServiceChanged || twoFactorChanged || webSsoChanged) {
            this._confirmApply()
                    .done($.proxy(function () {
                        authServiceChanged && this.authServiceController.onApply();
                        twoFactorChanged && this.twoFactorController.onApply();
                        webSsoChanged && this.webSsoController.onApply();
                    }, this));
        }
    },

    onResume: function() {
        this.authServiceController._loadData();
        this.twoFactorController._loadData();
        this.webSsoController._loadData();
    }
});

/** @class */
jscape.domain.AuthenticationServiceController = Class.extend(
    /** @lends {jscape.domain.AuthenticationServiceController.prototype} */
{
    DEFAULT_DESCRIPTOR: jscape.AuthService.domain(),

    templates: null,
    descriptor: null,

    /** @constructor */
    initialize: function(resources) {
        this.templates = [];
        this.resources = resources;

        this.view = new jscape.domain.AuthenticationServiceView(resources);
        this.controllers = { // lazy controllers creation
            DOMAIN: jscape.domain.DomainAuthServiceController,
            DB: jscape.domain.DbAuthServiceController,
            DB_QUERY: jscape.domain.DbQueryAuthServiceController,
            LDAP: jscape.domain.LdapAuthServiceController,
            LDAP_QUERY: jscape.domain.LdapQueryAuthServiceController,
            NTLM: jscape.domain.NtlmAuthServiceController,
            PAM: jscape.domain.PamAuthServiceController,
            CUSTOM: jscape.domain.CustomAuthServiceController
        };
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.descriptor && !Object._equals(this.descriptor, this._createDescriptor());
    },

    onApply: function () {
        var descriptor = this._createDescriptor();
        if (descriptor == null) {
            return $.Deferred().reject().promise();
        }

        return this.managementService.setAuthenticationService(this.domainName, descriptor)
                .done($.proxy(function () {
                    this.descriptor = descriptor;
                    this._fireApplySuccess();
                }, this));
    },

    onDiscard: function () {
        $.each(this.controllers, function (_, controller) {
            if ($.type(controller) === "object") {
                controller.discardChanges();
            }
        });
        this._setupView();
    },

    onChangeType: function () {
        var serviceType = (this.view.getType() || "").toUpperCase();
        $.each(this.controllers, $.proxy(function(type, controller) {
            var t = $.type(controller);
            if (serviceType === type) {
                if (t === "function") {
                    this.controllers[type] = new controller(this.resources);
                    this.controllers[type].start(this.managementService, this.domainName, this.descriptor, this.templates);
                } else {
                    controller.resume();
                }
            } else {
                if (t === "object") {
                    controller.pause();
                }
            }
        }, this));
    },

    _loadData: function () {
        $.when(this.managementService.getAuthenticationService(this.domainName),
               this.managementService.getTemplates(this.domainName))
                .done($.proxy(function (descriptor, templates) {
                    this.descriptor = descriptor || this.DEFAULT_DESCRIPTOR;
                    this.templates = (templates || []).sort();
                }, this))
                .fail($.proxy(function () {
                    this.descriptor = this.DEFAULT_DESCRIPTOR;
                }, this))
                .always($.proxy(this._setupView, this));
    },

    _createDescriptor: function() {
        var controller = this.controllers[(this.view.getType() || "").toUpperCase()];
        return $.type(controller) === "object" ? controller.applyChanges() : null;
    },

    _setupView: function () {
        this.view.setType((this.descriptor && this.descriptor.type) || this.DEFAULT_DESCRIPTOR.type);
    },

    _fireApplySuccess: function() {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.AuthenticationServiceView = Class.extend(
    /** @lends {jscape.domain.AuthenticationServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=authservicefields]");

        this.type = $("select[name=type]", this.fieldset);
        this.type.combobox({required: true, editable: false, selectOnNavigation: false, panelHeight: "auto", onChange: $.proxy(this._onChange$Type, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.AuthenticationServiceView.LISTENER;
    },

    getType: function() {
        return this.type.combobox("getValue");
    },

    setType: function(value) {
        var selectedType = this.type.combobox("getValue");
        this.type.combobox("clear");
        this.type.combobox("select", value);
        if (selectedType === value) {
            this._onChange$Type();
        }
    },

    _onChange$Type: function() {
        this.listener.onChangeType(this);
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard(this);
    }
});
jscape.domain.AuthenticationServiceView.LISTENER = {
    onChangeType: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.BaseAuthServiceController = Class.extend(
    /** @lends {jscape.domain.BaseAuthServiceController.prototype} */
{
    view: null,

    /** @constructor */
    initialize: function(resources) {
        this.testController = new jscape.domain.TestAuthServiceController(resources);
    },

    start: function(managementService, domainName, descriptor, templates) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.descriptor = descriptor;
        this.templates = templates;

        this._setupView(this.descriptor);
        this.view.show(this);
    },

    resume: function () {
        this.view.show(this);
    },

    pause: function () {
        this.view.hide();
    },

    applyChanges: function () {
        return this.view.valid() ? this._createDescriptor() : null;
    },

    discardChanges: function() {
        this._setupView(this.descriptor);
    },

    onTest: function() {
        this.testController.start(this.managementService, this.domainName, this._createDescriptor());
    },

    _createDescriptor: function () {
        return null;
    },

    _setupView: function (descriptor) {
    }
});

/** @class */
jscape.domain.NoneAuthServiceController = jscape.domain.BaseAuthServiceController.extend(
    /** @lends {jscape.domain.BaseAuthServiceController.prototype} */
{
    /** @constructor */
    initialize: function () {},
    start: function () {},
    resume: function () {},
    pause: function () {},
    applyChanges: function () { return this._createDescriptor(); },
    discardChanges: function () {},
    _createDescriptor: function () { return jscape.AuthService.NONE; },
    _setupView: function () {}
});

/** @class */
jscape.domain.BaseAuthServiceView = Class.extend(
    /** @lends {jscape.domain.BaseAuthServiceView.prototype} */
{
    show: function(listener) {
        this.listener = listener || jscape.domain.BaseAuthServiceController.LISTENER;
        this.fieldset.form("validate");
        this.fieldset.show();
    },

    hide: function() {
        this.fieldset.hide();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$TestButton: function() {
        if (this.valid()) {
            this.listener.onTest(this);
        }
    }
});
jscape.domain.BaseAuthServiceController.LISTENER = {
    onTest: $.noop,
    onAddVariable: $.noop
};

/** @class */
jscape.domain.DomainAuthServiceController = jscape.domain.BaseAuthServiceController.extend(
    /** @lends {jscape.domain.DomainAuthServiceController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.view = new jscape.domain.DomainAuthServiceView(resources);
    },

    /** @override */
    _createDescriptor: function () {
        return jscape.AuthService.domain();
    }
});

/** @class */
jscape.domain.DomainAuthServiceView = jscape.domain.BaseAuthServiceView.extend(
    /** @lends {jscape.domain.DomainAuthServiceView.prototype} */
{
    /** @constructor */
    initialize: function() {
        this.fieldset = $("fieldset[name=domainauthfields]").css("border", "none").hide();
    }
});

/** @class */
jscape.domain.DbAuthServiceController = jscape.domain.BaseAuthServiceController.extend(
    /** @lends {jscape.domain.DbAuthServiceController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
        this.view = new jscape.domain.DbAuthServiceView(resources);
    },

    /** @override */
    _createDescriptor: function () {
        return jscape.AuthService.db(this.view.getUrl(),
                                     this.view.getTemplate(),
                                     jscape.UsernameConversion.valueOf(this.view.getUsernameConversion()));
    },

    /** @override */
    _setupView: function (descriptor) {
        this.view.setTemplates(this.templates);

        if (descriptor && descriptor.type == jscape.AuthService.TYPE.DB) {
            this.view.setUrl(descriptor.url);
            this.view.setTemplate(descriptor.accountTemplate);
            this.view.setUsernameConversion(descriptor.usernameConversion == jscape.UsernameConversion.NONE ? null : descriptor.usernameConversion);
        }
    }
});

/** @class */
jscape.domain.DbAuthServiceView = jscape.domain.BaseAuthServiceView.extend(
    /** @lends {jscape.domain.DbAuthServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=dbauthfields]").hide();

        this.url = $("input[name=dburl]", this.fieldset);
        this.url.validatebox({required: true, validType: "length[1,2048]", missingMessage: resources.getString("AUTH_ERROR_BAD_DB_URL"), invalidMessage: resources.getString("AUTH_ERROR_BAD_DB_URL")});

        this.useTemplate = $("input[name=usetemplate]:checkbox", this.fieldset);
        this.useTemplate.change($.proxy(function() {
            var enabled = this.useTemplate.is(":checked");
            this.template.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.template = $("select[name=template]", this.fieldset);
        this.template.combobox({
            required: true,
            editable: false,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.template.combobox("resize", 170).combobox("clear").combobox("select", data[0].value);
                    this.useTemplate.prop("checked", true).change();
                } else {
                    this.useTemplate.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.useUsernameConversion = $("input[name=useusernameconversion]:checkbox", this.fieldset);
        this.useUsernameConversion.change($.proxy(function() {
            var enabled = this.useUsernameConversion.is(":checked");
            this.usernameConversion.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.usernameConversion = $("select[name=usernameconversion]", this.fieldset);
        this.usernameConversion.combobox({
            required: true,
            editable: false,
            width: 100,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.usernameConversion.combobox("clear").combobox("select", data[0].value);
                    this.useUsernameConversion.change();
                } else {
                    this.useUsernameConversion.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.testButton = $("a[role=button][name=test]", this.fieldset);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getUrl: function() {
        return this.url.val().trim();
    },

    setUrl: function(value) {
        this.url.val(value);
    },

    getTemplate: function() {
        return this.useTemplate.is(":checked") ? this.template.combobox("getValue") : null;
    },

    setTemplate: function(value) {
        this.template.combobox("select", value);
        this.useTemplate.prop("checked", value != null).change();
    },

    setTemplates: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.template.combobox("loadData", data);
    },

    getUsernameConversion: function() {
        return this.useUsernameConversion.is(":checked") ? this.usernameConversion.combobox("getValue") : null;
    },

    setUsernameConversion: function(value) {
        this.useUsernameConversion.prop("checked", value != null).change();
        if (value != null) {
            this.usernameConversion.combobox("select", value);
        }
    }
});

/** @class */
jscape.domain.DbQueryAuthServiceController = jscape.domain.BaseAuthServiceController.extend(
    /** @lends {jscape.domain.DbQueryAuthServiceController.prototype} */
{
    VARIABLES: ["username", "password"],

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
        this.view = new jscape.domain.DbQueryAuthServiceView(resources);
        this.variableController = new jscape.VariableController();
    },

    onAddVariable: function(input) {
        this.variableController.start(input, this.VARIABLES);
    },

    /** @override */
    _createDescriptor: function () {
        return jscape.AuthService.dbQuery(this.view.getUrl(),
                                          this.view.getUsername(),
                                          this.view.getPassword(),
                                          this.view.getQuery(),
                                          this.view.getHashClass(),
                                          this.view.getTemplate(),
                                          jscape.UsernameConversion.valueOf(this.view.getUsernameConversion()));
    },

    /** @override */
    _setupView: function (descriptor) {
        this.view.setTemplates(this.templates);

        if (descriptor && descriptor.type == jscape.AuthService.TYPE.DB_QUERY) {
            this.view.setUrl(descriptor.url);
            this.view.setUsername(descriptor.username);
            this.view.setPassword(descriptor.password);
            this.view.setQuery(descriptor.sql);
            this.view.setHashClass(descriptor.hasher);
            this.view.setTemplate(descriptor.accountTemplate);
            this.view.setUsernameConversion(descriptor.usernameConversion == jscape.UsernameConversion.NONE ? null : descriptor.usernameConversion);
        }
    }
});

/** @class */
jscape.domain.DbQueryAuthServiceView = jscape.domain.BaseAuthServiceView.extend(
    /** @lends {jscape.domain.DbQueryAuthServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=dbqueryauthfields]").hide();

        this.url = $("input[name=dburl]", this.fieldset);
        this.url.validatebox({required: true, validType: "length[1,2048]", missingMessage: resources.getString("AUTH_ERROR_BAD_DB_URL"), invalidMessage: resources.getString("AUTH_ERROR_BAD_DB_URL")});

        this.username = $("input[name=dbuser]", this.fieldset);
        this.username.validatebox({required: true, validType: "length[1,50]", missingMessage: resources.getString("AUTH_ERROR_BAD_DB_USER"), invalidMessage: resources.getString("AUTH_ERROR_BAD_DB_USER")});

        this.password = $("input[name=dbpwd]", this.fieldset);

        this.query = $("textarea[name=query]", this.fieldset).css({"width": "50em", "height": "3em"});
        this.query.validatebox({required: true, validType: "length[1,2000]", missingMessage: resources.getString("AUTH_ERROR_BAD_DB_QUERY"), invalidMessage: resources.getString("AUTH_ERROR_BAD_DB_QUERY")});
        this.query.focus($.proxy(function() { this.addVariable.linkbutton("enable"); }, this));

        this.hashClass = $("input[name=hashclass]", this.fieldset);

        this.useTemplate = $("input[name=usetemplate]:checkbox", this.fieldset);
        this.useTemplate.change($.proxy(function() {
            var enabled = this.useTemplate.is(":checked");
            this.template.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.template = $("select[name=template]", this.fieldset);
        this.template.combobox({
            required: true,
            editable: false,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.template.combobox("resize", 170).combobox("select", data[0].value);
                    this.useTemplate.prop("checked", true).change();
                } else {
                    this.useTemplate.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.useUsernameConversion = $("input[name=useusernameconversion]:checkbox", this.fieldset);
        this.useUsernameConversion.change($.proxy(function() {
            var enabled = this.useUsernameConversion.is(":checked");
            this.usernameConversion.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.usernameConversion = $("select[name=usernameconversion]", this.fieldset);
        this.usernameConversion.combobox({
            required: true,
            editable: false,
            width: 100,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.usernameConversion.combobox("clear").combobox("select", data[0].value);
                    this.useUsernameConversion.change();
                } else {
                    this.useUsernameConversion.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.addVariable = $("a[role=button][name=addvar]", this.fieldset);
        this.addVariable.linkbutton({disabled: true, onClick: $.proxy(this._onClick$AddVariableButton, this)});

        this.testButton = $("a[role=button][name=test]", this.fieldset);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getUrl: function() {
        return this.url.val().trim();
    },

    setUrl: function(value) {
        this.url.val(value);
    },

    getUsername: function() {
        return this.username.val();
    },

    setUsername: function(value) {
        this.username.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getQuery: function() {
        return this.query.val();
    },

    setQuery: function(value) {
        this.query.val(value);
    },

    getHashClass: function() {
        return this.hashClass.val();
    },

    setHashClass: function(value) {
        this.hashClass.val(value);
    },

    getTemplate: function() {
        return this.useTemplate.is(":checked") ? this.template.combobox("getValue") : null;
    },

    setTemplate: function(value) {
        this.template.combobox("select", value);
        this.useTemplate.prop("checked", value != null).change();
    },

    setTemplates: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.template.combobox("loadData", data);
    },

    getUsernameConversion: function() {
        return this.useUsernameConversion.is(":checked") ? this.usernameConversion.combobox("getValue") : null;
    },

    setUsernameConversion: function(value) {
        this.useUsernameConversion.prop("checked", value != null).change();
        if (value != null) {
            this.usernameConversion.combobox("select", value);
        }
    },

    _onClick$AddVariableButton: function() {
        this.addVariable.linkbutton("disable");
        this.listener.onAddVariable(this.query);
    }
});


/** @class */
jscape.domain.LdapAuthServiceController = jscape.domain.BaseAuthServiceController.extend(
    /** @lends {jscape.domain.LdapAuthServiceController.prototype} */
{
    VARIABLES: ["username", "password"],

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
        this.view = new jscape.domain.LdapAuthServiceView(resources);
        this.variableController = new jscape.VariableController();
    },

    onAddVariable: function(input) {
        this.variableController.start(input, this.VARIABLES);
    },

    /** @override */
    _createDescriptor: function () {
        var failover = this.view.getFailoverHost() != null  ? {host: this.view.getFailoverHost(), port: this.view.getFailoverPort()} : null;
        return jscape.AuthService.ldap(this.view.getHost(),
                                       this.view.getPort(),
                                       this.view.getTimeout(),
                                       this.view.getUserDN(),
                                       this.view.isUseSsl(),
                                       this.view.isAnonymousAllowed(),
                                       this.view.getTemplate(),
                                       jscape.UsernameConversion.valueOf(this.view.getUsernameConversion()),
                                       failover);
    },

    /** @override */
    _setupView: function (descriptor) {
        this.view.setTemplates(this.templates);

        if (descriptor && descriptor.type == jscape.AuthService.TYPE.LDAP) {
            this.view.setHost(descriptor.host);
            this.view.setPort(descriptor.port);
            this.view.setTimeout(descriptor.timeout);
            this.view.setUserDN(descriptor.userDN);
            this.view.setUseSsl(descriptor.useSsl);
            this.view.setAnonymousAllowed(descriptor.anonymousAllowed);
            this.view.setTemplate(descriptor.accountTemplate);
            this.view.setUsernameConversion(descriptor.usernameConversion == jscape.UsernameConversion.NONE ? null : descriptor.usernameConversion);

            if (descriptor.failoverServers.length) {
                this.view.setFailoverHost(descriptor.failoverServers[0].host);
                this.view.setFailoverPort(descriptor.failoverServers[0].port);
            }
        }
    }
});

/** @class */
jscape.domain.LdapAuthServiceView = jscape.domain.BaseAuthServiceView.extend(
    /** @lends {jscape.domain.LdapAuthServiceView.prototype} */
{
    TIMEOUT_FACTOR: 1000,
    DEFAULT_PORT: 389,

    focusInField: null,

    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=ldapauthfields]").hide();
        this.fieldset.find(":input").focus($.proxy(function (e) {
            var field = $(e.target);
            if (field.is("[type=text]") || field.is("textarea")) {
                this.addVariable.linkbutton("enable");
                this.focusInField = field;
            } else {
                this.focusInField = null;
                this.addVariable.linkbutton("disable");
            }
        }, this));

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, validType: "length[1,255]", missingMessage: resources.getString("AUTH_ERROR_BAD_LDAP_HOST"), invalidMessage: resources.getString("AUTH_ERROR_BAD_LDAP_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, value: this.DEFAULT_PORT});

        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({required: true, min: 1, max: 999, value: 30});

        this.userDN = $("input[name=userdn]", this.fieldset);
        this.userDN.validatebox({required: true, validType: "length[1,255]", missingMessage: resources.getString("AUTH_ERROR_BAD_USER_DN"), invalidMessage: resources.getString("AUTH_ERROR_BAD_USER_DN")});

        this.useSsl = $("input[name=usessl]", this.fieldset);
        this.anonymousAllowed = $("input[name=allowanonymous]", this.fieldset);

        this.useTemplate = $("input[name=usetemplate]:checkbox", this.fieldset);
        this.useTemplate.change($.proxy(function() {
            var enabled = this.useTemplate.is(":checked");
            this.template.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.template = $("select[name=template]", this.fieldset);
        this.template.combobox({
            required: true,
            editable: false,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.template.combobox("resize", 170).combobox("select", data[0].value);
                    this.useTemplate.prop("checked", true).change();
                } else {
                    this.useTemplate.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.useUsernameConversion = $("input[name=useusernameconversion]:checkbox", this.fieldset);
        this.useUsernameConversion.change($.proxy(function() {
            var enabled = this.useUsernameConversion.is(":checked");
            this.usernameConversion.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.usernameConversion = $("select[name=usernameconversion]", this.fieldset);
        this.usernameConversion.combobox({
            required: true,
            editable: false,
            width: 100,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.usernameConversion.combobox("clear").combobox("select", data[0].value);
                    this.useUsernameConversion.change();
                } else {
                    this.useUsernameConversion.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.useFailover = $("input[name=usefailover]", this.fieldset);
        this.useFailover.change($.proxy(function() {
            var enabled = this.useFailover.is(":checked");
            this.failoverHost.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation").validatebox("isValid");
            this.failoverPort.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        }, this));

        this.failoverHost = $("input[name=failoverhost]", this.fieldset).prop("disabled", true);
        this.failoverHost.validatebox({required: true, validType: "length[1,255]", missingMessage: resources.getString("AUTH_ERROR_BAD_FAILOVER_HOST"), invalidMessage: resources.getString("AUTH_ERROR_BAD_FAILOVER_HOST")});

        this.failoverPort = $("input[name=failoverport]", this.fieldset);
        this.failoverPort.numberspinner({required: true, disabled: true, min: 1, max: 65535, value: this.DEFAULT_PORT});

        this.addVariable = $("a[role=button][name=addvar]", this.fieldset);
        this.addVariable.linkbutton({disabled: true, onClick: $.proxy(this._onClick$AddVariableButton, this)});
        this.userDN.focus($.proxy(function() { this.addVariable.linkbutton("enable"); }, this));

        this.testButton = $("a[role=button][name=test]", this.fieldset);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", parseInt(value) || this.DEFAULT_PORT);
    },

    getTimeout: function() {
        return parseInt(this.timeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    getUserDN: function() {
        return this.userDN.val();
    },

    setUserDN: function(value) {
        this.userDN.val(value);
    },

    isUseSsl: function() {
        return this.useSsl.is(":checked");
    },

    setUseSsl: function(value) {
        this.useSsl.prop("checked", !!value);
    },

    isAnonymousAllowed: function() {
        return this.anonymousAllowed.is(":checked");
    },

    setAnonymousAllowed: function(value) {
        this.anonymousAllowed.prop("checked", !!value);
    },

    getTemplate: function() {
        return this.useTemplate.is(":checked") ? this.template.combobox("getValue") : null;
    },

    setTemplate: function(value) {
        this.template.combobox("select", value);
        this.useTemplate.prop("checked", value != null).change();
    },

    setTemplates: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.template.combobox("loadData", data);
    },

    getUsernameConversion: function() {
        return this.useUsernameConversion.is(":checked") ? this.usernameConversion.combobox("getValue") : null;
    },

    setUsernameConversion: function(value) {
        this.useUsernameConversion.prop("checked", value != null).change();
        if (value != null) {
            this.usernameConversion.combobox("select", value);
        }
    },

    getFailoverHost: function() {
        return this.useFailover.is(":checked") ? this.failoverHost.val() : null;
    },

    setFailoverHost: function(value) {
        this.failoverHost.val(value);
        this.useFailover.prop("checked", !!value).change();
    },

    getFailoverPort: function() {
        return parseInt(this.failoverPort.numberspinner("getValue"));
    },

    setFailoverPort: function(value) {
        this.failoverPort.numberspinner("setValue", parseInt(value) || this.DEFAULT_PORT);
    },

    _onClick$AddVariableButton: function () {
        if (this.focusInField && this.focusInField.length) {
            this.addVariable.linkbutton("disable");
            this.listener.onAddVariable(this.focusInField);
        }
    }
});

/** @class */
jscape.domain.LdapQueryAuthServiceController = jscape.domain.BaseAuthServiceController.extend(
    /** @lends {jscape.domain.LdapQueryAuthServiceController.prototype} */
{
    VARIABLES: ["username", "password"],

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
        this.view = new jscape.domain.LdapQueryAuthServiceView(resources);
        this.variableController = new jscape.VariableController();
    },

    onAddVariable: function(input) {
        this.variableController.start(input, this.VARIABLES);
    },

    /** @override */
    _createDescriptor: function () {
        var failover = this.view.getFailoverHost() != null  ? {host: this.view.getFailoverHost(), port: this.view.getFailoverPort()} : null;
        return jscape.AuthService.ldapQuery(this.view.getHost(),
                                            this.view.getPort(),
                                            this.view.getTimeout(),
                                            this.view.getUserDN(),
                                            this.view.getSearchUserDN(),
                                            this.view.getBaseDN(),
                                            this.view.getPassword(),
                                            this.view.getFilter(),
                                            this.view.isUseSsl(),
                                            this.view.isAnonymousAllowed(),
                                            this.view.getTemplate(),
                                            jscape.UsernameConversion.valueOf(this.view.getUsernameConversion()),
                                            failover);
    },

    /** @override */
    _setupView: function (descriptor) {
        this.view.setTemplates(this.templates);

        if (descriptor && descriptor.type == jscape.AuthService.TYPE.LDAP_QUERY) {
            this.view.setHost(descriptor.host);
            this.view.setPort(descriptor.port);
            this.view.setTimeout(descriptor.timeout);
            this.view.setUserDN(descriptor.userDN);
            this.view.setSearchUserDN(descriptor.searchUserDN);
            this.view.setBaseDN(descriptor.baseDN);
            this.view.setPassword(descriptor.password);
            this.view.setFilter(descriptor.filter);
            this.view.setUseSsl(descriptor.useSsl);
            this.view.setAnonymousAllowed(descriptor.anonymousAllowed);
            this.view.setTemplate(descriptor.accountTemplate);
            this.view.setUsernameConversion(descriptor.usernameConversion == jscape.UsernameConversion.NONE ? null : descriptor.usernameConversion);

            if (descriptor.failoverServers.length) {
                this.view.setFailoverHost(descriptor.failoverServers[0].host);
                this.view.setFailoverPort(descriptor.failoverServers[0].port);
            }
        }
    }
});

/** @class */
jscape.domain.LdapQueryAuthServiceView = jscape.domain.BaseAuthServiceView.extend(
    /** @lends {jscape.domain.LdapQueryAuthServiceView.prototype} */
{
    TIMEOUT_FACTOR: 1000,
    DEFAULT_PORT: 389,

    focusInField: null,

    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=ldapqueryauthfields]").hide();
        this.fieldset.find(":input").focus($.proxy(function (e) {
            var field = $(e.target);
            if (field.is("[type=text]") || field.is("textarea")) {
                this.addVariable.linkbutton("enable");
                this.focusInField = field;
            } else {
                this.focusInField = null;
                this.addVariable.linkbutton("disable");
            }
        }, this));

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, validType: "length[1,255]", missingMessage: resources.getString("AUTH_ERROR_BAD_LDAP_HOST"), invalidMessage: resources.getString("AUTH_ERROR_BAD_LDAP_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, value: this.DEFAULT_PORT});

        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({required: true, min: 1, max: 999, value: 30});

        this.userDN = $("input[name=userdn]", this.fieldset);
        this.userDN.validatebox({required: true, validType: "length[1,255]", missingMessage: resources.getString("AUTH_ERROR_BAD_USER_DN"), invalidMessage: resources.getString("AUTH_ERROR_BAD_USER_DN")});

        this.searchUserDN = $("input[name=searchuserdn]", this.fieldset);
        this.searchUserDN.validatebox({required: true, validType: "length[1,255]", missingMessage: resources.getString("AUTH_ERROR_BAD_SEARCH_USER_DN"), invalidMessage: resources.getString("AUTH_ERROR_BAD_SEARCH_USER_DN")});

        this.password = $("input[name=password]", this.fieldset);
        this.baseDN = $("input[name=basedn]", this.fieldset);

        this.filter = $("textarea[name=filter]", this.fieldset).css({"width": "50em", "height": "3em"});
        this.filter.validatebox({required: true, validType: "length[1,2000]", missingMessage: resources.getString("AUTH_ERROR_BAD_FILTER"), invalidMessage: resources.getString("AUTH_ERROR_BAD_FILTER")});

        this.useSsl = $("input[name=usessl]", this.fieldset);
        this.anonymousAllowed = $("input[name=allowanonymous]", this.fieldset);

        this.useTemplate = $("input[name=usetemplate]:checkbox", this.fieldset);
        this.useTemplate.change($.proxy(function() {
            var enabled = this.useTemplate.is(":checked");
            this.template.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.template = $("select[name=template]", this.fieldset);
        this.template.combobox({
            required: true,
            editable: false,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.template.combobox("resize", 170).combobox("select", data[0].value);
                    this.useTemplate.prop("checked", true).change();
                } else {
                    this.useTemplate.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.useUsernameConversion = $("input[name=useusernameconversion]:checkbox", this.fieldset);
        this.useUsernameConversion.change($.proxy(function() {
            var enabled = this.useUsernameConversion.is(":checked");
            this.usernameConversion.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.usernameConversion = $("select[name=usernameconversion]", this.fieldset);
        this.usernameConversion.combobox({
            required: true,
            editable: false,
            width: 100,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.usernameConversion.combobox("clear").combobox("select", data[0].value);
                    this.useUsernameConversion.change();
                } else {
                    this.useUsernameConversion.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.useFailover = $("input[name=usefailover]", this.fieldset);
        this.useFailover.change($.proxy(function() {
            var enabled = this.useFailover.is(":checked");
            this.failoverHost.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation").validatebox("isValid");
            this.failoverPort.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        }, this));

        this.failoverHost = $("input[name=failoverhost]", this.fieldset).prop("disabled", true);
        this.failoverHost.validatebox({required: true, validType: "length[1,255]", missingMessage: resources.getString("AUTH_ERROR_BAD_FAILOVER_HOST"), invalidMessage: resources.getString("AUTH_ERROR_BAD_FAILOVER_HOST")});

        this.failoverPort = $("input[name=failoverport]", this.fieldset);
        this.failoverPort.numberspinner({required: true, disabled: true, min: 1, max: 65535, value: this.DEFAULT_PORT});

        this.addVariable = $("a[role=button][name=addvar]", this.fieldset);
        this.addVariable.linkbutton({disabled: true, onClick: $.proxy(this._onClick$AddVariableButton, this)});

        this.testButton = $("a[role=button][name=test]", this.fieldset);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", parseInt(value) || this.DEFAULT_PORT);
    },

    getTimeout: function() {
        return parseInt(this.timeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    getUserDN: function() {
        return this.userDN.val();
    },

    setUserDN: function(value) {
        this.userDN.val(value);
    },

    getSearchUserDN: function() {
        return this.searchUserDN.val();
    },

    setSearchUserDN: function(value) {
        this.searchUserDN.val(value);
    },

    getBaseDN: function() {
        return this.baseDN.val();
    },

    setBaseDN: function(value) {
        this.baseDN.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getFilter: function() {
        return this.filter.val();
    },

    setFilter: function(value) {
        this.filter.val(value);
    },

    isUseSsl: function() {
        return this.useSsl.is(":checked");
    },

    setUseSsl: function(value) {
        this.useSsl.prop("checked", !!value);
    },

    isAnonymousAllowed: function() {
        return this.anonymousAllowed.is(":checked");
    },

    setAnonymousAllowed: function(value) {
        this.anonymousAllowed.prop("checked", !!value);
    },

    getTemplate: function() {
        return this.useTemplate.is(":checked") ? this.template.combobox("getValue") : null;
    },

    setTemplate: function(value) {
        this.template.combobox("select", value);
        this.useTemplate.prop("checked", value != null).change();
    },

    setTemplates: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.template.combobox("loadData", data);
    },

    getUsernameConversion: function() {
        return this.useUsernameConversion.is(":checked") ? this.usernameConversion.combobox("getValue") : null;
    },

    setUsernameConversion: function(value) {
        this.useUsernameConversion.prop("checked", value != null).change();
        if (value != null) {
            this.usernameConversion.combobox("select", value);
        }
    },

    getFailoverHost: function() {
        return this.useFailover.is(":checked") ? this.failoverHost.val() : null;
    },

    setFailoverHost: function(value) {
        this.failoverHost.val(value);
        this.useFailover.prop("checked", !!value).change();
    },

    getFailoverPort: function() {
        return parseInt(this.failoverPort.numberspinner("getValue"));
    },

    setFailoverPort: function(value) {
        this.failoverPort.numberspinner("setValue", parseInt(value) || this.DEFAULT_PORT);
    },

    _onClick$AddVariableButton: function() {
        if (this.focusInField && this.focusInField.length) {
            this.addVariable.linkbutton("disable");
            this.listener.onAddVariable(this.focusInField);
        }
    }
});

/** @class */
jscape.domain.NtlmAuthServiceController = jscape.domain.BaseAuthServiceController.extend(
    /** @lends {jscape.domain.NtlmAuthServiceController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
        this.view = new jscape.domain.NtlmAuthServiceView(resources);
    },

    /** @override */
    _createDescriptor: function () {
        return jscape.AuthService.ntlm(this.view.getHost(),
                                       this.view.getDomain(),
                                       this.view.getTemplate(),
                                       jscape.UsernameConversion.valueOf(this.view.getUsernameConversion()));
    },

    /** @override */
    _setupView: function (descriptor) {
        this.view.setTemplates(this.templates);

        if (descriptor && descriptor.type == jscape.AuthService.TYPE.NTLM) {
            this.view.setHost(descriptor.host);
            this.view.setDomain(descriptor.windowsDomain);
            this.view.setTemplate(descriptor.accountTemplate);
            this.view.setUsernameConversion(descriptor.usernameConversion == jscape.UsernameConversion.NONE ? null : descriptor.usernameConversion);
        }
    }
});

/** @class */
jscape.domain.NtlmAuthServiceView = jscape.domain.BaseAuthServiceView.extend(
    /** @lends {jscape.domain.NtlmAuthServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=ntlmauthfields]").hide();

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, validType: "length[1,255]", missingMessage: resources.getString("AUTH_ERROR_BAD_NTLM_HOST"), invalidMessage: resources.getString("AUTH_ERROR_BAD_NTLM_HOST")});

        this.domain = $("input[name=domain]", this.fieldset);
        this.domain.validatebox({required: true, validType: "length[1,255]", missingMessage: resources.getString("AUTH_ERROR_BAD_NTLM_DOMAIN"), invalidMessage: resources.getString("AUTH_ERROR_BAD_NTLM_DOMAIN")});

        this.useTemplate = $("input[name=usetemplate]:checkbox", this.fieldset);
        this.useTemplate.change($.proxy(function() {
            var enabled = this.useTemplate.is(":checked");
            this.template.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.template = $("select[name=template]", this.fieldset);
        this.template.combobox({
            required: true,
            editable: false,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.template.combobox("resize", 170).combobox("select", data[0].value);
                    this.useTemplate.prop("checked", true).change();
                } else {
                    this.useTemplate.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.useUsernameConversion = $("input[name=useusernameconversion]:checkbox", this.fieldset);
        this.useUsernameConversion.change($.proxy(function() {
            var enabled = this.useUsernameConversion.is(":checked");
            this.usernameConversion.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.usernameConversion = $("select[name=usernameconversion]", this.fieldset);
        this.usernameConversion.combobox({
            required: true,
            editable: false,
            width: 100,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.usernameConversion.combobox("clear").combobox("select", data[0].value);
                    this.useUsernameConversion.change();
                } else {
                    this.useUsernameConversion.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.testButton = $("a[role=button][name=test]", this.fieldset);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getDomain: function() {
        return this.domain.val();
    },

    setDomain: function(value) {
        this.domain.val(value);
    },

    getTemplate: function() {
        return this.useTemplate.is(":checked") ? this.template.combobox("getValue") : null;
    },

    setTemplate: function(value) {
        this.template.combobox("select", value);
        this.useTemplate.prop("checked", value != null).change();
    },

    setTemplates: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.template.combobox("loadData", data);
    },

    getUsernameConversion: function() {
        return this.useUsernameConversion.is(":checked") ? this.usernameConversion.combobox("getValue") : null;
    },

    setUsernameConversion: function(value) {
        this.useUsernameConversion.prop("checked", value != null).change();
        if (value != null) {
            this.usernameConversion.combobox("select", value);
        }
    }
});

/** @class */
jscape.domain.PamAuthServiceController = jscape.domain.BaseAuthServiceController.extend(
    /** @lends {jscape.domain.PamAuthServiceController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
        this.view = new jscape.domain.PamAuthServiceView(resources);
    },

    /** @override */
    _createDescriptor: function () {
        return jscape.AuthService.pam(this.view.isDebugEnabled(),
                                      this.view.getTemplate(),
                                      jscape.UsernameConversion.valueOf(this.view.getUsernameConversion()));
    },

    /** @override */
    _setupView: function (descriptor) {
        this.view.setTemplates(this.templates);

        if (descriptor && descriptor.type == jscape.AuthService.TYPE.PAM) {
            this.view.setDebugEnabled(descriptor.debugEnabled);
            this.view.setTemplate(descriptor.accountTemplate);
            this.view.setUsernameConversion(descriptor.usernameConversion == jscape.UsernameConversion.NONE ? null : descriptor.usernameConversion);
        }
    }
});

/** @class */
jscape.domain.PamAuthServiceView = jscape.domain.BaseAuthServiceView.extend(
    /** @lends {jscape.domain.PamAuthServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=pamauthfields]").hide();

        this.debug = $("input[name=debug]:checkbox", this.fieldset);

        this.useTemplate = $("input[name=usetemplate]:checkbox", this.fieldset);
        this.useTemplate.change($.proxy(function() {
            var enabled = this.useTemplate.is(":checked");
            this.template.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.template = $("select[name=template]", this.fieldset);
        this.template.combobox({
            required: true,
            editable: false,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.template.combobox("resize", 170).combobox("select", data[0].value);
                    this.useTemplate.prop("checked", true).change();
                } else {
                    this.useTemplate.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.useUsernameConversion = $("input[name=useusernameconversion]:checkbox", this.fieldset);
        this.useUsernameConversion.change($.proxy(function() {
            var enabled = this.useUsernameConversion.is(":checked");
            this.usernameConversion.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.usernameConversion = $("select[name=usernameconversion]", this.fieldset);
        this.usernameConversion.combobox({
            required: true,
            editable: false,
            width: 100,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.usernameConversion.combobox("clear").combobox("select", data[0].value);
                    this.useUsernameConversion.change();
                } else {
                    this.useUsernameConversion.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.testButton = $("a[role=button][name=test]", this.fieldset);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    isDebugEnabled: function() {
        return this.debug.is(":checked");
    },

    setDebugEnabled: function(value) {
        this.debug.prop("checked", !!value);
    },

    getTemplate: function() {
        return this.useTemplate.is(":checked") ? this.template.combobox("getValue") : null;
    },

    setTemplate: function(value) {
        this.template.combobox("select", value);
        this.useTemplate.prop("checked", value != null).change();
    },

    setTemplates: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.template.combobox("loadData", data);
    },

    getUsernameConversion: function() {
        return this.useUsernameConversion.is(":checked") ? this.usernameConversion.combobox("getValue") : null;
    },

    setUsernameConversion: function(value) {
        this.useUsernameConversion.prop("checked", value != null).change();
        if (value != null) {
            this.usernameConversion.combobox("select", value);
        }
    }
});

/** @class */
jscape.domain.CustomAuthServiceController = jscape.domain.BaseAuthServiceController.extend(
    /** @lends {jscape.domain.CustomAuthServiceController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
        this.view = new jscape.domain.CustomAuthServiceView(resources);
    },

    /** @override */
    _createDescriptor: function () {
        return jscape.AuthService.custom(this.view.getClassName(),
                                         this.view.getTemplate(),
                                         jscape.UsernameConversion.valueOf(this.view.getUsernameConversion()));
    },

    /** @override */
    _setupView: function (descriptor) {
        this.view.setTemplates(this.templates);

        if (descriptor && descriptor.type == jscape.AuthService.TYPE.CUSTOM) {
            this.view.setClassName(descriptor.className);
            this.view.setTemplate(descriptor.accountTemplate);
            this.view.setUsernameConversion(descriptor.usernameConversion == jscape.UsernameConversion.NONE ? null : descriptor.usernameConversion);
        }
    }
});

/** @class */
jscape.domain.CustomAuthServiceView = jscape.domain.BaseAuthServiceView.extend(
    /** @lends {jscape.domain.CustomAuthServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=customauthfields]").hide();

        this.className = $("input[name=classname]", this.fieldset);
        this.className.validatebox({required: true, validType: "length[1,255]", missingMessage: resources.getString("AUTH_ERROR_BAD_CLASS"), invalidMessage: resources.getString("AUTH_ERROR_BAD_CLASS")});

        this.useTemplate = $("input[name=usetemplate]:checkbox", this.fieldset);
        this.useTemplate.change($.proxy(function() {
            var enabled = this.useTemplate.is(":checked");
            this.template.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.template = $("select[name=template]", this.fieldset);
        this.template.combobox({
            required: true,
            editable: false,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.template.combobox("resize", 170).combobox("select", data[0].value);
                    this.useTemplate.prop("checked", true).change();
                } else {
                    this.useTemplate.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.useUsernameConversion = $("input[name=useusernameconversion]:checkbox", this.fieldset);
        this.useUsernameConversion.change($.proxy(function() {
            var enabled = this.useUsernameConversion.is(":checked");
            this.usernameConversion.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.usernameConversion = $("select[name=usernameconversion]", this.fieldset);
        this.usernameConversion.combobox({
            required: true,
            editable: false,
            width: 100,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.usernameConversion.combobox("clear").combobox("select", data[0].value);
                    this.useUsernameConversion.change();
                } else {
                    this.useUsernameConversion.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.testButton = $("a[role=button][name=test]", this.fieldset);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getClassName: function() {
        return this.className.val();
    },

    setClassName: function(value) {
        this.className.val(value);
    },

    getTemplate: function() {
        return this.useTemplate.is(":checked") ? this.template.combobox("getValue") : null;
    },

    setTemplate: function(value) {
        this.template.combobox("select", value);
        this.useTemplate.prop("checked", value != null).change();
    },

    setTemplates: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.template.combobox("loadData", data);
    },

    getUsernameConversion: function() {
        return this.useUsernameConversion.is(":checked") ? this.usernameConversion.combobox("getValue") : null;
    },

    setUsernameConversion: function(value) {
        this.useUsernameConversion.prop("checked", value != null).change();
        if (value != null) {
            this.usernameConversion.combobox("select", value);
        }
    }
});

/** @class */
jscape.domain.TestAuthServiceController = Class.extend(
    /** @lends {jscape.domain.TestAuthServiceController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, descriptor) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.descriptor = descriptor;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.TestAuthCredentialsDialog(this.resources);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var username = dialog.getUsername();
        var password = dialog.getPassword();
        var clientIp = dialog.getClientIp();
        dialog.destroy();

        this.managementService.testAuthenticationService(this.domainName, this.descriptor, username, password, clientIp)
                .done($.proxy(this._testPassed, this))
                .fail(this.deferred.reject);
    },

    onCancel: function (dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _testPassed: function () {
        $.messager.alert(this.resources.getString("AUTH_ERROR_TEST_TITLE"),
                         this.resources.getString("AUTH_ERROR_TEST_SUCCESS_MESSAGE"),
                         "info");
        this.deferred.resolve();
    }
});

/** @class */
jscape.domain.TestAuthCredentialsDialog = Class.extend(
    /** @lends {jscape.domain.TestAuthCredentialsDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-auth-credentials").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=testauthcredentialsfields]", this.dialog);

        this.username = $("input[name=username]", this.fieldset);
        this.password = $("input[name=password]", this.fieldset);
        this.clientIp = $("input[name=clientip]", this.fieldset);
    },

    getUsername: function() {
        return this.username.val();
    },

    getPassword: function() {
        return this.password.val();
    },

    getClientIp: function() {
        return this.clientIp.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});



/** @class */
jscape.domain.TwoFactorAuthServiceController = Class.extend(
    /** @lends {jscape.domain.TwoFactorAuthServiceController.prototype} */
{
    descriptor: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.view = new jscape.domain.TwoFactorAuthServiceServiceView(resources);
        this.controllers = { // lazy controller creation
            NONE: jscape.domain.NoneAuthServiceController,
            PHONE: jscape.domain.PhoneFactorServiceController
        };
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.descriptor && !Object._equals(this.descriptor, this._createDescriptor());
    },

    onApply: function () {
        var descriptor = this._createDescriptor();
        if (descriptor == null) {
            return $.Deferred().reject().promise();
        }

        return this.managementService.setPhoneFactorService(this.domainName, descriptor)
                .done($.proxy(function () {
                    this.descriptor = descriptor;
                    this._fireApplySuccess();
                }, this));
    },

    onDiscard: function () {
        $.each(this.controllers, function (_, controller) {
            if ($.type(controller) === "object") {
                controller.discardChanges();
            }
        });
        this._setupView();
    },

    onChangeType: function () {
        var serviceType = (this.view.getType() || "").toUpperCase();
        $.each(this.controllers, $.proxy(function(type, controller) {
            var t = $.type(controller);
            if (serviceType === type) {
                if (t === "function") {
                    this.controllers[type] = new controller(this.resources);
                    this.controllers[type].start(this.managementService, this.domainName, this.descriptor);
                } else {
                    controller.resume();
                }
            } else {
                if (t === "object") {
                    controller.pause();
                }
            }
        }, this));
    },

    _loadData: function () {
        this.managementService.getPhoneFactorService(this.domainName)
                .done($.proxy(function (descriptor) {
                    this.descriptor = descriptor;
                }, this))
                .always($.proxy(this._setupView, this));
    },

    _createDescriptor: function() {
        var controller = this.controllers[(this.view.getType() || "").toUpperCase()];
        return $.type(controller) === "object" ? controller.applyChanges() : null;
    },

    _setupView: function () {
        this.view.setType(this.descriptor == jscape.AuthService.NONE ? jscape.AuthService.TYPE.NONE : jscape.AuthService.TYPE.PHONE);
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.TwoFactorAuthServiceServiceView = Class.extend(
    /** @lends {jscape.domain.TwoFactorAuthServiceServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=twofactorauthfields]");

        this.type = $("select[name=type]", this.fieldset);
        this.type.combobox({required: true, editable: false, panelHeight: "auto", onChange: $.proxy(this._onChange$Type, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.TwoFactorAuthServiceServiceView.LISTENER;
    },

    getType: function() {
        return this.type.combobox("getValue");
    },

    setType: function(value) {
        var selectedType = this.type.combobox("getValue");
        this.type.combobox("clear");
        this.type.combobox("select", value);
        if (selectedType === value) {
            this._onChange$Type();
        }
    },

    _onChange$Type: function() {
        this.listener.onChangeType(this);
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard(this);
    }
});
jscape.domain.TwoFactorAuthServiceServiceView.LISTENER = {
    onChangeType: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.PhoneFactorServiceController = jscape.domain.BaseAuthServiceController.extend(
    /** @lends {jscape.domain.PhoneFactorServiceController.prototype} */
{
    VARIABLES: ["installdir"],

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
        this.view = new jscape.domain.PhoneFactorServiceView(resources);
        this.variableController = new jscape.VariableController();
    },

    onAddVariable: function(input) {
        this.variableController.start(input, this.VARIABLES);
    },

    onSelectPath: function() {
        var loadFn = $.proxy(this.managementService.listFileSystem, this.managementService);
        var createFn = $.proxy(this.managementService.createFileSystemDir, this.managementService);
        new SelectPathController(loadFn, createFn).selectPath($.proxy(function(entry) { this.view.setDirectory(entry.path); }, this));
    },

    /** @override */
    _createDescriptor: function () {
        return new jscape.PhoneFactorService(this.view.getDirectory(), this.view.getPassword(), this.view.isInternationalCallsAllowed());
    },

    /** @override */
    _setupView: function (descriptor) {
        if (descriptor && descriptor.licenseDirectory) {
            this.view.setDirectory(descriptor.licenseDirectory);
            this.view.setPassword(descriptor.password);
            this.view.setInternationalCallsAllowed(descriptor.internationalCallsAllowed);
        }
    }
});

/** @class */
jscape.domain.PhoneFactorServiceView = jscape.domain.BaseAuthServiceView.extend(
    /** @lends {jscape.domain.PhoneFactorServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=phonefactorauthfields]").hide();

        this.directory = $("input[name=dir]", this.fieldset);
        this.directory.validatebox({required: true, validType: "length[1,500]", missingMessage: resources.getString("AUTH_ERROR_BAD_PHONE_DIR"), invalidMessage: resources.getString("AUTH_ERROR_BAD_PHONE_DIR"), tipPosition: "bottom"});
        this.directory.focus($.proxy(function() { this.addVariable.linkbutton("enable"); }, this));

        this.password = $("input[name=password]", this.fieldset);
        this.allowInternationalCall = $("input[name=allowinterncall]:checkbox", this.fieldset);

        this.browseDir = $("a[role=button][name=browsedir]", this.fieldset);
        this.browseDir.linkbutton({onClick: $.proxy(this._onClick$BrowseDirectory, this)});

        this.addVariable = $("a[role=button][name=addvar]", this.fieldset);
        this.addVariable.linkbutton({disabled: true, onClick: $.proxy(this._onClick$AddVariableButton, this)});
    },

    getDirectory: function() {
        return this.directory.val();
    },

    setDirectory: function(value) {
        this.directory.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    isInternationalCallsAllowed: function() {
        return this.allowInternationalCall.is(":checked");
    },

    setInternationalCallsAllowed: function(value) {
        this.allowInternationalCall.prop("checked", !!value);
    },

    _onClick$BrowseDirectory: function() {
        this.listener.onSelectPath(this);
    },

    _onClick$AddVariableButton: function() {
        this.addVariable.linkbutton("disable");
        this.listener.onAddVariable(this.directory);
    }
});

/** @class */
jscape.domain.WebSsoAuthServiceController = Class.extend(
    /** @lends {jscape.WebSsoAuthServiceController.prototype} */
{
    templates: null,
    descriptor: null,

    /** @constructor */
    initialize: function(resources) {
        this.templates = [];
        this.resources = resources;

        this.view = new jscape.domain.WebSsoAuthServiceView(resources);
        this.controllers = {
            NONE: jscape.domain.NoneAuthServiceController,
            OPEN_ID: jscape.domain.OpenIdAuthServiceController,
            SAML: jscape.domain.SamlAuthServiceController
        };
    },

    hasChanged: function() {
        return this.descriptor && !Object._equals(this.descriptor, this._createDescriptor());
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onApply: function () {
        var descriptor = this._createDescriptor();
        if (descriptor == null) {
            return $.Deferred().reject().promise();
        }

        return this.managementService.setWebSsoService(this.domainName, descriptor)
                .done($.proxy(function () {
                    this.descriptor = descriptor;
                    this._fireApplySuccess();
                }, this));
    },

    onDiscard: function () {
        $.each(this.controllers, function(_, controller) {
            if ($.type(controller) === "object") {
                controller.discardChanges();
            }
        });
        this._setupView();
    },

    onChangeType: function () {
        var serviceType = (this.view.getType() || "").toUpperCase();
        $.each(this.controllers, $.proxy(function(type, controller) {
            var t = $.type(controller);
            if (serviceType === type) {
                if (t === "function") {
                    this.controllers[type] = new controller(this.resources);
                    this.controllers[type].start(this.managementService, this.domainName, this.descriptor, this.templates);
                } else {
                    controller.resume(this.descriptor, this.templates);
                }
            } else {
                if (t === "object") {
                    controller.pause();
                }
            }
        }, this));
    },

    _loadData: function () {
        $.when(this.managementService.getWebSsoService(this.domainName),
               this.managementService.getTemplates(this.domainName))
                .done($.proxy(function (descriptor, templates) {
                    this.descriptor = descriptor;
                    this.templates = (templates || []).sort();
                }, this))
                .fail($.proxy(function() { this.descriptor = jscape.AuthService.NONE; }, this))
                .always($.proxy(this._setupView, this));
    },

    _createDescriptor: function() {
        var controller = this.controllers[(this.view.getType() || "").toUpperCase()];
        return $.type(controller) === "object" ? controller.applyChanges() : null;
    },

    _setupView: function () {
        this.view.setType((this.descriptor && this.descriptor.type) || jscape.AuthService.TYPE.NONE);
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.WebSsoAuthServiceView = Class.extend(
    /** @lends {jscape.domain.WebSsoAuthServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=webssoauthfields]");

        this.type = $("select[name=type]", this.fieldset);
        this.type.combobox({required: true, editable: false, panelHeight: "auto", onChange: $.proxy(this._onChange$Type, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.WebSsoAuthServiceView.LISTENER;
    },

    getType: function() {
        return this.type.combobox("getValue");
    },

    setType: function(value) {
        var selectedType = this.type.combobox("getValue");
        this.type.combobox("clear");
        this.type.combobox("select", value);
        if (selectedType === value) {
            this._onChange$Type();
        }
    },

    _onChange$Type: function() {
        this.listener.onChangeType(this);
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard(this);
    }
});
jscape.domain.WebSsoAuthServiceView.LISTENER = {
    onChangeType: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.SamlAuthServiceController = jscape.domain.BaseAuthServiceController.extend(
    /** @lends {jscape.domain.SamlAuthServiceController.prototype} */
{
    hostKeys: [],

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
        this.view = new jscape.domain.SamlAuthServiceView(resources);
    },

    /** @override */
    start: function (managementService, domainName, descriptor, templates) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.descriptor = descriptor;
        this.templates = templates;

        this._loadHostKeys().always($.proxy(this._setupView, this, descriptor), this);

        this.view.show(this);
    },

    _loadHostKeys: function() {
        return this.managementService.getHostCertificates().done($.proxy(function(keys) {
            this.hostKeys = $.map(keys || [], function(key) { return key.alias; });
        }, this));
    },

    /** @override */
    _createDescriptor: function () {
        return jscape.WebSsoService.saml(this.view.getSignInUrl(),
                                         this.view.getSignOutUrl(),
                                         this.view.getChangePasswordUrl(),
                                         this.view.getHostKey(),
                                         this.view.isNonSsoLoginAllowed(),
                                         this.view.getTemplate(),
                                         jscape.UsernameConversion.valueOf(this.view.getUsernameConversion()));
    },

    /** @override */
    _setupView: function (descriptor) {
        this.view.setTemplates(this.templates);
        this.view.setHostKeys(this.hostKeys);

        if (descriptor && descriptor.type == jscape.WebSsoService.TYPE.SAML) {
            this.view.setSignInUrl(descriptor.signInUrl);
            this.view.setSignOutUrl(descriptor.signOutUrl);
            this.view.setChangePasswordUrl(descriptor.changePasswordUrl);
            this.view.setHostKey(descriptor.verificationHostKey);
            this.view.setNonSsoLoginAllowed(descriptor.nonSsoLoginAllowed);
            this.view.setTemplate(descriptor.accountTemplate);
            this.view.setUsernameConversion(descriptor.usernameConversion == jscape.UsernameConversion.NONE ? null : descriptor.usernameConversion);
        }
    }
});

/** @class */
jscape.domain.SamlAuthServiceView = jscape.domain.BaseAuthServiceView.extend(
    /** @lends {jscape.domain.SamlAuthServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=webssoauthsamlfields]").hide();

        this.signInUrl = $("input[name=signinurl]", this.fieldset);
        this.signInUrl.validatebox({required: true, validType: "length[1,500]", missingMessage: resources.getString("AUTH_ERROR_BAD_SIGNIN_URL"), invalidMessage: resources.getString("AUTH_ERROR_BAD_SIGNIN_URL")});

        this.signOutUrl = $("input[name=signouturl]", this.fieldset);
        this.signOutUrl.validatebox({required: true, validType: "length[1,500]", missingMessage: resources.getString("AUTH_ERROR_BAD_SIGNOUT_URL"), invalidMessage: resources.getString("AUTH_ERROR_BAD_SIGNOUT_URL")});

        this.changePasswordUrl = $("input[name=changepwdurl]", this.fieldset);
        this.nonSsoLoginAllowed = $("input[name=allownonsso]:checkbox", this.fieldset);

        this.hostKey = $("select[name=hostkey]", this.fieldset);
        this.hostKey.combobox({
            required: true,
            editable: false,
            width: 170,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.hostKey.combobox("select", data[0].value);
                } else {
                    this.hostKey.combobox("disable").combobox("disableValidation");
                }
            }, this)
        });

        this.useTemplate = $("input[name=usetemplate]:checkbox", this.fieldset);
        this.useTemplate.change($.proxy(function() {
            var enabled = this.useTemplate.is(":checked");
            this.template.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.template = $("select[name=template]", this.fieldset);
        this.template.combobox({
            required: true,
            editable: false,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function (data) {
                if (data.length) {
                    this.template.combobox("resize", 170).combobox("select", data[0].value);
                    this.useTemplate.prop("checked", true).change();
                } else {
                    this.useTemplate.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.useUsernameConversion = $("input[name=useusernameconversion]:checkbox", this.fieldset);
        this.useUsernameConversion.change($.proxy(function() {
            var enabled = this.useUsernameConversion.is(":checked");
            this.usernameConversion.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.usernameConversion = $("select[name=usernameconversion]", this.fieldset);
        this.usernameConversion.combobox({
            required: true,
            editable: false,
            width: 100,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.usernameConversion.combobox("clear").combobox("select", data[0].value);
                    this.useUsernameConversion.change();
                } else {
                    this.useUsernameConversion.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });
    },

    getSignInUrl: function() {
        return this.signInUrl.val();
    },

    setSignInUrl: function(value) {
        this.signInUrl.val(value).validatebox("validate");
    },

    getSignOutUrl: function() {
        return this.signOutUrl.val();
    },

    setSignOutUrl: function(value) {
        this.signOutUrl.val(value).validatebox("validate");
    },

    getChangePasswordUrl: function() {
        return this.changePasswordUrl.val();
    },

    setChangePasswordUrl: function(value) {
        this.changePasswordUrl.val(value);
    },

    getHostKey: function() {
        return this.hostKey.combobox("getValue");
    },

    setHostKey: function(value) {
        this.hostKey.combobox("setValue", value);
    },

    setHostKeys: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.hostKey.combobox("loadData", data);
    },

    isNonSsoLoginAllowed: function() {
        return this.nonSsoLoginAllowed.is(":checked");
    },

    setNonSsoLoginAllowed: function(value) {
        this.nonSsoLoginAllowed.prop("checked", !!value).change();
    },

    getTemplate: function() {
        return this.useTemplate.is(":checked") ? this.template.combobox("getValue") : null;
    },

    setTemplate: function(value) {
        this.template.combobox("select", value);
        this.useTemplate.prop("checked", value != null).change();
    },

    setTemplates: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.template.combobox("loadData", data);
    },

    getUsernameConversion: function() {
        return this.useUsernameConversion.is(":checked") ? this.usernameConversion.combobox("getValue") : null;
    },

    setUsernameConversion: function(value) {
        this.useUsernameConversion.prop("checked", value != null).change();
        if (value != null) {
            this.usernameConversion.combobox("select", value);
        }
    }
});

/** @class */
jscape.domain.OpenIdAuthServiceController = jscape.domain.BaseAuthServiceController.extend(
    /** @lends {jscape.domain.OpenIdAuthServiceController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
        this.view = new jscape.domain.OpenIdAuthServiceView(resources);
    },

    /** @override */
    _createDescriptor: function () {
        return jscape.WebSsoService.openId(this.view.getSignInUrl(),
                                           this.view.getSignOutUrl(),
                                           this.view.isNonSsoLoginAllowed(),
                                           this.view.getTemplate(),
                                           jscape.UsernameConversion.valueOf(this.view.getUsernameConversion()));
    },

    /** @override */
    _setupView: function (descriptor) {
        this.view.setTemplates(this.templates);

        if (descriptor && descriptor.type == jscape.WebSsoService.TYPE.OPEN_ID) {
            this.view.setSignInUrl(descriptor.signInUrl);
            this.view.setSignOutUrl(descriptor.signOutUrl);
            this.view.setNonSsoLoginAllowed(descriptor.nonSsoLoginAllowed);
            this.view.setTemplate(descriptor.accountTemplate);
            this.view.setUsernameConversion(descriptor.usernameConversion == jscape.UsernameConversion.NONE ? null : descriptor.usernameConversion);
        }
    }
});

/** @class */
jscape.domain.OpenIdAuthServiceView = jscape.domain.BaseAuthServiceView.extend(
    /** @lends {jscape.domain.OpenIdAuthServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=webssoauthopenidfields]").hide();

        this.signInUrl = $("input[name=signinurl]", this.fieldset);
        this.signInUrl.validatebox({required: true, validType: "length[1,500]", missingMessage: resources.getString("AUTH_ERROR_BAD_SIGNIN_URL"), invalidMessage: resources.getString("AUTH_ERROR_BAD_SIGNIN_URL")});

        this.signOutUrl = $("input[name=signouturl]", this.fieldset);
        this.signOutUrl.validatebox({required: true, validType: "length[1,500]", missingMessage: resources.getString("AUTH_ERROR_BAD_SIGNOUT_URL"), invalidMessage: resources.getString("AUTH_ERROR_BAD_SIGNOUT_URL")});

        this.nonSsoLoginAllowed = $("input[name=allownonsso]:checkbox", this.fieldset);

        this.useTemplate = $("input[name=usetemplate]:checkbox", this.fieldset);
        this.useTemplate.change($.proxy(function() {
            var enabled = this.useTemplate.is(":checked");
            this.template.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.template = $("select[name=template]", this.fieldset);
        this.template.combobox({
            required: true,
            editable: false,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.template.combobox("resize", 170).combobox("select", data[0].value);
                    this.useTemplate.prop("checked", true).change();
                } else {
                    this.useTemplate.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.useUsernameConversion = $("input[name=useusernameconversion]:checkbox", this.fieldset);
        this.useUsernameConversion.change($.proxy(function() {
            var enabled = this.useUsernameConversion.is(":checked");
            this.usernameConversion.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.usernameConversion = $("select[name=usernameconversion]", this.fieldset);
        this.usernameConversion.combobox({
            required: true,
            editable: false,
            width: 100,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.usernameConversion.combobox("clear").combobox("select", data[0].value);
                    this.useUsernameConversion.change();
                } else {
                    this.useUsernameConversion.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });
    },

    getSignInUrl: function() {
        return this.signInUrl.val();
    },

    setSignInUrl: function(value) {
        this.signInUrl.val(value);
    },

    getSignOutUrl: function() {
        return this.signOutUrl.val();
    },

    setSignOutUrl: function(value) {
        this.signOutUrl.val(value);
    },

    isNonSsoLoginAllowed: function() {
        return this.nonSsoLoginAllowed.is(":checked");
    },

    setNonSsoLoginAllowed: function(value) {
        this.nonSsoLoginAllowed.prop("checked", !!value).change();
    },

    getTemplate: function() {
        return this.useTemplate.is(":checked") ? this.template.combobox("getValue") : null;
    },

    setTemplate: function(value) {
        this.template.combobox("select", value);
        this.useTemplate.prop("checked", value != null).change();
    },

    setTemplates: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.template.combobox("loadData", data);
    },

    getUsernameConversion: function() {
        return this.useUsernameConversion.is(":checked") ? this.usernameConversion.combobox("getValue") : null;
    },

    setUsernameConversion: function(value) {
        this.useUsernameConversion.prop("checked", value != null).change();
        if (value != null) {
            this.usernameConversion.combobox("select", value);
        }
    }
});