jscape.domain = jscape.domain || {};

/** @class */
jscape.BannedFiles = Class.extend(
    /** @lends {jscape.BannedFiles.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.enabled = a || false;
        this.rules = b || [];
    }
});
jscape.BannedFiles.fromJSON = function(data) {
    return new jscape.BannedFiles(data.enabled, $.map(data.rules, jscape.FileOperationRule.fromJSON));
};

/** @class */
jscape.FileOperationRule = Class.extend(
    /** @lends {jscape.FileOperationRule.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.path = a == "" ? "/" : a;
        this.pattern = b || "";
        this.recursive = c != undefined ? c : true;
        this.scopes = d || [];
    }
});
jscape.FileOperationRule.fromJSON = function(data) {
    var scopes = jscape.FileOperationRule.Scopes.fromJSON(data.scopes);
    return new jscape.FileOperationRule(data.path, data.pattern, !!data.recursive, scopes);
};
/** @class */
jscape.FileOperationRule.Scopes = Class.extend(
    /** @lends {jscape.FileOperationRule.Scopes.prototype} */
{
    /** @constructor */
    initialize: function(code, value) {
        this.code = code;
        this.value = value;
    },

    toJSON: function() {
        return this.value;
    }
});
jscape.FileOperationRule.Scopes.fromJSON = function(data) {
    function _equals(o1, o2) {
        if (o1.length != o2.length) { return false; }
        for (var i = 0; i < o1.length; i++) {
            if (o1[i] !== o2[i]) { return false; }
        }
        return true;
    }
    var ret = $.grep(jscape.FileOperationRule.Scopes.values(), function(scope) { return _equals(data, scope.value)});
    return ret.length ? ret[0] : jscape.FileOperationRule.Scopes.FILE;
};
jscape.FileOperationRule.Scopes.fromCode = function(code) {
    var ret = $.grep(jscape.FileOperationRule.Scopes.values(), function(scope) { return code == scope.code; });
    return ret.length ? ret[0] :  jscape.FileOperationRule.Scopes.FILE;
};
jscape.FileOperationRule.Scopes.values = function() {
    return [jscape.FileOperationRule.Scopes.FILE, jscape.FileOperationRule.Scopes.DIRECTORY, jscape.FileOperationRule.Scopes.FILE_AND_DIRECTORY];
};
jscape.FileOperationRule.Scopes.FILE = new jscape.FileOperationRule.Scopes("FILE", ["File"]);
jscape.FileOperationRule.Scopes.DIRECTORY = new jscape.FileOperationRule.Scopes("DIR", ["Directory"]);
jscape.FileOperationRule.Scopes.FILE_AND_DIRECTORY = new jscape.FileOperationRule.Scopes("FILE_AND_DIR", ["File", "Directory"]);

/** @class */
jscape.domain.BannedFilesScopeFormat = Class.extend(
    /** @lends {jscape.domain.BannedFilesScopeFormat.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    format: function (scope) {
        switch (scope) {
            case jscape.FileOperationRule.Scopes.FILE:
                return this.resources.getString("BANNED_FILES_SCOPE_FILE");
            case jscape.FileOperationRule.Scopes.DIRECTORY:
                return this.resources.getString("BANNED_FILES_SCOPE_DIRECTORY");
            case jscape.FileOperationRule.Scopes.FILE_AND_DIRECTORY:
                return this.resources.getString("BANNED_FILES_SCOPE_FILE_AND_DIRECTORY");
            default:
                return "";
        }
    }
});

/** @class */
jscape.domain.BannedFilesModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.BannedFilesModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.BannedFilesController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onPause: function() {
        if (this.controller.hasChanged()) {
            this._confirmApply()
                    .done($.proxy(function () { this.controller.onApply(); }, this));
        }
    },

    onResume: function() {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.BannedFilesController = Class.extend(
    /** @lends {jscape.domain.BannedFilesController.prototype} */
{
    descriptor: null,
    copy: null,
    editIndex: -1,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.BannedFilesView(resources);
        this.addController = new jscape.domain.AddBannedFilesController(resources);
        this.editController = new jscape.domain.EditBannedFilesController(resources);
        this.deleteController = new jscape.domain.DeleteBannedFilesController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.copy && !Object._equals(this.copy, this._createDescriptor());
    },

    onAddRule: function() {
        this.addController.start()
                .done($.proxy(this._ruleAdded, this));
    },

    onEditRule: function(rule) {
        var index = $.inArray(rule, this.descriptor.rules);
        if (index != -1) {
            this.editIndex = index;
            this.editController.start(rule)
                    .done($.proxy(this._ruleEdited, this, index))
                    .always($.proxy(function() { this.editIndex = -1 }, this));
        }
    },

    onDeleteRule: function(rules) {
        this.deleteController.start(rules)
                .done($.proxy(this._ruleDeleted, this, rules));
    },

    onApply: function () {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var descriptor = this._createDescriptor();
        return this.managementService.setBannedFiles(this.domainName, descriptor)
                .done($.proxy(function () {
                    this.descriptor = descriptor;
                    this.copy = this._copyDescriptor();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function () {
        this.descriptor = this.copy;
        this.copy = this._copyDescriptor();
        this._setupView();
    },

    _ruleAdded: function (rule) {
        this.descriptor.rules.push(rule);
        this.view.setRules(this.descriptor.rules);
        this.view.selectRule(rule);
    },

    _ruleEdited: function (index, rule) {
        this.descriptor.rules[index] = rule;
        this.view.setRules(this.descriptor.rules);
        this.view.selectRule(rule);
    },

    _ruleDeleted: function (rules) {
        var index = -1;
        while (rules.length) {
            index = $.inArray(rules.pop(), this.descriptor.rules);
            this.descriptor.rules.splice(index, 1);
        }
        this.view.setRules(this.descriptor.rules);

        if (this.descriptor.rules.length) {
            index = Math.max(0, Math.min(index, this.descriptor.rules.length - 1));
            this.view.selectRule(this.descriptor.rules[index]);
        }
    },

    _loadData: function() {
        this.managementService.getBannedFiles(this.domainName)
                .done($.proxy(function(descriptor) {
                    this.descriptor = descriptor;
                    this.copy = this._copyDescriptor();
                    this._setupView();
                }, this));
    },

    _createDescriptor: function () {
        var descriptor = this._copyDescriptor();
        descriptor.enabled = this.view.getEnabled();
        return descriptor;
    },

    _copyDescriptor: function() {
        var rules = $.map(this.descriptor.rules, function(rule) { return $.extend(new jscape.FileOperationRule(), rule); });
        return new jscape.BannedFiles(this.descriptor.enabled, rules);
    },

    _setupView: function() {
        this.view.setEnabled(this.descriptor.enabled);
        this.view.setRules(this.descriptor.rules);
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("bannedfiles_path", resources.getString("BANNED_FILES_ERROR_BAD_PATH"));
        v.rule("bannedfiles_pattern", $.proxy(this._patternValid, this), resources.getString("BANNED_FILES_ERROR_BAD_PATTERN"));
        v.rule("bannedfiles_unique", $.proxy(this._ruleNotExists, this), resources.getString("BANNED_FILES_ERROR_ALREADY_EXISTS"));
    },

    _patternValid: function(value) {
        var response = this.managementService.testBannedFilesPattern(this.domainName, value);
        return response && "true" === response.trim();
    },

    _ruleNotExists: function (_, params) {
        if (params.length > 1) {
            var path = $(params[0]).val();
            var pattern = $(params[1]).val();
            var excludeIndex = this.editIndex;
            return $.grep(this.descriptor.rules, function (rule, index) {
                return excludeIndex != index && rule.path == path && rule.pattern == pattern;
            }).length == 0;
        }
        return true;
    }
});

/** @class */
jscape.domain.BannedFilesView = Class.extend(
    /** @lends {jscape.domain.BannedFilesView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.selection = [];
        this.resources = resources;
        this.scopeFormat = new jscape.domain.BannedFilesScopeFormat(resources);

        this.fieldset = $("fieldset[name=bannedfilesfields]");

        this.enabled = $("[name=enabled]:checkbox", this.fieldset);
        this.enabled.on("change", $.proxy(this._adjustButtonsState, this));

        this.data = $("table[role=grid][aria-label=bannedfiles]", this.fieldset);
        this.data.datagrid({
            singleSelect: true,
            fitColumns: true,
            pagination: true,
            search: true,
            columns: [[
                {field: "path", title: $("thead th:nth-child(1)", this.data).text(), width: 100},
                {field: "recursive", title: $("thead th:nth-child(2)", this.data).text(), width: 40},
                {field: "pattern", title: $("thead th:nth-child(3)", this.data).text(), width: 120},
                {field: "scopes", title: $("thead th:nth-child(4)", this.data).text(), width: 80, formatter: $.proxy(this._formatScope, this)}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onClickRow: $.proxy(function(index) { if (!this.getEnabled()) { this.data.datagrid("unselectRow", index); }}, this),
            onDblClickRow: $.proxy(function() { if (this.getEnabled()) { this._onClick$EditButton(); }}, this)
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getEnabled: function() {
        return this.enabled.is(":checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", value);
    },

    setRules: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectRule: function(rule) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", rule));
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.BannedFilesView.LISTENER;
        this._adjustButtonsState();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _formatScope: function(value) {
        return this.scopeFormat.format(value);
    },

    _onClick$AddButton: function() {
        this.listener.onAddRule();
    },

    _onClick$EditButton: function() {
        this.listener.onEditRule(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteRule(this.selection);
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard();
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        this.addButton.linkbutton(this.getEnabled() ? "enable" : "disable");
        this.editButton.linkbutton(this.getEnabled() && this.selection.length == 1 ? "enable" : "disable");
        this.deleteButton.linkbutton(this.getEnabled() && this.selection.length != 0 ? "enable" : "disable");
    }
});
jscape.domain.BannedFilesView.LISTENER = {
    onAddRule: $.noop,
    onEditRule: $.noop,
    onDeleteRule: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.BannedFilesRuleController = Class.extend(
    /** @lends {jscape.domain.BannedFilesRuleController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function() {
        this.deferred = $.Deferred();
        this._setupView();
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var rule = this._createRule(dialog);
        dialog.destroy();
        this.deferred.resolve(rule);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _setupView: function() {
    },

    _createRule: function(dialog) {
        return new jscape.FileOperationRule(dialog.getPath(), dialog.getPattern(), dialog.getRecursive(), dialog.getScope());
    }
});

/** @class */
jscape.domain.AddBannedFilesController = jscape.domain.BannedFilesRuleController.extend(
    /** @lends {jscape.domain.AddBannedFilesController.prototype} */
{
    /** @override */
    _setupView: function() {
        var dialog = new jscape.domain.BannedFilesRuleDialog(this.resources);
        dialog.setTitle(this.resources.getString("BANNED_FILES_DIALOG_ADD_TITLE"));
        dialog.setPath("/");
        dialog.setRecursive(true);
        dialog.setScopes(jscape.FileOperationRule.Scopes.values());
        dialog.setScope(jscape.FileOperationRule.Scopes.FILE);
        dialog.show(this);
    }
});

/** @class */
jscape.domain.EditBannedFilesController = jscape.domain.BannedFilesRuleController.extend(
    /** @lends {jscape.domain.EditBannedFilesController.prototype} */
{
    /** @override */
    start: function(rule) {
        this.rule = rule;
        return this._super();
    },

    /** @override */
    _setupView: function () {
        var dialog = new jscape.domain.BannedFilesRuleDialog(this.resources);
        dialog.setTitle(this.resources.getString("BANNED_FILES_DIALOG_EDIT_TITLE"));
        dialog.setPath(this.rule.path);
        dialog.setPattern(this.rule.pattern);
        dialog.setRecursive(this.rule.recursive);
        dialog.setScopes(jscape.FileOperationRule.Scopes.values());
        dialog.setScope(this.rule.scopes);
        dialog.show(this);
    }
});

/** @class */
jscape.domain.BannedFilesRuleDialog = Class.extend(
    /** @lends {jscape.domain.BannedFilesRuleDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.dialog = $("#d-bannedfiles-rule").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=bannedfilesrulefields]", this.dialog);

        var pathId = "bf_path_" + new Date().getTime().toString(36);
        var patternId = "bf_pattern_" + new Date().getTime().toString(36);
        this.path = $("input[name=path]", this.fieldset).attr("id", pathId);
        this.pattern = $("input[name=pattern]", this.fieldset).attr("id", patternId);
        this.path.validatebox({
            required: true,
            validType: ["bannedfiles_path", "bannedfiles_unique['#{0}','#{1}']".supplant({0: pathId, 1: patternId})],
            missingMessage: resources.getString("BANNED_FILES_ERROR_BAD_PATH")
        });
        this.pattern.validatebox({
            required: true,
            validType: ["bannedfiles_pattern", "bannedfiles_unique['#{0}','#{1}']".supplant({0: pathId, 1: patternId})],
            missingMessage: resources.getString("BANNED_FILES_ERROR_BAD_PATTERN")
        });

        this.recursive = $("input[name=recursive]:checkbox", this.fieldset);
        this.scope = $("select[name=scope]", this.fieldset);
        this.scope.combobox({required: true, editable: false, width: 120, panelHeight: "auto"});
    },

    setTitle: function(value) {
        this.dialog.dialog("setTitle", value);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this.fieldset.form("validate");
        this.path.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    getPath: function() {
        return this.path.val();
    },

    setPath: function(value) {
        this.path.val(value);
    },

    getPattern: function() {
        return this.pattern.val();
    },

    setPattern: function(value) {
        this.pattern.val(value);
    },

    getRecursive: function() {
        return this.recursive.is(":checked");
    },

    setRecursive: function(value) {
        this.recursive.prop("checked", !!value);
    },

    setScopes: function(values) {
        var format = new jscape.domain.BannedFilesScopeFormat(this.resources);
        var data = new Array(values.length);
        for (var i = 0; i < values.length; i++) {
            data[i] = {text: format.format(values[i]), value: values[i].code};
        }
        this.scope.combobox("loadData", data);
    },

    getScope: function() {
        return jscape.FileOperationRule.Scopes.fromCode(this.scope.combobox("getValue"));
    },

    setScope: function(value) {
        if (value && value.code) {
            this.scope.combobox("select", value.code);
        }
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.DeleteBannedFilesController = Class.extend(
    /** @lends {jscape.domain.DeleteBannedFilesController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(rules) {
        var text = $.map(rules, function(rule) { return rule.pattern; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("BANNED_FILES_CONFIRM_DELETE_TITLE"),
                this.resources.getString("BANNED_FILES_CONFIRM_DELETE_MESSAGE", {0: text}));
    }
});