jscape.domain = jscape.domain || {};

/** @class */
jscape.PasswordCompliance = Class.extend(
    /** @lends {jscape.PasswordCompliance.prototype} */
{
    /** @constructor */
    initialize: function (a, b, c, d, e, f, g, h, i, j) {
        this.minPasswordLength = parseInt(a) || null;
        this.maxPasswordAge = parseInt(b) || null;
        this.passwordExpirationNotificationDays = parseInt(c) || null;
        this.equalPasswordsHistoryLength = parseInt(d) || null;
        this.uppercaseRequired = e || false;
        this.lowercaseRequired = f || false;
        this.numericRequired = g || false;
        this.nonAlphaNumericRequired = h || false;
        this.nonCompliantPasswordLoginDenied = i || false;
        this.firstTimeLoginResetPasswordRequired = j || false;
    }
});
jscape.PasswordCompliance.fromJSON = function(data) {
    return $.extend(new jscape.PasswordCompliance(), data);
};

/** @class */
jscape.domain.ComplianceModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.ComplianceModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.ComplianceController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onPause: function() {
        if (this.controller.hasChanged()) {
            this._confirmApply()
                    .done($.proxy(function () { this.controller.onApply(); }, this));
        }
    },

    onResume: function() {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.ComplianceController = Class.extend(
    /** @lends {jscape.domain.ComplianceController.prototype} */
{
    descriptor: null,
    copy: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.ComplianceView(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.copy && !Object._equals(this.copy, this._createDescriptor());
    },

    onApply: function () {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var descriptor = this._createDescriptor();
        return this.managementService.setPasswordCompliance(this.domainName, descriptor)
                .done($.proxy(function () {
                    this.descriptor = descriptor;
                    this.copy = this._copyDescriptor();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.descriptor = this.copy;
        this.copy = this._copyDescriptor();
        this._setupView();
    },

    _loadData: function() {
        this.managementService.getPasswordCompliance(this.domainName).done($.proxy(function(descriptor) {
            this.descriptor = descriptor;
            this.copy = this._copyDescriptor();
            this._setupView();
        }, this));
    },

    _copyDescriptor: function() {
        return $.extend(new jscape.PasswordCompliance(), this.descriptor);
    },

    _createDescriptor: function () {
        return new jscape.PasswordCompliance(this.view.getMinPasswordLength(),
                                             this.view.getMaxPasswordAge(),
                                             this.view.getPasswordExpirationNotificationDays(),
                                             this.view.getEqualPasswordsHistoryLength(),
                                             this.view.getUppercaseRequired(),
                                             this.view.getLowercaseRequired(),
                                             this.view.getNumericRequired(),
                                             this.view.getNonAlphaNumericRequired(),
                                             this.view.getNonCompliantPasswordDenied(),
                                             this.view.getFirstTimeLoginResetPasswordRequired());
    },

    _setupView: function () {
        this.view.setMinPasswordLength(this.descriptor.minPasswordLength);
        this.view.setMaxPasswordAge(this.descriptor.maxPasswordAge);
        this.view.setPasswordExpirationNotificationDays(this.descriptor.passwordExpirationNotificationDays);
        this.view.setEqualPasswordsHistoryLength(this.descriptor.equalPasswordsHistoryLength);
        this.view.setUppercaseRequired(this.descriptor.uppercaseRequired);
        this.view.setLowercaseRequired(this.descriptor.lowercaseRequired);
        this.view.setNumericRequired(this.descriptor.numericRequired);
        this.view.setNonAlphaNumericRequired(this.descriptor.nonAlphaNumericRequired);
        this.view.setNonCompliantPasswordDenied(this.descriptor.nonCompliantPasswordLoginDenied);
        this.view.setFirstTimeLoginResetPasswordRequired(this.descriptor.firstTimeLoginResetPasswordRequired);
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.ComplianceView = Class.extend(
    /** @lends {jscape.domain.ComplianceView.prototype} */
{
    /** @const */
    DEFAULT_PASSWORD_LENGTH: 8,
    DEFAULT_PASSWORD_AGE: 90,
    DEFAULT_REMINDER_DAYS: 10,
    DEFAULT_HISTORY_LENGTH: 3,
    TIME_FACTOR: 24 * 60 * 60 * 1000,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=passwordcompliancefields]");

        this.passwordLengthEnabled = $("input[name=pwdlengthenabled]:checkbox", this.fieldset);
        this.passwordLengthEnabled.change($.proxy(function() { this._adjustSpinner(this.passwordLengthEnabled, this.passwordLength); }, this));
        this.passwordLength = $("input[name=pwdlength]", this.fieldset);
        this.passwordLength.numberspinner({required: true, disabled: true, min: 1, max: 100, value: this.DEFAULT_PASSWORD_LENGTH, width: 50});

        this.passwordAgeEnabled = $("input[name=pwdageenabled]:checkbox", this.fieldset);
        this.passwordAgeEnabled.change($.proxy(function() { this._adjustSpinner(this.passwordAgeEnabled, this.passwordAge); }, this));
        this.passwordAge = $("input[name=pwdage]", this.fieldset);
        this.passwordAge.numberspinner({required: true, disabled: true, min: 1, max: 999, value: this.DEFAULT_PASSWORD_AGE, width: 50});

        this.passwordReminderEnabled = $("input[name=pwdreminderenabled]:checkbox", this.fieldset);
        this.passwordReminderEnabled.change($.proxy(function() { this._adjustSpinner(this.passwordReminderEnabled, this.passwordReminder); }, this));
        this.passwordReminder = $("input[name=pwdreminder]", this.fieldset);
        this.passwordReminder.numberspinner({required: true, disabled: true, min: 1, max: 100, value: this.DEFAULT_REMINDER_DAYS, width: 50});

        this.passwordHistoryEnabled = $("input[name=pwdhistoryenabled]:checkbox", this.fieldset);
        this.passwordHistoryEnabled.change($.proxy(function() { this._adjustSpinner(this.passwordHistoryEnabled, this.passwordHistory); }, this));
        this.passwordHistory = $("input[name=pwdhistory]", this.fieldset);
        this.passwordHistory.numberspinner({required: true, disabled: true, min: 1, max: 100, value: this.DEFAULT_HISTORY_LENGTH, width: 50});

        this.firstTimeLoginResetPassword = $("input[name=ftlresetpwd]:checkbox", this.fieldset);
        this.uppercase = $("input[name=uppercase]:checkbox", this.fieldset);
        this.lowercase = $("input[name=lowercase]:checkbox", this.fieldset);
        this.numeric = $("input[name=numeric]:checkbox", this.fieldset);
        this.nonAlphaNumeric = $("input[name=nonalphanumeric]:checkbox", this.fieldset);
        this.denyLogin = $("input[name=denylogin]:checkbox", this.fieldset);

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getMinPasswordLength: function() {
        return this.passwordLengthEnabled.is(":checked") ? this.passwordLength.numberspinner("getValue") : null;
    },

    setMinPasswordLength: function(value) {
        this.passwordLengthEnabled.prop("checked", value != null);
        if (value != null) {
            this.passwordLength.numberspinner("setValue", value);
        }
        this._adjustSpinner(this.passwordLengthEnabled, this.passwordLength);
    },
    getMaxPasswordAge: function() {
        return this.passwordAgeEnabled.is(":checked") ? this.passwordAge.numberspinner("getValue") * this.TIME_FACTOR : null;
    },

    setMaxPasswordAge: function(value) {
        this.passwordAgeEnabled.prop("checked", value != null);
        if (value != null) {
            this.passwordAge.numberspinner("setValue", value / this.TIME_FACTOR);
        }
        this._adjustSpinner(this.passwordAgeEnabled, this.passwordAge);
    },

    getPasswordExpirationNotificationDays: function() {
        return this.passwordReminderEnabled.is(":checked") ? this.passwordReminder.numberspinner("getValue") : null;
    },

    setPasswordExpirationNotificationDays: function(value) {
        this.passwordReminderEnabled.prop("checked", value != null);
        if (value != null) {
            this.passwordReminder.numberspinner("setValue", value);
        }
        this._adjustSpinner(this.passwordReminderEnabled, this.passwordReminder);
    },

    getEqualPasswordsHistoryLength: function() {
        return this.passwordHistoryEnabled.is(":checked") ? this.passwordHistory.numberspinner("getValue") : null;
    },

    setEqualPasswordsHistoryLength: function(value) {
        this.passwordHistoryEnabled.prop("checked", value != null);
        if (value != null) {
            this.passwordHistory.numberspinner("setValue", value);
        }
        this._adjustSpinner(this.passwordHistoryEnabled, this.passwordHistory);
    },

    getUppercaseRequired: function() {
        return this.uppercase.is(":checked");
    },

    setUppercaseRequired: function(value) {
        this.uppercase.prop("checked", !!value);
    },

    getLowercaseRequired: function() {
        return this.lowercase.is(":checked");
    },

    setLowercaseRequired: function(value) {
        this.lowercase.prop("checked", !!value);
    },

    getNumericRequired: function() {
        return this.numeric.is(":checked");
    },

    setNumericRequired: function(value) {
        this.numeric.prop("checked", !!value);
    },

    getNonAlphaNumericRequired: function() {
        return this.nonAlphaNumeric.is(":checked");
    },

    setNonAlphaNumericRequired: function(value) {
        this.nonAlphaNumeric.prop("checked", !!value);
    },

    getNonCompliantPasswordDenied: function() {
        return this.denyLogin.is(":checked");
    },

    setNonCompliantPasswordDenied: function(value) {
        this.denyLogin.prop("checked", !!value);
    },

    getFirstTimeLoginResetPasswordRequired: function() {
        return this.firstTimeLoginResetPassword.is(":checked");
    },

    setFirstTimeLoginResetPasswordRequired: function(value) {
        this.firstTimeLoginResetPassword.prop("checked", !!value);
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.ComplianceView.LISTENER;
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$ApplyButton: function () {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function () {
        this.listener.onDiscard();
    },

    _adjustSpinner: function(checkbox, spinner) {
        var enabled = checkbox.is(":checked");
        spinner.numberspinner(enabled ? "enable" : "disable").validatebox(enabled ? "enableValidation" : "disableValidation");
    }
});
jscape.domain.ComplianceView.LISTENER = {
    onApply: $.noop,
    onDiscard: $.noop
};