jscape.domain = jscape.domain || {};

/** @class */
jscape.DomainRestrictions = Class.extend(
    /** @lends {jscape.DomainRestrictions.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e, f, g, h, i, j, k, l) {
        this.maxConcurrentConnections = parseInt(a) || null;
        this.maxIPConnections = parseInt(b) || null;
        this.maxUserConnections = parseInt(c) || null;
        this.maxSessionDownloads = parseInt(d) || null;
        this.maxSessionUploads = parseInt(e) || null;
        this.maxFileDownloadSize = parseInt(f) || null;
        this.maxFileUploadSize = parseInt(g) || null;
        this.maxDownloads = h || null;
        this.maxUploads = i || null;
        this.maxTransfers = j || null;
        this.maxAccountAuthorizationAttempts = k || null;
        this.maxIPAuthorizationAttempts = l || null;
    },

    toJSON: function() {
        var self = this;
        return $.grep($.map(this, function(value, key) {
            if (self.hasOwnProperty(key) && value != null) {
                var obj = {};
                if (value instanceof jscape.DomainRestrictions.Resettable) {
                    obj[key] = value.bytes;
                    obj.resetPeriod = value.resetPeriod;
                } else if (value instanceof jscape.DomainRestrictions.Attemptable) {
                    obj[key] = value.maxAttempts;
                    obj.maxTimeout = value.maxTimeout;
                    obj.restoringTimeout = value.restoringTimeout;
                } else {
                    obj[key] = value;
                }
                return obj;
            } else {
                return null;
            }
        }), function(entry) { return entry != null; });
    }
});
jscape.DomainRestrictions.fromJSON = function(data) {
    var obj = new jscape.DomainRestrictions();
    $.each(data, function(_, entry) {
        $.each(entry, function(key, value) {
            if (obj.hasOwnProperty(key)) {
                if (entry.hasOwnProperty("resetPeriod")) {
                    obj[key] = new jscape.DomainRestrictions.Resettable(value, entry["resetPeriod"]);
                } else if (entry.hasOwnProperty("maxTimeout") && entry.hasOwnProperty("restoringTimeout")) {
                    obj[key] = new jscape.DomainRestrictions.Attemptable(value, entry["maxTimeout"], entry["restoringTimeout"]);
                } else {
                    obj[key] = value;
                }
                return false;
            }
        });
    });
    return obj;
};
/** @class */
jscape.DomainRestrictions.Resettable = Class.extend(
    /** @lends {jscape.DomainRestrictions.Resettable.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.bytes = parseInt(a) || 0;
        this.resetPeriod = parseInt(b) || 0;
    }
});
/** @class */
jscape.DomainRestrictions.Attemptable = Class.extend(
    /** @lends {jscape.DomainRestrictions.Attemptable.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.maxAttempts = parseInt(a) || 0;
        this.maxTimeout = parseInt(b) || 0;
        this.restoringTimeout = parseInt(c) || 0;
    }
});

/** @class */
jscape.domain.ConnectionsModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.ConnectionsModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.ConnectionsController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onPause: function() {
        if (this.controller.hasChanged()) {
            this._confirmApply()
                    .done($.proxy(function () { this.controller.onApply(); }, this));
        }
    },

    onResume: function() {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.ConnectionsController = Class.extend(
    /** @lends {jscape.domain.ConnectionsController.prototype} */
{
    descriptor: null,
    copy: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.ConnectionsView(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.copy && !Object._equals(this.copy, this._createDescriptor());
    },

    onApply: function () {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var descriptor = this._createDescriptor();
        return this.managementService.setDomainRestrictions(this.domainName, descriptor)
                .done($.proxy(function () {
                    this.descriptor = descriptor;
                    this.copy = this._copyDescriptor();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.descriptor = this.copy;
        this.copy = this._copyDescriptor();
        this._setupView();
    },

    _loadData: function() {
        this.managementService.getDomainRestrictions(this.domainName)
                .done($.proxy(function(descriptor) {
                    this.descriptor = descriptor;
                    this.copy = this._copyDescriptor();
                    this._setupView();
                }, this));
    },

    _createDescriptor: function () {
        return new jscape.DomainRestrictions(this.view.getMaxConcurrentConnections(),
                                             this.view.getMaxIpConnections(),
                                             this.view.getMaxUserConnections(),
                                             this.view.getMaxSessionDownloads(),
                                             this.view.getMaxSessionUploads(),
                                             this.view.getMaxFileDownloadSize(),
                                             this.view.getMaxFileUploadSize(),
                                             this.view.getMaxDownloads(),
                                             this.view.getMaxUploads(),
                                             this.view.getMaxTransfers(),
                                             this.view.getAccountAuthorization(),
                                             this.view.getIpAuthorization());
    },

    _copyDescriptor: function() {
        return $.extend(new jscape.DomainRestrictions(), this.descriptor);
    },

    _setupView: function () {
        this.view.setMaxConcurrentConnections(this.descriptor.maxConcurrentConnections);
        this.view.setMaxIpConnections(this.descriptor.maxIPConnections);
        this.view.setMaxUserConnections(this.descriptor.maxUserConnections);
        this.view.setMaxSessionDownloads(this.descriptor.maxSessionDownloads);
        this.view.setMaxSessionUploads(this.descriptor.maxSessionUploads);
        this.view.setMaxFileDownloadSize(this.descriptor.maxFileDownloadSize);
        this.view.setMaxFileUploadSize(this.descriptor.maxFileUploadSize);
        this.view.setMaxDownloads(this.descriptor.maxDownloads);
        this.view.setMaxUploads(this.descriptor.maxUploads);
        this.view.setMaxTransfers(this.descriptor.maxTransfers);
        this.view.setAccountAuthorization(this.descriptor.maxAccountAuthorizationAttempts);
        this.view.setIpAuthorization(this.descriptor.maxIPAuthorizationAttempts);
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.ConnectionsView = Class.extend(
    /** @lends {jscape.domain.ConnectionsView.prototype} */
{
    /** @const */
    SIZE_FACTOR: 1024 * 1024,
    RESET_PERIOD_FACTOR: 24 * 60 * 60 * 1000,
    RESTORE_TIMEOUT_FACTOR: 60 * 1000,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=connectionsfields]");

        this.maxConcurrentConnections = new jscape.domain.ConnectionsView.Restriction(
                $("input[name=maxconcurconenabled]:checkbox", this.fieldset),
                $("input[name=maxconcurcon]", this.fieldset).numberspinner({min: 1, max: 1000, value: 100, required: true, disabled: true, width: 60})
        );
        this.maxIpConnections = new jscape.domain.ConnectionsView.Restriction(
                $("input[name=maxipconenabled]:checkbox", this.fieldset),
                $("input[name=maxipcon]", this.fieldset).numberspinner({min: 1, max: 1000, value: 100, required: true, disabled: true, width: 60})
        );
        this.maxUserConnections = new jscape.domain.ConnectionsView.Restriction(
                $("input[name=maxuserconenabled]:checkbox", this.fieldset),
                $("input[name=maxusercon]", this.fieldset).numberspinner({min: 1, max: 1000, value: 100, required: true, disabled: true, width: 60})
        );
        this.maxSessionDownloads = new jscape.domain.ConnectionsView.Restriction(
                $("input[name=maxsessiondloadsenabled]:checkbox", this.fieldset),
                $("input[name=maxsessiondloads]", this.fieldset).numberspinner({min: 1, max: 1000, increment: 10, value: 100, required: true, disabled: true, width: 60})
        );
        this.maxSessionUploads = new jscape.domain.ConnectionsView.Restriction(
                $("input[name=maxsessionuploadsenabled]:checkbox", this.fieldset),
                $("input[name=maxsessionuploads]", this.fieldset).numberspinner({min: 1, max: 1000, increment: 10, value: 100, required: true, disabled: true, width: 60})
        );
        this.maxFileDownloadSize = new jscape.domain.ConnectionsView.Restriction(
                $("input[name=maxfiledloadsizeenabled]:checkbox", this.fieldset),
                $("input[name=maxfiledloadsize]", this.fieldset).numberspinner({min: 1, max: 1000, increment: 10, value: 100, required: true, disabled: true, width: 60})
        );
        this.maxFileUploadSize = new jscape.domain.ConnectionsView.Restriction(
                $("input[name=maxfileuploadsizeenabled]:checkbox", this.fieldset),
                $("input[name=maxfileuploadsize]", this.fieldset).numberspinner({min: 1, max: 1000, increment: 10, value: 100, required: true, disabled: true, width: 60})
        );
        this.maxDownloads = new jscape.domain.ConnectionsView.RestrictionResetable(
                $("input[name=maxdloadsenabled]:checkbox", this.fieldset),
                $("input[name=maxdloads]", this.fieldset).numberspinner({min: 1, max: 1000000, increment: 10, value: 1000, required: true, disabled: true, width: 90}),
                $("input[name=maxdloadsperiod]", this.fieldset).numberspinner({min: 1, max: 1000, value: 1})
        );
        this.maxUploads = new jscape.domain.ConnectionsView.RestrictionResetable(
                $("input[name=maxuploadsenabled]:checkbox", this.fieldset),
                $("input[name=maxuploads]", this.fieldset).numberspinner({min: 1, max: 1000000, increment: 10, value: 1000, required: true, disabled: true, width: 90}),
                $("input[name=maxuploadsperiod]", this.fieldset).numberspinner({min: 1, max: 1000, value: 1})
        );
        this.maxTransfers = new jscape.domain.ConnectionsView.RestrictionResetable(
                $("input[name=maxtransfersenabled]:checkbox", this.fieldset),
                $("input[name=maxtransfers]", this.fieldset).numberspinner({min: 1, max: 1000000, increment: 10, value: 1000, required: true, disabled: true, width: 90}),
                $("input[name=maxtransfersperiod]", this.fieldset).numberspinner({min: 1, max: 1000, value: 1})
        );

        this.accountAuthorizationEnabled = $("input[name=disableaccountenabled]:checkbox", this.fieldset);
        this.accountAuthorization = $("input[name=disableaccount]", this.fieldset);
        this.accountAuthorization.numberspinner({min: 1, max: 100, value: 3, required: true, disabled: true, width: 60});
        this.accountAuthorizationTimeout = $("input[name=disableaccounttimeout]", this.fieldset);
        this.accountAuthorizationTimeout.numberspinner({min: 1, max: 1000, value: 3, required: true, disabled: true, width: 60});
        this.accountAuthorizationRestore = $("input[name=disableaccountrestore]", this.fieldset);
        this.accountAuthorizationRestore.numberspinner({min: 0, max: 1000, value: 60, required: true, disabled: true, width: 60});
        this.accountAuthorizationEnabled.change($.proxy(this._adjustSpinners, this, this.accountAuthorizationEnabled, [this.accountAuthorization, this.accountAuthorizationTimeout, this.accountAuthorizationRestore]));

        this.ipAuthorizationEnabled = $("input[name=disableipenabled]:checkbox", this.fieldset);
        this.ipAuthorization = $("input[name=disableip]", this.fieldset);
        this.ipAuthorization.numberspinner({min: 1, max: 100, value: 3, required: true, disabled: true, width: 60});
        this.ipAuthorizationTimeout = $("input[name=disableiptimeout]", this.fieldset);
        this.ipAuthorizationTimeout.numberspinner({min: 1, max: 1000, value: 3, required: true, disabled: true, width: 60});
        this.ipAuthorizationRestore = $("input[name=disableiprestore]", this.fieldset);
        this.ipAuthorizationRestore.numberspinner({min: 0, max: 1000, value: 60, required: true, disabled: true, width: 60});
        this.ipAuthorizationEnabled.change($.proxy(this._adjustSpinners, this, this.ipAuthorizationEnabled, [this.ipAuthorization, this.ipAuthorizationTimeout, this.ipAuthorizationRestore]));

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getMaxConcurrentConnections: function() {
        return this.maxConcurrentConnections.val();
    },

    setMaxConcurrentConnections: function(value) {
        this.maxConcurrentConnections.val(value);
    },

    getMaxIpConnections: function() {
        return this.maxIpConnections.val();
    },

    setMaxIpConnections: function(value) {
        this.maxIpConnections.val(value);
    },

    getMaxUserConnections: function() {
        return this.maxUserConnections.val();
    },

    setMaxUserConnections: function(value) {
        this.maxUserConnections.val(value);
    },

    getMaxSessionDownloads: function() {
        return this.maxSessionDownloads.val();
    },

    setMaxSessionDownloads: function(value) {
        this.maxSessionDownloads.val(value);
    },

    getMaxSessionUploads: function() {
        return this.maxSessionUploads.val();
    },

    setMaxSessionUploads: function(value) {
        this.maxSessionUploads.val(value);
    },

    getMaxFileDownloadSize: function() {
        var value = this.maxFileDownloadSize.val();
        return value != null ? value * this.SIZE_FACTOR : value;
    },

    setMaxFileDownloadSize: function(value) {
        this.maxFileDownloadSize.val(value ? value / this.SIZE_FACTOR : value);
    },

    getMaxFileUploadSize: function() {
        var value = this.maxFileUploadSize.val();
        return value != null ? value * this.SIZE_FACTOR : value;
    },

    setMaxFileUploadSize: function(value) {
        this.maxFileUploadSize.val(value ? value / this.SIZE_FACTOR : value);
    },

    getMaxDownloads: function() {
        return this._getResettable(this.maxDownloads);
    },

    setMaxDownloads: function(value) {
        this._setResettable(this.maxDownloads, value);
    },

    getMaxUploads: function() {
        return this._getResettable(this.maxUploads);
    },

    setMaxUploads: function(value) {
        this._setResettable(this.maxUploads, value);
    },

    getMaxTransfers: function() {
        return this._getResettable(this.maxTransfers);
    },

    setMaxTransfers: function(value) {
        this._setResettable(this.maxTransfers, value);
    },

    getAccountAuthorization: function() {
        return this.accountAuthorizationEnabled.is(":checked")
                ? new jscape.DomainRestrictions.Attemptable(parseInt(this.accountAuthorization.numberspinner("getValue")),
                                                            parseInt(this.accountAuthorizationTimeout.numberspinner("getValue")) *  this.RESTORE_TIMEOUT_FACTOR,
                                                            parseInt(this.accountAuthorizationRestore.numberspinner("getValue")) * this.RESTORE_TIMEOUT_FACTOR)
                : null
    },

    setAccountAuthorization: function(value) {
        this.accountAuthorizationEnabled.prop("checked", value != null);
        if (value != null) {
            this.accountAuthorization.numberspinner("setValue", value.maxAttempts);
            this.accountAuthorizationTimeout.numberspinner("setValue", value.maxTimeout / this.RESTORE_TIMEOUT_FACTOR);
            this.accountAuthorizationRestore.numberspinner("setValue", value.restoringTimeout / this.RESTORE_TIMEOUT_FACTOR);
        }
        this._adjustSpinners(this.accountAuthorizationEnabled, [this.accountAuthorization, this.accountAuthorizationTimeout, this.accountAuthorizationRestore]);
    },

    getIpAuthorization: function() {
        return this.ipAuthorizationEnabled.is(":checked")
                ? new jscape.DomainRestrictions.Attemptable(parseInt(this.ipAuthorization.numberspinner("getValue")),
                                                            parseInt(this.ipAuthorizationTimeout.numberspinner("getValue")) *  this.RESTORE_TIMEOUT_FACTOR,
                                                            parseInt(this.ipAuthorizationRestore.numberspinner("getValue")) * this.RESTORE_TIMEOUT_FACTOR)
                : null
    },

    setIpAuthorization: function(value) {
        this.ipAuthorizationEnabled.prop("checked", value != null);
        if (value != null) {
            this.ipAuthorization.numberspinner("setValue", value.maxAttempts);
            this.ipAuthorizationTimeout.numberspinner("setValue", value.maxTimeout / this.RESTORE_TIMEOUT_FACTOR);
            this.ipAuthorizationRestore.numberspinner("setValue", value.restoringTimeout / this.RESTORE_TIMEOUT_FACTOR);
        }
        this._adjustSpinners(this.ipAuthorizationEnabled, [this.ipAuthorization, this.ipAuthorizationTimeout, this.ipAuthorizationRestore]);
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.ConnectionsView.LISTENER;
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _getResettable: function(input) {
        var size = input.val();
        var period = input.resetPeriod();
        return size && period ? new jscape.DomainRestrictions.Resettable(size * this.SIZE_FACTOR, period * this.RESET_PERIOD_FACTOR) : null;
    },

    _setResettable: function(input, resettable) {
        input.val(resettable && resettable.bytes ? resettable.bytes / this.SIZE_FACTOR : null);
        input.resetPeriod(resettable && resettable.resetPeriod ? resettable.resetPeriod / this.RESET_PERIOD_FACTOR : null);
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard(this);
    },

    _adjustSpinners: function (checkbox, spinners) {
        var enabled = checkbox.is(":checked");
        $.each(spinners, function(_, spinner) {
            spinner.numberspinner(enabled ? "enable" : "disable").validatebox(enabled ? "enableValidation" : "disableValidation");
        });
    }
});
jscape.domain.ConnectionsView.LISTENER = {
    onApply: $.noop,
    onDiscard: $.noop
};
/** @class */
jscape.domain.ConnectionsView.Restriction = Class.extend(
    /** @lends {jscape.domain.ConnectionsView.Restriction.prototype} */
{
    /** @constructor */
    initialize: function(checkbox, spinner) {
        this.enabled = checkbox;
        this.enabled.change($.proxy(this._adjustSpinner, this));
        this.input = spinner;
        $.extend(this.input.numberspinner("options"), {required: true, disabled: true});
    },

    val: function(value) {
        if (!arguments.length) {
            return this.enabled.is(":checked") ? parseInt(this.input.numberspinner("getValue")) : null;
        } else {
            this.enabled.prop("checked", value != null);
            if (value != null){
                this.input.numberspinner("setValue", value);
            }
            this._adjustSpinner();
        }
    },

    _adjustSpinner: function () {
        if (this.enabled.is(":checked")) {
            this.input.numberspinner("enable").validatebox("enableValidation");
        } else {
            this.input.numberspinner("disable").validatebox("disableValidation");
        }
    }
});
/** @class */
jscape.domain.ConnectionsView.RestrictionResetable = jscape.domain.ConnectionsView.Restriction.extend(
    /** @lends {jscape.domain.ConnectionsView.RestrictionResetable.prototype} */
{
    /** @constructor */
    initialize: function(checkbox, spinner, resetSpinner) {
        this._super(checkbox, spinner);
        this.resetSpinner = resetSpinner;
    },

    resetPeriod: function(value) {
        if (!arguments.length) {
            return this.enabled.is(":checked") ? parseInt(this.resetSpinner.numberspinner("getValue")) : null;
        } else if (value != null) {
            this.resetSpinner.numberspinner("setValue", value);
            this._adjustResetSpinner();
        }
    },

    /** @override */
    _adjustSpinner: function() {
        this._super();
        this._adjustResetSpinner();
    },

    _adjustResetSpinner: function() {
        if (this.enabled.is(":checked")) {
            this.resetSpinner.numberspinner("enable").validatebox("enableValidation");
        } else {
            this.resetSpinner.numberspinner("disable").validatebox("disableValidation");
        }
    }
});
