jscape.domain = jscape.domain || {};

/** @class */
jscape.Contact = Class.extend(
    /** @lends {jscape.Contact.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.name = a || "" ;
        this.email = b || "";
        this.company = c || null;
        this.owner = d || null;
    }
});
jscape.Contact.fromJSON = function(data) {
    return data ? $.extend(new jscape.Contact(), data) : null;
};

/** @class */
jscape.domain.ContactsModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.ContactsModule.prototype} */
{
    onCreate: function() {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.ContactsController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onResume: function () {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.ContactsController = Class.extend(
    /** @lends {jscape.domain.ContactsController.prototype} */
{
    contacts: null,

    /** @constructor */
    initialize: function(resources) {
        this.view = new jscape.domain.ContactsView(resources);
        this.addController = new jscape.domain.AddContactController(resources);
        this.editController = new jscape.domain.EditContactController(resources);
        this.deleteController = new jscape.domain.DeleteContactController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onAddContact: function() {
        this.addController.start(this.managementService, this.domainName)
                .done($.proxy(this._contactAdded, this));
    },

    onEditContact: function(contact) {
        var index = $.inArray(contact, this.contacts);
        if (index != -1) {
            this.editController.start(this.managementService, this.domainName, contact)
                    .done($.proxy(this._contactEdited, this, index));
        }
    },

    onDeleteContact: function(contacts) {
        this.deleteController.start(this.managementService, this.domainName, contacts)
                .done($.proxy(this._contactDeleted, this, contacts));
    },

    onRefresh: function () {
        this._loadData();
    },

    _loadData: function() {
        this.managementService.getContacts(this.domainName)
                .done($.proxy(function(contacts) { this.contacts = contacts || []; }, this))
                .fail($.proxy(function() { this.contacts = []; }, this))
                .always($.proxy(function() { this.view.setContacts(this.contacts); }, this));
    },

    _contactAdded: function(contact) {
        this.contacts.push(contact);
        this.view.setContacts(this.contacts);
        this.view.selectContact(contact);
    },

    _contactEdited: function(index, contact) {
        this.contacts[index] = contact;
        this.view.setContacts(this.contacts);
        this.view.selectContact(contact);
    },

    _contactDeleted: function (contacts) {
        var index = -1;
        while (contacts.length) {
            index = $.inArray(contacts.pop(), this.contacts);
            this.contacts.splice(index, 1);
        }
        this.view.setContacts(this.contacts);

        if (this.contacts.length) {
            index = Math.max(0, Math.min(index, this.contacts.length - 1));
            this.view.selectContact(this.contacts[index]);
        }
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("contact_name_nonempty", resources.getString("CONTACTS_ERROR_BAD_NAME"));
    }
});

/** @class */
jscape.domain.ContactsView = Class.extend(
    /** @lends {jscape.domain.ContactsView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources, container) {
        this.selection = [];

        this.fieldset = $("fieldset[name=contactslistfields]", container);

        this.data = $("table[role=grid][aria-label=contacts]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            remoteSort: false,
            ctrlSelect: true,
            pagination: true,
            search: true,
            sortName: "name",
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), width: 100, sortable: true},
                {field: "company", title: $("thead th:nth-child(2)", this.data).text(), width: 80, sortable: true, sorter: function(a, b) { return a == b ? 0 : a == null || a < b ? -1 : 1; }},
                {field: "email", title: $("thead th:nth-child(3)", this.data).text(), width: 100, sortable: false},
                {field: "owner", title: $("thead th:nth-child(4)", this.data).text(), width: 80, sortable: true, sorter: function(a, b) { return a == b ? 0 : a == null || a < b ? -1 : 1; }}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this)
        });

        this.refreshButton = $("a[role=button][name=refresh]", this.fieldset);
        this.refreshButton.linkbutton({onClick: $.proxy(this._onClick$RefreshButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.ContactsView.LISTENER;
        this._adjustButtonsState();
    },

    setContacts: function (values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectContact: function (value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    _onClick$AddButton: function () {
        this.listener.onAddContact();
    },

    _onClick$EditButton: function () {
        this.listener.onEditContact(this.selection[0]);
    },

    _onClick$DeleteButton: function () {
        this.listener.onDeleteContact(this.selection);
    },

    _onClick$RefreshButton: function () {
        this.listener.onRefresh();
    },

    _adjustRowSelection: function () {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        this.editButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }
});
jscape.domain.ContactsView.LISTENER = {
    onAddContact: $.noop,
    onEditContact: $.noop,
    onDeleteContact: $.noop,
    onRefresh: $.noop
};

/** @class */
jscape.domain.AddContactController = Class.extend(
    /** @lends {jscape.domain.AddContactController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._setupDialog().show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var contact = this._createContact(dialog);
        dialog.destroy();

        this.managementService.storeContact(this.domainName, contact)
                .done($.proxy(function() { this.deferred.resolve(contact); }, this))
                .fail(this.deferred.reject);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createContact: function(dialog) {
        return new jscape.Contact(dialog.getName(), dialog.getEmail(), dialog.getCompany(), dialog.getOwner());
    },

    _setupDialog: function () {
        var dialog = new jscape.domain.AddContactDialog(this.resources);
        this.managementService.getAccounts(this.domainName)
                .done($.proxy(function (accounts) {
                    var owners = $.map(accounts, function (account) { return account.login; });
                    dialog.setOwners(owners.sort());
                }, this));
        return dialog;
    }
});

/** @class */
jscape.domain.EditContactController = Class.extend(
    /** @lends {jscape.domain.EditContactController.prototype} */
{
    contact: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, contact) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.contact = contact;
        this.deferred = $.Deferred();

        this._setupDialog(contact).show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var contact = this._createContact(dialog);
        dialog.destroy();

        this.managementService.deleteContact(this.domainName, this.contact)
                .done($.proxy(function() {
                    this.managementService.storeContact(this.domainName, contact)
                            .done($.proxy(function() { this.deferred.resolve(contact); }, this))
                            .fail($.proxy(function() { this.deferred.reject(); }, this));
                    }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createContact: function(dialog) {
        return new jscape.Contact(dialog.getName(), dialog.getEmail(), dialog.getCompany(), dialog.getOwner());
    },

    _setupDialog: function (contact) {
        var dialog = new jscape.domain.EditContactDialog(this.resources);
        dialog.setName(contact.name);
        dialog.setEmail(contact.email);
        dialog.setCompany(contact.company);

        this.managementService.getAccounts(this.domainName)
                .done($.proxy(function (accounts) {
                    var owners = $.map(accounts, function (account) { return account.login; });
                    dialog.setOwners(owners.sort());
                    dialog.setOwner(contact.owner);
                }, this));
        return dialog;
    }
});

/** @class */
jscape.domain.DeleteContactController = Class.extend(
    /** @lends {jscape.domain.DeleteContactController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function (managementService, domainName, contacts) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._confirmDelete(contacts)
                .done($.proxy(this._deleteConfirmed, this, contacts))
                .fail($.proxy(function() { this.deferred.reject(); }, this));

        return this.deferred.promise();
    },

    _confirmDelete: function(contacts) {
        var text = $.map(contacts || [], function(contact) { return contact.name; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("CONTACTS_CONFIRM_DELETE_TITLE"),
                this.resources.getString("CONTACTS_CONFIRM_DELETE_MESSAGE", {0: text}));
    },

    _deleteConfirmed: function(contacts) {
        var promises = [];
        for (var i = 0; i < contacts.length; i++) {
            promises.push(this.managementService.deleteContact(this.domainName, contacts[i]));
        }

        $.when.apply(jQuery, promises)
                .done($.proxy(function () { this.deferred.resolve(contacts); }, this))
                .fail($.proxy(function () { this.deferred.reject(); }, this));
    }
});

/** @class */
jscape.domain.ContactDialog = Class.extend(
    /** @lends {jscape.domain.AddContactDialog.prototype} */
{
    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=contactfields]", this.dialog);

        this.name = $("[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: "contact_name_nonempty", missingMessage: resources.getString("CONTACTS_ERROR_BAD_NAME")});

        this.email = $("[name=email]", this.fieldset);
        this.email.validatebox({required: true, validType: "multi_value['email',',']", missingMessage: resources.getString("CONTACTS_ERROR_BAD_EMAIL"), invalidMessage: resources.getString("CONTACTS_ERROR_BAD_EMAIL")});

        this.company = $("[name=company]", this.fieldset);

        this.ownedBy = $("[name=ownedby]:checkbox", this.fieldset);
        this.ownedBy.on("change", $.proxy(this._adjustFieldState, this));

        this.owner = $("[name=owner]", this.fieldset);
        this.owner.combobox({
            required: false,
            editable: false,
            disabled: true,
            width: 120,
            loadFilter: function(data) {return $.map(data, function(name) { return {text: name, value: name}; })}}
        ).combobox("autocomplete", true);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this.fieldset.form("validate");
        this.name.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    getName: function() {
        return this.name.val();
    },

    setName: function(value) {
        this.name.val(value);
    },

    getEmail: function() {
        return this.email.val();
    },

    setEmail: function(value) {
        this.email.val(value);
    },

    getCompany: function() {
        return this.company.val();
    },

    setCompany: function(value) {
        this.company.val(value);
    },

    setOwners: function(values) {
        if (values && values.length) {
            this.owner.combobox("loadData", values).combobox("setValue", values[0]);
            this.ownedBy.prop("disabled", false);
        } else {
            this.ownedBy.prop("disabled", true);
        }
        this.ownedBy.prop("checked", false);
    },

    getOwner: function() {
        return this.ownedBy.is(":checked") ? this.owner.combobox("getValue") : null;
    },

    setOwner: function(value) {
        if (value != null && value.trim().length) {
            this.ownedBy.prop("checked", true);
            this.owner.combobox("setValue", value);
            this._adjustFieldState();
        } else {
            this.ownedBy.prop("checked", false);
        }
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _adjustFieldState: function() {
        var enabled = this.ownedBy.is(":checked");
        this.owner.combobox(enabled ? "enable" : "disable");
    }
});

/** @class */
jscape.domain.AddContactDialog = jscape.domain.ContactDialog.extend(
    /** @lends {jscape.domain.AddContactDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-contact-add"), resources);
    }
});

/** @class */
jscape.domain.EditContactDialog = jscape.domain.ContactDialog.extend(
    /** @lends {jscape.domain.AddContactDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-contact-edit"), resources);

        this.title = this.dialog.dialog("header").text();
    },

    setName: function(value) {
        this._super(value);
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});