jscape.domain = jscape.domain || {};

/** @class */
jscape.DatastoreService = Class.extend(
    /** @lends {jscape.DatastoreService.prototype} */
{
    /** @constructor */
    initialize: function(a) {
        this.type = a || null;
    }
});
jscape.DatastoreService.fromJSON = function(data) {
    return data ? $.extend(new jscape.DatastoreService(), data) : null;
};
jscape.DatastoreService.Type = {FILE: "FILE", FILE_RECORD: "FILE_RECORD", DB: "DB"};
jscape.DatastoreService.file = function(directory) {
    return $.extend(new jscape.DatastoreService(jscape.DatastoreService.Type.FILE), {directory: directory});
};
jscape.DatastoreService.fileRecord = function(directory) {
    return $.extend(new jscape.DatastoreService(jscape.DatastoreService.Type.FILE_RECORD), {directory: directory});
};
jscape.DatastoreService.db = function(url, user, password, poolSize, ttl) {
    return $.extend(new jscape.DatastoreService(jscape.DatastoreService.Type.DB), {url: url, username: user, password: password, poolSize: poolSize, idleConnectionTTL: ttl});
};


/** @class */
jscape.domain.DatastoreModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.DatastoreModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.DatastoreController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onPause: function() {
        if (this.controller.hasChanged()) {
            this._confirmApply()
                    .done($.proxy(function () { this.controller.onApply(); }, this));
        }
    },

    onResume: function () {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.DatastoreController = Class.extend(
    /** @lends {jscape.domain.DatastoreController.prototype} */
{
    DEFAULT_DESCRIPTOR: jscape.DatastoreService.fileRecord(""),

    descriptor: null,
    copy: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.DatastoreView(resources);
        this.controllers = {
            FILE: new jscape.domain.FileDatastoreServiceController(resources),
            FILE_RECORD: new jscape.domain.FileRecordDatastoreServiceController(resources),
            DB: new jscape.domain.DbDatastoreServiceController(resources)
        };
    },

    start: function(managementService, domainName) {
        this.managementService= managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.copy && !Object._equals(this.copy, this._createDescriptor());
    },

    onApply: function () {
        var descriptor = this._createDescriptor();
        if (descriptor == null) {
            return $.Deferred().reject().promise();
        }

        return this.managementService.setDatastoreService(this.domainName, descriptor)
                .done($.proxy(function () {
                    this.descriptor = descriptor;
                    this.copy = this._copyDescriptor();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function () {
        this.descriptor = this.copy;
        this.copy = this._copyDescriptor();
        this._setupView();
    },

    onChangeType: function () {
        var serviceType = (this.view.getType() || "").toUpperCase();
        $.each(this.controllers, $.proxy(function (type, controller) {
            if (serviceType === type) {
                controller.resume();
            } else {
                controller.pause();
            }
        }, this));
    },

    _loadData: function() {
        this.managementService.getDatastoreService(this.domainName)
                .done($.proxy(function(descriptor) {
                    this.descriptor = descriptor || this.DEFAULT_DESCRIPTOR;
                }, this))
                .fail($.proxy(function() {
                    this.descriptor = this.DEFAULT_DESCRIPTOR;
                }, this))
                .always($.proxy(function() {
                    this.copy = this._copyDescriptor();
                    this._setupView();
                }, this));
    },

    _createDescriptor: function () {
        var controller = this.controllers[(this.view.getType() || "").toUpperCase()];
        return $.type(controller) === "object" ? controller.applyChanges() : null;
    },

    _copyDescriptor: function () {
        return this.descriptor != null ? $.extend(new jscape.DatastoreService(), this.descriptor) : null;
    },

    _setupView: function () {
        for (var type in this.controllers) {
            if (this.controllers.hasOwnProperty(type)) {
                this.controllers[type].start(this.managementService, this.domainName, this.descriptor);
            }
        }
        this.view.setType(this.descriptor.type);
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.DatastoreView = Class.extend(
    /** @lends {jscape.domain.DatastoreView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=datastorefields]");

        this.type = $("select[name=type]", this.fieldset);
        this.type.combobox({required: true, editable: false, width: 120, panelHeight: "auto", onChange: $.proxy(this._onChange$Type, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getType: function() {
        return this.type.combobox("getValue");
    },

    setType: function(value) {
        var select = this.type.combobox("getValue");
        this.type.combobox("setValue", value);
        if (select === value) {
            this._onChange$Type();
        }
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.DatastoreView.LISTENER;
    },

    _onChange$Type: function() {
        this.listener.onChangeType(this);
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard(this);
    }
});
jscape.domain.DatastoreView.LISTENER = {
    onChangeType: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.DatastoreServiceController = Class.extend(
    /** @lends {jscape.domain.DatastoreServiceController.prototype} */
{
    view: null,

    start: function(managementService, domainName, descriptor) {
        this.managementService = managementService;
        this.domainName = domainName;

        this._setupView(descriptor);
        this.view.show(this)
    },

    resume: function () {
        this.view.show(this);
    },

    pause: function () {
        this.view.hide();
    },

    valid: function() {
        return this.view.valid();
    },

    applyChanges: function () {
        return this.view.valid() ? this._createDescriptor() : null;
    },

    _createDescriptor: function () {
        return null;
    },

    _setupView: function (descriptor) {
    }
});

/** @class */
jscape.domain.FileDatastoreServiceController = jscape.domain.DatastoreServiceController.extend(
    /** @lends {jscape.domain.FileDatastoreServiceController.prototype} */
{
    DEFAULT_DIR: "%installdir%/users/%domain%",
    VARIABLES: ["installdir", "domain"],

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.FileDatastoreServiceView(resources, container);
        this.variableController = new jscape.VariableController();
    },

    onSelectPath: function() {
        var loadFn = $.proxy(this.managementService.listDatastoreDir, this.managementService);
        var createFn = $.proxy(this.managementService.createDatastoreDir , this.managementService);
        new SelectPathController(loadFn, createFn).selectPath($.proxy(function(entry) { this.view.setDirectory(entry.path); }, this));
    },

    onAddVariable: function(input) {
        this.variableController.start(input, this.VARIABLES);
    },

    /** @override */
    _createDescriptor: function () {
        return jscape.DatastoreService.file(this.view.getDirectory());
    },

    /** @override */
    _setupView: function (descriptor) {
        if (descriptor && descriptor.type == jscape.DatastoreService.Type.FILE) {
            this.view.setDirectory(descriptor.directory);
        } else {
            this.view.setDirectory(this.DEFAULT_DIR);
        }
    }
});

/** @class */
jscape.domain.FileDatastoreServiceView = Class.extend(
    /** @lends {jscape.domain.FileDatastoreServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=filedatastorefields]", container);

        this.directory = $("input[name=directory]", this.fieldset);
        this.directory.validatebox({required: true, validType: ["length[1,500]"], missingMessage: resources.getString("DATASTORE_ERROR_BAD_DIR"), invalidMessage: resources.getString("DATASTORE_ERROR_BAD_DIR"), tipPosition: "left"});
        this.directory.focus($.proxy(function() { this.addVariable.linkbutton("enable"); }, this));

        this.browseButton = $("a[role=button][name=browse]", this.fieldset);
        this.browseButton.linkbutton({onClick: $.proxy(this._onClick$BrowseButton, this)});

        this.addVariable = $("a[role=button][name=addvariable]", this.fieldset);
        this.addVariable.linkbutton({disabled: true, onClick: $.proxy(this._onClick$AddVariable, this)});
    },

    getDirectory: function() {
        return this.directory.val();
    },

    setDirectory: function(value) {
        this.directory.val(value).validatebox("validate");
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.FileDatastoreServiceView.LISTENER;
        this.fieldset.show();
    },

    hide: function() {
        this.fieldset.hide();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$AddVariable: function() {
        this.addVariable.linkbutton("disable");
        this.listener.onAddVariable(this.directory);
    },

    _onClick$BrowseButton: function() {
        this.listener.onSelectPath(this);
    }
});
jscape.domain.FileDatastoreServiceView.LISTENER = {
    onSelectPath: $.noop,
    onAddVariable: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.FileRecordDatastoreServiceController = jscape.domain.DatastoreServiceController.extend(
    /** @lends {jscape.domain.FileRecordDatastoreServiceController.prototype} */
{
    DEFAULT_DIR: "%installdir%/users/%domain%",
    VARIABLES: ["installdir", "domain"],

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.FileRecordDatastoreServiceView(resources, container);
        this.variableController = new jscape.VariableController();
    },

    onSelectPath: function() {
        var loadFn = $.proxy(this.managementService.listDatastoreDir, this.managementService);
        var createFn = $.proxy(this.managementService.createDatastoreDir , this.managementService);
        new SelectPathController(loadFn, createFn).selectPath($.proxy(function(entry) { this.view.setDirectory(entry.path); }, this));
    },

    onAddVariable: function(input) {
        this.variableController.start(input, this.VARIABLES);
    },

    /** @override */
    _createDescriptor: function () {
        return jscape.DatastoreService.fileRecord(this.view.getDirectory());
    },

    /** @override */
    _setupView: function (descriptor) {
        if (descriptor && descriptor.type == jscape.DatastoreService.Type.FILE_RECORD) {
            this.view.setDirectory(descriptor.directory);
        } else {
            this.view.setDirectory(this.DEFAULT_DIR);
        }
    }
});

/** @class */
jscape.domain.FileRecordDatastoreServiceView = jscape.domain.FileDatastoreServiceView.extend(
    /** @lends {jscape.domain.FileRecordDatastoreServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=filerecorddatastorefields]", container);

        this.directory = $("input[name=directory]", this.fieldset);
        this.directory.validatebox({required: true, validType: ["length[1,500]"], missingMessage: resources.getString("DATASTORE_ERROR_BAD_DIR"), invalidMessage: resources.getString("DATASTORE_ERROR_BAD_DIR"), tipPosition: "left"});
        this.directory.focus($.proxy(function() { this.addVariable.linkbutton("enable"); }, this));

        this.browseButton = $("a[role=button][name=browse]", this.fieldset);
        this.browseButton.linkbutton({onClick: $.proxy(this._onClick$BrowseButton, this)});

        this.addVariable = $("a[role=button][name=addvariable]", this.fieldset);
        this.addVariable.linkbutton({disabled: true, onClick: $.proxy(this._onClick$AddVariable, this)});
    }
});

/** @class */
jscape.domain.DbDatastoreServiceController = jscape.domain.DatastoreServiceController.extend(
    /** @lends {jscape.domain.DbDatastoreServiceController.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.resources = resources;
        this.view = new jscape.domain.DbDatastoreServiceView(resources, container);
    },

    onTest: function() {
        var descriptor = this._createDescriptor();
        this.managementService.testDatastoreService(this.domainName, descriptor)
                .done($.proxy(this._testPassed, this));
    },

    /** @override */
    _createDescriptor: function () {
        return jscape.DatastoreService.db(this.view.getUrl(),
                                          this.view.getUsername(),
                                          this.view.getPassword(),
                                          this.view.getPoolSize(),
                                          this.view.getPoolTimeout());
    },

    /** @override */
    _setupView: function (descriptor) {
        if (descriptor && descriptor.type == jscape.DatastoreService.Type.DB) {
            this.view.setUrl(descriptor.url);
            this.view.setUsername(descriptor.username);
            this.view.setPassword(descriptor.password);
            this.view.setPoolSize(descriptor.poolSize);
            this.view.setPoolTimeout(descriptor.idleConnectionTTL);
        }
    },

    _testPassed: function () {
        $.messager.alert(this.resources.getString("DATASTORE_TEST_DB_TITLE"),
                         this.resources.getString("DATASTORE_TEST_DB_PASSED_MESSAGE"),
                         "info");
    }
});

/** @class */
jscape.domain.DbDatastoreServiceView = Class.extend(
    /** @lends {jscape.domain.DbDatastoreServiceView.prototype} */
{
    /** @const */
    TIMEOUT_FACTOR: 60 * 1000,

    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=dbdatastorefields]", container).hide();

        this.url = $("input[name=dburl]", this.fieldset);
        this.url.validatebox({required: true, validType: "length[1,2048]", missingMessage: resources.getString("DATASTORE_ERROR_BAD_DB_URL"), invalidMessage: resources.getString("DATASTORE_ERROR_BAD_DB_URL")});

        this.username = $("input[name=username]", this.fieldset);
        this.username.validatebox({required: true, validType: "length[1,50]", missingMessage: resources.getString("DATASTORE_ERROR_BAD_USER"), invalidMessage: resources.getString("DATASTORE_ERROR_BAD_USER")});

        this.password = $("input[name=password]", this.fieldset);

        this.poolSize = $("input[name=poolsize]", this.fieldset);
        this.poolSize.numberspinner({min: 1, max: 100, value: 5, width: 50});

        this.poolTimeout = $("input[name=pooltimeout]", this.fieldset);
        this.poolTimeout.numberspinner({min: 1, max: 100, value: 5, width: 50});

        this.testButton = $("a[role=button][name=testdb]", this.fieldset);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getUrl: function() {
        return this.url.val();
    },

    setUrl: function(value) {
        this.url.val(value);
    },

    getUsername: function() {
        return this.username.val();
    },

    setUsername: function(value) {
        this.username.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getPoolSize: function() {
        return parseInt(this.poolSize.numberspinner("getValue"));
    },

    setPoolSize: function(value) {
        this.poolSize.numberspinner("setValue", parseInt(value));
    },

    getPoolTimeout: function() {
        return parseInt(this.poolTimeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setPoolTimeout: function(value) {
        this.poolTimeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.DbDatastoreServiceView.LISTENER;
        this.fieldset.show().form("validate");
    },

    hide: function() {
        this.fieldset.hide();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$TestButton: function() {
        if (this.valid()) {
            this.listener.onTest(this);
        }
    }
});
jscape.domain.DbDatastoreServiceView.LISTENER = {
    onTest: $.noop
};