jscape.domain = jscape.domain || {};

/** @class */
jscape.domain.DescriptionModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.DescriptionModule.prototype} */
{
    description: null,

    onCreate: function() {
        this._super.apply(this, arguments);

        this.view = new jscape.domain.DescriptionView(this.resources);
    },

    onStart: function() {
        this.view.show(this);
    },

    onPause: function() {
        if (this.hasChanged()) {
            this._confirmApply()
                    .done($.proxy(this.onApply, this));
        }
    },

    onResume: function() {
        this._loadData();
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var description = this.view.getDescription();
        return this.managementService.setDomainDescription(this.domainName, description)
                .done($.proxy(function() {
                    this.description = description;
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function () {
        this._setupView();
    },

    hasChanged: function () {
        return this.description != null && this.description != this.view.getDescription();
    },

    _loadData: function() {
        this.managementService.getDomainDescription(this.domainName)
                .done($.proxy(function(desc) {
                    this.description = desc;
                    this._setupView();
                }, this));
    },

    _setupView: function() {
        this.view.setDescription(this.description);
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.DescriptionView = Class.extend(
    /** @lends {jscape.domain.DescriptionView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=descriptionfields]");
        this.description = $("textarea[name=description]", this.fieldset);
        this.description.on("input", $.proxy(this._adjustButtonsState, this));

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getDescription: function() {
        return this.description.val();
    },

    setDescription: function(value) {
        this.description.val(value);
        this.description.data("originalValue", value);
        this._adjustButtonsState();
    },

    show: function(listener) {
        this.listener = listener || jscape.DescriptionView.LISTENER;
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply()
                .always($.proxy(this._adjustButtonsState, this));
    },

    _onClick$DiscardButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onDiscard();
    },

    _adjustButtonsState: function() {
        var modified = this.description.val() != this.description.data("originalValue");
        this.applyButton.linkbutton(modified ? "enable" : "disable");
        this.discardButton.linkbutton(modified ? "enable" : "disable");
    }
});
jscape.domain.DescriptionView.LISTENER = {
    onApply: $.noop,
    onDiscard: $.noop
};