jscape.domain = jscape.domain || {};

/** @class */
jscape.DirectoryMonitor = Class.extend(
    /** @lends {jscape.DirectoryMonitor.prototype} */
{
    /** @constructor */
    initialize: function (a, b, c, d, e, f, g, h, i) {
        this.name = a || "";
        this.path = b || "";
        this.recursive = c || false;
        this.timeout = parseInt(d) || null;
        this.latency = parseInt(e) || null;
        this.owner = f || null;
        this.quota = g || null;
        this.events = h || jscape.DirectoryMonitor.Event();
    },

    toJSON: function() {
        return $.extend({
            name: this.name,
            path: this.path,
            timeout: this.timeout,
            latency: this.latency,
            recursive: this.recursive,
            quota: this.quota,
            owner: this.owner
        }, this.events.toJSON());
    }
});
jscape.DirectoryMonitor.fromJSON = function(data) {
    return new jscape.DirectoryMonitor(data.name,
                                       data.path,
                                       data.recursive,
                                       data.timeout,
                                       data.latency,
                                       data.owner,
                                       data.quota ? jscape.DirectoryQuota.fromJSON(data.quota) : null,
                                       jscape.DirectoryMonitor.Event.fromJSON(data));
};
/** @class */
jscape.DirectoryMonitor.Event = Class.extend(
    /** @lends {jscape.DirectoryMonitor.Event.prototype} */
{
    /** @const */
    MASK_ADD: 1 << 0,
    MASK_CHANGE: 1 << 1,
    MASK_DELETE: 1 << 2,

    /** @constructor */
    initialize: function(adding, changing, deleting, maxFileAge) {
        this.mask = 0;
        this.monitorAdd(!!adding);
        this.monitorChange(!!changing);
        this.monitorDelete(!!deleting);
        this.maxFileAge = parseInt(maxFileAge) || null;
    },

    monitorAdd: function(value) {
        return this._mask(value, this.MASK_ADD);
    },

    monitorChange: function(value) {
        return this._mask(value, this.MASK_CHANGE);
    },

    monitorDelete: function(value) {
        return this._mask(value, this.MASK_DELETE);
    },

    monitorFileAge: function(value) {
        if (!arguments.length) {
            return this.maxFileAge != null;
        }
        this.maxFileAge = parseInt(value) || null;
        return undefined;
    },

    _mask: function (value, flag) {
        if (value !== undefined) {
            this.mask |= value ? flag : 0;
            return undefined;
        }
        return (this.mask & flag) != 0;
    },

    toJSON: function() {
        return {
            monitorAdding: this.monitorAdd(),
            monitorChanging: this.monitorChange(),
            monitorDeleting: this.monitorDelete(),
            maxFileAge: this.maxFileAge
        }
    }
});
jscape.DirectoryMonitor.Event.fromJSON = function(data) {
    return new jscape.DirectoryMonitor.Event(data.monitorAdding,
                                             data.monitorChanging,
                                             data.monitorDeleting,
                                             data.maxFileAge);
};
/** @class */
jscape.DirectoryQuota = Class.extend(
    /** @lends {jscape.DirectoryQuota.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.value = parseInt(a) || 0;
        this.currentValue = parseInt(b) || 0;
        this.exceeded = c || false;
    }
});
jscape.DirectoryQuota.fromJSON = function(data) {
    return new jscape.DirectoryQuota(data.value, data.currentValue, data.exceeded);
};

/** @class */
jscape.DirectoryMonitorsConfiguration = Class.extend(
    /** @lends {jscape.DirectoryMonitorsConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a) {
        this.maxThreadCount = parseInt(a) || 10;
    }
});
jscape.DirectoryMonitorsConfiguration.fromJSON = function(data) {
    return new jscape.DirectoryMonitorsConfiguration(data.maxThreadCount);
};

/** @class */
jscape.domain.DirectoryMonitorsModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.DirectoryMonitorsModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.monitorsController = new jscape.domain.DirectoryMonitorsController(this.resources);
        this.configController = new jscape.domain.DirectoryMonitorsConfigurationController(this.resources);
    },

    onStart: function () {
        this.monitorsController.start(this.managementService, this.domainName);
        this.configController.start(this.managementService, this.domainName);
    },

    onPause: function () {
        if (this.configController.hasChanged()) {
            this._confirmApply()
                    .done($.proxy(function () { this.configController.onApply(); }, this));
        }
    },

    onResume: function() {
        this.monitorsController._loadData();
        this.configController._loadData();
    }
});

/** @class */
jscape.domain.DirectoryMonitorsController = Class.extend(
    /** @lends {jscape.DirectoryMonitorsController.prototype} */
{
    monitors: null,

    /** @constructor */
    initialize: function(resources) {
        this.monitors = [];
        this.view = new jscape.domain.DirectoryMonitorsView(resources);
        this.addController = new jscape.domain.AddDirectoryMonitorController(resources);
        this.editController = new jscape.domain.EditDirectoryMonitorController(resources);
        this.deleteController = new jscape.domain.DeleteDirectoryMonitorController(resources);

        this._initValidators(resources);
    },

    start: function(managemementService, domainName) {
        this.managementService = managemementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onAddMonitor: function() {
        this.addController.start(this.managementService, this.domainName)
                .done($.proxy(this._monitorAdded, this));
    },

    onEditMonitor: function(monitor) {
        var index = $.inArray(monitor, this.monitors);
        if (index != -1) {
            this.editController.start(this.managementService, this.domainName, monitor)
                    .done($.proxy(this._monitorEdited, this, index));
        }
    },

    onDeleteMonitor: function(monitors) {
        this.deleteController.start(this.managementService, this.domainName, monitors)
                .done($.proxy(this._monitorDeleted, this));
    },

    onRunMonitor: function(monitor) {
        this.managementService.runDirectoryMonitor(this.domainName, monitor.name);
    },

    onRefresh: function() {
        this._loadData();
    },

    _loadData: function() {
        this.managementService.getDirectoryMonitors(this.domainName)
                .done($.proxy(function(monitors) {
                    this.monitors = monitors || [];
                    this.view.setMonitors(this.monitors);
                }, this));
    },

    _monitorAdded: function(monitor) {
        this.monitors.push(monitor);
        this.view.setMonitors(this.monitors);
        this.view.selectMonitor(monitor);
    },

    _monitorEdited: function(index, monitor) {
        this.monitors[index] = monitor;
        this.view.setMonitors(this.monitors);
        this.view.selectMonitor(monitor);
    },

    _monitorDeleted: function (monitors) {
        var index = -1;
        while (monitors.length) {
            index = $.inArray(monitors.pop(), this.monitors);
            this.monitors.splice(index, 1);
        }
        this.view.setMonitors(this.monitors);

        if (this.monitors.length) {
            index = Math.min(Math.max(0, index), this.monitors.length - 1);
            this.view.selectMonitor(this.monitors[index]);
        }
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("dirmonitor_name_nonempty", resources.getString("DIR_MONITOR_ERROR_BAD_NAME"));
        v.nonEmptyRule("dirmonitor_path_nonempty", resources.getString("DIR_MONITOR_ERROR_BAD_PATH"));
        v.rule("dirmonitor_unique", $.proxy(this._monitorNotExists, this), resources.getString("DIR_MONITOR_ERROR_ALREADY_EXIST"));
        v.rule("dirmonitor_filename", $.proxy(this._filenameValid, this), resources.getString("DIR_MONITOR_ERROR_BAD_FILENAME"));
    },

    _monitorNotExists: function (monitorName) {
        return $.grep(this.monitors, function (monitor) { return monitor.name == monitorName; }).length == 0;
    },

    _filenameValid: function(value) {
        return !/[\*\?\<\>\|"]/g.test(value);
    }
});

/** @class */
jscape.domain.DirectoryMonitorsView = Class.extend(
    /** @lends {jscape.domain.DirectoryMonitorsView.prototype} */
{
    /** @const */
    TIMEOUT_FACTOR: 1000,

    selection: [],

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.sizeFormat = jscape.SizeFormat();

        this.fieldset = $("fieldset[name=dirmonitorslistfields]");

        this.data = $("table[role=grid][aria-label=dirmonitors]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            singleSelect: true,
            pagination: true,
            search: true,
            remoteSort: false,
            sortName: "name",
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 120},
                {field: "path", title: $("thead th:nth-child(2)", this.data).text(), width: 200},
                {field: "timeout", title: $("thead th:nth-child(3)", this.data).text(), align: "right", formatter: $.proxy(this._formatTimeout, this), width: 70},
                {field: "quota", title: $("thead th:nth-child(4)", this.data).text(), width: 135, align: "right", formatter: $.proxy(this._formatQuota, this), styler: $.proxy(this._styleQuota, this)},
                {field: "owner", title: $("thead th:nth-child(5)", this.data).text(), sortable: true, width: 70},
                {field: "events", title: $("thead th:nth-child(6)", this.data).text(), formatter: $.proxy(this._formatEvents, this), width: 170},
                {field: "recursive", title: $("thead th:nth-child(7)", this.data).text(), width: 50}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this)
        });

        this.refreshButton = $("a[role=button][name=refresh]", this.fieldset);
        this.refreshButton.linkbutton({onClick: $.proxy(this._onClick$RefreshButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.runButton = $("a[role=button][name=run]", this.fieldset);
        this.runButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$RunButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.DirectoryMonitorsView.LISTENER;
        this.sizeFormat = new jscape.SizeFormat();
        this._adjustButtonsState();
    },

    setMonitors: function (values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectMonitor: function (value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    _onClick$RefreshButton: function() {
        this.listener.onRefresh();
    },
    _onClick$AddButton: function() {
        this.listener.onAddMonitor();
    },

    _onClick$EditButton: function() {
        this.listener.onEditMonitor(this.selection[0]);
    },

    _onClick$RunButton: function() {
        this.listener.onRunMonitor(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteMonitor(this.selection);
    },

    _formatTimeout: function (timeout) {
        return timeout != null
                ? this.resources.getString("DIR_MONITOR_TEMPLATE_INTERVAL", {0: timeout / this.TIMEOUT_FACTOR})
                : this.resources.getString("DIR_MONITOR_TEMPLATE_INTERVAL_NONE");
    },

    _formatQuota: function (quota) {
        return quota ? this.resources.getString("DIR_MONITOR_TEMPLATE_QUOTA", {0: this.sizeFormat.format(quota.currentValue), 1: this.sizeFormat.format(quota.value)}) : "";
    },

    _styleQuota: function(quota) {
        return quota && !quota.exceeded ? "color:green" : "color:red";
    },

    _formatEvents: function(events) {
        var t = [];
        events.monitorAdd() && t.push(this.resources.getString("DIR_MONITOR_EVENTS_ADDING"));
        events.monitorChange() && t.push(this.resources.getString("DIR_MONITOR_EVENTS_CHANGING"));
        events.monitorDelete() && t.push(this.resources.getString("DIR_MONITOR_EVENTS_DELETING"));
        events.monitorFileAge() && t.push(this.resources.getString("DIR_MONITOR_EVENTS_FILE_AGING"));
        return t.join(", ");
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function () {
        var singleSelection = this.selection.length == 1;
        this.editButton.linkbutton(singleSelection ? "enable" : "disable");
        this.runButton.linkbutton(singleSelection ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }
});
/** @const */
jscape.domain.DirectoryMonitorsView.LISTENER = {
    onAddMonitor: $.noop,
    onEditMonitor: $.noop,
    onDeleteMonitor: $.noop,
    onRunMonitor: $.noop,
    onRefresh: $.noop
};

/** @class */
jscape.domain.AddDirectoryMonitorController = Class.extend(
    /** @lends {jscape.domain.AddDirectoryMonitorController.prototype} */
{
    /** @const */
    DEFAULT_TIMEOUT: 600 * 1000,
    DEFAULT_LATENCY: 5 * 1000,

    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.AddDirectoryMonitorDialog(this.resources);
        this._setupDialog(dialog);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var monitor = this._createMonitor(dialog);
        dialog.destroy();

        this.managementService.storeDirectoryMonitor(this.domainName, monitor)
                .done($.proxy(function() { this.deferred.resolve(monitor); }, this))
                .fail(this.deferred.reject);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    onSelectPath: function(dialog) {
        var loadFn = $.proxy(this.managementService.listMonitorsDir, this.managementService);
        var createFn = $.proxy(this.managementService.createMonitorsDir, this.managementService);
        var c = new SelectPathController(loadFn, createFn);
        c.selectDirectory($.proxy(function(entry) { dialog.setPath(entry.path); }, this));
    },

    _setupDialog: function(dialog) {
        dialog.setRecursive(true);
        dialog.setTimeout(this.DEFAULT_TIMEOUT);
        dialog.setLatency(this.DEFAULT_LATENCY);
        dialog.setMonitorAdding(true);
        dialog.setMonitorChanging(true);
        dialog.setMonitorDeleting(true);

        this.managementService.getAccounts(this.domainName)
                .done(function (accounts) {
                    var owners = $.map(accounts, function (account) { return account.login; });
                    dialog.setOwners(owners.sort());
                });
    },

    _createMonitor: function (dialog) {
        return new jscape.DirectoryMonitor(dialog.getName(),
                                           dialog.getPath(),
                                           dialog.getRecursive(),
                                           dialog.getTimeout(),
                                           dialog.getLatency(),
                                           dialog.getOwner(),
                                           this._createQuota(dialog),
                                           this._createEvents(dialog));
    },

    _createQuota: function(dialog) {
        var value = dialog.getQuota();
        return value != null ? new jscape.DirectoryQuota(value) : null;
    },

    _createEvents: function(dialog) {
        return new jscape.DirectoryMonitor.Event(dialog.getMonitorAdding(),
                                                 dialog.getMonitorChanging(),
                                                 dialog.getMonitorDeleting(),
                                                 dialog.getMaxFileAge());
    }
});

/** @class */
jscape.domain.EditDirectoryMonitorController = Class.extend(
    /** @lends {jscape.domain.EditDirectoryMonitorController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, monitor) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.monitor = monitor;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.EditDirectoryMonitorDialog(this.resources);
        this._setupDialog(dialog);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var monitor = this._createMonitor(dialog);
        dialog.destroy();

        this.managementService.storeDirectoryMonitor(this.domainName, monitor)
                .done($.proxy(function() { this.deferred.resolve(monitor); }, this))
                .fail(this.deferred.reject);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    onSelectPath: function(dialog) {
        var loadFn = $.proxy(this.managementService.listMonitorsDir, this.managementService);
        var createFn = $.proxy(this.managementService.createMonitorsDir, this.managementService);
        var c = new SelectPathController(loadFn, createFn);
        c.selectDirectory($.proxy(function(entry) { dialog.setPath(entry.path); }, this));
    },

    _setupDialog: function(dialog) {
        dialog.setName(this.monitor.name);
        dialog.setPath(this.monitor.path);
        dialog.setRecursive(this.monitor.recursive);
        dialog.setTimeout(this.monitor.timeout);
        dialog.setLatency(this.monitor.latency);
        dialog.setQuota(this.monitor.quota);
        dialog.setMonitorAdding(this.monitor.events.monitorAdd());
        dialog.setMonitorChanging(this.monitor.events.monitorChange());
        dialog.setMonitorDeleting(this.monitor.events.monitorDelete());
        dialog.setMaxFileAge(this.monitor.events.maxFileAge);

        this.managementService.getAccounts(this.domainName)
                .done($.proxy(function (accounts) {
                    var owners = $.map(accounts, function (account) { return account.login; });
                    dialog.setOwners(owners.sort());
                    dialog.setOwner(this.monitor.owner);
                }, this));
    },

    _createMonitor: function (dialog) {
        return new jscape.DirectoryMonitor(this.monitor.name,
                                           dialog.getPath(),
                                           dialog.getRecursive(),
                                           dialog.getTimeout(),
                                           dialog.getLatency(),
                                           dialog.getOwner(),
                                           this._createQuota(dialog),
                                           this._createEvents(dialog));
    },

    _createQuota: function(dialog) {
        var value = dialog.getQuota();
        return value != null ? new jscape.DirectoryQuota(value) : null;
    },

    _createEvents: function(dialog) {
        return new jscape.DirectoryMonitor.Event(dialog.getMonitorAdding(),
                                                 dialog.getMonitorChanging(),
                                                 dialog.getMonitorDeleting(),
                                                 dialog.getMaxFileAge());
    }
});

/** @class */
jscape.domain.DirectoryMonitorDialog = Class.extend(
    /** @lends {jscape.domain.DirectoryMonitor.prototype} */
{
    /** @const */
    TIMEOUT_FACTOR: 1000,
    LATENCY_FACTOR: 1000,
    QUOTA_FACTOR: 1024 * 1024,
    AGE_FACTOR: 24 * 60 * 60 * 1000,

    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=dirmonitorsfields]", this.dialog);

        this.path = $("input[name=path]", this.fieldset);
        this.path.validatebox({required: true, validType: ["dirmonitor_path_nonempty", "dirmonitor_filename"], missingMessage: resources.getString("DIR_MONITOR_ERROR_BAD_PATH")});
        this.browseButton = $("a[role=button][name=browse]", this.fieldset);
        this.browseButton.linkbutton({onClick: $.proxy(this._onClick$BrowseButton, this)});

        this.recursive = $("input[name=recursive]:checkbox", this.fieldset);

        this.timeoutEnabled = $("input[name=timeoutenabled]:checkbox", this.fieldset);
        this.timeoutEnabled.change($.proxy(function() { this._adjustSpinner(this.timeoutEnabled, this.timeout); }, this));
        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({min: 1, max: 3600, value: 600, increment: 600, required: true, disabled: true, width: 60});

        this.latencyEnabled = $("input[name=latencyenabled]:checkbox", this.fieldset);
        this.latencyEnabled.change($.proxy(function() { this._adjustSpinner(this.latencyEnabled, this.latency); }, this));
        this.latency = $("input[name=latency]", this.fieldset);
        this.latency.numberspinner({min: 1, max: 3600, value: 5, increment: 5, required: true, disabled: true, width: 60});

        this.ownedBy = $("input[name=ownedby]:checkbox", this.fieldset);
        this.ownedBy.change($.proxy(function() { this.owner.combobox(this.ownedBy.is(":checked:not(:disabled)") ? "enable" : "disable"); }, this));
        this.owner = $("select[name=owner]", this.fieldset);
        this.owner.combobox({
            editable: false,
            disabled: true,
            width: 120,
            loadFilter: function(data) {return $.map(data, function(value) { return {text: value, value: value}; })},
            onLoadSuccess: $.proxy(this._adjustComboDefaults, this, this.owner)
        }).combobox("autocomplete", true);

        this.quotaEnabled = $("input[name=quotaenabled]:checkbox", this.fieldset);
        this.quotaEnabled.change($.proxy(function() { this._adjustSpinner(this.quotaEnabled, this.quota); }, this));
        this.quota = $("input[name=quota]", this.fieldset);
        this.quota.numberspinner({min: 1, max: 1024 * 1024, value: 100, increment: 10, required: true, disabled: true, width: 90});

        this.eventAdd = $("input[name=monitoradd]:checkbox", this.fieldset);
        this.eventChange = $("input[name=monitorchange]:checkbox", this.fieldset);
        this.eventDelete = $("input[name=monitordelete]:checkbox", this.fieldset);
        this.eventAge = $("input[name=maxfileageenabled]:checkbox", this.fieldset);
        this.eventAge.change($.proxy(function() { this._adjustSpinner(this.eventAge, this.maxFileAge); }, this));
        this.maxFileAge = $("input[name=maxfileage]", this.fieldset);
        this.maxFileAge.numberspinner({min: 1, max: 10000, value: 10, required: true, disabled: true, width: 60});
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    getPath: function() {
        return this.path.val();
    },

    setPath: function(value) {
        this.path.val(value).focus();
    },

    getRecursive: function() {
        return this.recursive.is(":checked");
    },

    setRecursive: function(value) {
        this.recursive.prop("checked", !!value);
    },

    getTimeout: function() {
        return this.timeoutEnabled.is(":checked") ? parseInt(this.timeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR : null;
    },

    setTimeout: function(value) {
        this.timeoutEnabled.prop("checked", value != null).change();
        if (value != null) {
            this.timeout.numberspinner("setValue", value / this.TIMEOUT_FACTOR);
        }
    },

    getLatency: function() {
        return this.latencyEnabled.is(":checked") ? parseInt(this.latency.numberspinner("getValue")) * this.LATENCY_FACTOR : null;
    },

    setLatency: function(value) {
        this.latencyEnabled.prop("checked", value != null).change();
        if (value != null) {
            this.latency.numberspinner("setValue", value / this.LATENCY_FACTOR);
        }
    },

    getOwner: function () {
        return this.ownedBy.is(":checked:not(:disabled)") ? this.owner.combobox("getValue") : null;
    },

    setOwner: function (value) {
        var empty = this.owner.combobox("getData").length == 0;
        if (value != null && !empty) {
            this.owner.combobox("select", value);
        }
        this.ownedBy.prop("checked", value != null && !empty).change();
    },

    setOwners: function (values) {
        this.ownedBy.prop("disabled", values.length == 0);
        this.owner.combobox("loadData", values);
    },

    getQuota: function() {
        return this.quotaEnabled.is(":checked") ? parseInt(this.quota.numberspinner("getValue")) * this.QUOTA_FACTOR : null;
    },

    setQuota: function(value) {
        this.quotaEnabled.prop("checked", value != null).change();
        if (value != null) {
            this.quota.numberspinner("setValue", value.value / this.QUOTA_FACTOR);
        }
    },

    getMonitorAdding: function() {
        return this.eventAdd.is(":checked");
    },

    setMonitorAdding: function(value) {
        this.eventAdd.prop("checked", !!value);
    },

    getMonitorChanging: function() {
        return this.eventChange.is(":checked");
    },

    setMonitorChanging: function(value) {
        this.eventChange.prop("checked", !!value);
    },

    getMonitorDeleting: function() {
        return this.eventDelete.is(":checked");
    },

    setMonitorDeleting: function(value) {
        this.eventDelete.prop("checked", !!value);
    },

    getMaxFileAge: function() {
        return this.eventAge.is(":checked") ? this.maxFileAge.numberspinner("getValue") * this.AGE_FACTOR : null;
    },

    setMaxFileAge: function(value) {
        this.eventAge.prop("checked", value != null).change();
        if (value != null) {
            this.maxFileAge.numberspinner("setValue", value / this.AGE_FACTOR);
        }
    },

    _onClick$BrowseButton: function() {
        this.listener.onSelectPath(this);
    },

    _adjustComboDefaults: function (box, data) {
        if (data.length != 0) {
            box.combobox("setValue", data[0].value);
        } else {
            box.combobox("disable");
        }
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _adjustSpinner: function (checkbox, spinner) {
        if (checkbox.is(":checked")) {
            spinner.numberspinner("enable").validatebox("enableValidation");
        } else {
            spinner.numberspinner("disable").validatebox("disableValidation");
        }
    }
});
jscape.domain.DirectoryMonitorDialog.LISTENER = {
    onSubmit: $.noop,
    onCancel: $.noop,
    onSelectPath: $.noop
};

/** @class */
jscape.domain.AddDirectoryMonitorDialog = jscape.domain.DirectoryMonitorDialog.extend(
    /** @lends {jscape.domain.AddDirectoryMonitorDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-dirmonitors-add"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["dirmonitor_name_nonempty", "dirmonitor_unique"], missingMessage: resources.getString("DIR_MONITOR_ERROR_BAD_NAME")});
    },

    getName: function() {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditDirectoryMonitorDialog = jscape.domain.DirectoryMonitorDialog.extend(
    /** @lends {jscape.domain.EditDirectoryMonitorDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-dirmonitors-edit"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    setName: function(value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});

/** @class */
jscape.domain.DeleteDirectoryMonitorController = Class.extend(
    /** @lends {jscape.domain.DeleteDirectoryMonitorController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, monitors) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._confirmDelete(monitors)
                .done($.proxy(this._deleteConfirmed, this, monitors))
                .fail(this.deferred.reject);

        return this.deferred.promise();
    },

    _confirmDelete: function(monitors) {
        var text = $.map(monitors, function(monitor) { return monitor.name; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("DIR_MONITOR_CONFIRM_DELETE_TITLE"),
                this.resources.getString("DIR_MONITOR_CONFIRM_DELETE_MESSAGE", {0: text}));
    },

    _deleteConfirmed: function (monitors) {
        var promises = new Array(monitors.length);
        for (var i = 0; i < monitors.length; i++) {
            promises[i] = this.managementService.deleteDirectoryMonitor(this.domainName, monitors[i].name);
        }

        $.when.apply(jQuery, promises)
                .done($.proxy(function () { this.deferred.resolve(monitors); }, this))
                .fail(this.deferred.reject);
    }
});

/** @class */
jscape.domain.DirectoryMonitorsConfigurationController = Class.extend(
    /** @lends {jscape.domain.DirectoryMonitorsConfigurationController.prototype} */
{
    config: null,
    copy: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.DirectoryMonitorsConfigurationView(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.copy && !Object._equals(this.copy, this._createConfiguration());
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var config = this._createConfiguration();
        return this.managementService.setDirectoryMonitorsConfiguration(this.domainName, config)
                .done($.proxy(function() {
                    this.config = config;
                    this.copy = this._copyConfiguration();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.config = this.copy;
        this.copy = this._copyConfiguration();
        this._setupView();
    },

    _loadData: function() {
        this.managementService.getDirectoryMonitorsConfiguration(this.domainName)
                .done($.proxy(function(config) {
                    this.config = config;
                    this.copy = this._copyConfiguration();
                    this._setupView();
                }, this));
    },

    _createConfiguration: function () {
        var config = this._copyConfiguration();
        config.maxThreadCount = this.view.getMaxThreads();
        return config;
    },

    _copyConfiguration: function() {
        return $.extend(new jscape.DirectoryMonitorsConfiguration(), this.config);
    },

    _setupView: function() {
        this.view.setMaxThreads(this.config.maxThreadCount);
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.DirectoryMonitorsConfigurationView = Class.extend(
    /** @lends {jscape.domain.DirectoryMonitorsConfigurationView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=dirmonitorssettingsfields]");

        this.maxThreads = $("input[name=maxthreads]", this.fieldset);
        this.maxThreads.numberspinner({min: 10, max: 100, editable: false, required: true, width: 60});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getMaxThreads: function() {
        return parseInt(this.maxThreads.numberspinner("getValue"));
    },

    setMaxThreads: function(value) {
        if (parseInt(value)) {
            this.maxThreads.numberspinner("setValue", value);
        }
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.DirectoryMonitorsConfigurationView.LISTENER;
        this.fieldset.form("validate");
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard(this);
    }
});
jscape.domain.DirectoryMonitorsConfigurationView.LISTENER = {
    onApply: $.noop,
    onDiscard: $.noop
};