jscape.domain = jscape.domain || {};

/** @class */
jscape.DlpRule = Class.extend(
    /** @lends {jscape.DlpRule.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.name = a || "";
        this.description = b || "";
        this.scope = c == jscape.DlpRule.SCOPE.FILE_CONTENTS ? c : jscape.DlpRule.SCOPE.FILE_NAME;
        this.expression = d || ".*";
    }
});
/** @const */
jscape.DlpRule.SCOPE = {FILE_NAME: "FILE_NAME", FILE_CONTENTS: "FILE_CONTENTS"};
jscape.DlpRule.fromJSON = function(data) {
    return $.extend(new jscape.DlpRule(), data);
};

/** @class */
jscape.domain.DlpModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.DlpModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.DlpRulesController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onResume: function() {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.DlpRulesController = Class.extend(
    /** @lends {jscape.domain.DlpRulesController.prototype} */
{
    rules: null,

    /** @constructor */
    initialize: function(resources) {
        this.rules = [];

        this.view = new jscape.domain.DlpRulesView(resources);
        this.addController = new jscape.domain.AddDlpRuleController(resources);
        this.editController = new jscape.domain.EditDlpRuleController(resources);
        this.deleteController = new jscape.domain.DeleteDlpRuleController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onAddRule: function() {
        this.addController.start(this.managementService, this.domainName)
                .done($.proxy(this._ruleAdded, this));
    },

    onEditRule: function(rule) {
        var index = $.inArray(rule, this.rules);
        if (index != -1) {
            this.editController.start(this.managementService, this.domainName, rule)
                    .done($.proxy(this._ruleEdited, this, index));
        }
    },

    onDeleteRule: function(rules) {
        this.deleteController.start(this.managementService, this.domainName, rules)
                .done($.proxy(this._ruleDeleted, this, rules));
    },

    _loadData: function() {
        this.managementService.getDlpRules(this.domainName)
                .done($.proxy(function(rules) {
                    this.rules = rules || [];
                    this.view.setRules(this.rules);
                }, this));
    },

    _ruleAdded: function(rule) {
        this.rules.push(rule);
        this.view.setRules(this.rules);
        this.view.selectRule(rule);
    },

    _ruleEdited: function(index, rule) {
        this.rules[index] = rule;
        this.view.setRules(this.rules);
        this.view.selectRule(rule);
    },

    _ruleDeleted: function(rules) {
        var index = -1;
        while (rules.length) {
            index = $.inArray(rules.pop(), this.rules);
            this.rules.splice(index, 1);
        }
        this.view.setRules(this.rules);

        if (this.rules.length) {
            index = Math.max(0, Math.min(index, this.rules.length - 1));
            this.view.selectRule(this.rules[index]);
        }
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("dlp_name_nonempty", resources.getString("DLP_ERROR_BAD_NAME"));
        v.rule("dlp_rule_unique", $.proxy(this._ruleNotExists, this), resources.getString("DLP_ERROR_ALREADY_EXIST"));
        v.rule("dlp_expression", $.proxy(this._expressionValid, this), resources.getString("DLP_ERROR_BAD_EXPRESSION"));
    },

    _ruleNotExists: function (value, excludes) {
        return $.grep(this.rules, function (rule) {
            return rule.name == value && (excludes == undefined || $.inArray(rule.name, excludes) == -1);
        }).length == 0;
    },

    _expressionValid: function(value) {
        return value.trim().length != 0;
    }
});

/** @class */
jscape.domain.DlpRulesView = Class.extend(
    /** @lends {jscape.domain.DlpRulesView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources, container) {
        this.selection = [];
        this.resources = resources;

        this.fieldset = $("fieldset[name=dlprulesfields]", container);

        this.data = $("table[role=grid][aria-label=dlprules]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            singleSelect: true,
            pagination: true,
            search: true,
            remoteSort: false,
            sortName: "name",
            sortOrder: "asc",
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), width: 70, sortable: true},
                {field: "description", title: $("thead th:nth-child(2)", this.data).text(), width: 70},
                {field: "scope", title: $("thead th:nth-child(3)", this.data).text(), width: 40, resizable: false, formatter: $.proxy(this._formatScope, this)},
                {field: "expression", title: $("thead th:nth-child(4)", this.data).text(), resizable: true, width: 120}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustButtonsState, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this)
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.DlpRulesView.LISTENER;
        this._adjustButtonsState();
    },

    setRules: function(values) {
        this.selection = [];
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectRule: function (rule) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", rule));
    },

    _formatScope: function(value) {
        return value ? this.resources.getString("DLP_RULES_SCOPE_" + (value + "").toUpperCase()) : "";
    },

    _onClick$AddButton: function() {
        this.listener.onAddRule();
    },

    _onClick$EditButton: function() {
        this.listener.onEditRule(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteRule(this.selection);
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function () {
        this.editButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }
});
/** @const */
jscape.domain.DlpRulesView.LISTENER = {
    onAddRule: $.noop,
    onEditRule: $.noop,
    onDeleteRule: $.noop
};

/** @class */
jscape.domain.AddDlpRuleController = Class.extend(
    /** @lends {jscape.domain.AddDlpRuleController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.DlpRuleDialog(this.resources, true);
        dialog.setTitle(this.resources.getString("DLP_DIALOG_ADD_RULE_TITLE"));
        dialog.setScope(jscape.DlpRule.SCOPE.FILE_NAME);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var rule = this._createRule(dialog);
        dialog.destroy();

        this.managementService.storeDlpRule(this.domainName, rule)
                .done(this.deferred.resolve)
                .fail(this.deferred.reject);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createRule: function(dialog) {
        return new jscape.DlpRule(dialog.getName(),
                                  dialog.getDescription(),
                                  dialog.getScope(),
                                  dialog.getExpression());
    }
});

/** @class */
jscape.domain.EditDlpRuleController = Class.extend(
    /** @lends {jscape.domain.EditDlpRuleController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, rule) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.DlpRuleDialog(this.resources, false);
        dialog.setTitle(this.resources.getString("DLP_DIALOG_EDIT_RULE_TITLE").supplant({0: rule.name}));
        dialog.setName(rule.name);
        dialog.setDescription(rule.description);
        dialog.setScope(rule.scope);
        dialog.setExpression(rule.expression);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var rule = this._createRule(dialog);
        dialog.destroy();

        this.managementService.storeDlpRule(this.domainName, rule)
                .done(this.deferred.resolve)
                .fail(this.deferred.reject);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createRule: function(dialog) {
        return new jscape.DlpRule(dialog.getName(),
                                  dialog.getDescription(),
                                  dialog.getScope(),
                                  dialog.getExpression());
    }
});


/** @class */
jscape.domain.DeleteDlpRuleController = Class.extend(
    /** @lends {jscape.vfs.DeleteDlpRuleController.prototype} */
{
    rules: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function (managementService, domainName, rules) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.rules = rules || [];
        this.deferred = $.Deferred();

        this._confirmDelete()
                .done($.proxy(this._deleteConfirmed, this))
                .fail(this.deferred.reject);

        return this.deferred.promise();
    },

    _confirmDelete: function() {
        var text = $.map(this.rules, function(rule) { return rule.name; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("DLP_CONFIRM_DELETE_TITLE"),
                this.resources.getString("DLP_CONFIRM_DELETE_MESSAGE", {0: text}));
    },

    _deleteConfirmed: function() {
        var promises = new Array(this.rules.length);
        for (var i = 0; i < this.rules.length; i++) {
            promises[i] = this.managementService.deleteDlpRule(this.domainName, this.rules[i].name);
        }

        $.when.apply(jQuery, promises)
                .done($.proxy(function () { this.deferred.resolve(this.rules); }, this))
                .fail(this.deferred.reject);
    }
});

/** @class */
jscape.domain.DlpRuleDialog = Class.extend(
    /** @lends {jscape.domain.DlpRuleDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources, newRule) {
        this.dialog = $("#d-dlp-rule").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=dlprulefields]", this.dialog);

        this.name = $("input[name=name]", this.fieldset);
        if (newRule) {
            this.name.validatebox({
                required: true,
                validType: ["dlp_name_nonempty", "dlp_rule_unique"],
                missingMessage: resources.getString("DLP_ERROR_BAD_NAME")
            });
        } else {
            this.name.prop("disabled", true);
        }

        this.description = $("input[name=description]", this.fieldset);
        this.scope = $("input[name=scope]:radio", this.fieldset);

        this.expression = $("textarea[name=expression]", this.fieldset);
        this.expression.validatebox({
            required: true,
            validType: "dlp_expression",
            missingMessage: resources.getString("DLP_ERROR_BAD_EXPRESSION")
        });
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this.fieldset.form("validate");

        if (!this.name.prop("disabled")) {
            this.name.focus();
        } else {
            this.description.focus();
        }
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    setTitle: function(value) {
        this.dialog.dialog("setTitle", value);
    },

    getName: function() {
        return this.name.val();
    },

    setName: function(value) {
        this.name.val(value);
        if (!this.name.prop("disabled") && value && value.trim().length) {
            this.name.validatebox("options").validType = ["dlp_name_nonempty", "dlp_rule_unique['{0}']".supplant({0: value})];
        }
    },

    getDescription: function() {
        return this.description.val();
    },

    setDescription: function(value) {
        this.description.val(value);
    },

    getScope: function() {
        return this.scope.filter(":checked").val();
    },

    setScope: function(value) {
        this.scope.filter("[value=" + value + "]").prop("checked", true);
    },

    getExpression: function() {
        return this.expression.val();
    },

    setExpression: function(value) {
        this.expression.val(value);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});