jscape.domain = jscape.domain || {};

/** @class */
jscape.DomainDescriptor = Class.extend(
    /** @lends {jscape.DomainDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.name = a || "";
        this.description = b || "";
        this.serviceDescriptors = c || [];
        this.configurations = d || [];
    }
});
jscape.DomainDescriptor.fromJSON = function(data) {
    return data ? $.extend(new jscape.DomainDescriptor(), data) : null;
};

/** @class */
jscape.domain.DomainModule = jscape.ApplicationModule.extend(
    /** @lends {jscape.domain.DomainModule.prototype} */
{
    domainName: null,

    /** @override */
    onCreate: function(managementService, domainName) {
        this.domainName = domainName;
        this._super(managementService);
    },

    update: function(domainStatistics) {
    }
});

/** @class */
jscape.DomainController = jscape.Application.extend(
    /** @lends {jscape.domain.DomainController.prototype} */
{
    /** @constructor */
    initialize: function(view, resources, modules) {
        this._super(modules);

        this.view = view;
        this.resources = resources;
    },

    /** @override */
    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;

        this._super(this.managementService, this.domainName);
    },

    /** @override */
    onMenuItemSelected: function (item) {
        if (item && item.id) {
            switch (item.id) {
                case jscape.Menu.ITEM.Home:
                case jscape.Menu.ITEM.KeyManager:
                case jscape.Menu.ITEM.ServerSettings:
                    setTimeout($.proxy(function() { this.onRedirect(item.url || "/"); }, this), 10);
                    return true;
                case jscape.Menu.ITEM.Logout:
                    this.onLogout();
                    return true;
            }
        }
        return this._super(item);
    },

    /** @override */
    onReady: function() {
        this._super();

        this.view.show(this, this.domainName);
        this._updateDomainState();
        $.parser.parse();
    },

    onViewStateChanged: function(id, index, visible) {
        this._super.call(this, index, visible);
    },

    onStartDomain: function() {
        this.managementService.startDomain(this.domainName)
                .done($.proxy(this._updateDomainState, this));
    },

    onStopDomain: function() {
        this.managementService.stopDomain(this.domainName)
                .done($.proxy(this._updateDomainState, this));
    },

    onPauseDomain: function() {
        this.managementService.pauseDomain(this.domainName)
                .done($.proxy(this._updateDomainState, this));
    },

    onResumeDomain: function() {
        this.managementService.resumeDomain(this.domainName)
                .done($.proxy(this._updateDomainState, this));
    },

    onRedirect: function (url) {
        window.location = url;
    },

    onLogout: function () {
        this.managementService.logout();
    },

    _updateDomainState: function () {
        this.managementService.getDomainStatistics(this.domainName)
                .done($.proxy(function (statistics) {
                    this.view.setDomainState(jscape.DomainState.stateFor(statistics));
                    this._updateModules(statistics);
                }, this));
    },

    _updateModules: function(statistics) {
        for (var i = 0; i < this.modules.length; i++) {
            this.modules[i].update(statistics);
        }
    }
});

/** @class */
jscape.domain.DomainView = Class.extend(
    /** @lends {jscape.DomainView.prototype} */
{
    domainName: null,
    domainState: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.throbber = $("#throbber");
        this.panel = $("#viewpane");
        this.panel.layout({fit: true, border: false}).css({visibility: "hidden"});

        this.modules = $("#modules[role=tablist]");
        this.modules.tabs({
            fit: true,
            tabPosition: "left",
            onSelect: $.proxy(this._onSelect$Tab, this),
            onUnselect: $.proxy(this._onUnselect$Tab, this)
        });
        this.modules.find("div[role=tablist]")
                .filter(function() { return $(this).parents("div.dialog").length == 0; })
                .each(function(_, tablist) { $(tablist).tabs({border:false}); });
        this.modules.find(">div.tabs-header>div.tabs-wrap")
                .hover(
                    function () { $(this).css({overflowY: "auto", overflowX: "hidden"}); },
                    function () { $(this).css({overflowY: "", overflowX: ""}); }
                );

        this.startButton = $("a[role=button][name=startdomain]", this.panel);
        this.startButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$StartButton, this)});

        this.stopButton = $("a[role=button][name=stopdomain]", this.panel);
        this.stopButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$StopButton, this)});

        this.pauseButton = $("a[role=button][name=pausedomain]", this.panel);
        this.pauseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$PauseButton, this)});

        this.resumeButton = $("a[role=button][name=resumedomain]", this.panel);
        this.resumeButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ResumeButton, this)});
    },

    show: function(listener, domainName) {
        this.listener = listener || jscape.domain.DomainView.LISTENER;
        this.domainName = domainName || "";
        this.throbber.remove();
        this.throbber = null;
        this.panel.css({visibility: "visible"});
        this._adjustDomainState();
        this._adjustViewState();
    },

    setDomainState: function(state) {
        this.domainState = state;
        this._adjustDomainState();
        this._adjustButtonsState();
    },

    _onSelect$Tab: function(title, index) {
        if (this.listener) {
            this.listener.onViewStateChanged(title, index, true);
        }
    },

    _onUnselect$Tab: function(title, index) {
        if (this.listener) {
            this.listener.onViewStateChanged(title, index, false);
        }
    },

    _onClick$StartButton: function() {
        this.listener.onStartDomain();
    },

    _onClick$StopButton: function() {
        this.listener.onStopDomain();
    },

    _onClick$PauseButton: function() {
        this.listener.onPauseDomain();
    },

    _onClick$ResumeButton: function() {
        this.listener.onResumeDomain();
    },

    _adjustViewState: function() {
        var tab = this.modules.tabs("getSelected");
        var index = this.modules.tabs("getTabIndex", tab);
        this._onSelect$Tab(tab.title, index);
    },

    _adjustDomainState: function() {
        var state = this.domainState ? this.resources.getString("DOMAIN_STATE_" + this.domainState) : this.resources.getString("DOMAIN_MESSAGE_LOADING");
        var title = this.resources.getString("DOMAIN_TITLE", {0: this.domainName, 1: state});
        this.panel.layout("panel", "center").panel("setTitle", title);
        document.title = title;
    },

    _adjustButtonsState: function () {
        this.startButton.linkbutton(jscape.DomainState.STOPPED == this.domainState ? "enable" : "disable");
        this.stopButton.linkbutton(jscape.DomainState.STOPPED != this.domainState ? "enable" : "disable");
        this.pauseButton.linkbutton(jscape.DomainState.RUNNING == this.domainState ? "enable" : "disable");
        this.resumeButton.linkbutton(jscape.DomainState.PAUSED == this.domainState ? "enable" : "disable");
    }
});
jscape.domain.DomainView.LISTENER = {
    onStartDomain: $.noop,
    onStopDomain: $.noop,
    onPauseDomain: $.noop,
    onResumeDomain: $.noop,
    onViewStateChanged: $.noop
};