jscape.domain = jscape.domain || {};

/** @class */
jscape.DropZoneDescriptor = Class.extend(
    /** @lends {jscape.DropZoneDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e, f, g) {
        this.name = a || "";
        this.account = b || "";
        this.path = c || "";
        this.createDirectoryIfNotExists = d != undefined ? !!d : true;
        this.overwriteFileIfExists = e || false;
        this.theme = f || null;
        this.owner = g || null;
    }
});
jscape.DropZoneDescriptor.fromJSON = function(data) {
    return data ? $.extend(new jscape.DropZoneDescriptor(), data) : null;
};

/** @class */
jscape.domain.DropZonesModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.DropZonesModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.DropZonesController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onResume: function () {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.DropZonesController = Class.extend(
    /** @lends {jscape.domain.DropZonesController.prototype} */
{
    zones: null,

    /** @constructor */
    initialize: function(resources) {
        this.zones = [];

        this.view = new jscape.domain.DropZonesView(resources);
        this.addController = new jscape.domain.AddDropZoneController(resources);
        this.editController = new jscape.domain.EditDropZoneController(resources);
        this.deleteController = new jscape.domain.DeleteDropZoneController(resources);
        this.purgeController = new jscape.domain.PurgeDropZoneController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onAddZone: function() {
        this.addController.start(this.managementService, this.domainName)
                .then($.proxy(function(zone) { return this._loadDropZoneURIs([zone])[0]; }, this))
                .done($.proxy(this._zoneAdded, this));
    },

    onEditZone: function(zone) {
        var index = $.inArray(zone, this.zones);
        if (index != -1) {
            this.editController.start(this.managementService, this.domainName, zone)
                    .then($.proxy(function(zone) { return this._loadDropZoneURIs([zone])[0]; }, this))
                    .done($.proxy(this._zoneEdited, this, index));
        }
    },

    onDeleteZone: function(zones) {
        this.deleteController.start(this.managementService, this.domainName, zones)
                .done($.proxy(this._zoneDeleted, this, zones));
    },

    onPurgeZone: function(zone) {
        this.purgeController.start(this.managementService, this.domainName, zone);
    },

    _loadData: function() {
        this.managementService.getDropZones(this.domainName)
                .then($.proxy(this._loadDropZoneURIs, this))
                .done($.proxy(function(items) {
                    this.zones = items || [];
                    this.view.setZones(this.zones);
                }, this));
    },

    _loadDropZoneURIs: function(dropzones) {
        for (var i = 0; i < dropzones.length; i++) {
            var dropzone = dropzones[i];
            dropzone.uri = this.managementService.getDropZoneURI(this.domainName, dropzone.name);
        }
        return dropzones;
    },

    _zoneAdded: function(zone) {
        this.zones.push(zone);
        this.view.setZones(this.zones);
        this.view.selectZone(zone);
    },

    _zoneEdited: function(index, zone) {
        this.zones[index] = zone;
        this.view.setZones(this.zones);
        this.view.selectZone(zone);
    },

    _zoneDeleted: function (zones) {
        var index = -1;
        while (zones.length) {
            index = $.inArray(zones.pop(), this.zones);
            this.zones.splice(index, 1);
        }
        this.view.setZones(this.zones);

        if (this.zones.length) {
            index = Math.min(Math.max(0, index), this.zones.length - 1);
            this.view.selectZone(this.zones[index]);
        }
    },

    _initValidators: function (resources) {
        var v = new jscape.Validator();
        v.rule("dropzone_name", $.proxy(this._dropZoneNameValid, this), resources.getString("DROP_ZONE_ERROR_BAD_NAME"));
        v.rule("dropzone_unique", $.proxy(this._dropZoneNotExists, this), resources.getString("DROP_ZONE_ERROR_ALREADY_EXIST"));
        v.nonEmptyRule("dropzone_path_nonempty", resources.getString("DROP_ZONE_ERROR_BAD_PATH"));
    },

    _dropZoneNameValid: function(name) {
        return /^\w+$/g.test(name);
    },

    _dropZoneNotExists: function(name) {
        return $.grep(this.zones, function (zone) { return zone.name == name; }).length == 0;
    }
});

/** @class */
jscape.domain.DropZonesView = Class.extend(
    /** @lends {jscape.domain.DropZonesView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.selection = [];
        this.fieldset = $("fieldset[name=dropzonelistfields]");

        this.data = $("table[role=grid][aria-label=dropzones]", this.fieldset);
        this.data.datagrid({
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this),
            fitColumns: true,
            singleSelect: true,
            pagination: true,
            search: true,
            remoteSort: false,
            sortName: "name",
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 50},
                {field: "account", title: $("thead th:nth-child(2)", this.data).text(), sortable: true, width: 50},
                {field: "path", title: $("thead th:nth-child(3)", this.data).text(), width: 90},
                {field: "uri", title: $("thead th:nth-child(4)", this.data).text(), width: 90},
                {field: "owner", title: $("thead th:nth-child(5)", this.data).text(), sortable: true, width: 50, sorter: function(a, b) { return a == b ? 0 : a == null || a < b ? -1 : 1; }},
                {field: "theme", title: $("thead th:nth-child(6)", this.data).text(), width: 50}
            ]]
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});

        this.purgeButton = $("a[role=button][name=purge]", this.fieldset);
        this.purgeButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$PurgeButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.DropZonesView.LISTENER;
        this._adjustButtonsState();
    },

    setZones: function (values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectZone: function (value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    _onClick$AddButton: function() {
        this.listener.onAddZone();
    },

    _onClick$EditButton: function() {
        this.listener.onEditZone(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteZone(this.selection);
    },

    _onClick$PurgeButton: function() {
        this.listener.onPurgeZone(this.selection[0]);
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function () {
        var singleSelection = this.selection.length == 1;
        this.editButton.linkbutton(singleSelection ? "enable" : "disable");
        this.purgeButton.linkbutton(singleSelection ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }
});
jscape.domain.DropZonesView.LISTENER = {
    onAddZone: $.noop,
    onEditZone: $.noop,
    onDeleteZone: $.noop,
    onPurgeZone: $.noop
};

/** @class */
jscape.domain.AddDropZoneController = Class.extend(
    /** @lends {jscape.domain.AddDropZoneController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._setupDialog().show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var descriptor = this._createDescriptor(dialog);
        dialog.destroy();

        this.managementService.storeDropZone(this.domainName, descriptor)
                .done($.proxy(function() { this.deferred.resolve(descriptor); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this))
                .always($.proxy(function() { this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    dropZoneUriFor: function(zoneName) {
        return zoneName && zoneName.trim().length
                ? this.managementService.getDropZoneURI(this.domainName, zoneName) || ""
                : "";
    },

    _setupDialog: function() {
        var dialog = new jscape.domain.AddDropZoneDialog(this.resources);
        dialog.setCreatePathIfNotExists(true);
        dialog.setOverwriteFileIfExists(true);

        $.when(this.managementService.getAccounts(this.domainName),
               this.managementService.getWebThemes(this.domainName))
                .done($.proxy(function (accounts, themes) {
                    var users = $.map(accounts, function (account) { return account.login; });
                    var owners = $.map($.grep(accounts, function (account) { return account.administrator; }), function (account) { return account.login; });
                    var themeNames = $.map(themes, function (theme) { return theme.name; });
                    dialog.setUsers(users.sort());
                    dialog.setOwners(owners.sort());
                    dialog.setWebThemes(themeNames.sort());
                }, this));
        return dialog;
    },

    _createDescriptor: function(dialog) {
        return new jscape.DropZoneDescriptor(dialog.getName(),
                                             dialog.getUser(),
                                             dialog.getPath(),
                                             dialog.getCreatePathIfNotExists(),
                                             dialog.getOverwriteFileIfExists(),
                                             dialog.getWebTheme(),
                                             dialog.getOwner(),
                                             dialog.getUri());
    }
});

/** @class */
jscape.domain.EditDropZoneController = Class.extend(
    /** @lends {jscape.domain.EditDropZoneController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function (resources) {
        this.resources = resources;
    },

    start: function (managementService, domainName, descriptor) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.zoneName = descriptor.name;
        this.deferred = $.Deferred();

        this._setupDialog(descriptor).show(this);

        return this.deferred.promise();
    },

    onSubmit: function (dialog) {
        var descriptor = this._createDescriptor(dialog);
        dialog.destroy();

        this.managementService.storeDropZone(this.domainName, descriptor)
                .done($.proxy(function() { this.deferred.resolve(descriptor); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this))
                .always($.proxy(function() { this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function(descriptor) {
        var dialog = new jscape.domain.EditDropZoneDialog(this.resources);
        dialog.setName(descriptor.name);
        dialog.setPath(descriptor.path);
        dialog.setCreatePathIfNotExists(descriptor.createDirectoryIfNotExists);
        dialog.setOverwriteFileIfExists(descriptor.overwriteFileIfExists);
        dialog.setUri(descriptor.uri);

        $.when(this.managementService.getAccounts(this.domainName),
               this.managementService.getWebThemes(this.domainName))
                .done(function (accounts, themes) {
                    var users = $.map(accounts, function (account) { return account.login; });
                    var owners = $.map($.grep(accounts, function (account) { return account.administrator; }), function (account) { return account.login; });
                    var themeNames = $.map(themes, function (theme) { return theme.name; });
                    dialog.setUsers(users.sort());
                    dialog.setUser(descriptor.account);
                    dialog.setWebThemes(themeNames.sort());
                    dialog.setWebTheme(descriptor.theme);
                    dialog.setOwners(owners.sort());
                    dialog.setOwner(descriptor.owner);
                });
        return dialog;
    },

    _createDescriptor: function(dialog) {
        return new jscape.DropZoneDescriptor(this.zoneName,
                                             dialog.getUser(),
                                             dialog.getPath(),
                                             dialog.getCreatePathIfNotExists(),
                                             dialog.getOverwriteFileIfExists(),
                                             dialog.getWebTheme(),
                                             dialog.getOwner(),
                                             dialog.getUri());
    }
});

/** @class */
jscape.domain.DropZoneDialog = Class.extend(
    /** @lends {jscape.domain.DropZoneDialog.prototype} */
{
    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=dropzonefields]", this.dialog);

        this.path = $("input[name=path]", this.fieldset);
        this.path.validatebox({required: true, validType: "dropzone_path_nonempty", missingMessage: resources.getString("DROP_ZONE_ERROR_BAD_PATH")});

        this.user = $("select[name=account]", this.fieldset);
        this.user.combobox({required: true, editable: false, missingMessage: resources.getString("DROP_ZONE_ERROR_BAD_ACCOUNT"), width: 120});
        this.user.combobox("autocomplete", true);

        this.ownedBy = $("input[name=ownedby]:checkbox", this.fieldset);
        this.ownedBy.change($.proxy(function() {
            this.owner.combobox(this.ownedBy.is(":checked") ? "enable" : "disable");
        }, this));
        this.owner = $("select[name=owner]", this.fieldset);
        this.owner.combobox({
            editable: false,
            disabled: true,
            width: 120,
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.owner.combobox("select", data[0].value);
                } else {
                    this.ownedBy.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        }).combobox("autocomplete", true);

        this.brandedBy = $("input[name=brandedby]:checkbox", this.fieldset);
        this.brandedBy.change($.proxy(function() {
            this.theme.combobox(this.brandedBy.is(":checked") ? "enable" : "disable");
        }, this));
        this.theme = $("select[name=webtheme]", this.fieldset);
        this.theme.combobox({
            editable: false, disabled: true,
            width: 120, panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.theme.combobox("select", data[0].value);
                } else {
                    this.brandedBy.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        }).combobox("autocomplete", true);

        this.createPath = $("input[name=createpath]:checkbox", this.fieldset);
        this.overwrite = $("input[name=overwrite]:checkbox", this.fieldset);
        this.uri = $("span[aria-label=zoneuri]", this.fieldset);
    },

    getPath: function() {
        return this.path.val();
    },

    setPath: function(value) {
        this.path.val(value);
    },

    getUser: function() {
        return this.user.combobox("getValue");
    },

    setUser: function(value) {
        this.user.combobox("select", value);
    },

    setUsers: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.user.combobox("loadData", data);
    },

    getOwner: function () {
        return this.ownedBy.is(":checked:not(:disabled)") ? this.owner.combobox("getValue") : null;
    },

    setOwner: function (value) {
        this.ownedBy.prop("checked", value != null).change();
        if (value != null) {
            this.owner.combobox("select", value);
        }
    },

    setOwners: function (values) {
        var data = $.map(values, function(value) { return {text: value, value: value}; });
        this.owner.combobox("loadData", data);
    },

    getWebTheme: function () {
        return this.brandedBy.is(":checked:not(:disabled)") ? this.theme.combobox("getValue") : null;
    },

    setWebTheme: function (value) {
        this.brandedBy.prop("checked", value != null).change();
        if (value != null) {
            this.theme.combobox("select", value);
        }
    },

    setWebThemes: function (values) {
        var data = $.map((values || []).sort(), function(value) { return {text: value, value: value}; });
        this.theme.combobox("loadData", data);
    },

    getCreatePathIfNotExists: function() {
        return this.createPath.is(":checked");
    },

    setCreatePathIfNotExists: function(value) {
        this.createPath.prop("checked", !!value);
    },

    getOverwriteFileIfExists: function() {
        return this.overwrite.is(":checked");
    },

    setOverwriteFileIfExists: function(value) {
        this.overwrite.prop("checked", !!value);
    },

    getUri: function() {
        return this.uri.text();
    },

    setUri: function(value) {
        this.uri.text(value || "");
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.DropZoneDialog.LISTENER = {
    onSubmit: $.noop,
    onCancel: $.noop
};

/** @class */
jscape.domain.AddDropZoneDialog = jscape.domain.DropZoneDialog.extend(
    /** @lends {jscape.domain.AddDropZoneDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-dropzone-add"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["dropzone_name", "dropzone_unique"], missingMessage: resources.getString("DROP_ZONE_ERROR_BAD_NAME")});
        this.name.change($.proxy(this._adjustZoneUri, this));
    },

    getName: function() {
        return this.name.val();
    },

    /** @override */
    show: function (listener) {
        this._super(listener || jscape.domain.AddDropZoneDialog.LISTENER);
    },

    _adjustZoneUri: function() {
        this.setUri(this.name.validatebox("isValid") ? this.listener.dropZoneUriFor(this.getName()) : null);
    }
});
jscape.domain.AddDropZoneDialog.LISTENER = $.extend({
    dropZoneUriFor: $.noop
}, jscape.domain.DropZoneDialog.LISTENER);


/** @class */
jscape.domain.EditDropZoneDialog = jscape.domain.DropZoneDialog.extend(
    /** @lends {jscape.domain.EditDropZoneDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-dropzone-edit"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    setName: function(value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});


/** @class */
jscape.domain.DeleteDropZoneController = Class.extend(
    /** @lends {jscape.domain.DeleteDropZoneController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, items) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._confirmDelete(items)
                .done($.proxy(this._deleteConfirmed, this, items))
                .fail($.proxy(function() { this.deferred.reject(); this.deferred = null; }, this));

        return this.deferred.promise();
    },

    _confirmDelete: function(items) {
        var text = $.map(items, function(dropzone) { return dropzone.name; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("DROP_ZONE_CONFIRM_DELETE_TITLE"),
                this.resources.getString("DROP_ZONE_CONFIRM_DELETE_MESSAGE", {0: text}));
    },

    _deleteConfirmed: function (dropzones) {
        var promises = [];
        for (var i = 0; i < dropzones.length; i++) {
            promises.push(this.managementService.deleteDropZone(this.domainName, dropzones[i].name));
        }

        $.when.apply(jQuery, promises)
                .done($.proxy(function () { this.deferred.resolve(dropzones); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this))
                .always($.proxy(function() { this.deferred = null; }, this));
    }
});

/** @class */
jscape.domain.PurgeDropZoneController = Class.extend(
    /** @lends {jscape.domain.PurgeDropZoneController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, descriptor) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._confirmPurge(descriptor.name)
                .done($.proxy(this._purgeConfirmed, this, descriptor.name))
                .fail($.proxy(function() { this.deferred.reject(); }, this));

        return this.deferred.promise();
    },

    _confirmPurge: function(zoneName) {
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("DROP_ZONE_CONFIRM_PURGE_TITLE"),
                this.resources.getString("DROP_ZONE_CONFIRM_PURGE_MESSAGE", {0: zoneName}));
    },

    _purgeConfirmed: function (zoneName) {
        this.managementService.purgeDropZone(this.domainName, zoneName)
                .done($.proxy(function() { this.deferred.resolve(zoneName); } , this))
                .fail($.proxy(function() { this.deferred.reject(); }, this));
    }
});