jscape.domain = jscape.domain || {};

/** @class */
jscape.Group = Class.extend(
    /** @lends {jscape.Group.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.name = a || "";
        this.resources = b || [];
    }
});
jscape.Group.fromJSON = function(data) {
    return data ? new jscape.Group(data.name, $.map(data.resources || [], jscape.VirtualFile.fromJSON)) : null;
};

/** @class */
jscape.domain.GroupsModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.GroupsModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.GroupsController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onResume: function() {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.GroupsController = Class.extend(
    /** @lends {jscape.domain.GroupsController.prototype} */
{
    groups: null,

    /** @constructor */
    initialize: function(resources) {
        this.groups = [];

        this.view = new jscape.domain.GroupsView(resources);
        this.addController = new jscape.domain.AddGroupController(resources);
        this.editController = new jscape.domain.EditGroupController(resources);
        this.copyController = new jscape.domain.CopyGroupController(resources);
        this.usersController = new jscape.domain.GroupUsersController(resources);
        this.deleteController = new jscape.domain.DeleteGroupController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onAddGroup: function() {
        this.addController.start(this.managementService, this.domainName)
                .done($.proxy(this._groupAdded, this));
    },

    onCopyGroup: function(group) {
        this.copyController.start(this.managementService, this.domainName, group.name)
                .done($.proxy(this._groupAdded, this));
    },

    onEditGroup: function(group) {
        var index = $.inArray(group, this.groups);
        if (index != -1) {
            this.editController.start(this.managementService, this.domainName, group)
                    .done($.proxy(this._groupEdited, this, index));
        }
    },

    onDeleteGroup: function(groups) {
        this.deleteController.start(this.managementService, this.domainName, groups)
                .done($.proxy(this._groupDeleted, this, groups));
    },

    onGroupAccounts: function(group) {
        this.usersController.start(this.managementService, this.domainName, group.name);
    },

    _groupAdded: function(group) {
        this.groups.push(group);
        this.view.setGroups(this.groups);
        this.view.selectGroup(group);
    },

    _groupEdited: function(index, group) {
        this.groups[index] = group;
        this.view.setGroups(this.groups);
        this.view.selectGroup(group);
    },

    _groupDeleted: function (groups) {
        var index = -1;
        while (groups.length) {
            index = $.inArray(groups.pop(), this.groups);
            this.groups.splice(index, 1);
        }
        this.view.setGroups(this.groups);

        if (this.groups.length) {
            index = Math.min(Math.max(0, index), this.groups.length - 1);
            this.view.selectGroup(this.groups[index]);
        }
    },

    _loadData: function() {
        this.managementService.getGroups(this.domainName)
                .done($.proxy(function(groups) {
                    this.groups = groups || [];
                    this.view.setGroups(this.groups);
                }, this));
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();

        v.nonEmptyRule("group_name_nonempty", resources.getString("GROUPS_ERROR_BAD_NAME"));
        v.rule("group_unique", $.proxy(this._groupNotExists, this), resources.getString("GROUPS_ERROR_ALREADY_EXIST"));
        v.nonEmptyRule("group_path_nonempty", resources.getString("GROUPS_ERROR_BAD_PATH"));
        v.nonEmptyRule("group_realpath_nonempty", resources.getString("GROUPS_ERROR_BAD_REAL_PATH"));
    },

    _groupNotExists: function(groupName) {
        groupName = (groupName || "").toUpperCase();
        for (var i = 0; i < this.groups.length; ++i) {
            if (this.groups[i].name.toUpperCase() === groupName) {
                return false;
            }
        }
        return true;
    }
});

/** @class */
jscape.domain.GroupsView = Class.extend(
    /** @lends {jscape.domain.GroupsView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.selection = [];
        this.resources = resources;

        this.fieldset = $("fieldset[name=groupsfields]");

        this.data = $("table[role=grid][aria-label=groups]", this.fieldset);
        this.data.datagrid({
            singleSelect: true,
            remoteSort: false,
            fitColumns: true,
            pagination: true,
            search: true,
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 100}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this)
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.copyButton = $("a[role=button][name=copy]", this.fieldset);
        this.copyButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$CopyButton, this)});

        this.usersButton = $("a[role=button][name=users]", this.fieldset);
        this.usersButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$UsersButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.GroupsView.LISTENER;
        this._adjustButtonsState();
    },

    setGroups: function(values) {
        this.selection = [];
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectGroup: function(value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    _onClick$AddButton: function () {
        this.listener.onAddGroup();
    },

    _onClick$EditButton: function () {
        this.listener.onEditGroup(this.selection[0]);
    },

    _onClick$DeleteButton: function () {
        this.listener.onDeleteGroup(this.selection);
    },

    _onClick$CopyButton: function () {
        this.listener.onCopyGroup(this.selection[0]);
    },

    _onClick$UsersButton: function () {
        this.listener.onGroupAccounts(this.selection[0]);
    },

    _adjustRowSelection: function () {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var singleSelection = this.selection.length == 1;
        this.editButton.linkbutton(singleSelection ? "enable" : "disable");
        this.copyButton.linkbutton(singleSelection ? "enable" : "disable");
        this.usersButton.linkbutton(singleSelection ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }
});
jscape.domain.GroupsView.LISTENER = {
    onAddGroup: $.noop,
    onEditGroup: $.noop,
    onCopyGroup: $.noop,
    onDeleteGroup: $.noop,
    onGroupAccounts: $.noop
};

/** @class */
jscape.domain.AddGroupController = Class.extend(
    /** @lends {jscape.domain.AddGroupController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.AddGroupDialog(this.resources);
        dialog.show(this);

        this.managementService.getReverseProxies(this.domainName).done(function (items) {
            var names = $.map(items, function (proxy) { return proxy.name; });
            dialog.setReverseProxies(names);
        });

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var group = this._createGroup(dialog);
        dialog.destroy();

        this.managementService.storeGroup(this.domainName, group)
                .done(this.deferred.resolve)
                .fail(this.deferred.reject);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    onSelectPath: function(dialog) {
        var loadFn = $.proxy(this.managementService.listFileSystem, this.managementService);
        var createFn = $.proxy(this.managementService.createFileSystemDir, this.managementService);
        var c = new SelectPathController(loadFn, createFn);
        c.selectDirectory($.proxy(function(entry) { dialog.setRealPath(entry.path); }, this));
    },

    _createGroup: function(dialog) {
        var permissions = new jscape.AccessPermissions();
        var file = dialog.getReverseProxy() != null
                   ? jscape.VirtualFile.remote(dialog.getPath(), permissions, false, false, dialog.getReverseProxy())
                   : jscape.VirtualFile.local(dialog.getPath(), permissions, false, false, true, dialog.getRealPath(), true);

        return new jscape.Group(dialog.getGroupName(), [file]);
    }
});

/** @class */
jscape.domain.AddGroupDialog = Class.extend(
    /** @lends {jscape.domain.AddGroupDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-group-add").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 570, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function() {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=addgroupfields]", this.dialog);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["group_name_nonempty", "group_unique"], missingMessage: resources.getString("GROUPS_ERROR_BAD_NAME")});

        this.path = $("input[name=path]", this.fieldset);
        this.path.validatebox({required: true, validType: "group_path_nonempty", missingMessage: resources.getString("GROUPS_ERROR_BAD_PATH")});

        this.type = $("input[name=type]:radio", this.fieldset);
        this.typeRealPath = this.type.filter("[value=realpath]");
        this.typeReverseProxy = this.type.filter("[value=reverseproxy]");
        this.typeRealPath.on("change", $.proxy(this._adjustFieldState, this));
        this.typeReverseProxy.on("change", $.proxy(this._adjustFieldState, this));

        this.realPath = $("input[name=realpath]", this.fieldset);
        this.realPath.validatebox({required: true, validType: "group_realpath_nonempty", missingMessage: resources.getString("GROUPS_ERROR_BAD_REAL_PATH")});

        this.browseButton = $("a[role=button][name=browse]", this.fieldset);
        this.browseButton.linkbutton({onClick: $.proxy(this._onClick$BrowseButton, this)});

        this.reverseProxy = $("select[name=reverseproxy]", this.fieldset);
        this.reverseProxy.combobox({
            required: true, editable: false, disabled: true, missingMessage: resources.getString("GROUPS_ERROR_BAD_REVERSE_PROXY"), width: 120, panelHeight: "auto",
            onLoadSuccess: function() {
                var data = $(this).combobox("getData");
                var value = $(this).combobox("getValue") || (data.length && data[0].value);
                value && $(this).combobox("select", value);
            }
        }).combobox("autocomplete", true);
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.AddGroupDialog.LISTENER;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this.typeRealPath.prop("checked", true);
        this.setReverseProxies(null);
        this.fieldset.form("validate");
        this.name.focus();
    },

    hide: function() {
        if (!this._closed()) {
            this.closing = true;
            this.fieldset.form("disableValidation");
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    getGroupName: function() {
        return this.name.val();
    },

    getPath: function() {
        return this.path.val();
    },

    getRealPath: function() {
        return this.typeRealPath.is(":checked") ? this.realPath.val() : null;
    },

    setRealPath: function(value) {
        this.typeRealPath.prop("checked", true).change();
        this.realPath.val(value).focus();
    },

    setReverseProxies: function(values) {
        if (values && values.length) {
            values = $.map(values, function(value) { return {text: value, value: value}; });
            this.reverseProxy.combobox("loadData", values);
            this.typeReverseProxy.prop("disabled", false);
        } else {
            this.reverseProxy.combobox("disable").combobox("disableValidation");
            this.typeReverseProxy.prop("disabled", true);
        }
    },

    getReverseProxy: function() {
        return this.typeReverseProxy.is(":checked") ? this.reverseProxy.combobox("getValue") : null;
    },

    _onClick$BrowseButton: function() {
        this.listener.onSelectPath(this);
    },

    _onSubmit: function() {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function() {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function() {
        return !!this.dialog.dialog("options").closed;
    },

    _adjustFieldState: function() {
        if (this.typeRealPath.is(":checked")) {
            this.realPath.prop("disabled", false).validatebox("enableValidation").validatebox("isValid");
            this.browseButton.linkbutton("enable");
            this.reverseProxy.combobox("disable").combobox("disableValidation");
        } else if (this.typeReverseProxy.is(":checked")) {
            this.realPath.prop("disabled", true).validatebox("disableValidation");
            this.browseButton.linkbutton("disable");
            this.reverseProxy.combobox("enable").combobox("enableValidation").combobox("isValid");
        }
    }
});
jscape.domain.AddGroupDialog.LISTENER = {
    onSelectPath: $.noop,
    onSubmit: $.noop,
    onCancel: $.noop
};

/** @class */
jscape.domain.EditGroupController = Class.extend(
    /** @lends {jscape.domain.EditGroupController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function (resources) {
        this.resources = resources;
        this.vfsController = null;
    },

    start: function (managementService, domainName, group) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.group = group;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.EditGroupDialog(this.resources);
        dialog.setGroupName(this.group.name);

        this.vfsController = new jscape.vfs.VirtualFilesController(this.resources, dialog.container());
        this.vfsController.start(this.managementService, this.domainName, this.group.resources);

        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function (dialog) {
        var group = new jscape.Group(this.group.name, this.vfsController.getDescriptors());
        dialog.destroy();

        this.managementService.storeGroup(this.domainName, group)
                .done(this.deferred.resolve)
                .fail(this.deferred.reject);
    },

    onCancel: function (dialog) {
        dialog.destroy();
        this.deferred.reject();
    }
});

/** @class */
jscape.domain.EditGroupDialog = Class.extend(
    /** @lends {jscape.domain.EditGroupDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-group-edit").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.title = this.dialog.dialog("header").text() || "";
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    container: function() {
        return this.dialog;
    },

    setGroupName: function(name) {
        this.dialog.dialog("setTitle", this.title.supplant({0: name}));
    },

    _onSubmit: function () {
        this.hide();
        this.listener.onSubmit(this);
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.CopyGroupController = Class.extend(
    /** @lends {jscape.domain.CopyGroupController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, groupName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.groupName = groupName;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.CopyGroupDialog(this.resources);
        dialog.setCopyGroup(groupName);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var targetGroupName = dialog.getGroupName();
        dialog.destroy();

        this.managementService.copyGroup(this.domainName, this.groupName, targetGroupName)
                .done(this.deferred.resolve)
                .fail(this.deferred.reject)
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    }
});

/** @class */
jscape.domain.CopyGroupDialog = Class.extend(
    /** @lends {jscape.domain.CopyGroupDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-group-copy").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.title = this.dialog.dialog("header").text() || "";

        this.fieldset = $("fieldset[name=copygroupfields]", this.dialog);

        this.name = $("[name=name]", this.fieldset);
        this.name.validatebox({
            required: true,
            validType: ["group_name_nonempty", "group_unique"],
            missingMessage: resources.getString("GROUPS_ERROR_BAD_NAME")
        });
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this.fieldset.form("enableValidation");
        this.fieldset.form("validate");
        this.name.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    getGroupName: function() {
        return this.name.val();
    },

    setCopyGroup: function(value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.DeleteGroupController = Class.extend(
    /** @lends {jscape.domain.DeleteGroupController.prototype} */
{
    groups: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, groups) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.groups = groups || [];
        this.deferred = $.Deferred();

        this._showConfirmDelete().done($.proxy(this._deleteConfirmed, this)).fail(this.deferred.reject);

        return this.deferred.promise();
    },

    _showConfirmDelete: function() {
        var text = $.map(this.groups, function(group) { return group.name; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("GROUPS_CONFIRM_DELETE_TITLE"),
                this.resources.getString("GROUPS_CONFIRM_DELETE_MESSAGE", {0: text}));
    },

    _deleteConfirmed: function () {
        var promises = new Array(this.groups.length);
        for (var i = 0; i < this.groups.length; i++) {
            promises[i] = this.managementService.deleteGroup(this.domainName, this.groups[i].name);
        }

        $.when.apply(jQuery, promises)
                .done($.proxy(function () { this.deferred.resolve(this.groups); }, this))
                .fail(this.deferred.reject);
    }
});

/** @class */
jscape.domain.GroupUsersController = Class.extend(
    /** @lends {jscape.domain.GroupUsersController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, groupName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.groupName = groupName;
        this.deferred = $.Deferred();

        this._setupDialog().show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var users = dialog.getGroupUsers();
        dialog.destroy();

        this.managementService.setGroupUsers(this.domainName, this.groupName, users)
                .done(this.deferred.resolve)
                .fail(this.deferred.reject);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _setupDialog: function() {
        var dialog = new jscape.domain.GroupUsersDialog(this.resources);
        dialog.setGroupName(this.groupName);

        $.when(this.managementService.getGroupUsers(this.domainName, this.groupName),
               this.managementService.getAccounts(this.domainName))
                .done(function(groupUsers, accounts) {
                        var users = $.map(accounts, function (account) { return account.login; });
                        dialog.setUsers(users.sort());
                        dialog.setGroupUsers(groupUsers);
                })
                .fail(this.deferred.reject);

        return dialog;
    }
});

/** @class */
jscape.domain.GroupUsersDialog = Class.extend(
    /** @lends {jscape.domain.GroupUsersDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-group-users").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=groupusersfields]", this.dialog);
        this.data = $("table[role=grid][aria-label=users]", this.fieldset);
        this.data.datagrid({
            fitColumns:true,
            singleSelect: false,
            checkOnSelect: true,
            selectOnCheck: true,
            remoteSort: false,
            height: 300,
            columns: [[
                {field: "checkbox", checkbox: true},
                {field: "value", sortable: true, width: 100, formatter: function (_, row) {return (row || "").escapeXml(); }}
            ]]
        });

        this.title = this.dialog.dialog("header").text();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this.fieldset.form("validate");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    setGroupName: function (name) {
        this.dialog.dialog("setTitle", this.title.supplant({0: name}));
    },

    setUsers: function (values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    getGroupUsers: function () {
        return this.data.datagrid("getChecked");
    },

    setGroupUsers: function (values) {
        if (values && values.length) {
            var rows = this.data.datagrid("getRows");
            for (var i = 0; i < values.length; i++) {
                var index = $.inArray(values[i], rows);
                this.data.datagrid("checkRow", index);
            }
        } else {
            this.data.datagrid("clearChecked");
        }
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});