jscape.domain = jscape.domain || {};

/** @class */
jscape.Report = Class.extend(
    /** @lends {jscape.Report.prototype} */
{
    /** @constructor */
    initialize: function (a, b, c, d, e, f, g, h) {
        this.name = a || "";
        this.date = parseInt(b) || 0;
        this.description = c || "";
        this.searchName = d || null;
        this.reRunSearch = e !== undefined ? !!e : false;
        this.metricses = f || [];
        this.progress = parseInt(g) || 0;
        this.errorMessage = h || null;
    },

    succeed: function() {
        return this.progress === 100 && !this.failed();
    },

    failed: function() {
        return this.errorMessage && this.errorMessage.trim().length != 0;
    }
});
jscape.Report.fromJSON = function(data) {
    return data ? $.extend(new jscape.Report(), data) : null;
};

/** @class */
jscape.domain.ReportsModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.ReportsModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.ReportsController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onResume: function() {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.ReportsController = Class.extend(
    /** @lends {jscape.domain.ReportsController.prototype} */
{
    reports: null,

    /** @constructor */
    initialize: function(resources) {
        this.reports = [];

        this.view = new jscape.domain.ReportsView(resources);
        this.addController = new jscape.domain.AddReportController(resources);
        this.editController = new jscape.domain.EditReportController(resources);
        this.deleteController = new jscape.domain.DeleteReportController(resources);
        this.viewController = new jscape.domain.ViewReportController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onRefresh: function() {
        this._loadData();
    },

    onAddReport: function() {
        this.addController.start(this.managementService, this.domainName)
                .done($.proxy(this._reportAdded, this));
    },

    onEditReport: function(report) {
        var index = $.inArray(report, this.reports);
        if (index != -1) {
            this.editController.start(this.managementService, this.domainName, report)
                    .done($.proxy(this._reportEdited, this, index));
        }
    },

    onDeleteReport: function(reports) {
        this.deleteController.start(this.managementService, this.domainName, reports)
                .done($.proxy(this._reportDeleted, this, reports));
    },

    onViewReport: function(report) {
        this.viewController.start(report);
    },

    onReRunReport: function(report) {
        var index = $.inArray(report, this.reports);
        if (index != -1) {
            report.progress = 0;
            report.errorMessage = null;
            this.managementService.reRunReport(this.domainName, report.name)
                    .done($.proxy(this._reportEdited, this, index));
        }
    },

    _reportAdded: function(report) {
        this.reports.push(report);
        this.view.setReports(this.reports);
        this.view.selectItem(report);
    },

    _reportEdited: function (index, report) {
        this.reports[index] = report;
        this.view.setReports(this.reports);
        this.view.selectItem(report);
    },

    _reportDeleted: function (reports) {
        var index = -1;
        while (reports.length) {
            index = $.inArray(reports.pop(), this.reports);
            this.reports.splice(index, 1);
        }
        this.view.setReports(this.reports);

        if (this.reports.length) {
            index = Math.max(0, Math.min(index, this.reports.length - 1));
            this.view.selectItem(this.reports[index]);
        }
    },

    _loadData: function() {
        this.managementService.getReports(this.domainName)
                .done($.proxy(function(reports) {
                    this.reports = reports || [];
                    this.view.setReports(this.reports);
                }, this));
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("report_name_nonempty", resources.getString("REPORTS_ERROR_BAD_NAME"));
        v.rule("report_unique", $.proxy(this._reportNotExists, this), resources.getString("REPORTS_ERROR_ALREDY_EXISTS"));
    },

    _reportNotExists: function(value) {
        return $.grep(this.reports, function(report) { return report.name == value; }).length == 0;
    }
});


/** @class */
jscape.domain.ReportsView = Class.extend(
    /** @lends {jscape.domain.ReportsView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.selection = [];
        this.resources = resources;
        this.dateFormat = new jscape.DateFormat();

        this.fieldset = $("fieldset[name=reportsfields]");

        this.data = $("table[role=grid][aria-label=reports]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            singleSelect: true,
            remoteSort: false,
            pagination: true,
            search: true,
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 100},
                {field: "date", title: $("thead th:nth-child(2)", this.data).text(), sortable: true, width: 150, align: "right", formatter: function(value) { return new Date(value).toLocaleString(); }},
                {field: "description", title: $("thead th:nth-child(3)", this.data).text(), sortable: false, width: 250},
                {field: "searchName", title: $("thead th:nth-child(4)", this.data).text(), sortable: true, width: 150, sorter: function(a, b) { return a == b ? 0 : (a || "" > b || "" ? 1 : -1); }},
                {field: "status", title: $("thead th:nth-child(5)", this.data).text(), sortable: false, width: 70, formatter: $.proxy(function(_, row) { return this._formatStatus(row); }, this), styler: $.proxy(function(_, row) { return this._formatStatusStyle(row); }, this)}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$ViewButton, this)
        });

        this.refreshButton = $("a[role=button][name=refresh]", this.fieldset);
        this.refreshButton.linkbutton({onClick: $.proxy(this._onClick$RefreshButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});

        this.viewButton = $("a[role=button][name=view]", this.fieldset);
        this.viewButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ViewButton, this)});

        this.runButton = $("a[role=button][name=run]", this.fieldset);
        this.runButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$RunButton, this)});
    },

    setReports: function(values) {
        this.selection = [];
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectItem: function (value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.ReportsView.LISTENER;
        this._adjustButtonsState();
    },

    _formatStatus: function(report) {
        return report.failed()
               ? this.resources.getString("REPORTS_STATUS_FAILED", {0: report.errorMessage})
               : report.succeed()
                 ? this.resources.getString("REPORTS_STATUS_SUCCEED") :
                 this.resources.getString("REPORTS_STATUS_PROGRESS", {0: report.progress});
    },

    _formatStatusStyle: function(report) {
        return report.failed() ? "color:red" : report.succeed() ? "color:green" : "color:black";
    },

    _onClick$RefreshButton: function() {
        this.listener.onRefresh();
    },

    _onClick$AddButton: function() {
        this.listener.onAddReport();
    },

    _onClick$EditButton: function() {
        this.listener.onEditReport(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteReport(this.selection);
    },

    _onClick$ViewButton: function() {
        this.listener.onViewReport(this.selection[0]);
    },

    _onClick$RunButton: function() {
        var index = this.data.datagrid("getRowIndex", this.selection[0]);
        this.listener.onReRunReport(this.selection[0]);

        this.data.datagrid("refreshRow", index);
        this._adjustButtonsState();
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var succeed = this.selection.length == 1 && this.selection[0].succeed();
        var failed = this.selection.length == 1 && this.selection[0].failed();
        this.editButton.linkbutton(succeed || failed ? "enable" : "disable");
        this.deleteButton.linkbutton(succeed || failed ? "enable" : "disable");
        this.viewButton.linkbutton(succeed ? "enable" : "disable");
        this.runButton.linkbutton(succeed || failed ? "enable" : "disable");
    }
});
jscape.domain.ReportsView.LISTENER = {
    onRefresh: $.noop,
    onAddReport: $.noop,
    onEditReport: $.noop,
    onDeleteReport: $.noop,
    onViewReport: $.noop,
    onReRunReport: $.noop
};

/** @class */
jscape.domain.AddReportController = Class.extend(
    /** @lends {jscape.domain.AddReportController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._setupDialog().show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var report = this._createReport(dialog);
        dialog.destroy();

        this.managementService.storeReport(this.domainName, report)
                .done($.proxy(function() { this.deferred.resolve(report); }, this))
                .fail($.proxy(this.deferred.reject, this.deferred));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createReport: function(dialog) {
        return new jscape.Report(dialog.getName(),
                                 $.now(),
                                 dialog.getDescription(),
                                 dialog.getSearchName(),
                                 dialog.isReRunSearch(),
                                 dialog.getMetrics());
    },

    _setupDialog: function () {
        var dialog = new jscape.domain.AddReportDialog(this.resources);
        dialog.selectAllMetrics();
        this.managementService.getLogSearches(this.domainName).done($.proxy(function (searches) {
            dialog.setSearches($.map(searches || [], function (search) { return search.name; }));
        }, this));
        return dialog;
    }
});

/** @class */
jscape.domain.EditReportController = Class.extend(
    /** @lends {jscape.domain.EditReportController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, report) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.report = report;
        this.deferred = $.Deferred();

        this._setupDialog().show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var report = this._createReport(dialog);
        dialog.destroy();

        this.managementService.storeReport(this.domainName, report)
                .done($.proxy(function() { this.deferred.resolve(report); }, this))
                .fail($.proxy(this.deferred.reject, this.deferred));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createReport: function(dialog) {
        return new jscape.Report(this.report.name,
                                 $.now(),
                                 dialog.getDescription(),
                                 dialog.getSearchName(),
                                 dialog.isReRunSearch(),
                                 dialog.getMetrics());
    },

    _setupDialog: function () {
        var dialog = new jscape.domain.EditReportDialog(this.resources);
        dialog.setReportName(this.report.name);
        dialog.setDescription(this.report.description);
        dialog.setMetrics(this.report.metricses);
        dialog.setReRunSearch(this.report.reRunSearch);
        this.managementService.getLogSearches(this.domainName).done($.proxy(function(searches) {
            searches = $.map(searches || [], function(search) { return search.name; });
            dialog.setSearches(searches);
            dialog.setSearchName(this.report.searchName);
        }, this));
        return dialog;
    }
});

/** @class */
jscape.domain.ReportDialog = Class.extend(
    /** @lends {jscape.domain.ReportDialog.prototype} */
{
    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=reportfields]", this.dialog);

        this.description = $("textarea[name=description]", this.fieldset).css({width: "23em", height: "3em"});

        this.useSearch = $("input[name=usesearch]:checkbox", this.fieldset);
        this.useSearch.change($.proxy(function() {
            var enabled = this.useSearch.is(":checked");
            this.searches.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
            this.reRunSearch.prop("disabled", !enabled);
        }, this));
        this.searches = $("select[name=searchname]", this.fieldset);
        this.searches.combobox({required: true, editable: false, disabled: true, width: 150, panelHeight: "auto", onLoadSuccess: $.proxy(function(data) {
            if (data.length) {
                this.searches.combobox("select", data[0].value);
            } else {
                this.searches.combobox("disable");
                this.useSearch.prop("disabled", true);
            }
        }, this)}).combobox("autocomplete", true);

        this.reRunSearch = $("input[name=rerunsearch]:checkbox", this.fieldset).prop("disabled", true);

        this.metrics = $("table[role=grid][aria-label=metrics]", this.fieldset);
        this.metrics.datagrid({
            fitColumns: true,
            singleSelect: false,
            checkOnSelect: true,
            selectOnCheck: true,
            remoteSort: false,
            width: "auto", height: 160,
            onLoadSuccess: function(data) { $(this).datagrid("getPanel").find("div.datagrid-header-check input:checkbox").prop("disabled", data.rows.length == 0); },
            idField: "value",
            columns: [[
                    {field: "value", checkbox: true, sortable: false},
                    {field: "displayName", title: $("thead th:nth-child(1)", this.metrics).text(), sortable: false, width: 100, formatter: function(_, row) { return row.value.replace(/([A-Z])/g, " $1").trim(); }}
                ]]
            });
    },

    getDescription: function() {
        return this.description.val();
    },

    setDescription: function(value) {
        this.description.val(value);
    },

    getSearchName: function() {
        return this.useSearch.is(":checked") ? this.searches.combobox("getValue") : null;
    },

    setSearchName: function(value) {
        if (value) {
            this.searches.combobox("setValue", value);
        }
        this.useSearch.prop("checked", value != null).change();
    },

    setSearches: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.searches.combobox("loadData", data);
    },

    isReRunSearch: function() {
        return this.reRunSearch.is(":checked") && !this.reRunSearch.is(":disabled");
    },

    setReRunSearch: function(value) {
        this.reRunSearch.prop("checked", !!value);
    },

    getMetrics: function() {
        return $.map(this.metrics.datagrid("getChecked"), function(entry) { return entry.value; });
    },

    setMetrics: function(values) {
        if (values && values.length) {
            for (var i = 0; i < values.length; i++) {
                this.metrics.datagrid("selectRecord", values[i]);
            }
        } else {
            this.metrics.datagrid("clearChecked");
        }
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.metrics.datagrid("fitColumns");
        this.fieldset.form("validate");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.AddReportDialog = jscape.domain.ReportDialog.extend(
    /** @lends {jscape.domain.AddReportDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-reports-add"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["report_name_nonempty", "report_unique"], missingMessage: resources.getString("REPORTS_ERROR_BAD_NAME")});
    },

    getName: function() {
        return this.name.val();
    },

    selectAllMetrics: function() {
        this.metrics.datagrid("checkAll");
    }
});

/** @class */
jscape.domain.EditReportDialog = jscape.domain.ReportDialog.extend(
    /** @lends {jscape.domain.EditReportDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-reports-edit"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    setReportName: function(value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});

/** @class */
jscape.domain.ViewReportController = Class.extend(
    /** @lends {jscape.domain.ViewReportController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(report) {
        var url = window.location.pathname + "/reports/{report}/".supplant({report: encodeURIComponent(report.name)});
        window.open(url, "jscape-report-result");
    }
});


/** @class */
jscape.domain.DeleteReportController = Class.extend(
    /** @lends {jscape.domain.DeleteReportController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function (managementService, domainName, items) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._confirmDelete(items)
                .done($.proxy(this._deleteConfirmed, this, items))
                .fail(this.deferred.reject);

        return this.deferred.promise();
    },

    _confirmDelete: function(items) {
        var text = $.map(items, function(proxy) { return proxy.name; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("REPORTS_CONFIRM_DELETE_TITLE"),
                this.resources.getString("REPORTS_CONFIRM_DELETE_MESSAGE", {0: text}));
    },

    _deleteConfirmed: function(items) {
        var promises = new Array(items.length);
        for (var i = 0; i < items.length; i++) {
            promises[i] = this.managementService.deleteReport(this.domainName, items[i].name);
        }

        $.when.apply(jQuery, promises)
                .done($.proxy(this.deferred.resolve, this.deferred, items))
                .fail(this.deferred.reject);
    }
});
