jscape.domain = jscape.domain || {};

/** @class */
jscape.ResourceDescriptor = Class.extend(
    /** @lends {jscape.ResourceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e, f, g, h, i, j, k) {
        this.name = a || "";
        this.protocol = b || null;
        this.host = c || "";
        this.port = parseInt(d) || null;
        this.timeout = parseInt(e) || null;
        this.username = f !== undefined ? f : null;
        this.password = g !== undefined ? g : null;
        this.usingCredentials = h || false;
        this.remoteDirectory = i || null;
        this.logDirectory = j || null;
        this.timeToLive = parseInt(k) || null;
    }
});
jscape.ResourceDescriptor.fromJSON = function(data) {
    var remoteDir = jscape.RemoteDirectory.fromJSON(data.remoteDirectory);
    if (data.protocol == jscape.ResourceDescriptor.Protocol.WEBDAV) {
        return jscape.ResourceDescriptor.webdav(data.name, data.host, data.port, data.domain, data.path, !!data.useSsl, data.username, data.password, data.usingCredentials, remoteDir, data.logDirectory);
    } else {
        var keyProvider = jscape.FileKeyProvider.fromJSON(data.keyProvider);
        if (data.protocol == jscape.ResourceDescriptor.Protocol.AFTP) {
            return jscape.ResourceDescriptor.aftp(data.name, data.host, data.port, data.timeout, data.securityMode, data.username, data.password, data.usingCredentials, keyProvider, remoteDir, data.logDirectory, data.timeToLive);
        } else if (data.protocol == jscape.ResourceDescriptor.Protocol.FTP) {
            return jscape.ResourceDescriptor.ftp(data.name, data.host, data.port, data.timeout, data.connectionType, data.passive, data.username, data.password, data.usingCredentials, keyProvider, remoteDir, data.logDirectory, data.timeToLive);
        } else if (data.protocol == jscape.ResourceDescriptor.Protocol.SFTP) {
            return jscape.ResourceDescriptor.sftp(data.name, data.host, data.port, data.timeout, data.useAdaptiveConnection, data.startSubsystemReplyRequired, data.username, data.password, data.usingCredentials, keyProvider, remoteDir, data.logDirectory, data.timeToLive);
        } else if (data.protocol == jscape.ResourceDescriptor.Protocol.AMAZON_S3) {
            return jscape.ResourceDescriptor.amazonS3(data.name, data.encryptionRequired, data.username, data.password, data.usingCredentials, remoteDir, data.logDirectory);
        } else if (data.protocol == jscape.ResourceDescriptor.Protocol.REST) {
            return jscape.ResourceDescriptor.rest(data.name, data.host, data.port, data.timeout, data.sslRequired, data.domain, data.username, data.password, data.usingCredentials, keyProvider, remoteDir, data.logDirectory, data.timeToLive);
        } else if (data.protocol == jscape.ResourceDescriptor.Protocol.OFTP) {
            var hostKeyProvider = null;
            var encryptionKeyProvider = jscape.ServerKeyProvider.fromJSON(data.encryptionKeyProvider);
            var signingKeyProvider = jscape.ServerKeyProvider.fromJSON(data.signingKeyProvider);
            return jscape.ResourceDescriptor.oftp(data.name, data.host, data.port, data.sslRequired, data.username, data.password, data.usingCredentials, keyProvider, hostKeyProvider, data.destination, encryptionKeyProvider, signingKeyProvider, data.cipherSuite, data.logDirectory, data.timeToLive);
        } else if (data.protocol == jscape.ResourceDescriptor.Protocol.TFTP) {
            return jscape.ResourceDescriptor.tftp(data.name, data.host, data.port, data.retransmitTimeout, data.maxRetransmitAttempts, data.logDirectory);
        }
    }
    throw "Unknown resource type: " + (data && data.protocol);
};
jscape.ResourceDescriptor.aftp = function(name, host, port, timeout, securityMode, username, password, usingCredentials, keyProvider, remoteDir, logDir, timeToLive) {
    return $.extend(new jscape.ResourceDescriptor(name, jscape.ResourceDescriptor.Protocol.AFTP, host, port, timeout, username, password, usingCredentials, remoteDir, logDir, timeToLive),
                    {securityMode: securityMode, keyProvider: keyProvider});
};
jscape.ResourceDescriptor.ftp = function(name, host, port, timeout, connectionType, passive, username, password, usingCredentials, keyProvider, remoteDir, logDir, timeToLive) {
    return $.extend(new jscape.ResourceDescriptor(name, jscape.ResourceDescriptor.Protocol.FTP, host, port, timeout, username, password, usingCredentials, remoteDir, logDir, timeToLive),
                    {connectionType: connectionType, passive: !!passive, keyProvider: keyProvider});
};
jscape.ResourceDescriptor.sftp = function(name, host, port, timeout, useAdaptiveConnection, startSubsystemReplyRequired, username, password, usingCredentials, keyProvider, remoteDir, logDir, timeToLive) {
    return $.extend(new jscape.ResourceDescriptor(name, jscape.ResourceDescriptor.Protocol.SFTP, host, port, timeout, username, password, usingCredentials, remoteDir, logDir, timeToLive),
                    {useAdaptiveConnection: !!useAdaptiveConnection, startSubsystemReplyRequired: !!startSubsystemReplyRequired, keyProvider: keyProvider});
};
jscape.ResourceDescriptor.webdav = function(name, host, port, domain, path, useSsl, username, password, usingCredentials, remoteDir, logDir) {
    return $.extend(new jscape.ResourceDescriptor(name, jscape.ResourceDescriptor.Protocol.WEBDAV, host, port, undefined, username, password, usingCredentials, remoteDir, logDir),
                    {domain: domain, path: path, useSsl: !!useSsl});
};
jscape.ResourceDescriptor.amazonS3 = function(name, encryptionRequired, username, password, usingCredentials, remoteDir, logDir) {
    return $.extend(new jscape.ResourceDescriptor(name, jscape.ResourceDescriptor.Protocol.AMAZON_S3, undefined, undefined, undefined, username, password, usingCredentials, remoteDir, logDir, null),
                    {encryptionRequired: !!encryptionRequired});
};
jscape.ResourceDescriptor.rest = function(name, host, port, timeout, sslRequired, domain, username, password, usingCredentials, keyProvider, remoteDir, logDir, timeToLive) {
    return $.extend(new jscape.ResourceDescriptor(name, jscape.ResourceDescriptor.Protocol.REST, host, port, timeout, username, password, usingCredentials, remoteDir, logDir, timeToLive),
                    {sslRequired: !!sslRequired, domain: domain, keyProvider: keyProvider});
};
jscape.ResourceDescriptor.oftp = function(name, host, port, sslRequired, username, password, usingCredentials, keyProvider, hostKeyProvider, destination, encryptionKeyProvider, signingKeyProvider, cipherSuite, logDir, timeToLive) {
    return $.extend(new jscape.ResourceDescriptor(name, jscape.ResourceDescriptor.Protocol.OFTP, host, port, undefined, username, password, usingCredentials, undefined, logDir, timeToLive),
                    {sslRequired: !!sslRequired, keyProvider: keyProvider, hostKeyProvider: hostKeyProvider, destination: destination, encryptionKeyProvider: encryptionKeyProvider, signingKeyProvider: signingKeyProvider, cipherSuite: cipherSuite});
};
jscape.ResourceDescriptor.tftp = function(name, host, port, retransmitTimeout, maxRetransmitAttempts, logDir) {
    return $.extend(new jscape.ResourceDescriptor(name, jscape.ResourceDescriptor.Protocol.TFTP, host, port, undefined, undefined, undefined, false, undefined, logDir),
                    {retransmitTimeout: retransmitTimeout, maxRetransmitAttempts: maxRetransmitAttempts});
};
jscape.ResourceDescriptor.Protocol = {FTP: "FTP", AFTP: "AFTP", SFTP: "SFTP", WEBDAV: "WEBDAV", AMAZON_S3: "AMAZONS3", OFTP: "OFTP", REST: "REST", TFTP: "TFTP"};

/** @class */
jscape.RemoteDirectory = Class.extend(
    /** @lends {jscape.RemoteDirectory.prototype} */
{
    /** @constructor */
    initialize: function(mapLocalToRemote) {
        this.mapLocalToRemote = !!mapLocalToRemote;
    },

    toJSON: function() {
        return {"@type": this.mapLocalToRemote ? "LOCAL" : "REMOTE"};
    }
});
jscape.RemoteDirectory.fromJSON = function(data) {
    return new jscape.RemoteDirectory(data["@type"] === "LOCAL");
};

/** @class */
jscape.domain.ReverseProxiesModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.ReverseProxiesModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.ReverseProxiesController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onResume: function() {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.ReverseProxiesController = Class.extend(
    /** @lends {jscape.domain.ReverseProxiesController.prototype} */
{
    proxies: null,

    /** @constructor */
    initialize: function(resources) {
        this.proxies = [];
        this.view = new jscape.domain.ReverseProxiesView(resources);
        this.addController = new jscape.domain.AddReverseProxyController(resources);
        this.editController = new jscape.domain.EditReverseProxyController(resources);
        this.testController = new jscape.domain.TestReverseProxyController(resources);
        this.deleteController = new jscape.domain.DeleteReverseProxyController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onAddProxy: function() {
        this.addController.start(this, this.managementService, this.domainName)
                .done($.proxy(this._proxyAdded, this));
    },

    onEditProxy: function(proxy) {
        var index = $.inArray(proxy, this.proxies);
        if (index != -1) {
            this.editController.start(this, this.managementService, this.domainName, proxy)
                    .done($.proxy(this._proxyEdited, this, index));
        }
    },

    onDeleteProxy: function(proxies) {
        this.deleteController.start(this.managementService, this.domainName, proxies)
                .done($.proxy(this._proxyDeleted, this, proxies));
    },

    onTestProxy: function(descriptor) {
        this.testController.start(this.managementService, this.domainName, descriptor);
    },

    onTestServer: function(descriptor) {
        this.onTestProxy(descriptor);
    },

    _proxyAdded: function(item) {
        this.proxies.push(item);
        this.view.setReverseProxies(this.proxies);
        this.view.selectProxy(item);
    },

    _proxyEdited: function(index, item) {
        this.proxies[index] = item;
        this.view.setReverseProxies(this.proxies);
        this.view.selectProxy(item);
    },

    _proxyDeleted: function (items) {
        var index = -1;
        while (items.length) {
            index = $.inArray(items.pop(), this.proxies);
            this.proxies.splice(index, 1);
        }
        this.view.setReverseProxies(this.proxies);

        if (this.proxies.length) {
            index = Math.max(0, Math.min(index, this.proxies.length - 1));
            this.view.selectProxy(this.proxies[index]);
        }
    },

    _loadData: function() {
        this.managementService.getReverseProxies(this.domainName)
                .done($.proxy(function(proxies) {
                    this.proxies = proxies || [];
                    this.view.setReverseProxies(this.proxies);
                }, this));
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("proxy_name_nonempty", resources.getString("PROXIES_ERROR_BAD_NAME"));
        v.nonEmptyRule("proxy_host_nonempty", resources.getString("PROXIES_ERROR_BAD_HOST"));
        v.nonEmptyRule("proxy_debugdir_nonempty", resources.getString("PROXIES_ERROR_BAD_DEBUG_DIR"));
        v.rule("proxy_unique", $.proxy(this._proxyNotExists, this), resources.getString("PROXIES_ERROR_ALREADY_EXISTS"));
    },

    _proxyNotExists: function (value, excludes) {
        return $.grep(this.proxies, function (proxy) {
            return proxy.name == value && (excludes == undefined || $.inArray(proxy.name, excludes) == -1);
        }).length == 0;
    }
});

/** @class */
jscape.domain.ReverseProxiesView = Class.extend(
    /** @lends {jscape.domain.ReverseProxiesView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources, container) {
        this.selection = [];
        this.resources = resources;

        this.fieldset = $("fieldset[name=reverseproxiesfields]", container);

        this.data = $("table[role=grid][aria-label=reverseproxies]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            singleSelect: true,
            pagination: true,
            search: true,
            remoteSort: false,
            sortName: "name",
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 120},
                {field: "protocol", title: $("thead th:nth-child(2)", this.data).text(), sortable: false, width: 60, formatter: $.proxy(this._formatProtocol, this)},
                {field: "host", title: $("thead th:nth-child(3)", this.data).text(), width: 100},
                {field: "port", title: $("thead th:nth-child(4)", this.data).text(), align: "right", width: 30},
                {field: "username", title: $("thead th:nth-child(5)", this.data).text(), width: 80},
                {field: "details", title: $("thead th:nth-child(6)", this.data).text(), formatter: $.proxy(function(_, row) { return this._formatDetails(row); }, this), width: 150}
            ]],
            onDblClickRow: $.proxy(this._onClick$EditButton, this),
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this)
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.testButton = $("a[role=button][name=test]", this.fieldset);
        this.testButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$TestButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.ReverseProxiesView.LISTENER;
        this._adjustButtonsState();
    },

    setReverseProxies: function (values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectProxy: function (value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    _formatProtocol: function(value) {
        return this.resources.getString("APPLICATION_PROTOCOL_" + String(value).toUpperCase());
    },

    _formatDetails: function(descriptor) {
        if (descriptor.connectionType) {
            return descriptor.connectionType;
        } else if (descriptor.securityMode) {
            return this.resources.getString("PROXIES_SECURITY_MODE_" + descriptor.securityMode.toUpperCase());
        } else {
            return "";
        }
    },

    _onClick$AddButton: function() {
        this.listener.onAddProxy();
    },

    _onClick$EditButton: function() {
        this.listener.onEditProxy(this.selection[0]);
    },

    _onClick$TestButton: function() {
        this.listener.onTestProxy(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteProxy(this.selection);
    },

    _adjustRowSelection: function () {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var singleSelection = this.selection.length == 1;
        this.editButton.linkbutton(singleSelection ? "enable" : "disable");
        this.testButton.linkbutton(singleSelection ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }
});
jscape.domain.ReverseProxiesView.LISTENER = {
    onAddProxy: $.noop,
    onEditProxy: $.noop,
    onTestProxy: $.noop,
    onDeleteProxy: $.noop
};

/** @class */
jscape.domain.BaseReverseProxyController = Class.extend(
    /** @lends {jscape.domain.BaseReverseProxyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(listener, managementService, domainName) {
        this.listener = listener || jscape.domain.BaseReverseProxyController.LISTENER;
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._setupDialog().show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var descriptor = this._createDescriptor(dialog);
        dialog.destroy();
        this.deferred.resolve(descriptor);
        this.deferred = null;
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    onTest: function(dialog) {
        this.listener.onTestServer(this._createDescriptor(dialog));
    },

    onSelectKeyFile: function(dialog) {
        var loadFn = $.proxy(this.managementService.listKeystorePath, this.managementService);
        var createFn = $.proxy(this.managementService.createKeystoreDir, this.managementService);
        new SelectPathController(loadFn, createFn).selectFile(function(entry) { dialog.setKeyFile(entry.path); });
    },

    onSelectDebugDir: function(dialog) {
        var loadFn = $.proxy(this.managementService.listDebugDir, this.managementService);
        var createFn = $.proxy(this.managementService.createDebugDir, this.managementService);
        new SelectPathController(loadFn, createFn).selectDirectory(function(entry) { dialog.setDebugDirectory(entry.path); });
    },

    _setupView: function() {
        throw "Abstract method";
    },

    _createDescriptor: function (dialog) {
        throw "Abstract method";
    }
});
jscape.domain.BaseReverseProxyController.LISTENER = {
    onTestServer: $.noop
};

/** @class */
jscape.domain.AddFtpReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.AddFtpReverseProxyController.prototype} */
{
    /** @const */
    SECURITY_MODE_KEY_USAGE: {'regular': false, 'explicit SSL': true, 'implicit SSL': true},
    DEFAULT_PORTS: {'regular': 21, 'explicit SSL': 21, 'implicit SSL': 990},
    DEFAULT_TIMEOUT: 1000,
    DEFAULT_MAX_AGE: 10 * 60 * 1000,

    /** @override */
    _setupDialog: function() {
        var dialog = new jscape.domain.AddFtpReverseProxyDialog(this.resources);
        dialog.setKeyUsage(this.SECURITY_MODE_KEY_USAGE);
        dialog.setDefaultPorts(this.DEFAULT_PORTS);
        dialog.setPort(this.DEFAULT_PORTS[dialog.getConnectionType()]);
        dialog.setTimeout(this.DEFAULT_TIMEOUT);
        dialog.setUsername("");
        dialog.setPassword("");
        dialog.setUsePassiveMode(true);
        dialog.setMapLocalDirectoryToRemote(false);
        dialog.setDebugDirectory(null);
        dialog.setMaxAge(this.DEFAULT_MAX_AGE);
        return dialog;
    },

    /** @override */
    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.ftp(dialog.getName(),
                                             dialog.getHost(),
                                             dialog.getPort(),
                                             dialog.getTimeout(),
                                             dialog.getConnectionType(),
                                             dialog.isUsePassiveMode(),
                                             dialog.getUsername(),
                                             dialog.getPassword(),
                                             dialog.getUsername() == null,
                                             dialog.getKeyFile().length != 0 ? new jscape.FileKeyProvider(dialog.getKeyFile(), dialog.getKeyFilePassword()) : null,
                                             new jscape.RemoteDirectory(dialog.isMapLocalDirectoryToRemote()),
                                             dialog.getDebugDirectory(),
                                             dialog.getMaxAge());
    }
});

/** @class */
jscape.domain.EditFtpReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.EditFtpReverseProxyController.prototype} */
{
    /** @const */
    SECURITY_MODE_KEY_USAGE: {'regular': false, 'explicit SSL': true, 'implicit SSL': true},
    DEFAULT_PORTS: {'REGULAR': 21, 'EXPLICIT_SSL': 21, 'IMPLICIT_SSL': 990},

    descriptor: null,

    /** @override */
    start: function(listener, managementService, domainName, descriptor) {
        this.descriptor = descriptor;
        return this._super(listener, managementService, domainName);
    },

    /** @override */
    _setupDialog: function() {
        var dialog = new jscape.domain.EditFtpReverseProxyDialog(this.resources);
        dialog.setName(this.descriptor.name);
        dialog.setHost(this.descriptor.host);
        dialog.setPort(this.descriptor.port);
        dialog.setConnectionType(this.descriptor.connectionType);
        dialog.setKeyUsage(this.SECURITY_MODE_KEY_USAGE);
        dialog.setTimeout(this.descriptor.timeout);
        if (this.descriptor.keyProvider) {
            dialog.setKeyFile(this.descriptor.keyProvider.file);
            dialog.setKeyFilePassword(this.descriptor.keyProvider.filePassword);
        }
        dialog.setUsername(this.descriptor.username);
        dialog.setPassword(this.descriptor.password);
        dialog.setUsePassiveMode(this.descriptor.passive);
        dialog.setMapLocalDirectoryToRemote(this.descriptor.remoteDirectory.mapLocalToRemote);
        dialog.setDebugDirectory(this.descriptor.logDirectory);
        dialog.setMaxAge(this.descriptor.timeToLive);
        return dialog;
    },

    /** @override */
    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.ftp(this.descriptor.name,
                                             dialog.getHost(),
                                             dialog.getPort(),
                                             dialog.getTimeout(),
                                             dialog.getConnectionType(),
                                             dialog.isUsePassiveMode(),
                                             dialog.getUsername(),
                                             dialog.getPassword(),
                                             dialog.getUsername() == null,
                                             dialog.getKeyFile().length != 0 ? new jscape.FileKeyProvider(dialog.getKeyFile(), dialog.getKeyFilePassword()) : null,
                                             new jscape.RemoteDirectory(dialog.isMapLocalDirectoryToRemote()),
                                             dialog.getDebugDirectory(),
                                             dialog.getMaxAge());
    }
});

/** @class */
jscape.domain.FtpReverseProxyDialog = Class.extend(
    /** @lends {jscape.domain.FtpReverseProxyDialog.prototype} */
{
    /** @cons */
    TIMEOUT_FACTOR: 60 * 1000,
    DEFAULT_MAX_AGE: 10,

    keyUsage: null,
    defaultPorts: null,

    /** @constructor */
    initialize: function(dialog, resources) {
        this.keyUsage = {};
        this.defaultPorts = {};

        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=reverseproxyfields]", this.dialog);

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, validType: "proxy_host_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 100});

        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({required: true, min: 1, max: 60, width: 60});

        this.connectionType = $("select[name=connectiontype]", this.fieldset);
        this.connectionType.combobox({required: true, editable: false, panelHeight: "auto", onSelect: $.proxy(this._adjustSecurityMode, this)});

        this.keyFile = $("input[name=keyfile]", this.fieldset);
        this.keyFilePassword = $("input[name=keyfilepwd]", this.fieldset);
        this.keyFileBrowseButton = $("a[role=button][name=keyfilebrowse]", this.fieldset);
        this.keyFileBrowseButton.linkbutton({onClick: $.proxy(this._onClick$KeyFileBrowseButton, this)});

        this.credentials = $("input[name=credentials]:radio", this.fieldset);
        this.enterCredentials = this.credentials.filter("[value=enter]");
        this.replayCredentials = this.credentials.filter("[value=replay]");
        this.credentials.change($.proxy(function() {
            var enabled = this.enterCredentials.is(":checked");
            this.username.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation").validatebox("validate");
            this.password.prop("disabled", !enabled);
        }, this));

        this.username = $("input[name=username]", this.fieldset);
        this.username.validatebox({required: true, validType: "proxy_username_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_USER")});
        this.password = $("input[name=userpwd]", this.fieldset);

        this.passive = $("input[name=passivemode]:checkbox", this.fieldset);
        this.mapLocalDir = $("input[name=maplocaldir]:checkbox", this.fieldset);

        this.debugLog = $("input[name=debuglog]:checkbox", this.fieldset);
        this.debugLog.change($.proxy(function() {
            var enabled = this.debugLog.is(":checked");
            this.debugDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.debugDirBrowseButton.linkbutton(enabled ? "enable" : "disable");
        }, this));
        this.debugDir = $("input[name=debugdir]", this.fieldset);
        this.debugDir.validatebox({required: true, validaType: "proxy_debugdir_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_DEBUG_DIR")});
        this.debugDirBrowseButton = $("a[role=button][name=debugdirbrowse]", this.fieldset);
        this.debugDirBrowseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DebugDirBrowseButton, this)});

        this.maxAge = $("input[name=maxage]:checkbox", this.fieldset);
        this.maxAge.change($.proxy(function() {
            var enabled = this.maxAge.is(":checked");
            this.ttl.numberspinner(enabled ? "enable" : "disable").validatebox(enabled ? "enableValidation" : "disableValidation")
        }, this));
        this.ttl = $("input[name=ttl]", this.fieldset);
        this.ttl.numberspinner({required: true, disabled: true, min: 1, max: 1000, width: 70});

        this.testButton = $("a[role=button][name=test]", this.dialog);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getName: function() {
        throw "Abstract method";
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value);
    },

    getTimeout: function() {
        return this.timeout.numberspinner("getValue") * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    setKeyUsage: function(values) {
        this.keyUsage = $.extend({}, values);
    },

    setDefaultPorts: function(values) {
        this.defaultPorts = $.extend({}, values);
    },

    getConnectionType: function() {
        return this.connectionType.combobox("getValue");
    },

    setConnectionType: function(value) {
        this.connectionType.combobox("setValue", value);
    },

    getUsername: function() {
        return this.enterCredentials.is(":checked") ? this.username.val() : null;
    },

    setUsername: function(value) {
        if (value != null) {
            this.username.val(value);
            this.enterCredentials.prop("checked", true).change();
        } else {
            this.replayCredentials.prop("checked", true).change();
        }
    },

    getPassword: function() {
        return this.enterCredentials.is(":checked") ? this.password.val() : null;
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getKeyFile: function() {
        return this.keyFile.val().trim();
    },

    setKeyFile: function(value) {
        this.keyFile.val(value);
    },

    getKeyFilePassword: function() {
        return this.keyFilePassword.val();
    },

    setKeyFilePassword: function(value) {
        this.keyFilePassword.val(value);
    },

    isUsePassiveMode: function() {
        return this.passive.is(":checked");
    },

    setUsePassiveMode: function(value) {
        this.passive.prop("checked", !!value);
    },

    isMapLocalDirectoryToRemote: function() {
        return this.mapLocalDir.is(":checked");
    },

    setMapLocalDirectoryToRemote: function(value) {
        this.mapLocalDir.prop("checked", !!value);
    },

    getDebugDirectory: function() {
        return this.debugLog.is(":checked") ? this.debugDir.val() : null;
    },

    setDebugDirectory: function(value) {
        this.debugLog.prop("checked", value != null).change();
        this.debugDir.val(value || "");
    },

    getMaxAge: function() {
        return this.maxAge.is(":checked") ? this.ttl.numberspinner("getValue") * this.TIMEOUT_FACTOR : null;
    },

    setMaxAge: function(value) {
        value = parseInt(value);
        this.maxAge.prop("checked", !isNaN(value)).change();
        this.ttl.numberspinner("setValue", !isNaN(value) ? value / this.TIMEOUT_FACTOR : this.DEFAULT_MAX_AGE);
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.FtpReverseProxyDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
        this._adjustSecurityMode();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$TestButton: function() {
        if (this.fieldset.form("validate")) {
            this.listener.onTest(this);
        }
    },

    _onClick$KeyFileBrowseButton: function () {
        this.listener.onSelectKeyFile(this);
    },

    _onClick$DebugDirBrowseButton: function () {
        this.listener.onSelectDebugDir(this);
    },

    _adjustSecurityMode: function () {
        var type = this.getConnectionType();
        var enabled = !!this.keyUsage[type];
        this.keyFile.prop("disabled", !enabled);
        this.keyFilePassword.prop("disabled", !enabled);
        this.keyFileBrowseButton.linkbutton(enabled ? "enable" : "disable");

        var port = parseInt(this.defaultPorts[type]);
        if (!isNaN(port)) {
            this.port.numberspinner("setValue", port);
        }
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.FtpReverseProxyDialog.LISTENER = {
    onTest: $.noop,
    onSelectKeyFile: $.noop,
    onSelectDebugDir: $.noop
};

/** @class */
jscape.domain.AddFtpReverseProxyDialog = jscape.domain.FtpReverseProxyDialog.extend(
    /** @lends {jscape.domain.AddFtpReverseProxyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-add-ftp"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["proxy_name_nonempty", "proxy_unique"], missingMessage: resources.getString("PROXIES_ERROR_BAD_NAME")});
    },

    getName: function() {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditFtpReverseProxyDialog = jscape.domain.FtpReverseProxyDialog.extend(
    /** @lends {jscape.domain.EditFtpReverseProxyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-edit-ftp"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    setName: function(value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});


/** @class */
jscape.domain.AddSftpReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.AddSftpReverseProxyController.prototype} */
{
    /** @const */
    DEFAULT_PORT: 22,
    DEFAULT_TIMEOUT: 1000,
    DEFAULT_MAX_AGE: 10 * 60 * 1000,

    /** @override */
    _setupDialog: function() {
        var dialog = new jscape.domain.AddSftpReverseProxyDialog(this.resources);
        dialog.setPort(this.DEFAULT_PORT);
        dialog.setTimeout(this.DEFAULT_TIMEOUT);
        dialog.setUsername("");
        dialog.setPassword("");
        dialog.setMapLocalDirectoryToRemote(false);
        dialog.setDebugDirectory(null);
        dialog.setMaxAge(this.DEFAULT_MAX_AGE);
        return dialog;
    },

    /** @override */
    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.sftp(dialog.getName(),
                                              dialog.getHost(),
                                              dialog.getPort(),
                                              dialog.getTimeout(),
                                              dialog.isUseAdaptiveMode(),
                                              dialog.isStartSubsystemReplyRequired(),
                                              dialog.getUsername(),
                                              dialog.getPassword(),
                                              dialog.getUsername() == null,
                                              dialog.getKeyFile().length != 0 ? new jscape.FileKeyProvider(dialog.getKeyFile(), dialog.getKeyFilePassword()) : null,
                                              new jscape.RemoteDirectory(dialog.isMapLocalDirectoryToRemote()),
                                              dialog.getDebugDirectory(),
                                              dialog.getMaxAge());
    }
});

/** @class */
jscape.domain.EditSftpReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.EditSftpReverseProxyController.prototype} */
{
    descriptor: null,

    /** @override */
    start: function(listener, managementService, domainName, descriptor) {
        this.descriptor = descriptor;
        return this._super(listener, managementService, domainName);
    },

    /** @override */
    _setupDialog: function() {
        var dialog = new jscape.domain.EditSftpReverseProxyDialog(this.resources);
        dialog.setName(this.descriptor.name);
        dialog.setHost(this.descriptor.host);
        dialog.setPort(this.descriptor.port);
        if (this.descriptor.keyProvider) {
            dialog.setKeyFile(this.descriptor.keyProvider.file);
            dialog.setKeyFilePassword(this.descriptor.keyProvider.filePassword);
        }
        dialog.setTimeout(this.descriptor.timeout);
        dialog.setUsername(this.descriptor.username);
        dialog.setPassword(this.descriptor.password);
        dialog.setUseAdaptiveMode(this.descriptor.useAdaptiveConnection);
        dialog.setStartSubsystemReplyRequired(this.descriptor.startSubsystemReplyRequired);
        dialog.setMapLocalDirectoryToRemote(this.descriptor.remoteDirectory.mapLocalToRemote);
        dialog.setDebugDirectory(this.descriptor.logDirectory);
        dialog.setMaxAge(this.descriptor.timeToLive);
        return dialog;
    },

    /** @override */
    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.sftp(this.descriptor.name,
                                              dialog.getHost(),
                                              dialog.getPort(),
                                              dialog.getTimeout(),
                                              dialog.isUseAdaptiveMode(),
                                              dialog.isStartSubsystemReplyRequired(),
                                              dialog.getUsername(),
                                              dialog.getPassword(),
                                              dialog.getUsername() == null,
                                              dialog.getKeyFile().length != 0 ? new jscape.FileKeyProvider(dialog.getKeyFile(), dialog.getKeyFilePassword()) : null,
                                              new jscape.RemoteDirectory(dialog.isMapLocalDirectoryToRemote()),
                                              dialog.getDebugDirectory(),
                                              dialog.getMaxAge());
    }
});

/** @class */
jscape.domain.SftpReverseProxyDialog = Class.extend(
    /** @lends {jscape.domain.SftpReverseProxyDialog.prototype} */
{
    /** @cons */
    TIMEOUT_FACTOR: 60 * 1000,
    DEFAULT_MAX_AGE: 10,

    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=reverseproxyfields]", this.dialog);

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, validType: "proxy_host_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 100});

        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({required: true, min: 1, max: 60, width: 60});

        this.keyFile = $("input[name=keyfile]", this.fieldset);
        this.keyFilePassword = $("input[name=keyfilepwd]", this.fieldset);
        this.keyFileBrowseButton = $("a[role=button][name=keyfilebrowse]", this.fieldset);
        this.keyFileBrowseButton.linkbutton({onClick: $.proxy(this._onClick$KeyFileBrowseButton, this)});

        this.credentials = $("input[name=credentials]:radio", this.fieldset);
        this.enterCredentials = this.credentials.filter("[value=enter]");
        this.replayCredentials = this.credentials.filter("[value=replay]");
        this.credentials.change($.proxy(function() {
            var enabled = this.enterCredentials.is(":checked");
            this.username.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.password.prop("disabled", !enabled);
        }, this));

        this.username = $("input[name=username]", this.fieldset);
        this.username.validatebox({required: true, validType: "proxy_username_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_USER")});
        this.password = $("input[name=userpwd]", this.fieldset);

        this.useAdaptiveMode = $("input[name=adaptivemode]:checkbox");
        this.startSubsystemReply = $("input[name=startsubsystemreply]:checkbox");
        this.mapLocalDir = $("input[name=maplocaldir]:checkbox", this.fieldset);

        this.debugLog = $("input[name=debuglog]:checkbox", this.fieldset);
        this.debugLog.change($.proxy(function() {
            var enabled = this.debugLog.is(":checked");
            this.debugDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.debugDirBrowseButton.linkbutton(enabled ? "enable" : "disable");
        }, this));
        this.debugDir = $("input[name=debugdir]", this.fieldset);
        this.debugDir.validatebox({required: true, validaType: "proxy_debugdir_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_DEBUG_DIR")});
        this.debugDirBrowseButton = $("a[role=button][name=debugdirbrowse]", this.fieldset);
        this.debugDirBrowseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DebugDirBrowseButton, this)});

        this.maxAge = $("input[name=maxage]:checkbox", this.fieldset);
        this.maxAge.change($.proxy(function() {
            var enabled = this.maxAge.is(":checked");
            this.ttl.numberspinner(enabled ? "enable" : "disable").validatebox(enabled ? "enableValidation" : "disableValidation")
        }, this));
        this.ttl = $("input[name=ttl]", this.fieldset);
        this.ttl.numberspinner({required: true, disabled: true, min: 1, max: 1000, width: 70});

        this.testButton = $("a[role=button][name=test]", this.dialog);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getName: function() {
        throw "Abstract method.";
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value);
    },

    getTimeout: function() {
        return this.timeout.numberspinner("getValue") * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    getUsername: function() {
        return this.enterCredentials.is(":checked") ? this.username.val() : null;
    },

    setUsername: function(value) {
        if (value != null) {
            this.username.val(value);
            this.enterCredentials.prop("checked", true).change();
        } else {
            this.replayCredentials.prop("checked", true).change();
        }
    },

    getPassword: function() {
        return this.enterCredentials.is(":checked") ? this.password.val() : null;
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getKeyFile: function() {
        return this.keyFile.val().trim();
    },

    setKeyFile: function(value) {
        this.keyFile.val(value);
    },

    getKeyFilePassword: function() {
        return this.keyFilePassword.val();
    },

    setKeyFilePassword: function(value) {
        this.keyFilePassword.val(value);
    },

    isUseAdaptiveMode: function() {
        return this.useAdaptiveMode.is(":checked");
    },

    setUseAdaptiveMode: function(value) {
        this.useAdaptiveMode.prop("checked", !!value);
    },

    isStartSubsystemReplyRequired: function() {
        return this.startSubsystemReply.is(":checked");
    },

    setStartSubsystemReplyRequired: function(value) {
        this.startSubsystemReply.prop("checked", !!value);
    },

    isMapLocalDirectoryToRemote: function() {
        return this.mapLocalDir.is(":checked");
    },

    setMapLocalDirectoryToRemote: function(value) {
        this.mapLocalDir.prop("checked", !!value);
    },

    getDebugDirectory: function() {
        return this.debugLog.is(":checked") ? this.debugDir.val() : null;
    },

    setDebugDirectory: function(value) {
        this.debugLog.prop("checked", value != null).change();
        this.debugDir.val(value || "");
    },

    getMaxAge: function() {
        return this.maxAge.is(":checked") ? this.ttl.numberspinner("getValue") * this.TIMEOUT_FACTOR : null;
    },

    setMaxAge: function(value) {
        value = parseInt(value);
        this.maxAge.prop("checked", !isNaN(value)).change();
        this.ttl.numberspinner("setValue", !isNaN(value) ? value / this.TIMEOUT_FACTOR : this.DEFAULT_MAX_AGE);
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.SftpReverseProxyDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$TestButton: function() {
        if (this.fieldset.form("validate")) {
            this.listener.onTest(this);
        }
    },

    _onClick$KeyFileBrowseButton: function () {
        this.listener.onSelectKeyFile(this);
    },

    _onClick$DebugDirBrowseButton: function () {
        this.listener.onSelectDebugDir(this);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.SftpReverseProxyDialog.LISTENER = {
    onTest: $.noop,
    onSelectKeyFile: $.noop,
    onSelectDebugDir: $.noop
};

/** @class */
jscape.domain.AddSftpReverseProxyDialog = jscape.domain.SftpReverseProxyDialog.extend(
    /** @lends {jscape.domain.AddSftpReverseProxyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-add-sftp"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["proxy_name_nonempty", "proxy_unique"], missingMessage: resources.getString("PROXIES_ERROR_BAD_NAME")});
    },

    getName: function() {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditSftpReverseProxyDialog = jscape.domain.SftpReverseProxyDialog.extend(
    /** @lends {jscape.domain.EditSftpReverseProxyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-edit-sftp"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    setName: function(value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});


/** @class */
jscape.domain.AddAftpReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.AddAftpReverseProxyController.prototype} */
{
    /** @const */
    SECURITY_MODE_KEY_USAGE: {'NO_PROTECTION': false, 'FULL_TIME_PROTECTED': true, 'AUTHENTICATION_PROTECTED': true},
    DEFAULT_MAX_AGE: 10 * 60 * 1000,
    DEFAULT_PORT: 3000,
    DEFAULT_TIMEOUT: 1000,

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.AddAftpReverseProxyDialog(this.resources);
        dialog.setPort(this.DEFAULT_PORT);
        dialog.setTimeout(this.DEFAULT_TIMEOUT);
        dialog.setKeyUsage(this.SECURITY_MODE_KEY_USAGE);
        dialog.setUsername("");
        dialog.setPassword("");
        dialog.setMapLocalDirectoryToRemote(false);
        dialog.setDebugDirectory(null);
        dialog.setMaxAge(this.DEFAULT_MAX_AGE);
        return dialog;
    },

    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.aftp(dialog.getName(),
                                              dialog.getHost(),
                                              dialog.getPort(),
                                              dialog.getTimeout(),
                                              dialog.getSecurityMode(),
                                              dialog.getUsername(),
                                              dialog.getPassword(),
                                              dialog.getUsername() == null,
                                              dialog.getKeyFile().length != 0 ? new jscape.FileKeyProvider(dialog.getKeyFile(), dialog.getKeyFilePassword()) : null,
                                              new jscape.RemoteDirectory(dialog.isMapLocalDirectoryToRemote()),
                                              dialog.getDebugDirectory(),
                                              dialog.getMaxAge());
    }
});

/** @class */
jscape.domain.EditAftpReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.EditAftpReverseProxyController.prototype} */
{
    /** @const */
    SECURITY_MODE_KEY_USAGE: {'NO_PROTECTION': false, 'FULL_TIME_PROTECTED': true, 'AUTHENTICATION_PROTECTED': true},

    /** @override */
    start: function(listener, managementService, domainName, descriptor) {
        this.descriptor = descriptor;
        return this._super(listener, managementService, domainName);
    },

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.EditAftpReverseProxyDialog(this.resources);
        dialog.setName(this.descriptor.name);
        dialog.setHost(this.descriptor.host);
        dialog.setPort(this.descriptor.port);
        dialog.setSecurityMode(this.descriptor.securityMode);
        dialog.setKeyUsage(this.SECURITY_MODE_KEY_USAGE);
        if (this.descriptor.keyProvider) {
            dialog.setKeyFile(this.descriptor.keyProvider.file);
            dialog.setKeyFilePassword(this.descriptor.keyProvider.filePassword);
        }
        dialog.setTimeout(this.descriptor.timeout);
        dialog.setUsername(this.descriptor.username);
        dialog.setPassword(this.descriptor.password);
        dialog.setMapLocalDirectoryToRemote(this.descriptor.remoteDirectory.mapLocalToRemote);
        dialog.setDebugDirectory(this.descriptor.logDirectory);
        dialog.setMaxAge(this.descriptor.timeToLive);
        return dialog;
    },

    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.aftp(this.descriptor.name,
                                              dialog.getHost(),
                                              dialog.getPort(),
                                              dialog.getTimeout(),
                                              dialog.getSecurityMode(),
                                              dialog.getUsername(),
                                              dialog.getPassword(),
                                              dialog.getUsername() == null,
                                              dialog.getKeyFile().length != 0 ? new jscape.FileKeyProvider(dialog.getKeyFile(), dialog.getKeyFilePassword()) : null,
                                              new jscape.RemoteDirectory(dialog.isMapLocalDirectoryToRemote()),
                                              dialog.getDebugDirectory(),
                                              dialog.getMaxAge());
    }
});

/** @class */
jscape.domain.AftpReverseProxyDialog = Class.extend(
    /** @lends {jscape.domain.AftpReverseProxyDialog.prototype} */
{
    /** @cons */
    TIMEOUT_FACTOR: 60 * 1000,
    DEFAULT_MAX_AGE: 10,

    keyUsage: {},

    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=reverseproxyfields]", this.dialog);

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, validType: "proxy_host_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 100});

        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({required: true, min: 1, max: 60, width: 60});

        this.security = $("select[name=security]", this.fieldset);
        this.security.combobox({required: true, editable: false, panelHeight: "auto", onSelect: $.proxy(this._adjustSecurityMode, this)});

        this.keyFile = $("input[name=keyfile]", this.fieldset);
        this.keyFilePassword = $("input[name=keyfilepwd]", this.fieldset);
        this.keyFileBrowseButton = $("a[role=button][name=keyfilebrowse]", this.fieldset);
        this.keyFileBrowseButton.linkbutton({onClick: $.proxy(this._onClick$KeyFileBrowseButton, this)});

        this.credentials = $("input[name=credentials]:radio", this.fieldset);
        this.enterCredentials = this.credentials.filter("[value=enter]");
        this.replayCredentials = this.credentials.filter("[value=replay]");
        this.credentials.change($.proxy(function() {
            var enabled = this.enterCredentials.is(":checked");
            this.username.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.password.prop("disabled", !enabled);
        }, this));

        this.username = $("input[name=username]", this.fieldset);
        this.username.validatebox({required: true, validType: "proxy_username_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_USER")});
        this.password = $("input[name=userpwd]", this.fieldset);

        this.mapLocalDir = $("input[name=maplocaldir]:checkbox", this.fieldset);

        this.debugLog = $("input[name=debuglog]:checkbox", this.fieldset);
        this.debugLog.change($.proxy(function() {
            var enabled = this.debugLog.is(":checked");
            this.debugDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.debugDirBrowseButton.linkbutton(enabled ? "enable" : "disable");
        }, this));
        this.debugDir = $("input[name=debugdir]", this.fieldset);
        this.debugDir.validatebox({required: true, validaType: "proxy_debugdir_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_DEBUG_DIR")});
        this.debugDirBrowseButton = $("a[role=button][name=debugdirbrowse]", this.fieldset);
        this.debugDirBrowseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DebugDirBrowseButton, this)});

        this.maxAge = $("input[name=maxage]:checkbox", this.fieldset);
        this.maxAge.change($.proxy(function() {
            var enabled = this.maxAge.is(":checked");
            this.ttl.numberspinner(enabled ? "enable" : "disable").validatebox(enabled ? "enableValidation" : "disableValidation")
        }, this));
        this.ttl = $("input[name=ttl]", this.fieldset);
        this.ttl.numberspinner({required: true, disabled: true, min: 1, max: 1000, width: 70});

        this.testButton = $("a[role=button][name=test]", this.dialog);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getName: function() {
        throw "Abstract method.";
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value);
    },

    getTimeout: function() {
        return this.timeout.numberspinner("getValue") * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    setKeyUsage: function(keyUsage) {
        this.keyUsage = $.extend({}, keyUsage);
    },

    getSecurityMode: function() {
        return this.security.combobox("getValue");
    },

    setSecurityMode: function(value) {
        this.security.combobox("setValue", value);
    },

    getUsername: function() {
        return this.enterCredentials.is(":checked") ? this.username.val() : null;
    },

    setUsername: function(value) {
        if (value != null) {
            this.username.val(value);
            this.enterCredentials.prop("checked", true).change();
        } else {
            this.replayCredentials.prop("checked", true).change();
        }
    },

    getPassword: function() {
        return this.enterCredentials.is(":checked") ? this.password.val() : null;
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getKeyFile: function() {
        return this.keyFile.val().trim();
    },

    setKeyFile: function(value) {
        this.keyFile.val(value);
    },

    getKeyFilePassword: function() {
        return this.keyFilePassword.val();
    },

    setKeyFilePassword: function(value) {
        this.keyFilePassword.val(value);
    },

    isMapLocalDirectoryToRemote: function() {
        return this.mapLocalDir.is(":checked");
    },

    setMapLocalDirectoryToRemote: function(value) {
        this.mapLocalDir.prop("checked", !!value);
    },

    getDebugDirectory: function() {
        return this.debugLog.is(":checked") ? this.debugDir.val() : null;
    },

    setDebugDirectory: function(value) {
        this.debugLog.prop("checked", value != null).change();
        this.debugDir.val(value || "");
    },

    getMaxAge: function() {
        return this.maxAge.is(":checked") ? this.ttl.numberspinner("getValue") * this.TIMEOUT_FACTOR : null;
    },

    setMaxAge: function(value) {
        value = parseInt(value);
        this.maxAge.prop("checked", !isNaN(value)).change();
        this.ttl.numberspinner("setValue", !isNaN(value) ? value / this.TIMEOUT_FACTOR : this.DEFAULT_MAX_AGE);
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.AftpReverseProxyDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
        this._adjustSecurityMode();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$TestButton: function() {
        if (this.fieldset.form("validate")) {
            this.listener.onTest(this);
        }
    },

    _onClick$KeyFileBrowseButton: function () {
        this.listener.onSelectKeyFile(this);
    },

    _onClick$DebugDirBrowseButton: function () {
        this.listener.onSelectDebugDir(this);
    },

    _adjustSecurityMode: function () {
        var enabled = !!this.keyUsage[this.getSecurityMode()];
        this.keyFile.prop("disabled", !enabled);
        this.keyFilePassword.prop("disabled", !enabled);
        this.keyFileBrowseButton.linkbutton(enabled ? "enable" : "disable");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.AftpReverseProxyDialog.LISTENER = {
    onTest: $.noop,
    onSelectKeyFile: $.noop,
    onSelectDebugDir: $.noop
};

/** @class */
jscape.domain.AddAftpReverseProxyDialog = jscape.domain.AftpReverseProxyDialog.extend(
    /** @lends {jscape.domain.AddAftpReverseProxyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-add-aftp"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["proxy_name_nonempty", "proxy_unique"], missingMessage: resources.getString("PROXIES_ERROR_BAD_NAME")});
    },

    /** @override */
    getName: function() {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditAftpReverseProxyDialog = jscape.domain.AftpReverseProxyDialog.extend(
    /** @lends {jscape.domain.EditAftpReverseProxyDialog.prototype} */
{
    proxyName: "",

    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-edit-aftp"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    /** @override */
    getName: function() {
        return this.proxyName;
    },

    setName: function(value) {
        this.proxyName = value;
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});

/** @class */
jscape.domain.AddWebDavReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.AddWebDavReverseProxyController.prototype} */
{
    /** @const */
    DEFAULT_PORT: 80,

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.AddWebDavReverseProxyDialog(this.resources);
        dialog.setPort(this.DEFAULT_PORT);
        dialog.setUsername("");
        dialog.setPassword("");
        dialog.setPath("/");
        dialog.setMapLocalDirectoryToRemote(false);
        dialog.setDebugDirectory(null);
        return dialog;
    },

    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.webdav(dialog.getName(),
                                                dialog.getHost(),
                                                dialog.getPort(),
                                                dialog.getDomain(),
                                                dialog.getPath(),
                                                dialog.isUseSsl(),
                                                dialog.getUsername(),
                                                dialog.getPassword(),
                                                dialog.getUsername() == null,
                                                new jscape.RemoteDirectory(dialog.isMapLocalDirectoryToRemote()),
                                                dialog.getDebugDirectory());
    }
});

/** @class */
jscape.domain.EditWebDavReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.EditWebDavReverseProxyController.prototype} */
{
    /** @override */
    start: function(listener, managementService, domainName, descriptor) {
        this.descriptor = descriptor;
        return this._super(listener, managementService, domainName);
    },

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.EditWebDavReverseProxyDialog(this.resources);
        dialog.setName(this.descriptor.name);
        dialog.setHost(this.descriptor.host);
        dialog.setPort(this.descriptor.port);
        dialog.setUsername(this.descriptor.username);
        dialog.setPassword(this.descriptor.password);
        dialog.setPath(this.descriptor.path);
        dialog.setDomain(this.descriptor.domain);
        dialog.setUseSsl(this.descriptor.useSsl);
        dialog.setMapLocalDirectoryToRemote(this.descriptor.remoteDirectory.mapLocalToRemote);
        dialog.setDebugDirectory(this.descriptor.logDirectory);
        return dialog;
    },

    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.webdav(this.descriptor.name,
                                                dialog.getHost(),
                                                dialog.getPort(),
                                                dialog.getDomain(),
                                                dialog.getPath(),
                                                dialog.isUseSsl(),
                                                dialog.getUsername(),
                                                dialog.getPassword(),
                                                dialog.getUsername() == null,
                                                new jscape.RemoteDirectory(dialog.isMapLocalDirectoryToRemote()),
                                                dialog.getDebugDirectory());
    }
});

/** @class */
jscape.domain.WebDavReverseProxyDialog = Class.extend(
    /** @lends {jscape.domain.WebDavReverseProxyDialog.prototype} */
{
    /** @cons */
    TIMEOUT_FACTOR: 60 * 1000,

    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=reverseproxyfields]", this.dialog);

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, validType: "proxy_host_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 100});

        this.path = $("input[name=path]", this.fieldset);
        this.domain = $("input[name=domain]", this.fieldset);

        this.credentials = $("input[name=credentials]:radio", this.fieldset);
        this.enterCredentials = this.credentials.filter("[value=enter]");
        this.replayCredentials = this.credentials.filter("[value=replay]");
        this.credentials.change($.proxy(function() {
            var enabled = this.enterCredentials.is(":checked");
            this.username.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.password.prop("disabled", !enabled);
        }, this));

        this.username = $("input[name=username]", this.fieldset);
        this.username.validatebox({required: true, validType: "proxy_username_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_USER")});
        this.password = $("input[name=userpwd]", this.fieldset);

        this.useSsl = $("input[name=ssl]:checkbox", this.fieldset);
        this.mapLocalDir = $("input[name=maplocaldir]:checkbox", this.fieldset);

        this.debugLog = $("input[name=debuglog]:checkbox", this.fieldset);
        this.debugLog.change($.proxy(function() {
            var enabled = this.debugLog.is(":checked");
            this.debugDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.debugDirBrowseButton.linkbutton(enabled ? "enable" : "disable");
        }, this));
        this.debugDir = $("input[name=debugdir]", this.fieldset);
        this.debugDir.validatebox({required: true, validaType: "proxy_debugdir_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_DEBUG_DIR")});
        this.debugDirBrowseButton = $("a[role=button][name=debugdirbrowse]", this.fieldset);
        this.debugDirBrowseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DebugDirBrowseButton, this)});

        this.testButton = $("a[role=button][name=test]", this.dialog);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getName: function() {
        throw "Abstract method.";
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value);
    },

    getPath: function() {
        return this.path.val();
    },

    setPath: function(value) {
        this.path.val(value);
    },

    getDomain: function() {
        return this.domain.val();
    },

    setDomain: function(value) {
        this.domain.val(value);
    },

    getUsername: function() {
        return this.enterCredentials.is(":checked") ? this.username.val() : null;
    },

    setUsername: function(value) {
        if (value != null) {
            this.username.val(value);
            this.enterCredentials.prop("checked", true).change();
        } else {
            this.replayCredentials.prop("checked", true).change();
        }
    },

    getPassword: function() {
        return this.enterCredentials.is(":checked") ? this.password.val() : null;
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    isUseSsl: function() {
        return this.useSsl.is(":checked");
    },

    setUseSsl: function(value) {
        this.useSsl.prop("checked", !!value);
    },

    isMapLocalDirectoryToRemote: function() {
        return this.mapLocalDir.is(":checked");
    },

    setMapLocalDirectoryToRemote: function(value) {
        this.mapLocalDir.prop("checked", !!value);
    },

    getDebugDirectory: function() {
        return this.debugLog.is(":checked") ? this.debugDir.val() : null;
    },

    setDebugDirectory: function(value) {
        this.debugLog.prop("checked", value != null).change();
        this.debugDir.val(value || "");
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.WebDavReverseProxyDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$TestButton: function() {
        if (this.fieldset.form("validate")) {
            this.listener.onTest(this);
        }
    },

    _onClick$DebugDirBrowseButton: function () {
        this.listener.onSelectDebugDir(this);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.WebDavReverseProxyDialog.LISTENER = {
    onTest: $.noop,
    onSelectDebugDir: $.noop
};

/** @class */
jscape.domain.AddWebDavReverseProxyDialog = jscape.domain.WebDavReverseProxyDialog.extend(
    /** @lends {jscape.domain.AddWebDavReverseProxyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-add-webdav"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["proxy_name_nonempty", "proxy_unique"], missingMessage: resources.getString("PROXIES_ERROR_BAD_NAME")});
    },

    /** @override */
    getName: function() {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditWebDavReverseProxyDialog = jscape.domain.WebDavReverseProxyDialog.extend(
    /** @lends {jscape.domain.EditWebDavReverseProxyDialog.prototype} */
{
    proxyName: "",

    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-edit-webdav"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    /** @override */
    getName: function() {
        return this.proxyName;
    },

    setName: function(value) {
        this.proxyName = value;
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});

/** @class */
jscape.domain.AddRestReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.AddRestReverseProxyController.prototype} */
{
    /** @const */
    DEFAULT_PORT: 80,
    DEFAULT_TIMEOUT: 1000,
    DEFAULT_MAX_AGE: 10 * 60 * 1000,

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.AddRestReverseProxyDialog(this.resources);
        dialog.setPort(this.DEFAULT_PORT);
        dialog.setTimeout(this.DEFAULT_TIMEOUT);
        dialog.setUsername("");
        dialog.setPassword("");
        dialog.setMapLocalDirectoryToRemote(false);
        dialog.setDebugDirectory(null);
        dialog.setMaxAge(this.DEFAULT_MAX_AGE);
        return dialog;
    },

    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.rest(dialog.getName(),
                                              dialog.getHost(),
                                              dialog.getPort(),
                                              dialog.getTimeout(),
                                              dialog.isSslRequired(),
                                              dialog.getDomain(),
                                              dialog.getUsername(),
                                              dialog.getPassword(),
                                              dialog.getUsername() == null,
                                              dialog.getKeyFile().length != 0 ? new jscape.FileKeyProvider(dialog.getKeyFile(), dialog.getKeyFilePassword()) : null,
                                              new jscape.RemoteDirectory(dialog.isMapLocalDirectoryToRemote()),
                                              dialog.getDebugDirectory(),
                                              dialog.getMaxAge());
    }
});

/** @class */
jscape.domain.EditRestReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.EditRestReverseProxyController.prototype} */
{
    /** @override */
    start: function(listener, managementService, domainName, descriptor) {
        this.descriptor = descriptor;
        return this._super(listener, managementService, domainName);
    },

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.EditRestReverseProxyDialog(this.resources);
        dialog.setName(this.descriptor.name);
        dialog.setHost(this.descriptor.host);
        dialog.setPort(this.descriptor.port);
        dialog.setTimeout(this.descriptor.timeout);
        dialog.setSslRequired(this.descriptor.sslRequired);
        dialog.setDomain(this.descriptor.domain);
        dialog.setUsername(this.descriptor.username);
        dialog.setPassword(this.descriptor.password);
        if (this.descriptor.keyProvider) {
            dialog.setKeyFile(this.descriptor.keyProvider.file);
            dialog.setKeyFilePassword(this.descriptor.keyProvider.filePassword);
        }
        dialog.setMapLocalDirectoryToRemote(this.descriptor.remoteDirectory.mapLocalToRemote);
        dialog.setDebugDirectory(this.descriptor.logDirectory);
        dialog.setMaxAge(this.descriptor.timeToLive);
        return dialog;
    },

    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.rest(this.descriptor.name,
                                              dialog.getHost(),
                                              dialog.getPort(),
                                              dialog.getTimeout(),
                                              dialog.isSslRequired(),
                                              dialog.getDomain(),
                                              dialog.getUsername(),
                                              dialog.getPassword(),
                                              dialog.getUsername() == null,
                                              dialog.getKeyFile().length != 0 ? new jscape.FileKeyProvider(dialog.getKeyFile(), dialog.getKeyFilePassword()) : null,
                                              new jscape.RemoteDirectory(dialog.isMapLocalDirectoryToRemote()),
                                              dialog.getDebugDirectory(),
                                              dialog.getMaxAge());
    }
});

/** @class */
jscape.domain.RestReverseProxyDialog = Class.extend(
    /** @lends {jscape.domain.RestReverseProxyDialog.prototype} */
{
    /** @cons */
    TIMEOUT_FACTOR: 60 * 1000,

    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=reverseproxyfields]", this.dialog);

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, validType: "proxy_host_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 100});

        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({required: true, min: 1, max: 60, width: 60});

        this.sslRequired = $("input[name=ssl]:checkbox", this.fieldset);
        this.domain = $("input[name=domain]", this.fieldset);
        this.domain.validatebox({required: true});

        this.credentials = $("input[name=credentials]:radio", this.fieldset);
        this.enterCredentials = this.credentials.filter("[value=enter]");
        this.replayCredentials = this.credentials.filter("[value=replay]");
        this.credentials.change($.proxy(function() {
            var enabled = this.enterCredentials.is(":checked");
            this.username.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.password.prop("disabled", !enabled);
        }, this));

        this.username = $("input[name=username]", this.fieldset);
        this.username.validatebox({required: true, validType: "proxy_username_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_USER")});
        this.password = $("input[name=userpwd]", this.fieldset);

        this.keyFile = $("input[name=keyfile]", this.fieldset);
        this.keyFilePassword = $("input[name=keyfilepwd]", this.fieldset);
        this.keyFileBrowseButton = $("a[role=button][name=keyfilebrowse]", this.fieldset);
        this.keyFileBrowseButton.linkbutton({onClick: $.proxy(this._onClick$KeyFileBrowseButton, this)});

        this.mapLocalDir = $("input[name=maplocaldir]:checkbox", this.fieldset);

        this.debugLog = $("input[name=debuglog]:checkbox", this.fieldset);
        this.debugLog.change($.proxy(function() {
            var enabled = this.debugLog.is(":checked");
            this.debugDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.debugDirBrowseButton.linkbutton(enabled ? "enable" : "disable");
        }, this));
        this.debugDir = $("input[name=debugdir]", this.fieldset);
        this.debugDir.validatebox({required: true, validaType: "proxy_debugdir_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_DEBUG_DIR")});
        this.debugDirBrowseButton = $("a[role=button][name=debugdirbrowse]", this.fieldset);
        this.debugDirBrowseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DebugDirBrowseButton, this)});

        this.maxAge = $("input[name=maxage]:checkbox", this.fieldset);
        this.maxAge.change($.proxy(function () {
            var enabled = this.maxAge.is(":checked");
            this.ttl.numberspinner(enabled ? "enable" : "disable").validatebox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.ttl = $("input[name=ttl]", this.fieldset);
        this.ttl.numberspinner({required: true, disabled: true, min: 1, max: 1000, width: 70});

        this.testButton = $("a[role=button][name=test]", this.dialog);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getName: function() {
        throw "Abstract method.";
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value);
    },

    getTimeout: function() {
        return this.timeout.numberspinner("getValue") * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    isSslRequired: function() {
        return this.sslRequired.is(":checked");
    },

    setSslRequired: function(value) {
        this.sslRequired.prop("checked", !!value);
    },

    getDomain: function() {
        return this.domain.val();
    },

    setDomain: function(value) {
        this.domain.val(value);
    },

    getUsername: function() {
        return this.enterCredentials.is(":checked") ? this.username.val() : null;
    },

    setUsername: function(value) {
        if (value != null) {
            this.username.val(value);
            this.enterCredentials.prop("checked", true).change();
        } else {
            this.replayCredentials.prop("checked", true).change();
        }
    },

    getPassword: function() {
        return this.enterCredentials.is(":checked") ? this.password.val() : null;
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getKeyFile: function() {
        return this.keyFile.val().trim();
    },

    setKeyFile: function(value) {
        this.keyFile.val(value);
    },

    getKeyFilePassword: function() {
        return this.keyFilePassword.val();
    },

    setKeyFilePassword: function(value) {
        this.keyFilePassword.val(value);
    },

    isMapLocalDirectoryToRemote: function() {
        return this.mapLocalDir.is(":checked");
    },

    setMapLocalDirectoryToRemote: function(value) {
        this.mapLocalDir.prop("checked", !!value);
    },

    getDebugDirectory: function() {
        return this.debugLog.is(":checked") ? this.debugDir.val() : null;
    },

    setDebugDirectory: function(value) {
        this.debugLog.prop("checked", value != null).change();
        this.debugDir.val(value || "");
    },

    getMaxAge: function () {
        return this.maxAge.is(":checked") ? this.ttl.numberspinner("getValue") * this.TIMEOUT_FACTOR : null;
    },

    setMaxAge: function (value) {
        value = parseInt(value);
        this.maxAge.prop("checked", !isNaN(value)).change();
        this.ttl.numberspinner("setValue", !isNaN(value) ? value / this.TIMEOUT_FACTOR : this.DEFAULT_MAX_AGE);
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.RestReverseProxyDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$TestButton: function() {
        if (this.fieldset.form("validate")) {
            this.listener.onTest(this);
        }
    },

    _onClick$KeyFileBrowseButton: function () {
        this.listener.onSelectKeyFile(this);
    },

    _onClick$DebugDirBrowseButton: function () {
        this.listener.onSelectDebugDir(this);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.RestReverseProxyDialog.LISTENER = {
    onTest: $.noop,
    onSelectKeyFile: $.noop,
    onSelectDebugDir: $.noop
};

/** @class */
jscape.domain.AddRestReverseProxyDialog = jscape.domain.RestReverseProxyDialog.extend(
    /** @lends {jscape.domain.AddRestReverseProxyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-add-rest"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["proxy_name_nonempty", "proxy_unique"], missingMessage: resources.getString("PROXIES_ERROR_BAD_NAME")});
    },

    /** @override */
    getName: function() {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditRestReverseProxyDialog = jscape.domain.RestReverseProxyDialog.extend(
    /** @lends {jscape.domain.EditRestReverseProxyDialog.prototype} */
{
    proxyName: "",

    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-edit-rest"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    /** @override */
    getName: function() {
        return this.proxyName;
    },

    setName: function(value) {
        this.proxyName = value;
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});

/** @class */
jscape.domain.AddAmazonS3ReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.AddAmazonS3ReverseProxyController.prototype} */
{
    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.AddAmazonS3ReverseProxyDialog(this.resources);
        dialog.setUsername("");
        dialog.setPassword("");
        dialog.setMapLocalDirectoryToRemote(false);
        dialog.setDebugDirectory(null);
        return dialog;
    },

    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.amazonS3(dialog.getName(),
                                                  dialog.isUseSsl(),
                                                  dialog.getUsername(),
                                                  dialog.getPassword(),
                                                  dialog.getUsername() == null,
                                                  new jscape.RemoteDirectory(dialog.isMapLocalDirectoryToRemote()),
                                                  dialog.getDebugDirectory());
    }
});

/** @class */
jscape.domain.EditAmazonS3ReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.EditAmazonS3ReverseProxyController.prototype} */
{
    /** @override */
    start: function(listener, managementService, domainName, descriptor) {
        this.descriptor = descriptor;
        return this._super(listener, managementService, domainName);
    },

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.EditAmazonS3ReverseProxyDialog(this.resources);
        dialog.setName(this.descriptor.name);
        dialog.setUseSsl(this.descriptor.encryptionRequired);
        dialog.setUsername(this.descriptor.username);
        dialog.setPassword(this.descriptor.password);
        dialog.setMapLocalDirectoryToRemote(this.descriptor.remoteDirectory.mapLocalToRemote);
        dialog.setDebugDirectory(this.descriptor.logDirectory);
        return dialog;
    },

    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.amazonS3(this.descriptor.name,
                                                  dialog.isUseSsl(),
                                                  dialog.getUsername(),
                                                  dialog.getPassword(),
                                                  dialog.getUsername() == null,
                                                  new jscape.RemoteDirectory(dialog.isMapLocalDirectoryToRemote()),
                                                  dialog.getDebugDirectory());
    }
});

/** @class */
jscape.domain.AmazonS3ReverseProxyDialog = Class.extend(
    /** @lends {jscape.domain.AmazonS3ReverseProxyDialog.prototype} */
{
    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=reverseproxyfields]", this.dialog);

        this.ssl = $("input[name=ssl]:checkbox", this.fieldset);

        this.credentials = $("input[name=credentials]:radio", this.fieldset);
        this.enterCredentials = this.credentials.filter("[value=enter]");
        this.replayCredentials = this.credentials.filter("[value=replay]");
        this.credentials.change($.proxy(function() {
            var enabled = this.enterCredentials.is(":checked");
            this.username.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.password.prop("disabled", !enabled);
        }, this));

        this.username = $("input[name=username]", this.fieldset);
        this.username.validatebox({required: true, validType: "proxy_username_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_USER")});
        this.password = $("input[name=password]", this.fieldset);

        this.mapLocalDir = $("input[name=maplocaldir]:checkbox", this.fieldset);

        this.debugLog = $("input[name=debuglog]:checkbox", this.fieldset);
        this.debugLog.change($.proxy(function() {
            var enabled = this.debugLog.is(":checked");
            this.debugDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.debugDirBrowseButton.linkbutton(enabled ? "enable" : "disable");
        }, this));
        this.debugDir = $("input[name=debugdir]", this.fieldset);
        this.debugDir.validatebox({required: true, validaType: "proxy_debugdir_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_DEBUG_DIR")});
        this.debugDirBrowseButton = $("a[role=button][name=debugdirbrowse]", this.fieldset);
        this.debugDirBrowseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DebugDirBrowseButton, this)});

        this.testButton = $("a[role=button][name=test]", this.dialog);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getName: function() {
        throw "Abstract method.";
    },

    isUseSsl: function() {
        return this.ssl.is(":checked");
    },

    setUseSsl: function(value) {
        this.ssl.prop("checked", !!value);
    },

    getUsername: function() {
        return this.enterCredentials.is(":checked") ? this.username.val() : null;
    },

    setUsername: function(value) {
        if (value != null) {
            this.username.val(value);
            this.enterCredentials.prop("checked", true).change();
        } else {
            this.replayCredentials.prop("checked", true).change();
        }
    },

    getPassword: function() {
        return this.enterCredentials.is(":checked") ? this.password.val() : null;
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    isMapLocalDirectoryToRemote: function() {
        return this.mapLocalDir.is(":checked");
    },

    setMapLocalDirectoryToRemote: function(value) {
        this.mapLocalDir.prop("checked", !!value);
    },

    getDebugDirectory: function() {
        return this.debugLog.is(":checked") ? this.debugDir.val() : null;
    },

    setDebugDirectory: function(value) {
        this.debugLog.prop("checked", value != null).change();
        this.debugDir.val(value || "");
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.RestReverseProxyDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$TestButton: function() {
        if (this.fieldset.form("validate")) {
            this.listener.onTest(this);
        }
    },

    _onClick$DebugDirBrowseButton: function () {
        this.listener.onSelectDebugDir(this);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.AmazonS3ReverseProxyDialog.LISTENER = {
    onTest: $.noop,
    onSelectDebugDir: $.noop
};

/** @class */
jscape.domain.AddAmazonS3ReverseProxyDialog = jscape.domain.AmazonS3ReverseProxyDialog.extend(
    /** @lends {jscape.domain.AddAmazonS3ReverseProxyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-add-amazons3"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["proxy_name_nonempty", "proxy_unique"], missingMessage: resources.getString("PROXIES_ERROR_BAD_NAME")});
    },

    /** @override */
    getName: function() {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditAmazonS3ReverseProxyDialog = jscape.domain.AmazonS3ReverseProxyDialog.extend(
    /** @lends {jscape.domain.EditAmazonS3ReverseProxyDialog.prototype} */
{
    proxyName: "",

    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-edit-amazons3"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    /** @override */
    getName: function() {
        return this.proxyName;
    },

    setName: function(value) {
        this.proxyName = value;
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});

/** @class */
jscape.domain.AddOftpReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.AddOftpReverseProxyController.prototype} */
{
    DEFAULT_PORT: 3305,
    DEFAULT_MAX_AGE: 10 * 60 * 1000,

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.AddOftpReverseProxyDialog(this.resources);
        dialog.setPort(this.DEFAULT_PORT);
        dialog.setUseSsl(false);
        dialog.setUsername("");
        dialog.setDebugDirectory(null);
        dialog.setMaxAge(this.DEFAULT_MAX_AGE);
        this._loadEncryptionKeys()
                .done($.proxy(function(keys) {
                    dialog.setEncryptionKeys($.map(keys, function(key) { return key.alias; }));
                }, this));
        this._loadSigningKeys()
                .done($.proxy(function(keys) {
                    dialog.setSigningKeys($.map(keys, function(key) { return key.alias; }));
                }, this));
        return dialog;
    },

    _createDescriptor: function (dialog) {
        var signingKeyProvider = dialog.getSigningKey() != null ? new jscape.ServerKeyProvider(dialog.getSigningKey()) : null;
        var encryptionKeyProvider = dialog.getEncryptionKey() != null ? new jscape.ServerKeyProvider(dialog.getEncryptionKey()) : null;
        var cipherSuite = dialog.getEncryptionKey() != null || dialog.getSigningKey() != null ? dialog.getCipher() : null;
        return jscape.ResourceDescriptor.oftp(dialog.getName(),
                                              dialog.getHost(),
                                              dialog.getPort(),
                                              dialog.isUseSsl(),
                                              dialog.getUsername(),
                                              dialog.getPassword(),
                                              dialog.getUsername() == null,
                                              null,
                                              null,
                                              dialog.getRecipient(),
                                              encryptionKeyProvider,
                                              signingKeyProvider,
                                              cipherSuite,
                                              dialog.getDebugDirectory(),
                                              dialog.getMaxAge());
    },

    _loadEncryptionKeys: function() {
        return this.managementService.getHostCertificates()
                .then(function(entries) { return $.grep(entries || [], function(entry) { return "RSA" == entry.algorithm; }); });
    },

    _loadSigningKeys: function() {
        return this.managementService.getServerKeys()
                .then(function(entries) { return $.grep(entries || [], function(entry) { return "RSA" == entry.algorithm; }); });
    }
});

/** @class */
jscape.domain.EditOftpReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.EditOftpReverseProxyController.prototype} */
{
    /** @override */
    start: function(listener, managementService, domainName, descriptor) {
        this.descriptor = descriptor;
        return this._super(listener, managementService, domainName);
    },

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.EditOftpReverseProxyDialog(this.resources);
        dialog.setName(this.descriptor.name);
        dialog.setHost(this.descriptor.host);
        dialog.setPort(this.descriptor.port);
        dialog.setUseSsl(this.descriptor.sslRequired);
        dialog.setUsername(this.descriptor.username);
        dialog.setPassword(this.descriptor.password);
        dialog.setRecipient(this.descriptor.destination);
        dialog.setCipher(this.descriptor.cipherSuite);
        dialog.setDebugDirectory(this.descriptor.logDirectory);
        dialog.setMaxAge(this.descriptor.timeToLive);

        this._loadEncryptionKeys(dialog)
                .done($.proxy(function(keys) {
                    dialog.setEncryptionKeys($.map(keys, function(key) { return key.alias; }));
                    dialog.setEncryptionKey(this.descriptor.encryptionKeyProvider ? this.descriptor.encryptionKeyProvider.keyAlias : null);
                }, this));

        this._loadSigningKeys(dialog)
                .done($.proxy(function(keys) {
                    dialog.setSigningKeys($.map(keys, function(key) { return key.alias; }));
                    dialog.setSigningKey(this.descriptor.signingKeyProvider ? this.descriptor.signingKeyProvider.keyAlias : null);
                }, this));

        return dialog;
    },

    _createDescriptor: function (dialog) {
        var signingKeyProvider = dialog.getSigningKey() != null ? new jscape.ServerKeyProvider(dialog.getSigningKey()) : null;
        var encryptionKeyProvider = dialog.getEncryptionKey() != null ? new jscape.ServerKeyProvider(dialog.getEncryptionKey()) : null;
        var cipherSuite = dialog.getEncryptionKey() != null || dialog.getSigningKey() != null ? dialog.getCipher() : null;
        return jscape.ResourceDescriptor.oftp(this.descriptor.name,
                                              dialog.getHost(),
                                              dialog.getPort(),
                                              dialog.isUseSsl(),
                                              dialog.getUsername(),
                                              dialog.getPassword(),
                                              dialog.getUsername() == null,
                                              null,
                                              null,
                                              dialog.getRecipient(),
                                              encryptionKeyProvider,
                                              signingKeyProvider,
                                              cipherSuite,
                                              dialog.getDebugDirectory(),
                                              dialog.getMaxAge());
    },

    _loadEncryptionKeys: function() {
        return this.managementService.getHostCertificates()
                .then(function(entries) { return $.grep(entries || [], function(entry) { return "RSA" == entry.algorithm; }); });
    },

    _loadSigningKeys: function() {
        return this.managementService.getServerKeys()
                .then(function(entries) { return $.grep(entries || [], function(entry) { return "RSA" == entry.algorithm; }); });
    }
});

/** @class */
jscape.domain.OftpReverseProxyDialog = Class.extend(
    /** @lends {jscape.domain.OftpReverseProxyDialog.prototype} */
{
    TIMEOUT_FACTOR: 60 * 1000,
    DEFAULT_MAX_AGE: 10,

    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=reverseproxyfields]", this.dialog);

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, validType: "proxy_host_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 100});

        this.ssl = $("input[name=ssl]:checkbox", this.fieldset);

        this.credentials = $("input[name=credentials]:radio", this.fieldset);
        this.enterCredentials = this.credentials.filter("[value=enter]");
        this.replayCredentials = this.credentials.filter("[value=replay]");
        this.credentials.change($.proxy(function() {
            var enabled = this.enterCredentials.is(":checked");
            this.username.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation").validatebox("validate");
            this.password.prop("disabled", !enabled);
        }, this));

        this.username = $("input[name=username]", this.fieldset);
        this.username.validatebox({required: true, validType: "proxy_username_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_USER")});
        this.password = $("input[name=userpwd]", this.fieldset);

        this.recipient = $("input[name=recipient]", this.fieldset);
        this.recipient.validatebox({required: true, missingMessage: resources.getString("PROXIES_ERROR_BAD_RECIPIENT")});

        this.useEncryptionKey = $("input[name=useencryptionkey]:checkbox", this.fieldset).prop("disabled", true);
        this.useEncryptionKey.change($.proxy(function() {
            this.encryptionKey.combobox(this.useEncryptionKey.is(":checked") ? "enable" : "disable");
            this._adjustCipherFieldState();
        }, this));
        this.encryptionKey = $("select[name=encryptionkey]", this.fieldset);
        this.encryptionKey.combobox({
            editable: false,
            disabled: true,
            width: 170,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                this.useEncryptionKey.prop("disabled", data.length == 0);
                if (data.length != 0) {
                    this.encryptionKey.combobox("select", data[0].value);
                }
            }, this)
        });

        this.useSigningKey = $("input[name=usesigningkey]:checkbox", this.fieldset).prop("disabled", true);
        this.useSigningKey.change($.proxy(function() {
            this.signingKey.combobox(this.useSigningKey.is(":checked") ? "enable" : "disable");
            this._adjustCipherFieldState();
        }, this));
        this.signingKey = $("select[name=signingkey]", this.fieldset);
        this.signingKey.combobox({
            editable: false,
            disabled: true,
            width: 170,
            panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                this.useSigningKey.prop("disabled", data.length == 0);
                if (data.length != 0) {
                    this.signingKey .combobox("select", data[0].value);
                }
            }, this)
        });

        this.cipher = $("select[name=cipher]", this.fieldset);
        this.cipher.combobox({required: true, editable: false, panelHeight: "auto"});

        this.debugLog = $("input[name=debuglog]:checkbox", this.fieldset);
        this.debugLog.change($.proxy(function() {
            var enabled = this.debugLog.is(":checked");
            this.debugDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.debugDirBrowseButton.linkbutton(enabled ? "enable" : "disable");
        }, this));
        this.debugDir = $("input[name=debugdir]", this.fieldset);
        this.debugDir.validatebox({required: true, validaType: "proxy_debugdir_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_DEBUG_DIR")});
        this.debugDirBrowseButton = $("a[role=button][name=debugdirbrowse]", this.fieldset);
        this.debugDirBrowseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DebugDirBrowseButton, this)});

        this.maxAge = $("input[name=maxage]:checkbox", this.fieldset);
        this.maxAge.change($.proxy(function() {
            var enabled = this.maxAge.is(":checked");
            this.ttl.numberspinner(enabled ? "enable" : "disable").validatebox(enabled ? "enableValidation" : "disableValidation")
        }, this));
        this.ttl = $("input[name=ttl]", this.fieldset);
        this.ttl.numberspinner({required: true, disabled: true, min: 1, max: 1000, width: 70});

        this.testButton = $("a[role=button][name=test]", this.dialog);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getName: function() {
        throw "Abstract method.";
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", parseInt(value)).numberspinner("validate");
    },

    isUseSsl: function() {
        return this.ssl.is(":checked");
    },

    setUseSsl: function(value) {
        this.ssl.prop("checked", !!value);
    },

    getUsername: function() {
        return this.enterCredentials.is(":checked") ? this.username.val() : null;
    },

    setUsername: function(value) {
        if (value != null) {
            this.username.val(value);
            this.enterCredentials.prop("checked", true).change();
        } else {
            this.replayCredentials.prop("checked", true).change();
        }
    },

    getPassword: function() {
        return this.enterCredentials.is(":checked") ? this.password.val() : null;
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getRecipient: function() {
        return this.recipient.val();
    },

    setRecipient: function(value) {
        this.recipient.val(value);
    },

    getEncryptionKey: function() {
        return this.useEncryptionKey.is(":checked") ? this.encryptionKey.combobox("getValue") : null;
    },

    setEncryptionKey: function(value) {
        this.useEncryptionKey.prop("checked", value != null).change();
        if (value != null) {
            this.encryptionKey.combobox("setValue", value);
        }
    },

    setEncryptionKeys: function (values) {
        values = $.map(values, function (value) { return {text: value, value: value}; });
        this.encryptionKey.combobox("loadData", values);
    },

    getSigningKey: function() {
        return this.useSigningKey.is(":checked") ? this.signingKey.combobox("getValue") : null;
    },

    setSigningKey: function(value) {
        this.useSigningKey.prop("checked", value != null).change();
        if (value != null) {
            this.signingKey.combobox("setValue", value);
        }
    },

    setSigningKeys: function (values) {
        values = $.map(values, function (value) { return {text: value, value: value}; });
        this.signingKey.combobox("loadData", values);
    },

    getCipher: function() {
        return this.cipher.is(":disabled") ? null : this.cipher.combobox("getValue");
    },

    setCipher: function(value) {
        this.cipher.combobox("select", value);
    },

    getDebugDirectory: function() {
        return this.debugLog.is(":checked") ? this.debugDir.val() : null;
    },

    setDebugDirectory: function(value) {
        this.debugLog.prop("checked", value != null).change();
        this.debugDir.val(value || "");
    },

    getMaxAge: function() {
        return this.maxAge.is(":checked") ? this.ttl.numberspinner("getValue") * this.TIMEOUT_FACTOR : null;
    },

    setMaxAge: function(value) {
        value = parseInt(value);
        this.maxAge.prop("checked", !isNaN(value)).change();
        this.ttl.numberspinner("setValue", !isNaN(value) ? value / this.TIMEOUT_FACTOR : this.DEFAULT_MAX_AGE);
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.OftpReverseProxyDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this._adjustCipherFieldState();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$TestButton: function() {
        if (this.fieldset.form("validate")) {
            this.listener.onTest(this);
        }
    },

    _onClick$DebugDirBrowseButton: function () {
        this.listener.onSelectDebugDir(this);
    },

    _adjustCipherFieldState: function(){
        this.cipher.combobox(this.getEncryptionKey() != null || this.getSigningKey() != null ? "enable" : "disable");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.OftpReverseProxyDialog.LISTENER = {
    onTest: $.noop,
    onSelectDebugDir: $.noop
};

/** @class */
jscape.domain.AddOftpReverseProxyDialog = jscape.domain.OftpReverseProxyDialog.extend(
    /** @lends {jscape.domain.AddOftpReverseProxyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-add-oftp"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["proxy_name_nonempty", "proxy_unique"], missingMessage: resources.getString("PROXIES_ERROR_BAD_NAME")});
    },

    /** @override */
    getName: function() {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditOftpReverseProxyDialog = jscape.domain.OftpReverseProxyDialog.extend(
    /** @lends {jscape.domain.EditOftpReverseProxyDialog.prototype} */
{
    proxyName: "",

    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-edit-oftp"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    /** @override */
    getName: function() {
        return this.proxyName;
    },

    setName: function(value) {
        this.proxyName = value;
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});

/** @class */
jscape.domain.AddTftpReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.AddTftpReverseProxyController.prototype} */
{
    DEFAULT_PORT: 69,
    DEFAULT_INTERVAL: 3000,
    DEFAULT_ATTEMPTS: 6,

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.AddTftpReverseProxyDialog(this.resources);
        dialog.setPort(this.DEFAULT_PORT);
        dialog.setRetransmitInterval(this.DEFAULT_INTERVAL);
        dialog.setMaxRetransmitAttempts(this.DEFAULT_ATTEMPTS);
        dialog.setDebugDirectory(null);
        return dialog;
    },

    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.tftp(dialog.getName(),
                                              dialog.getHost(),
                                              dialog.getPort(),
                                              dialog.getRetransmitInterval(),
                                              dialog.getMaxRetransmitAttempts(),
                                              dialog.getDebugDirectory());
    }
});

/** @class */
jscape.domain.EditTftpReverseProxyController = jscape.domain.BaseReverseProxyController.extend(
    /** @lends {jscape.domain.EditTftpReverseProxyController.prototype} */
{
    /** @override */
    start: function(listener, managementService, domainName, descriptor) {
        this.descriptor = descriptor;
        return this._super(listener, managementService, domainName);
    },

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.EditTftpReverseProxyDialog(this.resources);
        dialog.setName(this.descriptor.name);
        dialog.setHost(this.descriptor.host);
        dialog.setPort(this.descriptor.port);
        dialog.setRetransmitInterval(this.descriptor.retransmitTimeout);
        dialog.setMaxRetransmitAttempts(this.descriptor.maxRetransmitAttempts);
        dialog.setDebugDirectory(this.descriptor.logDirectory);
        return dialog;
    },

    _createDescriptor: function (dialog) {
        return jscape.ResourceDescriptor.tftp(this.descriptor.name,
                                              dialog.getHost(),
                                              dialog.getPort(),
                                              dialog.getRetransmitInterval(),
                                              dialog.getMaxRetransmitAttempts(),
                                              dialog.getDebugDirectory());
    }
});

/** @class */
jscape.domain.TftpReverseProxyDialog = Class.extend(
    /** @lends {jscape.domain.TftpReverseProxyDialog.prototype} */
{
    TIME_FACTOR: 1000,

    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=reverseproxyfields]", this.dialog);

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, validType: "proxy_host_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 100});

        this.retransmitInterval = $("input[name=retransmitinterval]", this.fieldset);
        this.retransmitInterval.numberspinner({required: true, min: 1, max: 255, width: 60});

        this.retransmitAttempts = $("input[name=retransmitattempts]", this.fieldset);
        this.retransmitAttempts.numberspinner({required: true, min: 1, max: 100, width: 60});

        this.debugLog = $("input[name=debuglog]:checkbox", this.fieldset);
        this.debugLog.change($.proxy(function() {
            var enabled = this.debugLog.is(":checked");
            this.debugDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            this.debugDirBrowseButton.linkbutton(enabled ? "enable" : "disable");
        }, this));
        this.debugDir = $("input[name=debugdir]", this.fieldset);
        this.debugDir.validatebox({required: true, validaType: "proxy_debugdir_nonempty", missingMessage: resources.getString("PROXIES_ERROR_BAD_DEBUG_DIR")});
        this.debugDirBrowseButton = $("a[role=button][name=debugdirbrowse]", this.fieldset);
        this.debugDirBrowseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DebugDirBrowseButton, this)});

        this.testButton = $("a[role=button][name=test]", this.dialog);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getName: function() {
        throw "Abstract method.";
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getPort: function() {
        return this.port.numberspinner("getValue");
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", parseInt(value)).numberspinner("validate");
    },

    getRetransmitInterval: function() {
        return parseInt(this.retransmitInterval.numberspinner("getValue")) * this.TIME_FACTOR;
    },

    setRetransmitInterval: function(value) {
        this.retransmitInterval.numberspinner("setValue", parseInt(value) / this.TIME_FACTOR);
    },

    getMaxRetransmitAttempts: function() {
        return parseInt(this.retransmitAttempts.numberspinner("getValue"));
    },

    setMaxRetransmitAttempts: function(value) {
        this.retransmitAttempts.numberspinner("setValue", parseInt(value)).numberspinner("validate");
    },

    getDebugDirectory: function() {
        return this.debugLog.is(":checked") ? this.debugDir.val() : null;
    },

    setDebugDirectory: function(value) {
        this.debugLog.prop("checked", value != null).change();
        this.debugDir.val(value || "");
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.RestReverseProxyDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$TestButton: function() {
        if (this.fieldset.form("validate")) {
            this.listener.onTest(this);
        }
    },

    _onClick$DebugDirBrowseButton: function () {
        this.listener.onSelectDebugDir(this);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.TftpReverseProxyDialog.LISTENER = {
    onTest: $.noop,
    onSelectDebugDir: $.noop
};

/** @class */
jscape.domain.AddTftpReverseProxyDialog = jscape.domain.TftpReverseProxyDialog.extend(
    /** @lends {jscape.domain.AddTftpReverseProxyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-add-tftp"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["proxy_name_nonempty", "proxy_unique"], missingMessage: resources.getString("PROXIES_ERROR_BAD_NAME")});
    },

    /** @override */
    getName: function() {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditTftpReverseProxyDialog = jscape.domain.TftpReverseProxyDialog.extend(
    /** @lends {jscape.domain.EditTftpReverseProxyDialog.prototype} */
{
    proxyName: "",

    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-proxy-edit-tftp"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    /** @override */
    getName: function() {
        return this.proxyName;
    },

    setName: function(value) {
        this.proxyName = value;
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});

/** @class */
jscape.domain.AddReverseProxyController = Class.extend(
    /** @lends {jscape.domain.AddReverseProxyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.controllerMap = {};
        this.controllerMap[jscape.ResourceDescriptor.Protocol.AFTP] = jscape.domain.AddAftpReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.AMAZON_S3] = jscape.domain.AddAmazonS3ReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.FTP] = jscape.domain.AddFtpReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.OFTP] = jscape.domain.AddOftpReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.REST] = jscape.domain.AddRestReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.SFTP] = jscape.domain.AddSftpReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.WEBDAV] = jscape.domain.AddWebDavReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.TFTP] = jscape.domain.AddTftpReverseProxyController;
    },

    start: function(listener, managementService, domainName) {
        this.listener = listener || jscape.domain.AddReverseProxyController.LISTENER;
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.AddReverseProxyProtocolDialog(this.resources);
        dialog.show(this);

        return this.deferred.promise();
    },

    onTestServer: function(descriptor) {
        this.listener.onTestServer(descriptor);
    },

    onSubmit: function(dialog) {
        var protocol = dialog.getProtocol();
        dialog.destroy();

        var theClass = this.controllerMap[protocol.toUpperCase()];
        var controller = new theClass(this.resources);
        controller.start(this, this.managementService, this.domainName)
                .done($.proxy(this._storeDescriptor, this))
                .fail(this.deferred.reject);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _storeDescriptor: function (descriptor) {
        this.managementService.storeReverseProxy(this.domainName, descriptor)
                .done($.proxy(function () { this.deferred.resolve(descriptor); }, this))
                .fail($.proxy(function () { this.deferred.reject(); }, this));
    }
});
jscape.domain.AddReverseProxyController.LISTENER = {
    onTestServer: $.noop
};

/** @class */
jscape.domain.AddReverseProxyProtocolDialog = Class.extend(
    /** @lends {jscape.domain.AddReverseProxyProtocolDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-proxy-add-protocol").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=proxyprotocolfileds]", this.dialog);
        this.protocol = $("select[name=protocol]", this.fieldset);
        this.protocol.combobox({required: true, editable: false, width: 150, panelHeight: "auto"});
    },

    getProtocol: function() {
        return this.protocol.combobox("getValue");
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.protocol.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.EditReverseProxyController = Class.extend(
    /** @lends {jscape.domain.EditReverseProxyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.controllerMap = {};
        this.controllerMap[jscape.ResourceDescriptor.Protocol.AFTP] = jscape.domain.EditAftpReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.AMAZON_S3] = jscape.domain.EditAmazonS3ReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.FTP] = jscape.domain.EditFtpReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.OFTP] = jscape.domain.EditOftpReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.REST] = jscape.domain.EditRestReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.SFTP] = jscape.domain.EditSftpReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.WEBDAV] = jscape.domain.EditWebDavReverseProxyController;
        this.controllerMap[jscape.ResourceDescriptor.Protocol.TFTP] = jscape.domain.EditTftpReverseProxyController;
    },

    start: function(listener, managementService, domainName, descriptor) {
        this.listener = listener || jscape.domain.EditReverseProxyController.LISTENER;
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        var theClass = this.controllerMap[descriptor.protocol.toUpperCase()];
        var controller = new theClass(this.resources);
        controller.start(this, this.managementService, this.domainName, descriptor)
                        .done($.proxy(this._storeDescriptor, this))
                        .fail(this.deferred.reject);

        return this.deferred.promise();
    },

    onTestServer: function(descriptor) {
        this.listener.onTestServer(descriptor);
    },

    _storeDescriptor: function (descriptor) {
        this.managementService.storeReverseProxy(this.domainName, descriptor)
                .done($.proxy(function () { this.deferred.resolve(descriptor); }, this))
                .fail(this.deferred.reject);
    }
});
jscape.domain.EditReverseProxyController.LISTENER = {
    onTestServer: $.noop
};


/** @class */
jscape.domain.TestReverseProxyController = Class.extend(
    /** @lends {jscape.domain.TestReverseProxyController.prototype} */
{
    descriptor: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, descriptor) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.descriptor = descriptor;
        this.deferred = $.Deferred();

        if (this.descriptor.usingCredentials) {
            var dialog = new jscape.domain.ProxyCredentialsDialog(this.resources);
            dialog.show(this);
        } else {
            this._test(this.descriptor);
        }

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var descriptor = $.extend({}, this.descriptor, {username: dialog.getUsername(), password: dialog.getPassword()});
        dialog.destroy();

        this._test(descriptor);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _test: function(descriptor) {
        this.managementService.testReverseProxy(this.domainName, descriptor)
                .done($.proxy(this._testPassed, this));
    },

    _testPassed: function() {
        $.messager.alert(this.resources.getString("PROXIES_SUCCESS_TEST_TITLE"),
                         this.resources.getString("PROXIES_SUCCESS_TEST_MESSAGE"),
                         "info");
        this.deferred.resolve();
    }
});

/** @class */
jscape.domain.ProxyCredentialsDialog = Class.extend(
    /** @lends {jscape.domain.ProxyCredentialsDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-proxies-test").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=testreverseproxyfields]", this.dialog);

        this.username = $("[name=username]", this.fieldset);
        this.password = $("[name=password]", this.fieldset);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.username.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    getUsername: function() {
        return this.username.val();
    },

    getPassword: function() {
        return this.password.val();
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.DeleteReverseProxyController = Class.extend(
    /** @lends {jscape.domain.DeleteReverseProxyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function (managementService, domainName, proxies) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._confirmDelete(proxies)
                .done($.proxy(this._deleteConfirmed, this, proxies))
                .fail(this.deferred.reject);

        return this.deferred.promise();
    },

    _confirmDelete: function(proxies) {
        var text = $.map(proxies, function(proxy) { return proxy.name; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("PROXIES_CONFIRM_DELETE_TITLE"),
                this.resources.getString("PROXIES_CONFIRM_DELETE_MESSAGE", {0: text}));
    },

    _deleteConfirmed: function(proxies) {
        var promises = new Array(proxies.length);
        for (var i = 0; i < proxies.length; i++) {
            promises[i] = this.managementService.deleteReverseProxy(this.domainName, proxies[i].name);
        }

        $.when.apply(jQuery, promises)
                .done($.proxy(this.deferred.resolve, this.deferred, proxies))
                .fail(this.deferred.reject);
    }
});
