jscape.domain = jscape.domain || {};

/** @class */
jscape.ServiceConfiguration = Class.extend({
    clone: function() {
        var attrs = {};
        $.each(["uploadForms", "languages"], $.proxy(function(_, prop) {
            if ($.type(this[prop]) == "array") {
                attrs[prop] = $.map(this[prop], function(entry) { return entry.clone(); });
            }
        }, this));
        return $.extend(new jscape.ServiceConfiguration(), this, attrs);
    }
});
jscape.ServiceConfiguration.fromJSON = function(data) {
    if (data) {
        if (data.hasOwnProperty("uploadForms")) {
            data["uploadForms"] = $.map(data.uploadForms || [], jscape.UploadForm.fromJSON);
        }
        if (data.hasOwnProperty("languages")) {
            data["languages"] = $.map(data.languages || [], jscape.WebResources.fromJSON);
        }
        return $.extend(new jscape.ServiceConfiguration(), data);
    }
    return null;
};
jscape.ServiceConfiguration.ftp = function(timeout, dataTimeout, banner, passiveIp, portRange, sendBuffer, receiveBuffer, transferMode, ciphers, shutdownCommandSsl, shutdownDataSsl, commandClientCertificate, dataClientCertificate, forceDataEncryption, blockPasv, blockBounce, networkCharset, fileCharset) {
    return $.extend(new jscape.ServiceConfiguration(), {
        timeout: parseInt(timeout) || 0, dataTimeout: parseInt(dataTimeout) || 0, welcomeMessage: banner || "", passiveIp: passiveIp || "", portRange: portRange || null,
        sendBufferSize: parseInt(sendBuffer) || null, receiveBufferSize: parseInt(receiveBuffer) || null, defaultTransferMode: transferMode == "IMAGE" ? transferMode : "ASCII", cipherSuites: ciphers || undefined,
        shutdownSsl: !!shutdownCommandSsl, shutdownDataSsl: !!shutdownDataSsl, commandRequireClientCertificate: !!commandClientCertificate, dataRequireClientCertificate: !!dataClientCertificate, forceDataEncryption: !!forceDataEncryption,
        blockPasvAttack: !!blockPasv, blockBounceAttack: !!blockBounce, networkCharset: networkCharset, fileCharset: fileCharset
    });
};
jscape.ServiceConfiguration.aftp = function(timeout, dataTimeout, maxLossListSize, ciphers) {
    return $.extend(new jscape.ServiceConfiguration(), {timeout: parseInt(timeout) || 0, dataTimeout: parseInt(dataTimeout) || 0, maxLossListSize: parseInt(maxLossListSize) || 0, cipherSuites: ciphers || undefined});
};
jscape.ServiceConfiguration.sftp = function(timeout, version, startupBanner, authBanner, sendBuffer, receiveBuffer, disableExpandedLongName, sshKexes, sshCiphers, sshMacs, sshCompressions) {
    return $.extend(new jscape.ServiceConfiguration(), {
        timeout: parseInt(timeout) || 0, softwareVersion: version || "JSCAPE", startupBanner: startupBanner || "", authBanner: authBanner || "", sendBufferSize: parseInt(sendBuffer) || null, receiveBufferSize: parseInt(receiveBuffer) || null,
        disableRealPathExpandedLongName: !!disableExpandedLongName, supportedKexes: sshKexes, supportedCiphers: sshCiphers, supportedMacs: sshMacs, supportedCompressions: sshCompressions
    });
};
jscape.ServiceConfiguration.oftp = function(timeout, maxDataBufferSize, maxCredit, ciphers, protocols) {
    return $.extend(new jscape.ServiceConfiguration(), {
        timeout: parseInt(timeout) || 0, maxDataExchangeBufferSize: parseInt(maxDataBufferSize) || 0, maxCredit: parseInt(maxCredit) || 0,
        cipherSuites: ciphers || undefined, enabledProtocols: protocols || undefined
    });
};
jscape.ServiceConfiguration.tftp = function(retransmitAttempts, retransmitInterval, dirListingGenerationEnabled, fileHashGenerationEnabled) {
    return $.extend(new jscape.ServiceConfiguration(), {
        maxRetransmitAttempts: parseInt(retransmitAttempts) || 0, retransmitInterval: parseInt(retransmitInterval) || 0, directoryListingGenerationEnabled: !!dirListingGenerationEnabled, fileHashGenerationEnabled: !!fileHashGenerationEnabled
    });
};
jscape.ServiceConfiguration.http = function(logoData, timeout, logoutUrl, autoLogout, selfRegistration, docViewerEnabled, adHocEnabled, showButtonsShortcuts, showTransferType, showLoginInfo, showAccountLink, showSearch, uploadForms, uploadFormRequired, languages, aftpNatHost) {
    return $.extend(new jscape.ServiceConfiguration(), {
        logoData: logoData || "", timeout: parseInt(timeout) || 0, logoutUrl: logoutUrl || "", autoLogout: autoLogout || null, selfRegistration: selfRegistration || new jscape.SelfRegistrationConfiguration(), documentViewerEnabled: !!docViewerEnabled, adHocTransferEnabled: !!adHocEnabled,
        showButtonsShortcuts: !!showButtonsShortcuts, showTransferType: !!showTransferType, showLoginInfo: !!showLoginInfo, showAccountLink: !!showAccountLink, showSearch: !!showSearch,
        uploadForms: uploadForms || [], uploadFormRequired: !!uploadFormRequired, languages: languages, aftpNatHost: aftpNatHost || null
    });
};
jscape.ServiceConfiguration.webdav = function(logoData, appletEnabled, docViewerEnabled, logoutUrl, showLoginInfo, showAccountLink, languages) {
    return $.extend(new jscape.ServiceConfiguration(), {logoData: logoData || "", appletEnabled: !!appletEnabled, documentViewerEnabled: !!docViewerEnabled, logoutUrl: logoutUrl || "", showLoginInfo: !!showLoginInfo, showAccountLink: !!showAccountLink, languages: languages});
};
/** @class */
jscape.SelfRegistrationConfiguration = Class.extend(
    /** @lends {jscape.SelfRegistrationConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.enabled = a || false;
        this.accountTemplate = b || "Default";
    }
});
jscape.SelfRegistrationConfiguration.fromJSON = function(data) {
    return new jscape.SelfRegistrationConfiguration(data || data.enabled, data && data.accountTemplate);
};

/** @class */
jscape.UploadForm = Class.extend(
    /** @lends {jscape.UploadForm.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e, f) {
        this.name = a || "";
        this.description = b || "";
        this.enabled = !!c;
        this.batch = !!d;
        this.group = e || null;
        this.inputs = f || [];
    },

    clone: function() {
        return new jscape.UploadForm(this.name, this.description, this.enabled, this.batch, this.group, $.map(this.inputs, function(entry) { return entry.clone(); }));
    }
});
jscape.UploadForm.fromJSON = function(data) {
    return $.extend(new jscape.UploadForm(), data, {inputs: $.map(data.inputs || [], jscape.UploadForm.Input.fromJSON)});
};
/** @class */
jscape.UploadForm.Input = Class.extend(
    /** @lends {jscape.UploadForm.Input.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e) {
        this.type = a || jscape.UploadForm.Input.TYPE.TEXT;
        this.name = b || "";
        this.label = c || "";
        this.value = $.isArray(d) ? d.join(",") : d || "";
        this.required = e !== undefined ? !!e : true;
    },

    clone: function() {
        return new jscape.UploadForm.Input(this.type, this.name, this.label, this.value, this.required);
    },

    toJSON: function() {
        var multivalue = $.inArray(this.type, [jscape.UploadForm.Input.TYPE.DROPDOWN, jscape.UploadForm.Input.TYPE.MULTISELECT, jscape.UploadForm.Input.TYPE.RADIO]) != -1;
        return $.extend({type: this.type, name: this.name, label: this.label, required: this.required}, multivalue ? {values: this.value.split(",")} : {value: this.value});
    }
});
jscape.UploadForm.Input.fromJSON = function(data) {
    return new jscape.UploadForm.Input(data.type, data.name, data.label, data.value || data.values, data.required);
};
jscape.UploadForm.Input.TYPE = {TEXT: 'TEXT', MEMO: 'MEMO', DROPDOWN: 'DROPDOWN', MULTISELECT: 'MULTISELECT', RADIO: 'RADIO'};


/** @class */
jscape.domain.FtpConfigurationController = Class.extend(
    /** @lends {jscape.domain.FtpConfigurationController.prototype} */
{
    DEFAULT_CONFIG: jscape.ServiceConfiguration.ftp(300000, 60000),

    config: null,
    copy: null,
    sslCiphers: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.FtpConfigurationView(resources);
        this.ciphersController = new jscape.domain.SslCiphersController(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.config && !Object._equals(this.copy, this._createConfiguration());
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var config = this._createConfiguration();
        return this.managementService.setServiceConfiguration(this.domainName, config)
                .done($.proxy(function() {
                    this.config = config;
                    this.copy = this.config.clone();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.config = this.copy;
        this.copy = this.config.clone();
        this.sslCiphers = this.config && this.config.cipherSuites || null;
        this._setupView();
    },

    onSslCiphers: function() {
        this.ciphersController.start(this.managementService, this.domainName, this.sslCiphers)
                .done($.proxy(function(ciphers) { this.sslCiphers = ciphers; }, this));
    },

    _loadData: function() {
        this.managementService.getServiceConfiguration(this.domainName, jscape.Service.Protocol.FTP)
                .done($.proxy(function(config) {
                    this.config = config || this.DEFAULT_CONFIG;
                    this.copy = this.config.clone();
                    this.sslCiphers = this.config && this.config.cipherSuites || null;
                    this._setupView();
                }, this));
    },

    _createConfiguration: function() {
        return jscape.ServiceConfiguration.ftp(this.view.getTimeout(),
                                               this.view.getDataTimeout(),
                                               this.view.getBanner(),
                                               this.view.getPassiveIp(),
                                               this.view.getPassivePortRange(),
                                               this.view.getSendBufferSize(),
                                               this.view.getReceiveBufferSize(),
                                               this.view.getDefaultTransferMode(),
                                               this.sslCiphers,
                                               this.view.isShutdownCommandSsl(),
                                               this.view.isShutdownDataSsl(),
                                               this.view.isCommandRequireClientCertificate(),
                                               this.view.isDataRequireClientCertificate(),
                                               this.view.isForceDataEncryption(),
                                               this.view.isBlockPasvAttack(),
                                               this.view.isBlockBounceAttack(),
                                               this.view.getNetworkCharset(),
                                               this.view.getFileCharset());
    },

    _setupView: function() {
        if (this.config) {
            this.view.setTimeout(this.config.timeout);
            this.view.setDataTimeout(this.config.dataTimeout);
            this.view.setBanner(this.config.welcomeMessage);
            this.view.setPassiveIp(this.config.passiveIp);
            this.view.setPassivePortRange(this.config.portRange);
            this.view.setSendBufferSize(this.config.sendBufferSize);
            this.view.setReceiveBufferSize(this.config.receiveBufferSize);
            this.view.setDefaultTransferMode(this.config.defaultTransferMode);
            this.view.setShutdownCommandSsl(this.config.shutdownSsl);
            this.view.setShutdownDataSsl(this.config.shutdownDataSsl);
            this.view.setCommandRequireClientCertificate(this.config.commandRequireClientCertificate);
            this.view.setDataRequireClientCertificate(this.config.dataRequireClientCertificate);
            this.view.setForceDataEncryption(this.config.forceDataEncryption);
            this.view.setBlockPasvAttack(this.config.blockPasvAttack);
            this.view.setBlockBounceAttack(this.config.blockBounceAttack);
            this.view.setNetworkCharset(this.config.networkCharset);
            this.view.setFileCharset(this.config.fileCharset);
        }
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.FtpConfigurationView = Class.extend(
    /** @lends {jscape.domain.FtpConfigurationView.prototype} */
{
    TIMEOUT_FACTOR: 60 * 1000,
    BUFFER_FACTOR: 1024,
    DEFAULT_CHARSET: "UTF-8",

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=ftpconfigfields]");

        this.banner = $("textarea[name=banner]", this.fieldset);

        this.timeout = $("input[name=cmdtimeout]", this.fieldset);
        this.timeout.numberspinner({required: true, min: 0, max: 999, value: 5, width: 80});

        this.dataTimeout = $("input[name=datatimeout]", this.fieldset);
        this.dataTimeout.numberspinner({required: true, min: 0, max: 999, value: 1, width: 80});

        this.passiveIp = $("input[name=passiveip]", this.fieldset);
        this.passiveIp.validatebox({required: false, validType: "length[1,255]", invalidMessage: resources.getString("SERVICES_ERROR_BAD_PASSIVE_IP")});

        this.usePortRange = $("input[name=passiveportrange]:checkbox", this.fieldset);
        this.usePortRange.change($.proxy(function() {
            var enabled = this.usePortRange.is(":checked");
            $.each([this.portRangeStart, this.portRangeEnd], function(_, input) {
                input.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
            });
        }, this));
        this.portRangeStart = $("input[name=passiveportpstart]", this.fieldset);
        this.portRangeStart.numberspinner({required: true, min: 1, max: 65535, value: 10000, width: 80, onChange: $.proxy(this._adjustPortRange, this)});
        this.portRangeEnd = $("input[name=passiveportpend]", this.fieldset);
        this.portRangeEnd.numberspinner({required: true, min: 1, max: 65535, value: 20000, width: 80, onChange: $.proxy(this._adjustPortRange, this)});

        this.useSendBuffer = $("input[name=usesendbuffer]:checkbox", this.fieldset);
        this.useSendBuffer.change($.proxy(function() {
            var enabled = this.useSendBuffer.is(":checked");
            this.sendBufferSize.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.sendBufferSize = $("input[name=sendbuffer]", this.fieldset);
        this.sendBufferSize.numberspinner({required: true, min: 64, max: 1024, increment: 64, value: 64, width: 80});

        this.useReceiveBuffer = $("input[name=usereceivebuffer]:checkbox", this.fieldset);
        this.useReceiveBuffer.change($.proxy(function() {
            var enabled = this.useReceiveBuffer.is(":checked");
            this.receiveBufferSize.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.receiveBufferSize = $("input[name=receivebuffer]", this.fieldset);
        this.receiveBufferSize.numberspinner({required: true, min: 64, max: 1024, increment: 64, value: 64, width: 80});

        this.defaultTransferMode = $("input[name=transfermode]:radio", this.fieldset);
        this.forceDataEncryption = $("input[name=forcedataencryption]:checkbox", this.fieldset);
        this.commandRequireClientCertificate = $("input[name=cmdrequireclientcrt]:checkbox", this.fieldset);
        this.dataRequireClientCertificate = $("input[name=datarequireclientcrt]:checkbox", this.fieldset);
        this.shutdownCommandSsl = $("input[name=shutdowncmdssl]:checkbox", this.fieldset);
        this.shutdownDataSsl = $("input[name=shutdowndatassl]:checkbox", this.fieldset);
        this.blockBounceAttack = $("input[name=blockbounce]:checkbox", this.fieldset);
        this.blockPasvAttack = $("input[name=blockpasv]:checkbox", this.fieldset);

        this.networkCharset = $("select[name=networkcharset]", this.fieldset);
        this.networkCharset.combobox({required: true, editable: false, width: 140});
        this.fileCharset = $("select[name=filecharset]", this.fieldset);
        this.fileCharset.combobox({required: true, editable: false, width: 140});
        this.threatAsciiAsImage = $("input[name=asciiasimage]:checkbox", this.fieldset);
        this.threatAsciiAsImage.change($.proxy(function() {
            var enabled = !this.threatAsciiAsImage.is(":checked");
            this.fileCharset.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));

        this.ciphersButton = $("a[role=button][name=sslciphers]", this.fieldset);
        this.ciphersButton.linkbutton({onClick: $.proxy(this._onClick$CiphersButton, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getBanner: function() {
        return this.banner.val().trim();
    },

    setBanner: function(value) {
        this.banner.val(value);
    },

    getTimeout: function() {
        return parseInt(this.timeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    getDataTimeout: function() {
        return parseInt(this.dataTimeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setDataTimeout: function(value) {
        this.dataTimeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    getPassiveIp: function() {
        return this.passiveIp.val().trim();
    },

    setPassiveIp: function(value) {
        this.passiveIp.val(value);
    },

    getPassivePortRange: function() {
        return this.usePortRange.is(":checked")
                ? {lowerBound: parseInt(this.portRangeStart.numberspinner("getValue")), upperBound: parseInt(this.portRangeEnd.numberspinner("getValue"))}
                : null;
    },

    setPassivePortRange: function(value) {
        if (value && value.lowerBound && value.upperBound) {
            this.usePortRange.prop("checked", true).change();
            this.portRangeEnd.numberspinner("setValue", parseInt(value.upperBound));
            this.portRangeStart.numberspinner("setValue", parseInt(value.lowerBound));
        } else {
            this.usePortRange.prop("checked", false).change();
        }
    },

    getSendBufferSize: function() {
        return this.useSendBuffer.is(":checked") ? parseInt(this.sendBufferSize.numberspinner("getValue")) * this.BUFFER_FACTOR : null;
    },

    setSendBufferSize: function(value) {
        this.useSendBuffer.prop("checked", value != null).change();
        if (value != null) {
            this.sendBufferSize.numberspinner("setValue", parseInt(value) / this.BUFFER_FACTOR);
        }
    },

    getReceiveBufferSize: function() {
        return this.useReceiveBuffer.is(":checked") ? parseInt(this.receiveBufferSize.numberspinner("getValue")) * this.BUFFER_FACTOR : null;
    },

    setReceiveBufferSize: function(value) {
        this.useReceiveBuffer.prop("checked", value != null).change();
        if (value != null) {
            this.receiveBufferSize.numberspinner("setValue", parseInt(value) / this.BUFFER_FACTOR);
        }
    },

    getDefaultTransferMode: function () {
        return this.defaultTransferMode.filter(":checked").val();
    },

    setDefaultTransferMode: function (value) {
        this.defaultTransferMode.filter("[value=" + value + "]").prop("checked", true);
    },

    isForceDataEncryption: function() {
        return this.forceDataEncryption.is(":checked");
    },

    setForceDataEncryption: function(value) {
        this.forceDataEncryption.prop("checked", !!value);
    },

    isCommandRequireClientCertificate: function() {
        return this.commandRequireClientCertificate.is(":checked");
    },

    setCommandRequireClientCertificate: function(value) {
        this.commandRequireClientCertificate.prop("checked", !!value);
    },

    isDataRequireClientCertificate: function() {
        return this.dataRequireClientCertificate.is(":checked");
    },

    setDataRequireClientCertificate: function(value) {
        this.dataRequireClientCertificate.prop("checked", !!value);
    },

    isShutdownCommandSsl: function() {
        return this.shutdownCommandSsl.is(":checked");
    },

    setShutdownCommandSsl: function(value) {
        this.shutdownCommandSsl.prop("checked", !!value);
    },

    isShutdownDataSsl: function() {
        return this.shutdownDataSsl.is(":checked");
    },

    setShutdownDataSsl: function(value) {
        this.shutdownDataSsl.prop("checked", !!value);
    },

    isBlockBounceAttack: function() {
        return this.blockBounceAttack.is(":checked");
    },

    setBlockBounceAttack: function(value) {
        this.blockBounceAttack.prop("checked", !!value);
    },

    isBlockPasvAttack: function() {
        return this.blockPasvAttack.is(":checked");
    },

    setBlockPasvAttack: function(value) {
        this.blockPasvAttack.prop("checked", !!value).change();
    },

    getNetworkCharset: function() {
        return this.networkCharset.combobox("getValue");
    },

    setNetworkCharset: function(value) {
        this.networkCharset.combobox("setValue", value || this.DEFAULT_CHARSET);
    },

    getFileCharset: function() {
        return this.threatAsciiAsImage.is(":checked") ? null : this.fileCharset.combobox("getValue");
    },

    setFileCharset: function(value) {
        this.threatAsciiAsImage.prop("checked", value == null).change();
        this.fileCharset.combobox("setValue", value != null ? value : this.DEFAULT_CHARSET);
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.FtpConfigurationView.LISTENER;
        this._adjustPortRange();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$CiphersButton: function() {
        this.listener.onSslCiphers();
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard();
    },

    _adjustPortRange: function() {
        var range = this.getPassivePortRange();
        if (range != null) {
            function _validate(spinner, validType, validParams) {
                $.extend($(spinner).validatebox("options"), {validType: validType, validParams: validParams});
                return spinner.numberspinner("isValid");
            }
            _validate(this.portRangeStart, "in_range", [this.portRangeStart.numberspinner("options").min, range.upperBound]);
            _validate(this.portRangeEnd, "in_range", [range.lowerBound, this.portRangeEnd.numberspinner("options").max]);
        }
    }
});
jscape.domain.FtpConfigurationView.LISTENER = {
    onSslCiphers: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.SftpConfigurationController = Class.extend(
    /** @lends {jscape.domain.SftpConfigurationController.prototype} */
{
    DEFAULT_CONFIG: jscape.ServiceConfiguration.sftp(300000, "JSCAPE"),

    config: null,
    copy: null,
    algorithms: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.SftpConfigurationView(resources);
        this.algorithmsController = new jscape.domain.SftpAlgorithmsController(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.algorithms = {};
        this.view.show(this);
    },

    hasChanged: function() {
        return this.config && !Object._equals(this.copy, this._createConfiguration());
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var config = this._createConfiguration();
        return this.managementService.setServiceConfiguration(this.domainName, config)
                .done($.proxy(function() {
                    this.config = config;
                    this.copy = this.config.clone();
                    this._initAlgorithms();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.config = this.copy;
        this.copy = this.config.clone();
        this._initAlgorithms();
        this._setupView();
    },

    onSftpAlgorithms: function() {
        this.algorithmsController.start(this.algorithms)
                .done($.proxy(function(algorithms) { this.algorithms = algorithms; }, this));
    },

    _loadData: function() {
        this.managementService.getServiceConfiguration(this.domainName, jscape.Service.Protocol.SFTP)
                .done($.proxy(function(config) {
                    this.config = config || this.DEFAULT_CONFIG;
                    this.copy = this.config.clone();
                    this._initAlgorithms();
                    this._setupView();
                }, this));
    },

    _initAlgorithms: function() {
        this.algorithms = this.config
                ? {kexes: this.config.supportedKexes, ciphers: this.config.supportedCiphers, macs: this.config.supportedMacs, compressions: this.config.supportedCompressions}
                : {};
    },

    _createConfiguration: function() {
        return jscape.ServiceConfiguration.sftp(this.view.getTimeout(),
                                                this.view.getVersion(),
                                                this.view.getStartupBanner(),
                                                this.view.getAuthenticationBanner(),
                                                this.view.getSendBufferSize(),
                                                this.view.getReceiveBufferSize(),
                                                this.view.isDisableExpandedLongName(),
                                                this.algorithms.kexes,
                                                this.algorithms.ciphers,
                                                this.algorithms.macs,
                                                this.algorithms.compressions);
    },

    _setupView: function () {
        if (this.config) {
            this.view.setTimeout(this.config.timeout);
            this.view.setVersion(this.config.softwareVersion);
            this.view.setStartupBanner(this.config.startupBanner);
            this.view.setAuthenticationBanner(this.config.authBanner);
            this.view.setSendBufferSize(this.config.sendBufferSize);
            this.view.setReceiveBufferSize(this.config.receiveBufferSize);
            this.view.setDisableExpandedLongName(this.config.disableRealPathExpandedLongName);
        }
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.SftpConfigurationView = Class.extend(
    /** @lends {jscape.domain.SftpConfigurationView.prototype} */
{
    TIMEOUT_FACTOR: 60 * 1000,
    BUFFER_FACTOR: 1024,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=sftpconfigfields]");

        this.version = $("input[name=version]", this.fieldset);
        this.startupBanner = $("textarea[name=startupbanner]", this.fieldset);
        this.authBanner = $("textarea[name=authbanner]", this.fieldset);

        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({required: true, min: 0, max: 999, value: 5, width: 80});

        this.useSendBuffer = $("input[name=usesendbuffer]:checkbox", this.fieldset);
        this.useSendBuffer.change($.proxy(function() {
            var enabled = this.useSendBuffer.is(":checked");
            this.sendBufferSize.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.sendBufferSize = $("input[name=sendbuffer]", this.fieldset);
        this.sendBufferSize.numberspinner({required: true, min: 64, max: 1024, increment: 64, value: 64, width: 80});

        this.useReceiveBuffer = $("input[name=usereceivebuffer]:checkbox", this.fieldset);
        this.useReceiveBuffer.change($.proxy(function() {
            var enabled = this.useReceiveBuffer.is(":checked");
            this.receiveBufferSize.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.receiveBufferSize = $("input[name=receivebuffer]", this.fieldset);
        this.receiveBufferSize.numberspinner({required: true, min: 64, max: 1024, increment: 64, value: 64, width: 80});

        this.disableLongName = $("input[name=disablelongname]:checkbox", this.fieldset);

        this.algorithmsButton = $("a[role=button][name=algorithms]", this.fieldset);
        this.algorithmsButton.linkbutton({onClick: $.proxy(this._onClick$AlgorithmsButton, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getTimeout: function() {
        return parseInt(this.timeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    getVersion: function() {
        return this.version.val().trim();
    },

    setVersion: function(value) {
        this.version.val(value);
    },

    getStartupBanner: function() {
        return this.startupBanner.val();
    },

    setStartupBanner: function(value) {
        this.startupBanner.val(value);
    },

    getAuthenticationBanner: function() {
        return this.authBanner.val();
    },

    setAuthenticationBanner: function(value) {
        this.authBanner.val(value);
    },

    getSendBufferSize: function() {
        return this.useSendBuffer.is(":checked") ? parseInt(this.sendBufferSize.numberspinner("getValue")) * this.BUFFER_FACTOR : null;
    },

    setSendBufferSize: function(value) {
        this.useSendBuffer.prop("checked", value != null).change();
        if (value != null) {
            this.sendBufferSize.numberspinner("setValue", parseInt(value) / this.BUFFER_FACTOR);
        }
    },

    getReceiveBufferSize: function() {
        return this.useReceiveBuffer.is(":checked") ? parseInt(this.receiveBufferSize.numberspinner("getValue")) * this.BUFFER_FACTOR : null;
    },

    setReceiveBufferSize: function(value) {
        this.useReceiveBuffer.prop("checked", value != null).change();
        if (value != null) {
            this.receiveBufferSize.numberspinner("setValue", parseInt(value) / this.BUFFER_FACTOR);
        }
    },

    isDisableExpandedLongName: function() {
        return this.disableLongName.is(":checked");
    },

    setDisableExpandedLongName: function(value) {
        this.disableLongName.prop("checked", !!value).change();
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.SftpConfigurationView.LISTENER;
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$AlgorithmsButton: function() {
        this.listener.onSftpAlgorithms();
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard();
    }
});
jscape.domain.SftpConfigurationView.LISTENER = {
    onSftpAlgorithms: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.AftpConfigurationController = Class.extend(
    /** @lends {jscape.domain.AftpConfigurationController.prototype} */
{
    DEFAULT_CONFIG: jscape.ServiceConfiguration.aftp(300000, 60000, 10),

    config: null,
    copy: null,
    sslCiphers: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.AftpConfigurationView(resources);
        this.ciphersController = new jscape.domain.SslCiphersController(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.config && !Object._equals(this.copy, this._createConfiguration());
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var config = this._createConfiguration();
        return this.managementService.setServiceConfiguration(this.domainName, config)
                .done($.proxy(function() {
                    this.config = config;
                    this.copy = this.config.clone();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.config = this.copy;
        this.copy = this.config.clone();
        this.sslCiphers = this.config && this.config.cipherSuites || null;
        this._setupView();
    },

    onSslCiphers: function() {
        this.ciphersController.start(this.managementService, this.domainName, this.sslCiphers)
                .done($.proxy(function(ciphers) { this.sslCiphers = ciphers; }, this));
    },

    _loadData: function() {
        this.managementService.getServiceConfiguration(this.domainName, jscape.Service.Protocol.AFTP)
                .done($.proxy(function(config) {
                    this.config = config || this.DEFAULT_CONFIG;
                    this.copy = this.config.clone();
                    this.sslCiphers = this.config && this.config.cipherSuites || null;
                    this._setupView();
                }, this));
    },

    _createConfiguration: function() {
        return jscape.ServiceConfiguration.aftp(this.view.getTimeout(),
                                                this.view.getDataTimeout(),
                                                this.view.getMaxLossListSize(),
                                                this.sslCiphers);
    },

    _setupView: function() {
        if (this.config) {
            this.view.setTimeout(this.config.timeout);
            this.view.setDataTimeout(this.config.dataTimeout);
            this.view.setMaxLossListSize(this.config.maxLossListSize);
        }
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.AftpConfigurationView = Class.extend(
    /** @lends {jscape.domain.AftpConfigurationView.prototype} */
{
    TIMEOUT_FACTOR: 60 * 1000,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=aftpconfigfields]");

        this.timeout = $("input[name=cmdtimeout]", this.fieldset);
        this.timeout.numberspinner({required: true, min: 0, max: 999, value: 5, width: 80});

        this.dataTimeout = $("input[name=datatimeout]", this.fieldset);
        this.dataTimeout.numberspinner({required: true, min: 0, max: 999, value: 5, width: 80});

        this.lossListSize = $("input[name=losslistsize]", this.fieldset);
        this.lossListSize.numberspinner({required: true, min: 1, max: 99, value: 10, width: 60});

        this.ciphersButton = $("a[role=button][name=sslciphers]", this.fieldset);
        this.ciphersButton.linkbutton({onClick: $.proxy(this._onClick$CiphersButton, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getTimeout: function() {
        return parseInt(this.timeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    getDataTimeout: function() {
        return parseInt(this.dataTimeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setDataTimeout: function(value) {
        this.dataTimeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    getMaxLossListSize: function() {
        return parseInt(this.lossListSize.numberspinner("getValue"));
    },

    setMaxLossListSize: function(value) {
        this.lossListSize.numberspinner("setValue", parseInt(value));
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.AftpConfigurationView.LISTENER;
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$CiphersButton: function() {
        this.listener.onSslCiphers();
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard();
    }
});
jscape.domain.AftpConfigurationView.LISTENER = {
    onSslCiphers: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.OftpConfigurationController = Class.extend(
    /** @lends {jscape.domain.OftpConfigurationController.prototype} */
{
    DEFAULT_CONFIG: jscape.ServiceConfiguration.oftp(30 * 1000, 4096, 10),

    config: null,
    copy: null,
    sslCiphers: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.OftpConfigurationView(resources);
        this.ciphersController = new jscape.domain.SslCiphersController(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.config && !Object._equals(this.copy, this._createConfiguration());
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var config = this._createConfiguration();
        return this.managementService.setServiceConfiguration(this.domainName, config)
                .done($.proxy(function() {
                    this.config = config;
                    this.copy = this.config.clone();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.config = this.copy;
        this.copy = this.config.clone();
        this.sslCiphers = this.config && this.config.cipherSuites || null;
        this._setupView();
    },

    onSslCiphers: function() {
        this.ciphersController.start(this.managementService, this.domainName, this.sslCiphers)
                .done($.proxy(function(ciphers) { this.sslCiphers = ciphers; }, this));
    },

    _loadData: function() {
        this.managementService.getServiceConfiguration(this.domainName, jscape.Service.Protocol.OFTP)
                .done($.proxy(function(config) { this.config = config || this.DEFAULT_CONFIG; }, this))
                .fail($.proxy(function() { this.config = this.DEFAULT_CONFIG; }, this))
                .always($.proxy(function() {
                    this.copy = this.config.clone();
                    this.sslCiphers = this.config && this.config.cipherSuites || null;
                    this._setupView();
                }, this));
    },

    _createConfiguration: function() {
        return jscape.ServiceConfiguration.oftp(this.view.getTimeout(),
                                                this.view.getMaxDataBufferSize(),
                                                this.view.getMaxCredit(),
                                                this.sslCiphers,
                                                this.config.enabledProtocols);
    },

    _setupView: function() {
        if (this.config != null) {
            this.view.setTimeout(this.config.timeout);
            this.view.setMaxDataBufferSize(this.config.maxDataExchangeBufferSize);
            this.view.setMaxCredit(this.config.maxCredit);
        }
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.OftpConfigurationView = Class.extend(
    /** @lends {jscape.domain.OftpConfigurationView.prototype} */
{
    TIMEOUT_FACTOR: 1000,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=oftpconfigfields]");

        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({required: true, min: 0, max: 999, width: 80});

        this.dataBuffer = $("input[name=buffer]", this.fieldset);
        this.dataBuffer.numberspinner({required: true, min: 128, max: 99999, increment: 10, width: 100});

        this.credit = $("input[name=credit]", this.fieldset);
        this.credit.numberspinner({required: true, min: 1, max: 999, width: 80});

        this.ciphersButton = $("a[role=button][name=sslciphers]", this.fieldset);
        this.ciphersButton.linkbutton({onClick: $.proxy(this._onClick$CiphersButton, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getTimeout: function() {
        return parseInt(this.timeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR).numberspinner("validate");
    },

    getMaxDataBufferSize: function() {
        return parseInt(this.dataBuffer.numberspinner("getValue"));
    },

    setMaxDataBufferSize: function(value) {
        this.dataBuffer.numberspinner("setValue", parseInt(value)).numberspinner("validate");
    },

    getMaxCredit: function() {
        return parseInt(this.credit.numberspinner("getValue"));
    },

    setMaxCredit: function(value) {
        this.credit.numberspinner("setValue", parseInt(value)).numberspinner("validate");
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.OftpConfigurationView.LISTENER;
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$CiphersButton: function () {
        this.listener.onSslCiphers();
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard();
    }
});
jscape.domain.OftpConfigurationView.LISTENER = {
    onSslCiphers: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.TftpConfigurationController = Class.extend(
    /** @lends {jscape.domain.TftpConfigurationController.prototype} */
{
    DEFAULT_CONFIG: jscape.ServiceConfiguration.tftp(6, 3000, true, true),

    config: null,
    copy: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.TftpConfigurationView(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.config && !Object._equals(this.copy, this._createConfiguration());
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var config = this._createConfiguration();
        return this.managementService.setServiceConfiguration(this.domainName, config)
                .done($.proxy(function() {
                    this.config = config;
                    this.copy = this.config.clone();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.config = this.copy;
        this.copy = this.config.clone();
        this._setupView();
    },

    _loadData: function() {
        this.managementService.getServiceConfiguration(this.domainName, jscape.Service.Protocol.TFTP)
                .done($.proxy(function(config) { this.config = config || this.DEFAULT_CONFIG; }, this))
                .fail($.proxy(function() { this.config = this.DEFAULT_CONFIG; }, this))
                .always($.proxy(function() {
                    this.copy = this.config.clone();
                    this._setupView();
                }, this));
    },

    _createConfiguration: function() {
        return jscape.ServiceConfiguration.tftp(this.view.getMaxRetransmitAttempts(),
                                                this.view.getRetransmitInterval(),
                                                this.view.isDirectoryListingGenerationEnabled(),
                                                this.view.isFileHashGenerationEnabled());
    },

    _setupView: function() {
        if (this.config != null) {
            this.view.setMaxRetransmitAttempts(this.config.maxRetransmitAttempts);
            this.view.setRetransmitInterval(this.config.retransmitInterval);
            this.view.setDirectoryListingGenerationEnabled(this.config.directoryListingGenerationEnabled);
            this.view.setFileHashGenerationEnabled(this.config.fileHashGenerationEnabled);
        }
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.TftpConfigurationView = Class.extend(
    /** @lends {jscape.domain.TftpConfigurationView.prototype} */
{
    TIME_FACTOR: 1000,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=tftpconfigfields]");

        this.retransmitAttempts = $("input[name=retransmitattempts]", this.fieldset);
        this.retransmitAttempts.numberspinner({required: true, min: 0, max: 100, width: 80});

        this.retransmitInterval = $("input[name=retransmitinterval]", this.fieldset);
        this.retransmitInterval.numberspinner({required: true, min: 1, max: 255, width: 80});

        this.dirListingGeneration = $("input[name=generatedirlisting]", this.fieldset);
        this.fileHashGeneration = $("input[name=generatefilehash]", this.fieldset);

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getMaxRetransmitAttempts: function() {
        return parseInt(this.retransmitAttempts.numberspinner("getValue"));
    },

    setMaxRetransmitAttempts: function(value) {
        this.retransmitAttempts.numberspinner("setValue", parseInt(value)).numberspinner("validate");
    },

    getRetransmitInterval: function() {
        return parseInt(this.retransmitInterval.numberspinner("getValue")) * this.TIME_FACTOR;
    },

    setRetransmitInterval: function(value) {
        this.retransmitInterval.numberspinner("setValue", parseInt(value) / this.TIME_FACTOR).numberspinner("validate");
    },

    isDirectoryListingGenerationEnabled: function() {
        return this.dirListingGeneration.is(":checked");
    },

    setDirectoryListingGenerationEnabled: function(value) {
        this.dirListingGeneration.prop("checked", !!value);
    },

    isFileHashGenerationEnabled: function() {
        return this.fileHashGeneration.is(":checked");
    },

    setFileHashGenerationEnabled: function(value) {
        this.fileHashGeneration.prop("checked", !!value);
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.TftpConfigurationView.LISTENER;
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard();
    }
});
jscape.domain.TftpConfigurationView.LISTENER = {
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.HttpConfigurationController = Class.extend(
    /** @lends {jscape.domain.HttpConfigurationController.prototype} */
{
    DEFAULT_CONFIG: jscape.ServiceConfiguration.http("", 600000, "", null, null, false, true, true, true, true, true, true),

    config: null,
    copy: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.HttpConfigurationView(resources);
        this.formsController = new jscape.domain.UploadFormsController(resources);
        this.resourcesController = new jscape.domain.WebResourcesController(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.config && !Object._equals(this.copy, this._createConfiguration());
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var config = this._createConfiguration();
        return this.managementService.setServiceConfiguration(this.domainName, config)
                .done($.proxy(function() {
                    this.config = config;
                    this.copy = this.config.clone();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.config = this.copy;
        this.copy = this.config.clone();
        this._setupView();
    },

    onSetupForms: function() {
        var forms = $.map(this.config.uploadForms, function(form) { return form.clone(); });
        this.managementService.getGroups(this.domainName)
                .always($.proxy(function(groups) {
                    this.formsController.start(groups, forms)
                            .done($.proxy(function(values) {
                                this.config.uploadForms = values;
                            }, this));
                }, this));
    },

    onSetupResources: function() {
        var languages = $.map(this.config.languages, function(lang) { return lang.clone(); });
        this.resourcesController.start(languages)
                .done($.proxy(function(values) {
                    this.config.languages = values;
                }, this));
    },

    onChangeLogo: function (file) {
        if (file && file.type.match("image.*") && window.FileReader) {
            var reader = new FileReader();
            reader.onload = $.proxy(function (e) { this.view.setLogoData(e.target.result); }, this);
            reader.readAsDataURL(file);
        } else {
            this.view.setLogoData(null);
        }
    },

    _loadData: function() {
        this.managementService.getServiceConfiguration(this.domainName, jscape.Service.Protocol.HTTP)
                .done($.proxy(function(config) {
                    this.config = config || this.DEFAULT_CONFIG;
                    this.copy = this.config.clone();
                    this._setupView();
                    this._loadAccountTemplates();
                }, this));
    },

    _loadAccountTemplates: function() {
        this.managementService.getTemplates(this.domainName)
                .done($.proxy(function(templates) {
                    this.view.setSelfRegistrationTemplates((templates || []).sort());
                }, this));
    },

    _createConfiguration: function() {
        var signupTemplate = this.view.getSelfRegistrationTemplate();
        return jscape.ServiceConfiguration.http(this.view.getLogoData(),
                                                this.view.getTimeout(),
                                                this.view.getLogoutUrl(),
                                                this.view.getAutoLogout(),
                                                new jscape.SelfRegistrationConfiguration(signupTemplate != null, signupTemplate),
                                                this.view.isDocumentViewerEnabled(),
                                                this.view.isAdHocTransferEnabled(),
                                                this.view.isShowButtonsShortcuts(),
                                                this.view.isShowTransferMode(),
                                                this.view.isShowLoginInfo(),
                                                this.view.isShowAccountLink(),
                                                this.view.isShowSearch(),
                                                this.config.uploadForms,
                                                this.view.isUploadFormRequired(),
                                                this.config.languages,
                                                this.view.getAftpNatHost());
    },

    _setupView: function() {
        if (this.config) {
            var signup = this.config.selfRegistration;
            this.view.setLogoData(this.config.logoData);
            this.view.setTimeout(this.config.timeout);
            this.view.setLogoutUrl(this.config.logoutUrl);
            this.view.setAutoLogout(this.config.autoLogout);
            this.view.setSelfRegistrationTemplate(signup && signup.enabled ? signup.accountTemplate : null);
            this.view.setDocumentViewerEnabled(this.config.documentViewerEnabled);
            this.view.setAdHocTransferEnabled(this.config.adHocTransferEnabled);
            this.view.setShowButtonsShortcuts(this.config.showButtonsShortcuts);
            this.view.setShowTransferMode(this.config.showTransferType);
            this.view.setShowLoginInfo(this.config.showLoginInfo);
            this.view.setShowAccountLink(this.config.showAccountLink);
            this.view.setShowSearch(this.config.showSearch);
            this.view.setAftpNatHost(this.config.aftpNatHost);
            this.view.setUploadFormRequired(this.config.uploadFormRequired);
        }
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.HttpConfigurationView = Class.extend(
    /** @lends {jscape.domain.HttpConfigurationView.prototype} */
{
    DATA_URL: {pattern: /data:image\/.*;base64,(.+)/, prefix: "data:image/png;base64,"},
    DEFAULT_TEMPLATE_NAME: "Default",
    TIMEOUT_FACTOR: 60 * 1000,
    GRACE_PERIOD_FACTOR: 1000,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=httpconfigfields]");

        this.logo = $("div.logo-preview>img", this.fieldset);
        this.logoFile = $("input[name=logo]", this.fieldset).css({width: 0, height: 0, opacity: 0, position: "absolute", left: "-10000px"});
        this.logoFile.change($.proxy(this._onChange$Logo, this));
        this.logoButton = $("a[role=button][name=changelogo]", this.fieldset);
        this.logoButton.linkbutton({onClick: $.proxy(function() { this.logoFile.click(); }, this)});
        this.logo.parent().click($.proxy(function() { this.logoFile.click(); }, this));

        this.logoutUrl = $("input[name=logouturl]", this.fieldset);

        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({required: true, min: 1, max: 180, width: 70});

        this.autoLogoutEnabled = $("input[name=autologout]:checkbox", this.fieldset);
        this.autoLogoutEnabled.change($.proxy(function() {
            var enabled = this.autoLogoutEnabled.is(":checked");
            $.each([this.autoLogoutTimeout, this.autoLogoutGracePeriod], function(_, input) {
                input.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
            })
        }, this));
        this.autoLogoutTimeout = $("input[name=autologouttimeout]", this.fieldset);
        this.autoLogoutTimeout.numberspinner({required: true, min: 1, max: 180, value: 1, width: 70});
        this.autoLogoutGracePeriod = $("input[name=autologoutgraceperiod]", this.fieldset);
        this.autoLogoutGracePeriod.numberspinner({required: true, min: 10, max: 600, value: 10, increment: 10, width: 70});

        this.signupEnabled = $("input[name=signupallowed]:checkbox", this.fieldset).prop("disabled", true);
        this.signupEnabled.change($.proxy(function() {
            var enabled = this.signupEnabled.is(":checked");
            this.signupAccountTemplate.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.signupAccountTemplate = $("select[name=signuptemplate]", this.fieldset);
        this.signupAccountTemplate.combobox({required: true, editable: false, disabled: true, panelHeight: "auto", onLoadSuccess: $.proxy(function(data) {
            if (data.length) {
                this.signupEnabled.prop("disabled", false);
                this.signupAccountTemplate.combobox("select", this.DEFAULT_TEMPLATE_NAME).combobox("resize", 170);
            } else {
                this.signupEnabled.prop("disabled", true);
            }
        }, this)}).combobox("autocomplete", true);

        this.docViewerEnabled = $("input[name=docviewerallowed]:checkbox", this.fieldset);
        this.adHocTransferEnabled = $("input[name=adhoctransferallowed]:checkbox", this.fieldset);
        this.showButtonsShortcuts = $("input[name=showbuttonsshortcut]:checkbox", this.fieldset);
        this.showLogin = $("input[name=showlogin]:checkbox", this.fieldset);
        this.showSearch = $("input[name=showsearch]:checkbox", this.fieldset);
        this.showTransferMode = $("input[name=showtransfermode]:checkbox", this.fieldset);
        this.showAccount = $("input[name=showaccount]:checkbox", this.fieldset);

        this.aftpNatHost = $("input[name=aftpnathost]", this.fieldset);

        this.resourcesButton = $("a[role=button][name=resources]", this.fieldset);
        this.resourcesButton.linkbutton({onClick: $.proxy(this._onClick$ResourcesButton, this)});

        this.uploadFormsButton = $("a[role=button][name=uploadforms]", this.fieldset);
        this.uploadFormsButton.linkbutton({onClick: $.proxy(this._onClick$FormsButton, this)});
        this.uploadFormRequired = $("input[name=uploadformrequired]:checkbox", this.fieldset);

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getLogoData: function () {
        return (this.logo.attr("src") || "").replace(this.DATA_URL.pattern, "$1");
    },

    setLogoData: function (data) {
        this.logo[0].src = this.DATA_URL.pattern.test(data) ? data : this.DATA_URL.prefix + (data || "");
    },

    getTimeout: function() {
        return parseInt(this.timeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    getLogoutUrl: function() {
        return this.logoutUrl.val();
    },

    setLogoutUrl: function(value) {
        this.logoutUrl.val(value);
    },

    getAutoLogout: function() {
        return this.autoLogoutEnabled.is(":checked")
                ? {timeout: parseInt(this.autoLogoutTimeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR, gracePeriod: parseInt(this.autoLogoutGracePeriod.numberspinner("getValue")) * this.GRACE_PERIOD_FACTOR}
                : null;
    },

    setAutoLogout: function(value) {
        if (value && value.timeout) {
            this.autoLogoutEnabled.prop("checked", true).change();
            this.autoLogoutTimeout.numberspinner("setValue", parseInt(value.timeout) / this.TIMEOUT_FACTOR).validatebox("isValid");
            this.autoLogoutGracePeriod.numberspinner("setValue", parseInt(value.gracePeriod) / this.GRACE_PERIOD_FACTOR).validatebox("isValid");
        } else {
            this.autoLogoutEnabled.prop("checked", false).change();
        }
    },

    getSelfRegistrationTemplate: function() {
        return this.signupEnabled.is(":checked") ? this.signupAccountTemplate.combobox("getValue") : null;
    },

    setSelfRegistrationTemplate: function(value) {
        this.signupEnabled.prop("checked", value != null).change();
        if (value != null) {
            this.signupAccountTemplate.combobox("select", value);
        }
    },

    setSelfRegistrationTemplates: function(values) {
        var data = $.map(values, function(value) { return {text: value, value: value}; });
        this.signupAccountTemplate.combobox("loadData", data);
    },

    isDocumentViewerEnabled: function() {
        return this.docViewerEnabled.is(":checked");
    },

    setDocumentViewerEnabled: function(value) {
        this.docViewerEnabled.prop("checked", !!value);
    },

    isAdHocTransferEnabled: function() {
        return this.adHocTransferEnabled.is(":checked");
    },

    setAdHocTransferEnabled: function(value) {
        this.adHocTransferEnabled.prop("checked", !!value).change();
    },

    isShowButtonsShortcuts: function() {
        return this.showButtonsShortcuts.is(":checked");
    },

    setShowButtonsShortcuts: function(value) {
        this.showButtonsShortcuts.prop("checked", !!value);
    },

    isShowTransferMode: function() {
        return this.showTransferMode.is(":checked");
    },
    setShowTransferMode: function(value) {
        this.showTransferMode.prop("checked", !!value);
    },

    isShowSearch: function() {
        return this.showSearch.is(":checked");
    },

    setShowSearch: function(value) {
        this.showSearch.prop("checked", !!value);
    },

    isShowLoginInfo: function() {
        return this.showLogin.is(":checked");
    },

    setShowLoginInfo: function(value) {
        this.showLogin.prop("checked", !!value);
    },

    isShowAccountLink: function() {
        return this.showAccount.is(":checked");
    },

    setShowAccountLink: function(value) {
        this.showAccount.prop("checked", !!value).change();
    },

    getAftpNatHost: function() {
        return this.aftpNatHost.val().trim();
    },

    setAftpNatHost: function(value) {
        this.aftpNatHost.val(value);
    },

    isUploadFormRequired: function() {
        return this.uploadFormRequired.is(":checked");
    },

    setUploadFormRequired: function(value) {
        this.uploadFormRequired.prop("checked", !!value);
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.HttpConfigurationView.LISTENER;
        this.fieldset.form("validate");
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$FormsButton: function() {
        this.listener.onSetupForms();
    },

    _onClick$ResourcesButton: function() {
        this.listener.onSetupResources();
    },

    _onChange$Logo: function (e) {
        var files = e.target.files;
        if (files && files.length) {
            this.listener.onChangeLogo(files[0]);
        }
        return false;
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard();
    }
});
jscape.domain.HttpConfigurationView.LISTENER = {
    onSetupForms: $.noop,
    onSetupResources: $.noop,
    onChangeLogo: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.WebDavConfigurationController = Class.extend(
    /** @lends {jscape.domain.WebDavConfigurationController.prototype} */
{
    DEFAULT_CONFIG: jscape.ServiceConfiguration.webdav("", true, false, "", true),

    config: null,
    copy: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.WebDavConfigurationView(resources);
        this.resourcesController = new jscape.domain.WebResourcesController(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.config && !Object._equals(this.copy, this._createConfiguration());
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var config = this._createConfiguration();
        return this.managementService.setServiceConfiguration(this.domainName, config)
                .done($.proxy(function() {
                    this.config = config;
                    this.copy = this.config.clone();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.config = this.copy;
        this.copy = this.config.clone();
        this._setupView();
    },

    onSetupResources: function() {
        var resources = $.map(this.config.languages, function(entry) { return entry.clone(); });
        this.resourcesController.start(resources)
                .done($.proxy(function(values) {
                    this.config.languages = values;
                }, this));
    },

    onChangeLogo: function (file) {
        if (file && file.type.match("image.*") && window.FileReader) {
            var reader = new FileReader();
            reader.onload = $.proxy(function (e) { this.view.setLogoData(e.target.result); }, this);
            reader.readAsDataURL(file);
        } else {
            this.view.setLogoData(null);
        }
    },

    _loadData: function() {
        this.managementService.getServiceConfiguration(this.domainName, jscape.Service.Protocol.WEBDAV)
                .done($.proxy(function(config) {
                    this.config = config || this.DEFAULT_CONFIG;
                    this.copy = this.config.clone();
                    this._setupView();
                }, this));
    },

    _createConfiguration: function() {
        return jscape.ServiceConfiguration.webdav(this.view.getLogoData(),
                                                  this.view.isAppletEnabled(),
                                                  this.view.isDocumentViewerEnabled(),
                                                  this.view.getLogoutUrl(),
                                                  this.view.isShowLoginInfo(),
                                                  this.view.isShowAccountLink(),
                                                  this.config.languages);
    },

    _setupView: function() {
        if (this.config) {
            this.view.setLogoData(this.config.logoData);
            this.view.setLogoutUrl(this.config.logoutUrl);
            this.view.setAppletEnabled(this.config.appletEnabled);
            this.view.setDocumentViewerEnabled(this.config.documentViewerEnabled);
            this.view.setShowLoginInfo(this.config.showLoginInfo);
            this.view.setShowAccountLink(this.config.showAccountLink);
        }
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.WebDavConfigurationView = Class.extend(
    /** @lends {jscape.domain.WebDavConfigurationView.prototype} */
{
    DATA_URL: {pattern: /data:image\/.*;base64,(.+)/, prefix: "data:image/png;base64,"},

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=webdavconfigfields]");

        this.logo = $("div.logo-preview>img", this.fieldset);
        this.logoFile = $("input[name=logo]", this.fieldset).css({width: 0, height: 0, opacity: 0, position: "absolute", left: "-10000px"});
        this.logoFile.change($.proxy(this._onChange$Logo, this));
        this.logoButton = $("a[role=button][name=changelogo]", this.fieldset);
        this.logoButton.linkbutton({onClick: $.proxy(function() { this.logoFile.click(); }, this)});
        this.logo.parent().click($.proxy(function() { this.logoFile.click(); }, this));

        this.logoutUrl = $("input[name=logouturl]", this.fieldset);

        this.appletEnabled = $("input[name=appletallowed]:checkbox", this.fieldset);
        this.docViewerEnabled = $("input[name=docviewerallowed]:checkbox", this.fieldset);
        this.showLogin = $("input[name=showlogin]:checkbox", this.fieldset);
        this.showAccount = $("input[name=showaccount]:checkbox", this.fieldset);

        this.resourcesButton = $("a[role=button][name=resources]", this.fieldset);
        this.resourcesButton.linkbutton({onClick: $.proxy(this._onClick$ResourcesButton, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getLogoData: function () {
        return (this.logo.attr("src") || "").replace(this.DATA_URL.pattern, "$1");
    },

    setLogoData: function (data) {
        this.logo[0].src = this.DATA_URL.pattern.test(data) ? data : this.DATA_URL.prefix + (data || "");
    },

    getLogoutUrl: function() {
        return this.logoutUrl.val();
    },

    setLogoutUrl: function(value) {
        this.logoutUrl.val(value);
    },

    isAppletEnabled: function() {
        return this.appletEnabled.is(":checked");
    },

    setAppletEnabled: function(value) {
        this.appletEnabled.prop("checked", !!value);
    },

    isDocumentViewerEnabled: function() {
        return this.docViewerEnabled.is(":checked");
    },

    setDocumentViewerEnabled: function(value) {
        this.docViewerEnabled.prop("checked", !!value);
    },

    isShowLoginInfo: function() {
        return this.showLogin.is(":checked");
    },

    setShowLoginInfo: function(value) {
        this.showLogin.prop("checked", !!value);
    },

    isShowAccountLink: function() {
        return this.showAccount.is(":checked");
    },

    setShowAccountLink: function(value) {
        this.showAccount.prop("checked", !!value).change();
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.WebDavConfigurationView.LISTENER;
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$ResourcesButton: function() {
        this.listener.onSetupResources();
    },

    _onChange$Logo: function (e) {
        var files = e.target.files;
        if (files && files.length) {
            this.listener.onChangeLogo(files[0]);
        }
        return false;
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard();
    }
});
jscape.domain.WebDavConfigurationView.LISTENER = {
    onSetupResources: $.noop,
    onChangeLogo: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.SslCiphersController = Class.extend(
    /** @lends {jscape.domain.SslCiphersController.prototype} */
{
    /** @const */
    STRONG_CIPHERS: [
        "SSL_RSA_WITH_RC4_128_MD5",
        "SSL_RSA_WITH_RC4_128_SHA",
        "SSL_RSA_WITH_3DES_EDE_CBC_SHA",
        "SSL_RSA_WITH_AES_128_CBC_SHA",
        "SSL_RSA_FIPS_WITH_3DES_EDE_CBC_SHA",
        "SSL_DHE_RSA_WITH_AES_128_CBC_SHA",
        "SSL_DHE_DSS_WITH_AES_128_CBC_SHA",
        "SSL_DHE_RSA_WITH_3DES_EDE_CBC_SHA",
        "SSL_DHE_DSS_WITH_3DES_EDE_CBC_SHA",
        "SSL_DHE_DSS_WITH_RC4_128_SHA",
        "SSL_KRB5_WITH_RC4_128_SHA",
        "SSL_KRB5_WITH_RC4_128_MD5",
        "SSL_KRB5_WITH_3DES_EDE_CBC_SHA",
        "SSL_KRB5_WITH_3DES_EDE_CBC_MD5",
        "TLS_RSA_WITH_AES_128_CBC_SHA",
        "TLS_DHE_RSA_WITH_AES_128_CBC_SHA",
        "TLS_DHE_DSS_WITH_AES_128_CBC_SHA",
        "TLS_KRB5_WITH_RC4_128_SHA",
        "TLS_KRB5_WITH_RC4_128_MD5",
        "TLS_KRB5_WITH_3DES_EDE_CBC_SHA",
        "TLS_KRB5_WITH_3DES_EDE_CBC_MD5"
    ],
    MEDIUM_CIPHERS: [
        "SSL_DHE_RSA_WITH_DES_CBC_SHA",
        "SSL_DHE_DSS_WITH_DES_CBC_SHA",
        "SSL_RSA_WITH_DES_CBC_SHA",
        "SSL_RSA_FIPS_WITH_DES_CBC_SHA",
        "SSL_KRB5_WITH_DES_CBC_SHA",
        "SSL_KRB5_WITH_DES_CBC_MD5",
        "TLS_KRB5_WITH_DES_CBC_SHA",
        "TLS_KRB5_WITH_DES_CBC_MD5"
    ],

    deferred: null,
    ciphers: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.MEDIUM_CIPHERS = this.MEDIUM_CIPHERS.concat(this.STRONG_CIPHERS);
    },

    start: function(managementService, domainName, ciphers) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._setupDialog(ciphers).show(this);

        return this.deferred.promise();
    },

    onSelectMedium: function(dialog) {
        dialog.setEnabledCiphers(this.MEDIUM_CIPHERS);
    },

    onSelectStrong: function(dialog) {
        dialog.setEnabledCiphers(this.STRONG_CIPHERS);
    },

    onSubmit: function(dialog) {
        var ciphers = dialog.getEnabledCiphers();
        dialog.destroy();
        this.deferred.resolve(ciphers);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _setupDialog: function(ciphers) {
        var dialog = new jscape.domain.SslCiphersDialog(this.resources);
        if (this.ciphers == null) {
            this.managementService.getServerParameters().done($.proxy(function(parameters) {
                this.ciphers = parameters && parameters.getSslCiphers() || null;
                dialog.setCiphers(this.ciphers);
                dialog.setEnabledCiphers(ciphers);
            }, this));
        } else {
            dialog.setCiphers(this.ciphers);
            dialog.setEnabledCiphers(ciphers);
        }
        return dialog;
    }
});

/** @class */
jscape.domain.SslCiphersDialog = Class.extend(
    /** @lends {jscape.domain.SslCiphersDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-services-ciphers").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=sslciphersfields]", this.dialog);
        this.data = $("table[role=grid][aria-label=sslciphers]", this.fieldset);
        this.data.datagrid({
            width: "auto", height: 250, fitColumns: true,
            remoteSort: false, showHeader: false,
            singleSelect: false, selectOnCheck: true, checkOnSelect: true,
            toolbar: "#{0} div.viewpane-toolbar".supplant({0: this.dialog.attr("id")}),
            onLoadSuccess: $.proxy(this._adjustButtonsState, this),
            idField: "value",
            columns: [[
                {field: "value", checkbox: true},
                {field: "text", sortable: true}
            ]]
        });
        this.data.filter(".datagrid-body td").css("border", "transparent");

        this.selectAllButton = $("a[role=button][name=all]", this.fieldset);
        this.selectAllButton.linkbutton({toggle: true, onClick: $.proxy(this._onClick$SelectAllButton, this)});

        this.selectMediumButton = $("a[role=button][name=medium]", this.fieldset);
        this.selectMediumButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$SelectMediumButton, this)});

        this.selectStrongButton = $("a[role=button][name=strong]", this.fieldset);
        this.selectStrongButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$SelectStrongButton, this)});
    },

    getEnabledCiphers: function() {
        return $.map(this.data.datagrid("getChecked"), function(row) { return row.value; });
    },

    setEnabledCiphers: function (values) {
        if (values && values.length) {
            this.data.datagrid("unselectAll");
            for (var i = 0; i < values.length; i++) {
                this.data.datagrid("selectRecord", values[i]);
            }
        } else {
            this.data.datagrid("clearChecked");
        }
    },

    setCiphers: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", $.map(values, function(value) { return {text: value, value: value};}));
        this.data.datagrid("loaded");
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.SslCiphersDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$SelectAllButton: function() {
        var text = this.selectAllButton.text();
        this.selectAllButton.html(this.selectAllButton.html().replace(text, this.selectAllButton.data("toggle-text")));
        this.selectAllButton.data("toggle-text", text);

        if (this.selectAllButton.linkbutton("options").selected) {
            this.data.datagrid("selectAll");
        } else {
            this.data.datagrid("unselectAll");
        }
    },

    _onClick$SelectMediumButton: function() {
        this.listener.onSelectMedium(this);
    },

    _onClick$SelectStrongButton: function() {
        this.listener.onSelectStrong(this);
    },

    _adjustButtonsState: function() {
        var enabled = this.data.datagrid("getData").length != 0;
        this.selectAllButton.linkbutton(enabled ? "enable" : "disable");
        this.selectMediumButton.linkbutton(enabled ? "enable" : "disable");
        this.selectStrongButton.linkbutton(enabled ? "enable" : "disable");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.SslCiphersDialog.LISTENER = {
    onSelectMedium: $.noop,
    onSelectStrong: $.noop,
    onSubmit: $.noop,
    onCancel: $.noop
};

/** @class */
jscape.domain.SftpAlgorithmsController = Class.extend(
    /** @lends {jscape.domain.SftpAlgorithmsController.prototype} */
{
    /** @const */
    DEFAULT_ALGORITHMS: {kexes: [], ciphers: [], macs: [], compressions: []},

    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(algorithms) {
        this.deferred = $.Deferred();

        this._setupDialog(algorithms || this.DEFAULT_ALGORITHMS).show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var algorithms = this._createAlgorithms(dialog);
        dialog.destroy();
        this.deferred.resolve(algorithms);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createAlgorithms: function(dialog) {
        return {kexes: dialog.getKexes(), ciphers: dialog.getCiphers(), macs: dialog.getMacs(), compressions: dialog.getCompressions()}
    },

    _setupDialog: function (algorithms) {
        var dialog = new jscape.domain.SftpAlgorithmsDialog(this.resources);
        dialog.setKexes(algorithms.kexes);
        dialog.setCiphers(algorithms.ciphers);
        dialog.setMacs(algorithms.macs);
        dialog.setCompressions(algorithms.compressions);
        return dialog;
    }
});

/** @class */
jscape.domain.SftpAlgorithmsDialog = Class.extend(
    /** @lends {jscape.domain.SftpAlgorithmsDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-services-sftpalgorithms").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=sftpalgorithmsfields]", this.dialog);
        this.tabs = $("[role=tablist]", this.fieldset);
        this.tabs.tabs({plain: true, width: 440, height: 230});

        this.kexes = $("table[role=grid][aria-label=kexes]", this.fieldset);
        this.kexes.datagrid({
            border: false, fitColumns: true, remoteSort: false, showHeader: false,
            singleSelect: false, selectOnCheck: true, checkOnSelect: true,
            idField: "value",
            columns: [[{field: "value", checkbox: true}, {field: "text", formatter: function(_, row) { return row.value; }}]]
        });

        this.ciphers = $("table[role=grid][aria-label=ciphers]", this.fieldset);
        this.ciphers.datagrid({
            border: false, fitColumns: true, remoteSort: false, showHeader: false,
            singleSelect: false, selectOnCheck: true, checkOnSelect: true,
            idField: "value",
            columns: [[{field: "value", checkbox: true}, {field: "text", formatter: function(_, row) { return row.value; }}]]
        });
        this.macs = $("table[role=grid][aria-label=macs]", this.fieldset);
        this.macs.datagrid({
            border: false, fitColumns: true, remoteSort: false, showHeader: false,
            singleSelect: false, selectOnCheck: true, checkOnSelect: true,
            idField: "value",
            columns: [[{field: "value", checkbox: true}, {field: "text", formatter: function(_, row) { return row.value; }}]]
        });
        this.compressions = $("table[role=grid][aria-label=compressions]", this.fieldset);
        this.compressions.datagrid({
            border: false, fitColumns: true, remoteSort: false, showHeader: false,
            singleSelect: false, selectOnCheck: true, checkOnSelect: true,
            idField: "value",
            columns: [[{field: "value", checkbox: true}, {field: "text", formatter: function(_, row) { return row.value; }}]]
        });
    },

    getKexes: function() {
        return this._getAlgorithms(this.kexes);
    },

    setKexes: function (values) {
        this._setAlgorithms(this.kexes, values);
    },

    getCiphers: function() {
        return this._getAlgorithms(this.ciphers);
    },

    setCiphers: function (values) {
        this._setAlgorithms(this.ciphers, values);
    },

    getMacs: function() {
        return this._getAlgorithms(this.macs);
    },

    setMacs: function (values) {
        this._setAlgorithms(this.macs, values);
    },

    getCompressions: function() {
        return this._getAlgorithms(this.compressions);
    },

    setCompressions: function (values) {
        this._setAlgorithms(this.compressions, values);
    },


    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _getAlgorithms: function(grid) {
        return $.map(grid.datagrid("getChecked"), function(row) { return row.value; });
    },

    _setAlgorithms: function (grid, values) {
        if (values && values.length) {
            grid.datagrid("unselectAll");
            for (var i = 0; i < values.length; i++) {
                grid.datagrid("selectRecord", values[i]);
            }
        } else {
            grid.datagrid("clearChecked");
        }
    }
});

/** @class */
jscape.domain.WebResourcesController = Class.extend(
    /** @lends {jscape.domain.WebResourcesController.prototype} */
{
    languages: null,
    copy: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(languages) {
        this.deferred = $.Deferred();
        this.languages = languages || [];
        this.copy = this._copyWebResources();

        this.dialog = this._setupDialog();
        this.dialog.show(this);

        return this.deferred.promise();
    },

    onAddLanguage: function() {
        var dialog = new jscape.domain.AddLanguageDialog(this.resources);
        dialog.disableLanguages($.map(this.languages, function(item) { return item.lang; }));
        dialog.show({
            onSubmit: $.proxy(function(dialog) {
                var language = this._createWebResources(dialog);
                dialog.destroy();
                this.languages.push(language);
                this.dialog.setLanguages(this.languages);
                this.dialog.selectLanguage(language);
            }, this),
            onCancel: function(d) { d.destroy(); }
        });
    },

    onResetToDefault: function() {
        this.languages = this.copy;
        this.copy = this._copyWebResources();
        this.dialog.setLanguages(this.languages);
    },

    onSubmit: function(dialog) {
        var languages = dialog.getLanguages();
        dialog.destroy();
        this.deferred.resolve(languages);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createWebResources: function(dialog) {
        var defaultWebResources = $.grep(this.languages, function(item) { return item.isDefault(); });
        var properties = defaultWebResources.length ? $.extend({}, defaultWebResources[0].properties) : {};
        return new jscape.WebResources(dialog.getLanguage(), dialog.getDisplayName(), properties);
    },

    _copyWebResources: function() {
        return $.map(this.languages, function(item) { return item.clone(); });
    },

    _setupDialog: function() {
        var dialog = new jscape.domain.WebResourcesDialog(this.resources);
        dialog.setLanguages(this.languages);
        return dialog;
    }
});

/** @class */
jscape.domain.WebResourcesDialog = Class.extend(
    /** @lends {jscape.domain.WebResourcesDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-services-webresources").clone().attr("id", "");
        this.dialog.dialog({
            width: 700, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=webresourcesfields]", this.dialog);

        this.language = $("select[name=language]", this.fieldset);
        this.language.combobox({
            required: true, editable: false, width: 170, panelHeight: "auto",
            valueField: "lang", textField: "displayName", selectOnNavigation: false,
            onChange: $.proxy(this._onChange$Language, this),
            onLoadSuccess: function(data) { data.length && $(this).combobox("select", data[0].lang).combobox("options").onChange.call(this); }
        });

        this.data = $("table[role=grid][aria-label=resources]", this.fieldset);
        this.data.propertygrid({
            showGroup: false,
            scrollbarSize: 0,
            sortName: "name",
            sortOrder: "asc",
            onAfterEdit: $.proxy(this._commitChanges, this)
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.resetButton = $("a[role=button][name=reset]", this.fieldset);
        this.resetButton.linkbutton({onClick: $.proxy(this._onClick$ResetButton, this)});
    },

    getLanguages: function() {
        var data = this.language.combobox("getData");
        return $.map(data, function(item) { if (item.isDefault()) { item.displayName = ""; } return item; });
    },

    setLanguages: function (values) {
        var data = $.map(values || [], function(item) { if (item.isDefault()) { item.displayName = "default"; } return item; });
        this.language.combobox("loadData", data);
    },

    selectLanguage: function(value) {
        this.language.combobox("select", value.lang);
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.WebResourcesDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.language.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$AddButton: function() {
        this.listener.onAddLanguage();
    },

    _onClick$ResetButton: function() {
        this.listener.onResetToDefault();
    },

    _onChange$Language: function() {
        var language = this.language.combobox("getValue");
        var properties = this._propertiesFor(language);
        if (properties != null) {
            this._selectProperties(properties);
        }
        return false;
    },

    _commitChanges: function(_, data) {
        var language = this.language.combobox("getValue");
        var properties = this._propertiesFor(language);
        if (properties != null) {
            properties[data.name] = data.value;
        }
    },

    _selectProperties: function(properties) {
        var data = $.map(properties, function(value, key) { return {name: key, value: value, editor: "text"}});
        this.data.propertygrid("loading");
        this.data.propertygrid("loadData", data);
        this.data.propertygrid("loaded");
    },

    _propertiesFor: function (language) {
        var data = this.language.combobox("getData");
        for (var i = 0; i < data.length; i++) {
            var webResources = data[i];
            if (webResources.lang == language) {
                return webResources.properties;
            }
        }
        return null;
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.WebResourcesDialog.LISTENER = {
    onAddLanguage: $.noop,
    onResetToDefault: $.noop,
    onSubmit: $.noop,
    onCancel: $.noop
};

/** @class */
jscape.domain.AddLanguageDialog = Class.extend(
    /** @lends {jscape.domain.AddLanguageDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-services-addwebresource").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.language = $("select[name=language]", this.dialog);
    },

    getLanguage: function() {
        return this.language.val();
    },

    getDisplayName: function() {
        return this.language.find("option:selected").html();
    },

    disableLanguages: function(values) {
        for (var i = 0; i < values.length; i++) {
            this.language.find("option[value=" + values[i] + "]").prop("disabled", true);
        }
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.language.val(this.language.find("option:not(:disabled):first").val()).focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        this.hide();
        this.listener.onSubmit(this);
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.UploadFormsController = Class.extend(
    /** @lends {jscape.domain.UploadFormsController.prototype} */
{
    forms: null,
    dialog: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.addController = new jscape.domain.AddFormController(resources);
        this.editController = new jscape.domain.EditFormController(resources);

        this._initValidators(resources);
    },

    start: function(groups, forms) {
        this.groups = $.map(groups || [], function(group) { return group.name; });
        this.forms = forms || [];
        this.deferred = $.Deferred();

        this.dialog = this._setupDialog(forms);
        this.dialog.show(this);

        return this.deferred.promise();
    },

    onFormUp: function (form) {
        var index = $.inArray(form, this.forms);
        if (index > 0) {
            this.forms.splice(index - 1, 2, this.forms[index], this.forms[index - 1]);
            this.dialog.setForms(this.forms);
            this.dialog.selectItem(form);
        }
    },

    onFormDown: function(form) {
        var index = $.inArray(form, this.forms);
        if (index != -1 && index < this.forms.length - 1) {
            this.forms.splice(index, 2, this.forms[index + 1], this.forms[index]);
            this.dialog.setForms(this.forms);
            this.dialog.selectItem(form);
        }
    },

    onAddForm: function() {
        this.addController.start(this.groups).done($.proxy(this._formAdded, this));
    },

    onEditForm: function(form) {
        var index = $.inArray(form, this.forms);
        if (index != -1) {
            this.editController.start(this.groups, form).done($.proxy(this._formEdited, this, index));
        }
    },

    onDeleteForm: function(forms) {
        this._confirmDelete(forms).done($.proxy(this._formDeleted, this, forms));
    },

    onSubmit: function(dialog) {
        dialog.destroy();
        this.deferred.resolve(this.forms);
        this.dialog = null;
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.dialog = null;
    },

    _formAdded: function (form) {
        this.forms.push(form);
        this.dialog.setForms(this.forms);
        this.dialog.selectItem(form);
    },

    _formEdited: function (index, form) {
        this.forms[index] = form;
        this.dialog.setForms(this.forms);
        this.dialog.selectItem(form);
    },

    _formDeleted: function (forms) {
        var index = -1;
        while (forms.length) {
            index = $.inArray(forms.pop(), this.forms);
            this.forms.splice(index, 1);
        }
        this.dialog.setForms(this.forms);
        if (this.forms.length) {
            index = Math.max(0, Math.min(index, this.forms.length - 1));
            this.dialog.selectItem(this.forms[index]);
        }
    },

    _confirmDelete: function(forms) {
        var text = $.map(forms || [], function(form) { return form.name; }).join(", ");
        return jscape.ConfirmDialog.confirm(this.resources.getString("UPLOAD_FORMS_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("UPLOAD_FORMS_CONFIRM_DELETE_MESSAGE", {0: text}));
    },

    _setupDialog: function(forms) {
        var dialog = new jscape.domain.UploadFormsDialog(this.resources);
        dialog.setForms(forms);
        return dialog;
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.rule("uploadform_name", $.proxy(this._uploadFormNameValid, this), resources.getString("UPLOAD_FORMS_ERROR_BAD_NAME"));
        v.rule("uploadform_unique", $.proxy(this._uploadFormUnique, this), resources.getString("UPLOAD_FORMS_ERROR_ALREADY_EXISTS"));
    },

    _uploadFormNameValid: function(value) {
        return /^[^\s_.]+$/.test(value);
    },

    _uploadFormUnique: function(value) {
        return $.grep(this.forms, function(form) { return form.name === value; }).length == 0;
    }
});

/** @class */
jscape.domain.UploadFormsDialog = Class.extend(
    /** @lends {jscape.domain.UploadFormsDialog.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-services-uploadforms").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=uploadformsfields]", this.dialog);

        this.data = $("table[role=grid][aria-label=uploadforms]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            singleSelect: true,
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            idField: "name",
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), width: 150},
                {field: "enabled", title: $("thead th:nth-child(2)", this.data).text(), width: 70}
            ]]
        });

        this.moveUpButton = $("a[role=button][name=moveup]", this.fieldset);
        this.moveUpButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveUpButton, this)});

        this.moveDownButton = $("a[role=button][name=movedown]", this.fieldset);
        this.moveDownButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveDownButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    setForms: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectItem: function(value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.UploadFormsDialog.LISTENER;
        this.selection = [];
        this.dialog.dialog("center").dialog("open");
        this._adjustButtonsState();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$MoveUpButton: function() {
        this.listener.onFormUp(this.selection[0]);
    },

    _onClick$MoveDownButton: function() {
        this.listener.onFormDown(this.selection[0]);
    },

    _onClick$AddButton: function() {
        this.listener.onAddForm();
    },

    _onClick$EditButton: function() {
        this.listener.onEditForm(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteForm(this.selection);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _adjustRowSelection: function () {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function () {
        var index = this.selection.length == 1 ? this.data.datagrid("getRowIndex", this.selection[0]) : -1;
        var count = this.data.datagrid("getRows").length;

        this.moveUpButton.linkbutton(index > 0 && index <= count - 1 ? "enable" : "disable");
        this.moveDownButton.linkbutton(index >= 0 && index < count - 1 ? "enable" : "disable");
        this.editButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }
});
jscape.domain.UploadFormsDialog.LISTENER = {
    onFormUp: $.noop,
    onFormDown: $.noop,
    onAddForm: $.noop,
    onEditForm: $.noop,
    onDeleteForm: $.noop,
    onSubmit: $.noop,
    onCancel: $.noop
};

/** @class */
jscape.domain.BaseFormController = Class.extend(
    /** @lends {jscape.domain.BaseFormController.prototype} */
{
    deferred: null,
    dialog: null,
    inputs: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.addController = new jscape.domain.AddFormInputController(resources);
        this.editController = new jscape.domain.EditFormInputController(resources);
    },

    start: function(groups, inputs) {
        this.inputs = inputs || [];
        this.deferred = $.Deferred();
        this.dialog = this._setupDialog(groups).show(this);
        return this.deferred.promise();
    },

    onInputUp: function (input) {
        var index = $.inArray(input, this.inputs);
        if (index > 0) {
            this.inputs.splice(index - 1, 2, this.inputs[index], this.inputs[index - 1]);
            this.dialog.setInputs(this.inputs);
            this.dialog.selectItem(input);
        }
    },

    onInputDown: function (input) {
        var index = $.inArray(input, this.inputs);
        if (index != -1 && index < this.inputs.length - 1) {
            this.inputs.splice(index, 2, this.inputs[index + 1], this.inputs[index]);
            this.dialog.setInputs(this.inputs);
            this.dialog.selectItem(input);
        }
    },

    onAddInput: function() {
        this.addController.start().done($.proxy(function(input) {
            this.inputs.push(input);
            this.dialog.setInputs(this.inputs);
            this.dialog.selectItem(input);
        }, this));
    },

    onEditInput: function(input) {
        var index = $.inArray(input, this.inputs);
        if (index != -1) {
            this.editController.start(input).done($.proxy(function(input) {
                this.inputs[index] = input;
                this.dialog.setInputs(this.inputs);
                this.dialog.selectItem(input);
            }, this));
        }
    },

    onDeleteInput: function (inputs) {
        var index = -1;
        while (inputs.length) {
            index = $.inArray(inputs.pop(), this.inputs);
            this.inputs.splice(index, 1);
        }
        this.dialog.setInputs(this.inputs);
    },

    onSubmit: function(dialog) {
        var form = this._createForm(dialog);
        dialog.destroy();
        this.deferred.resolve(form);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createForm: function(dialog) {
        throw "Abstract method";
    },

    _setupDialog: function(groups) {
    },

    _initValidators: function (resources) {
        var v = new jscape.Validator();
        v.rule("uploadform_input_name", $.proxy(this._uploadFormInputNameValid, this), resources.getString("UPLOAD_FORMS_ERROR_BAD_INPUT_NAME"));
        v.rule("uploadform_input_unique", $.proxy(this._uploadFormInputNotExists, this), resources.getString("UPLOAD_FORMS_ERROR_INPUT_ALREADY_EXISTS"));
    },

    _uploadFormInputNameValid: function(value) {
        return /^[^\s_.]+$/gi.test(value);
    },

    _uploadFormInputNotExists: function (value) {
        return $.grep(this.inputs,function (input) { return input.name == value; }).length == 0;
    }
});

/** @class */
jscape.domain.AddFormController = jscape.domain.BaseFormController.extend(
    /** @lends {jscape.domain.AddFormController.prototype} */
{
    /** @override */
    start: function (groups) {
        this._initValidators(this.resources);
        return this._super(groups, []);
    },

    /** @override */
    _createForm: function(dialog) {
        return new jscape.UploadForm(dialog.getName(),
                                     dialog.getDescription(),
                                     dialog.isEnabled(),
                                     dialog.isBatch(),
                                     dialog.getGroup(),
                                     dialog.getInputs());
    },

    /** @override */
    _setupDialog: function(groups) {
        var dialog = new jscape.domain.AddFormDialog(this.resources);
        dialog.setGroups(groups);
        dialog.setBatch(true);
        dialog.setEnabled(true);
        return dialog;
    }
});

/** @class */
jscape.domain.EditFormController = jscape.domain.BaseFormController.extend(
    /** @lends {jscape.domain.EditFormController.prototype} */
{
    /** @override */
    start: function(groups, form) {
        this.form = form;
        this._initValidators(this.resources);
        return this._super(groups, form.inputs);
    },

    /** @override */
    _createForm: function(dialog) {
        return new jscape.UploadForm(this.form.name,
                                     dialog.getDescription(),
                                     dialog.isEnabled(),
                                     dialog.isBatch(),
                                     dialog.getGroup(),
                                     dialog.getInputs());
    },

    /** @override */
    _setupDialog: function(groups) {
        var dialog = new jscape.domain.EditFormDialog(this.resources);
        dialog.setFormName(this.form.name);
        dialog.setDescription(this.form.description);
        dialog.setEnabled(this.form.enabled);
        dialog.setBatch(this.form.batch);
        dialog.setGroups(groups);
        dialog.setGroup(this.form.group);
        dialog.setInputs(this.form.inputs);
        return dialog;
    }
});

/** @class */
jscape.domain.BaseFormDialog = Class.extend(
    /** @lends {jscape.domain.BaseFormDialog.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=uploadformfields]", this.dialog);

        this.description = $("input[name=description]", this.fieldset);
        this.prompt = $("input[name=prompt]:radio", this.fieldset);

        this.groupEnabled = $("input[name=grouprequired]:checkbox", this.fieldset);
        this.groupEnabled.change($.proxy(function() {
            var enabled = this.groupEnabled.is(":checked");
            this.group.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.group = $("select[name=group]", this.fieldset);
        this.group.combobox({
            required: true, editable: false, disabled: true, width: 150, panelHeight: "auto",
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.group.combobox("select", data[0].value);
                } else {
                    this.groupEnabled.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });

        this.enabled = $("input[name=enabled]:checkbox", this.fieldset);

        this.inputs = $("table[role=grid][aria-label=inputs]", this.fieldset);
        this.inputs.datagrid({
            height: 200,
            fitColumns: true,
            remoteSort: false,
            ctrlSelect: true,
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.inputs).text(), width: 100},
                {field: "type", title: $("thead th:nth-child(2)", this.inputs).text(), width: 80},
                {field: "value", title: $("thead th:nth-child(3)", this.inputs).text(), width: 150},
                {field: "required", title: $("thead th:nth-child(4)", this.inputs).text(), width: 60}
            ]]
        });

        this.moveUpButton = $("a[role=button][name=moveup]", this.fieldset);
        this.moveUpButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveUpButton, this)});

        this.moveDownButton = $("a[role=button][name=movedown]", this.fieldset);
        this.moveDownButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveDownButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    getDescription: function() {
        return this.description.val();
    },

    setDescription: function(value) {
        this.description.val(value);
    },

    isBatch: function() {
        return this.prompt.filter("[value=batch]").is(":checked");
    },

    setBatch: function(value) {
        this.prompt.filter(!!value ? "[value=batch]" : "[value=file]").prop("checked", true);
    },

    getGroup: function() {
        return this.groupEnabled.is(":checked") ? this.group.combobox("getValue") : null;
    },

    setGroup: function(value) {
        this.groupEnabled.prop("checked", value != null).change();
        if (value != null) {
            this.group.combobox("setValue", value);
        }
    },

    setGroups: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.group.combobox("loadData", data);
    },

    isEnabled: function() {
        return this.enabled.is(":checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", !!value);
    },

    getInputs: function() {
        return this.inputs.datagrid("getRows");
    },

    setInputs: function(values) {
        this.inputs.datagrid("loading");
        this.inputs.datagrid("loadData", values);
        this.inputs.datagrid("loaded");
    },

    selectItem: function(input) {
        this.inputs.datagrid("selectRow", this.inputs.datagrid("getRowIndex", input));
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.BaseFormDialog.LISTENER;
        this.selection = [];
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this._adjustButtonsState();
        return this;
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
        return this;
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$MoveUpButton: function() {
        this.listener.onInputUp(this.selection[0]);
    },

    _onClick$MoveDownButton: function () {
        this.listener.onInputDown(this.selection[0]);
    },

    _onClick$AddButton: function() {
        this.listener.onAddInput();
    },

    _onClick$EditButton: function() {
        this.listener.onEditInput(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteInput(this.selection);
    },

    _adjustRowSelection: function() {
        this.selection = this.inputs.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var index = this.selection.length == 1 ? this.inputs.datagrid("getRowIndex", this.selection[0]) : -1;
        var count = this.inputs.datagrid("getRows").length;

        this.moveUpButton.linkbutton(index > 0 && index <= count - 1 ? "enable" : "disable");
        this.moveDownButton.linkbutton(index >= 0 && index < count - 1 ? "enable" : "disable");
        this.editButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.BaseFormDialog.LISTENER = {
    onInputUp: $.noop,
    onInputDown: $.noop,
    onAddInput: $.noop,
    onEditInput: $.noop,
    onDeleteInput: $.noop
};

/** @class */
jscape.domain.AddFormDialog = jscape.domain.BaseFormDialog.extend(
    /** @lends {jscape.domain.AddFormDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-uploadforms-add"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["uploadform_name", "uploadform_unique"], missingMessage: resources.getString("UPLOAD_FORMS_ERROR_BAD_NAME")});
    },

    getName: function() {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditFormDialog = jscape.domain.BaseFormDialog.extend(
    /** @lends {jscape.domain.EditFormDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-uploadforms-edit"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    setFormName: function(value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});

/** @class */
jscape.domain.BaseFormInputController = Class.extend(
    /** @lends {jscape.domain.BaseFormInputController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function (resources) {
        this.resources = resources;
    },

    start: function() {
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var input = this._createFormInput(dialog);
        dialog.destroy();
        this.deferred.resolve(input);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createFormInput: function(dialog) {
        throw "Abstract method";
    },

    _setupDialog: function() {
    }
});

/** @class */
jscape.domain.AddFormInputController = jscape.domain.BaseFormInputController.extend(
    /** @lends {jscape.domain.AddFormInputController.prototype} */
{
    _createFormInput: function(dialog) {
        return new jscape.UploadForm.Input(dialog.getType(), dialog.getName(), dialog.getLabel(), dialog.getValue(), dialog.isRequired());
    },

    _setupDialog: function() {
        var dialog = new jscape.domain.AddFormInputDialog(this.resources);
        dialog.setType(jscape.UploadForm.Input.TYPE.TEXT);
        dialog.setRequired(true);
        return dialog;
    }
});

/** @class */
jscape.domain.EditFormInputController = jscape.domain.BaseFormInputController.extend(
    /** @lends {jscape.domain.EditFormInputController.prototype} */
{
    /** @override */
    start: function(input) {
        this.input = input;
        return this._super();
    },

    _createFormInput: function(dialog) {
        return new jscape.UploadForm.Input(dialog.getType(), this.input.name, dialog.getLabel(), dialog.getValue(), dialog.isRequired());
    },

    _setupDialog: function () {
        var dialog = new jscape.domain.EditFormInputDialog(this.resources);
        dialog.setName(this.input.name);
        dialog.setLabel(this.input.label);
        dialog.setType(this.input.type);
        dialog.setValue(this.input.value);
        dialog.setRequired(this.input.required);
        return dialog;
    }
});

/** @class */
jscape.domain.BaseFormInputDialog = Class.extend(
    /** @lends {jscape.domain.BaseFormInputDialog.prototype} */
{
    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=uploadforminputfields]", this.dialog);

        this.label = $("input[name=label]", this.fieldset);

        this.type = $("select[name=type]", this.fieldset);
        this.type.combobox({required: true, editable: false, panelHeight: "auto"});

        this.value = $("input[name=value]", this.fieldset);
        this.required = $("input[name=required]:checkbox", this.fieldset);
    },

    getLabel: function() {
        return this.label.val();
    },

    setLabel: function(value) {
        this.label.val(value);
    },

    getType: function() {
        return this.type.combobox("getValue");
    },

    setType: function(value) {
        this.type.combobox("setValue", value);
    },

    getValue: function() {
        return this.value.val();
    },

    setValue: function(value) {
        this.value.val(value);
    },

    isRequired: function() {
        return this.required.is(":checked");
    },

    setRequired: function(value) {
        this.required.prop("checked", !!value);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:not(:disabled):first").focus();
        return this;
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
        return this;
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.AddFormInputDialog = jscape.domain.BaseFormInputDialog.extend(
    /** @lends {jscape.domain.AddFormInputDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-uploadforms-addfield"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["uploadform_input_name", "uploadform_input_unique"], missingMessage: resources.getString("UPLOAD_FORMS_ERROR_BAD_INPUT_NAME")});
    },

    getName: function() {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditFormInputDialog = jscape.domain.BaseFormInputDialog.extend(
    /** @lends {jscape.domain.EditFormInputDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-uploadforms-editfield"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    setName: function(value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});