jscape.domain = jscape.domain || {};

/** @class */
jscape.Service = Class.extend(
    /** @lends {jscape.Service.prototype} */
{
    initialize: function(a, b, c) {
        this.protocol = a || undefined;
        this.host = b || undefined;
        this.port = parseInt(c) || undefined;
    }
});
jscape.Service.fromJSON = function(data) {
    return data ? $.extend(new jscape.Service(), data) : null;
};
jscape.Service.Protocol = {AFTP: "AFTP", FTP: "FTP", SFTP: "SFTP", OFTP: "OFTP", TFTP: "TFTP", HTTP: "HTTP", WEBDAV: "WEBDAV", AS2: "AS2"};
jscape.Service.FTP_TYPE = {REGULAR: "regular", EXPLICIT_SSL: "explicit SSL", FORCED_EXPLICIT_SSL: "forced explicit SSL", IMPLICIT_SSL: "implicit SSL"};
jscape.Service.ftp = function(host, port, type, key) {
    switch (type) {
        case jscape.Service.FTP_TYPE.EXPLICIT_SSL:
        case jscape.Service.FTP_TYPE.FORCED_EXPLICIT_SSL:
        case jscape.Service.FTP_TYPE.IMPLICIT_SSL:
            return $.extend(new jscape.Service(jscape.Service.Protocol.FTP, host, port), {type: type, keyAlias: key});
        default:
            return $.extend(new jscape.Service(jscape.Service.Protocol.FTP, host, port), {type: jscape.Service.FTP_TYPE.REGULAR});
    }
};
jscape.Service.sftp = function(host, port, authentication, key) {
    return $.extend(new jscape.Service(jscape.Service.Protocol.SFTP, host, port), {authenticationType: authentication, keyAlias: key});
};
jscape.Service.aftp = function(host, port, key, protect) {
    return $.extend(new jscape.Service(jscape.Service.Protocol.AFTP, host, port), {keyAlias: key, protectionRequired: !!protect});
};
jscape.Service.oftp = function(host, port, code, key, useSsl, encryptFile, signFile) {
    return $.extend(new jscape.Service(jscape.Service.Protocol.OFTP, host, port), {identificationCode: code + "", keyAlias: key + "", connectionInitiallySecured: !!useSsl, fileEncryptionRequired: !!encryptFile, fileSignatureRequired: !!signFile});
};
jscape.Service.tftp = function(host, port, username) {
    return $.extend(new jscape.Service(jscape.Service.Protocol.TFTP, host, port), {username: username});
};
jscape.Service.http = function(plain, secure) {
    return $.extend(new jscape.Service(jscape.Service.Protocol.HTTP), {plain: !!plain, secure: !!secure});
};
jscape.Service.webdav = function(plain, secure) {
    return $.extend(new jscape.Service(jscape.Service.Protocol.WEBDAV), {plain: !!plain, secure: !!secure});
};
jscape.Service.as2 = function(plain, secure) {
    return $.extend(new jscape.Service(jscape.Service.Protocol.AS2), {plain: !!plain, secure: !!secure});
};

/** @class */
jscape.domain.ServicesModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.ServicesModule.prototype} */
{
    onCreate: function() {
        this._super.apply(this, arguments);

        this.servicesController = new jscape.domain.ServicesController(this.resources);
        this.ftpController = new jscape.domain.FtpConfigurationController(this.resources);
        this.sftpController = new jscape.domain.SftpConfigurationController(this.resources);
        this.aftpController = new jscape.domain.AftpConfigurationController(this.resources);
        this.oftpController = new jscape.domain.OftpConfigurationController(this.resources);
        this.tftpController = new jscape.domain.TftpConfigurationController(this.resources);
        this.httpController = new jscape.domain.HttpConfigurationController(this.resources);
        this.webDavController = new jscape.domain.WebDavConfigurationController(this.resources);
    },

    onStart: function () {
        this.servicesController.start(this.managementService, this.domainName);
        this.ftpController.start(this.managementService, this.domainName);
        this.sftpController.start(this.managementService, this.domainName);
        this.aftpController.start(this.managementService, this.domainName);
        this.oftpController.start(this.managementService, this.domainName);
        this.tftpController.start(this.managementService, this.domainName);
        this.httpController.start(this.managementService, this.domainName);
        this.webDavController.start(this.managementService, this.domainName);
    },

    onPause: function() {
        var ftpChanged = this.ftpController.hasChanged();
        var sftpChanged = this.sftpController.hasChanged();
        var aftpChanged = this.aftpController.hasChanged();
        var oftpChanged = this.oftpController.hasChanged();
        var tftpChanged = this.tftpController.hasChanged();
        var httpChanged = this.httpController.hasChanged();
        var webDavChanged = this.webDavController.hasChanged();

        if (ftpChanged || sftpChanged || aftpChanged || tftpChanged || httpChanged || webDavChanged) {
            this._confirmApply()
                    .done($.proxy(function () {
                        ftpChanged && this.ftpController.onApply();
                        sftpChanged && this.sftpController.onApply();
                        aftpChanged && this.aftpController.onApply();
                        oftpChanged && this.oftpController.onApply();
                        tftpChanged && this.tftpController.onApply();
                        httpChanged && this.httpController.onApply();
                        webDavChanged && this.webDavController.onApply();
                    }, this));
        }
    },

    onResume: function() {
        this.servicesController._loadData();
        this.ftpController._loadData();
        this.sftpController._loadData();
        this.aftpController._loadData();
        this.oftpController._loadData();
        this.tftpController._loadData();
        this.httpController._loadData();
        this.webDavController._loadData();
    }
});

/** @class */
jscape.domain.ServicesController = Class.extend(
    /** @lends {jscape.domain.ServicesController.prototype} */
{
    services: null,

    /** @constructor */
    initialize: function(resources) {
        this.services = [];
        this.view = new jscape.domain.ServicesView(resources);
        this.addController = new jscape.domain.AddServiceController(resources);
        this.deleteController = new jscape.domain.DeleteServiceController(resources);
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onAddService: function() {
        this.addController.start(this.managementService, this.domainName)
                .done($.proxy(this._serviceAdded, this))
                .done($.proxy(this._autoStartService, this));
    },

    onDeleteService: function(service) {
        this.deleteController.start(this.managementService, this.domainName, service)
                .done($.proxy(this._serviceDeleted, this, service));
    },

    _loadData: function() {
        this.managementService.getWebServiceConfiguration()
                .done($.proxy(function(descriptor) {
                    if (descriptor) {
                        this.view.setWebConfiguration(descriptor.configuration);
                    }
                }, this));

        this.managementService.getServices(this.domainName)
                .done($.proxy(function(services) {
                    this.services = services || [];
                    this.view.setServices(this.services);
                }, this));
    },

    _autoStartService: function (service) {
        this.managementService.getDomainStatistics(this.domainName)
                .done($.proxy(function(stats) {
                    if (service && stats && stats.isRunning()) {
                        this.managementService.startService(this.domainName, service);
                    }
                }, this));
    },

    _serviceAdded: function(service) {
        this.services.push(service);
        this.view.setServices(this.services);
        this.view.selectItem(service);
    },

    _serviceDeleted: function (services) {
        var index = -1;
        while (services.length) {
            index = $.inArray(services.pop(), this.services);
            this.services.splice(index, 1);
        }
        this.view.setServices(this.services);

        if (this.services.length) {
            index = Math.max(0, Math.min(index, this.services.length - 1));
            this.view.selectItem(this.services[index]);
        }
    }
});

/** @class */
jscape.domain.ServicesView = Class.extend(
    /** @lends {jscape.domain.ServicesView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.fieldset = $("fieldset[name=servicesfields]");

        this.data = $("table[role=grid][aria-label=services]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            remoteSort: false,
            singleSelect: true,
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            sortName: "protocol",
            columns: [[
                {field: "protocol", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 60, formatter: $.proxy(function(_, row) { return this._formatProtocol(row); }, this)},
                {field: "host", title: $("thead th:nth-child(2)", this.data).text(), sortable: false, width: 120, formatter: $.proxy(function(_, row) { return this._formatHost(row); }, this)},
                {field: "port", title: $("thead th:nth-child(3)", this.data).text(), sortable: false, width: 100, align: "right", formatter: $.proxy(function(_, row) { return this._formatPort(row); }, this)},
                {field: "key", title: $("thead th:nth-child(4)", this.data).text(), sortable: false, width: 120, formatter: $.proxy(function(_, row) { return this._formatKey(row); }, this)},
                {field: "details", title: $("thead th:nth-child(5)", this.data).text(), sortable: false, width: 120, formatter: $.proxy(function(_, row) { return this._formatDetails(row); }, this)}
            ]]
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    setServices: function(values) {
        this.rows = values;
        this.data.datagrid("loading");
        this.data.datagrid("loadData", this.rows);
        this.data.datagrid("loaded");
    },

    selectItem: function(value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    setWebConfiguration: function(config) {
        this.config = config;

        if (this.rows) {
            this.data.datagrid("loading");
            this.data.datagrid("loadData", this.rows);
            this.data.datagrid("loaded");
        }
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.ServicesView.LISTENER;
        this.selection = [];
        this._adjustButtonsState();
    },

    _formatProtocol: function(service) {
        return this.resources.getString("APPLICATION_PROTOCOL_" + service.protocol.toUpperCase()) + (service.type ? " " + service.type : "");
    },

    _formatHost: function (service) {
        switch (service.protocol) {
            case jscape.Service.Protocol.HTTP:
            case jscape.Service.Protocol.WEBDAV:
            case jscape.Service.Protocol.AS2:
                return this._webHostFor(service.plain, service.secure);
            default:
                return service.host;
        }
    },

    _formatPort: function(service) {
        switch (service.protocol) {
            case jscape.Service.Protocol.HTTP:
            case jscape.Service.Protocol.WEBDAV:
            case jscape.Service.Protocol.AS2:
                return this._webPortFor(service.plain, service.secure);
            default:
                return service.port;
        }
    },

    _formatKey: function(service) {
        return service.keyAlias || "";
    },

    _formatDetails: function(service) {
        return service.protocol == jscape.Service.Protocol.SFTP
                ? this.resources.getString("SERVICES_TEMPLATE_SFTP_DETAILS", {0: service.authenticationType})
                : "";
    },

    _webHostFor: function (plain, secure) {
        var disabled = this.resources.getString("SERVICES_TEMPLATE_DISABLED");
        return this.resources.getString("SERVICES_TEMPLATE_WEB", {
            0: plain && this.config && this.config.httpEnabled ? this.config.httpAddress : disabled,
            1: secure && this.config && this.config.httpsEnabled ? this.config.httpsAddress : disabled
        });
    },

    _webPortFor: function(plain, secure) {
        var disabled = this.resources.getString("SERVICES_TEMPLATE_DISABLED");
        return this.resources.getString("SERVICES_TEMPLATE_WEB", {
            0: plain && this.config && this.config.httpEnabled ? this.config.httpPort : disabled,
            1: secure && this.config && this.config.httpsEnabled ? this.config.httpsPort : disabled
        });
    },

    _onClick$AddButton: function() {
        this.listener.onAddService();
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteService(this.selection);
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }
});
jscape.domain.ServicesView.LISTENER = {
    onAddService: $.noop,
    onDeleteService: $.noop
};

/** @class */
jscape.domain.AddServiceController = Class.extend(
    /** @lends {jscape.domain.AddServiceController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.controllerMap = {
            AFTP: jscape.domain.AftpServiceController,
            AS2: jscape.domain.As2ServiceController,
            FTP: jscape.domain.FtpServiceController,
            HTTP: jscape.domain.HttpServiceController,
            OFTP: jscape.domain.OftpServiceController,
            SFTP: jscape.domain.SftpServiceController,
            TFTP: jscape.domain.TftpServiceController,
            WEBDAV: jscape.domain.WebDavServiceController
        };
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.AddServiceProtocolDialog(this.resources);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var protocol = dialog.getProtocol();
        dialog.destroy();

        var theClass = this.controllerMap[protocol.toUpperCase()];
        var controller = new theClass(this.resources);
        controller.start(this.managementService, this.domainName)
                .done($.proxy(this._storeDescriptor, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _storeDescriptor: function(descriptor) {
        this.managementService.addService(this.domainName, descriptor)
                        .done($.proxy(function() { this.deferred.resolve(descriptor); }, this))
                        .fail($.proxy(function() { this.deferred.reject(); }, this));
    }
});

/** @class */
jscape.domain.AddServiceProtocolDialog = Class.extend(
    /** @lends {jscape.domain.AddServiceProtocolDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-service-add-protocol").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=serviceprotocolfileds]", this.dialog);
        this.protocol = $("select[name=protocol]", this.fieldset);
        this.protocol.combobox({required: true, editable: false, width: 150, panelHeight: "auto"});
    },

    getProtocol: function() {
        return this.protocol.combobox("getValue");
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.protocol.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.DeleteServiceController = Class.extend(
    /** @lends {jscape.domain.DeleteServiceController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, descriptors) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._confirmDelete(descriptors)
                .done($.proxy(this._deleteConfirmed, this, descriptors))
                .fail(this.deferred.reject, this.deferred);

        return this.deferred.promise();
    },

    _deleteConfirmed: function (descriptors) {
        var promises = [];
        for (var i = 0; i < descriptors.length; i++) {
            promises.push(this.managementService.deleteService(this.domainName, descriptors[i]));
        }
        $.when.apply(jQuery, promises)
                .done($.proxy(this.deferred.resolve, this.deferred, descriptors))
                .fail(this.deferred.reject);
    },

    _confirmDelete: function (descriptor) {
        //todo: warning dialog
        return jscape.ConfirmDialog.confirm(this.resources.getString("SERVICES_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("SERVICES_CONFIRM_DELETE_MESSAGE"));
    }
});

/** @class */
jscape.domain.FtpServiceController = Class.extend(
    /** @lends {jscape.domain.FtpServiceController.prototype} */
{
    DEFAULTS: {
        "regular": {port: 21, keyUsed: false},
        "explicit SSL": {port: 21, keyUsed: true},
        "forced explicit SSL": {port: 21, keyUsed: true},
        "implicit SSL": {port: 990, keyUsed: true}
    },

    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService) {
        this.deferred = $.Deferred();
        this._setupDialog(managementService).show(this);
        return this.deferred.promise();
    },

    onChangeType: function(dialog) {
        var type = dialog.getType();
        if (this.DEFAULTS.hasOwnProperty(type)) {
            dialog.setPort(this.DEFAULTS[type].port);
            dialog.setKeyUsed(this.DEFAULTS[type].keyUsed);
        }
    },

    onSubmit: function(dialog) {
        var descriptor = this._createDescriptor(dialog);
        dialog.destroy();
        this.deferred.resolve(descriptor);
        this.deferred = null;
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _createDescriptor: function (dialog) {
        return jscape.Service.ftp(dialog.getHost(), dialog.getPort(), dialog.getType(), dialog.getKeyAlias());
    },

    _setupDialog: function (managementService) {
        var dialog = new jscape.domain.FtpServiceDialog(this.resources);
        dialog.setType(jscape.Service.FTP_TYPE.EXPLICIT_SSL);
        this.onChangeType(dialog);

        $.when(managementService.getServerKeys(), managementService.getServerParameters())
                .done($.proxy(function (keys, parameters) {
                    var keys = $.map(keys || [], function (key) { return key.alias; });
                    var hosts = parameters && parameters.getInetAddresses();
                    dialog.setKeys(keys);
                    dialog.setHosts(hosts);
                }, this));

        return dialog;
    }
});

/** @class */
jscape.domain.FtpServiceDialog = Class.extend(
    /** @lends {jscape.domain.FtpServiceDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-services-ftp").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=ftpservicefields]", this.dialog);

        this.host = $("select[name=host]", this.fieldset);
        this.host.combobox({required: true, width: 200,  panelHeight: "auto", validType: "length[1,500]", missingMessage: resources.getString("SERVICES_ERROR_BAD_HOST"), invalidMessage: resources.getString("SERVICES_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 80});

        this.type = $("select[name=type]", this.fieldset);
        this.type.combobox({required: true, editable: false, panelHeight: "auto", onChange: $.proxy(this._onChange$Type, this)});

        this.keyAlias = $("select[name=privatekey]", this.fieldset);
        this.keyAlias.combobox({required: true, editable: false, panelHeight: "auto", width: 170});
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHosts: function(values) {
        var data = $.map(values || [], function(host) { return {text: host, value: host}; });
        this.host.combobox("loadData", data);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", parseInt(value)).numberspinner("isValid");
    },

    getType: function() {
        return this.type.combobox("getValue");
    },

    setType: function (value) {
        var select = this.type.combobox("getValue");
        this.type.combobox("setValue", value);
        if (select === value) {
            this._onChange$Type();
        }
    },

    getKeyAlias: function() {
        return !this.keyAlias.combobox("options").disabled ? this.keyAlias.combobox("getValue") : null;
    },

    setKeys: function(values) {
        var data = $.map(values || [], function(key) { return {text: key, value: key}; });
        this.keyAlias.combobox("loadData", data);
    },

    setKeyUsed: function(value) {
        this.keyAlias.combobox(!!value ? "enable" : "disable").combobox(!!value ? "enableValidation" : "disableValidation");
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.FtpServiceDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.host.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _onChange$Type: function() {
        if (this.listener) {
            this.listener.onChangeType(this);
        }
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.FtpServiceDialog.LISTENER = {
    onChangeType: $.noop,
    onSubmit: $.noop,
    onCancel: $.noop
};

/** @class */
jscape.domain.SftpServiceController = Class.extend(
    /** @lends {jscape.domain.SftpServiceController.prototype} */
{
    DEFAULT_PORT: 22,

    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService) {
        this.deferred = $.Deferred();
        this._setupDialog(managementService).show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var descriptor = this._createDescriptor(dialog);
        dialog.destroy();
        this.deferred.resolve(descriptor);
        this.deferred = null;
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _createDescriptor: function (dialog) {
        return jscape.Service.sftp(dialog.getHost(), dialog.getPort(), dialog.getAuthentication(), dialog.getKeyAlias());
    },

    _setupDialog: function(managementService) {
        var dialog = new jscape.domain.SftpServiceDialog(this.resources);
        dialog.setPort(this.DEFAULT_PORT);

        $.when(managementService.getServerKeys(), managementService.getServerParameters())
                .done($.proxy(function (keys, parameters) {
                    var hosts = parameters && parameters.getInetAddresses();
                    var keyAliases = $.map(keys || [], function (key) { return key.alias; });
                    dialog.setHosts(hosts);
                    dialog.setKeys(keyAliases);
                }, this));

        return dialog;
    }
});

/** @class */
jscape.domain.SftpServiceDialog = Class.extend(
    /** @lends {jscape.domain.SftpServiceDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-services-sftp").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=sftpservicefields]", this.dialog);

        this.host = $("select[name=host]", this.fieldset);
        this.host.combobox({required: true, width: 200,  panelHeight: "auto", validType: "length[1,500]", missingMessage: resources.getString("SERVICES_ERROR_BAD_HOST"), invalidMessage: resources.getString("SERVICES_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, value: this.DEFAULT_PORT, width: 80});

        this.keyAlias = $("select[name=privatekey]", this.fieldset);
        this.keyAlias.combobox({required: true, editable: false, width: 170,  panelHeight: "auto"});

        this.auth = $("select[name=auth]", this.fieldset);
        this.auth.combobox({required: true, editable: false, panelHeight: "auto"});
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHosts: function(values) {
        var data = $.map(values || [], function(host) { return {text: host, value: host}; });
        this.host.combobox("loadData", data);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", parseInt(value)).numberspinner("validate");
    },

    getAuthentication: function() {
        return this.auth.combobox("getValue");
    },

    getKeyAlias: function() {
        return !this.keyAlias.combobox("options").disabled ? this.keyAlias.combobox("getValue") : null;
    },

    setKeys: function(values) {
        var data = $.map(values || [], function(key) { return {text: key, value: key}; });
        this.keyAlias.combobox("loadData", data);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.host.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.AftpServiceController = Class.extend(
    /** @lends {jscape.domain.AftpServiceController.prototype} */
{
    DEFAULT_PORT: 3000,

    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService) {
        this.deferred = $.Deferred();
        this._setupDialog(managementService).show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var descriptor = this._createDescriptor(dialog);
        dialog.destroy();
        this.deferred.resolve(descriptor);
        this.deferred = null;
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _createDescriptor: function (dialog) {
        return jscape.Service.aftp(dialog.getHost(), dialog.getPort(), dialog.getKeyAlias(), dialog.isProtectionRequired());
    },

    _setupDialog: function(managementService) {
        var dialog = new jscape.domain.AftpServiceDialog(this.resources);
        dialog.setPort(this.DEFAULT_PORT);

        $.when(managementService.getServerKeys(), managementService.getServerParameters())
                .done($.proxy(function (keys, parameters) {
                    var hosts = parameters && parameters.getInetAddresses();
                    var keyAliases = $.map(keys || [], function (key) { return key.alias; });
                    dialog.setHosts(hosts);
                    dialog.setKeys(keyAliases);
                }, this));

        return dialog;
    }
});

/** @class */
jscape.domain.AftpServiceDialog = Class.extend(
    /** @lends {jscape.domain.AftpServiceDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-services-aftp").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=aftpservicefields]", this.dialog);

        this.host = $("select[name=host]", this.fieldset);
        this.host.combobox({required: true, width: 200,  panelHeight: "auto", validType: "length[1,500]", missingMessage: resources.getString("SERVICES_ERROR_BAD_HOST"), invalidMessage: resources.getString("SERVICES_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 80});

        this.keyAlias = $("select[name=privatekey]", this.fieldset);
        this.keyAlias.combobox({required: true, editable: false, width: 170,  panelHeight: "auto"});

        this.protectionRequired = $("input[name=requiresecure]:checkbox", this.fieldset);
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHosts: function(values) {
        var data = $.map(values || [], function(host) { return {text: host, value: host}; });
        this.host.combobox("loadData", data);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", parseInt(value)).numberspinner("validate");
    },

    getKeyAlias: function() {
        return !this.keyAlias.combobox("options").disabled ? this.keyAlias.combobox("getValue") : null;
    },

    setKeys: function(values) {
        var data = $.map(values || [], function(key) { return {text: key, value: key}; });
        this.keyAlias.combobox("loadData", data);
    },

    isProtectionRequired: function() {
        return this.protectionRequired.is(":checked");
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.host.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.OftpServiceController = Class.extend(
    /** @lends {jscape.domain.OftpServiceController.prototype} */
{
    DEFAULT_PORT: 3305,
    DEFAULT_SSL_PORT: 6619,

    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function (managementService, domainName) {
        this.deferred = $.Deferred();
        this._setupDialog(managementService, domainName).show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var descriptor = this._createDescriptor(dialog);
        dialog.destroy();
        this.deferred.resolve(descriptor);
        this.deferred = null;
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _createDescriptor: function (dialog) {
        return jscape.Service.oftp(dialog.getHost(),
                                   dialog.getPort(),
                                   dialog.getIdCode(),
                                   dialog.getKeyAlias(),
                                   dialog.isUseSsl(),
                                   dialog.isFileEncryptionRequired(),
                                   dialog.isFileSignatureRequired());
    },

    _setupDialog: function(managementService, domainName) {
        var dialog = new jscape.domain.OftpServiceDialog(this.resources);
        dialog.setPort(this.DEFAULT_PORT);
        dialog.setUseSsl(true);
        dialog.setFileEncryptionRequired(true);
        dialog.setFileSignatureRequired(true);

        $.when(managementService.getServerParameters(), managementService.getServerKeys())
                .done($.proxy(function (parameters, serverKeys) {
                    var hosts = parameters && parameters.getInetAddresses();
                    var keys = $.map(serverKeys || [], function (key) { return key.alias; });
                    dialog.setHosts(hosts);
                    dialog.setKeys(keys);
                }, this));

        return dialog;
    }
});

/** @class */
jscape.domain.OftpServiceDialog = Class.extend(
    /** @lends {jscape.domain.OftpServiceDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-services-oftp").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=tftpservicefields]", this.dialog);

        this.host = $("select[name=host]", this.fieldset);
        this.host.combobox({required: true, width: 200,  panelHeight: "auto", validType: "length[1,500]", missingMessage: resources.getString("SERVICES_ERROR_BAD_HOST"), invalidMessage: resources.getString("SERVICES_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 80});

        this.idCode = $("input[name=idcode]", this.fieldset);
        this.idCode.validatebox({required: true, validType: "length[1,500]", missingMessage: resources.getString("SERVICES_ERROR_BAD_ID_CODE"), invalidMessage: resources.getString("SERVICES_BAD_ID_CODE")});

        this.keyAlias = $("select[name=privatekey]", this.fieldset);
        this.keyAlias.combobox({required: true, editable: false, width: 170,  panelHeight: "auto"});

        this.useSsl = $("input[name=usessl]:checkbox", this.fieldset);
        this.fileEncryption = $("input[name=fileencryption]:checkbox", this.fieldset);
        this.fileSignature = $("input[name=filesignature]:checkbox", this.fieldset);
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHosts: function(values) {
        var data = $.map(values || [], function(host) { return {text: host, value: host}; });
        this.host.combobox("loadData", data);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value);
    },

    getIdCode: function() {
        return this.idCode.val();
    },

    setIdCode: function(value) {
        this.idCode.val(value);
    },

    getKeyAlias: function() {
        return !this.keyAlias.combobox("options").disabled ? this.keyAlias.combobox("getValue") : null;
    },

    setKeys: function(values) {
        var data = $.map(values || [], function(key) { return {text: key, value: key}; });
        this.keyAlias.combobox("loadData", data);
    },

    isUseSsl: function() {
        return this.useSsl.is(":checked");
    },

    setUseSsl: function(value) {
        this.useSsl.prop("checked", !!value);
    },

    isFileEncryptionRequired: function() {
        return this.fileEncryption.is(":checked");
    },

    setFileEncryptionRequired: function(value) {
        this.fileEncryption.prop("checked", !!value);
    },

    isFileSignatureRequired: function() {
        return this.fileSignature.is(":checked");
    },

    setFileSignatureRequired: function(value) {
        this.fileSignature.prop("checked", !!value);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.host.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.TftpServiceController = Class.extend(
    /** @lends {jscape.domain.TftpServiceController.prototype} */
{
    DEFAULT_PORT: 69,

    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function (managementService, domainName) {
        this.deferred = $.Deferred();
        this._setupDialog(managementService, domainName).show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var descriptor = this._createDescriptor(dialog);
        dialog.destroy();
        this.deferred.resolve(descriptor);
        this.deferred = null;
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _createDescriptor: function (dialog) {
        return jscape.Service.tftp(dialog.getHost(), dialog.getPort(), dialog.getAccount());
    },

    _setupDialog: function(managementService, domainName) {
        var dialog = new jscape.domain.TftpServiceDialog(this.resources);
        dialog.setPort(this.DEFAULT_PORT);

        $.when(managementService.getAccounts(domainName), managementService.getServerParameters())
                .done($.proxy(function (accounts, parameters) {
                    var hosts = parameters && parameters.getInetAddresses();
                    var users = $.map(accounts || [], function (account) { return account.login; });
                    dialog.setHosts(hosts);
                    if (users.length != 0) {
                        dialog.setAccounts(users.sort());
                    } else {
                        this._noAccountsErrorMessage();
                    }
                }, this));

        return dialog;
    },

    _noAccountsErrorMessage: function() {
        $.messager.alert(this.resources.getString("APPLICATION_ERROR_TITLE"), this.resources.getString("SERVICES_ERROR_NO_ACCOUNTS"), "error");
    }
});

/** @class */
jscape.domain.TftpServiceDialog = Class.extend(
    /** @lends {jscape.domain.TftpServiceDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-services-tftp").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=tftpservicefields]", this.dialog);

        this.host = $("select[name=host]", this.fieldset);
        this.host.combobox({required: true, width: 200,  panelHeight: "auto", validType: "length[1,500]", missingMessage: resources.getString("SERVICES_ERROR_BAD_HOST"), invalidMessage: resources.getString("SERVICES_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 80});

        this.account = $("select[name=username]", this.fieldset);
        this.account.combobox({required: true, editable: false, width: 170,  panelHeight: "auto", missingMessage: resources.getString("SERVICES_ERROR_BAD_ACCOUNT"), invalidMessage: resources.getString("SERVICES_ERROR_BAD_ACCOUNT")});
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHosts: function(values) {
        var data = $.map(values || [], function(host) { return {text: host, value: host}; });
        this.host.combobox("loadData", data);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value);
    },

    getAccount: function() {
        return this.account.combobox("getValue");
    },

    setAccounts: function(values) {
        var data = $.map(values, function(key) { return {text: key, value: key}; });
        this.account.combobox("loadData", data);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.host.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.BaseWebServiceController = Class.extend(
    /** @lends {jscape.domain.BaseWebServiceController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function() {
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var descriptor = this._createDescriptor(dialog);
        dialog.destroy();
        this.deferred.resolve(descriptor);
        this.deferred = null;
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _createDescriptor: function (dialog) {
        throw "Abstract method";
    },

    _setupDialog: function() {
        throw "Abstract method";
    }
});

/** @class */
jscape.domain.HttpServiceController = jscape.domain.BaseWebServiceController.extend(
    /** @lends {jscape.domain.HttpServiceController.prototype} */
{
    /** @override */
    _createDescriptor: function (dialog) {
        return jscape.Service.http(dialog.isPlain(), dialog.isSecure());
    },

    /** @override */
    _setupDialog: function() {
        var dialog = new jscape.domain.HttpServiceDialog(this.resources);
        dialog.setPlain(true);
        dialog.setSecure(true);
        return dialog;
    }
});

/** @class */
jscape.domain.WebDavServiceController = jscape.domain.BaseWebServiceController.extend(
    /** @lends {jscape.domain.WebDavServiceController.prototype} */
{
    /** @override */
    _createDescriptor: function (dialog) {
        return jscape.Service.webdav(dialog.isPlain(), dialog.isSecure());
    },

    /** @override */
    _setupDialog: function() {
        var dialog = new jscape.domain.WebDavServiceDialog(this.resources);
        dialog.setPlain(true);
        dialog.setSecure(true);
        return dialog;
    }
});

/** @class */
jscape.domain.As2ServiceController = jscape.domain.BaseWebServiceController.extend(
    /** @lends {jscape.domain.As2ServiceController.prototype} */
{
    /** @override */
    _createDescriptor: function (dialog) {
        return jscape.Service.as2(dialog.isPlain(), dialog.isSecure());
    },

    /** @override */
    _setupDialog: function() {
        var dialog = new jscape.domain.As2ServiceDialog(this.resources);
        dialog.setPlain(true);
        dialog.setSecure(true);
        return dialog;
    }
});

/** @class */
jscape.domain.BaseWebServiceDialog = Class.extend(
    /** @lends {jscape.domain.BaseWebServiceDialog.prototype} */
{
    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset", this.dialog).first();

        this.plain = $("input[name=plain]:checkbox", this.fieldset);
        this.secure = $("input[name=secure]:checkbox", this.fieldset);
    },

    isPlain: function() {
        return this.plain.is(":checked");
    },

    setPlain: function(value) {
        this.plain.prop("checked", !!value);
    },

    isSecure: function() {
        return this.secure.is(":checked");
    },

    setSecure: function(value) {
        this.secure.prop("checked", !!value);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.HttpServiceDialog = jscape.domain.BaseWebServiceDialog.extend(
    /** @lends {jscape.domain.HttpServiceDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-services-http"), resources);
    }
});

jscape.domain.WebDavServiceDialog = jscape.domain.BaseWebServiceDialog.extend(
    /** @lends {jscape.domain.WebDavServiceDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-services-webdav"), resources);
    }
});

jscape.domain.As2ServiceDialog = jscape.domain.BaseWebServiceDialog.extend(
    /** @lends {jscape.domain.As2ServiceDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-services-as2"), resources);
    }
});