jscape.domain = jscape.domain || {};

/** @class */
jscape.ConnectionStatistics = Class.extend(
    /** @lends {jscape.ConnectionStatistics.prototype} */
{
    /** @constructor */
    initialize: function (a, b, c, d, e, f, g, h, i) {
        this.type = a || "";
        this.creationTime = parseInt(b) || 0;
        this.upTimeMillis = parseInt(c) || 0;
        this.localAddress = d || "";
        this.localPort = parseInt(e) || 0;
        this.remoteAddress = f || "";
        this.remotePort = parseInt(g) || 0;
        this.username = h || "";
        this.path = i || "";
    }
});
jscape.ConnectionStatistics.fromJSON = function(data) {
    return $.extend(new jscape.ConnectionStatistics(), data);
};

/** @class */
jscape.domain.StatisticsModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.StatisticsModule.prototype} */
{
    /** @const */
    DELAY: 5 * 1000,

    timerId: null,

    onCreate: function() {
        this._super.apply(this, arguments);

        this.statisticsController = new jscape.domain.StatisticsController(this.resources);
        this.sessionsController = new jscape.domain.SessionsController(this.resources);
    },

    onStart: function () {
        this.statisticsController.start(this.managementService, this.domainName);
        this.sessionsController.start(this.managementService, this.domainName);
    },

    onPause: function() {
        this._stopPoll();
    },

    onResume: function() {
        this._poll();
    },

    _poll: function (delay) {
        this._stopPoll();
        this.timerId = setTimeout($.proxy(function () {
            this.timerId = null;
            this._loadDomainStatistics()
                    .done($.proxy(this._poll, this, this.DELAY))
                    .fail($.proxy(this._stopPoll, this));
        }, this), delay || 0);
    },

    _stopPoll: function () {
        if (this.timerId) {
            clearInterval(this.timerId);
        }
    },

    _loadDomainStatistics: function () {
        return this.managementService.getDomainStatistics(this.domainName)
                .done($.proxy(function (statistics) {
                    this.statisticsController.setStatistics(statistics);
                    this.sessionsController.setSessions(statistics.connectionStatistics);
                }, this));
    }
});

/** @class */
jscape.domain.StatisticsController = Class.extend(
    /** @lends {jscape.domain.StatisticsController.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.DomainStatisticsView(resources, container);
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    setStatistics: function(stats) {
        this.view.setStatistics(stats);
    }
});

/** @class */
jscape.domain.DomainStatisticsView = Class.extend(
    /** @lends {jscape.domain.DomainStatisticsView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.resources = resources;
        this.dateFormat = new jscape.DateFormat();

        this.fieldset = $("fieldset[name=domainstatisticsfields]", container);

        this.status = $("dd[aria-label=status]", this.fieldset);
        this.startDate = $("dd[aria-label=startdate]", this.fieldset);
        this.stopDate = $("dd[aria-label=stopdate]", this.fieldset);
        this.connections = $("dd[aria-label=connections]", this.fieldset);
        this.transfers = $("dd[aria-label=transfers]", this.fieldset);
        this.totalConnections = $("dd[aria-label=totalconnections]", this.fieldset);
        this.sessionUploaded = $("dd[aria-label=sessionuploaded]", this.fieldset);
        this.sessionDownloaded = $("dd[aria-label=sessiondownloaded]", this.fieldset);
        this.uploadsQuota = $("dd[aria-label=uploadsquota]", this.fieldset);
        this.downloadsQuota = $("dd[aria-label=downloadsquota]", this.fieldset);
        this.transfersQuota = $("dd[aria-label=transfersquota]", this.fieldset);
    },

    setStatistics: function(stats) {
        this.status.text(this._formatState(stats));
        this.startDate.text(this._formatDate(stats.startDate));
        this.stopDate.text(this._formatDate(stats.stopDate));
        this.connections.text(stats.currentSessions);
        this.transfers.text(stats.currentTransfers);
        this.totalConnections.text(stats.totalSessions);
        this.sessionUploaded.text(this._formatTransfers(stats.bytesUploaded, stats.filesUploaded));
        this.sessionDownloaded.text(this._formatTransfers(stats.bytesDownloaded, stats.filesDownloaded));
        this.uploadsQuota.text(this._formatQuota(stats.uploadsQuota, stats.currentUploadedBytes));
        this.downloadsQuota.text(this._formatQuota(stats.downloadsQuota, stats.currentDownloadedBytes));
        this.transfersQuota.text(this._formatQuota(stats.transfersQuota, stats.currentTransferredBytes));
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.DomainStatisticsView.LISTENER;
    },

    _formatState: function(stats) {
        return this.resources.getString("DOMAIN_STATE_" + jscape.DomainState.stateFor(stats).toUpperCase());
    },

    _formatDate: function(value) {
        return value ? this.dateFormat.format(new Date(value)) : "-";
    },

    _formatTransfers: function(bytes, files) {
        return this.resources.getString("STATISTICS_TEMPLATE_TRANSFERS", {0: jscape.SizeFormat.format(bytes), 1: files});
    },

    _formatQuota: function(quota, bytes) {
        return quota
                ? this.resources.getString("STATISTICS_TEMPLATE_QUOTA", {0: jscape.SizeFormat.format(bytes), 1: jscape.SizeFormat.format(quota)})
                : "-";
    }
});
jscape.domain.DomainStatisticsView.LISTENER = {};


/** @class */
jscape.domain.SessionsController = Class.extend(
    /** @lends {jscape.domain.StatisticsController.prototype} */
{
    sessions: [],

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.SessionsView(resources, container);
        this.closeController = new jscape.domain.CloseSessionController(resources);
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    setSessions: function(sessions) {
        this.sessions = sessions || [];
        this.view.setSessions(this.sessions);
    },

    onCloseConnection: function(session) {
        this.closeController.start(this.managementService, this.domainName, session)
                .done($.proxy(this._sessionClosed, this));
    },

    _sessionClosed: function (session) {
        var index = $.inArray(session, this.sessions);
        if (index != -1) {
            this.sessions.splice(index, 1);
            this.view.setSessions(this.sessions);
            if (this.sessions.length) {
                index = Math.max(0, Math.min(index, this.sessions.length - 1));
                this.view.selectItem(this.sessions[index]);
            }
        }
    }
});

/** @class */
jscape.domain.SessionsView = Class.extend(
    /** @lends {jscape.domain.SessionsView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources, container) {
        this.selection = [];
        this.resources = resources;
        this.timeFormat = new jscape.DateFormat("{hh}:{mm}:{ss}");

        this.fieldset = $("fieldset[name=sessionsfields]", container);
        this.data = $("table[role=grid][aria-label=sessions]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            singleSelect: true,
            remoteSort: false,
            sortName: "type",
            columns: [[
                {field: "type", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 100},
                {field: "server", title: $("thead th:nth-child(2)", this.data).text(), sortable: true, width: 220, formatter: $.proxy(function(_, row) { return this._formatServerAddress(row); }, this)},
                {field: "client", title: $("thead th:nth-child(3)", this.data).text(), sortable: true, width: 220, formatter: $.proxy(function(_, row) { return this._formatClientAddress(row); }, this)},
                {field: "username", title: $("thead th:nth-child(4)", this.data).text(), sortable: true, width: 170},
                {field: "created", title: $("thead th:nth-child(5)", this.data).text(), sortable: true, width: 200, align: "right", formatter: $.proxy(function(_, row) { return this._formatCreationTime(row); }, this)},
                {field: "uptime", title: $("thead th:nth-child(6)", this.data).text(), sortable: false, width: 170, align: "right", formatter: $.proxy(function(_, row) { return this._formatUptime(row); }, this)},
                {field: "path", title: $("thead th:nth-child(7)", this.data).text(), sortable: false, width: 170}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this)
        });

        this.closeButton = $("a[role=button][name=close]", this.fieldset);
        this.closeButton.linkbutton({onClick: $.proxy(this._onClick$CloseButton, this)});
    },

    setSessions: function(values) {
        this.selection = [];
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectItem: function (value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.SessionsView.LISTENER;
        this._adjustButtonsState();
    },

    _formatCreationTime: function(stats) {
        return new Date(stats.creationTime).toLocaleString();
    },

    _formatUptime: function(stats) {
        // millis are greater than logical range, the adjacent value will be adjusted
        return this.timeFormat.format(new Date(0, 0, 1, 0, 0, stats.upTimeMillis / 1000));
    },

    _formatServerAddress: function(stats) {
        return stats.localAddress + ":" + stats.localPort;
    },

    _formatClientAddress: function(stats) {
        return stats.remoteAddress + ":" + stats.remotePort;
    },

    _onClick$CloseButton: function() {
        this.listener.onCloseConnection(this.selection[0]);
    },

    _adjustRowSelection: function () {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        this.closeButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
    }
});
/** @const */
jscape.domain.SessionsView.LISTENER = {
    onCloseConnection: $.noop
};


/** @class */
jscape.domain.CloseSessionController = Class.extend(
    /** @lends {jscape.domain.CloseSessionController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName,stats) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.stats = stats;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.CloseSessionDialog(this.resources);
        dialog.setStatistics(stats);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var scope = dialog.getScope();
        dialog.destroy();
        this.managementService.closeConnection(this.domainName, this.stats, scope)
                .done($.proxy(function() { this.deferred.resolve(this.stats); }, this))
                .fail(this.deferred.reject);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    }
});

/** @class */
jscape.domain.CloseSessionDialog = Class.extend(
    /** @lends {jscape.domain.CloseSessionDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-sessions-close").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=closesessionfields]", this.dialog);
        this.scope = $("input[name=scope]:radio", this.fieldset);
    },

    getScope: function() {
        return this.scope.filter(":checked").val();
    },

    setStatistics: function(stats) {
        this._adjustLabel(this.scope.filter("[value=CURRENT_SESSION]"), {0: stats.remoteAddress, 1: stats.remotePort, 2: stats.localAddress, 3: stats.localPort});
        this._adjustLabel(this.scope.filter("[value=ALL_USER_SESSIONS]"), {0: stats.username}).prop("disabled", stats.username.trim().length == 0);
        this._adjustLabel(this.scope.filter("[value=ALL_CLIENT_IP_SESSIONS]"), {0: stats.remoteAddress});
        this._adjustLabel(this.scope.filter("[value=ALL_SERVICE_SESSIONS]"), {0: stats.localAddress, 1: stats.localPort});

        this.scope.first().prop("checked", true);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _adjustLabel: function(radio, args) {
        var node = $(radio[0].nextSibling);
        node.replaceWith(node.text().supplant(args));
        return radio;
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});