/** @class */
jscape.domain.TriggerActionsController = jscape.view.WizardDialog.Page.extend(
    /** @lends {jscape.domain.TriggerActionsController.prototype} */
{
    items: null,
    types: null,
    functions: null,
    eventProperties: null,

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.TriggerActionsView(resources, container);
        this.addController = new jscape.domain.AddTriggerActionController(resources);
        this.editController = new jscape.domain.EditTriggerActionController(resources);
        this.deleteController = new jscape.domain.DeleteTriggerActionController(resources);
    },

    start: function(managementService, domainName, types, functions, actionContexts) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.types = types || [];
        this.functions = functions || [];
        this.eventProperties = [];
        this.items = $.map(actionContexts || [], $.proxy(function(entry) { return this._rowDataFor(entry); }, this));

        this.view.setItems(this.items);
        this.view.show(this);
    },

    setEventProperties: function(values) {
        this.eventProperties = values || [];
    },

    /** @overrde */
    pause: function() {
        this.view.hide();
    },

    /** @overrde */
    resume: function() {
        this.view.show(this);
    },

    createActionContexts: function() {
        return $.map(this.items, $.proxy(function(entry) { return this._actionContextOf(entry); }, this));
    },

    onActionUp: function (item) {
        var index = $.inArray(item, this.items);
        if (index != -1) {
            this.items.splice(index - 1, 2, this.items[index], this.items[index - 1]);
            this.view.setItems(this.items);
            this.view.selectItem(item);
        }
    },

    onActionDown: function (item) {
        var index = $.inArray(item, this.items);
        if (index != -1 && index < this.items.length - 1) {
            this.items.splice(index, 2, this.items[index + 1], this.items[index]);
            this.view.setItems(this.items);
            this.view.selectItem(item);
        }
    },

    onAddAction: function () {
        this.addController.start(this.managementService, this.domainName, this.types, this.functions, this.eventProperties)
                .then($.proxy(function(actionContext) { return this._rowDataFor(actionContext); }, this))
                .done($.proxy(this._actionAdded, this));
    },

    onEditAction: function (item) {
        var index = $.inArray(item, this.items);
        if (index != -1) {
            var actionContext = this._actionContextOf(item);
            this.editController.start(this.managementService, this.domainName, this.functions, this.eventProperties, actionContext, item.name)
                    .then($.proxy(function(actionContext) { return this._rowDataFor(actionContext); }, this))
                    .done($.proxy(this._actionEdited, this, index));
        }
    },

    onDeleteAction: function (items) {
        this.deleteController.start(items)
                .done($.proxy(this._actionDeleted, this, items));
    },

    _actionAdded: function(item) {
        this.items.push(item);
        this.view.setItems(this.items);
        this.view.selectItem(item);
    },

    _actionEdited: function(index, item) {
        this.items[index] = item;
        this.view.setItems(this.items);
        this.view.selectItem(item);
    },

    _actionDeleted: function (items) {
        var index = -1;
        while (items.length) {
            index = $.inArray(items.pop(), this.items);
            this.items.splice(index, 1);
        }
        this.view.setItems(this.items);

        if (this.items.length) {
            index = Math.max(0, Math.min(index, this.items.length - 1));
            this.view.selectItem(this.items[index]);
        }
    },

    _actionContextOf: function(rowData) {
        var context = $.extend(true, {}, rowData);
        delete context.name;
        delete context.description;
        return context;
    },

    _rowDataFor: function(actionContext) {
        for (var i = 0; i < this.types.length; i++) {
            var candidate = this.types[i];
            if (candidate.actionClass === actionContext.actionClass) {
                return $.extend(true, {}, actionContext, {name: candidate.name, description: candidate.description});
            }
        }
        return null;
    },

    _setupView: function() {
        this.view.setItems(this.item)
    }
});

/** @class */
jscape.domain.TriggerActionsView = Class.extend(
    /** @lends {jscape.domain.TriggerActionsView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources, container) {
        this.selection = [];

        this.fieldset = $("fieldset[name=triggeractionsfields]", container);

        this.data = $("table[role=grid][aria-label=triggeractions]", this.fieldset);
        this.data.datagrid({
            height: 172,
            fitColumns: true,
            remoteSort: false,
            ctrlSelect: true,
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this),
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), width: 100},
                {field: "description", title: $("thead th:nth-child(2)", this.data).text(), width: 200}
            ]]
        });

        this.moveUpButton = $("a[role=button][name=moveup]", this.fieldset);
        this.moveUpButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveUpButton, this)});

        this.moveDownButton = $("a[role=button][name=movedown]", this.fieldset);
        this.moveDownButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveDownButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    getItems: function() {
        return this.data.datagrid("getRows");
    },

    setItems: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectItem: function(value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.TriggerActionsView.LISTENER;
        this.fieldset.show();
        this.data.datagrid("resize").datagrid("fitColumns");
    },

    hide: function() {
        this.fieldset.hide();
    },

    _onClick$MoveUpButton: function() {
        this.listener.onActionUp(this.selection[0]);
    },

    _onClick$MoveDownButton: function() {
        this.listener.onActionDown(this.selection[0]);
    },

    _onClick$AddButton: function() {
        this.listener.onAddAction();
    },

    _onClick$EditButton: function() {
        this.listener.onEditAction(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteAction(this.selection);
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var index = this.selection.length == 1 ? this.data.datagrid("getRowIndex", this.selection[0]) : -1;
        var count = this.data.datagrid("getRows").length;

        this.moveUpButton.linkbutton(index > 0 && index <= count - 1 ? "enable" : "disable");
        this.moveDownButton.linkbutton(index >= 0 && index < count - 1 ? "enable" : "disable");
        this.editButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }
});
jscape.domain.TriggerActionsView.LISTENER = {
    onActionUp: $.noop,
    onActionDown: $.noop,
    onAddAction: $.noop,
    onEditAction: $.noop,
    onDeleteAction: $.noop
};

/** @class */
jscape.domain.BaseTriggerActionController = Class.extend(
    /** @lends {jscape.domain.BaseTriggerActionController.prototype} */
{
    controllers: null,
    functions: null,
    eventProperties: null,
    actionClass: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.variableController = new jscape.VariableController();
    },

    start: function(managementService, domainName, functions, eventProperties) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.controllers = [];
        this.functions = functions || [];
        this.eventProperties = eventProperties || [];
        this.deferred = $.Deferred();

        this._loadActionProperties()
                .done($.proxy(function(properties) { this._createActionDialog(properties).show(this); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this));

        return this.deferred.promise();
    },

    onAddVariable: function(input) {
        var variables = $.map(this.eventProperties, function(entry) { return {text: entry.property, description: entry.description}; });
        this.variableController.start(input, variables);
    },

    onAddFunction: function(input) {
        var items = [];
        for (var i = 0; i < this.functions.length; i++) {
            $.each(this.functions[i].templates, function (key, value) {
                items.push({value: key, text: key.replace(/(.*)(\(.*)/, "$1 $2"), description: value});
            });
        }
        this.variableController.start(input, items);
    },

    onSubmit: function(dialog) {
        var action = this._createTriggerAction(dialog);
        dialog.destroy();
        this.deferred.resolve(action);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _loadActionProperties: function() {
        return this.managementService.getTriggerAction(this.domainName, this.actionClass)
                .fail($.proxy(function() { this.deferred.reject(); }, this));
    },

    _createTriggerAction: function(dialog) {
        var context = new jscape.ActionContext(this.actionClass);
        for (var i = 0; i < this.controllers.length; i++) {
            this.controllers[i].apply(context);
        }
        return context;
    },

    _createController: function(property, container) {
        var type = (property.field.type || "").toLowerCase();
        var input = $("#" + property.id, container);
        if (type == "integer") {
            return new jscape.domain.PropertyController(new jscape.domain.NumberPropertyView(input));
        } else if (type == "boolean" || type == "logaction") {
            return new jscape.domain.PropertyController(new jscape.domain.BooleanPropertyView(input));
        } else if (type == "string" || type == "memo" || type == "password") {
            return new jscape.domain.PropertyController(new jscape.domain.StringPropertyView(input));
        } else if (type == "choice") {
            return new jscape.domain.PropertyController(new jscape.domain.ComboboxPropertyView(input));
        } else if (type == "path") {
            return new jscape.domain.PathPropertyController(input, new jscape.domain.PathPropertyController.PathProvider());
        } else if (type == "file") {
            return new jscape.domain.PathPropertyController(input, new jscape.domain.PathPropertyController.FileProvider());
        } else if (type == "directory") {
            return new jscape.domain.PathPropertyController(input, new jscape.domain.PathPropertyController.DirectoryProvider());
        } else if (type == "account") {
            return new jscape.domain.DisableUsersPropertyController(input);
        } else if (type == "hostkey") {
            return new jscape.domain.HostKeysPropertyController(input);
        } else if (type == "pgppublickey") {
            return new jscape.domain.PgpPublicKeysPropertyController(input);
        } else if (type == "pgpsecretkey") {
            return new jscape.domain.PgpSecretKeysPropertyController(input);
        } else if (type == "partner") {
            return new jscape.domain.TradingPartnersPropertyController(input, property.field.supportedProtocols);
        } else if (type == "directorymonitor") {
            return new jscape.domain.DirectoryMonitorsPropertyController(input);
        }
        throw "Unsupported field: " + type;
    },

    _createActionDialog: function() {
        throw "Not implemented";
    }
});

/** @class */
jscape.domain.AddTriggerActionController = jscape.domain.BaseTriggerActionController.extend(
    /** @lends {jscape.domain.BaseTriggerActionController.prototype} */
{
    action: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);
    },

    /** @override */
    start: function(managementService, domainName, types, functions, eventProperties) {
        this.types = types;
        return this._super(managementService, domainName, functions, eventProperties);
    },

    /** @override */
    _loadActionProperties: function () {
        var _super = this._super;
        return this._selectActionType(this.types)
                .then($.proxy(function (action) {
                    this.action = action;
                    this.actionClass = action.actionClass;
                    return _super.apply(this);
                }, this));
    },

    _selectActionType: function (types) {
        var deferred = $.Deferred();
        var dialog = new jscape.domain.TriggerActionTypeDialog(this.resources);
        dialog.setActions(types);
        dialog.show({
            onSubmit: $.proxy(function(d) { var action = d.getAction(); d.destroy(); deferred.resolve(action); }, this),
            onCancel: $.proxy(function(d) { d.destroy(); deferred.reject(); }, this)
        });
        return deferred.promise();
    },

    _createActionDialog: function (properties) {
        var dialog = new jscape.domain.AddTriggerActionDialog(this.resources);
        dialog.setActionName(this.action.name);
        var container = dialog.container();

        $.each(properties, $.proxy(function(_, property) {
            if (!!property.advanced) {
                dialog.addAdvancedField(property.html);
            } else {
                dialog.addFormField(property.html);
            }
            var controller = this._createController(property, container);
            controller.start(this.managementService, this.domainName, property);
            this.controllers.push(controller);
        }, this));

        dialog.finish();
        return dialog;
    }
});

/** @class */
jscape.domain.EditTriggerActionController = jscape.domain.BaseTriggerActionController.extend(
    /** @lends {jscape.domain.EditTriggerActionController.prototype} */
{
    actionContext: null,

    /** @override */
    start: function(managementService, domainName, functions, eventProperties, actionContext, actionName) {
        this.actionClass = actionContext.actionClass;
        this.actionContext = actionContext;
        this.actionName = actionName;
        return this._super(managementService, domainName, functions, eventProperties);
    },

    _createActionDialog: function (properties) {
        var dialog = new jscape.domain.EditTriggerActionDialog(this.resources);
        dialog.setActionName(this.actionName);
        var container = dialog.container();

        $.each(properties, $.proxy(function(_, property) {
            if (!!property.advanced) {
                dialog.addAdvancedField(property.html);
            } else {
                dialog.addFormField(property.html);
            }
            var controller = this._createController(property, container);
            controller.start(this.managementService, this.domainName, property, this.actionContext);
            this.controllers.push(controller);
        }, this));

        dialog.finish();
        return dialog;
    }
});

/** @class */
jscape.domain.TriggerActionTypeDialog = Class.extend(
    /** @lends {jscape.domain.TriggerActionTypeDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-triggers-actiontype").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=triggeractiontypefields]", this.dialog);

        this.action = $("select[name=action]", this.fieldset);
        this.action.combobox({
            required: true,
            editable: false,
            width: 310,
            textField: "name",
            valueField: "actionClass",
            formatter: function(row) { return '<span title="' + (row.description || "").escapeXml() + '">' + (row.name || "").escapeXml() + '</span>'; },
            onChange: $.proxy(this._adjustActionHelp, this)
        }).combobox("autocomplete", true);

        this.actionHelp = $("#trigger-action-help", this.fieldset);
        this.actionHelp.tooltip({position: "right", showDelay: 0});
    },

    getAction: function() {
        var data = this.action.combobox("getData");
        var value = this.action.combobox("getValue");
        for (var i = 0; i < data.length; i++) {
            var candidate = data[i];
            if (candidate.actionClass == value) {
                return candidate;
            }
        }
        return null;
    },

    setActions: function(values) {
        this.action.combobox("loadData", values);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this._adjustActionHelp();
        this.action.combobox("textbox").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _adjustActionHelp: function () {
        var action = this.getAction();
        this.actionHelp.tooltip("update", action ? action.description : "");
    }
});

/** @class */
jscape.domain.DeleteTriggerActionController = Class.extend(
    /** @lends {jscape.domain.DeleteTriggerActionController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(actions) {
        var text = $.map(actions, function(action) { return action.name; }).join(", ");
        return jscape.ConfirmDialog.confirm(this.resources.getString("TRIGGERS_CONFIRM_DELETE_ACTION_TITLE"),
                                            this.resources.getString("TRIGGERS_CONFIRM_DELETE_ACTION_MESSAGE", {0: text}));
    }
});

/** @class */
jscape.domain.TriggerActionDialog = Class.extend(
    /** @lends {jscape.domain.TriggerActionDialog.prototype} */
{
    currentInput: null,

    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", $.now().toString(36));
        this.dialog.dialog({
            width: 680, height: "auto",
            modal: true, resizable: true, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.title = this.dialog.dialog("header").text() || "";

        this.fieldset = $("fieldset[name=triggeractionfields]", this.dialog);
        this.tabs = $("div[role=tablist]", this.fieldset);
        this.tabs.tabs({plain: true, width: 590, height: 350});

        this.parametersFieldset = $("fieldset[name=parametersfields]", this.fieldset);
        this.advancedFieldset = $("fieldset[name=advancedparametersfields]", this.fieldset);

        this.variablesButton = $("a[role=button][name=addvariable]", this.fieldset);
        this.variablesButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$VariablesButton, this)});

        this.functionsButtons = $("a[role=button][name=addfunction]", this.fieldset);
        this.functionsButtons.linkbutton({disabled: true, onClick: $.proxy(this._onClick$FunctionsButtons, this)});
    },

    setActionName: function(value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value || ""}));
    },

    addFormField: function(html, advanced) {
        this.parametersFieldset.append(html);
    },

    addAdvancedField: function(html) {
        this.advancedFieldset.append(html);
    },

    finish: function() {
        this.parametersFieldset.focusin($.proxy(this._onForm$FocusIn, this));
        this.advancedFieldset.focusin($.proxy(this._onForm$FocusIn, this));
        this.tabs.tabs({onSelect: function() { $(this).tabs("getSelected").find("fieldset").trigger("focusin"); }});;
        this.currentInput = null;

        this.dialog.dialog("resize");
        this.fieldset.form("validate");
        this.fieldset.find("input:not(:disabled):first").focus();
    },

    container: function() {
        return this.fieldset;
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
        this.currentInput = null;
    },

    destroy: function() {
        this.currentInput = null;
        this.dialog.dialog("destroy");
    },

    _onClick$VariablesButton: function() {
        if (this.currentInput && this.currentInput.is(":not(:disabled)")) {
            this.listener.onAddVariable(this.currentInput);
        }
    },

    _onClick$FunctionsButtons: function() {
        if (this.currentInput && this.currentInput.is(":not(:disabled)")) {
            this.listener.onAddFunction(this.currentInput);
        }
    },

    _onForm$FocusIn: function (e) {
        var field = $(e.target);
        if (field.is("input[type=text]:visible:not([readonly])") || field.is("textarea:visible")) {
            this.variablesButton.linkbutton("enable");
            this.functionsButtons.linkbutton("enable");
            this.currentInput = field;
        } else {
            this.variablesButton.linkbutton("disable");
            this.functionsButtons.linkbutton("disable");
            this.currentInput = null;
        }
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.TriggerActionDialog.LISTENER = {
    onAddVariable: $.noop,
    onAddFunction: $.noop
};

/** @class */
jscape.domain.AddTriggerActionDialog = jscape.domain.TriggerActionDialog.extend(
    /** @lends {jscape.domain.AddTriggerActionDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-triggers-addaction"), resources);
    }
});

/** @class */
jscape.domain.EditTriggerActionDialog = jscape.domain.TriggerActionDialog.extend(
    /** @lends {jscape.domain.EditTriggerActionDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-triggers-editaction"), resources);
    }
});

/** @class */
jscape.domain.PropertyController = Class.extend(
    /** @lends {jscape.domain.PropertyController.prototype} */
{
    property: null,

    /** @constructor */
    initialize: function(view) {
        this.view = view;
    },

    start: function(managementService, domainName, property, context) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.property = property;

        this._setupView(context);
        this.view.show(this);
    },

    apply: function(context) {
        if (this.view.valid()) {
            context.setParameter(this.property.id, this.view.val());
        }
    },

    _setupView: function (context) {
        if (context) {
            if (context.hasParameter(this.property.id)) {
                this.view.val(context.getParameter(this.property.id));
            }
        }
    }
});

/** @class */
jscape.domain.PathPropertyController = jscape.domain.PropertyController.extend(
    /** @lends {jscape.domain.PathPropertyController.prototype} */
{
    provider: null,

    /** @constructor */
    initialize: function(input, provider) {
        this._super(new jscape.domain.PathPropertyView(input));
        this.provider = provider || new jscape.domain.PathPropertyController.PathProvider();
    },

    onBrowse: function(view) {
        this.provider.browse(this.managementService)
                .done(function(value) { view.val(value); });
    }
});

/** @class */
jscape.domain.PathPropertyController.FileProvider = Class.extend(
    /** @lends {jscape.domain.PathPropertyController.FileProvider.prototype} */
{
    browse: function(managementService) {
        var deferred = $.Deferred();
        var loadFn = $.proxy(managementService.listFileSystem, managementService);
        var createFn = $.proxy(managementService.createFileSystemDir, managementService);
        new SelectPathController(loadFn, createFn).selectFile(function(entry) { deferred.resolve(entry.path); });
        return deferred.promise();
    }
});

/** @class */
jscape.domain.PathPropertyController.DirectoryProvider = Class.extend(
    /** @lends {jscape.domain.PathPropertyController.DirectoryProvider.prototype} */
{
    browse: function(managementService) {
        var deferred = $.Deferred();
        var loadFn = $.proxy(managementService.listFileSystem, managementService);
        var createFn = $.proxy(managementService.createFileSystemDir, managementService);
        new SelectPathController(loadFn, createFn).selectDirectory(function(entry) { deferred.resolve(entry.path); });
        return deferred.promise();
    }
});

/** @class */
jscape.domain.PathPropertyController.PathProvider = Class.extend(
    /** @lends {jscape.domain.PathPropertyController.PathProvider.prototype} */
{
    browse: function(managementService) {
        var deferred = $.Deferred();
        var loadFn = $.proxy(managementService.listFileSystem, managementService);
        var createFn = $.proxy(managementService.createFileSystemDir, managementService);
        new SelectPathController(loadFn, createFn).selectPath(function(entry) { deferred.resolve(entry.path); });
        return deferred.promise();
    }
});

/** @class */
jscape.domain.HostKeysPropertyController = jscape.domain.PropertyController.extend(
    /** @lends {jscape.domain.HostKeysPropertyController.prototype} */
{
    /** @constructor */
    initialize: function(input) {
        this._super(new jscape.domain.OptionalComboboxPropertyView(input));
    },

    /** @override */
    _setupView: function(context) {
        var _super = this._super;
        $.when(this.managementService.getHostKeys(),
               this.managementService.getHostCertificates())
                .done($.proxy(function(keys, certificates) {
                    var keyAliases = $.map(keys || [], function(key) { return key.alias; });
                    var certAliases = $.map(certificates || [], function(certificate) { return certificate.alias; });

                    this.view.variants(keyAliases.concat(certAliases));
                    _super.call(this, context);
                }, this));
    }
});

/** @class */
jscape.domain.PgpPublicKeysPropertyController = jscape.domain.PropertyController.extend(
    /** @lends {jscape.domain.PgpPublicKeysPropertyController.prototype} */
{
    SECRET_KEY_PREFIX: "secret: ",
    PUBLIC_KEY_PREFIX: "public: ",

    /** @constructor */
    initialize: function(input) {
        this._super(new jscape.domain.OptionalComboboxPropertyView(input));
    },

    /** @override */
    _setupView: function(context) {
        var _super = this._super;
        $.when(this.managementService.getPgpPublicKeys(),
               this.managementService.getPgpSecretKeys())
                .done($.proxy(function(publicKeys, secretKeys) {
                    var pubKeyAliases = $.map(publicKeys || [], $.proxy(function(key) { return this.PUBLIC_KEY_PREFIX + key.alias; }, this));
                    var secKeyAliases = $.map(secretKeys || [], $.proxy(function(key) { return this.SECRET_KEY_PREFIX + key.alias; }, this));

                    this.view.variants(pubKeyAliases.concat(secKeyAliases));
                    _super.call(this, context);
                }, this));
    }
});

/** @class */
jscape.domain.PgpSecretKeysPropertyController = jscape.domain.PropertyController.extend(
    /** @lends {jscape.domain.PgpSecretKeysPropertyController.prototype} */
{
    /** @constructor */
    initialize: function(input) {
        this._super(new jscape.domain.OptionalComboboxPropertyView(input));
    },

    /** @override */
    _setupView: function(context) {
        var _super = this._super;
        this.managementService.getPgpSecretKeys()
                .done($.proxy(function(keys) {
                    var aliases = $.map(keys || [], function(key) { return key.alias; });
                    this.view.variants(aliases);
                    _super.call(this, context);
                }, this));
    }
});

/** @class */
jscape.domain.TradingPartnersPropertyController = jscape.domain.PropertyController.extend(
    /** @lends {jscape.domain.TradingPartnersPropertyController.prototype} */
{
    supportedProtocols: null,

    /** @constructor */
    initialize: function(input, supportedProtocols) {
        this._super(new jscape.domain.ComboboxPropertyView(input, true));
        this.supportedProtocols = supportedProtocols || [];
    },

    /** @override */
    _setupView: function(context) {
        var _super = this._super;
        var protocols = this.supportedProtocols;
        this.managementService.getTradingPartners(this.domainName)
                .done($.proxy(function(partners) {
                    partners = $.grep(partners, function(partner) { return $.inArray(partner.protocol, protocols) != -1; });
                    this.view.variants($.map(partners, function(partner) { return partner.name; }));
                    _super.call(this, context);
                }, this));
    }
});

/** @class */
jscape.domain.DirectoryMonitorsPropertyController = jscape.domain.PropertyController.extend(
    /** @lends {jscape.domain.DirectoryMonitorsPropertyController.prototype} */
{
    /** @constructor */
    initialize: function(input) {
        this._super(new jscape.domain.ComboboxPropertyView(input));
    },

    /** @override */
    _setupView: function(context) {
        var _super = this._super;
        this.managementService.getDirectoryMonitors(this.domainName)
                .done($.proxy(function(monitors) {
                    var names = $.map(monitors || [], function(monitor) { return monitor.name; });
                    this.view.variants(names);
                    _super.call(this, context);
                }, this));
    }
});

/** @class */
jscape.domain.StringPropertyView = Class.extend(
    /** @lends {jscape.domain.StringPropertyView.prototype} */
{
    input: null,

    /** @constructor */
    initialize: function(input) {
        this.input = $(input);
        this.input.validatebox();
        var opts = this.input.validatebox("options");
        var maxLength = parseInt(this.input.prop("maxlength"));
        if (opts.required && !isNaN(maxLength)) {
            if (maxLength > 0) {
                opts.validType = "length[1," + maxLength + "]";
            } else {
                opts.validType = "length[1," + Number.MAX_VALUE + "]";
                opts.invalidMessage = opts.missingMessage;
            }
        }
        this.help = $("#" + this.input.attr("aria-describedby"));
        this.help.tooltip({position: "right", showDelay: 0, content: this.help.html()});
        this.help.empty();
    },

    show: function(listener) {
        this.listener = listener;
    },

    valid: function() {
        return this.input.validatebox("isValid");
    },

    val: function(value) {
        if (!arguments.length) {
            return this.input.is(":not(:disabled)") ? this.input.val() : null;
        }
        this.input.val(value).change();
        return this;
    }
});

/** @class */
jscape.domain.BooleanPropertyView = Class.extend(
    /** @lends {jscape.domain.BooleanPropertyView.prototype} */
{
    input: null,

    /** @constructor */
    initialize: function (input) {
        this.input = $(input);

        this.help = $("#" + this.input.attr("aria-describedby"));
        this.help.tooltip({position: "right", showDelay: 0, content: this.help.html()});
        this.help.empty();
    },

    show: function() {
    },

    valid: function() {
        return true;
    },

    val: function(value) {
        if (!arguments.length) {
            return this.input.is(":checked");
        }
        this.input.prop("checked", !!value).change();
        return this;
    }
});

/** @class */
jscape.domain.NumberPropertyView = Class.extend(
    /** @lends {jscape.domain.NumberPropertyView.prototype} */
{
    input: null,

    /** @constructor */
    initialize: function (input) {
        this.input = $(input);

        function _expression(value) {
            return value.length && /%\w+[()%]?/.test(value);
        }
        function _expressionChar(c) {
            return /[%\w\s(),.]/.test(c);
        }
        this.input.numberspinner({
            filter: function(e) {
                if (_expression($(this).numberspinner("getValue"))
                    && _expressionChar(String.fromCharCode(e.which))) {
                    return true;
                }
                return $.fn.numberbox.defaults.filter.call(this, e);
            },
            formatter: function(value) {
                return _expression(value) ? value : $.fn.numberbox.defaults.formatter.call(this, value);
            },
            parser: function(value) {
                return _expression(value) ? value : $.fn.numberbox.defaults.parser.call(this, value);
            }
        });

        this.help = $("#" + this.input.attr("aria-describedby"));
        this.help.tooltip({position: "right", showDelay: 0, content: this.help.html()});
        this.help.empty();
    },

    show: function() {
    },

    valid: function() {
        return this.input.numberspinner("isValid");
    },

    val: function(value) {
        if (!arguments.length) {
            return this.input.numberspinner("getValue");
        }
        this.input.numberspinner("setValue", value);
        return this;
    }
});

/** @class */
jscape.domain.ComboboxPropertyView = Class.extend(
    /** @lends {jscape.domain.ComboboxPropertyView.prototype} */
{
    input: null,

    /** @constructor */
    initialize: function (input, autocomplete) {
        this.input = $(input);
        this.input.combobox({
            onLoadSuccess: function(data) {
                var c = $(this);
                c.combobox("clear");
                if (data.length) {
                    c.combobox("select", data[0].value);
                    c.combo("options").panelHeight = "auto";
                    c.combo("resize");
                }
            }
        });
        if (autocomplete) {
            this.input.combobox("autocomplete", true);
        }

        this.help = $("#" + this.input.attr("aria-describedby"));
        this.help.tooltip({position: "right", showDelay: 0, content: this.help.html()});
        this.help.empty();
    },

    show: function() {
    },

    valid: function() {
        return this.input.combobox("isValid");
    },

    variants: function(values) {
        values = $.map(values || [], function(value) { return {text: value, value: value}});
        this.input.combobox("loadData", values);
    },

    val: function(value) {
        if (!arguments.length) {
            return this.input.combobox("getValue");
        }
        if (this.input.combobox("options").editable) {
            this.input.combobox("clear");
            this.input.combobox("setValue", value);
        } else {
            if (this.contains(value)) {
                this.input.combobox("clear");
                this.input.combobox("select", value);
            }
        }
        return this;
    },

    contains: function (value) {
        var options = this.input.combobox("options");
        var data = this.input.combobox("getData");
        for (var i = 0; i < data.length; i++) {
            if (data[i][options.valueField] === value) {
                return true;
            }
        }
        return false;
    }
});

/** @class */
jscape.domain.OptionalComboboxPropertyView = Class.extend(
    /** @lends {jscape.domain.OptionalComboboxPropertyView.prototype} */
{
    enableInput: null,
    valueInput: null,

    /** @constructor */
    initialize: function (input) {
        this.valueInput = $(input);
        this.enableInput = $("input[name=use" + this.valueInput.attr("name") + "]:checkbox", this.valueInput.parent());

        this.valueInput.combobox({
            onLoadSuccess: $.proxy(function(data) {
                this.valueInput.combobox("clear");
                if (data.length) {
                    this.valueInput.combobox("setValue", data[0].value);
                    this.valueInput.combo("options").panelHeight = "auto";
                    this.valueInput.combo("resize");
                    this.enableInput.change();
                } else {
                    this.enableInput.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        });
        this.enableInput.change($.proxy(function() {
            var enabled = this.enableInput.is(":checked");
            this.valueInput.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));

        this.help = $("#" + this.valueInput.attr("aria-describedby"));
        this.help.tooltip({position: "right", showDelay: 0, content: this.help.html()});
        this.help.empty();
    },

    show: function () {
    },

    valid: function () {
        return this.valueInput.combobox("isValid");
    },

    variants: function (values) {
        values = $.map(values || [], function(value) { return {text: value, value: value}});
        this.valueInput.combobox("loadData", values);
    },

    val: function (value) {
        if (!arguments.length) {
            return this.enableInput.is(":checked:not(:disabled)") ? this.valueInput.combobox("getValue") : null;
        }
        if (value != null) {
            this.valueInput.combobox("setValue", value);
        }
        this.enableInput.prop("checked", value != null).change();
        return this;
    }
});

/** @class */
jscape.domain.PathPropertyView = jscape.domain.StringPropertyView.extend(
    /** @lends {jscape.domain.PathPropertyView.prototype} */
{
    /** @constructor */
    initialize: function (input) {
        this._super(input);

        this.input.change(function() { $(this).validatebox("validate"); });
        this.browseButton = $("a[role=button][name=browse" + $(input).attr("name" ) + "]", $(input).parent());
        this.browseButton.linkbutton({onClick: $.proxy(this._onBrowse, this)});
    },

    _onBrowse: function() {
        if (this.listener) {
            this.listener.onBrowse(this);
        }
    }
});

/** @class */
jscape.domain.DisableUsersPropertyController = jscape.domain.PropertyController.extend(
    /** @lends {jscape.domain.DisableUsersPropertyController.prototype} */
{
    /** @constructor */
    initialize: function(input) {
        this._super(new jscape.domain.DisableUsersPropertyView(input));
    },

    /** @override */
    _setupView: function(context) {
        var _super = this._super;
        this.managementService.getAccounts(this.domainName)
                .done($.proxy(function(monitors) {
                    var users = $.map(monitors || [], function(user) { return user.login; });
                    this.view.variants(users.sort());
                    _super.call(this, context);
                }, this));
    }
});

/** @class */
jscape.domain.DisableUsersPropertyView = Class.extend(
    /** @lends {jscape.domain.DisableUsersPropertyView.prototype} */
{
    input: null,
    disableUser: null,

    /** @constructor */
    initialize: function (input) {
        this.input = $(input);
        this.input.combobox({
            onLoadSuccess: $.proxy(function(data) {
                this.input.combobox("clear");
                if (data.length) {
                    this.input.combobox("setValue", data[0].value);
                    this.input.combo("options").panelHeight = "auto";
                    this.input.combo("resize");
                } else {
                    this.disableElectedUser.prop("disabled", true);
                }
                this.disableElectedUser.prop("checked", false).change();
            }, this)
        });

        this.disableUser = $("input[name=disableuser]:radio", this.input.parents("fieldset"));
        this.disableUser.change($.proxy(function() {
            var enabled = this.disableElectedUser.is(":checked");
            this.input.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.disableElectedUser = this.disableUser.filter("[value=elected]");
        this.disableCurrentUser = this.disableUser.filter("[value=current]");

        this.help = $("#" + this.input.attr("aria-describedby"));
        this.help.tooltip({position: "right", showDelay: 0, content: this.help.html()});
        this.help.empty();
    },

    show: function() {
    },

    valid: function() {
        return this.disableElectedUser.is(":checked") ? this.input.combobox("isValid") : true;
    },

    variants: function(values) {
        values = $.map(values || [], function(value) { return {text: value, value: value}});
        this.input.combobox("loadData", values);
    },

    val: function(value) {
        if (!arguments.length) {
            return this.disableElectedUser.is(":checked") ? this.input.combobox("getValue") : null;
        }
        if (value != null) {
            this.input.combobox("setValue", value);
            this.disableElectedUser.prop("checked", this.input.combobox("getValue") == value).change();
        } else {
            this.disableCurrentUser.prop("checked", true).change();
        }
        return this;
    }
});