jscape.domain = jscape.domain || {};

/** @class */
jscape.Trigger = Class.extend(
    /** @lends {jscape.Trigger.prototype} */
{
    CURRENT_TIME_EVENT: "com.jscape.inet.mft.common.CurrentTimeEvent",

    /** @constructor */
    initialize: function(a, b, c, d, e, f, g, h, i, j) {
        this.name = a || "";
        this.description = b || "";
        this.eventClass = c || null;
        this.expression = d || "";
        this.contexts = e || [];
        this.allowErrorEvent = !!f;
        this.enabled = !!g;
        this.asynchronous = !!h;
        this.asynchronousActions = !!i;
        this.ignoredDomainState = j || null;
    },

    clone: function() {
        return $.extend(new jscape.Trigger(), this);
    },

    runnable: function() {
        return this.eventClass === this.CURRENT_TIME_EVENT;
    }
});
jscape.Trigger.fromJSON = function(data) {
    return data ? $.extend(new jscape.Trigger(), data, {contexts: $.map(data.contexts, jscape.ActionContext.fromJSON)}) : null;
};
jscape.Trigger.Status = {RUNNING: {name: "RUNNING", code: 1}, COMPLETED: {name: "COMPLETED", code: 2}, FAILED: {name: "FAILED", code: 3}, CANCELLED: {name: "CANCELLED", code: 4}, QUEUED: {name: "QUEUED", code: 5}, PAUSED: {name: "PAUSED", code: 6}};
jscape.Trigger.Status.fromCode = function(code, defaultValue) {
    code = parseInt(code);
    var status = null;
    $.each(jscape.Trigger.Status, function(_, entry) {
        if (entry.code == code) {
            status = entry;
            return false;
        }
    });
    return status ? status : defaultValue;
};

/** @class */
jscape.TriggerState = Class.extend(
    /** @lends {jscape.TriggerState.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d,e, f, g, h, i, j) {
        this.id = parseInt(a) || 0;
        this.triggerName = b || "";
        this.eventId = c || 0;
        this.eventClassName = d || "";
        this.eventParameters = e || {};
        this.status = f || jscape.Trigger.Status.QUEUED;
        this.startTime = parseInt(g) || 0;
        this.endTime = parseInt(h) || 0;
        this.runtime = parseInt(i) || 0;
        this.actionStates = j || [];
    },

    cancelable: function () {
        return this.status == jscape.Trigger.Status.QUEUED || this.status == jscape.Trigger.Status.RUNNING || this.status == jscape.Trigger.Status.PAUSED;
    },

    pausable: function () {
        return this.status == jscape.Trigger.Status.QUEUED || this.status == jscape.Trigger.Status.RUNNING;
    },

    resumable: function () {
        return this.status == jscape.Trigger.Status.PAUSED;
    }
});
jscape.TriggerState.fromJSON = function(data) {
    return data ? $.extend(new jscape.TriggerState(), data, {status: jscape.Trigger.Status.fromCode(data.status, jscape.Trigger.Status.QUEUED), actionStates: $.map(data.actionStates || [], jscape.ActionState.fromJSON)}) : null;
};

/** @class */
jscape.TriggerFunction = Class.extend(
    /** @lends {jscape.TriggerFunction.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.name = a || "";
        this.templates = b || {};
        this.description = c || "";
    }
});
jscape.TriggerFunction.fromJSON = function(data) {
    return data ? $.extend(new jscape.TriggerFunction(), data) : null;
};

/** @class */
jscape.TriggerEvent = Class.extend(
    /** @lends {jscape.TriggerEvent.prototype} */
{
    /** @constructor */
    initialize: function(a, b,c) {
        this.type = a || "";
        this.name = b || "";
        this.description = c || "";
    }
});
jscape.TriggerEvent.fromJSON = function(data) {
    return data ? $.extend(new jscape.TriggerEvent(), data) : null;
};

/** @class */
jscape.TriggerAction = Class.extend(
    /** @lends {jscape.TriggerAction.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.name = a || "";
        this.actionClass = b || "";
        this.description = c || "";
    }
});
jscape.TriggerAction.fromJSON = function(data) {
    return data ? $.extend(new jscape.TriggerAction(), data) : null;
};

/** @class */
jscape.ActionContext = Class.extend(
    /** @lends {jscape.ActionContext.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.actionClass = a || null;
        this.parametersMap = b || {};
    },

    hasParameter: function(name) {
        return this.parametersMap.hasOwnProperty(name);
    },

    getParameter: function(name) {
        if (this.hasParameter(name)) {
            var value = this.parametersMap[name];
            if (/true|false/i.test(value)) {
                return "true" === value.toLowerCase();
            } else if (!/null/.test(value)) {
                var num = Number(value);
                return "" === value.trim() || isNaN(num) ? value : num;
            }
        }
        return null;
    },

    setParameter: function(name, value) {
        this.parametersMap[name + ""] = value != null ? value + "" : null;
    }
});
jscape.ActionContext.fromJSON = function(data) {
    return data ? new jscape.ActionContext(data.actionClass, data.parametersMap) : null;
};

/** @class */
jscape.ActionState = Class.extend(
    /** @lends {jscape.ActionState.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e) {
        this.actionClass = a || null;
        this.startTime = parseInt(b) || 0;
        this.endTime = parseInt(c) || 0;
        this.runTime = parseInt(d) || 0;
        this.status = e || jscape.ActionState.Status.QUEUED;
    }
});
jscape.ActionState.fromJSON = function(data) {
    return data ? new jscape.ActionState(data.actionClass, data.startTime, data.endTime, data.runTime, jscape.ActionState.Status.fromCode(data.status, jscape.ActionState.Status.QUEUED)) : null;
};
jscape.ActionState.Status = {QUEUED: {name: "QUEUED", code: 1}, RUNNING: {name: "RUNNING", code: 2}, INTERRUPTED: {name: "INTERRUPTED", code: 3}, COMPLETED: {name: "COMPLETED", code: 4}, FAILED: {name: "FAILED", code: 5}, CANCELLED: {name: "CANCELLED", code: 6}, OMITTED: {name: "OMITTED", code: 7}};
jscape.ActionState.Status.fromCode = function(code, defaultValue) {
    var status = null;
    code = parseInt(code);
    $.each(jscape.ActionState.Status, function(_, entry) {
        if (entry.code == code) {
            status = entry;
            return false;
        }
    });
    return status ? status : defaultValue;
};

/** @class */
jscape.TriggerServiceConfiguration = Class.extend(
    /** @lends {jscape.TriggerServiceConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.maxThreadCount = parseInt(a) || null;
        this.maxEventQueueSize = parseInt(b) || null;
        this.maxTransferCount = parseInt(c) || null;
        this.globalVariables = d || {}
    },

    toJSON: function() {
        return this.maxThreadCount == null || this.maxEventQueueSize == null ? {executionParameters: null, maxTransferCount: this.maxTransferCount, globalVariables: this.globalVariables}: this;
    }
});
jscape.TriggerServiceConfiguration.fromJSON = function(data) {
    return data ? $.extend(new jscape.TriggerServiceConfiguration(), data) : null;
};

/** @class */
jscape.domain.TriggersModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.TriggersModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.triggersController = new jscape.domain.TriggersController(this.resources);
        this.recentTriggersController = new jscape.domain.RecentTriggersController(this.resources);
        this.settingsController = new jscape.domain.TriggersSettingsController(this.resources);
        this.actionsController = new jscape.domain.ActionsController(this.resources);
        this.functionsController = new jscape.domain.FunctionsController(this.resources);
    },

    onStart: function() {
        this.triggersController.start(this.managementService, this.domainName);
        this.recentTriggersController.start(this.managementService, this.domainName);
        this.settingsController.start(this.managementService, this.domainName);
        this.actionsController.start(this.managementService, this.domainName);
        this.functionsController.start(this.managementService, this.domainName);
    },

    onPause: function() {
        var triggersChanged = this.triggersController.hasChanged();
        var settingsChanged = this.settingsController.hasChanged();
        if (triggersChanged || settingsChanged) {
            this._confirmApply()
                    .done($.proxy(function () {
                        triggersChanged && this.triggersController.onApply();
                        settingsChanged && this.settingsController.onApply();
                    }, this));
        }
    },

    onResume: function() {
        this.triggersController._loadData();
        this.recentTriggersController._loadData();
        this.settingsController._loadData();
    }
});
jscape.domain.TriggersModule.displayClassNameOf = function(className) {
    return (className || "").replace(/.+\.([^\.]+)(Event|Action)$/, "$1").replace(/([a-z])([A-Z])/g, "$1 $2");
    //.replace(/([A-Z][^A-Z]){1}/g, function(c) { return " " + c; })
};

/** @class */
jscape.domain.TriggersController = Class.extend(
    /** @lends {jscape.domain.TriggersController.prototype} */
{
    triggers: null,
    copy: null,
    modified: false,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.TriggersView(resources);
        this.addController = new jscape.domain.AddTriggerController(resources);
        this.editController = new jscape.domain.EditTriggerController(resources);
        this.copyController = new jscape.domain.CopyTriggerController(resources);
        this.deleteController = new jscape.domain.DeleteTriggerController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.copy && !Object._equals(this.copy, this.triggers);
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var triggers = this._copyTriggers();
        return this.managementService.setTriggers(this.domainName, triggers)
                .done($.proxy(function() {
                    this.triggers = triggers;
                    this.copy = this._copyTriggers();
                    this.modified = false;
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.triggers = this.copy;
        this.copy = this._copyTriggers();
        this.modified = false;
        this._setupView();
    },

    onTriggerUp: function(trigger) {
        var index = $.inArray(trigger, this.triggers);
        if (index != -1) {
            this.triggers.splice(index - 1, 2, this.triggers[index], this.triggers[index - 1]);
            this.view.setTriggers(this.triggers);
            this.view.selectItem(trigger);
            this.modified = true;
        }
    },

    onTriggerDown: function(trigger) {
        var index = $.inArray(trigger, this.triggers);
        if (index != -1) {
            this.triggers.splice(index, 2, this.triggers[index + 1], this.triggers[index]);
            this.view.setTriggers(this.triggers);
            this.view.selectItem(trigger);
            this.modified = true;
        }
    },

    onAddTrigger: function() {
        this.addController.start(this.managementService, this.domainName)
                .done($.proxy(this._triggerAdded, this));
    },

    onEditTrigger: function(trigger) {
        var index = $.inArray(trigger, this.triggers);
        if (index != -1) {
            this.editController.start(this.managementService, this.domainName, trigger)
                    .done($.proxy(this._triggerEdited, this, index));
        }
    },

    onCopyTrigger: function(trigger) {
        this.copyController.start(trigger)
                .done($.proxy(this._triggerAdded, this));
    },

    onRunTrigger: function(triggers) {
        if (this.modified) {
            this.view.showStateNotAppliedMessage();
            return;
        }
        for (var i = 0; i < triggers.length; i++) {
            this.managementService.runTrigger(this.domainName, triggers[i].name);
        }
    },

    onDeleteTrigger: function(triggers) {
        this.deleteController.start(triggers)
                .done($.proxy(this._triggerDeleted, this));
    },

    _loadData: function() {
        this.managementService.getTriggers(this.domainName)
                .done($.proxy(function(triggers) {
                    this.triggers = triggers || [];
                }, this))
                .fail($.proxy(function() {
                    this.triggers = [];
                }, this))
                .always($.proxy(function() {
                    this.copy = this._copyTriggers();
                    this._setupView();
                }, this));
    },

    _triggerAdded: function(trigger) {
        this.triggers.push(trigger);
        this.view.setTriggers(this.triggers);
        this.view.selectItem(trigger);
        this.modified = true;
    },

    _triggerEdited: function(index, trigger) {
        this.triggers[index] = trigger;
        this.view.setTriggers(this.triggers);
        this.view.selectItem(trigger);
        this.modified = true;
    },

    _triggerDeleted: function (triggers) {
        var index = -1;
        while (triggers.length) {
            index = $.inArray(triggers.pop(), this.triggers);
            this.triggers.splice(index, 1);
            this.modified = true;
        }
        this.view.setTriggers(this.triggers);

        if (this.triggers.length) {
            index = Math.max(0, Math.min(index, this.triggers.length - 1));
            this.view.selectItem(this.triggers[index]);
        }
    },

    _copyTriggers: function() {
        return $.map(this.triggers, function(trigger) { return trigger.clone(); });
    },

    _setupView: function() {
        this.view.setTriggers(this.triggers);
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("trigger_name_nonempty", resources.getString("TRIGGERS_ERROR_BAD_TRIGGER_NAME"));
        v.rule("trigger_unique", $.proxy(this._triggerNotExists, this), resources.getString("TRIGGERS_ERROR_ALREADY_EXISTS"));
    },

    _triggerNotExists: function(value) {
        return $.grep(this.triggers, function(trigger) { return trigger.name === value; }).length == 0;
    }
});

/** @class */
jscape.domain.TriggersView = Class.extend(
    /** @lends {jscape.domain.TriggersView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.selection = [];

        this.fieldset = $("fieldset[name=triggersfields]");

        this.data = $("table[role=grid][aria-label=triggers]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            remoteSort: false,
            ctrlSelect: true,
            pagination: true,
            search: true,
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 80},
                {field: "eventClass", title: $("thead th:nth-child(2)", this.data).text(), sortable: true, width: 100, formatter: $.proxy(this._formatEvent, this), sorter: $.proxy(this._sortEvent, this)},
                {field: "enabled", title: $("thead th:nth-child(3)", this.data).text(), sortable: true, width: 50},
                {field: "expression", title: $("thead th:nth-child(4)", this.data).text(), sortable: true, width: 200},
                {field: "actions", title: $("thead th:nth-child(5)", this.data).text(), sortable: false, width: 100, formatter: $.proxy(function(_, rowData) { return this._formatActions(rowData); },  this)}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this)
        });

        this.moveUpButton = $("a[role=button][name=moveup]", this.fieldset);
        this.moveUpButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveUpButton, this)});

        this.moveDownButton = $("a[role=button][name=movedown]", this.fieldset);
        this.moveDownButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveDownButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.copyButton = $("a[role=button][name=copy]", this.fieldset);
        this.copyButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$CopyButton, this)});

        this.runButton = $("a[role=button][name=run]", this.fieldset);
        this.runButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$RunButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    setTriggers: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectItem: function(item) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", item));
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.TriggersView.LISTENER;
        this._adjustButtonsState();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    showStateNotAppliedMessage: function() {
        $.messager.alert(this.resources.getString("APPLICATION_ERROR_TITLE"),
                         this.resources.getString("TRIGGERS_ERROR_STATE_NOT_APPLIED"),
                         "warning",
                         $.proxy(function() { this.applyButton.focus(); }, this));
    },

    _formatEvent: function(value) {
        return jscape.domain.TriggersModule.displayClassNameOf(value);
    },

    _sortEvent: function(a, b) {
        a = jscape.domain.TriggersModule.displayClassNameOf(a);
        b = jscape.domain.TriggersModule.displayClassNameOf(b);
        return a == b ? 0 : (a > b ? 1 : -1);
    },

    _formatActions: function(trigger) {
        return $.map(trigger.contexts, function(entry) { return jscape.domain.TriggersModule.displayClassNameOf(entry.actionClass); }).join(", ");
    },

    _onClick$MoveUpButton: function() {
        this.listener.onTriggerUp(this.selection[0]);
    },

    _onClick$MoveDownButton: function() {
        this.listener.onTriggerDown(this.selection[0]);
    },

    _onClick$AddButton: function() {
        this.listener.onAddTrigger();
    },

    _onClick$EditButton: function() {
        this.listener.onEditTrigger(this.selection[0]);
    },

    _onClick$CopyButton: function() {
        this.listener.onCopyTrigger(this.selection[0]);
    },

    _onClick$RunButton: function() {
        this.listener.onRunTrigger(this.selection);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteTrigger(this.selection);
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard();
    },

    _adjustRowSelection: function () {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var index = this.selection.length == 1 ? this.data.datagrid("getRowIndex", this.selection[0]) : -1;
        var count = this.data.datagrid("getRows").length;
        var runnable = this.selection.length != 0;
        $.each(this.selection, function(_, trigger) { if (!trigger.runnable()) { runnable = false; return false; }});

        this.moveUpButton.linkbutton(index > 0 && index <= count - 1 ? "enable" : "disable");
        this.moveDownButton.linkbutton(index >= 0 && index < count - 1 ? "enable" : "disable");
        this.editButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.copyButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.runButton.linkbutton(runnable ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }
});
jscape.domain.TriggersView.LISTENER = {
    onTriggerUp: $.noop,
    onTriggerDown: $.noop,
    onAddTrigger: $.noop,
    onEditTrigger: $.noop,
    onCopyTrigger: $.noop,
    onRunTrigger: $.noop,
    onDeleteTrigger: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.BaseTriggerController = Class.extend(
    /** @lends {jscape.domain.BaseTriggerController.prototype} */
{
    events: null,
    actions: null,
    functions: null,
    eventType: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();
        this.eventType = null;

        this._loadData();

        return this.deferred.promise();
    },

    onPageSelected: function(dialog) {
        var page = dialog.getItem(dialog.getCurrentItem());
        if (page != null && page == this.expressionController) {
            var eventType = this.parametersController.getEventType();
            if (this.eventType != eventType) {
                this._loadEventProperties(eventType);
            }
        }
    },

    onSubmit: function(dialog) {
        var trigger = this._createTrigger(dialog);
        dialog.destroy();
        this.deferred.resolve(trigger);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _loadData: function() {
        if (this.events != null && this.actions != null && this.functions != null) {
            this._setupDialog().show(this);
        } else {
            $.when(this.managementService.getTriggerEvents(this.domainName),
                   this.managementService.getTriggerActions(this.domainName),
                   this.managementService.getTriggerFunctions(this.domainName))
                    .done($.proxy(function(events, actions, functions) {
                        this.events = events || [];
                        this.actions = actions || [];
                        this.functions = functions || [];
                        this._setupDialog().show(this);
                   }, this));
        }
    },

    _loadEventProperties: function (eventType) {
        return this.managementService.getTriggerEvent(this.domainName, eventType)
                .done($.proxy(function (properties) {
                    this.eventType = eventType;
                    this.expressionController.setEventProperties(eventType, properties);
                    this.actionsController.setEventProperties(properties);
                }, this));
    },

    _createTrigger: function (dialog) {
        throw "Not implemeted";
    },

    _setupDialog: function () {
        throw "Not implemeted";
    }
});

/** @class */
jscape.domain.AddTriggerController = jscape.domain.BaseTriggerController.extend(
    /** @lends {jscape.domain.AddTriggerController.prototype} */
{

    /** @override */
    _createTrigger: function (dialog) {
        var trigger = this.parametersController.createTrigger();
        trigger.expression = this.expressionController.createExpression();
        trigger.contexts = this.actionsController.createActionContexts();
        return trigger;
    },

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.AddTriggerDialog(this.resources);

        this.parametersController = new jscape.domain.AddTriggerParametersController(this.resources, dialog.container());
        dialog.addItem(this.parametersController);

        this.expressionController = new jscape.domain.TriggerExpressionController(this.resources, dialog.container());
        dialog.addItem(this.expressionController);

        this.actionsController = new jscape.domain.TriggerActionsController(this.resources, dialog.container());
        dialog.addItem(this.actionsController);

        this.parametersController.start(this.events);
        this.expressionController.start(this.managementService, this.domainName, this.functions);
        this.actionsController.start(this.managementService, this.domainName, this.actions, this.functions);

        return dialog;
    }
});

/** @class */
jscape.domain.EditTriggerController = jscape.domain.BaseTriggerController.extend(
    /** @lends {jscape.domain.EditTriggerController.prototype} */
{
    /** @override */
    start: function(managementService, domainName, trigger) {
        this.trigger = trigger;
        return this._super(managementService, domainName);
    },

    /** @override */
    _createTrigger: function (dialog) {
        var trigger = this.parametersController.createTrigger();
        trigger.expression = this.expressionController.createExpression();
        trigger.contexts = this.actionsController.createActionContexts();
        return trigger;
    },

    /** @override */
    _setupDialog: function () {
        var dialog = new jscape.domain.EditTriggerDialog(this.resources);
        dialog.setTriggerName(this.trigger.name);

        this.parametersController = new jscape.domain.EditTriggerParametersController(this.resources, dialog.container());
        dialog.addItem(this.parametersController);

        this.expressionController = new jscape.domain.TriggerExpressionController(this.resources, dialog.container());
        dialog.addItem(this.expressionController);

        this.actionsController = new jscape.domain.TriggerActionsController(this.resources, dialog.container());
        dialog.addItem(this.actionsController);

        this.parametersController.start(this.events, this.trigger);
        this.expressionController.start(this.managementService, this.domainName, this.functions, this.trigger.expression);
        this.actionsController.start(this.managementService, this.domainName, this.actions, this.functions, this.trigger.contexts);

        return dialog;
    }
});

/** @class */
jscape.domain.AddTriggerDialog = jscape.view.WizardDialog.extend(
    /** @lends {jscape.domain.AddTriggerDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-triggers-add"), resources);
    }
});

/** @class */
jscape.domain.EditTriggerDialog = jscape.view.WizardDialog.extend(
    /** @lends {jscape.domain.EditTriggerDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-triggers-edit"), resources);
    },

    setTriggerName: function(value) {
        this.triggerName = value || "";
    },

    /** @override */
    _adjustDialogTitle: function() {
        this.dialog.dialog("setTitle", this.title.supplant({0: this.triggerName, 1: this.index + 1, 2: this.items.length}));
    }
});

/** @class */
jscape.domain.TriggerParametersView = Class.extend(
    /** @lends {jscape.domain.TriggerParametersView.prototype} */
{
    TRIGGER_ERROR_EVENT: "com.jscape.inet.mft.workflow.TriggerErrorEvent",

    /** @constructor */
    initialize: function (fieldset, resources) {
        this.fieldset = fieldset.hide();

        this.event = $("select[name=eventtype]", this.fieldset);
        this.event.combobox({required: true, editable: false, width: 200, textField: "name", valueField: "type", onChange: $.proxy(this._adjustEventHelp, this), onLoadSuccess: function (data) { data.length && $(this).combobox("select", data[0].type); }});
        this.eventHelp = $("#trigger-event-help", this.fieldset);
        this.eventHelp.tooltip({position: "right", showDelay: 0});
        this.description = $("textarea[name=description]", this.fieldset).css({"width": "30em", "height": "3em"});

        this.ignoreDomainState = $("input[name=ignoredomainstate]:checkbox", this.fieldset);
        this.ignoreDomainState.change($.proxy(function () {
            var enabled = this.ignoreDomainState.is(":checked");
            this.domainState.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.domainState = $("select[name=domainstate]", this.fieldset);
        this.domainState.combobox({required: true, editable: false, panelHeight: "auto", onLoadSuccess: $.proxy(function (data) {
            if (data.length) {
                this.domainState.combobox("select", data[0].value);
                this.ignoreDomainState.change();
            } else{
                this.ignoreDomainState.prop("disabled", true).prop("checked", false).change();
            }
        }, this)});

        this.enabled = $("input[name=enabled]:checkbox", this.fieldset);
        this.triggerAsynchronous = $("input[name=triggerasync]:checkbox", this.fieldset);
        this.actionsAsynchronous = $("input[name=actionsasync]:checkbox", this.fieldset);
        this.fireError = $("input[name=fireerror]:checkbox", this.fieldset);
    },

    getEvent: function () {
        var data = this.event.combobox("getData");
        var value = this.event.combobox("getValue");
        for (var i = 0; i < data.length; i++) {
            var event = data[i];
            if (event.type == value) {
                return event;
            }
        }
        return null;
    },

    setEvent: function (value) {
        this.event.combobox("select", value);
    },

    setAvailableEvents: function (values) {
        this.event.combobox("loadData", values);
    },

    getDescription: function () {
        return this.description.val();
    },

    setDescription: function (value) {
        this.description.val(value);
    },

    getIgnoredDomainState: function () {
        return this.ignoreDomainState.is(":checked") ? this.domainState.combobox("getValue") : null;
    },

    setIgnoredDomainState: function (value) {
        this.ignoreDomainState.prop("checked", value != null).change();
        if (value != null) {
            this.domainState.combobox("select", value);
        }
    },

    isEnabled: function () {
        return this.enabled.is(":checked");
    },

    setEnabled: function (value) {
        this.enabled.prop("checked", !!value);
    },

    isTriggerAsynchronous: function() {
        return this.triggerAsynchronous.is(":checked");
    },

    setTriggerAsynchronous: function(value) {
        this.triggerAsynchronous.prop("checked", !!value);
    },

    isActionsAsynchronous: function() {
        return this.actionsAsynchronous.is(":checked");
    },

    setActionsAsynchronous: function(value) {
        this.actionsAsynchronous.prop("checked", !!value);
    },

    isFireError: function () {
        return this.fireError.is(":checked");
    },

    setFireError: function (value) {
        this.fireError.prop("checked", !!value);
    },

    show: function() {
        this.fieldset.show();
        this._adjustEventHelp();
    },

    hide: function() {
        this.fieldset.hide();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _adjustEventHelp: function () {
        var event = this.getEvent();
        if (event != null) {
            this.eventHelp.tooltip("update", event.description || "");
            this.fireError.prop("disabled", this.TRIGGER_ERROR_EVENT === event.type).prop("checked", !this.fireError.is(":disabled"));
        }
    }
});

/** @class */
jscape.domain.AddTriggerParametersController = Class.extend(
    /** @lends {jscape.domain.AddTriggerParametersController.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.AddTriggerParametersView(resources, container);
    },

    start: function(events) {
        this._setupView(events);
    },

    pause: function() {
        this.view.hide();
    },

    resume: function() {
        this.view.show(this);
    },

    valid: function() {
        return this.view.valid();
    },

    getEventType: function() {
        return this.view.getEvent().type;
    },

    createTrigger: function() {
        return new jscape.Trigger(this.view.getName(),
                                  this.view.getDescription(),
                                  this.getEventType(),
                                  null,
                                  null,
                                  this.view.isFireError(),
                                  this.view.isEnabled(),
                                  this.view.isTriggerAsynchronous(),
                                  this.view.isActionsAsynchronous(),
                                  this.view.getIgnoredDomainState());
    },

    _setupView: function(events) {
        this.view.setAvailableEvents(events);

        this.view.setEnabled(true);
        this.view.setFireError(true);
    }
});
/** @class */
jscape.domain.EditTriggerParametersController = Class.extend(
    /** @lends {jscape.domain.EditTriggerParametersController.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.EditTriggerParametersView(resources, container);
    },

    start: function(events, trigger) {
        this.trigger = trigger;
        this._setupView(events);
    },

    pause: function() {
        this.view.hide();
    },

    resume: function() {
        this.view.show(this);
    },

    valid: function() {
        return this.view.valid();
    },

    getEventType: function() {
        return this.view.getEvent().type;
    },

    createTrigger: function() {
        return new jscape.Trigger(this.trigger.name,
                                  this.view.getDescription(),
                                  this.getEventType(),
                                  this.trigger.expression,
                                  this.trigger.contexts,
                                  this.view.isFireError(),
                                  this.view.isEnabled(),
                                  this.view.isTriggerAsynchronous(),
                                  this.view.isActionsAsynchronous(),
                                  this.view.getIgnoredDomainState());
    },

    _setupView: function(events) {
        this.view.setAvailableEvents(events);

        this.view.setDescription(this.trigger.description);
        this.view.setEvent(this.trigger.eventClass);
        this.view.setEnabled(this.trigger.enabled);
        this.view.setTriggerAsynchronous(this.trigger.asynchronous);
        this.view.setActionsAsynchronous(this.trigger.asynchronousActions);
        this.view.setFireError(this.trigger.allowErrorEvent);
        this.view.setIgnoredDomainState(this.trigger.ignoredDomainState);
    }
});

/** @class */
jscape.domain.AddTriggerParametersView = jscape.domain.TriggerParametersView.extend(
    /** @lends {jscape.domain.AddTriggerParametersView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=addtriggerparametersfields]", container), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["trigger_unique", "trigger_name_nonempty"], missingMessage: resources.getString("TRIGGERS_ERROR_BAD_TRIGGER_NAME")});
    },

    getName: function () {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditTriggerParametersView = jscape.domain.TriggerParametersView.extend(
    /** @lends {jscape.domain.EditTriggerParametersView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=edittriggerparametersfields]", container), resources);
    }
});

/** @class */
jscape.domain.TriggerExpressionController = jscape.view.WizardDialog.Page.extend(
    /** @lends {jscape.domain.TriggerExpressionController.prototype} */
{
    OPERATOR: {
        LOGICAL: {AND: "AND", OR: "OR"},
        COMPARISON: {EQUALS: "=", NOT_EQUAL: "!=", GREATER: ">", GREATER_OR_EQUAL: ">=", LESS: "<", LESS_OR_EQUAL: "<="},
        CONDITIONAL: {CONTAINS: "#", NOT_CONTAIN: "!#", MATCHES: "~", NOT_MATCH: "!~"}
    },

    eventType: null,
    eventProperties: null,

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.TriggerExpressionView(resources, container);
        this.autoCompleteController = new jscape.AutoCompleteController();
        this.constantsController = new jscape.domain.ConstantsController(resources);
        this.timeExpressionController = new jscape.domain.TimeExpressionController(resources);
        this.testController = new jscape.domain.ExpressionTestController(resources);
    },

    start: function(managementService, domainName, functions, expression) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.eventProperties = [];
        this.functions = functions || [];

        this.view.setExpression(expression || "");
        this.view.show(this);
    },

    setEventProperties: function(eventType, eventProperties) {
        this.eventType = eventType;
        this.eventProperties = eventProperties || [];
    },

    /** @override */
    pause: function() {
        this.view.hide();
    },

    /** @override */
    resume: function() {
        this.view.show(this);
    },

    /** @override */
    valid: function() {
        var response = this.managementService.validateTriggerExpression(this.domainName, this.view.getExpression(), this.eventType);
        return response && response.trim().toLowerCase() === "true";
    },

    onInsertVariable: function(input) {
        var variables = $.map(this.eventProperties, function(entry) {
            return {value: entry.property, text: entry.property, description: entry.description};
        });
        this.autoCompleteController.start(input, variables);
    },

    onInsertFunction: function (input) {
        var items = [];
        for (var i = 0; i < this.functions.length; i++) {
            $.each(this.functions[i].templates, function(key, value) {
                items.push({value: key, text: key.replace(/(.*)(\(.*)/, "$1 $2"), description: value});
            });
        }
        this.autoCompleteController.start(input, items);
    },

    onInsertConstant: function(input) {
        this.constantsController.start().done($.proxy(function(value) {
            this.view.insertText(value);
        }, this))
    },

    onInsertOperator: function(input) {
        var operators = $.map(this.OPERATOR.COMPARISON, function(value) { return value; })
                .concat($.map(this.OPERATOR.CONDITIONAL, function(value) { return value; }));
        this.autoCompleteController.start(input, operators);
    },

    onInsertAndOperator: function() {
        this.view.insertText(this.OPERATOR.LOGICAL.AND + " ");
    },

    onInsertOrOperator: function() {
        this.view.insertText(this.OPERATOR.LOGICAL.OR + " ");
    },

    onInsertTimeExpression: function() {
        this.timeExpressionController.start(this.eventProperties)
                .done($.proxy(function(expression) { this.view.insertText(expression); }, this));
    },

    onTestExpression: function() {
        if (this.valid()) {
            this.testController.start(this.managementService, this.domainName, this.eventProperties, this.createExpression());
        }
    },

    createExpression: function() {
        return this.view.getExpression();
    }
});

/** @class */
jscape.domain.TriggerExpressionView = Class.extend(
    /** @lends {jscape.domain.TriggerExpressionView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=triggerexpressionfields]", container);

        this.expression = $("textarea[name=expression]", this.fieldset).css({width: "45em", height: "8em"});
        this.expression.on("input change", $.proxy(function() {
            var enabled = this.expression.val().trim().length != 0;
            this.testButton.linkbutton(enabled ? "enable" : "disable");
        }, this));
        this.expressionHelp = $("#trigger-expression-help", this.fieldset);
        var help = this.expressionHelp.html();
        this.expressionHelp.html("").tooltip({position: "right", showDelay: 0, content: help});

        this.variablesButton = $("a[role=button][name=variables]", this.fieldset);
        this.variablesButton.linkbutton({onClick: $.proxy(this._onClick$VariablesButton, this)});

        this.functionsButtons = $("a[role=button][name=functions]", this.fieldset);
        this.functionsButtons.linkbutton({onClick: $.proxy(this._onClick$FunctionsButtons, this)});

        this.constantsButton = $("a[role=button][name=constants]", this.fieldset);
        this.constantsButton.linkbutton({onClick: $.proxy(this._onClick$ConstantsButton, this)});

        this.timeExpressionButton = $("a[role=button][name=timeexpression]", this.fieldset);
        this.timeExpressionButton.linkbutton({onClick: $.proxy(this._onClick$TimeExpressionButton, this)});

        this.operatorsButton = $("a[role=button][name=operators]", this.fieldset);
        this.operatorsButton.linkbutton({onClick: $.proxy(this._onClick$OperatorsButton, this)});

        this.operatorAndButton = $("a[role=button][name=operatorand]", this.fieldset);
        this.operatorAndButton.linkbutton({onClick: $.proxy(this._onClick$OperatorAndButton, this)});

        this.operatorOrButton = $("a[role=button][name=operatoror]", this.fieldset);
        this.operatorOrButton.linkbutton({onClick: $.proxy(this._onClick$OperatorOrButton, this)});

        this.testButton = $("a[role=button][name=testexpression]", this.fieldset);
        this.testButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getExpression: function() {
        return this.expression.val().trim();
    },

    setExpression: function(value) {
        this.expression.val(value).change();
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.TriggerExpressionView.LISTENER;
        this.fieldset.show();
        this.expression.focus();
    },

    hide: function() {
        this.fieldset.hide();
    },

    insertText: function(value) {
        var input = this.expression[0];
        var text = this.expression.val();
        text = text.substring(0, input.selectionStart) + value + text.substring(input.selectionEnd);
        this.expression.val(text);
        input.selectionStart = input.selectionEnd = input.selectionStart + value.length;
        return this.expression.change().focus();
    },

    _onClick$VariablesButton: function() {
        this.listener.onInsertVariable(this.expression);
    },

    _onClick$FunctionsButtons: function() {
        this.listener.onInsertFunction(this.expression);
    },

    _onClick$ConstantsButton: function() {
        this.listener.onInsertConstant(this.expression);
    },

    _onClick$OperatorsButton: function() {
        this.listener.onInsertOperator(this.expression);
    },

    _onClick$OperatorAndButton: function() {
        this.listener.onInsertAndOperator();
    },

    _onClick$OperatorOrButton: function() {
        this.listener.onInsertOrOperator();
    },

    _onClick$TimeExpressionButton: function() {
        this.listener.onInsertTimeExpression();
    },

    _onClick$TestButton: function() {
        this.listener.onTestExpression();
    }
});
jscape.domain.TriggerExpressionView.LISTENER = {
    onInsertVariable: $.noop,
    onInsertFunction: $.noop,
    onInsertConstant: $.noop,
    onInsertOperator: $.noop,
    onInsertAndOperator: $.noop,
    onInsertOrOperator: $.noop,
    onInsertTimeExpression: $.noop,
    onTestExpression: $.noop
};

/** @class */
jscape.domain.TimeExpressionController = Class.extend(
    /** @lends {jscape.domain.TimeExpressionController.prototype} */
{
    FIELD: {HOUR: "Hour", MINUTE: "Minute", DAY_OF_WEEK: "DayOfWeek", DAY_OF_MONTH: "DayOfMonth", DAY_OF_YEAR: "DayOfYear", WEEK_OF_MONTH: "WeekOfMonth", WEEK_OF_YEAR: "WeekOfYear", MONTH: "Month", YEAR: "Year"},
    VALUE_SEPARATOR_REGEXP: /\s*,\s*/,

    deferred: null,
    eventProperties: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this._initValidators(resources);
    },

    start: function(eventProperties) {
        this.eventProperties = eventProperties || {};
        this.deferred = $.Deferred();

        this._setupDialog().show(this);

        return this.deferred.promise();
    },

    onHelpRequested: function(name) {
        var help = null;
        $.each(this.eventProperties, function(_, entry) {
            if ((entry.property || "").toLowerCase() == name.toLowerCase()) {
                help = entry.description;
                return false;
            }
            return true;
        });
        return help;
    },

    onSubmit: function(dialog) {
        var expression = this._createExpression(dialog);
        dialog.destroy();
        this.deferred.resolve(expression);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _setupDialog: function() {
        return new jscape.domain.TimeExpressionDialog(this.resources);
    },

    _createExpression: function(dialog) {
        return $.map(
                $.grep([
                           this._fieldExpression(this.FIELD.HOUR, dialog.getHour()),
                           this._fieldExpression(this.FIELD.MINUTE, dialog.getMinute()),
                           this._fieldExpression(this.FIELD.DAY_OF_WEEK, dialog.getDayOfWeek()),
                           this._fieldExpression(this.FIELD.DAY_OF_MONTH, dialog.getDayOfMonth()),
                           this._fieldExpression(this.FIELD.DAY_OF_YEAR, dialog.getDayOfYear()),
                           this._fieldExpression(this.FIELD.WEEK_OF_MONTH, dialog.getWeekOfMonth()),
                           this._fieldExpression(this.FIELD.WEEK_OF_YEAR, dialog.getWeekOfYear()),
                           this._fieldExpression(this.FIELD.MONTH, dialog.getMonth()),
                           this._fieldExpression(this.FIELD.YEAR, dialog.getYear())
                       ],
                       function (entry) { return entry != null; }),
                function (expression) { return "(" + expression + ")"}
        ).join(" AND ");
    },

    _fieldExpression: function(field, value) {
        var values = value.trim().split(this.VALUE_SEPARATOR_REGEXP);
        return value.trim().length != 0 ? $.map(values, function(value) { return field + " = " + parseInt(value, 10); }).join(" OR ") : null;
    },

    _initValidators: function(value) {
        var v = new jscape.Validator();
        v.rule("timeexpression_comma_separated_numbers", $.proxy(this._commaSeparatedNumbers, this));
    },

    _commaSeparatedNumbers: function(value) {
        var values = value.trim().split(this.VALUE_SEPARATOR_REGEXP);
        return $.grep(values, function(value) { return /[^\d]+/.test(value) || isNaN(parseInt(value, 10)); }).length == 0;
    }
});

/** @class */
jscape.domain.TimeExpressionDialog = Class.extend(
    /** @lends {jscape.domain.TimeExpressionDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-triggers-timexpression").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=timeexpressionfields]", this.dialog);

        this.hour = $("input[name=hour]", this.fieldset);
        this.hour.validatebox({required: false, validType: "timeexpression_comma_separated_numbers", invalidMessage: resources.getString("TIME_EXPRESSION_ERROR_BAD_HOURS")});

        this.minute = $("input[name=minute]", this.fieldset);
        this.minute.validatebox({required: false, validType: "timeexpression_comma_separated_numbers", invalidMessage: resources.getString("TIME_EXPRESSION_ERROR_BAD_MINUTES")});

        this.dayOfWeek = $("input[name=dayofweek]", this.fieldset);
        this.dayOfWeek.validatebox({required: false, validType: "timeexpression_comma_separated_numbers", invalidMessage: resources.getString("TIME_EXPRESSION_ERROR_BAD_DAYSOFWEEK")});

        this.dayOfMonth = $("input[name=dayofmonth]", this.fieldset);
        this.dayOfMonth.validatebox({required: false, validType: "timeexpression_comma_separated_numbers", invalidMessage: resources.getString("TIME_EXPRESSION_ERROR_BAD_DAYSOFMONTH")});

        this.dayOfYear = $("input[name=dayofyear]", this.fieldset);
        this.dayOfYear.validatebox({required: false, validType: "timeexpression_comma_separated_numbers", invalidMessage: resources.getString("TIME_EXPRESSION_ERROR_BAD_DAYSOFYEAR")});

        this.weekOfMonth = $("input[name=weekofmonth]", this.fieldset);
        this.weekOfMonth.validatebox({required: false, validType: "timeexpression_comma_separated_numbers", invalidMessage: resources.getString("TIME_EXPRESSION_ERROR_BAD_WEEKSOFMONTH")});

        this.weekOfYear = $("input[name=weekofyear]", this.fieldset);
        this.weekOfYear.validatebox({required: false, validType: "timeexpression_comma_separated_numbers", invalidMessage: resources.getString("TIME_EXPRESSION_ERROR_BAD_WEEKSOFYEAR")});

        this.month = $("input[name=month]", this.fieldset);
        this.month.validatebox({required: false, validType: "timeexpression_comma_separated_numbers", invalidMessage: resources.getString("TIME_EXPRESSION_ERROR_BAD_MONTHS")});

        this.year = $("input[name=year]", this.fieldset);
        this.year.validatebox({required: false, validType: "timeexpression_comma_separated_numbers", invalidMessage: resources.getString("TIME_EXPRESSION_ERROR_BAD_YEARS")});

        this.helps = this.fieldset.find("span.help");
        this.helps.tooltip({position: "right", showDelay: 0});
        this.helps.mouseenter($.proxy(this._adjustPropertyHelp, this));
    },

    getHour: function() {
        return this.hour.val();
    },

    getMinute: function() {
        return this.minute.val();
    },

    getDayOfWeek: function() {
        return this.dayOfWeek.val();
    },

    getDayOfMonth: function() {
        return this.dayOfMonth.val();
    },

    getDayOfYear: function() {
        return this.dayOfYear.val();
    },

    getWeekOfMonth: function() {
        return this.weekOfMonth.val();
    },

    getWeekOfYear: function() {
        return this.weekOfYear.val();
    },

    getMonth: function() {
        return this.month.val();
    },

    getYear: function() {
        return this.year.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _adjustPropertyHelp: function(e) {
        var target = $(e.target);
        if (!target.data("updated")) {
            var name = this.fieldset.find("[aria-describedby=" + target.attr("id") + "]:input").attr("name");
            if (name) {
                var help = this.listener.onHelpRequested(name);
                target.tooltip("update", help || "");
            }
            target.data("updated", true);
        }
    }
});
jscape.domain.TimeExpressionDialog.LISTENER = {
    onHelpRequested: $.noop
};

/** @class */
jscape.domain.ConstantsController = Class.extend(
    /** @lends {jscape.domain.ConstantsController.prototype} */
{
    /** @const */
    BOOLEAN: {TRUE: "TRUE", FALSE: "FALSE"},

    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function() {
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var value = this._createConstant(dialog);
        dialog.destroy();
        this.deferred.resolve(value);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createConstant: function(dialog) {
        if (dialog.isString()) {
            return '"' + dialog.getString() + '"';
        } else if (dialog.isBoolean()) {
            return dialog.getBoolean() ? this.BOOLEAN.TRUE : this.BOOLEAN.FALSE;
        } else if (dialog.isNumber()) {
            return dialog.getNumber();
        }
        return "";
    },

    _setupDialog: function() {
        return new jscape.domain.ConstantsDialog(this.resources);
    }
});

/** @class */
jscape.domain.ConstantsDialog = Class.extend(
    /** @lends {jscape.domain.ConstantsDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-triggers-constant").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=triggerexpressionconstantsfields]", this.dialog);

        this.type = $("input[name=type]", this.fieldset);
        this.type.change($.proxy(this._adjustFieldState, this));
        this.stringValue = $("input[name=string]", this.fieldset);
        this.numberValue = $("input[name=number]", this.fieldset).numberspinner({value: 0});
        this.booleanValue = $("input[name=boolean]:radio", this.fieldset);
    },

    isString: function() {
        return this.type.filter("[value=string]").is(":checked");
    },

    getString: function() {
        return this.stringValue.val();
    },

    isNumber: function() {
        return this.type.filter("[value=number]").is(":checked");
    },

    getNumber: function() {
        return parseInt(this.numberValue.numberspinner("getValue"));
    },

    isBoolean: function() {
        return this.type.filter("[value=boolean]").is(":checked");
    },

    getBoolean: function() {
        return this.booleanValue.filter("[value=true]").is(":checked");
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.type.first().prop("checked", true).change();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _adjustFieldState: function() {
        this.stringValue.prop("disabled", !this.isString());
        this.numberValue.numberspinner(this.isNumber() ? "enable" : "disable");
        this.booleanValue.prop("disabled", !this.isBoolean());

        if (this.isString()) {
            this.stringValue.focus();
        } else if (this.isNumber()) {
            this.numberValue.focus();
        } else if (this.isBoolean()) {
            this.booleanValue.filter(":checked").focus();
        }
    }
});

/** @class */
jscape.domain.ExpressionTestController = Class.extend(
    /** @lends {jscape.domain.ExpressionTestController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, eventProperties, expression) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.eventProperties = eventProperties;
        this.expression = expression;
        this.deferred = $.Deferred();

        this._setupDialog().show(this);

        return this.deferred.promise();
    },

    onTest: function(dialog) {
        var parameters = this._createParameters(dialog);
        this.managementService.testTriggerExpression(this.domainName, this.expression, parameters)
                .done($.proxy(function(response) {
                    this._onComplete((response || "").toLowerCase() === "true");
                }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
    },

    _setupDialog: function() {
        var dialog = new jscape.domain.ExpressionTestDialog(this.resources);
        dialog.setExpression(this.expression);
        dialog.setProperties(this.eventProperties);
        return dialog;
    },

    _createParameters: function(dialog) {
        var parameters = {};
        $.each(this.eventProperties, function(_, entry) {
            var name = entry.property;
            parameters[name] = dialog.getParameter(name);
        });
        return parameters;
    },

    _onComplete: function(success) {
        if (success) {
            $.messager.alert(this.resources.getString("TRIGGERS_TEST_EXPRESSION_DIALOG_TITLE"),
                            this.resources.getString("TRIGGERS_SUCCESS_EXPRESSION_TEST_PASSED"),
                            "info");
        } else {
            $.messager.alert(this.resources.getString("TRIGGERS_TEST_EXPRESSION_DIALOG_TITLE"),
                            this.resources.getString("TRIGGERS_ERROR_EXPRESSION_TEST_FAILED"),
                            "error");
        }
    }
});
/** @class */
jscape.domain.ExpressionTestDialog = Class.extend(
    /** @lends {jscape.domain.ExpressionTestDialog.prototype} */
{
    panels: null,

    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-triggers-expressiontest").clone().attr("id", "");
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_TEST") || "Test", handler: $.proxy(this._onTest, this)},
                {text: resources.getString("DIALOG_BUTTON_CLOSE") || "Close", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=triggerexpressiontestfields]", this.dialog);
        this.expression = $("textarea[name=expression]", this.fieldset);
        this.parameters = $("fieldset[name=expressionparametersfields]", this.dialog);
    },

    setExpression: function(value) {
        this.expression.val(value).prop("readonly", true);
    },

    setProperties: function(properties) {
        this.panels = [];
        this.parameters.empty();
        $.each(properties, $.proxy(function(_, entry) {
            var panel = this._createValuePanel(entry);
            panel.appendTo(this.parameters);
            this.panels.push(panel);
        }, this));
    },

    getParameter: function(name) {
        for (var i = 0; i < this.panels.length; i++) {
            var panel = this.panels[i];
            if (name == panel.getName()) {
                return panel.getValue();
            }
        }
        return null;
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onTest: function () {
        if (this.fieldset.form("validate")) {
            this.listener.onTest(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _createValuePanel: function(eventProperty) {
        return new jscape.domain.ExpressionTestDialog.ValuePanel(eventProperty.type, eventProperty.property);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
/** @class */
jscape.domain.ExpressionTestDialog.ValuePanel = Class.extend(
    /** @lends {jscape.domain.ExpressionTestDialog.ValuePanel.prototype} */
{
    type: null,
    input: null,

    /** @constructor */
    initialize: function(type, name) {
        this.type = type;
        this.name = name;
    },

    appendTo: function (parent) {
        var html;
        if (this.isBoolean()) {
            html = '<div class="radio"><span style="width:16em;float:left">' + this.name + '</span>'
                   + '<label><input type="radio" name="' + this.name + '" value="true" />true</label>'
                   + '&nbsp;&nbsp;<label><input type="radio" name="' + this.name + '" value="false" />false</label>';
        } else if (this.isNumber()) {
            html = '<div><label>' + this.name + '</label><input type="text" name="' + this.name + '"/></div>';
        } else {
            html = '<div><label>' + this.name + '</label><input type="text" name="' + this.name + '" size="40" maxlength="50"/></div>';
        }

        var row = $(html).appendTo(parent);
        this.input = row.find(":input");
        if (this.isNumber()) {
            this.input.numberspinner({value: 0});
        }
    },

    getName: function() {
        return this.name;
    },

    getValue: function() {
        if (this.isBoolean()) {
            return this.input.filter("[value=true]").is(":checked");
        } else if (this.isNumber()) {
            return parseInt(this.input.numberspinner("getValue"));
        } else {
            return this.input.val();
        }
    },

    isBoolean: function() {
        return this.type === "boolean";
    },

    isNumber: function() {
        return $.inArray(this.type, ["byte", "short", "int", "long", "number"]) != -1;
    }
});

/** @class */
jscape.domain.CopyTriggerController = Class.extend(
    /** @lends {jscape.domain.CopyTriggerController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(trigger) {
        this.deferred = $.Deferred();
        this.trigger = trigger;

        var dialog = new jscape.domain.CopyTriggerDialog(this.resources);
        dialog.setName(trigger.name);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var name = dialog.getName();
        dialog.destroy();

        var copy = this.trigger.clone();
        copy.name = name;
        this.deferred.resolve(copy);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.defered.reject();
    }
});

/** @class */
jscape.domain.CopyTriggerDialog = Class.extend(
    /** @lends {jscape.domain.CopyTriggerDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-triggers-copy").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=copytriggerfields]", this.dialog);
        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["trigger_unique", "trigger_name_nonempty"], missingMessage: resources.getString("TRIGGERS_ERROR_BAD_TRIGGER_NAME")});

        this.title = this.dialog.dialog("header").text() || "";
    },

    getName: function() {
        return this.name.val();
    },

    setName: function(value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.name.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.DeleteTriggerController = Class.extend(
    /** @lends {jscape.domain.DeleteTriggerController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(triggers) {
        var deferred = $.Deferred();

        this._confirmDelete(triggers)
                .done(function() { deferred.resolve(triggers); })
                .fail(function() { deferred.reject(); });

        return deferred.promise();
    },

    _confirmDelete: function (triggers) {
        var text = $.map(triggers, function (trigger) { return trigger.name; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("TRIGGERS_CONFIRM_DELETE_TITLE"),
                this.resources.getString("TRIGGERS_CONFIRM_DELETE_MESSAGE", {0: text}));
    }
});

/** @class */
jscape.domain.RecentTriggersController = Class.extend(
    /** @lends {jscape.domain.RecentTriggersController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.view = new jscape.domain.RecentTriggersView(resources);
        this.viewController = new jscape.domain.TriggerStateViewController(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onRefresh: function() {
        this._loadData();
    },

    onViewTriggerState: function(state) {
        this.viewController.start(state);
    },

    onPauseTrigger: function(state) {
        this.managementService.pauseTrigger(this.domainName, state.id)
                .done($.proxy(this._loadData, this));
    },

    onResumeTrigger: function(state) {
        this.managementService.resumeTrigger(this.domainName, state.id)
                .done($.proxy(this._loadData, this));
    },

    onKillTrigger: function(state) {
        this.managementService.killTrigger(this.domainName, state.id)
                .done($.proxy(this._loadData, this));
    },

    _loadData: function() {
        this.managementService.getTriggerStates(this.domainName)
                .done($.proxy(function(states) {
                    this.states = states || [];
                    this._setupView();
                }, this));
    },

    _setupView: function() {
        this.view.setStates(this.states);
    }
});

/** @class */
jscape.domain.RecentTriggersView = Class.extend(
    /** @lends {jscape.domain.RecentTriggersView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.selection = [];
        this.resources = resources;

        this.fieldset = $("fieldset[name=recenttriggersfields]");

        this.data = $("table[role=grid][aria-label=recenttriggers]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            remoteSort: false,
            singleSelect: true,
            sortName: "eventId",
            sortOrder: "asc",
            columns: [[
                {field: "eventId", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 50},
                {field: "triggerName", title: $("thead th:nth-child(2)", this.data).text(), sortable: true, width: 100},
                {field: "status", title: $("thead th:nth-child(3)", this.data).text(), sortable: true, width: 70, formatter: $.proxy(this._formatStatus, this), styler: $.proxy(this._styleStatus, this)},
                {field: "startTime", title: $("thead th:nth-child(4)", this.data).text(), sortable: true, width: 110, align: "right", formatter: $.proxy(this._formatDate, this), sorter: function(a, b) { return a < b ? -1 : a > b ? 1 : 0; }},
                {field: "endTime", title: $("thead th:nth-child(5)", this.data).text(), width: 110, align: "right", formatter: $.proxy(this._formatDate, this)},
                {field: "runtime", title: $("thead th:nth-child(6)", this.data).text(), width: 70, align: "right", formatter: $.proxy(function(_, row) { return this._formatRuntime(row); }, this)},
                {field: "action", title: $("thead th:nth-child(7)", this.data).text(), width: 150, formatter: $.proxy(function(_, row) { return this._formatAction(row); }, this)}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$ViewButton, this)
        });

        this.refreshButton = $("a[role=button][name=refresh]", this.fieldset);
        this.refreshButton.linkbutton({onClick: $.proxy(this._onClick$RefreshButton, this)});

        this.viewButton = $("a[role=button][name=view]", this.fieldset);
        this.viewButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ViewButton, this)});

        this.pauseButton = $("a[role=button][name=pause]", this.fieldset);
        this.pauseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$PauseButton, this)});

        this.resumeButton = $("a[role=button][name=resume]", this.fieldset);
        this.resumeButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ResumeButton, this)});

        this.killButton = $("a[role=button][name=kill]", this.fieldset);
        this.killButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$KillButton, this)});
    },

    setStates: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.RecentTriggersView.LISTENER;
        this._adjustButtonsState();
    },

    _formatDate: function(value) {
        return $.isNumeric(value) && value != 0 ? new Date(value).toLocaleString() : "";
    },

    _formatRuntime: function(state) {
        var date = new Date(0, 0, 1, 0, 0, 0, state.runtime);
        var pattern = (date.getMinutes() != 0 ? "{m} min. " : "") + (date.getSeconds() != 0 ? "{s} sec. " : "") + "{S} ms.";
        return new jscape.DateFormat(pattern).format(date);
    },

    _formatAction: function(triggerState) {
        var names = [];
        var executed = 0;
        $.each(triggerState.actionStates, function(_, actionState) {
            var status = actionState.status;
            if (jscape.ActionState.Status.RUNNING == status) {
                names.push(jscape.domain.TriggersModule.displayClassNameOf(actionState.actionClass));
            } else if (jscape.ActionState.Status.COMPLETED == status) {
                executed++;
            }
        });
        return this.resources.getString("TRIGGERS_TEMPLATE_COLUMN_ACTION", {0: names.join(", "), 1: executed, 2: triggerState.actionStates.length});
    },

    _formatStatus: function(status) {
        return this.resources.getString("TRIGGERS_STATE_" + status.name);
    },

    _styleStatus: function(status) {
        switch (status) {
            case jscape.Trigger.Status.COMPLETED: return "color:green";
            case jscape.Trigger.Status.FAILED: return "color:red";
            case jscape.Trigger.Status.CANCELLED: return "color:gray";
            case jscape.Trigger.Status.PAUSED: return "color:blue";
            default: return "color:black";
        }
    },

    _onClick$RefreshButton: function() {
        this.listener.onRefresh();
    },

    _onClick$ViewButton: function() {
        this.listener.onViewTriggerState(this.selection[0]);
    },

    _onClick$PauseButton: function() {
        this.listener.onPauseTrigger(this.selection[0]);
    },

    _onClick$ResumeButton: function() {
        this.listener.onResumeTrigger(this.selection[0]);
    },

    _onClick$KillButton: function() {
        this.listener.onKillTrigger(this.selection[0]);
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var state = this.selection.length == 1 ? this.selection[0] : null;
        this.viewButton.linkbutton(state ? "enable" : "disable");
        this.pauseButton.linkbutton(state && state.pausable() ? "enable" : "disable");
        this.resumeButton.linkbutton(state && state.resumable() ? "enable" : "disable");
        this.killButton.linkbutton(state && state.cancelable() ? "enable" : "disable");
    }
});
jscape.domain.RecentTriggersView.LISTENER = {
    onRefresh: $.noop,
    onViewTriggerState: $.noop,
    onPauseTrigger: $.noop,
    onResumeTrigger: $.noop,
    onKillTrigger: $.noop
};

/** @class */
jscape.domain.TriggerStateViewController = Class.extend(
    /** @lends {jscape.domain.TriggerStateViewController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(state) {
        this.deferred = $.Deferred();
        this._setupDialog(state).show(this);
        return this.deferred.promise();
    },

    onClose: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function(state) {
        var dialog = new jscape.domain.TriggerStateDialog(this.resources);
        dialog.setData(state);
        return dialog;
    }
});

/** @class */
jscape.domain.TriggerStateDialog = Class.extend(
    /** @lends {jscape.domain.TriggerStateDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.dialog = $("#d-trigger-recent").clone().attr("id", "");
        this.dialog.dialog({
            width: 750, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [{text: resources.getString("DIALOG_BUTTON_CLOSE") || "Close", handler: $.proxy(this._onCancel, this)}],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=recenttriggerstatefields]", this.dialog);
        this.eventId = $("code[aria-label=event-id]", this.fieldset);
        this.eventType = $("code[aria-label=event-type]", this.fieldset);
        this.triggerName = $("code[aria-label=trigger-name]", this.fieldset);
        this.status = $("code[aria-label=status]", this.fieldset);
        this.startTime = $("code[aria-label=start-time]", this.fieldset);
        this.endTime = $("code[aria-label=end-time]", this.fieldset);
        this.actionHistory = $("table[role=grid][aria-label=action-history]", this.fieldset).height(110);
        this.actionHistory.datagrid({
            fitColumns: true,
            remoteSort: false,
            columns: [[
                    {field: "actionClass", title: $("thead th:nth-child(1)", this.actionHistory).text(), sortable: true, width: 100, formatter: $.proxy(this._formatAction, this)},
                    {field: "status", title: $("thead th:nth-child(2)", this.actionHistory).text(), sortable: true, width: 80, formatter: $.proxy(function(_, data) { return this._formatStatus(data.status); }, this), styler: $.proxy(function(_, data) { return this._styleActionStatus(data.status); }, this)},
                    {field: "startTime", title: $("thead th:nth-child(3)", this.actionHistory).text(), sortable: true, width: 110, align: "right", formatter: $.proxy(this._formatDate, this), sorter: function(a, b) { return a < b ? -1 : a > b ? 1 : 0; }},
                    {field: "endTime", title: $("thead th:nth-child(4)", this.actionHistory).text(), sortable: true, width: 110, align: "right", formatter: $.proxy(this._formatDate, this), sorter: function(a, b) { return a < b ? -1 : a > b ? 1 : 0; }}

            ]]
        });
        this.eventParameters = $("table[role=grid][aria-label=event-parameters]", this.fieldset).height(180);
        this.eventParameters.propertygrid({showGroup: false, scrollbarSize: 0, sortName: "name", sortOrder: "asc"});
    },

    setData: function(state) {
        this.eventId.text(state.eventId);
        this.eventType.text(jscape.domain.TriggersModule.displayClassNameOf(state.eventClassName));
        this.triggerName.text(state.triggerName);
        this.status.text(this._formatStatus(state.status)).css(this._styleTriggerStatus(state.status));
        this.startTime.text(this._formatDate(state.startTime));
        this.endTime.text(this._formatDate(state.endTime));
        this.endTime.text(this._formatDate(state.endTime));
        this.actionHistory.datagrid("loadData", state.actionStates);
        this.eventParameters.propertygrid("loadData", $.map(state.eventParameters, function(value, key) { return {name: key, value: value}; }));
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _formatAction: function(value) {
        return jscape.domain.TriggersModule.displayClassNameOf(value);
    },

    _formatStatus: function(status) {
        return this.resources.getString("TRIGGERS_STATE_" + status.name);
    },

    _styleActionStatus: function(status) {
        switch (status) {
            case jscape.ActionState.Status.COMPLETED: return "color:green";
            case jscape.ActionState.Status.FAILED: return "color:red";
            case jscape.ActionState.Status.CANCELLED: return "color:gray";
            case jscape.ActionState.Status.INTERRUPTED: return "color:blue";
            default: return "color:black";
        }
    },

    _styleTriggerStatus: function(status) {
        switch (status) {
            case jscape.Trigger.Status.COMPLETED: return {color: "green"};
            case jscape.Trigger.Status.FAILED: return {color: "red"};
            case jscape.Trigger.Status.CANCELLED: return {color: "gray"};
            case jscape.Trigger.Status.INTERRUPTED: return {color: "blue"};
            default: return "color:black";
        }
    },

    _formatDate: function(value) {
        return $.isNumeric(value) ? new Date(value).toLocaleString() : "";
    },

    _onCancel: function () {
        this.hide();
        this.listener.onClose(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.TriggersSettingsController = Class.extend(
    /** @lends {jscape.domain.TriggersSettingsController.prototype} */
{
    config: null,
    copy: null,
    variables: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.domain.TriggerSettingsView(resources);

        this._initValidators(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.copy && !Object._equals(this.copy, this._createConfiguration());
    },

    onAddVariable: function() {
        var dialog = new jscape.domain.AddGlobalVariableDialog(this.resources);
        dialog.show({
            onSubmit: $.proxy(function(d) { var name = d.getName(); var value = d.getValue(); d.destroy(); this._variableAdded(name, value); }, this),
            onCancel: function(d) { d. destroy(); }
        });
    },

    onDeleteVariable: function(name) {
        jscape.ConfirmDialog.confirm(this.resources.getString("TRIGGERS_CONFIRM_DELETE_VARIABLE_TITLE"),
                                     this.resources.getString("TRIGGERS_CONFIRM_DELETE_VARIABLE_MESSAGE", {0: name}))
                .done($.proxy(this._variableDeleted, this, name));
    },

    onChangeVariable: function(name, value) {
        for (var i = 0; i < this.variables.length; i++) {
            if (this.variables[i].name == name) {
                this.variables[i].value = value;
                break;
            }
        }
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var config = this._createConfiguration();
        return this.managementService.setTriggerServiceConfiguration(this.domainName, config)
                .done($.proxy(function() {
                    this.config = config;
                    this.copy = this._copyConfiguration();
                    this.variables = this._copyVariables();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.config = this.copy;
        this.copy = this._copyConfiguration();
        this.variables = this._copyVariables();
        this._setupView();
    },

    _variableAdded: function(name, value) {
        var variable = {name: name, value: value};
        this.variables.push(variable);
        this.view.setVariables(this.variables);
        this.view.selectItem(variable);
    },

    _variableDeleted: function (name) {
        var index = -1;
        for (var i = 0; i < this.variables.length; i++) {
            if (this.variables[i].name === name) {
                index = i;
                break;
            }
        }
        if (index != -1) {
            this.variables.splice(index, 1);
        }
        this.view.setVariables(this.variables);

        if (this.variables.length) {
            index = Math.max(0, Math.min(index, this.variables.length - 1));
            this.view.selectItem(this.variables[index]);
        }
    },

    _loadData: function() {
        this.managementService.getTriggerServiceConfiguration(this.domainName)
                .done($.proxy(function(config) {
                    this.config = config;
                }, this))
                .always($.proxy(function() {
                    this.copy = this._copyConfiguration();
                    this.variables = this._copyVariables();
                    this._setupView();
                }, this));
    },

    _copyConfiguration: function() {
        return $.extend(new jscape.TriggerServiceConfiguration(), this.config);
    },

    _copyVariables: function() {
        return $.map(this.config && this.config.globalVariables || {}, function(value, key) { return {name: key, value: value}; });
    },

    _createConfiguration: function() {
        var variables = {};
        $.each(this.variables, function(_, entry) { variables[entry.name] = entry.value; });
        return new jscape.TriggerServiceConfiguration(this.view.getMaxThreadCount(),
                                                      this.view.getMaxEventQueueSize(),
                                                      this.view.getMaxTransferCount(),
                                                      variables);
    },

    _setupView: function () {
        this.view.setMaxThreadCount(this.config.maxThreadCount);
        this.view.setMaxEventQueueSize(this.config.maxEventQueueSize);
        this.view.setMaxTransferCount(this.config.maxTransferCount);
        this.view.setVariables(this.variables);
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    },

    _initValidators: function() {
        var v = new jscape.Validator();
        v.nonEmptyRule("triggers_variable_nonempty", this.resources.getString("TRIGGERS_ERROR_BAD_VARIABLE_NAME"));
        v.rule("triggers_variable_unique", $.proxy(this._variableNotExists, this), this.resources.getString("TRIGGERS_ERROR_VARIABLE_EXISTS"));
    },

    _variableNotExists: function(value) {
        return $.grep(this.variables, function(entry) { return entry.name == value; }).length == 0;
    }
});

/** @class */
jscape.domain.TriggerSettingsView = Class.extend(
    /** @lends {jscape.domain.TriggerSettingsView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=triggerssettingsfields]");

        this.limitTriggers = $("input[name=limittriggers]:checkbox", this.fieldset);
        this.limitTriggers.change($.proxy(function() {
            var enabled = this.limitTriggers.is(":checked");
            $.each([this.maxThreads, this.maxQueueSize], function(_, input) {
                input.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
            })
        }, this));
        this.maxThreads = $("input[name=maxthreads]", this.fieldset);
        this.maxThreads.numberspinner({required: true, disabled: true, min: 1, max: 1000, value: 50, width: 70});
        this.maxQueueSize = $("input[name=triggersqueuesize]", this.fieldset);
        this.maxQueueSize.numberspinner({required: true, disabled: true, min: 1, max: 1000, value: 100, width: 70});

        this.limitTransfers = $("input[name=limittransfers]:checkbox", this.fieldset);
        this.limitTransfers.change($.proxy(function() {
            var enabled = this.limitTransfers.is(":checked");
            this.maxTransfers.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.maxTransfers = $("input[name=maxtransfers]", this.fieldset);
        this.maxTransfers.numberspinner({required: true, disabled: true, min: 1, max: 1000, value: 10, width: 70});

        this.variables = $("table[role=grid][aria-label=variables]", this.fieldset);
        this.variables.propertygrid({
            onSelect: $.proxy(this._adjustButtonsState, this),
            onUnselect: $.proxy(this._adjustButtonsState, this),
            onLoadSuccess: $.proxy(this._adjustButtonsState, this),
            onEndEdit: $.proxy(this._onAfterEdit, this),
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.variables).text(), sortable: true, width: 100},
                {field: "value", title: $("thead th:nth-child(2)", this.variables).text(), width: 100, resizable: false}
            ]]
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getMaxThreadCount: function() {
        return this.limitTriggers.is(":checked") ? parseInt(this.maxThreads.numberspinner("getValue")) : null;
    },

    setMaxThreadCount: function(value) {
        this.limitTriggers.prop("checked", value != null).change();
        if (value != null) {
            this.maxThreads.numberspinner("setValue", parseInt(value));
        }
    },

    getMaxEventQueueSize: function() {
        return this.limitTriggers.is(":checked") ? parseInt(this.maxQueueSize.numberspinner("getValue")) : null;
    },

    setMaxEventQueueSize: function(value) {
        if (value) {
            this.maxQueueSize.numberspinner("setValue", parseInt(value));
        }
    },

    getMaxTransferCount: function() {
        return this.limitTransfers.is(":checked") ? parseInt(this.maxTransfers.numberspinner("getValue")) : null;
    },

    setMaxTransferCount: function(value) {
        this.limitTransfers.prop("checked", value != null).change();
        if (value != null) {
            this.maxTransfers.numberspinner("setValue", parseInt(value));
        }
    },

    setVariables: function(values) {
        var data = $.map(values || [], function(entry) { return $.extend({editor: "text"}, entry); });
        this.variables.propertygrid("loading");
        this.variables.propertygrid("loadData", data);
        this.variables.propertygrid("loaded");
    },

    selectItem: function(value) {
        this.variables.propertygrid("selectRow", this.variables.propertygrid("getRowIndex", value));
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.TriggerSettingsView.LISTENER;
        this._adjustButtonsState();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$AddButton: function() {
        this.listener.onAddVariable();
    },

    _onClick$DeleteButton: function() {
        var name = this.variables.propertygrid("getSelected").name;
        this.listener.onDeleteVariable(name);
    },

    _onAfterEdit: function(index, data, changes) {
        this.listener.onChangeVariable(data.name, changes.value);
    },

    _onClick$ApplyButton: function() {
        this.variables.propertygrid("acceptChanges");
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.variables.propertygrid("rejectChanges");
        this.listener.onDiscard();
    },

    _adjustButtonsState: function() {
        var selections = this.variables.propertygrid("getSelections");
        this.deleteButton.linkbutton(selections.length != 0 ? "enable" : "disable");
    }
});
jscape.domain.TriggerSettingsView.LISTENER = {
    onAddVariable: $.noop,
    onDeleteVariable: $.noop,
    onChangeVariable: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.AddGlobalVariableDialog = Class.extend(
    /** @lends {jscape.domain.AddGlobalVariableDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-triggers-addglobalvar").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=globalvariablesfields]", this.dialog);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["triggers_variable_nonempty", "triggers_variable_unique"], missingMessage: resources.getString("TRIGGERS_ERROR_BAD_VARIABLE_NAME"), width: 170});

        this.value = $("input[name=value]", this.fieldset);
    },

    getName: function() {
        return this.name.val();
    },

    getValue: function() {
        return this.value.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.name.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.ActionsController = Class.extend(
    /** @lends {jscape.domain.ActionsController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.view = new jscape.domain.ActionsView(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);

        this._loadActions();
    },

    _loadActions: function() {
        this.managementService.getTriggerActions(this.domainName)
                .done($.proxy(function(actions) {
                    this.view.setActions(actions);
                }, this));
    }
});

/** @class */
jscape.domain.ActionsView = Class.extend(
    /** @lends {jscape.domain.ActionsView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=triggeractioncontentsfields]");

        this.data = $("table[role=grid][aria-label=triggeractioncontents]", this.fieldset);
        this.data.datagrid({
            height: "auto",
            fitColumns: true,
            remoteSort: false,
            singleSelect: true,
            sortName: "name",
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 100},
                {field: "description", title: $("thead th:nth-child(2)", this.data).text(), sortable: false, width: 250}
            ]]
        });
    },

    setActions: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.ActionsView.LISTENER;
    }
});
jscape.domain.ActionsView.LISTENER = {};

/** @class */
jscape.domain.FunctionsController = Class.extend(
    /** @lends {jscape.domain.FunctionsController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.view = new jscape.domain.FunctionsView(resources);
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);

        this._loadFunctions();
    },

    _loadFunctions: function () {
        this.managementService.getTriggerFunctions(this.domainName)
                .done($.proxy(function (functions) {
                    this.view.setFunctions(functions);
                }, this));
    }
});

/** @class */
jscape.domain.FunctionsView = Class.extend(
    /** @lends {jscape.domain.FunctionsView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=triggerfunctioncontentsfields]");
        this.data = $("table[role=grid][aria-label=triggerfunctioncontents]", this.fieldset);
        this.data.datagrid({
            height: "auto",
            fitColumns: true,
            remoteSort: false,
            singleSelect: true,
            sortName: "name",
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 100, formatter: $.proxy(this._formatFunctionName, this)},
                {field: "syntax", title: $("thead th:nth-child(2)", this.data).text(), width: 150, formatter: function(_, rowData) { return $.map(rowData.templates, function(value, key) { return key; }).join("<br/>")}},
                {field: "description", title: $("thead th:nth-child(3)", this.data).text(), width: 250, formatter: function(_, rowData) { return $.map(rowData.templates, function(value, key) { return value; }).join("<br/>")}}
            ]]
        });
    },

    setFunctions: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.FunctionsView.LISTENER;
    },

    _formatFunctionName: function(name) {
        return name.replace(/([a-z])([A-Z])/g, "$1 $2");
    }
});
jscape.domain.FunctionsView.LISTENER = {};