jscape.domain = jscape.domain || {};

/** @class */
jscape.WebTheme = Class.extend(
    /** @lends {jscape.WebTheme.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.name = a || "";
        this.owner = b || "";
        this.encodedLogo = c || "";
        this.uri = d || "";
    }
});
jscape.WebTheme.fromJSON = function(data) {
    return $.extend(new jscape.WebTheme(), data);
};

/** @class */
jscape.domain.UrlBrandingModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.UrlBrandingModule.prototype} */
{
    onCreate: function() {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.UrlBrandingController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onResume: function() {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.UrlBrandingController = Class.extend(
    /** @lends {jscape.domain.UrlBrandingController.prototype} */
{
    themes: null,

    /** @constructor */
    initialize: function(resources) {
        this.themes = [];

        this.view = new jscape.domain.UrlBrandingView(resources);
        this.addController = new jscape.domain.AddUrlBrandingController(resources);
        this.editController = new jscape.domain.EditUrlBrandingController(resources);
        this.deleteController = new jscape.domain.DeleteUrlBrandingController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onAddTheme: function() {
        this.addController.start(this.managementService, this.domainName)
                .done($.proxy(this._themeAdded, this));
    },

    onEditTheme: function(theme) {
        var index = $.inArray(theme, this.themes);
        if (index != -1) {
            this.editController.start(this.managementService, this.domainName, theme)
                    .done($.proxy(this._themeEdited, this, index));
        }
    },

    onDeleteTheme: function(theme) {
        this.deleteController.start(this.managementService, this.domainName, theme)
                .done($.proxy(this._themeDeleted, this, theme));
    },

    _loadData: function() {
        this.managementService.getWebThemes(this.domainName)
                .then($.proxy(this._loadWebThemesURI, this))
                .done($.proxy(function(themes) {
                    this.themes = themes;
                    this.view.setThemes(themes);
                }, this));
    },

    _loadWebThemesURI: function(themes) {
        for (var i = 0; i < themes.length; i++) {
            var theme = themes[i];
            theme.uri = this.managementService.getWebThemeURI(this.domainName, theme.name) || "";
        }
        return themes;
    },

    _themeAdded: function (theme) {
        this.themes.push(theme);
        this.view.setThemes(this.themes);
        this.view.selectTheme(theme);
    },

    _themeEdited: function (index, theme) {
        this.themes[index] = theme;
        this.view.setThemes(this.themes);
        this.view.selectTheme(theme);
    },

    _themeDeleted: function (themes) {
        var index = -1;
        while (themes.length) {
            index = $.inArray(themes.pop(), this.themes);
            this.themes.splice(index, 1);
        }
        this.view.setThemes(this.themes);

        if (this.themes.length) {
            index = Math.min(Math.max(0, index), this.themes.length - 1);
            this.view.selectTheme(this.themes[index]);
        }
    },

    _initValidators: function (resources) {
        var v = new jscape.Validator();
        v.rule("webtheme_name", $.proxy(this._themeNameValid, this), resources.getString("WEB_THEME_ERROR_BAD_NAME"));
        v.rule("webtheme_unique", $.proxy(this._themeNotExists, this), resources.getString("WEB_THEME_ERROR_ALREADY_EXIST"));
        v.rule("webtheme_logo", $.proxy(this._logoLoaded, this), resources.getString("WEB_THEME_ERROR_LOAD_LOGO"));
    },

    _themeNameValid: function(name) {
        return name.trim().length > 0 && name.indexOf("/") == -1;
    },

    _themeNotExists: function (name) {
        return $.grep(this.themes, function (theme) { return theme.name == name; }).length == 0;
    },

    _logoLoaded: function(_, params) {
        return $(params[0]).data("error") == null;
    }
});

/** @class */
jscape.domain.UrlBrandingView = Class.extend(
    /** @lends {jscape.domain.UrlBrandingView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.selection = [];

        this.fieldset = $("fieldset[name=urlbrandingfields]");

        this.data = $("table[role=grid][aria-label=urlbranding]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            singleSelect: true,
            pagination: true,
            search: true,
            remoteSort: false,
            sortName: "name",
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 90},
                {field: "owner", title: $("thead th:nth-child(2)", this.data).text(), sortable: true, width: 90},
                {field: "uri", title: $("thead th:nth-child(3)", this.data).text(), width: 110}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this)
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});

        this.logoButton = $("a[role=button][name=logo]", this.fieldset);
        this.logoButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$LogoButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.UrlBrandingView.LISTENER;
        this._adjustButtonsState();
    },

    setThemes: function (values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectTheme: function (value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    _onClick$AddButton: function() {
        this.listener.onAddTheme();
    },

    _onClick$EditButton: function() {
        this.listener.onEditTheme(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteTheme(this.selection);
    },

    _onClick$LogoButton: function() {
        var theme = this.data.datagrid("getSelected");
        var row = this.data.datagrid("options").finder.getTr(this.data[0], null, "selected").find("td:first");
        row.tooltip({position: "bottom", content: "<img src=\"data:image/png;base64," + theme.encodedLogo + "\"/>"}).tooltip("show");
        this.logoButton.one("mouseleave", $.proxy(function() { row.tooltip("destroy"); }, this));
    },

    _adjustRowSelection: function () {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var singleSelection = this.selection.length == 1;
        this.editButton.linkbutton(singleSelection ? "enable" : "disable");
        this.logoButton.linkbutton(singleSelection ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }
});
/** @const */
jscape.domain.UrlBrandingView.LISTENER = {
    onAddTheme: $.noop,
    onEditTheme: $.noop,
    onDeleteTheme: $.noop
};

/** @class */
jscape.domain.AddUrlBrandingController = Class.extend(
    /** @lends {jscape.domain.AddUrlBrandingController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.AddUrlBrandingDialog(this.resources);
        dialog.show(this);

        this.managementService.getAccounts(this.domainName).done($.proxy(function(accounts) {
            var admins = $.grep(accounts, function(account) { return account.administrator; });
            var owners = $.map(admins, function(account) { return account.login; });
            dialog.setOwners(owners.sort());
        }, this));

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var theme = this._createWebTheme(dialog);
        dialog.destroy();

        this.managementService.storeWebTheme(this.domainName, theme)
                .done($.proxy(function() { this.deferred.resolve(theme); }, this))
                .fail(this.deferred.reject);
    },


    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    getWebThemeURI: function(themeName) {
        return themeName && themeName.trim().length
                ? this.managementService.getWebThemeURI(this.domainName, themeName) || ""
                : "";
    },

    _createWebTheme: function(dialog) {
        return new jscape.WebTheme(dialog.getName(), dialog.getOwner(), dialog.getLogoData(), dialog.getUri());
    }
});

/** @class */
jscape.domain.EditUrlBrandingController = Class.extend(
    /** @lends {jscape.domain.EditUrlBrandingController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function (resources) {
        this.resources = resources;
    },

    start: function (managementService, domainName, theme) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.EditUrlBrandingDialog(this.resources);
        dialog.setName(theme.name);
        dialog.setLogoData(theme.encodedLogo);
        dialog.setUri(theme.uri || "/");
        dialog.show(this);

        this.managementService.getAccounts(domainName).done(function(accounts) {
            var admins = $.grep(accounts, function(account) { return account.administrator; });
            var owners = $.map(admins, function(account) { return account.login; });
            dialog.setOwners(owners.sort());
            dialog.setOwner(theme.owner);
        });

        return this.deferred.promise();
    },

    onSubmit: function (dialog) {
        var theme = this._createWebTheme(dialog);
        dialog.destroy();

        this.managementService.storeWebTheme(this.domainName, theme)
                .done($.proxy(function() { this.deferred.resolve(theme); }, this))
                .fail(this.deferred.reject);
    },

    onCancel: function (dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createWebTheme: function (dialog) {
        return new jscape.WebTheme(dialog.getName(), dialog.getOwner(), dialog.getLogoData(), dialog.getUri());
    }
});

/** @class */
jscape.domain.UrlBrandingDialog = Class.extend(
    /** @lends {jscape.domain.UrlBrandingDialog.prototype} */
{
    /** @const */
    DATA_URL: {pattern: /data:image\/.*;base64,(.+)/, prefix: "data:image/png;base64,"},

    logoData: "",

    /** @constructor */
    initialize: function(dialog, resources) {
        this.resources = resources;

        this.dialog = dialog.clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 700, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=webthemefields]", this.dialog);

        this.logo = $("div[aria-label=logopreview] img", this.fieldset).attr("id", "logo_" + new Date().getTime().toString(36));
        this.logo.on("load", $.proxy(function() {
            this.logo.data("error", null);
            this.logoFile.validatebox("validate");
        }, this));
        this.logo.on("error", $.proxy(function() {
            this.logo.data("error", true);
            this.logoFile.validatebox("validate");
        }, this));
        this.logo.parent().css({width: 400, height: 100, overflow: "auto", border: "1px solid #DEDEDE"});
        this.logo.parent().click($.proxy(function() { this.logoFile.click(); }, this));

        this.logoFile = $("input[name=logo]", this.fieldset);
        this.logoFile.validatebox({required: true, validType: "webtheme_logo['#" + this.logo.attr("id") + "']", missingMessage: resources.getString("WEB_THEME_ERROR_BAD_LOGO")});
        this.logoFile.change($.proxy(this._onChange$Logo, this));

        this.ownedBy = $("input[name=ownedby]:checkbox", this.fieldset);
        this.ownedBy.change($.proxy(function() { this.owner.combobox(this.ownedBy.is(":checked:not(:disabled)") ? "enable" : "disable");}, this));
        this.owner = $("select[name=owner]", this.fieldset);
        this.owner.combobox({
            required: false,
            editable: false,
            disabled: true,
            width: 120,
            onLoadSuccess: $.proxy(function(data) {
                if (data.length) {
                    this.owner.combobox("clear").combobox("select", data[0].value);
                } else {
                    this.ownedBy.prop("disabled", true).prop("checked", false).change();
                }
            }, this)
        }).combobox("autocomplete", true);

        this.uri = $("span[aria-label=themeuri]", this.fieldset);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    getName: function() {
        return this.themeName;
    },

    setName: function(value) {
        this.themeName = value;
    },

    getLogoData: function () {
        var data = this.logo.attr("src");
        return data ? data.replace(this.DATA_URL.pattern, "$1") : "";
    },

    setLogoData: function (data) {
        this.logo[0].src = this.DATA_URL.pattern.test(data) ? data : this.DATA_URL.prefix + data;
    },

    getOwner: function () {
        return this.ownedBy.is(":checked:not(:disabled)") ? this.owner.combobox("getValue") : null;
    },

    setOwner: function (value) {
        this.ownedBy.prop("checked", !!value).change();
        if (value) {
            this.owner.combobox("clear").combobox("select", value);
        }
    },

    setOwners: function (values) {
        var data = $.map(values, function(value) { return {text: value, value: value} });
        this.owner.combobox("loadData", data);
    },

    getUri: function () {
        return this.uri.text();
    },

    setUri: function(value) {
        this.uri.text(value || "");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _onChange$Logo: function(e) {
        var files = $.grep(e.target.files || [], function(file) { return file && file.type.match("image.*"); });
        if (files.length && typeof FileReader != undefined) {
            var reader = new FileReader();
            reader.onload = $.proxy(function (e) { this.setLogoData(e.target.result); }, this);
            reader.readAsDataURL(files[0]);
        } else {
            this.setLogoData("");
        }
    }
});

/** @class */
jscape.domain.AddUrlBrandingDialog = jscape.domain.UrlBrandingDialog.extend(
    /** @lends {jscape.domain.AddUrlBrandingDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-webtheme-add"), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["webtheme_name", "webtheme_unique"], missingMessage: resources.getString("WEB_THEME_ERROR_BAD_NAME")});
        this.name.change($.proxy(this._adjustBrandingUri, this));
    },

    /** @override */
    getName: function() {
        return this.name.val();
    },

    _adjustBrandingUri: function () {
        this.setUri(this.name.validatebox("isValid") ? this.listener.getWebThemeURI(this.getName()) : null);
    }
});

/** @class */
jscape.domain.EditUrlBrandingDialog = jscape.domain.UrlBrandingDialog.extend(
    /** @lends {jscape.domain.EditUrlBrandingDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-webtheme-edit"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    /** @override */
    setName: function(value) {
        this._super(value);
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    },

    /** @override */
    setLogoData: function(data) {
        if (data && data.trim().length) {
            this.logoFile.validatebox("options").required = false;
        }
        this._super(data);
    }
});

/** @class */
jscape.domain.DeleteUrlBrandingController = Class.extend(
    /** @lends {jscape.domain.DeleteUrlBrandingController.prototype} */
{
    themes: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, themes) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.themes = themes || [];
        this.deferred = $.Deferred();

        this._confirmDelete()
                .done($.proxy(this._deleteConfirmed, this))
                .fail(this.deferred.reject);

        return this.deferred.promise();
    },

    _confirmDelete: function() {
        var text = $.map(this.themes, function(theme) { return theme.name; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("WEB_THEME_CONFIRM_DELETE_TITLE"),
                this.resources.getString("WEB_THEME_CONFIRM_DELETE_MESSAGE", {0: text}));
    },

    _deleteConfirmed: function () {
        var promises = new Array(this.themes.length);
        for (var i = 0; i < this.themes.length; i++) {
            promises[i] = this.managementService.deleteWebTheme(this.domainName, this.themes[i].name);
        }

        $.when.apply(jQuery, promises)
                .done($.proxy(function () { this.deferred.resolve(this.themes); }, this))
                .fail(this.deferred.reject);
    }
});