jscape.domain = jscape.domain || {};

/** @class */
jscape.Quota = Class.extend(
    /** @lends {jscape.Quota.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.bytes = parseInt(a) || 0;
        this.resetPeriod = parseInt(b) || 0;
        this.transferredBytes = parseInt(c) || 0;
    },

    toJSON: function() {
        return {bytes: this.bytes, resetPeriod: this.resetPeriod};
    }
});
jscape.Quota.fromJSON = function(data) {
    return data ? new jscape.Quota(data.bytes, data.resetPeriod, data.transferredBytes) : null;
};

/** @class */
jscape.AccountTransferQuotas = Class.extend(
    /** @lends {jscape.AccountTransferQuotas.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e) {
        this.uploadsQuota = a || null;
        this.downloadsQuota = b || null;
        this.transfersQuota = c || null;
        this.maxUploadsPerSession = parseInt(d) || null;
        this.maxDownloadsPerSession = parseInt(e) || null;
    }
});
jscape.AccountTransferQuotas.fromJSON = function (data) {
    return data
           ? new jscape.AccountTransferQuotas(jscape.Quota.fromJSON(data.uploadsQuota),
                                              jscape.Quota.fromJSON(data.downloadsQuota),
                                              jscape.Quota.fromJSON(data.transfersQuota),
                                              data.maxUploadsPerSession,
                                              data.maxDownloadsPerSession)
           : null;
};

/** @class */
jscape.IpAccessVerifier = Class.extend(
    /** @lends {jscape.IpAccessVerifier.prototype} */
{
    /** @constructor */
    initialize: function(masks) {
        this.masks = masks || [];
    }
});
jscape.IpAccessVerifier.fromJSON = function(data) {
    return new jscape.IpAccessVerifier($.map(data.masks || [], jscape.IpAccessVerifier.IPMask.fromJSON));
};

/** @class */
jscape.IpAccessVerifier.IPMask = Class.extend(
    /** @lends {jscape.IpAccessVerifier.IPMask.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.mask = a || "";
        this.reason = b || "";
        this.allowed = c || false;
    }
});
jscape.IpAccessVerifier.IPMask.fromJSON = function(data) {
    return $.extend(new jscape.IpAccessVerifier.IPMask(), data);
};

/** @class */
jscape.DomainAdministration = Class.extend(
    /** @lends {jscape.DomainAdministration.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e, f, g, h, i, j, k, l, m, n) {
        this.maxOwnedCount = parseInt(a) || null;
        this.manageNonOwned = b || false;
        this.allowGroupAssignment = c || false;
        this.allowResourceMapping = d || false;
        this.restrictDir = e || null;
        this.shareQuotas = f || false;
        this.allowSetPhoneAuth = g || false;
        this.allowUrlBranding = h || false;
        this.maxUrlBrandings = parseInt(i) || null;
        this.allowDropZone = j || false;
        this.maxDropZones = parseInt(k) || null;
        this.allowManagePublicContact = l || false;
        this.allowSystemPgpKeyAssignment = m || false;
        this.allowedTemplates = n || [];
    }
});
jscape.DomainAdministration.fromJSON = function (data) {
    return data ? $.extend(new jscape.DomainAdministration(), data) : null;
};

/** @class */
jscape.WebOptions = Class.extend(
    /** @lends {jscape.WebOptions.prototype} */
{
    /** @constructor */
    initialize: function (a, b, c, d, e, f, g, h) {
        this.loginRedirection = a || "storage";
        this.webPublicKeyAuthenticationAvailable = b != undefined ? b : true;
        this.webOpenPgpEncryptionAvailable = c != undefined ? c : true;
        this.webQuotasAvailable = d != undefined ? d : true;
        this.webContactsAvailable = e != undefined ? e : true;
        this.webAdHocActivityAvailable = f != undefined ? f : true;
        this.webDropZonesAvailable = g != undefined ? g : true;
        this.webAftpProtocolPreferable = h || false;
    },

    toJSON: function() {
        return $.extend({}, this, {loginRedirection: {"target": this.loginRedirection}});
    }
});
jscape.WebOptions.fromJSON = function(data) {
    var loginRedirection = data.loginRedirection && data.loginRedirection.target;
    return $.extend(new jscape.WebOptions(), data, {loginRedirection: loginRedirection});
};



/** @class */
jscape.domain.UserInfoView = Class.extend(
    /** @lends {jscape.domain.UserInfoView.prototype} */
{
    /** @constructor */
    initialize: function(fieldset, resources) {
        this.resources = resources;

        this.fieldset = fieldset;

        this.username = $("input[name=username]", this.fieldset);
        this.email = $("input[name=email]", this.fieldset);
        this.email.validatebox({validType: {any: ["variable", "multi_value['email',',']"]}, invalidMessage: resources.getString("USERS_ERROR_BAD_EMAIL")});
        this.company = $("input[name=company]", this.fieldset);

        this.phoneCode = $("input[name=phonecode]", this.fieldset);
        this.phoneCode.validatebox({validType: "pattern['^[\\\\d]+$']", invalidMessage: resources.getString("USERS_ERROR_BAD_PHONE")});
        this.phoneNumber = $("input[name=phonenum]", this.fieldset);
        this.phoneNumber.validatebox({validType: "pattern['^[\\\\d- ]+$']", invalidMessage: resources.getString("USERS_ERROR_BAD_PHONE")});
        this.phoneExtension = $("input[name=phoneext]", this.fieldset);
        this.phoneExtension.validatebox({validType: "pattern['^[\\\\d- ]+$']", invalidMessage: resources.getString("USERS_ERROR_BAD_PHONE")});

        this.groups = $("table[role=grid][aria-label=usergroups]", this.fieldset);
        this.groups.datagrid({
            fitColumns:true,
            singleSelect: false,
            checkOnSelect: true,
            selectOnCheck: true,
            remoteSort: false,
            width: 600,
            height: 80,
            onLoadSuccess: function(data) { $(this).datagrid("getPanel").find("div.datagrid-header-check input:checkbox").prop("disabled", data.rows.length == 0); },
            columns: [[
                  {field: "allowed", checkbox: true},
                  {field: "name", title: $("thead th:nth-child(1)", this.groups).text(), sortable: true, width: 100, formatter: function (_, row) {return (row || "").escapeXml(); }}
            ]]
        });

        this.enabled = $("input[name=enabled]:checkbox", this.fieldset);
        this.allowAdHocTransfers = $("input[name=adhocallowed]:checkbox", this.fieldset);

        this.ownedBy = $("input[name=ownedby]:checkbox", this.fieldset);
        this.ownedBy.on("change", $.proxy(function() {
            var enabled = this.ownedBy.is(":checked");
            this.owner.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        }, this));

        this.owner = $("select[name=owners]", this.fieldset);
        this.owner.combobox({required: true, disabled: true, width: 120}).combobox("autocomplete", true);

        this.expires = $("input[name=expires]", this.fieldset);
        this.expires.on("change", $.proxy(function() {
            var enabled = this.expires.is(":checked");
            this.expirationDate.datebox(enabled ? "enable" : "disable").datebox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.expirationDate = $("input[name=expirationdate]", this.fieldset);
        this.expirationDate.datebox({
            required: true,
            editable: false,
            disabled: true,
            width: 120,
            formatter: function(date) { return new jscape.DateFormat("{MM}/{dd}/{yy}").format(date); },
            parser: function(text) {
                var matches = /(\d{1,2})\/(\d{1,2})\/(\d{2,4})/.exec(text);
                if (matches != null && matches.length == 4) {
                    var m = parseInt(matches[1], 10), d = parseInt(matches[2], 10), y = parseInt(matches[3], 10);
                    if (!isNaN(d) && !isNaN(m) && !isNaN(y)) {
                        return new Date(y, m - 1, d);
                    }
                }
                return new Date();
            }
        });
        this.expirationDate.datebox("setValue", $.now());

        this.secured = $("input[name=secured]:checkbox", this.fieldset);
        this.usePhoneAuth = $("input[name=phoneauth]:checkbox", this.fieldset);
        this.allowPasswordChange = $("input[name=allowpwdchange]:checkbox", this.fieldset);
        this.ignorePasswordAging = $("input[name=ignorepwdaging]:checkbox", this.fieldset);

        this.clientKeys = $("table[role=grid][aria-label=clientkeys]", this.fieldset);
        this.clientKeys.datagrid({
            fitColumns:true,
            singleSelect: false,
            checkOnSelect: true,
            selectOnCheck: true,
            remoteSort: false,
            width: 600,
            height: 90,
            onLoadSuccess: function(data) { $(this).datagrid("getPanel").find("div.datagrid-header-check input:checkbox").prop("disabled", data.rows.length == 0); },
            columns: [[
                {field: "allowed", checkbox: true},
                {field: "alias", title: $("thead th:nth-child(1)", this.clientKeys).text(), sortable: true, width: 100, formatter: function (_, row) { return (row || "").escapeXml(); }}
            ]]
        });
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.UserInfoView.LISTENER;
        this.fieldset.css({visible: "visible"});
        this.fieldset.form("validate");
    },

    getUsername: function() {
        return this.username.val();
    },

    setUsername: function(value) {
        this.username.val(value);
    },

    getEmail: function() {
        return this.email.val();
    },

    setEmail: function(value) {
        this.email.val(value);
    },

    getCompany: function() {
        return this.company.val();
    },

    setCompany: function(value) {
        this.company.val(value);
    },

    getPhoneCode: function() {
        return this.phoneCode.validatebox("isValid") ? this.phoneCode.val() : null;
    },

    setPhoneCode: function(value) {
        if (value != null) {
            this.phoneCode.val(value).validatebox("validate");
        }
    },

    getPhoneNumber: function() {
        return this.phoneNumber.validatebox("isValid") ? this.phoneNumber.val() : null;
    },

    setPhoneNumber: function(value) {
        if (value != null) {
            this.phoneNumber.val(value).validatebox("validate");
        }
    },

    getPhoneExtension: function() {
        return this.phoneExtension.validatebox("isValid") ? this.phoneExtension.val() : null;
    },

    setPhoneExtension: function(value) {
        if (value != null) {
            this.phoneExtension.val(value).validatebox("validate");
        }
    },

    setAvailableClientKeys: function(values) {
        this.clientKeys.datagrid("loading");
        this.clientKeys.datagrid("loadData", values);
        this.clientKeys.datagrid("loaded");
    },

    getClientKeys: function() {
        return this.clientKeys.datagrid("getChecked");
    },

    setClientKeys: function(values) {
        if (values && values.length) {
            var rows = this.clientKeys.datagrid("getRows");
            for (var i = 0; i < values.length; i++) {
                this.clientKeys.datagrid("checkRow", $.inArray(values[i], rows));
            }
        } else {
            this.clientKeys.datagrid("clearChecked");
        }
    },

    setAvailableGroups: function(values) {
        this.groups.datagrid("loading");
        this.groups.datagrid("loadData", values);
        this.groups.datagrid("loaded");
    },

    getGroups: function() {
        return this.groups.datagrid("getChecked");
    },

    setGroups: function(values) {
        if (values && values.length) {
            var rows = this.groups.datagrid("getRows");
            for (var i = 0; i < values.length; i++) {
                this.groups.datagrid("checkRow", $.inArray(values[i], rows));
            }
        } else {
            this.groups.datagrid("clearChecked");
        }
    },

    setAvailableOwners: function(values) {
        var data = new Array(values.length);
        for (var i = 0; i < values.length; i++) {
            var owner = values[i].login;
            data[i] = {text: owner, value: owner};
        }
        this.owner.combobox("loadData", data);
        this.ownedBy.prop("disabled", data.length == 0);
    },

    getOwner: function() {
        return this.ownedBy.is(":checked") ? this.owner.combobox("getValue") : null;
    },

    setOwner: function(value) {
        if (value) {
            this.owner.combobox("enable").combobox("select", value);
            this.ownedBy.prop("checked", this.owner.combobox("getValue") === value);
        } else {
            this.ownedBy.prop("checked", false);
        }
    },

    getExpirationDate: function() {
        return this.expires.is(":checked")
                ? this.expirationDate.datebox("calendar").calendar("options").current
                : null;
    },

    setExpirationDate: function(value) {
        var formatter = this.expirationDate.datebox("options").formatter;
        if (value != null) {
            this.expires.prop("checked", true);
            this.expirationDate.datebox("setValue", formatter(value)).datebox("enable");
        } else {
            this.expires.prop("checked", false);
            this.expirationDate.datebox("setValue", formatter(new Date()));
        }
    },

    getEnabled: function() {
        return this.enabled.is(":checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", !!value);
    },

    getSecured: function() {
        return this.secured.is(":checked");
    },

    setSecured: function(value) {
        this.secured.prop("checked", !!value);
    },

    getUsePhoneAuthentication: function() {
        return this.usePhoneAuth.is(":checked");
    },

    setUsePhoneAuthentication: function(value) {
        this.usePhoneAuth.prop("checked", !!value);
    },

    getAllowAdHocTransfers: function() {
        return this.allowAdHocTransfers.is(":checked");
    },

    setAllowAdHocTransfers: function(value) {
        this.allowAdHocTransfers.prop("checked", !!value);
    },

    getAllowPasswordChanging: function() {
        return this.allowPasswordChange.is(":checked");
    },

    setAllowPasswordChanging: function(value) {
        this.allowPasswordChange.prop("checked", !!value);
    },

    getIgnorePasswordAgingRules: function() {
        return this.ignorePasswordAging.is(":checked");
    },

    setIgnorePasswordAgingRules: function(value) {
        this.ignorePasswordAging.prop("checked", !!value);
    }
});
jscape.domain.UserInfoView.LISTENER = {};





/** @class */
jscape.domain.QuotasController = Class.extend(
    /** @lends {jscape.domain.QuotasController.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.UserQuotaView(resources, container);
    },

    start: function(quotas) {
        this.view.show(this);

        this._setupView(quotas || null);
    },

    getQuotas: function () {
        return new jscape.AccountTransferQuotas(this.view.getUploadsQuota(),
                                                this.view.getDownloadsQuota(),
                                                this.view.getTransfersQuota(),
                                                this.view.getMaxUploadsPerSession(),
                                                this.view.getMaxDownloadsPerSession());
    },

    _setupView: function(quotas) {
        if (quotas != null) {
            this.view.setDownloadsQuota(quotas.downloadsQuota);
            this.view.setUploadsQuota(quotas.uploadsQuota);
            this.view.setTransfersQuota(quotas.transfersQuota);
            this.view.setMaxDownloadsPerSession(quotas.maxDownloadsPerSession);
            this.view.setMaxUploadsPerSession(quotas.maxUploadsPerSession);
        } else {
            this.view.setDownloadsQuota(null);
            this.view.setUploadsQuota(null);
            this.view.setTransfersQuota(null);
            this.view.setMaxDownloadsPerSession(null);
            this.view.setMaxUploadsPerSession(null);
        }
    }
});

/** @class */
jscape.domain.UserQuotaView = Class.extend(
    /** @lends {jscape.domain.UserQuotaView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=quotasfields]", container);

        this.downloadQuotaView = new jscape.domain.QuotaView(
                $("input[name=maxdload]:checkbox", this.fieldset),
                $("input[name=maxdloadcount]", this.fieldset),
                $("input[name=maxdloadresetperiod]", this.fieldset)
        );

        this.uploadQuotaView = new jscape.domain.QuotaView(
                $("input[name=maxupload]:checkbox", this.fieldset),
                $("input[name=maxuploadcount]", this.fieldset),
                $("input[name=maxuploadresetperiod]", this.fieldset)
        );

        this.transferQuotaView = new jscape.domain.QuotaView(
                $("input[name=maxtransfer]:checkbox", this.fieldset),
                $("input[name=maxtransfercount]", this.fieldset),
                $("input[name=maxtransferresetperiod]", this.fieldset)
        );

        this.downloadPerSessionEnabled = $("input[name=maxdloadsession]:checkbox", this.fieldset);
        this.downloadPerSessionEnabled.click($.proxy(this._adjustDownloadPerSession, this));

        this.downloadPerSession = $("input[name=maxdloadsessioncount]", this.fieldset);
        this.downloadPerSession.numberspinner({required: true, disabled: true, min: 1, max: 1000, value: 1});

        this.uploadPerSessionEnabled = $("input[name=maxuploadsession]:checkbox", this.fieldset);
        this.uploadPerSessionEnabled.click($.proxy(this._adjustUploadPerSession, this));

        this.uploadPerSession = $("input[name=maxuploadsessioncount]", this.fieldset);
        this.uploadPerSession.numberspinner({required: true, disabled: true, min: 1, max: 1000, value: 1});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.UserQuotaView.LISTENER;
        this.downloadQuotaView.show(this);
        this.uploadQuotaView.show(this);
        this.transferQuotaView.show(this);
    },

    onStateChanged: function(view) {
        var targets = view == this.transferQuotaView
                      ? [this.downloadQuotaView, this.uploadQuotaView]
                      : [this.transferQuotaView];
        if (view.isAllowed()) {
            for (var i = 0; i < targets.length; i++) {
                targets[i].setAllowed(false);
            }
        }
    },

    getDownloadsQuota: function() {
        return this._getQuota(this.downloadQuotaView);
    },

    setDownloadsQuota: function(value) {
        this._setQuota(this.downloadQuotaView, value);
    },

    getUploadsQuota: function() {
        return this._getQuota(this.uploadQuotaView);
    },

    setUploadsQuota: function(value) {
        this._setQuota(this.uploadQuotaView, value);
    },

    getTransfersQuota: function() {
        return this._getQuota(this.transferQuotaView);
    },

    setTransfersQuota: function(value) {
        this._setQuota(this.transferQuotaView, value);
    },

    getMaxDownloadsPerSession: function() {
        return this.downloadPerSessionEnabled.is(":checked")
               ? parseInt(this.downloadPerSession.numberspinner("getValue"))
               : null;
    },

    setMaxDownloadsPerSession: function(value) {
        value = parseInt(value);
        if (!isNaN(value)) {
            this.downloadPerSessionEnabled.prop("checked", true);
            this.downloadPerSession.numberspinner("setValue", value);
        } else {
            this.downloadPerSessionEnabled.prop("checked", false);
            this.downloadPerSession.numberspinner("reset");
        }
        this._adjustDownloadPerSession();
    },

    getMaxUploadsPerSession: function() {
        return this.uploadPerSessionEnabled.is(":checked")
               ? parseInt(this.uploadPerSession.numberspinner("getValue"))
               : null;
    },

    setMaxUploadsPerSession: function(value) {
        value = parseInt(value);
        if (!isNaN(value)) {
            this.uploadPerSessionEnabled.prop("checked", true);
            this.uploadPerSession.numberspinner("setValue", value);
        } else {
            this.uploadPerSessionEnabled.prop("checked", false);
            this.uploadPerSession.numberspinner("reset");
        }
        this._adjustUploadPerSession();
    },

    _getQuota: function(quotaView) {
        return quotaView.isAllowed()
                ? new jscape.Quota(quotaView.getSize(), quotaView.getResetPeriod())
                : null;
    },

    _setQuota: function(quotaView, quota) {
        if (quota != null) {
            quotaView.setAllowed(true);
            quotaView.setSize(quota.bytes);
            quotaView.setResetPeriod(quota.resetPeriod);
        } else {
            quotaView.setAllowed(false);
            quotaView.setSize(null);
            quotaView.setResetPeriod(null);
        }
    },

    _adjustDownloadPerSession: function() {
        var enabled = this.downloadPerSessionEnabled.is(":checked");
        this.downloadPerSession.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
    },

    _adjustUploadPerSession: function() {
        var enabled = this.uploadPerSessionEnabled.is(":checked");
        this.uploadPerSession.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
    }
});
/** @const */
jscape.domain.UserQuotaView.LISTENER = {};

/** @class */
jscape.domain.QuotaView = Class.extend(
    /** @lends {jscape.domain.QuotaView.prototype} */
{
    /** @const */
    SIZE_FACTOR: 1024 * 1024,
    PERIOD_FACTOR: 24 * 60 * 60 * 1000,
    DEFAULT_SIZE: 1000,
    DEFAULT_PERIOD: 1,

    /** @constructor */
    initialize: function(allowedInput, sizeInput, resetPeriodInput) {
        this.allowed = allowedInput;
        this.allowed.click($.proxy(this._onClick$Allowed, this));
        this.allowed.change($.proxy(function() {
            var enabled = this.isAllowed();
            $.each([this.size, this.period], function(_, input) {
                input.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
            });
        }, this));

        this.size = sizeInput;
        this.size.numberspinner({required: true, min: 0, max: 1000000, value: this.DEFAULT_SIZE, increment: 10, disabled: true});

        this.period = resetPeriodInput;
        this.period.numberspinner({required: true, min: 1, max: 1000, value: this.DEFAULT_PERIOD, disabled: true});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.QuotaView.LISTENER;
    },

    isAllowed: function() {
        return this.allowed.is(":checked");
    },

    setAllowed: function(value) {
        this.allowed.prop("checked", !!value).change();
    },

    getSize: function() {
        return parseInt(this.size.numberspinner("getValue")) * this.SIZE_FACTOR;
    },

    setSize: function(value) {
        this.size.numberspinner("setValue", value != null ? value / this.SIZE_FACTOR : this.DEFAULT_SIZE) ;
    },

    getResetPeriod: function() {
        return parseInt(this.period.numberspinner("getValue")) * this.PERIOD_FACTOR;
    },

    setResetPeriod: function(value) {
        this.period.numberspinner("setValue", value != null ? value / this.PERIOD_FACTOR : this.DEFAULT_PERIOD);
    },

    _onClick$Allowed: function() {
        this.listener.onStateChanged(this);
    }
});
jscape.domain.QuotaView.LISTENER = {
    onStateChanged: $.noop
};

/** @class */
jscape.domain.UserIpAccessController = Class.extend(
    /** @lends {jscape.domain.UserIpAccessController.prototype} */
{
    rules: null,

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.UserIpAccessView(resources, container);
        this.addController = new jscape.domain.AddUserIpAccessController(resources);
        this.editController = new jscape.domain.EditUserIpAccessController(resources);
        this.deleteController = new jscape.domain.DeleteUserIpAccessController(resources);

        this._initValidators(resources);
    },

    start: function(ipAccessVerifier) {
        this.rules = ipAccessVerifier ? [].concat(ipAccessVerifier.masks) : [];

        this.view.setRules(this.rules);
        this.view.show(this);
    },

    getIpAccess: function() {
        return new jscape.IpAccessVerifier(this.rules);
    },

    onAddRule: function() {
        this.addController.start().done($.proxy(this._ruleAdded, this));
    },

    onEditRule: function(rule) {
        var index = $.inArray(rule, this.rules);
        if (index != -1) {
            this.editController.start(rule).done($.proxy(this._ruleEdited, this, index));
        }
    },

    onDeleteRule: function(rules) {
        this.deleteController.start(rules).done($.proxy(this._ruleDeleted, this, rules));
    },

    onRuleUp: function(rule) {
        var index = $.inArray(rule, this.rules);
        if (index > 0) {
            this.rules.splice(index - 1, 2, this.rules[index], this.rules[index - 1]);
            this.view.setRules(this.rules);
            this.view.selectRule(rule);
        }
    },

    onRuleDown: function(rule) {
        var index = $.inArray(rule, this.rules);
        if (index != -1 && index < this.rules.length - 1) {
            this.rules.splice(index, 2, this.rules[index + 1], this.rules[index]);
            this.view.setRules(this.rules);
            this.view.selectRule(rule);
        }
    },

    _ruleAdded: function(rule) {
        this.rules.push(rule);
        this.view.setRules(this.rules);
        this.view.selectRule(rule);
    },

    _ruleEdited: function(index, rule) {
        this.rules[index] = rule;
        this.view.setRules(this.rules);
        this.view.selectRule(rule);
    },

    _ruleDeleted: function(rules) {
        var index = -1;
        while (rules.length) {
            index = $.inArray(rules.pop(), this.rules);
            this.rules.splice(index, 1);
        }
        this.view.setRules(this.rules);

        if (this.rules.length) {
            index = Math.max(0, Math.min(index, this.rules.length - 1));
            this.view.selectRule(this.rules[index]);
        }
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("user_ipaccess_mask_nonempty", resources.getString("USERS_IP_ACCESS_ERROR_BAD_MASK"));
        v.rule("user_ipaccess_unique", $.proxy(this._ruleNotExists, this), resources.getString("USERS_IP_ACCESS_ERROR_ALREADY_EXIST"));
    },

    _ruleNotExists: function(value, excludes) {
        return $.grep(this.rules, function(rule) {
            return rule.mask == value && (excludes == undefined || $.inArray(rule.mask, excludes) == -1);
        }).length == 0;
    }
});

/** @class */
jscape.domain.UserIpAccessView = Class.extend(
    /** @lends {jscape.domain.UserIpAccessView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources, container) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=useripaccessfields]", container);

        this.data = $("table[role=grid][aria-label=rules]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            singleSelect: true,
            onDblClick: $.proxy(this._onClick$EditButton, this),
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            columns: [[
                {field: "mask", title: $("thead th:nth-child(1)", this.data).text(), width: 60},
                {field: "allowed", title: $("thead th:nth-child(2)", this.data).text(), width: 15, formatter: $.proxy(this._formatAccess, this), styler: $.proxy(this._styleAccess, this)},
                {field: "reason", title: $("thead th:nth-child(3)", this.data).text(), width: 25}
            ]]
        });

        this.moveUpButton = $("a[role=button][name=moveup]", this.fieldset);
        this.moveUpButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveUpButton, this)});

        this.moveDownButton = $("a[role=button][name=movedown]", this.fieldset);
        this.moveDownButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveDownButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.UserIpAccessView.LISTENER;
        this.selection = [];
        this._adjustButtonsState();
    },

    setRules: function(values) {
        this.selection = [];
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectRule: function(rule) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", rule));
    },

    _formatAccess: function(value) {
        return this.resources.getString(!!value ? "USERS_IP_ACCESS_RULE_ALLOWED" : "USERS_IP_ACCESS_RULE_DENIED");
    },

    _styleAccess: function(value) {
        return !!value ? "color:green" : "color:red";
    },

    _onClick$MoveUpButton: function() {
        this.listener.onRuleUp(this.selection[0]);
    },

    _onClick$MoveDownButton: function() {
        this.listener.onRuleDown(this.selection[0]);
    },

    _onClick$AddButton: function() {
        this.listener.onAddRule();
    },

    _onClick$EditButton: function() {
        this.listener.onEditRule(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteRule(this.selection);
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var index = this.selection.length == 1 ? this.data.datagrid("getRowIndex", this.selection[0]) : -1;
        var count = this.data.datagrid("getRows").length;

        this.moveUpButton.linkbutton(index > 0 && index <= count - 1 ? "enable" : "disable");
        this.moveDownButton.linkbutton(index >= 0 && index < count - 1 ? "enable" : "disable");
        this.editButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length !=0 ? "enable" : "disable");
    }
});
jscape.domain.UserIpAccessView.LISTENER = {
    onAddRule: $.noop,
    onEditRule: $.noop,
    onDeleteRule: $.noop,
    onRuleUp: $.noop,
    onRuleDown: $.noop
};

/** @class */
jscape.domain.AddUserIpAccessController = Class.extend(
    /** @lends {jscape.domain.AddUserIpAccessController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function() {
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.UserIpAccessRuleDialog(this.resources);
        dialog.setTitle(this.resources.getString("USERS_IP_ACCESS_DIALOG_ADD_TITLE"));
        dialog.setAllowed(true);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function (dialog) {
        var rule = new jscape.IpAccessVerifier.IPMask(dialog.getMask(), dialog.getReason(), dialog.getAllowed());
        dialog.hide();

        this.deferred.resolve(rule);
    },

    onCancel: function (dialog) {
        dialog.hide();
        this.deferred.reject();
    }
});

/** @class */
jscape.domain.EditUserIpAccessController = Class.extend(
    /** @lends {jscape.domain.EditUserIpAccessController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(rule) {
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.UserIpAccessRuleDialog(this.resources);
        dialog.setTitle(this.resources.getString("USERS_IP_ACCESS_DIALOG_EDIT_TITLE"));
        dialog.setMask(rule.mask);
        dialog.setReason(rule.reason);
        dialog.setAllowed(rule.allowed);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var rule = new jscape.IpAccessVerifier.IPMask(dialog.getMask(), dialog.getReason(), dialog.getAllowed());
        dialog.hide();

        this.deferred.resolve(rule);
    },

    onCancel: function(dialog) {
        dialog.hide();
        this.deferred.reject();
    }
});

/** @class */
jscape.domain.DeleteUserIpAccessController = Class.extend(
    /** @lends {jscape.domain.DeleteUserIpAccessController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(rules) {
        var text = $.map(rules, function(rule) { return rule.mask; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("USERS_IP_ACCESS_CONFIRM_DELETE_TITLE"),
                this.resources.getString("USERS_IP_ACCESS_CONFIRM_DELETE_MESSAGE", {0: text}));
    }
});

/** @class */
jscape.domain.UserIpAccessRuleDialog = Class.extend(
    /** @lends {jscape.domain.UserIpAccessRuleDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-user-ipaccess-rule").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=ipaccessrulefields]", this.dialog);

        this.mask = $("input[name=mask]", this.fieldset);
        this.mask.validatebox({required: true, validType: ["user_ipaccess_mask_nonempty", "user_ipaccess_unique"], missingMessage: resources.getString("USERS_IP_ACCESS_ERROR_BAD_MASK")});

        this.reason = $("input[name=reason]", this.fieldset);
        this.allowed = $("input[name=allowed]:radio", this.fieldset);
    },

    setTitle: function(title) {
        this.dialog.dialog("setTitle", title);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.mask.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    getMask: function () {
        return this.mask.val();
    },

    setMask: function (value) {
        this.mask.val(value);
        if (value && value.trim().length) {
            this.mask.validatebox("options").validType = ["user_ipaccess_mask_nonempty", "user_ipaccess_unique['" + value + "']"];
        }
    },

    getReason: function () {
        return this.reason.val();
    },

    setReason: function (value) {
        this.reason.val(value);
    },

    getAllowed: function () {
        return (/^true$/i).test(this.allowed.filter(":checked").val());
    },

    setAllowed: function (value) {
        this.allowed.filter("[value=" + Boolean(value) + "]").prop("checked", true);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.DomainAdministrationController = Class.extend(
    /** @lends {jscape.domain.DomainAdministrationController.prototype} */
{
    /** @constructor */
    initialize: function(resources, container, vfsController) {
        this.view = new jscape.domain.DomainAdministrationView(resources, container);
        this.vfsController = vfsController;
    },

    start: function(managementService, administration, availableTemplates) {
        this.managementService = managementService;
        this.administration = administration || null;
        this.avalableTemplates = availableTemplates || [];
        this.view.show(this);

        this._setupView();
    },

    getDomainAdministration: function() {
        if (!this.view.isDomainAdministrationAllowed()) {
            return null;
        }
        return new jscape.DomainAdministration(this.view.getMaxOwnedUsers(),
                                               this.view.getManageNonOwnedUsersAllowed(),
                                               this.view.getGroupAssignmentAllowed(),
                                               this.view.getProxyMappingAllowed(),
                                               this.view.getRestrictionPath(),
                                               this.view.getShareQuotas(),
                                               this.view.getPhoneAuthSettingsAllowed(),
                                               this.view.getUrlBrandingAllowed(),
                                               this.view.getMaxUrlBranding(),
                                               this.view.getDropZoneAllowed(),
                                               this.view.getMaxDropZones(),
                                               this.view.getManagePublicContacts(),
                                               this.view.getSystemPgpKeysAvailable(),
                                               this.view.getTemplates());
    },

    onAdministrationAllowed: function(dialog) {
        var descriptors = this.vfsController.getDescriptors();
        for (var i = 0; i < descriptors.length; ++i) {
            var descriptor = descriptors[i];
            if (descriptor.path === "/" && descriptor.realPath) {
                this.view.setRestrictionPath(descriptor.realPath);
                return;
            }
        }
    },

    onSelectPath: function(dialog) {
        var loadFn = $.proxy(this.managementService.listUsersRestrictPath, this.managementService);
        var createFn = $.proxy(this.managementService.createUsersRestrictPath, this.managementService);
        var c = new SelectPathController(loadFn, createFn);
        c.selectDirectory($.proxy(function(entry) { dialog.setRestrictionPath(entry.path); }, this));
    },

    _setupView: function() {
        if (this.administration != null) {
            this.view.setMaxOwnedUsers(this.administration.maxOwnedCount);
            this.view.setManageNonOwnedUsersAllowed(this.administration.manageNonOwned);
            this.view.setGroupAssignmentAllowed(this.administration.allowGroupAssignment);
            this.view.setProxyMappingAllowed(this.administration.allowResourceMapping);
            this.view.setRestrictionPath(this.administration.restrictDir);
            this.view.setShareQuotas(this.administration.shareQuotas);
            this.view.setPhoneAuthSettingsAllowed(this.administration.allowSetPhoneAuth);
            this.view.setUrlBrandingAllowed(this.administration.allowUrlBranding);
            this.view.setMaxUrlBranding(this.administration.maxUrlBrandings);
            this.view.setDropZoneAllowed(this.administration.allowDropZone);
            this.view.setMaxDropZones(this.administration.maxDropZones);
            this.view.setManagePublicContacts(this.administration.allowManagePublicContact);
            this.view.setSystemPgpKeysAvailable(this.administration.allowSystemPgpKeyAssignment);
            this.view.setAvailableTemplates(this.avalableTemplates);
            this.view.setTemplates(this.administration.allowedTemplates);
        } else {
            this.view.setDomainAdministrationAllowed(false);
            this.view.setAvailableTemplates(this.avalableTemplates);
        }

        this.view.setDomainAdministrationAllowed(this.administration != null);
    }
});

/** @class */
jscape.domain.DomainAdministrationView = Class.extend(
    /** @lends {jscape.domain.DomainAdministrationView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=administrationfields]", container);

        this.allowed = $("input[name=domainadminallowed]:checkbox", this.fieldset);
        this.allowed.change($.proxy(this._adjustFieldState, this));

        this.manageNonOwned = $("input[name=managenonowned]", this.fieldset);

        this.maxOwnedEnabled = $("input[name=maxowned]:checkbox", this.fieldset);
        this.maxOwnedEnabled.click($.proxy(function() {
            var enabled = this.maxOwnedEnabled.is(":checked");
            this.maxOwned.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        }, this));

        this.maxOwned = $("input[name=maxownedcount]", this.fieldset);
        this.maxOwned.numberspinner({required: true, editable: false, min: 1, max: 1000, value: 1});

        this.systemPgpKeysAvailable = $("input[name=systempgpkeysavailable]", this.fieldset);
        this.managePublicContacts = $("input[name=managepubliccontacts]", this.fieldset);
        this.shareQuotas = $("input[name=shareqoutas]", this.fieldset);

        this.templates = $("table[role=grid][aria-label=templates]", this.fieldset);
        this.templates.datagrid({
            fitColumns:true,
            singleSelect: false,
            checkOnSelect: true,
            selectOnCheck: true,
            remoteSort: false,
            width: 600, height: 90,
            onClickRow: $.proxy(function(index) { if (!this.isDomainAdministrationAllowed()) { this.templates.datagrid("unselectRow", index); }}, this),
            onSortColumn: $.proxy(function() { if (!this.isDomainAdministrationAllowed()) { this.templates.datagrid("getPanel").find(":checkbox").prop("disabled", true); }}, this),
            onLoadSuccess: function(data) { $(this).datagrid("getPanel").find("div.datagrid-header-check input:checkbox").prop("disabled", data.rows.length == 0); },
            columns: [[
                  {field: "allowed", checkbox: true},
                  {field: "template", title: $("thead th:nth-child(1)", this.templates).text(), sortable: true, width: 100, formatter: function(_, row) { return (row || "").escapeXml(); }}
            ]]
        });

        this.restrictPathEnabled = $("input[name=restrictpath]:checkbox", this.fieldset);
        this.restrictPathEnabled.change($.proxy(function() {
            var enabled = this.restrictPathEnabled.is(":checked");
            this.restrictPath.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation").validatebox("validate");
            this.browsePathButton.linkbutton(enabled ? "enable" : "disable");
        }, this));

        this.restrictPath = $("input[name=restrictpathto]", this.fieldset);
        this.restrictPath.validatebox({required: true, disabled: true, tipPosition: "bottom"});

        this.browsePathButton = $("a[role=button][name=browse]", this.fieldset);
        this.browsePathButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$BrowsePathButton, this)});

        this.groupAssignmentAllowed = $("input[name=groupassignmentallowed]", this.fieldset);
        this.proxyMappingAllowed = $("input[name=proxymappingallowed]", this.fieldset);
        this.setPhoneAuthAllowed = $("input[name=setphoneauthallowed]", this.fieldset);

        this.dropZoneAllowed = $("input[name=dropzoneallowed]", this.fieldset);
        this.urlBrandingAllowed = $("input[name=urlbrandingallowed]", this.fieldset);

        this.maxDropZonesEnabled = $("input[name=maxdropzones]:checkbox", this.fieldset);
        this.maxDropZonesEnabled.click($.proxy(function() {
            var enabled = this.maxDropZonesEnabled.is(":checked");
            this.maxDropZones.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        }, this));

        this.maxDropZones = $("input[name=maxdropzonescount]", this.fieldset);
        this.maxDropZones.numberspinner({required: true, min: 1, max: 1000, value: 1});

        this.maxUrlBrandingsEnabled = $("input[name=maxurlbrandings]:checkbox", this.fieldset);
        this.maxUrlBrandingsEnabled.click($.proxy(function() {
            var enabled = this.maxUrlBrandingsEnabled.is(":checked");
            this.maxUrlBrandings.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        }, this));

        this.maxUrlBrandings = $("input[name=maxurlbrandingscount]", this.fieldset);
        this.maxUrlBrandings.numberspinner({required: true, editable: false, min: 1, max: 1000, value: 1});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.DomainAdministrationView.LISTENER;
        this.templates.datagrid("fitColumns");
        this._adjustFieldState();
    },

    isDomainAdministrationAllowed: function() {
        return this.allowed.is(":checked");
    },

    setDomainAdministrationAllowed: function(value) {
        this.allowed.prop("checked", !!value).change();
    },

    getMaxOwnedUsers: function() {
        return this.maxOwnedEnabled.is(":checked") ? this.maxOwned.numberspinner("getValue") : null;
    },

    setMaxOwnedUsers: function(value) {
        if (value != null) {
            this.maxOwnedEnabled.prop("checked", true);
            this.maxOwned.numberspinner("setValue", value);
        } else {
            this.maxOwnedEnabled.prop("checked", false);
            this.maxOwned.numberspinner("reset");
        }
    },

    getManageNonOwnedUsersAllowed: function() {
        return this.manageNonOwned.is(":checked");
    },

    setManageNonOwnedUsersAllowed: function(value) {
        this.manageNonOwned.prop("checked", !!value);
    },

    getGroupAssignmentAllowed: function() {
        return this.groupAssignmentAllowed.is(":checked");
    },

    setGroupAssignmentAllowed: function(value) {
        this.groupAssignmentAllowed.prop("checked", !!value);
    },

    getProxyMappingAllowed: function() {
        return this.proxyMappingAllowed.is(":checked");
    },

    setProxyMappingAllowed: function(value) {
        this.proxyMappingAllowed.prop("checked", !!value);
    },

    getRestrictionPath: function() {
        return this.restrictPathEnabled.is(":checked:not(:disabled)") ? this.restrictPath.val() : null;
    },

    setRestrictionPath: function(value) {
        if (value != null) {
            this.restrictPathEnabled.prop("checked", true).change();
            this.restrictPath.val(value).focus();
        } else {
            this.restrictPathEnabled.prop("checked", false).change();
            this.restrictPath.val("");
        }
    },

    getShareQuotas: function() {
        return this.shareQuotas.is(":checked");
    },

    setShareQuotas: function(value) {
        this.shareQuotas.prop("checked", !!value);
    },

    getPhoneAuthSettingsAllowed: function() {
        return this.setPhoneAuthAllowed.is(":checked");
    },

    setPhoneAuthSettingsAllowed: function(value) {
        this.setPhoneAuthAllowed.prop("checked", !!value);
    },

    getUrlBrandingAllowed: function() {
        return this.urlBrandingAllowed.is(":checked");
    },

    setUrlBrandingAllowed: function(value) {
        this.urlBrandingAllowed.prop("checked", !!value);
    },

    getMaxUrlBranding: function() {
        return this.maxUrlBrandingsEnabled.is(":checked") ? this.maxUrlBrandings.numberspinner("getValue") : null;
    },

    setMaxUrlBranding: function(value) {
        if (value != null) {
            this.maxUrlBrandingsEnabled.prop("checked", true);
            this.maxUrlBrandings.numberspinner("setValue", value);
        } else {
            this.maxUrlBrandingsEnabled.prop("checked", false);
            this.maxUrlBrandings.numberspinner("reset");
        }
    },

    getDropZoneAllowed: function() {
        return this.dropZoneAllowed.is(":checked");
    },

    setDropZoneAllowed: function(value) {
        this.dropZoneAllowed.prop("checked", !!value);
    },

    getMaxDropZones: function() {
        return this.maxDropZonesEnabled.is(":checked") ? this.maxDropZones.numberspinner("getValue") : null;
    },

    setMaxDropZones: function(value) {
        if (value != null) {
            this.maxDropZonesEnabled.prop("checked", true);
            this.maxDropZones.numberspinner("setValue", value);
        } else {
            this.maxDropZonesEnabled.prop("checked", false);
            this.maxDropZones.numberspinner("reset");
        }
    },

    getManagePublicContacts: function() {
        return this.managePublicContacts.is(":checked")
    },

    setManagePublicContacts: function(value) {
        this.managePublicContacts.prop("checked", !!value);
    },

    getSystemPgpKeysAvailable: function() {
        return this.systemPgpKeysAvailable.is(":checked")
    },

    setSystemPgpKeysAvailable: function(value) {
        this.systemPgpKeysAvailable.prop("checked", !!value);
    },

    setAvailableTemplates: function(values) {
        this.templates.datagrid("loading");
        this.templates.datagrid("loadData", values);
        this.templates.datagrid("loaded");
    },

    getTemplates: function () {
        return this.templates.datagrid("getChecked");
    },

    setTemplates: function (values) {
        if (values && values.length) {
            var rows = this.templates.datagrid("getRows");
            for (var i = 0; i < values.length; i++) {
                var index = $.inArray(values[i], rows);
                this.templates.datagrid("checkRow", index);
            }
        } else {
            this.templates.datagrid("clearChecked");
        }
    },

    _onClick$BrowsePathButton: function() {
        this.listener.onSelectPath(this);
    },

    _adjustFieldState: function() {
        var allowed = this.isDomainAdministrationAllowed();
        this.fieldset.find(":input:checkbox").not(this.allowed).prop("disabled", !allowed);

        var ownedEnabled = allowed && this.maxOwnedEnabled.is(":checked");
        var pathEnabled = allowed && this.restrictPathEnabled.is(":checked");
        var brandingEnabled = allowed && this.maxUrlBrandingsEnabled.is(":checked");
        var zoneEnabled = allowed && this.maxDropZonesEnabled.is(":checked");

        this.maxOwned.numberspinner(ownedEnabled ? "enable" : "disable").numberspinner(ownedEnabled ? "enableValidation" : "disableValidation");
        this.restrictPath.prop("disabled", !pathEnabled).validatebox(pathEnabled ? "enableValidation" : "disableValidation");
        this.browsePathButton.linkbutton(pathEnabled ? "enable" : "disable");
        this.maxDropZones.numberspinner(zoneEnabled ? "enable" : "disable").numberspinner(allowed ? "enableValidation" : "disableValidation");
        this.maxUrlBrandings.numberspinner(brandingEnabled ? "enable" : "disable").numberspinner(allowed ? "enableValidation" : "disableValidation");

        this._initRestrictionPath();

        this.fieldset.form("validate");
    },

    _initRestrictionPath: function() {
        if (this.isDomainAdministrationAllowed() && this.restrictPath.val().trim().length == 0) {
            this.listener.onAdministrationAllowed(this);
        }
    }
});
jscape.domain.DomainAdministrationView.LISTENER = {
    onSelectPath: $.noop
};

/** @class */
jscape.domain.WebOptionsController = Class.extend(
    /** @lends {jscape.domain.WebOptionsController.prototype} */
{
    options: null,

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.WebOptionsView(resources, container);
    },

    start: function(webOptions) {
        this.options = webOptions || new jscape.WebOptions();
        this.view.show(this);

        this._setupView();
    },

    getWebOptions: function() {
        return new jscape.WebOptions(this.view.getLoginRedirection(),
                                     this.view.getShowPublicKeyAuthentication(),
                                     this.view.getShowPgpEncryption(),
                                     this.view.getShowQuotas(),
                                     this.view.getShowContacts(),
                                     this.view.getShowAdHocActivity(),
                                     this.view.getShowDropZones(),
                                     this.view.getPreferAftpForApplet());
    },

    _setupView: function() {
        this.view.setLoginRedirection(this.options.loginRedirection);
        this.view.setShowPublicKeyAuthentication(this.options.webPublicKeyAuthenticationAvailable);
        this.view.setShowPgpEncryption(this.options.webOpenPgpEncryptionAvailable);
        this.view.setShowQuotas(this.options.webQuotasAvailable);
        this.view.setShowContacts(this.options.webContactsAvailable);
        this.view.setShowAdHocActivity(this.options.webAdHocActivityAvailable);
        this.view.setShowDropZones(this.options.webDropZonesAvailable);
        this.view.setPreferAftpForApplet(this.options.webAftpProtocolPreferable)
    }
});

/** @class */
jscape.domain.WebOptionsView = Class.extend(
    /** @lends {jscape.domain.WebOptionsView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=weboptionsfields]", container);

        this.loginRedirection = $("select[name=loginredirect]", this.fieldset);
        this.loginRedirection.combobox({required: true, editable: false, panelHeight: "auto"});

        this.showPublicKeyAuth = $("input[name=showpubkeyauth]", this.fieldset);
        this.showPgpEncryption = $("input[name=showpgpencrypt]", this.fieldset);
        this.showQuotas = $("input[name=showquotas]", this.fieldset);
        this.showContacts = $("input[name=showcontacts]", this.fieldset);
        this.showAdHocActivity = $("input[name=showadhoc]", this.fieldset);
        this.showDropZones = $("input[name=showdropzones]", this.fieldset);
        this.preferAftpForApplet= $("input[name=preferaftpforapplet]", this.fieldset);
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.WebOptionsView.LISTENER;
    },

    reset: function() {
        this.loginRedirection.combobox("reset");
    },

    getLoginRedirection: function() {
        return this.loginRedirection.combobox("getValue");
    },

    setLoginRedirection: function(value) {
        this.loginRedirection.combobox("setValue", value);
    },

    getShowPublicKeyAuthentication: function() {
        return this.showPublicKeyAuth.is(":checked");
    },

    setShowPublicKeyAuthentication: function(value) {
        this.showPublicKeyAuth.prop("checked", !!value);
    },

    getShowPgpEncryption: function() {
        return this.showPgpEncryption.is(":checked");
    },

    setShowPgpEncryption: function(value) {
        this.showPgpEncryption.prop("checked", !!value);
    },

    getShowQuotas: function() {
        return this.showQuotas.is(":checked");
    },

    setShowQuotas: function(value) {
        this.showQuotas.prop("checked", !!value);
    },

    getShowContacts: function() {
        return this.showContacts.is(":checked");
    },

    setShowContacts: function(value) {
        this.showContacts.prop("checked", !!value);
    },

    getShowAdHocActivity: function() {
        return this.showAdHocActivity.is(":checked");
    },

    setShowAdHocActivity: function(value) {
        this.showAdHocActivity.prop("checked", !!value);
    },

    getShowDropZones: function() {
        return this.showDropZones.is(":checked");
    },

    setShowDropZones: function(value) {
        this.showDropZones.prop("checked", !!value);
    },

    getPreferAftpForApplet: function() {
        return this.preferAftpForApplet.is(":checked");
    },

    setPreferAftpForApplet: function(value) {
        this.preferAftpForApplet.prop("checked", !!value);
    }
});
jscape.domain.WebOptionsView.LISTENER = {};