jscape.domain = jscape.domain || {};

/** @class */
jscape.AccountTemplate = Class.extend(
    /** @lends {jscape.AccountTemplate.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e, f, g, h, i, j, k, l, m, n, o) {
        this.name = a || "";
        this.username = b || "";
        this.emailAddress = c || "";
        this.company = d || null;
        this.phone = e || null;
        this.owner = f || "";
        this.expirationDate = g ||null;
        this.enabled = h != undefined ? !!h : true;
        this.secured = i || false;
        this.groups = j || [];
        this.bindedKeys = k || [];
        this.passwordChangingAllowed = l || false;
        this.emailFileTransferAllowed = m || false;
        this.usePhoneAuthentication = n || false;
        this.ignorePasswordAgingRules = o || false;
        this.resources = [];
        this.quotas = new jscape.AccountTransferQuotas();
        this.ipAccessVerifier = new jscape.IpAccessVerifier();
        this.administration = null;
        this.webOptions = new jscape.WebOptions();
    },

    toJSON: function () {
        return $.extend({
                name: this.name,
                username: this.username,
                emailAddress: this.emailAddress,
                company: this.company,
                phone: this.phone,
                owner: this.owner,
                expirationDate: this.expirationDate ? this.expirationDate.getTime() : null,
                enabled: this.enabled,
                secured: this.secured,
                groups: this.groups,
                bindedKeys: this.bindedKeys,
                resources: this.resources,
                passwordChangingAllowed: this.passwordChangingAllowed,
                emailFileTransferAllowed: this.emailFileTransferAllowed,
                usePhoneAuthentication: this.usePhoneAuthentication,
                ignorePasswordAgingRules: this.ignorePasswordAgingRules,
                ipAccessVerifier: this.ipAccessVerifier,
                administration: this.administration
            },
            $.parseJSON(JSON.stringify(this.quotas)),
            $.parseJSON(JSON.stringify(this.webOptions))
        );
    }
});
jscape.AccountTemplate.fromJSON = function(data) {
    return $.extend(new jscape.AccountTemplate(), data, {
        phone: jscape.Phone.fromJSON(data.phone),
        groups: $.isArray(data.groups) ? data.groups : [],
        expirationDate: $.isNumeric(data.expirationDate) ? new Date(data.expirationDate) : null,
        resources: $.map(data.resources || [], jscape.VirtualFile.fromJSON),
        quotas: jscape.AccountTransferQuotas.fromJSON(data),
        ipAccessVerifier: jscape.IpAccessVerifier.fromJSON(data.ipAccessVerifier),
        administration: jscape.DomainAdministration.fromJSON(data.administration),
        webOptions: jscape.WebOptions.fromJSON(data)
    });
};

/** @class */
jscape.domain.AccountTemplatesController = Class.extend(
    /** @lends {jscape.domain.AccountTemplatesController.prototype} */
{
    templateNames: null,

    /** @constructor */
    initialize: function(resources) {
        this.templateNames = [];

        this.view = new jscape.domain.AccountTemplatesView(resources);
        this.addController = new jscape.domain.AddAccountTemplateController(resources);
        this.editController = new jscape.domain.EditAccountTemplateController(resources);
        this.deleteController = new jscape.domain.DeleteAccountTemplateController(resources);

        this._initValidators(resources);
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onAddTemplate: function() {
        this.addController.start(this.managementService, this.domainName)
                .done($.proxy(this._templateAdded, this));
    },

    onEditTemplate: function(templateName) {
        var index = $.inArray(templateName, this.templateNames);
        if (index != -1) {
            this.editController.start(this.managementService, this.domainName, templateName)
                    .done($.proxy(this._templateEdited, this, index));
        }
    },

    onDeleteTemplate: function(templateName) {
        this.deleteController.start(this.managementService, this.domainName, templateName)
                .done($.proxy(this._templateDeleted, this, templateName));
    },

    _templateAdded: function(template) {
        this.templateNames.push(template.name);
        this.view.setTemplates(this.templateNames);
        this.view.selectItem(template.name);
    },

    _templateEdited: function(index, template) {
        this.templateNames[index] = template.name;
        this.view.setTemplates(this.templateNames);
        this.view.selectItem(template.name);
    },

    _templateDeleted: function (templateName) {
        var index = $.inArray(templateName, this.templateNames);
        this.templateNames.splice(index, 1);
        this.view.setTemplates(this.templateNames);

        if (this.templateNames.length) {
            index = Math.min(Math.max(0, index), this.templateNames.length - 1);
            this.view.selectItem(this.templateNames[index]);
        }
    },

    _loadData: function() {
        this.managementService.getTemplates(this.domainName)
                .done($.proxy(function(templateNames) {
                    this.templateNames = templateNames || [];
                    this.view.setTemplates(this.templateNames);
                }, this));
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("usertemplate_name_nonempty", resources.getString("USER_TEMPLATES_ERROR_BAD_NAME"));
        v.rule("usertemplate_unique", $.proxy(this._templateNotExists, this), resources.getString("USER_TEMPLATES_ERROR_ALREADY_EXIST"));
    },

    _templateNotExists: function (value) {
        value = (value || "").toUpperCase();
        return $.grep(this.templateNames, function(name) { return name.toUpperCase() == value; }).length == 0;
    }
});

/** @class */
jscape.domain.AccountTemplatesView = Class.extend(
    /** @lends {jscape.domain.AccountTemplatesView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.selection = [];
        this.fieldset = $("fieldset[name=usertemplatesfields]");

        this.data = $("table[role=grid][aria-label=usertemplates]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            singleSelect: true,
            ctrlSelect: true,
            remoteSort: false,
            pagination: true,
            search: true,
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 300}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this)
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.AccountTemplatesView.LISTENER;
    },

    setTemplates: function(values) {
        var data = $.map(values, function(value) { return {name: value}; });
        this.data.datagrid("loading");
        this.data.datagrid("loadData", data);
        this.data.datagrid("loaded");

        this._resetButtons();
    },

    selectItem: function(value) {
        var rows = this.data.datagrid("getRows");
        for (var i = 0; i < rows.length; i++) {
            if (rows[i].name === value) {
                this.data.datagrid("selectRow", i);
                break;
            }
        }
    },

    _onClick$AddButton: function() {
        this.listener.onAddTemplate();
    },

    _onClick$EditButton: function () {
        this.listener.onEditTemplate(this.template.name);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteTemplate(this.template.name);
    },

    _adjustRowSelection: function(row, template) {
        this.row = row;
        this.template = template;
        this.editButton.linkbutton("enable");
        this.deleteButton.linkbutton(template.name !== "Default" ? "enable" : "disable");
    },

    _resetButtons: function() {
        this.editButton.linkbutton("disable");
        this.deleteButton.linkbutton("disable");
    }
});
jscape.domain.AccountTemplatesView.LISTENER = {
    onAddTemplate: $.noop,
    onEditTemplate: $.noop,
    onDeleteTemplate: $.noop
};

/** @class */
jscape.domain.AddAccountTemplateController = Class.extend(
    /** @lends {jscape.domain.AddAccountTemplateController.prototype} */
{
    DEFAULT_RESOURCES: [jscape.VirtualFile.local("/", undefined, false, false, true, "%installdir%/users/%domain%/%username%", true)],
    VARIABLES: ["installdir", "domain", "username", "queryattr['name']"],

    infoController: null,
    vfsController: null,
    quotasController: null,
    ipAccessController: null,
    domainAdminController: null,
    webController: null,

    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.variableController = new jscape.VariableController();
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._loadData();

        return this.deferred.promise();
    },

    onAddVariable: function(input) {
        this.variableController.start(input, this.VARIABLES);
    },

    onSubmit: function (dialog) {
        var template = this._createTemplate(dialog);
        dialog.destroy();

        this.managementService.storeTemplate(this.domainName, template)
                .done($.proxy(function() { this.deferred.resolve(template); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this))
                .always($.proxy(this._cleanup, this));
    },

    onCancel: function (dialog) {
        dialog.destroy();
        this._cleanup();
        this.deferred.reject();
    },

    _loadData: function () {
        $.when(this.managementService.getTemplates(this.domainName),
               this.managementService.getAccounts(this.domainName),
               this.managementService.getGroups(this.domainName),
               this.managementService.getClientKeys(this.domainName),
               this.managementService.getClientCertificates(this.domainName))
                .done($.proxy(function (templates, accounts, groups, clientKeys, clientCertificates) {
                    var admins = $.grep(accounts || [], function(account) { return account.administrator; });
                    var groupNames = $.map(groups || [], function(group) { return group.name; });
                    var keyAliases = $.map(clientKeys.concat(clientCertificates), function(key) { return key.alias; });
                    this._dataLoaded(templates, groupNames.sort(), admins, keyAliases);
                }, this));
    },

    _dataLoaded: function (templates, groups, owners, keyAliases) {
        var dialog = new jscape.domain.AddAccountTemplateDialog(this.resources);
        dialog.show(this);

        try {
            this.infoController = new jscape.domain.AddUserTemplateInfoController(this.resources, dialog.container());
            this.vfsController = new jscape.vfs.VirtualFilesController(this.resources, dialog.container());
            this.quotasController = new jscape.domain.QuotasController(this.resources, dialog.container());
            this.ipAccessController = new jscape.domain.UserIpAccessController(this.resources, dialog.container());
            this.domainAdminController = new jscape.domain.DomainAdministrationController(this.resources, dialog.container(), this.vfsController);
            this.webController = new jscape.domain.WebOptionsController(this.resources, dialog.container());

            this.infoController.start(groups, owners, keyAliases);
            this.vfsController.start(this.managementService, this.domainName, this.DEFAULT_RESOURCES);
            this.quotasController.start();
            this.ipAccessController.start();
            this.domainAdminController.start(this.managementService, null, templates);
            this.webController.start();
        } catch (e) {
            dialog.destroy();
        }
    },

    _createTemplate: function (dialog) {
        var template = this.infoController.createAccountTemplate();
        return $.extend(template, {
            resources: this.vfsController.getDescriptors(),
            quotas: this.quotasController.getQuotas(),
            ipAccessVerifier: this.ipAccessController.getIpAccess(),
            administration: this.domainAdminController.getDomainAdministration(),
            webOptions: this.webController.getWebOptions()
        });
    },

    _cleanup: function () {
        this.infoController = null;
        this.vfsController = null;
        this.quotasController = null;
        this.ipAccessController = null;
        this.domainAdminController = null;
        this.webController = null;
    }
});

/** @class */
jscape.domain.EditAccountTemplateController = Class.extend(
    /** @lends {jscape.domain.EditAccountTemplateController.prototype} */
{
    VARIABLES: ["installdir", "domain", "username", "queryattr['name']"],

    infoController: null,
    vfsController: null,
    quotasController: null,
    ipAccessController: null,
    domainAdminController: null,
    webController: null,

    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.variableController = new jscape.VariableController();
    },

    start: function(managementService, domainName, templateName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._loadData(templateName);

        return this.deferred.promise();
    },

    onAddVariable: function(input) {
        this.variableController.start(input, this.VARIABLES);
    },

    onSubmit: function(dialog) {
        var template = this._createTemplate(dialog);
        dialog.destroy();

        this.managementService.storeTemplate(this.domainName, template)
                .done($.proxy(function() { this.deferred.resolve(template); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this))
                .always($.proxy(this._cleanup, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this._cleanup();
        this.deferred.reject();
    },

    _loadData: function (templateName) {
        $.when(this.managementService.getTemplate(this.domainName, templateName),
               this.managementService.getTemplates(this.domainName),
               this.managementService.getAccounts(this.domainName),
               this.managementService.getGroups(this.domainName),
               this.managementService.getClientKeys(this.domainName),
               this.managementService.getClientCertificates(this.domainName))
                .done($.proxy(function (template, templateNames, accounts, groups, clientKeys, clientCertificates) {
                    var admins = $.grep(accounts || [], function(account) { return account.administrator; });
                    var groupNames = $.map(groups || [], function(group) { return group.name; });
                    var keyAliases = $.map(clientKeys.concat(clientCertificates), function(key) { return key.alias; });
                    this._dataLoaded(template, templateNames, groupNames.sort(), admins, keyAliases);
                }, this));
    },

    _dataLoaded: function(template, templateNames, groups, owners, keyAliases) {
        var dialog = new jscape.domain.EditAccountTemplateDialog(this.resources);
        dialog.setTemplateName(template.name);
        dialog.show(this);

        try {
            this.infoController = new jscape.domain.EditUserTemplateInfoController(this.resources, dialog.container());
            this.vfsController = new jscape.vfs.VirtualFilesController(this.resources, dialog.container());
            this.quotasController = new jscape.domain.QuotasController(this.resources, dialog.container());
            this.ipAccessController = new jscape.domain.UserIpAccessController(this.resources, dialog.container());
            this.domainAdminController = new jscape.domain.DomainAdministrationController(this.resources, dialog.container(), this.vfsController);
            this.webController = new jscape.domain.WebOptionsController(this.resources, dialog.container());

            this.infoController.start(template, groups, owners, keyAliases);
            this.vfsController.start(this.managementService, this.domainName, template.resources);
            this.quotasController.start(template.quotas);
            this.ipAccessController.start(template.ipAccessVerifier);
            this.domainAdminController.start(this.managementService, template.administration, templateNames);
            this.webController.start(template.webOptions);
        } catch (e) {
            dialog.destroy();
        }
    },

    _createTemplate: function (dialog) {
        var template = this.infoController.createAccountTemplate();
        return $.extend(template, {
            resources: this.vfsController.getDescriptors(),
            quotas: this.quotasController.getQuotas(),
            ipAccessVerifier: this.ipAccessController.getIpAccess(),
            administration: this.domainAdminController.getDomainAdministration(),
            webOptions: this.webController.getWebOptions()
        });
    },

    _cleanup: function() {
        this.infoController = null;
        this.vfsController = null;
        this.quotasController = null;
        this.ipAccessController = null;
        this.domainAdminController = null;
        this.webController = null;
    }
});

/** @class */
jscape.domain.DeleteAccountTemplateController = Class.extend(
    /** @lends {jscape.domain.DeleteAccountTemplateController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, templateName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._showConfirmDelete(templateName)
                .done($.proxy(this._deleteConfirmed, this, templateName))
                .fail($.proxy(function() { this.deferred.reject(); }, this));

        return this.deferred.promise();
    },

    _showConfirmDelete: function(templateName) {
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("USER_TEMPLATES_CONFIRM_DELETE_TITLE"),
                this.resources.getString("USER_TEMPLATES_CONFIRM_DELETE_MESSAGE", {0: templateName}));
    },

    _deleteConfirmed: function (templateName) {
        var promises = [];
        promises.push(this.managementService.deleteTemplate(this.domainName, templateName));

        $.when.apply(jQuery, promises)
                .done($.proxy(function () { this.deferred.resolve(); }, this))
                .fail($.proxy(function () { this.deferred.reject(); }, this));
    }
});

/** @class */
jscape.domain.AccountTemplateDialog = Class.extend(
    /** @lends {jscape.domain.AccountTemplateDialog.prototype} */
{
    focusInField: null,

    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 800, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.tabs = $("div[role=tablist]", this.dialog);
        this.tabs.tabs({tabPosition: "top", plain: true});

        this.addVariable = $("a[role=button][name=addvar]", this.dialog);
        this.addVariable.linkbutton({disabled: true, onClick: $.proxy(this._onClick$AddVariableButton, this)});

        this.dialog.find(":input").focus($.proxy(function (e) {
            var field = $(e.target);
            if (field.is("[type=text]") || field.is("textarea")) {
                this.addVariable.linkbutton("enable");
                this.focusInField = field;
            } else {
                this.focusInField = null;
                this.addVariable.linkbutton("disable");
            }
        }, this));
    },

    container: function() {
        return this.dialog;
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$AddVariableButton: function () {
        if (this.focusInField && this.focusInField.length) {
            this.addVariable.linkbutton("disable");
            this.listener.onAddVariable(this.focusInField);
        }
    },

    _onSubmit: function () {
        if (this._valid()) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _valid: function () {
        return new jscape.Validator().validateTabs(this.tabs);
    }
});

/** @class */
jscape.domain.AddAccountTemplateDialog = jscape.domain.AccountTemplateDialog.extend(
    /** @lends {jscape.domain.AddAccountTemplateDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-usertemplate-add"), resources);
    }
});

/** @class */
jscape.domain.EditAccountTemplateDialog = jscape.domain.AccountTemplateDialog.extend(
    /** @lends {jscape.domain.EditAccountTemplateDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-usertemplate-edit"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    setTemplateName: function (value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});

/** @class */
jscape.domain.AddUserTemplateInfoController = Class.extend(
    /** @lends {jscape.domain.AddUserTemplateInfoController.prototype} */
{
    /** @constructor */
    initialize: function (resources, container) {
        this.view = new jscape.domain.AddUserTemplateInfoView(resources, container);
    },

    start: function(groups, owners, clientKeys) {
        this.groups = groups;
        this.owners = owners;
        this.clientKeys = clientKeys;
        this.view.show(this);

        this._setupView();
    },

    createAccountTemplate: function() {
        var company = this.view.getCompany().trim().length ? this.view.getCompany() : null;
        var phone = $.isNumeric(this.view.getPhoneCode()) ? new jscape.Phone(this.view.getPhoneCode(), this.view.getPhoneNumber(), this.view.getPhoneExtension()) : null;
        return new jscape.AccountTemplate(this.view.getName(),
                                          this.view.getUsername(),
                                          this.view.getEmail(),
                                          company,
                                          phone,
                                          this.view.getOwner(),
                                          this.view.getExpirationDate(),
                                          this.view.getEnabled(),
                                          this.view.getSecured(),
                                          this.view.getGroups(),
                                          this.view.getClientKeys(),
                                          this.view.getAllowPasswordChanging(),
                                          this.view.getAllowAdHocTransfers(),
                                          this.view.getUsePhoneAuthentication(),
                                          this.view.getIgnorePasswordAgingRules());
    },

    _setupView: function () {
        this.view.setAvailableClientKeys(this.clientKeys);
        this.view.setAvailableGroups(this.groups);
        this.view.setAvailableOwners(this.owners);

        this.view.setEnabled(true);
        this.view.setAllowAdHocTransfers(true);
        this.view.setAllowPasswordChanging(true);
    }
});

/** @class */
jscape.domain.EditUserTemplateInfoController = Class.extend(
    /** @lends {jscape.domain.EditUserTemplateInfoController.prototype} */
{
    /** @constructor */
    initialize: function (resources, container) {
        this.view = new jscape.domain.EditUserTemplateInfoView(resources, container);
    },

    start: function(template, groups, owners, clientKeys) {
        this.template = template;
        this.groups = groups;
        this.owners = owners;
        this.clientKeys = clientKeys;
        this.view.show(this);

        this._setupView();
    },

    createAccountTemplate: function() {
        var company = this.view.getCompany().trim().length ? this.view.getCompany() : null;
        var phone = $.isNumeric(this.view.getPhoneCode()) ? new jscape.Phone(this.view.getPhoneCode(), this.view.getPhoneNumber(), this.view.getPhoneExtension()) : null;
        return new jscape.AccountTemplate(this.template.name,
                                          this.view.getUsername(),
                                          this.view.getEmail(),
                                          company,
                                          phone,
                                          this.view.getOwner(),
                                          this.view.getExpirationDate(),
                                          this.view.getEnabled(),
                                          this.view.getSecured(),
                                          this.view.getGroups(),
                                          this.view.getClientKeys(),
                                          this.view.getAllowPasswordChanging(),
                                          this.view.getAllowAdHocTransfers(),
                                          this.view.getUsePhoneAuthentication(),
                                          this.view.getIgnorePasswordAgingRules());
    },

    _setupView: function () {
        this.view.setAvailableClientKeys(this.clientKeys);
        this.view.setAvailableGroups(this.groups);
        this.view.setAvailableOwners(this.owners);

        this.view.setUsername(this.template.username);
        this.view.setEmail(this.template.emailAddress);
        this.view.setCompany(this.template.company != null ? this.template.company.name : "");
        if (this.template.phone != null) {
            this.view.setPhoneCode(this.template.phone.code);
            this.view.setPhoneNumber(this.template.phone.number);
            this.view.setPhoneExtension(this.template.phone.extension);
        }
        this.view.setClientKeys(this.template.bindedKeys);
        this.view.setGroups(this.template.groups);
        this.view.setOwner(this.template.owner);
        this.view.setExpirationDate(this.template.expirationDate);
        this.view.setEnabled(this.template.enabled);
        this.view.setSecured(this.template.secured);
        this.view.setUsePhoneAuthentication(this.template.usePhoneAuthentication);
        this.view.setAllowAdHocTransfers(this.template.emailFileTransferAllowed);
        this.view.setAllowPasswordChanging(this.template.passwordChangingAllowed);
        this.view.setIgnorePasswordAgingRules(this.template.ignorePasswordAgingRules);
    }
});

/** @class */
jscape.domain.AddUserTemplateInfoView = jscape.domain.UserInfoView.extend(
    /** @lends {jscape.domain.AddUserTemplateInfoView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=templateinfoaddfields]", container), resources);

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["usertemplate_name_nonempty", "usertemplate_unique"], missingMessage: resources.getString("USER_TEMPLATES_ERROR_BAD_NAME")});
    },

    getName: function () {
        return this.name.val();
    }
});

/** @class */
jscape.domain.EditUserTemplateInfoView = jscape.domain.UserInfoView.extend(
    /** @lends {jscape.domain.EditUserTemplateInfoView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=templateinfoeditfields]", container), resources);
    }
});