jscape.domain = jscape.domain || {};

/** @class */
jscape.AccountInfo = Class.extend(
    /** @lends {jscape.AccountInfo.prototype} */
{
    /** @constructor */
    initialize: function (a, b, c, d, e, f) {
        this.name = a || "";
        this.login = b || "";
        this.owner = c || "";
        this.company = d || "";
        this.administrator = !!e || false;
        this.lastLoginDate = parseInt(f) || null;
    }
});
jscape.AccountInfo.fromJSON = function(data) {
    return data ? $.extend(new jscape.AccountInfo(), data) : null;
};

/** @class */
jscape.Phone = Class.extend(
    /** @lends {jscape.Phone.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.code = a || "";
        this.number = b || "";
        this.extension = c;
    }
});
jscape.Phone.fromJSON = function(data) {
    return data && data.code && String(data.code).trim().length ? new jscape.Phone(data.code, data.number, data.extension) : null;
};

/** @class */
jscape.Account = Class.extend(
    /** @lends {jscape.Account.prototype} */
{
    /** @constructor */
    initialize: function() {
        this.username = "";
        this.login = "";
        this.emailAddress = "";
        this.company = null;
        this.phone = null;
        this.owner = "";
        this.expirationDate = null;
        this.enabled = true;
        this.secured = false;
        this.groups = [];
        this.bindedKeys = [];
        this.resources = [];
        this.passwordChangingAllowed = false;
        this.emailFileTransferAllowed = false;
        this.usePhoneAuthentication = false;
        this.ignorePasswordAgingRules = false;
        this.quotas = new jscape.AccountTransferQuotas();
        this.ipAccessVerifier = new jscape.IpAccessVerifier();
        this.administration = null;
        this.webOptions = new jscape.WebOptions();
    },

    toJSON: function () {
        return $.extend({},
                        this,
                        {quotas: undefined, webOptions: undefined},
                        $.parseJSON(JSON.stringify(this.quotas)),
                        $.parseJSON(JSON.stringify(this.webOptions)),
                        {expirationDate: this.expirationDate ? this.expirationDate.getTime() : null});
    }
});
jscape.Account.fromJSON = function(data) {
    return $.extend(new jscape.Account(), data, {
        phone: jscape.Phone.fromJSON(data.phone),
        groups: $.isArray(data.groups) ? data.groups : [],
        expirationDate: $.isNumeric(data.expirationDate) ? new Date(data.expirationDate) : null,
        resources: $.map(data.resources || [], jscape.VirtualFile.fromJSON),
        ipAccessVerifier: jscape.IpAccessVerifier.fromJSON(data.ipAccessVerifier),
        administration: jscape.DomainAdministration.fromJSON(data.administration),
        quotas: jscape.AccountTransferQuotas.fromJSON(data),
        webOptions: jscape.WebOptions.fromJSON(data)
    });
};

/** @class */
jscape.domain.UsersModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.UsersModule.prototype} */
{
    onCreate: function() {
        this._super.apply(this, arguments);

        this.accountsController = new jscape.domain.AccountsController(this.resources);
        this.templatesController = new jscape.domain.AccountTemplatesController(this.resources);
    },

    onStart: function () {
        this.accountsController.start(this.managementService, this.domainName);
        this.templatesController.start(this.managementService, this.domainName);
    },

    onResume: function() {
        this.accountsController._loadData();
        this.templatesController._loadData();
    }
});

/** @class */
jscape.domain.AccountsController = Class.extend(
    /** @lends {jscape.domain.AccountsController.prototype} */
{
    accounts: null,

    /** @constructor */
    initialize: function(resources) {
        this.accounts = [];
        this.view = new jscape.domain.AccountsView(resources);
        this.addController = new jscape.domain.AddAccountController(resources);
        this.editController = new jscape.domain.EditAccountController(resources);
        this.deleteController = new jscape.domain.DeleteAccountController(resources);
        this.copyController = new jscape.domain.CopyAccountController(resources);
        this.checkQuotaController = new jscape.domain.CheckQuotaController(resources);
        this.reportController = new jscape.domain.UserReportController(resources);

        this._initValidators(resources);
    },

    /** @override */
    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    onAddAccount: function() {
        this.addController.start(this.managementService, this.domainName)
                .done($.proxy(this._accountAdded, this));
    },

    onCopyAccount: function(account) {
        this.copyController.start(this.managementService, this.domainName, account.login)
                .done($.proxy(this._accountAdded, this));
    },

    onEditAccount: function(account) {
        var index = $.inArray(account, this.accounts);
        if (index != -1) {
            this.editController.start(this.managementService, this.domainName, account.login)
                    .done($.proxy(this._accountEdited, this, index));
        }
    },

    onDeleteAccount: function(accounts) {
        this.deleteController.start(this.managementService, this.domainName, accounts)
                .done($.proxy(this._accountDeleted, this, accounts));
    },

    onCheckAccountQuotas: function(account) {
        this.checkQuotaController.start(this.managementService, this.domainName, account.login);
    },

    onAccountsReport: function() {
        this.reportController.start(this.managementService, this.domainName);
    },

    _accountAdded: function(account) {
        this.accounts.push(account);
        this.view.setAccounts(this.accounts);
        this.view.selectAccount(account);
    },

    _accountEdited: function(index, account) {
        this.accounts[index] = account;
        this.view.setAccounts(this.accounts);
        this.view.selectAccount(account);
    },

    _accountDeleted: function (accounts) {
        var index = -1;
        while (accounts.length) {
            index = $.inArray(accounts.pop(), this.accounts);
            this.accounts.splice(index, 1);
        }
        this.view.setAccounts(this.accounts);

        if (this.accounts.length) {
            index = Math.min(Math.max(0, index), this.accounts.length - 1);
            this.view.selectAccount(this.accounts[index]);
        }
    },

    _loadData: function() {
        this.managementService.getAccounts(this.domainName)
                .done($.proxy(function(accounts) {
                    this.accounts = accounts || [];
                    this.view.setAccounts(this.accounts);
                }, this));
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("user_login_nonempty", resources.getString("USERS_ERROR_BAD_LOGIN"));
        v.rule("user_unique", $.proxy(this._accountNotExists, this), resources.getString("USERS_ERROR_ALREADY_EXIST"));
        v.rule("user_password_compliance", $.proxy(this._passwordCompliant, this), resources.getString("USERS_ERROR_PASSWORD_NON_COMPLIANT"));
        v.rule("user_password_confirm", $.proxy(this._passwordConfirmed, this), resources.getString("USERS_ERROR_CONFIRM_PWD"));
    },

    _accountNotExists: function (login) {
        login = (login || "").toUpperCase();
        for (var i = 0; i < this.accounts.length; ++i) {
            if (this.accounts[i].login.toUpperCase() === login) {
                return false;
            }
        }
        return true;
    },

    _passwordCompliant: function (value, params) {
        var accountName = params && params.length ? params[0] : null;
        var ret = this.managementService.testPasswordCompliance(this.domainName, accountName, value);
        return ret && ret.trim() == "true";
    },

    _passwordConfirmed: function(value, params) {
        return params && params.length && $(params[0]).val() == value;
    }
});

/** @class */
jscape.domain.AccountsView = Class.extend(
    /** @lends {jscape.domain.AccountsView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.selection = [];

        this.fieldset = $("fieldset[name=usersfields]");

        this.data = $("table[role=grid][aria-label=users]", this.fieldset);
        this.data.datagrid({
            singleSelect: true,
            remoteSort: false,
            fitColumns: true,
            pagination: true,
            search: true,
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), width: 100, sortable: true},
                {field: "login", title: $("thead th:nth-child(2)", this.data).text(), width: 80, sortable: true},
                {field: "lastLoginDate", title: $("thead th:nth-child(3)", this.data).text(), width: 110, sortable: true, align: "right", formatter: function(value) { return $.isNumeric(value) ? new Date(value).toLocaleString() : ""; }},
                {field: "owner", title: $("thead th:nth-child(4)", this.data).text(), width: 80, sortable: true},
                {field: "company", title: $("thead th:nth-child(5)", this.data).text(), width: 120, sortable: true, sorter: function(a, b) { return a == b ? 0 : a == null || a < b ? -1 : 1; }},
                {field: "administrator", title: $("thead th:nth-child(6)", this.data).text(), width: 40, sortable: true,  formatter: function(value) { return value ? "yes" : ""; }, styler: function(value) { return value ? "color:green;font-weight:bold" : ""; }}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this)
        });

        this.reportButton = $("a[role=button][name=report]", this.fieldset);
        this.reportButton.linkbutton({onClick: $.proxy(this._onClick$ReportButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});

        this.copyButton = $("a[role=button][name=copy]", this.fieldset);
        this.copyButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$CopyButton, this)});

        this.quotasButton = $("a[role=button][name=quotas]", this.fieldset);
        this.quotasButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$QuotasButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.AccountsView.LISTENER;
        this._adjustButtonsState();
    },

    setAccounts: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectAccount: function(value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    _onClick$AddButton: function() {
        this.listener.onAddAccount();
    },

    _onClick$EditButton: function () {
        this.listener.onEditAccount(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteAccount(this.selection);
    },

    _onClick$CopyButton: function() {
        this.listener.onCopyAccount(this.selection[0]);
    },

    _onClick$QuotasButton: function() {
        this.listener.onCheckAccountQuotas(this.selection[0]);
    },

    _onClick$ReportButton: function() {
        this.listener.onAccountsReport();
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var singleSelection = this.selection.length == 1;
        this.editButton.linkbutton(singleSelection ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
        this.copyButton.linkbutton(singleSelection ? "enable" : "disable");
        this.quotasButton.linkbutton(singleSelection ? "enable" : "disable");
    }
});
jscape.domain.AccountsView.LISTENER = {
    onAddAccount: $.noop,
    onCopyAccount: $.noop,
    onEditAccount: $.noop,
    onDeleteAccount: $.noop,
    onCheckAccountQuotas: $.noop,
    onAccountsReport: $.noop
};

/** @class */
jscape.domain.AddAccountController = Class.extend(
    /** @lends {jscape.domain.AddAccountController.prototype} */
{
    DEFAULT_TEMPLATE_NAME: "Default",

    infoController: null,
    vfsController: null,
    quotasController: null,
    ipAccessController: null,
    domainAdminController: null,
    webController: null,
    groups: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this.managementService.getTemplates(this.domainName).done($.proxy(this._selectTemplate, this));

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var account = this._createAccount(dialog);
        if (!this._rootPathPresent(account)) {
            this._showMessageRootNotFound();
        } else {
            dialog.destroy();
            this.managementService.createAccount(this.domainName, account)
                    .done($.proxy(function (info) { this.deferred.resolve(info); }, this))
                    .fail($.proxy(function () { this.deferred.reject(); }, this))
                    .always($.proxy(this._cleanup, this));
        }
    },

    onCancel: function (dialog) {
        dialog.destroy();
        this._cleanup();
        this.deferred.reject();
    },

    _selectTemplate: function(templateNames) {
        var dialog = new jscape.domain.UserTemplateDialog(this.resources);
        dialog.setTemplates(templateNames.sort());
        dialog.selectItem(this.DEFAULT_TEMPLATE_NAME);
        dialog.show({
            onSubmit: $.proxy(this._templateSelected, this),
            onCancel: $.proxy(this.onCancel, this)
        });
    },

    _templateSelected: function(dialog) {
        var template = dialog.getTemplate();
        dialog.destroy();
        this._loadData(template);
    },

    _loadData: function (templateName) {
        $.when(this.managementService.getTemplate(this.domainName, templateName),
               this.managementService.getAccounts(this.domainName),
               this.managementService.getTemplates(this.domainName),
               this.managementService.getGroups(this.domainName),
               this.managementService.getClientKeys(this.domainName),
               this.managementService.getClientCertificates(this.domainName))
                .done($.proxy(function (template, accounts, templates, groups, clientKeys, clientCertificates) {
                    this.groups = groups || [];
                    var admins = $.grep(accounts, function(account) { return account.administrator;});
                    var groupNames = $.map(this.groups, function(group) { return group.name; });
                    var keyAliases = $.map(clientKeys.concat(clientCertificates), function(key) { return key.alias; });
                    this._dataLoaded(template, templates, groupNames.sort(), admins, keyAliases);
                }, this));
    },

    _dataLoaded: function (template, templates, groups, owners, keyAliases) {
        var dialog = new jscape.domain.AddAccountDialog(this.resources);
        dialog.show(this);

        try {
            this.infoController = new jscape.domain.AddUserInfoController(this.resources, dialog.container());
            this.vfsController = new jscape.vfs.VirtualFilesController(this.resources, dialog.container());
            this.quotasController = new jscape.domain.QuotasController(this.resources, dialog.container());
            this.ipAccessController = new jscape.domain.UserIpAccessController(this.resources, dialog.container());
            this.domainAdminController = new jscape.domain.DomainAdministrationController(this.resources, dialog.container(), this.vfsController);
            this.webController = new jscape.domain.WebOptionsController(this.resources, dialog.container());

            this.infoController.start(template, groups, owners, keyAliases);
            this.vfsController.start(this.managementService, this.domainName, template.resources);
            this.quotasController.start(template.quotas);
            this.ipAccessController.start(template.ipAccessVerifier);
            this.domainAdminController.start(this.managementService, template.administration, templates);
            this.webController.start(template.webOptions);
        } catch (e) {
            dialog.destroy();
        }
    },

    _createAccount: function (dialog) {
        var account = this.infoController.createAccount();
        return $.extend(account, {
            resources: this.vfsController.getDescriptors(),
            quotas: this.quotasController.getQuotas(),
            ipAccessVerifier: this.ipAccessController.getIpAccess(),
            administration: this.domainAdminController.getDomainAdministration(),
            webOptions: this.webController.getWebOptions()
        });
    },

    _rootPathPresent: function(account) {
        var v = new jscape.Validator();
        if (v.isSatisfied("vfs_root_found", account.resources)) {
            return true;
        }
        for (var i = 0; i < account.groups.length; i++) {
            var groupName = account.groups[i];
            for (var j = 0; j < this.groups.length; j++) {
                var group = this.groups[j];
                if (group.name == groupName && v.isSatisfied("vfs_root_found", group.resources)) {
                    return true;
                }
            }
        }
        return false;
    },

    _showMessageRootNotFound: function() {
        $.messager.alert(this.resources.getString("APPLICATION_ERROR_TITLE"),
                         this.resources.getString("VFS_ERROR_ROOT_PATH_NOT_FOUND"),
                         "error");
    },

    _cleanup: function () {
        this.infoController = null;
        this.vfsController = null;
        this.quotasController = null;
        this.ipAccessController = null;
        this.domainAdminController = null;
        this.webController = null;
    }
});

/** @class */
jscape.domain.EditAccountController = Class.extend(
    /** @lends {jscape.domain.EditAccountController.prototype} */
{
    infoController: null,
    vfsController: null,
    quotasController: null,
    ipAccessController: null,
    domainAdminController: null,
    webController: null,
    groups: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, accountName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._loadData(accountName);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var account = this._createAccount(dialog);
        if (!this._rootPathPresent(account)) {
            this._showMessageRootNotFound();
        } else {
            dialog.destroy();
            this.managementService.updateAccount(this.domainName, account)
                    .done($.proxy(function (info) { this.deferred.resolve(info); }, this))
                    .fail($.proxy(function () { this.deferred.reject(); }, this))
                    .always($.proxy(this._cleanup, this));
        }
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this._cleanup();
        this.deferred.reject();
    },

    _loadData: function (accountName) {
        $.when(this.managementService.getAccount(this.domainName, accountName),
               this.managementService.getAccounts(this.domainName),
               this.managementService.getTemplates(this.domainName),
               this.managementService.getGroups(this.domainName),
               this.managementService.getClientKeys(this.domainName),
               this.managementService.getClientCertificates(this.domainName))
                .done($.proxy(function (account, accounts, templates, groups, clientKeys, clientCertificates) {
                    this.groups = groups || [];
                    var admins = $.grep(accounts, function(account) { return account.administrator;});
                    var groupNames = $.map(this.groups, function(group) { return group.name; });
                    var keyAliases = $.map(clientKeys.concat(clientCertificates), function(key) { return key.alias; });
                    this._dataLoaded(account, templates, groupNames.sort(), admins, keyAliases);
                }, this));
    },

    _dataLoaded: function(account, templates, groups, owners, keyAliases) {
        var dialog = new jscape.domain.EditAccountDialog(this.resources);
        dialog.setAccountName(account.login);
        dialog.show(this);

        try {
            this.infoController = new jscape.domain.EditUserInfoController(this.resources, dialog.container());
            this.vfsController = new jscape.vfs.VirtualFilesController(this.resources, dialog.container());
            this.quotasController = new jscape.domain.QuotasController(this.resources, dialog.container());
            this.ipAccessController = new jscape.domain.UserIpAccessController(this.resources, dialog.container());
            this.domainAdminController = new jscape.domain.DomainAdministrationController(this.resources, dialog.container(), this.vfsController);
            this.webController = new jscape.domain.WebOptionsController(this.resources, dialog.container());

            this.infoController.start(this.managementService, this.domainName, account, groups, owners, keyAliases);
            this.vfsController.start(this.managementService, this.domainName, account.resources);
            this.quotasController.start(account.quotas);
            this.ipAccessController.start(account.ipAccessVerifier);
            this.domainAdminController.start(this.managementService, account.administration, templates);
            this.webController.start(account.webOptions);
        } catch (e) {
            dialog.destroy();
        }
    },

    _createAccount: function (dialog) {
        var account = this.infoController.createAccount();
        return $.extend(account, {
            resources: this.vfsController.getDescriptors(),
            quotas: this.quotasController.getQuotas(),
            ipAccessVerifier: this.ipAccessController.getIpAccess(),
            administration: this.domainAdminController.getDomainAdministration(),
            webOptions: this.webController.getWebOptions()
        });
    },

    _rootPathPresent: function(account) {
        var v = new jscape.Validator();
        if (v.isSatisfied("vfs_root_found", account.resources)) {
            return true;
        }
        for (var i = 0; i < account.groups.length; i++) {
            var groupName = account.groups[i];
            for (var j = 0; j < this.groups.length; j++) {
                var group = this.groups[j];
                if (group.name == groupName && v.isSatisfied("vfs_root_found", group.resources)) {
                    return true;
                }
            }
        }
        return false;
    },

    _showMessageRootNotFound: function() {
        $.messager.alert(this.resources.getString("APPLICATION_ERROR_TITLE"),
                         this.resources.getString("VFS_ERROR_ROOT_PATH_NOT_FOUND"),
                         "error");
    },

    _cleanup: function() {
        this.infoController = null;
        this.vfsController = null;
        this.quotasController = null;
        this.ipAccessController = null;
        this.domainAdminController = null;
        this.webController = null;
    }
});

/** @class */
jscape.domain.DeleteAccountController = Class.extend(
    /** @lends {jscape.domain.DeleteAccountController.prototype} */
{
    users: [],
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, users) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.users = users || [];
        this.deferred = $.Deferred();

        this._showConfirmDelete().done($.proxy(this._deleteConfirmed, this)).fail(this.deferred.reject);

        return this.deferred.promise();
    },

    _showConfirmDelete: function() {
        var text = $.map(this.users, function(user) { return user.login; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("USERS_CONFIRM_DELETE_TITLE"),
                this.resources.getString("USERS_CONFIRM_DELETE_MESSAGE", {0: text}));
    },

    _deleteConfirmed: function () {
        var promises = new Array(this.users.length);
        for (var i = 0; i < this.users.length; i++) {
            promises[i] = this.managementService.deleteAccount(this.domainName, this.users[i].login);
        }

        $.when.apply(jQuery, promises)
                .done($.proxy(function () { this.deferred.resolve(this.users); }, this))
                .fail(this.deferred.reject);
    }
});

/** @class */
jscape.domain.CopyAccountController = Class.extend(
    /** @lends {jscape.domain.CopyAccountController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, accountName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.accountName = accountName;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.CopyAccountDialog(this.resources);
        dialog.setAccountName(this.accountName);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var newLogin = dialog.getLogin();
        dialog.destroy();

        this.managementService.copyAccount(this.domainName, this.accountName, newLogin)
                .done(this.deferred.resolve)
                .fail(this.deferred.reject)
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    }
});

/** @class */
jscape.domain.CheckQuotaController = Class.extend(
    /** @lends {jscape.domain.CheckQuotaController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, accountName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.accountName = accountName;

        managementService.getAccountQuotas(domainName, accountName)
                .done($.proxy(this._showQuotas, this, accountName));
    },

    _showQuotas: function(accountName, quotas) {
        var dialog = new jscape.domain.CheckQuotaDialog(this.resources);
        dialog.setAccountName(accountName);
        dialog.setDownloadsQuota(quotas ? quotas.downloadsQuota : null);
        dialog.setUploadsQuota(quotas ? quotas.uploadsQuota : null);
        dialog.setTransfersQuota(quotas ? quotas.transfersQuota : null);
        dialog.show();
    }
});

/** @class */
jscape.domain.UserTemplateDialog = Class.extend(
    /** @lends {jscape.domain.UserTemplateDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-user-add-usertemplate").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=usertemplatefileds]", this.dialog);
        this.template = $("select[name=template]", this.fieldset);
        this.template.combobox({required: true, editable: false, width: 170, panelHeight: "auto"})
                .combobox("autocomplete", true);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this.fieldset.form("validate");
        this.template.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    getTemplate: function() {
        return this.template.combobox("getValue");
    },

    setTemplates: function(values) {
        var data = $.map(values, function(value) { return {text: value, value: value}; });
        this.template.combobox("loadData", data);
    },

    selectItem: function(value) {
        this.template.combobox("select", value);
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.AccountDialog = Class.extend(
    /** @lends {jscape.domain.AccountDialog.prototype} */
{
    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 800, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.tabs = $("div[role=tablist]", this.dialog);
        this.tabs.tabs({tabPosition: "top", plain: true});
    },

    container: function() {
        return this.dialog;
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this._valid()) {
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _valid: function () {
        return new jscape.Validator().validateTabs(this.tabs);
    }
});

/** @class */
jscape.domain.AddAccountDialog = jscape.domain.AccountDialog.extend(
    /** @lends {jscape.domain.EditAccountDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-user-add"), resources);
    }
});

/** @class */
jscape.domain.EditAccountDialog = jscape.domain.AccountDialog.extend(
    /** @lends {jscape.domain.EditAccountDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-user-edit"), resources);

        this.title = this.dialog.dialog("header").text() || "";
    },

    setAccountName: function (value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    }
});

/** @class */
jscape.domain.CopyAccountDialog = Class.extend(
    /** @lends {jscape.domain.CopyAccountDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-user-copy").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.title = this.dialog.dialog("header").text();
        this.fieldset = $("fieldset[name=copyuserfileds]", this.dialog);

        this.login = $("[name=login]", this.fieldset);
        this.login.validatebox({
            required: true,
            validType: ["user_login_nonempty", "user_unique"],
            missingMessage: resources.getString("USERS_ERROR_BAD_LOGIN")
        });
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this.fieldset.form("validate");
        this.login.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    getLogin: function() {
        return this.login.val();
    },

    setAccountName: function(value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.CheckQuotaDialog = Class.extend(
    /** @lends {jscape.domain.CheckQuotaDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.dialog = $("#d-user-quota").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 450, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [{text: resources.getString("DIALOG_BUTTON_CLOSE") || "Close", handler: $.proxy(this._onClose, this)}],
            onClose: $.proxy(this._onClose, this)
        }).show();

        this.fieldset = $("fieldset[name=userquotafields]", this.dialog);

        this.downloadQuota = $("[aria-label=dloadquota]", this.fieldset);
        this.uploadQuota = $("[aria-label=uploadquota]", this.fieldset);
        this.transferQuota = $("[aria-label=transferquota]", this.fieldset);

        this.title = this.dialog.dialog("header").text();
    },

    show: function () {
        this.dialog.dialog("center");
        this.dialog.dialog("open");
    },

    setAccountName: function(value) {
        this.dialog.dialog("setTitle", this.title.supplant({0: value}));
    },

    setDownloadsQuota: function(quota) {
        this.downloadQuota.prop("readonly", false);
        this.downloadQuota.text(this._formatQuota(quota));
        this.downloadQuota.prop("readonly", true);
    },

    setUploadsQuota: function(quota) {
        this.uploadQuota.prop("readonly", false);
        this.uploadQuota.text(this._formatQuota(quota));
        this.uploadQuota.prop("readonly", true);
    },

    setTransfersQuota: function(quota) {
        this.transferQuota.prop("readonly", false);
        this.transferQuota.text(this._formatQuota(quota));
        this.transferQuota.prop("readonly", true);
    },

    _formatQuota: function(quota) {
        if (quota != null) {
            var f = new jscape.SizeFormat();
            return this.resources.getString("USERS_QUOTA_TEMPLATE").supplant({0: f.format(quota.transferredBytes), 1: f.format(quota.bytes)});
        }
        return this.resources.getString("USERS_QUOTA_NONE");
    },

    _onClose: function () {
        this.dialog.dialog("destroy");
    }
});

/** @class */
jscape.domain.AddUserInfoController = Class.extend(
    /** @lends {jscape.domain.AddUserInfoController.prototype} */
{
    /** @constructor */
    initialize: function (resources, container) {
        this.view = new jscape.domain.AddUserInfoView(resources, container);
    },

    start: function(template, groups, owners, clientKeys) {
        this.groups = groups;
        this.owners = owners;
        this.clientKeys = clientKeys;
        this.view.show(this);

        this._setupView(template);
    },

    createAccount: function() {
        var account = new jscape.Account();
        account.login = this.view.getLogin();
        account.password = this.view.getPassword();
        account.username = this.view.getUsername();
        account.emailAddress = this.view.getEmail();
        account.company = this.view.getCompany() ? this.view.getCompany() : null;
        account.phone = this.view.getPhoneCode() != null || this.view.getPhoneNumber() != null ? new jscape.Phone(this.view.getPhoneCode(), this.view.getPhoneNumber(), this.view.getPhoneExtension()) :  null;
        account.bindedKeys = this.view.getClientKeys();
        account.groups = this.view.getGroups();
        account.owner = this.view.getOwner();
        account.expirationDate = this.view.getExpirationDate();
        account.enabled = this.view.getEnabled();
        account.secured = this.view.getSecured();
        account.usePhoneAuthentication = this.view.getUsePhoneAuthentication();
        account.passwordChangingAllowed = this.view.getAllowPasswordChanging();
        account.emailFileTransferAllowed = this.view.getAllowAdHocTransfers();
        account.ignorePasswordAgingRules = this.view.getIgnorePasswordAgingRules();
        return account;
    },

    _setupView: function (template) {
        this.view.setAvailableClientKeys(this.clientKeys);
        this.view.setAvailableGroups(this.groups);
        this.view.setAvailableOwners(this.owners);

        this.view.setUsername(template.username);
        this.view.setEmail(template.emailAddress);
        this.view.setCompany(template.company != null ? template.company.name : "");
        if (template.phone != null) {
            this.view.setPhoneCode(template.phone.code);
            this.view.setPhoneNumber(template.phone.number);
            this.view.setPhoneExtension(template.phone.extension);
        }
        this.view.setClientKeys(template.bindedKeys);
        this.view.setGroups(template.groups);
        this.view.setOwner(template.owner);
        this.view.setExpirationDate(template.expirationDate);
        this.view.setEnabled(template.enabled);
        this.view.setSecured(template.secured);
        this.view.setUsePhoneAuthentication(template.usePhoneAuthentication);
        this.view.setAllowAdHocTransfers(template.emailFileTransferAllowed);
        this.view.setAllowPasswordChanging(template.passwordChangingAllowed);
        this.view.setIgnorePasswordAgingRules(template.ignorePasswordAgingRules);
    }
});

/** @class */
jscape.domain.EditUserInfoController = Class.extend(
    /** @lends {jscape.domain.EditUserInfoController.prototype} */
{
    /** @constructor */
    initialize: function (resources, container) {
        this.resources = resources;
        this.view = new jscape.domain.EditUserInfoView(resources, container);
        this.passwordController = new jscape.domain.ChangePasswordController(resources);
    },

    start: function(managementService, domainName, account, groups, owners, clientKeys) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.account = account;
        this.groups = groups;
        this.owners = owners;
        this.clientKeys = clientKeys;
        this.view.show(this);

        this._setupView();
    },

    onChangePassword: function() {
        this.passwordController.start(this.managementService, this.domainName, this.account.login)
                .done($.proxy(function() {
                    $.messager.alert(this.resources.getString("USERS_CHANGE_PASSWORD_TITLE"),
                                     this.resources.getString("USERS_CHANGE_PASSWORD_MESSAGE"),
                                     "info");
                }, this));
    },

    createAccount: function() {
        var account = $.extend(new jscape.Account(), this.account);
        account.username = this.view.getUsername();
        account.emailAddress = this.view.getEmail();
        account.company = this.view.getCompany() ? this.view.getCompany() : null;
        account.phone = $.isNumeric(this.view.getPhoneCode()) ? new jscape.Phone(this.view.getPhoneCode(), this.view.getPhoneNumber(), this.view.getPhoneExtension()) : null;
        account.bindedKeys = this.view.getClientKeys();
        account.groups = this.view.getGroups();
        account.owner = this.view.getOwner();
        account.expirationDate = this.view.getExpirationDate();
        account.enabled = this.view.getEnabled();
        account.secured = this.view.getSecured();
        account.usePhoneAuthentication = this.view.getUsePhoneAuthentication();
        account.passwordChangingAllowed = this.view.getAllowPasswordChanging();
        account.emailFileTransferAllowed = this.view.getAllowAdHocTransfers();
        account.ignorePasswordAgingRules = this.view.getIgnorePasswordAgingRules();
        return account;
    },

    _setupView: function () {
        this.view.setAvailableClientKeys(this.clientKeys);
        this.view.setAvailableGroups(this.groups);
        this.view.setAvailableOwners(this.owners);

        this.view.setUsername(this.account.username);
        this.view.setEmail(this.account.emailAddress);
        this.view.setCompany(this.account.company != null ? this.account.company.name : "");
        if (this.account.phone != null) {
            this.view.setPhoneCode(this.account.phone.code);
            this.view.setPhoneNumber(this.account.phone.number);
            this.view.setPhoneExtension(this.account.phone.extension);
        }
        this.view.setClientKeys(this.account.bindedKeys);
        this.view.setGroups(this.account.groups);
        this.view.setOwner(this.account.owner);
        this.view.setExpirationDate(this.account.expirationDate);
        this.view.setEnabled(this.account.enabled);
        this.view.setSecured(this.account.secured);
        this.view.setUsePhoneAuthentication(this.account.usePhoneAuthentication);
        this.view.setAllowAdHocTransfers(this.account.emailFileTransferAllowed);
        this.view.setAllowPasswordChanging(this.account.passwordChangingAllowed);
        this.view.setIgnorePasswordAgingRules(this.account.ignorePasswordAgingRules);
    }
});


/** @class */
jscape.domain.AddUserInfoView = jscape.domain.UserInfoView.extend(
    /** @lends {jscape.domain.AddUserInfoView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=userinfoaddfields]", container), resources);

        this.login = $("input[name=login]", this.fieldset);
        this.login.validatebox({required: true, validType: ["user_login_nonempty", "user_unique"], missingMessage: resources.getString("USERS_ERROR_BAD_LOGIN")});

        this.password = $("input[name=password]", this.fieldset).attr("id", "pwd_" + new Date().getTime().toString(36));
        this.password.validatebox({delay: 2000, validType: "user_password_compliance", missingMessage: resources.getString("USERS_ERROR_PASSWORD_NON_COMPLIANT")});
        this.password.blur(function() { $(this).validatebox("validate"); });
        this.password.on("input", $.proxy(function() { this.confirmPassword.validatebox({required: this.getPassword().length != 0}); }, this));
        this.confirmPassword = $("input[name=confirmpwd]", this.fieldset);
        this.confirmPassword.validatebox({validType: "user_password_confirm['#" + this.password.attr("id") + "']", missingMessage: resources.getString("USERS_ERROR_CONFIRM_PWD")});
    },

    /** @override */
    show: function(listener) {
        this._adjustPasswordRequirements();
        this._super(listener);
    },

    getLogin: function () {
        return this.login.val();
    },

    getPassword: function() {
        return this.password.val();
    },

    _adjustPasswordRequirements: function() {
        var rule = this.password.validatebox("options").validType;
        if (!new jscape.Validator().isSatisfied(rule, "")) {
            this.password.validatebox({required: true});
        } else {
            this.password.validatebox({required: false});
        }
    }
});

/** @class */
jscape.domain.EditUserInfoView = jscape.domain.UserInfoView.extend(
    /** @lends {jscape.domain.EditUserInfoView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this._super($("fieldset[name=userinfoeditfields]", container), resources);

        this.changePasswordButton = $("a[role=button][name=changepwd]", this.fieldset);
        this.changePasswordButton.linkbutton({onClick: $.proxy(this._onClick$ChangePasswordButton, this)});
    },

    /** @override */
    show: function(listener) {
        this._super(listener || jscape.domain.EditUserInfoView.LISTENER);
    },

    _onClick$ChangePasswordButton: function() {
        this.listener.onChangePassword(this);
    }
});
jscape.domain.EditUserInfoView.LISTENER = {
    onChangePassword: $.noop
};

/** @class */
jscape.domain.ChangePasswordController = Class.extend(
    /** @lends {jscape.domain.ChangePasswordController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, accountName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.accountName = accountName;
        this.deferred = $.Deferred();

        this._setupDialog().show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var password = dialog.getPassword();
        dialog.destroy();

        this.managementService.changeAccountPassword(this.domainName, this.accountName, password)
                .done($.proxy(function() { this.deferred.resolve(); this.deferred = null; }, this))
                .fail($.proxy(function() { this.deferred.reject(); this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function() {
        var dialog = new jscape.domain.ChangePasswordDialog(this.resources);
        dialog.setAccountName(this.accountName);
        return dialog;
    }
});

/** @class */
jscape.domain.ChangePasswordDialog = Class.extend(
    /** @lends {jscape.domain.ChangePasswordDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-user-changepwd").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=userpasswordfields]", this.dialog);

        this.password = $("input[name=password]", this.fieldset).attr("id", "newpwd_" + new Date().getTime().toString(36));
        this.password.validatebox({delay: 2000, missingMessage: resources.getString("USERS_ERROR_PASSWORD_NON_COMPLIANT")});
        this.password.blur(function() { $(this).validatebox("validate"); });
        this.password.on("input", $.proxy(function() { this.confirmPassword.validatebox({required: this.getPassword().length != 0}); }, this));

        this.confirmPassword = $("input[name=confirmpwd]", this.fieldset);
        this.confirmPassword.validatebox({validType: "user_password_confirm['#" + this.password.attr("id") + "']", missingMessage: resources.getString("USERS_ERROR_CONFIRM_PWD")});
    },

    setAccountName: function(value) {
        this._adjustPasswordRequirements(value);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.password.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    getPassword: function() {
        return this.password.val();
    },

    _adjustPasswordRequirements: function(accountName) {
        var compliant = new jscape.Validator().isSatisfied("user_password_compliance", "", accountName);
        this.password.validatebox({validType: "user_password_compliance" + (accountName ? "['" + accountName + "']" : ""), required: !compliant});
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.UserReportController = Class.extend(
    /** @lends {jscape.domain.UserReportController.prototype} */
{
    defered: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._selectReportParameters().show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var detailed = dialog.isDetailedView();
        dialog.destroy();
        this._openReport(detailed);
        this.deferred.resolve();
        this.deferred = null;
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.resolve();
        this.deferred = null;
    },

    _openReport: function(detailed) {
        var url = window.location.pathname + "/accounts-report" + (detailed ? "?all=true" : "");
        window.open(url, "jscape-report-users");
    },

    _selectReportParameters: function() {
        return new jscape.domain.UserReportParametersDialog(this.resources);
    }
});

/** @class */
jscape.domain.UserReportParametersDialog = Class.extend(
    /** @lends {jscape.domain.UserReportParametersDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-users-reportparams").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=userreportparametersfields]", this.dialog);

        this.detailed = $("input[name=detailed]:checkbox", this.fieldset);
    },

    isDetailedView: function() {
        return this.detailed.is(":checked");
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});