jscape.vfs = jscape.vfs || {};

/** @class */
jscape.VirtualFile = Class.extend(
    /** @lends {jscape.VirtualFile.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e) {
        this.path = a == "" ? "/" : a;
        this.accessPermissions = b || new jscape.AccessPermissions();
        this.secured = c || false;
        this.denied = d || false;
        this.indexable = e || false;
    },

    isRoot: function() {
        return "/" === this.path;
    }
});
jscape.VirtualFile.local = function(path, permissions, secured, denied, indexable, realPath, create, pgpEncrypt, dlp) {
    return $.extend(new jscape.VirtualFile(path, permissions, secured, denied, indexable), {
        realPath: realPath,
        createIfNotExists: Boolean(create),
        pgpEncryption: pgpEncrypt || new jscape.PgpEncryption(),
        dataLossPrevention: dlp || new jscape.DataLossPrevention()
    });
};
jscape.VirtualFile.remote = function(path, permissions, secured, denied, reverseProxy) {
    return $.extend(new jscape.VirtualFile(path, permissions, secured, denied, false), {
        reverseProxy: reverseProxy,
        toJSON: function() { return $.extend({name: this.reverseProxy}, this); }
    });
};
jscape.VirtualFile.fromJSON = function(data) {
    var permissions = jscape.AccessPermissions.fromJSON(data.accessPermissions);
    return data.name && data.name.trim().length
            ? jscape.VirtualFile.remote(data.path, permissions, data.secured, data.denied, data.name)
            : jscape.VirtualFile.local(data.path, permissions, data.secured, data.denied, data.indexable, data.realPath,
                                       data.createIfNotExists,
                                       data.pgpEncryption ? jscape.PgpEncryption.fromJSON(data.pgpEncryption) : null,
                                       data.dataLossPrevention ? jscape.DataLossPrevention.fromJSON(data.dataLossPrevention) : null);
};

/** @class */
jscape.AccessPermissions = Class.extend(
    /** @lends {jscape.AccessPermissions.prototype} */
{
    FORMAT: {
        fileDownloading: "R", fileUploading: "W", fileAppending: "A", fileDeletion: "D", fileRenaming: "R", fileListing: "L",
        dirCreation: "C", dirDeletion: "D", dirListing: "L", dirBrowsing: "B",
        disabled: "-"
    },

    /** @constructor */
    initialize: function(a, b, c, d, e, f, g, h, i, j) {
        this.fileDownloadingAllowed = a != undefined ? a : true;
        this.fileUploadingAllowed = b != undefined ? b : true;
        this.fileDeletionAllowed = c != undefined ? c : true;
        this.fileAppendingAllowed = d != undefined ? d : true;
        this.fileListingAllowed = e != undefined ? e : true;
        this.fileRenamingAllowed = f != undefined ? f : true;
        this.directoriesListingAllowed = g != undefined ? g : true;
        this.directoryMakingAllowed = h != undefined ? h : true;
        this.directoryDeletionAllowed = i != undefined ? i : true;
        this.subdirectoriesBrowsingAllowed = j != undefined ? j : true;
    },

    format: function () {
        return this._valueOf(this.fileDownloadingAllowed, this.FORMAT.fileDownloading)
               + this._valueOf(this.fileUploadingAllowed, this.FORMAT.fileUploading)
               + this._valueOf(this.fileAppendingAllowed, this.FORMAT.fileAppending)
               + this._valueOf(this.fileDeletionAllowed, this.FORMAT.fileDeletion)
               + this._valueOf(this.fileRenamingAllowed, this.FORMAT.fileRenaming)
               + this._valueOf(this.fileListingAllowed, this.FORMAT.fileListing)
               + this._valueOf(this.directoryMakingAllowed, this.FORMAT.dirCreation)
               + this._valueOf(this.directoryDeletionAllowed, this.FORMAT.dirDeletion)
               + this._valueOf(this.directoriesListingAllowed, this.FORMAT.dirListing)
               + this._valueOf(this.subdirectoriesBrowsingAllowed, this.FORMAT.dirBrowsing);
    },

    _valueOf: function (value, str) {
        return value ? str : this.FORMAT.disabled;
    }
});
jscape.AccessPermissions.fromJSON = function(data) {
    return $.extend(new jscape.AccessPermissions(), data);
};

/** @class */
jscape.PgpEncryption = Class.extend(
    /** @lends {jscape.PgpEncryption.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.keyProvider = a || new jscape.PgpFileKeyProvider();
        this.enabled = b || false;
    }
});
jscape.PgpEncryption.fromJSON = function(data) {
    var provider = data.keyProvider && data.keyProvider.hasOwnProperty("keyAlias")
                   ? new jscape.ServerKeyProvider(data.keyProvider.keyAlias)
                   : new jscape.PgpFileKeyProvider();
    return new jscape.PgpEncryption(provider, !!data.enabled);
};

/** @class */
jscape.PgpFileKeyProvider = jscape.FileKeyProvider.extend(
    /** @lends {jscape.PgpFileKeyProvider.prototype} */
{
    /** @constructor */
    initialize: function() {
        this._super("/.pgp/key.pub", "");
    }
});

/** @class */
jscape.DataLossPrevention = Class.extend(
    /** @lends {jscape.DataLossPrevention.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.rules = $.isArray(a) ? a : [];
        this.enabled = b || false;
    }
});
jscape.DataLossPrevention.fromJSON = function(data) {
    var rules = $.isArray(data.rules) ? $.map(data.rules, function(entry) { return jscape.DataLossPrevention.Entry.fromJSON(entry); }) : [];
    return new jscape.DataLossPrevention(rules, !!data.enabled);
};
/** @class */
jscape.DataLossPrevention.Entry = Class.extend(
    /** @lends {jscape.DataLossPrevention.Entry.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.ruleName = a || null;
        this.access = b || null;
        this.enabled = c || false;
    }
});
jscape.DataLossPrevention.Entry.fromJSON = function(data) {
    return $.extend(new jscape.DataLossPrevention.Entry(), data);
};


/** @class */
jscape.vfs.VirtualFilesController = Class.extend(
    /** @lends {jscape.vfs.VirtualFilesController.prototype} */
{
    paths: [],

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.vfs.VirtualFilesView(resources, container);
        this.addController = new jscape.vfs.AddPathController(resources);
        this.editController = new jscape.vfs.EditPathController(resources);
        this.deleteController = new jscape.vfs.DeletePathController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, domainName, paths) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.paths = [].concat(paths);

        this._setupView();
        this.view.show(this);
    },

    getDescriptors: function() {
        return this.paths;
    },

    onAddPath: function() {
        this.addController.start(this.managementService, this.domainName)
                .done($.proxy(this._pathAdded, this));
    },

    onEditPath: function(path) {
        var index = $.inArray(path, this.paths);
        if (index != -1) {
            this.editController.start(this.managementService, this.domainName, path)
                    .done($.proxy(this._pathEdited, this, index));
        }
    },

    onDeletePath: function(paths) {
        this.deleteController.start(paths)
                .done($.proxy(this._pathDeleted, this, paths));
    },

    onPathUp: function(path) {
        var index = $.inArray(path, this.paths);
        if (index > 0) {
            this.paths.splice(index - 1, 2, this.paths[index], this.paths[index - 1]);
            this.view.setPaths(this.paths);
            this.view.selectPath(path);
        }
    },

    onPathDown: function(path) {
        var index = $.inArray(path, this.paths);
        if (index != -1 && index != this.paths.length - 1) {
            this.paths.splice(index, 2, this.paths[index + 1], this.paths[index]);
            this.view.setPaths(this.paths);
            this.view.selectPath(path);
        }
    },

    _pathAdded: function(path) {
        this.paths.push(path);
        this.view.setPaths(this.paths);
        this.view.selectPath(path);
    },

    _pathEdited: function(index, path) {
        this.paths[index] = path;
        this.view.setPaths(this.paths);
        this.view.selectPath(path);
    },

    _pathDeleted: function (paths) {
        var index;
        while (paths.length) {
            index = $.inArray(paths.pop(), this.paths);
            if (index != -1) {
                this.paths.splice(index, 1);
            }
        }

        this.view.setPaths(this.paths);

        if (this.paths.length) {
            index = Math.max(0, Math.min(index, this.paths.length - 1));
            this.view.selectPath(this.paths[index]);
        }
    },

    _setupView: function() {
        this.view.setPaths(this.paths);
    },

    _initValidators: function (resources) {
        var v = new jscape.Validator();
        v.rule("vfs_unique", $.proxy(this._pathNotExists, this), resources.getString("VFS_ERROR_BAD_PATH"));
        v.rule("vfs_root_found", $.proxy(this._rootPathFound, this), resources.getString("VFS_ERROR_ROOT_PATH_NOT_FOUND"));
        v.nonEmptyRule("vfs_path_nonempty", resources.getString("VFS_ERROR_BAD_PATH"));
        v.nonEmptyRule("vfs_realpath_nonempty", resources.getString("VFS_ERROR_BAD_REAL_PATH"));
    },

    _pathNotExists: function (path) {
        return $.grep(this.paths, function (file) { return file.path == path; }) == 0;
    },

    _rootPathFound: function(paths) {
        return $.grep(paths || [], function(path) { return path.isRoot(); }).length != 0;
    }
});

/** @class */
jscape.vfs.VirtualFilesView = Class.extend(
    /** @lends {jscape.vfs.VirtualFilesView.prototype} */
{
    selection: [],

    /** @constructor */
    initialize: function(resources, container) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=vfsfields]", container);

        this.data = $("table[role=grid][aria-label=paths]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            singleSelect: true,
            columns: [[
                {field: "path", title: $("thead th:nth-child(1)", this.data).text(), resizable: true, sortable: false, width: 60},
                {field: "mapping", title: $("thead th:nth-child(2)", this.data).text(), resizable: true, sortable: false, width: 100, formatter: function(_, row) { return (row.realPath ? row.realPath : (row.reverseProxy ? row.reverseProxy : "")).escapeXml(); }},
                {field: "permissions", title: $("thead th:nth-child(3)", this.data).text(), resizable: false, sortable: false, width: 70, formatter: function(_, row) { return row.accessPermissions.format(); }}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onSortColumn: $.proxy(this._adjustButtonsState, this),
            onLoadSuccess: $.proxy(this._adjustButtonsState, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this)
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});

        this.moveUpButton = $("a[role=button][name=moveup]", this.fieldset);
        this.moveUpButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveUpButton, this)});

        this.moveDownButton = $("a[role=button][name=movedown]", this.fieldset);
        this.moveDownButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveDownButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.vfs.VirtualFilesView.LISTENER;
        this.selection = [];
        this._initState();
    },

    setPaths: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectPath: function(path) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", path));
    },

    _onClick$AddButton: function() {
        this.listener.onAddPath();
    },

    _onClick$EditButton: function() {
        this.listener.onEditPath(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeletePath(this.selection);
    },

    _onClick$MoveUpButton: function() {
        this.listener.onPathUp(this.selection[0]);
    },

    _onClick$MoveDownButton: function() {
        this.listener.onPathDown(this.selection[0]);
    },

    _initState: function() {
        this._adjustButtonsState();
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var index = this.selection.length == 1 ? this.data.datagrid("getRowIndex", this.selection[0]) : -1;
        var rowCount = this.data.datagrid("getRows").length;
        var deleteEnabled = this.selection.length > 0 && new jscape.Validator().isSatisfied("vfs_delete_allowed", this.selection);

        this.moveUpButton.linkbutton(index > 0 && index <= rowCount - 1 ? "enable" : "disable");
        this.moveDownButton.linkbutton(index >= 0 && index < rowCount - 1 ? "enable" : "disable");
        this.editButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length > 0  ? "enable" : "disable");
    }
});
jscape.vfs.VirtualFilesView.LISTENER = {
    onAddPath: $.noop,
    onEditPath: $.noop,
    onDeletePath: $.noop,
    onPathUp: $.noop,
    onPathDown: $.noop
};

/** @class */
jscape.vfs.AddPathController = Class.extend(
    /** @lends {jscape.vfs.AddPathController.prototype} */
{
    VARIABLES: ["installdir", "domain", "sessionid", "username", "group", "queryattr['name']"],

    permissions: null,
    keyProvider: null,
    dlpEntries: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.pgpSettingsController = new jscape.vfs.PgpEncryptSettingsController(resources);
        this.dlpSettingsController = new jscape.vfs.DlpSettingsController(resources);
        this.variableController = new jscape.VariableController();
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.permissions = new jscape.AccessPermissions();
        this.keyProvider = new jscape.FileKeyProvider();
        this.dlpEntries = [];
        this.deferred = $.Deferred();

        var dialog = new jscape.vfs.VirtualPathDialog(this.resources);
        dialog.setTitle(this.resources.getString("VFS_DIALOG_ADD_PATH_TITLE"));
        dialog.setPath("/");
        dialog.setRealPath("");
        dialog.setCreateIfNotFound(true);
        dialog.setIncludeInSearchIndex(true);
        dialog.setSecured(false);
        dialog.setDenied(false);
        dialog.setPgpEncryptUploads(false);
        dialog.setDlpEnabled(false);
        dialog.show(this);

        this.managementService.getReverseProxies(domainName).done(function (items) {
            var names = $.map(items, function (proxy) { return proxy.name; });
            dialog.setReverseProxies(names);
        });

        return this.deferred.promise();
    },

    onAddVariable: function(input) {
        this.variableController.start(input, this.VARIABLES);
    },

    onSelectPath: function(dialog) {
        var loadFn = $.proxy(this.managementService.listFileSystem, this.managementService);
        var createFn = $.proxy(this.managementService.createFileSystemDir, this.managementService);
        var c = new SelectPathController(loadFn, createFn);
        c.selectDirectory($.proxy(function (entry) { dialog.setRealPath(entry.path); }, this));
    },

    onSetPgpEncryptUploadsSettings: function() {
        this.pgpSettingsController.start(this.managementService, this.domainName, this.keyProvider)
                .done($.proxy(function(provider) {
                    this.keyProvider = provider;
                }, this));
    },

    onSetDlpSettings: function() {
        this.dlpSettingsController.start(this.managementService, this.domainName, this.dlpEntries)
                .done($.proxy(function(entries) { this.dlpEntries = entries; }, this));
    },

    onSetPermission: function() {
        var dialog = new jscape.vfs.VirtualPathPermissionsDialog(this.resources);
        dialog.setFileDownloading(this.permissions.fileDownloadingAllowed);
        dialog.setFileUploading(this.permissions.fileUploadingAllowed);
        dialog.setFileDeletion(this.permissions.fileDeletionAllowed);
        dialog.setFileAppending(this.permissions.fileAppendingAllowed);
        dialog.setFileListing(this.permissions.fileListingAllowed);
        dialog.setFileRenaming(this.permissions.fileRenamingAllowed);
        dialog.setDirectoriesListing(this.permissions.directoriesListingAllowed);
        dialog.setDirectoryMaking(this.permissions.directoryMakingAllowed);
        dialog.setDirectoryDeletion(this.permissions.directoryDeletionAllowed);
        dialog.setSubdirectoriesBrowsing(this.permissions.subdirectoriesBrowsingAllowed);
        dialog.show({
            onSubmit: $.proxy(this._onSubmit$Permissions, this),
            onCancel: function(d) { d.destroy(); }
        });
    },

    onSubmit: function(dialog) {
        var file = this._createFile(dialog);
        file.accessPermissions = this.permissions;
        dialog.destroy();

        this.deferred.resolve(file);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _onSubmit$Permissions: function(dialog) {
        this.permissions = this._createAccessPermissions(dialog);
        dialog.destroy();
    },

    _createAccessPermissions: function (dialog) {
        return new jscape.AccessPermissions(dialog.getFileDownloading(),
                                            dialog.getFileUploading(),
                                            dialog.getFileDeletion(),
                                            dialog.getFileAppending(),
                                            dialog.getFileListing(),
                                            dialog.getFileRenaming(),
                                            dialog.getDirectoriesListing(),
                                            dialog.getDirectoryMaking(),
                                            dialog.getDirectoryDeletion(),
                                            dialog.getSubdirectoriesBrowsing());
    },

    _createFile: function(dialog) {
        return dialog.getReverseProxy() != null
               ? jscape.VirtualFile.remote(dialog.getPath(),
                                           this.permissions,
                                           dialog.getSecured(),
                                           dialog.getDenied(),
                                           dialog.getReverseProxy())
                : jscape.VirtualFile.local(dialog.getPath(),
                                           this.permissions,
                                           dialog.getSecured(),
                                           dialog.getDenied(),
                                           dialog.getIncludeInSearchIndex(),
                                           dialog.getRealPath(),
                                           dialog.getCreateIfNotFound(),
                                           new jscape.PgpEncryption(this.keyProvider, dialog.getPgpEncryptUploads()),
                                           new jscape.DataLossPrevention(this.dlpEntries, dialog.getDlpEnabled()));
    }
});

/** @class */
jscape.vfs.EditPathController = Class.extend(
    /** @lends {jscape.vfs.EditPathController.prototype} */
{
    VARIABLES: ["installdir", "domain", "sessionid", "username", "group", "queryattr['name']"],

    permissions: null,
    keyProvider: null,
    dlpEntries: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.pgpSettingsController = new jscape.vfs.PgpEncryptSettingsController(resources);
        this.dlpSettingsController = new jscape.vfs.DlpSettingsController(resources);
        this.variableController = new jscape.VariableController();
    },

    start: function(managementService, domainName, file) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.permissions = file.accessPermissions;
        this.keyProvider = file.pgpEncryption ? file.pgpEncryption.keyProvider : null;
        this.dlpEntries = file.dataLossPrevention ? file.dataLossPrevention.rules : [];
        this.deferred = $.Deferred();

        var dialog = new jscape.vfs.VirtualPathDialog(this.resources);
        dialog.setTitle(this.resources.getString("VFS_DIALOG_EDIT_PATH_TITLE").supplant({0: file.path}));
        dialog.setPath(file.path);
        if (file.realPath) {
            dialog.setRealPath(file.realPath);
            dialog.setCreateIfNotFound(file.createIfNotExists);
            dialog.setPgpEncryptUploads(file.pgpEncryption && file.pgpEncryption.enabled);
            dialog.setDlpEnabled(file.dataLossPrevention && file.dataLossPrevention.enabled);
        } else if (file.reverseProxy) {
            dialog.setReverseProxy(file.reverseProxy);
            dialog.setPgpEncryptUploads(false);
            dialog.setDlpEnabled(false);
        }
        dialog.setIncludeInSearchIndex(file.indexable);
        dialog.setSecured(file.secured);
        dialog.setDenied(file.denied);
        dialog.show(this);

        this.managementService.getReverseProxies(domainName).done(function (items) {
            var names = $.map(items, function (proxy) { return proxy.name; });
            dialog.setReverseProxies(names);
        });

        return this.deferred.promise();
    },

    onSubmit: function (dialog) {
        var file = this._createFile(dialog);
        file.accessPermissions = this.permissions;
        dialog.destroy();

        this.deferred.resolve(file);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    onAddVariable: function(input) {
        this.variableController.start(input, this.VARIABLES);
    },

    onSelectPath: function(dialog) {
        var loadFn = $.proxy(this.managementService.listFileSystem, this.managementService);
        var createFn = $.proxy(this.managementService.createFileSystemDir, this.managementService);
        var c = new SelectPathController(loadFn, createFn);
        c.selectDirectory($.proxy(function (entry) { dialog.setRealPath(entry.path); }, this));
    },

    onSetPgpEncryptUploadsSettings: function() {
        this.pgpSettingsController.start(this.managementService, this.domainName, this.keyProvider)
                .done($.proxy(function(provider) {
                    this.keyProvider = provider;
                }, this));
    },

    onSetDlpSettings: function() {
        this.dlpSettingsController.start(this.managementService, this.domainName, this.dlpEntries)
                .done($.proxy(function(entries) { this.dlpEntries = entries; }, this));
    },

    onSetPermission: function () {
        var dialog = new jscape.vfs.VirtualPathPermissionsDialog(this.resources);
        dialog.setFileDownloading(this.permissions.fileDownloadingAllowed);
        dialog.setFileUploading(this.permissions.fileUploadingAllowed);
        dialog.setFileDeletion(this.permissions.fileDeletionAllowed);
        dialog.setFileAppending(this.permissions.fileAppendingAllowed);
        dialog.setFileListing(this.permissions.fileListingAllowed);
        dialog.setFileRenaming(this.permissions.fileRenamingAllowed);
        dialog.setDirectoriesListing(this.permissions.directoriesListingAllowed);
        dialog.setDirectoryMaking(this.permissions.directoryMakingAllowed);
        dialog.setDirectoryDeletion(this.permissions.directoryDeletionAllowed);
        dialog.setSubdirectoriesBrowsing(this.permissions.subdirectoriesBrowsingAllowed);
        dialog.show({
            onSubmit: $.proxy(this._onSubmit$Permissions, this),
            onCancel: function(dialog) { dialog.destroy(); }
        });
    },

    _onSubmit$Permissions: function(dialog) {
        this.permissions = this._createAccessPermissions(dialog);
        dialog.hide();
    },

    _createAccessPermissions: function(dialog) {
        return new jscape.AccessPermissions(dialog.getFileDownloading(),
                                            dialog.getFileUploading(),
                                            dialog.getFileDeletion(),
                                            dialog.getFileAppending(),
                                            dialog.getFileListing(),
                                            dialog.getFileRenaming(),
                                            dialog.getDirectoriesListing(),
                                            dialog.getDirectoryMaking(),
                                            dialog.getDirectoryDeletion(),
                                            dialog.getSubdirectoriesBrowsing());
    },

    _createFile: function (dialog) {
        return dialog.getReverseProxy() != null
                ? jscape.VirtualFile.remote(dialog.getPath(),
                                            this.permissions,
                                            dialog.getSecured(),
                                            dialog.getDenied(),
                                            dialog.getReverseProxy())
                : jscape.VirtualFile.local(dialog.getPath(),
                                           this.permissions,
                                           dialog.getSecured(),
                                           dialog.getDenied(),
                                           dialog.getIncludeInSearchIndex(),
                                           dialog.getRealPath(),
                                           dialog.getCreateIfNotFound(),
                                           new jscape.PgpEncryption(this.keyProvider, dialog.getPgpEncryptUploads()),
                                           new jscape.DataLossPrevention(this.dlpEntries, dialog.getDlpEnabled()));
    }
});

/** @class */
jscape.vfs.DeletePathController = Class.extend(
    /** @lends {jscape.vfs.DeletePathController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function (files) {
        var text = $.map(files, function(file) { return file.path; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("VFS_CONFIRM_DELETE_TITLE"),
                this.resources.getString("VFS_CONFIRM_DELETE_MESSAGE", {0: text}));
    }
});

/** @class */
jscape.vfs.VirtualPathDialog = Class.extend(
    /** @lends {jscape.vfs.VirtualPathDialog.prototype} */
{
    currentField: null,

    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-vfs-path").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 600, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function() {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=pathfields]", this.dialog);
        this.path = $("input[name=path]", this.fieldset);
        this.path.validatebox({required: true, validaType: ["vfs_unique", "vfs_path_nonempty"], missingMessage: resources.getString("VFS_ERROR_BAD_PATH")});

        this.type = $("input[name=type]:radio", this.fieldset);
        this.type.on("change", $.proxy(this._adjustFieldState, this));
        this.typeRealPath = this.type.filter("[value=realpath]");
        this.typeReverseProxy = this.type.filter("[value=reverseproxy]");

        this.realPath = $("input[name=realpath]", this.fieldset);
        this.realPath.validatebox({required: true, validType: "vfs_realpath_nonempty", missingMessage: resources.getString("VFS_ERROR_BAD_REAL_PATH")});

        this.browseButton = $("a[role=button][name=browse]", this.fieldset);
        this.browseButton.linkbutton({onClick: $.proxy(this._onClick$BrowseButton, this)});

        this.reverseProxy = $("select[name=reverseproxy]", this.fieldset);
        this.reverseProxy.combobox({
            required: true, editable: false, disabled: true, missingMessage: resources.getString("VFS_ERROR_BAD_REVERSE_PROXY"), width: 120, panelHeight: "auto",
            onLoadSuccess: function() {
                var data = $(this).combobox("getData");
                var value = $(this).combobox("getValue") || (data.length && data[0].value);
                value && $(this).combobox("select", value);
            }
        }).combobox("autocomplete", true);

        this.createIfNotFound = $("input[name=createifnotfound]", this.fieldset);
        this.includeInSearchIndex = $("input[name=includeinsearchindex]", this.fieldset);
        this.secured = $("input[name=secured]", this.fieldset);
        this.denied = $("input[name=denied]", this.fieldset);

        this.pgpEncryptUploads = $("input[name=pgpencryptupload]:checkbox", this.fieldset);
        this.pgpEncryptUploads.on("change", $.proxy(this._adjustButtonsState, this));

        this.pgpSettingsButton = $("a[role=button][name=pgpsettings]", this.fieldset);
        this.pgpSettingsButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$PgpSettingsButton, this)});

        this.dlpEnabled = $("input[name=enabledlp]:checkbox", this.fieldset);
        this.dlpEnabled.on("change", $.proxy(this._adjustButtonsState, this));

        this.dlpSettingsButton = $("a[role=button][name=dlpsettings]", this.fieldset);
        this.dlpSettingsButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DlpSettingsButton, this)});

        this.permissionsButton = $("a[role=button][name=permissions]", this.fieldset);
        this.permissionsButton.linkbutton({onClick: $.proxy(this._onClick$PermissionsButton, this)});

        this.addVariableButton = $("a[role=button][name=addvar]", this.dialog);
        this.addVariableButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$AddVariableButton, this)});

        this.fieldset.focusin($.proxy(function(e) {
            var field = $(e.target);
            if (field.is("input[type=text]") || field.is("textarea")) {
                this.addVariableButton.linkbutton("enable");
                this.currentField = field;
            } else {
                this.addVariableButton.linkbutton("disable");
                this.currentField = null;
            }
        }, this));
    },

    show: function (listener) {
        this.currentField = null;
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this._adjustFieldState();
        this._adjustButtonsState();
        this.fieldset.form("validate");
        this.path.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.fieldset.form("disableValidation");
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.currentField = null;
        this.dialog.dialog("destroy");
    },

    setTitle: function(title) {
        this.dialog.dialog("setTitle", title);
    },

    getPath: function() {
        return this.path.val();
    },

    setPath: function(value) {
        this.path.val(value);
    },

    getRealPath: function() {
        return this.realPath.val();
    },

    setRealPath: function(value) {
        this.typeRealPath.prop("checked", true).change();
        this.realPath.val(value).focus();
    },

    setReverseProxies: function (values) {
        if (values && values.length) {
            values = $.map(values, function(value) { return {text: value, value: value}; });
            this.reverseProxy.combobox("loadData", values);
            this.typeReverseProxy.prop("disabled", false);
        } else {
            this.reverseProxy.combobox("disable").combobox("disableValidation");
            this.typeReverseProxy.prop("disabled", true);
        }
    },

    getReverseProxy: function() {
        return this.typeReverseProxy.is(":checked") ? this.reverseProxy.combobox("getValue") : null;
    },

    setReverseProxy: function(value) {
        this.typeReverseProxy.prop("checked", true);
        this.reverseProxy.combobox("setValue", value);
        this._adjustFieldState();
    },

    getCreateIfNotFound: function() {
        return this.createIfNotFound.is(":checked");
    },

    setCreateIfNotFound: function(value) {
        this.createIfNotFound.prop("checked", !!value);
    },

    getIncludeInSearchIndex: function() {
        return this.includeInSearchIndex.is(":checked");
    },

    setIncludeInSearchIndex: function(value) {
        this.includeInSearchIndex.prop("checked", !!value);
    },

    getSecured: function() {
        return this.secured.is(":checked");
    },

    setSecured: function(value) {
        this.secured.prop("checked", !!value);
    },

    getDenied: function() {
        return this.denied.is(":checked");
    },

    setDenied: function(value) {
        this.denied.prop("checked", !!value);
    },

    getPgpEncryptUploads: function() {
        return this.pgpEncryptUploads.is(":checked");
    },

    setPgpEncryptUploads: function(value) {
        this.pgpEncryptUploads.prop("checked", !!value);
        this._adjustButtonsState();
    },

    getDlpEnabled: function() {
        return this.dlpEnabled.is(":checked");
    },

    setDlpEnabled: function(value) {
        this.dlpEnabled.prop("checked", !!value);
        this._adjustButtonsState();
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _onClick$PgpSettingsButton: function() {
        this.listener.onSetPgpEncryptUploadsSettings();
    },

    _onClick$DlpSettingsButton: function() {
        this.listener.onSetDlpSettings();
    },

    _onClick$PermissionsButton: function() {
        this.listener.onSetPermission();
    },

    _onClick$BrowseButton: function() {
        this.listener.onSelectPath(this);
    },

    _onClick$AddVariableButton: function () {
        if (this.currentField != null && this.currentField.length && this.currentField.is(":not(:disabled)")) {
            this.addVariableButton.linkbutton("disable");
            this.listener.onAddVariable(this.currentField);
        }
    },

    _adjustFieldState: function () {
        var enabled = this.typeRealPath.is(":checked");

        this.realPath.prop("disabled", !enabled);
        this.browseButton.linkbutton(enabled ? "enable" : "disable");
        this.createIfNotFound.prop("disabled", !enabled);
        this.includeInSearchIndex.prop("disabled", !enabled);
        this.pgpEncryptUploads.prop("disabled", !enabled);
        this.pgpSettingsButton.linkbutton(enabled && this.getPgpEncryptUploads() ? "enable" : "disable");
        this.dlpEnabled.prop("disabled", !enabled);
        this.dlpSettingsButton.linkbutton(enabled && this.getDlpEnabled() ? "enable" : "disable");

        if (enabled) {
            this.realPath.validatebox("enableValidation").validatebox("isValid");
            this.reverseProxy.combobox("disable").combobox("disableValidation");
        } else if (this.typeReverseProxy.is(":checked")) {
            this.realPath.validatebox("disableValidation");
            this.reverseProxy.combobox("enable").combobox("enableValidation").combobox("isValid");
        }
    },

    _adjustButtonsState: function() {
        this.pgpSettingsButton.linkbutton(this.getPgpEncryptUploads() ? "enable" : "disable");
        this.dlpSettingsButton.linkbutton(this.getDlpEnabled() ? "enable" : "disable");
    }
});
/** @const */
jscape.vfs.VirtualPathDialog.LISTENER = {
    onSelectPath: $.noop,
    onSetPgpEncryptUploadsSettings: $.noop,
    onSetDlpSettings: $.noop,
    onSetPermission: $.noop,
    onSubmit: $.noop,
    onCancel: $.noop
};


/** @class */
jscape.vfs.VirtualPathPermissionsDialog = Class.extend(
    /** @lends {jscape.vfs.VirtualPathPermissionsDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-vfs-path-permissions").clone().attr("id", new Date().getTime().toString(36)) ;
        this.dialog.dialog({
            width: 500, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function() {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=permissionsfields]", this.dialog);

        this.fileDownload = $("input[name=filedownload]", this.fieldset);
        this.fileUpload = $("input[name=fileupload]", this.fieldset);
        this.fileAppend = $("input[name=fileappend]", this.fieldset);
        this.fileDelete = $("input[name=filedelete]", this.fieldset);
        this.fileRename = $("input[name=filerename]", this.fieldset);
        this.fileListing = $("input[name=filelisting]", this.fieldset);
        this.dirListing = $("input[name=dirlisting]", this.fieldset);
        this.dirCreate = $("input[name=dircreate]", this.fieldset);
        this.dirDelete = $("input[name=dirdelete]", this.fieldset);
        this.subdirBrowse = $("input[name=subdirbrowse]", this.fieldset);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    getFileDownloading: function() {
        return this.fileDownload.is(":checked");
    },

    setFileDownloading: function(value) {
        this.fileDownload.prop("checked", !!value);
    },

    getFileUploading: function() {
        return this.fileUpload.is(":checked");
    },

    setFileUploading: function(value) {
        this.fileUpload.prop("checked", !!value);
    },

    getFileAppending: function() {
        return this.fileAppend.is(":checked");
    },

    setFileAppending: function(value) {
        this.fileAppend.prop("checked", !!value);
    },

    getFileDeletion: function() {
        return this.fileDelete.is(":checked");
    },

    setFileDeletion: function(value) {
        this.fileDelete.prop("checked", !!value);
    },

    getFileRenaming: function() {
        return this.fileRename.is(":checked");
    },

    setFileRenaming: function(value) {
        this.fileRename.prop("checked", !!value);
    },

    getFileListing: function() {
        return this.fileListing.is(":checked");
    },

    setFileListing: function(value) {
        this.fileListing.prop("checked", !!value);
    },

    getDirectoriesListing: function() {
        return this.dirListing.is(":checked");
    },

    setDirectoriesListing: function(value) {
        this.dirListing.prop("checked", !!value);
    },

    getDirectoryMaking: function() {
        return this.dirCreate.is(":checked");
    },

    setDirectoryMaking: function(value) {
        this.dirCreate.prop("checked", !!value);
    },

    getDirectoryDeletion: function() {
        return this.dirDelete.is(":checked");
    },

    setDirectoryDeletion: function(value) {
        this.dirDelete.prop("checked", !!value);
    },

    getSubdirectoriesBrowsing: function() {
        return this.subdirBrowse.is(":checked");
    },

    setSubdirectoriesBrowsing: function(value) {
        this.subdirBrowse.prop("checked", !!value);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.vfs.PgpEncryptSettingsController = Class.extend(
    /** @lends {jscape.vfs.PgpEncryptSettingsController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, keyProvider) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._setupDialog(keyProvider).show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var provider = this._createKeyProvider(dialog);
        dialog.destroy();
        this.deferred.resolve(provider);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createKeyProvider: function(dialog) {
        return dialog.getUserKeySelected()
                ? new jscape.PgpFileKeyProvider()
                : new jscape.ServerKeyProvider(dialog.getSystemKey());
    },

    _setupDialog: function(keyProvider) {
        var dialog = new jscape.vfs.PgpEncryptSettingsDialog(this.resources);
        $.when(this.managementService.getPgpPublicKeys(),
               this.managementService.getPgpSecretKeys())
                .done(function(publicKeys, secretKeys) {
                    var aliases = $.map(publicKeys.concat(secretKeys), function(key) { return key.alias; });
                    dialog.setSystemKeys(aliases);

                    if (keyProvider instanceof jscape.ServerKeyProvider) {
                        dialog.setSystemKey(keyProvider.keyAlias);
                    } else {
                        dialog.setUserKeySelected(true);
                    }
                });
        return dialog;
    }
});

/** @class */
jscape.vfs.PgpEncryptSettingsDialog = Class.extend(
    /** @lends {jscape.vfs.PgpEncryptSettingsDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-vfs-pgpencrypt").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 600, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function() {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=pgpencryptfields]", this.dialog);

        this.type = $("input[name=type]:radio", this.fieldset);
        this.typeUserKey = this.type.filter("[value=user]");
        this.typeSystemKey = this.type.filter("[value=system]");
        this.typeUserKey.on("change", $.proxy(this._adjustFieldState, this));
        this.typeSystemKey.on("change", $.proxy(this._adjustFieldState, this));

        this.systemKeys = $("select[name=systemkeys]", this.fieldset);
        this.systemKeys.combobox({
            required: true,
            editable: false,
            disabled: true,
            width: 230, panelHeight: "auto",
            formatter: function(rowData) { var t = rowData[$(this).combobox("options").textField]; return t.escapeXml(); },
            onLoadSuccess: function(data) { var c = $(this); if (data.length) { c.combobox("clear").combobox("select", data[0].value); }}
        });
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this._adjustFieldState();
        this.fieldset.form("validate");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.fieldset.form("disableValidation");
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function () {
        this.dialog.dialog("destroy");
    },

    getUserKeySelected: function() {
        return this.typeUserKey.is(":checked");
    },

    setUserKeySelected: function(value) {
        this.typeUserKey.prop("checked", !!value);
    },

    setSystemKeys: function(values) {
        if (values && values.length) {
            var data = $.map(values, function(value) { return {text: value, value: value}; });
            this.systemKeys.combobox("loadData", data);
            this.typeSystemKey.prop("disabled", false);
        } else {
            this.systemKeys.combobox("disable").combobox("disableValidation");
            this.typeSystemKey.prop("disabled", true);
        }
    },

    getSystemKey: function() {
        return this.systemKeys.combobox("getValue");
    },

    setSystemKey: function(value) {
        this.typeSystemKey.prop("checked", true).change();
        this.systemKeys.combobox("clear").combobox("select", value);
        this._adjustFieldState();
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _adjustFieldState: function () {
        if (this.typeUserKey.is(":checked")) {
            this.systemKeys.combobox("disable").combobox("disableValidation");
        } else if (this.typeSystemKey.is(":checked")) {
            this.systemKeys.combobox("enable").combobox("enableValidation").combobox("isValid");
        }
    }
});

/** @class */
jscape.vfs.DlpSettingsController = Class.extend(
    /** @lends {jscape.vfs.DlpSettingsController.prototype} */
{
    view: null,
    entries: null,
    dlpRuleNames: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.deleteController = new jscape.vfs.DeleteDlpRuleController(resources);
    },

    start: function(managementService, domainName, entries) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.entries = [].concat(entries);
        this.dlpRuleNames = [];
        this.deferred = $.Deferred();
        this._loadDlpRules();

        var dialog = new jscape.vfs.DlpSettingsDialog(this.resources);
        dialog.setEntries(this.entries);
        dialog.show(this);
        this.view = dialog;

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        dialog.destroy();
        this.view = null;
        this.deferred.resolve(this.entries);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.view = null;
        this.deferred.reject();
    },

    onAddEntry: function() {
        var dialog = new jscape.vfs.DlpRuleDialog(this.resources);
        dialog.setTitle(this.resources.getString("VFS_DLP_DIALOG_ADD_RULE_TITLE"));
        dialog.setDlpRules(this._filterDlpRules());
        dialog.setEnabled(true);
        dialog.show({
            onSubmit: $.proxy(this._onSubmit$AddDialog, this),
            onCancel: function(d) { d.destroy(); }
        });
    },

    onEditEntry: function(entry) {
        var index = $.inArray(entry, this.entries);
        if (index != -1) {
            var dialog = new jscape.vfs.DlpRuleDialog(this.resources);
            dialog.setTitle(this.resources.getString("VFS_DLP_DIALOG_EDIT_RULE_TITLE").supplant({0: entry.ruleName}));
            dialog.setDlpRules(this._filterDlpRules(entry.ruleName));
            dialog.setRule(entry.ruleName);
            dialog.setAccess(entry.access);
            dialog.setEnabled(entry.enabled);
            dialog.show({
                onSubmit: $.proxy(this._onSubmit$EditDialog, this, index),
                onCancel: function(d) { d.destroy(); }
            });
        }
    },

    onDeleteEntry: function(entries) {
        this.deleteController.start(entries).done($.proxy(this._entryDeleted, this, entries));
    },

    onEntryUp: function(entry) {
        var index = $.inArray(entry, this.entries);
        if (index > 0) {
            this.entries.splice(index - 1, 2, this.entries[index], this.entries[index - 1]);
            this.view.setEntries(this.entries);
            this.view.selectEntry(entry);
        }
    },

    onEntryDown: function(entry) {
        var index = $.inArray(entry, this.entries);
        if (index != -1 && index != this.entries.length - 1) {
            this.entries.splice(index, 2, this.entries[index + 1], this.entries[index]);
            this.view.setEntries(this.entries);
            this.view.selectEntry(entry);
        }
    },

    _onSubmit$AddDialog: function(dialog) {
        var entry = this._createDlpEntry(dialog);
        dialog.destroy();

        this._entryAdded(entry);
    },

    _onSubmit$EditDialog: function(index, dialog) {
        var entry = this._createDlpEntry(dialog);
        dialog.destroy();

        this._entryEdited(index, entry);
    },

    _createDlpEntry: function(dialog) {
        return new jscape.DataLossPrevention.Entry(dialog.getRule(), dialog.getAccess(), dialog.getEnabled());
    },

    _entryAdded: function(entry) {
        this.entries.push(entry);
        this.view.setEntries(this.entries);
        this.view.selectEntry(entry);
    },

    _entryEdited: function(index, entry) {
        this.entries[index] = entry;
        this.view.setEntries(this.entries);
        this.view.selectEntry(entry);
    },

    _entryDeleted: function (entries) {
        var index;
        while (entries.length) {
            index = $.inArray(entries.pop(), this.entries);
            if (index != -1) {
                this.entries.splice(index, 1);
            }
        }

        this.view.setEntries(this.entries);

        if (this.entries.length) {
            index = Math.max(0, Math.min(index, this.entries.length - 1));
            this.view.selectEntry(this.entries[index]);
        }
    },

    _loadDlpRules: function() {
        this.managementService.getDlpRules(this.domainName).done($.proxy(function(rules) {
            this.dlpRuleNames = $.map(rules, function(rule) { return rule.name; });
        }, this));
    },

    _filterDlpRules: function(exclude) {
        var entryRuleNames = $.map(this.entries, function(entry) { return entry.ruleName; });
        return $.grep(this.dlpRuleNames, function(ruleName) {
            return $.inArray(ruleName, entryRuleNames) == -1 || exclude === ruleName;
        });
    }
});

/** @class */
jscape.vfs.DlpSettingsDialog = Class.extend(
    /** @lends {jscape.vfs.DlpSettingsDialog.prototype} */
{
    selection: [],

    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-vfs-dlpsettins").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 650, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function() {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=dlpfields]", this.dialog);
        this.data = $("table[role=grid][aria-label=entries]", this.fieldset);
        this.data.datagrid({
            height: 220,
            fitColumns: true,
            singleSelect: true,
            columns: [[
                {field: "ruleName", title: $("thead th:nth-child(1)", this.data).text(), resizable: true, sortable: false, width: 100},
                {field: "enabled", title: $("thead th:nth-child(2)", this.data).text(), resizable: false, sortable: false, width: 40},
                {field: "access", title: $("thead th:nth-child(3)", this.data).text(), resizable: true, sortable: false, width: 130, formatter: function(value) { return resources.getString("VFS_DLP_RULES_ACCESS_" + (value + "").toUpperCase()); }}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this)
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});

        this.moveUpButton = $("a[role=button][name=moveup]", this.fieldset);
        this.moveUpButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveUpButton, this)});

        this.moveDownButton = $("a[role=button][name=movedown]", this.fieldset);
        this.moveDownButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveDownButton, this)});
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this._adjustButtonsState();
        this.fieldset.form("validate");
        this.addButton.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.fieldset.form("disableValidation");
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    setEntries: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectEntry: function(entry) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", entry));
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _onClick$AddButton: function() {
        this.listener.onAddEntry();
    },

    _onClick$EditButton: function() {
        this.listener.onEditEntry(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteEntry(this.selection);
    },

    _onClick$MoveUpButton: function() {
        this.listener.onEntryUp(this.selection[0]);
    },

    _onClick$MoveDownButton: function() {
        this.listener.onEntryDown(this.selection[0]);
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var index = this.selection.length == 1 ? this.data.datagrid("getRowIndex", this.selection[0]) : -1;
        var count = this.data.datagrid("getRows").length;

        this.moveUpButton.linkbutton(index > 0 && index <= count - 1 ? "enable" : "disable");
        this.moveDownButton.linkbutton(index >= 0 && index < count - 1 ? "enable" : "disable");
        this.editButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length !=0 ? "enable" : "disable");
    }
});
/** @const */
jscape.vfs.DlpSettingsDialog.LISTENER = {
    onAddEntry: $.noop,
    onEditEntry: $.noop,
    onDeleteEntry: $.noop,
    onEntryUp: $.noop,
    onEntryDown: $.noop
};

/** @class */
jscape.vfs.DlpRuleDialog = Class.extend(
    /** @lends {jscape.vfs.DlpRuleDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        var id = "dlg_" + new Date().getTime().toString(36);
        var okBtnId = id + "_btn_ok";
        this.dialog = $("#d-vfs-dlp-rule").clone().attr("id", id);
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this), disabled: true, id: okBtnId},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function() {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.okButton = $("#" + okBtnId, this.dialog);
        this.fieldset = $("fieldset[name=dlprulefields]", this.dialog);

        this.rule = $("select[name=rule]", this.fieldset);
        this.rule.combobox({
            required: true,
            editable: false,
            width: 170,
            onLoadSuccess: $.proxy(this._adjustButtonsState, this),
            loadFilter: function(data) {
                return $.map(data.sort(), function(item) { return {text: item, value: item}; });
            }
        });

        this.access = $("select[name=access]", this.fieldset);
        this.access.combobox({required: true, editable: false, width: 120, panelHeight: "auto"});

        this.enabled = $("input[name=enabled]:checkbox", this.fieldset);
    },

    show: function (listener) {
         this.listener = listener;
         this.dialog.dialog("center");
         this.dialog.dialog("open");

         this._adjustButtonsState();
         this.fieldset.form("validate");
     },

     hide: function () {
         if (!this._closed()) {
             this.closing = true;
             this.fieldset.form("disableValidation");
             this.dialog.dialog("close");
             this.closing = false;
         }
     },

     destroy: function() {
         this.dialog.dialog("destroy");
     },

    setTitle: function(value) {
        this.dialog.dialog("setTitle", value);
    },

    setDlpRules: function(values) {
        this.rule.combobox("loadData", values);
        if (values.length) {
            this.rule.combobox("setValue", values[0]);
        }
    },

    getRule: function() {
        return this.rule.combobox("getValue");
    },

    setRule: function(value) {
        this.rule.combobox("setValue", value);
    },

    getAccess: function() {
        return this.access.combobox("getValue");
    },

    setAccess: function(value) {
        this.access.combobox("setValue", value);
    },

    getEnabled: function() {
        return this.enabled.is(":checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", !!value);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _adjustButtonsState: function() {
        var enabled = this.rule.combobox("getData").length != 0;
        this.okButton.linkbutton(enabled ? "enable" : "disable");
    }
});

/** @class */
jscape.vfs.DeleteDlpRuleController = Class.extend(
    /** @lends {jscape.vfs.DeleteDlpRuleController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function (entries) {
        var text = $.map(entries, function(entries) { return entries.ruleName; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("VFS_DLP_CONFIRM_DELETE_TITLE"),
                this.resources.getString("VFS_DLP_CONFIRM_DELETE_MESSAGE", {0: text}));
    }
});