(function ($) {
    function contains(target, value) {
        var opts = $.data(target, "combobox").options;
        var data = $.data(target, "combobox").data;
        for (var i = 0; i < data.length; ++i) {
            if (data[i][opts.valueField] === value) {
                return true;
            }
        }
        return false;
    }

    var selectMethod = $.fn.combobox.methods.select;
    $.fn.combobox.methods.select = function (jq, value) {
        return jq.each(function () {
            var opts = $(this).combobox("options");
            if (opts.editable) {
                $(this).combobox("reset").combobox("setValue", value);
            } else {
                if (contains(this, value)) {
                    $(this).combobox("reset");
                    selectMethod.call($.fn.combobox.methods, $(this), value);
                }
            }
        });
    };

    $.fn.combobox.defaults = $.extend({}, $.fn.combobox.defaults, {
        onLoadSuccess: function (data) {
            if (data.length) {
                var opts = $(this).combobox("options");
                $(this).combobox("select", data[0][opts.valueField]);
            }
        }
    });

    $.extend($.fn.combobox.methods, {
        "autocomplete": function(jq, enable) {
            var opts = jq.combobox("options");
            var comboOpts = jq.combo("options");
            var boxOpts = jq.combobox("textbox").validatebox("options");
            if (enable === false) {
                comboOpts.editable = opts.oldOptions.editable;
                opts.filter = opts.oldOptions.filter;
                opts.onLoadSuccess = opts.oldOptions.onLoadSuccess;
                boxOpts.validType = opts.oldOptions.validType;
                boxOpts.validParams = opts.oldOptions.validParams;
                opts.oldOptions = null;

                jq.combobox("readonly", false);
                jq.combobox("textbox").off(".autocomplete");
            } else {
                opts.oldOptions = {
                    editable: opts.editable,
                    filter: opts.filter,
                    onLoadSuccess: opts.onLoadSuccess,
                    validType: boxOpts.validType,
                    validParams: boxOpts.validParams
                };
                comboOpts.editable = true;
                jq.combobox("readonly", false);
                jq.combobox("textbox").on("focus.autocomplete", function () { $(this).select(); });

                opts.filter = function (query, row) {
                    var value = row[$(this).combobox("options").textField];
                    return value.toLowerCase().indexOf(query.toLowerCase()) != -1;
                };
                var onLoadSuccess = opts.onLoadSuccess;
                opts.onLoadSuccess = function(data) {
                    var textField = $(this).combobox("options").textField;
                    $(this).combobox("textbox").validatebox("options").validParams = $.map(data, function (row) { return row[textField]; });
                    onLoadSuccess && onLoadSuccess.apply(this, arguments);
                };
                boxOpts.validParams = $.map(jq.combobox("getData"), function(row) { return row[opts.textField]; });
                boxOpts.validType = $.isArray(boxOpts.validType)
                                    ? ["in_array"].concat(boxOpts.validType)
                                    : (typeof boxOpts.validType === "string" ? [boxOpts.validType, "in_array"] : "in_array");
            }
        }
    });

    var emailRuleValidator = $.fn.validatebox.defaults.rules.email.validator;
    $.extend($.fn.validatebox.defaults.rules, {
        "email": {
            validator: function(value) {
                value = /^(.+)?[ \t]*<(.+)>[ \t]*$/.test(value) ? RegExp.$2 : value;
                return emailRuleValidator.call(this, value);
            },
            message: $.fn.validatebox.defaults.rules.email.message
        },
        "pattern": {
            validator: function(value, params) {
                var pattern = params && params.length ? params[0] : "";
                var flags = params && params.length > 1 ? params[1] : "";
                try { return new RegExp(pattern, flags).test(value);} catch (e) { return false }
            },
            message: $.fn.validatebox.defaults.missingMessage || ""
        },
        "variable": {
            validator: function(value) {
                return /^%[^%]+%$/.test(value.trim());
            },
            message: $.fn.validatebox.defaults.missingMessage || ""
        },
        "in_array": {
            validator: function(value, params) {
                return $.inArray(value, params || []) != -1;
            },
            message: $.fn.validatebox.defaults.missingMessage || ""
        },
        "in_range": {
            validator: function(value, params) {
                var lowerBound = params && params.length ? parseInt(params[0]) : NaN;
                var upperBound = params && params.length > 1 ? parseInt(params[1]) : NaN;
                value = parseInt(value);
                return !isNaN(lowerBound) && !isNaN(upperBound) && !isNaN(value) && isFinite(value) ? lowerBound <= value && value <= upperBound : true
            },
            message: "Please enter a value between {0} and {1}."
        },
        "multi_value": {
            validator: function (value, params) {
                var rule = $(this).validatebox("options").rules[params[0]];
                var separator = params.length > 1 ? params[1] : " ";
                var values = value.split(separator);
                for (var i = 0; i < values.length; i++) {
                    var str = values[i].trim();
                    if (str.length && !rule["validator"].call(this, str)) {
                        return false;
                    }
                }
                return true;
            },
            message: ""
        },
        "any": {
            validator: function(value, params) {
                var opts = $(this).validatebox("options");
                var validType = opts.validType;
                try {
                    for (var i = 0; i < params.length; ++i) {
                        opts.validType = params[i];
                        if ($(this).validatebox("isValid")) {
                            return true;
                        }
                    }
                    return false;
                } finally {
                    opts.validType = validType;
                }
            },
            message: ""
        }
    });
})(jQuery);


(function ($) {
    function pageFilter(data) {
        var state = $.data(this, "datagrid");
        var opts = state.options;
        var dg = $(this);
        $(this).datagrid("getPager")
                .pagination({
                        pageNumber: opts.pageNumber || undefined,
                        pageSize: opts.pageSize || undefined,
                        pageList: pageList(dg, data.total),
                        total: data.total,
                        buttons: pageButtons(dg),
                        onSelectPage: function (pageNumber, pageSize) {
                            opts.pageNumber = pageNumber;
                            opts.pageSize = pageSize;
                            $(this).pagination("refresh", {pageNumber: pageNumber, pageSize: pageSize});
                            dg.datagrid("loadData", state.originalData || state.data);
                        },
                        onBeforeRefresh: function () {
                            dg.datagrid("reload");
                            return false;
                        }
                });
        var start = (opts.pageNumber - 1) * opts.pageSize;
        var rows = start < data.rows.length
                   ? data.rows.slice(start, start + opts.pageSize)
                   : data.rows.slice(0, opts.pageSize);
        return {total: data.total, rows: rows};
    }

    function pageList(jq, total) {
        var list = jq.datagrid("options").pageList;
        list = total > 0 ? [total].concat(list) : [].concat(list);
        list.sort(function (a, b) { return a - b; });
        return total > 0 ? list.slice(0, $.inArray(total, list) + 1) : list.slice(0);
    }

    function pageButtons(jq) {
        var opts = jq.datagrid("options");
        return opts.search ? [{iconCls: "icon-search", handler: function () { jq.datagrid("getFindbar").findbar("show"); }}] : [];
    }

    var loadFilter = $.fn.datagrid.defaults.loadFilter;
    $.fn.datagrid.defaults.loadFilter = function (data) {
        data = loadFilter.call($.fn.datagrid.methods, data);
        var state = $.data(this, "datagrid");
        if (state.options.pagination) {
            if (!state.originalData && !state.options.sorting) {
                state.originalData = data;
            }
            state.options.sorting = undefined;
            data = pageFilter.call(this, data);
        }
        return data;
    };
    var onBeforeSortColumn = $.fn.datagrid.defaults.onBeforeSortColumn;
    $.fn.datagrid.defaults.onBeforeSortColumn = function() {
        var ret = onBeforeSortColumn.apply(this, arguments);
        if (ret != false) {
            $.data(this, "datagrid").options.sorting = true;
        }
        return ret;
    };
    var loadData = $.fn.datagrid.methods.loadData;
    $.fn.datagrid.methods.loadData = function(jq, data) {
        jq.each(function() {
            $.data(this, "datagrid").originalData = null;
        });
        return loadData.call($.fn.datagrid.methods, jq, data);
    };
})(jQuery);


// XSS filters
(function ($) {
    function escapeXml(value) {
        var entities = {"<": "&lt;", ">": "&gt;", "&": "&amp;", '"': "&quot;"};
        return value.replace(/[<>&"]/g, function (entry) { return entities[entry]});
    }
    function xssFilter(value) {
        return typeof value === "string" ? escapeXml(value) : value;
    }
    var getColumnOptionMethod = $.fn.datagrid.methods.getColumnOption;
    $.fn.datagrid.methods.getColumnOption = function (jq, field) {
        var col = getColumnOptionMethod.call(this, jq, field);
        if (col && !$.isFunction(col.formatter)) {
            col.formatter = xssFilter;
        }
        return col;
    };

    var treeFormatter = $.fn.tree.defaults.formatter;
    $.fn.tree.defaults.formatter = function() { return xssFilter(treeFormatter.apply(this, arguments)); };

    var comboFormatter = $.fn.combobox.defaults.formatter;
    $.fn.combobox.defaults.formatter = function() { return xssFilter(comboFormatter.apply(this, arguments)); };

    var setTitleMethod = $.fn.panel.methods.setTitle;
    $.fn.panel.methods.setTitle = function(jq, value) { return setTitleMethod.call(this, jq, xssFilter(value)); };

    $.each($.messager, function(key) {
        var fn = $.messager[key];
        if ($.isFunction(fn)) {
            $.messager[key] = function() {
                return fn.apply(this, $.map(arguments, function(arg) { return xssFilter(arg); }));
            }
        }
    });
})(jQuery);


// FindBar plugin
(function ($) {
    function init(target) {
        var state = $.data(target, "findbar");
        var opts = state.options;
        target = $(target).hide().addClass(opts.cls);
        var initializing = true;
        if (target.length) {
            target.empty();
        }

        function _doSearch() { if (!initializing) { $(target).findbar("search"); }}
        function _doHide() { $(target).findbar("hide"); }

        $('<input name="query" type="search" size="30" maxlength="255"/>')
                .off(".findbar")
                .on("keydown.findbar", function (e) {
                        if (e.keyCode == 13) {
                            _doSearch();
                            return false;
                        } else if (e.keyCode == 27) {
                            _doHide();
                            return false;
                        }
                    })
                .on("input.findbar", function () {
                        if (this.timer) { clearTimeout(this.timer); }
                        opts.query = $(this).val();
                        this.timer = setTimeout(_doSearch, 200);
                    })
                .prependTo($('<label/>').appendTo(target));

        $.each(opts.nav || {}, function(_, btn) {
            $('<a/>').appendTo(target).linkbutton(btn);
        });

        $('<input type="checkbox" name="matchcase"/>').prependTo($('<label/>').text(opts.matchCaseText).appendTo(target))
                .click(function() { opts.ignoreCase = !$(this).is(":checked"); _doSearch(); });

        $('<input type="checkbox" name="regexp"/>').prependTo($('<label/>').text(opts.regexpText).appendTo(target))
                .click(function() { opts.regexp = $(this).is(":checked"); _doSearch(); });

        if (opts.fields.length) {
            var byField = $('<input type="checkbox"/>').prependTo($('<label/>').text(opts.fieldText).appendTo(target));
            var field = $('<select name="field"></select>').appendTo(target);
            byField.change(function() {
                if ($(this).is(":checked")) {
                    field.combobox("enable");
                    opts.field = field.combobox("getValue");
                } else {
                    field.combobox("disable");
                    opts.field = null;
                }
                _doSearch();
            });
            field.combobox({editable: false, data: opts.fields, width: 120, panelHeight: "auto", onChange: function () { byField.change(); }});
        }

        if (opts.closable) {
            $('<a/>').css({float: "right"}).appendTo(target)
                    .linkbutton({plain: true, iconCls: "panel-tool-close", onClick: function() { _doHide(); }});
        }

        initializing = false;
    }

    function doShow(target, e) {
        if (!$(target).is(":visible")) {
            $(target).show();
            var opts = $.data(target, "findbar").options;
            opts.onShow.call(target, e);
        }
        $(target).find(":input:first").focus();
    }

    function doHide(target, e) {
        var opts = $.data(target, "findbar").options;
        if (opts.closable) {
            $(target).hide();
            opts.onHide.call(target, e);
        }
    }

    function doSearch(target) {
        var state = $.data(target, "findbar");
        var opts = state.options;
        var query = $(target).find("input[name=query]");

        try {
            var pattern = opts.regexp ? opts.query : opts.query.replace(/([.*+?^${}()|\[\]\/\\])/g, "\\$1");
            state.re = new RegExp(pattern, opts.ignoreCase ? "i" : "");
            query.removeClass(opts.invalidCls);
        } catch (e) {
            state.re = null;
            query.addClass(opts.invalidCls);
        } finally {
            opts.onSearch.call(target);
        }
    }

    function doHighlight(target, node) {
        var state = $.data(target, "findbar");
        if ($.type(state.re) === "regexp") {
            node.each(function() {
                _highlight(this, state.re, state.options.highlightCls);
            });
        }

        function _highlight(target, re, cls) {
            if (target.nodeType == 3) {
                var matches = re.exec(target.data);
                if (matches) {
                    var node = target.splitText(matches.index);
                    node.splitText(matches[0].length);
                    var wrapper = document.createElement("span");
                    wrapper.className = cls;
                    wrapper.appendChild(node.cloneNode(true));
                    node.parentNode.replaceChild(wrapper, node);
                    return true;
                }
            } else if (target.nodeType == 1 && target.hasChildNodes() && !/(script|style)/i.test(target.tagName)) {
                for (var i = 0, size = target.childNodes.length; i < size; ++i) {
                    if (_highlight(target.childNodes[i], re, cls)) {
                        i++;
                    }
                }
            }
            return false;
        }
    }

    function doUnhighlight(target, node) {
        var opts = $.data(target, "findbar").options;
        var nodes = $(node).find("." + opts.highlightCls);
        nodes.replaceWith(function() { return $(this).text(); });
    }

    $.fn.findbar = function(opts, args) {
        if (typeof opts == "string") {
            return $.fn.findbar.methods[opts](this, args);
        }
        opts = opts || {};
        return this.each(function () {
            var state = $.data(this, "findbar");
            if (state) {
                $.extend(state.options, opts);
            } else {
                opts = $.extend({}, $.fn.findbar.defaults, $.fn.findbar.parseOptions(this), opts);
                $.data(this, "findbar", {options: opts});
            }
            init(this, args);
        });
    };

    $.fn.findbar.methods = {
        options: function (jq) { return $.data(jq[0], "findbar").options; },
        show: function (jq, e) { return jq.each(function () { doShow(this, e); }); },
        hide: function (jq, e) { return jq.each(function () { doHide(this, e); }); },
        search: function (jq, data) { return jq.each(function () { doSearch(this, data); })},
        highlight: function (jq, node) { return jq.each(function () { doHighlight(this, node); })},
        unhighlight: function (jq, node) { return jq.each(function () { doUnhighlight(this, node); })},
        test: function(jq, data) { var state = $.data(jq[0], "findbar"); return $.type(state.re) != "regexp" || state.re.test(data); }
    };

    $.fn.findbar.defaults = {
        query: "", ignoreCase: true, regexp: false, field: null, fields: [], closable: true,
        onShow: function (e) {}, onHide: function (e) {}, onSearch: function () {},
        cls: "datagrid-findbar", invalidCls: "query-invalid", highlightCls: "highlight",
        matchCaseText: "Match case", regexpText: "Regexp", fieldText: "Field",
        nav: {
            /*prev: {plain: true, iconCls: "icon-up", onClick: function() {}},
            next: {plain: true, iconCls: "icon-down", onClick: function() {}}*/
        }
    };

    $.fn.findbar.parseOptions = function (t) {
        return $.extend({}, $.parser.parseOptions($(t), ["query", {showMatchCase: "boolean", showRegexp: "boolean", showFields: "boolean", ignoreCase: "boolean", regexp: "boolean"}]));
    };


    (function($) {
        function matches(rows) {
            var fb = $(this).datagrid("getFindbar");
            var field = fb.findbar("options").field;
            var fields = $.grep($(this).datagrid("getColumnFields", true).concat($(this).datagrid("getColumnFields", false)),
                                function(value) { return field == null || field == value; });

            for (var i = 0; i < rows.length;) {
                var row = rows[i];
                var found = false;

                for (var j = 0; j < fields.length; ++j) {
                    var col = $(this).datagrid("getColumnOption", fields[j]);
                    var value = col && col.formatter ? col.formatter(row[fields[j]], row, i) : row[fields[j]];
                    if (fb.findbar("test", value)) {
                        found = true;
                        break;
                    }
                }
                if (found) {
                    ++i;
                } else {
                    rows.splice(i, 1);
                }
            }
            return rows;
        }

        function searchFilter(data) {
            if ($.isArray(data)) {
                data = {total: data.length, rows: data};
            }
            var state = $.data(this, "datagrid");
            if (!state.originalData) {
                state.originalData = data;
            }
            var rows = matches.call(this, [].concat(data.rows));
            state.options.pageSize = rows.length;

            return {total: rows.length, rows: rows};
        }

        function highlightSearch() {
            var state = $.data(this, "datagrid");
            var cells = state.dc.view.find(".datagrid-body").find("td");
            var fb = $(this).datagrid("getFindbar");
            var field = fb.findbar("options").field;
            fb.findbar("unhighlight", cells);
            fb.findbar("highlight", field ? cells.filter("[field=" + field + "]") : cells);
        }

        function create(target) {
            var dg = $(target);
            var state = $.data(target, "datagrid");

            var header = state.dc.header2.find("table.datagrid-htable");
            header.find("tr.datagrid-findbar-row").remove();

            var fields = dg.datagrid("getColumnFields");
            var tr = $('<tr class="datagrid-header-row datagrid-findbar-row"></tr>');
            var td = $('<td' + (fields.length ? ' colspan="' + fields.length + '"' : '') + '></td>');
            var bar = $('<div/>').hide().appendTo(td.appendTo(tr.appendTo(header.find("tbody"))));

            return bar.findbar({
                fields: $.map(fields, function (field) {
                    var col = dg.datagrid("getColumnOption", field);
                    return {text: (col && col.title) || field, value: field};
                }),
                onSearch: function () {
                    dg.datagrid("loadData", state.originalData || state.data);
                },
                onShow: function () {
                    var opts = dg.datagrid("options");
                    opts.oldOptions = {
                        loadFilter: opts.loadFilter,
                        onLoadSuccess: opts.onLoadSuccess,
                        pageNumber: opts.pageNumber,
                        pageSize: opts.pageSize,
                        checkOnSelect: opts.checkOnSelect
                    };
                    var loadFilter = opts.loadFilter;
                    opts.loadFilter = function(data) {
                        data = searchFilter.call(this, data);
                        return loadFilter.call(this, data);
                    };
                    var onLoadSuccess = opts.onLoadSuccess;
                    opts.onLoadSuccess = function(data) {
                        onLoadSuccess && onLoadSuccess.apply(this, arguments);
                        highlightSearch.call(this);
                    };
                    // ui lib bug fix: prevent of selecting checkboxes in the findbar;
                    opts.checkOnSelect = false;

                    dg.datagrid("resize");
                },
                onHide: function () {
                    var opts = dg.datagrid("options");
                    $.extend(opts, opts.oldOptions);
                    opts.oldOptions = null;

                    $(this).closest("tr.datagrid-findbar-row").remove();
                    dg.datagrid("resize");
                    dg.datagrid("loadData", state.originalData || state.data);
                }
            });
        }

        $.extend($.fn.datagrid.methods, {
            getFindbar: function (jq) {
                var bar = jq.datagrid("getPanel").find("div.datagrid-findbar");
                if (bar.length == 0) {
                    bar = create(jq[0]);
                }
                return bar;
            }
        });
    })($)
})(jQuery);