jscape.keystore = jscape.keystore || {};

/** @class */
jscape.keystore.ClientKeysModule = jscape.keystore.KeystoreModule.extend(
    /** @lends {jscape.keystore.ClientKeysModule.prototype} */
{
    onCreate: function() {
        this._super.apply(this, arguments);

        this.controller = new jscape.keystore.ClientKeysController(this.resources);
    },

    onStart: function() {
        this.controller.start(this.managementService, this.keys);
    }
});

/** @class */
jscape.keystore.ClientKeysController = Class.extend(
    /** @lends {jscape.keystore.ClientKeysController.prototype} */
{
    publicKeys: null,
    certificates: null,

    /** @constructor */
    initialize: function(resources) {
        this.view = new jscape.keystore.ClientKeysView(resources);
        this.importController = new jscape.keystore.ImportClientKeyController(resources);
        this.generateController = new jscape.keystore.GenerateClientKeyController(resources);
        this.exportPublicKeyController = new jscape.keystore.ExportClientPublicKeyController(resources);
        this.exportCertificateController = new jscape.keystore.ExportClientCertificateController(resources);
        this.deleteController = new jscape.keystore.DeleteClientKeyController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, keys) {
        this.managementService = managementService;
        this.publicKeys = keys.getClientKeys();
        this.certificates = keys.getClientCertificates();

        this.view.setCertificates(this.certificates);
        this.view.setPublicKeys(this.publicKeys);
        this.view.show(this);
    },

    onImport: function() {
        this.importController.start(this.managementService)
                .done($.proxy(this._keyImported, this));
    },

    onExportCertificate: function(certificate) {
        this.exportCertificateController.start(this.managementService, certificate.alias);
    },

    onExportCertificatePublicKey: function(certificate) {
        this.exportPublicKeyController.start(this.managementService, certificate.alias, true);
    },

    onExportPublicKey: function(key) {
        this.exportPublicKeyController.start(this.managementService, key.alias, false);
    },

    onGenerate: function() {
        this.generateController.start(this.managementService)
                .done($.proxy(this._keyGenerated, this));
    },

    onDeletePublicKey: function(key) {
        this.deleteController.start(this.managementService, key.alias, false)
                .done($.proxy(this._publicKeyDeleted, this, key));
    },

    onDeleteCertificate: function(certificate) {
        this.deleteController.start(this.managementService, certificate.alias, true)
                .done($.proxy(this._certificateDeleted, this, certificate));
    },

    _keyImported: function(key) {
        if (key.serial) {
            this.certificates.push(key);
            this.view.setCertificates(this.certificates);
            this.view.selectItem(key);
        } else {
            this.publicKeys.push(key);
            this.view.setPublicKeys(this.publicKeys);
            this.view.selectItem(key);
        }
    },

    _keyGenerated: function(certificate) {
        this.certificates.push(certificate);
        this.view.setCertificates(this.certificates);
        this.view.selectItem(certificate);
    },

    _publicKeyDeleted: function(key) {
        var index = $.inArray(key, this.publicKeys);
        if (index != -1) {
            this.publicKeys.splice(index, 1);
            this.view.setPublicKeys(this.publicKeys, index);

            if (this.publicKeys.length) {
                index = Math.max(0, Math.min(index, this.publicKeys.length - 1));
                this.view.selectItem(this.publicKeys[index]);
            }
        }
    },

    _certificateDeleted: function(certificate) {
        var index = $.inArray(certificate, this.certificates);
        if (index != -1) {
            this.certificates.splice(index, 1);
            this.view.setCertificates(this.certificates);

            if (this.certificates.length) {
                index = Math.max(0, Math.min(index, this.certificates.length - 1));
                this.view.selectItem(this.certificates[index]);
            }
        }
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("clientkey_nonempty", resources.getString("KEY_MANAGER_CLIENT_KEYS_ERROR_BAD_ALIAS"));
        v.rule("clientkey_notexists", $.proxy(this._keyAliasNotExists, this), resources.getString("KEY_MANAGER_CLIENT_KEYS_ERROR_ALIAS_EXISTS"));
    },

    _keyAliasNotExists: function(value, excludes) {
        var keys = [].concat(this.publicKeys).concat(this.certificates);
        for (var i = 0; i < keys.length; i++) {
            var alias = keys[i].alias;
            if (excludes && $.inArray(alias, excludes) != -1) {
                continue;
            }
            if (alias == value) {
                return false;
            }
        }
        return true;
    }
});


/** @class */
jscape.keystore.ClientKeysView = Class.extend(
    /** @lends {jscape.keystore.ClientKeysView.prototype} */
{
    CERTIFICATES_ROOT_ID: "certificates",
    PUBLIC_KEYS_ROOT_ID: "public-keys",

    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=clientkeysfields]");

        this.keyInfo = new jscape.keystore.ClientKeyInfoView(resources);

        this.tree = $("ul[role=tree]", this.fieldset);
        this.tree.tree({lines: true, onSelect: $.proxy(this._adjustSelection, this)});
        this.certificatesRoot = this.tree.tree("find", this.CERTIFICATES_ROOT_ID);
        this.publicKeysRoot = this.tree.tree("find", this.PUBLIC_KEYS_ROOT_ID);

        this.importButton = $("a[role=button][name=import]", this.fieldset);
        this.importButton.linkbutton({onClick: $.proxy(this._onClick$ImportButton, this)});

        this.exportButton = $("a[role=button][name=export]", this.fieldset);
        this.exportButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ExportButton, this)});
        this.exportMenu = $("div[role=menu][aria-label=export-menu]", this.fieldset);
        this.exportMenu.menu({onClick: $.proxy(this._onClick$ExportMenu, this)});

        this.generateButton = $("a[role=button][name=generate]", this.fieldset);
        this.generateButton.linkbutton({onClick: $.proxy(this._onClick$GenerateButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.keystore.ClientKeysView.LISTENER;
        this._adjustSelection();
    },

    getCertificates: function() {
        var nodes = this.tree.tree("getChildren", this.certificatesRoot.target);
        return $.map(nodes || [], $.proxy(this._node2Key, this));
    },

    setCertificates: function (values) {
        this._updateChildren(this.certificatesRoot, values || []);
    },

    getPublicKeys: function() {
        var nodes = this.tree.tree("getChildren", this.publicKeysRoot.target);
        return $.map(nodes || [], $.proxy(this._node2Key, this));
    },

    setPublicKeys: function (values) {
        this._updateChildren(this.publicKeysRoot, values || []);
    },

    selectItem: function(value) {
        var node = this.tree.tree("find", this._key2Node(value).id);
        if (node) {
            this.tree.tree("select", node.target);
        }
    },

    _onClick$ImportButton: function() {
        this.listener.onImport();
    },

    _onClick$GenerateButton: function() {
        this.listener.onGenerate();
    },

    _onClick$ExportButton: function () {
        if (this._publicKeySelected()) {
            this.listener.onExportPublicKey(this.selection);
        } else if (this._certificateSelected()) {
            this.exportMenu.menu("show", {alignTo: this.exportButton});
            this.exportButton.blur();
        }
    },

    _onClick$ExportMenu: function(item) {
        var type = $(item.target).attr("aria-label");
        if ("menu-crt" == type) {
            this.listener.onExportCertificate(this.selection);
        } else if ("menu-pubkey" == type) {
            this.listener.onExportCertificatePublicKey(this.selection);
        }
    },

    _onClick$DeleteButton: function() {
        if (this._publicKeySelected()) {
            this.listener.onDeletePublicKey(this.selection);
        } else if (this._certificateSelected()) {
            this.listener.onDeleteCertificate(this.selection);
        }
    },

    _key2Node: function (key) {
        return {id: key.alias, text: key.alias, iconCls: "icon-key", attributes: {key: key}};
    },

    _node2Key: function (node) {
        return node && node.attributes ? node.attributes.key : null;
    },

    _updateChildren: function(root, values) {
        if (!this.tree.tree("isLeaf", root.target)) {
            var children = this.tree.tree("getChildren", root.target);
            for (var i = 0; i < children.length; i++) {
                this.tree.tree("remove", children[i].target);
            }
        }
        this.tree.tree("append", {parent: root.target, data: $.map(values, $.proxy(this._key2Node, this))});
    },

    _adjustSelection: function() {
        var node = this.tree.tree("getSelected");
        this.selection = $.inArray(node, this.tree.tree("getRoots")) == -1 ? this._node2Key(node) : null;
        this._adjustKeyInfo();
        this._adjustButtonsState();
    },

    _adjustKeyInfo: function () {
        if (this.selection != null) {
            this.keyInfo.show(this.selection);
        } else {
            this.keyInfo.hide();
        }
    },

    _adjustButtonsState: function() {
        var enabled = this.selection != null;
        this.exportButton.linkbutton(enabled ? "enable" : "disable");
        this.deleteButton.linkbutton(enabled ? "enable" : "disable");
    },

    _certificateSelected: function() {
        var node = this.tree.tree("getSelected");
        return node != null && this.tree.tree("getParent", node.target) == this.certificatesRoot;
    },

    _publicKeySelected: function() {
        var node = this.tree.tree("getSelected");
        return node != null && this.tree.tree("getParent", node.target) == this.publicKeysRoot;
    }
});
jscape.keystore.ClientKeysView.LISTENER = {
    onImport: $.noop,
    onExportCertificate: $.noop,
    onExportCertificatePublicKey: $.noop,
    onExportPublicKey: $.noop,
    onGenerate: $.noop,
    onDeletePublicKey: $.noop,
    onDeleteCertificate: $.noop
};

/** @class */
jscape.keystore.ClientKeyInfoView = jscape.keystore.BaseKeyInfoView.extend(
    /** @lends {jscape.keystore.ClientKeyInfoView.prototype} */
{
    /** @constructor */
    initialize: function() {
        this._super($("fieldset[name=clientkeyinfofields]"));
    }
});

/** @class */
jscape.keystore.ImportClientKeyController = Class.extend(
    /** @lends {jscape.keystore.ImportClientKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService) {
        this.managementService = managementService;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function (dialog) {
        this.managementService.importClientKey(dialog.getAlias(), dialog.getFile())
                .done($.proxy(function (key) { this.deferred.resolve(key); }, this))
                .fail($.proxy(function () { this.deferred.reject(); }, this))
                .always($.proxy(function () {
                            dialog.destroy();
                            this.deferred = null;
                        }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function() {
        return new jscape.keystore.ImportClientKeyDialog(this.resources);
    }
});

/** @class */
jscape.keystore.ImportClientKeyDialog = jscape.keystore.ImportPublicKeyDialog.extend(
    /** @lends {jscape.keystore.ImportClientKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.alias.validatebox({required: true, validType: ["clientkey_nonempty", "clientkey_notexists"], missingMessage: resources.getString("KEY_MANAGER_CLIENT_KEYS_ERROR_BAD_ALIAS")});
        this.file.validatebox({required: true, missingMessage: resources.getString("KEY_MANAGER_CLIENT_KEYS_ERROR_BAD_FILE")});
    }
});

/** @class */
jscape.keystore.GenerateClientKeyController = Class.extend(
    /** @lends {jscape.keystore.GenerateClientKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService) {
        this.managementService = managementService;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var alias = dialog.getAlias();

        var deferred = $.Deferred();
        var exportDialog = new jscape.keystore.ExportPrivateKeyDialog(this.resources);
        exportDialog.setFilename(alias + ".prv");
        exportDialog.show({onSubmit: deferred.resolve, onCancel: deferred.reject});

        deferred.promise()
                .done($.proxy(function() {
                                  dialog.hide();
                                  this.managementService.generateClientKey(alias,
                                                                           dialog.getAlgorithm(),
                                                                           dialog.getKeyLength(),
                                                                           dialog.getPeriod(),
                                                                           dialog.getCommonName(),
                                                                           dialog.getOrganizationUnit(),
                                                                           dialog.getOrganization(),
                                                                           dialog.getLocality(),
                                                                           dialog.getState(),
                                                                           dialog.getCountry(),
                                                                           exportDialog.getFilename(),
                                                                           exportDialog.getFormat(),
                                                                           exportDialog.getPassword())
                                          .then($.proxy(function() { return this.managementService.getClientCertificate(alias); }, this))
                                          .done($.proxy(function(certificate) { dialog.destroy(); this.deferred.resolve(certificate); this.deferred = null; }, this))
                                          .fail($.proxy(this.onCancel, this, dialog))
                              }, this))
                .fail($.proxy(this.onCancel, this, dialog))
                .always(function() { exportDialog.destroy(); });
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function() {
        var dialog = new jscape.keystore.GenerateClientKeyDialog(this.resources);
        dialog.setAlgorithms(jscape.KeyPairAlgorithm.values());
        return dialog;
    }
});

/** @class */
jscape.keystore.GenerateClientKeyDialog = Class.extend(
    /** @lends {jscape.keystore.GenerateClientKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-clientkey-generate").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=generateclientkeyfields]", this.dialog);

        this.alias = $("input[name=alias]", this.fieldset);
        this.alias.validatebox({
            required: true,
            validType: ["clientkey_nonempty", "clientkey_notexists"],
            missingMessage: resources.getString("KEY_MANAGER_CLIENT_KEYS_ERROR_BAD_ALIAS")
        });

        this.keyAlgorithm = $("select[name=keyalgorithm]", this.fieldset);
        this.keyAlgorithm.combobox({editable: false, width: 80, panelHeight: "auto", onChange: $.proxy(this._adjustKeyLength, this)});

        this.keyLength = $("select[name=keylength]", this.fieldset);
        this.keyLength.combobox({editable: false, width: 80, panelHeight: "auto"});

        this.period = $("input[name=period]", this.fieldset);
        this.period.numberspinner({min: 1, max: 10000, value: 365, increment: 10, width: 80});

        this.commonName = $("input[name=cn]", this.fieldset);
        this.organizationUnit = $("input[name=ou]", this.fieldset);
        this.organization = $("input[name=o]", this.fieldset);
        this.locality = $("input[name=l]", this.fieldset);
        this.state = $("input[name=s]", this.fieldset);
        this.country = $("input[name=c]", this.fieldset);
    },

    getAlias: function() {
        return this.alias.val();
    },

    getAlgorithm: function() {
        return this.keyAlgorithm.combobox("getValue");
    },

    setAlgorithms: function(values) {
        var data = $.map(values || [], function(item) { return {text: item.name, value: item.name}; });
        this.keyAlgorithm.combobox("loadData", data);
    },

    getKeyLength: function() {
        return this.keyLength.combobox("getValue");
    },

    setKeyLengths: function (values) {
        var data = $.map(values || [], function (value) { return {text: value, value: value}; });
        this.keyLength.combobox("loadData", data);
    },

    getPeriod: function() {
        return this.period.numberspinner("getValue");
    },

    getCommonName: function() {
        return this.commonName.val();
    },

    getOrganizationUnit: function() {
        return this.organizationUnit.val();
    },

    getOrganization: function() {
        return this.organization.val();
    },

    getLocality: function() {
        return this.locality.val();
    },

    getState: function() {
        return this.state.val();
    },

    getCountry: function() {
        return this.country.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _adjustKeyLength: function () {
        this.setKeyLengths(jscape.KeyPairAlgorithm.lengthsOf(this.getAlgorithm()));
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.keystore.DeleteClientKeyController = Class.extend(
    /** @lends {jscape.keystore.DeleteClientKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, alias, isCertificate) {
        this.managementService = managementService;
        this.deferred = $.Deferred();

        this._confirmDelete(alias, isCertificate)
                .done($.proxy(this._deleteConfirmed, this, alias, isCertificate))
                .fail($.proxy(function() { this.deferred.reject(); }, this));

        return this.deferred.promise();
    },

    _confirmDelete: function(alias, isCertificate) {
        if (!!isCertificate) {
            return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_CLIENT_KEYS_DELETE_CERTIFICATE_TITLE"),
                                                this.resources.getString("KEY_MANAGER_CLIENT_KEYS_DELETE_CERTIFICATE_MESSAGE", {0: alias}));
        } else {
            return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_CLIENT_KEYS_DELETE_PUBLIC_KEY_TITLE"),
                                                this.resources.getString("KEY_MANAGER_CLIENT_KEYS_DELETE_PUBLIC_KEY_MESSAGE", {0: alias}));
        }
    },

    _deleteConfirmed: function (alias, isCertificate) {
        var promise = !!isCertificate
                      ? this.managementService.deleteClientCertificate(alias)
                      : this.managementService.deleteClientPublicKey(alias);
        promise.done($.proxy(function () { this.deferred.resolve(); }, this))
                .fail($.proxy(function () { this.deferred.reject(); }, this));
    }
});

/** @class */
jscape.keystore.ExportClientPublicKeyController = Class.extend(
    /** @lends {jscape.keystore.ExportClientPublicKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, alias, isCertificate) {
        this.managementService = managementService;
        this.alias = alias;
        this.isCertificate = !!isCertificate;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var format = dialog.getFormat();
        var filename = dialog.getFilename();
        dialog.destroy();

        var promise = this.isCertificate
                ? this.managementService.exportClientCertificatePublicKey(this.alias, format, filename)
                : this.managementService.exportClientPublicKey(this.alias, format, filename);
        promise.done($.proxy(function () { this.deferred.resolve(); }, this))
                .fail($.proxy(function () { this.deferred.reject(); }, this))
                .always($.proxy(function () { this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function() {
        var dialog = new jscape.keystore.ExportPublicKeyDialog(this.resources);
        dialog.setFilename(this.alias + ".pub");
        return dialog;
    }
});

/** @class */
jscape.keystore.ExportClientCertificateController = Class.extend(
    /** @lends {jscape.keystore.ExportClientCertificateController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, alias) {
        this.managementService = managementService;
        this.alias = alias;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var format = dialog.getFormat();
        var filename = dialog.getFilename();
        dialog.destroy();

        this.managementService.exportClientCertificate(this.alias, format, filename)
                .done($.proxy(function() { this.deferred.resolve(); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this))
                .always($.proxy(function() { this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function() {
        var dialog = new jscape.keystore.ExportCertificateDialog(this.resources);
        dialog.setFilename(this.alias + ".crt");
        return dialog;
    }
});