jscape.keystore = jscape.keystore || {};

/** @class */
jscape.keystore.HostKeysModule = jscape.keystore.KeystoreModule.extend(
    /** @lends {jscape.keystore.HostKeysModule.prototype} */
{
    onCreate: function() {
        this._super.apply(this, arguments);

        this.controller = new jscape.keystore.HostKeysController(this.resources);
    },

    onStart: function() {
        this.controller.start(this.managementService, this.keys);
    }
});

/** @class */
jscape.keystore.HostKeysController = Class.extend(
    /** @lends {jscape.keystore.HostKeysController.prototype} */
{
    publicKeys: null,
    certificates: null,

    /** @constructor */
    initialize: function(resources) {
        this.view = new jscape.keystore.HostKeysView(resources);
        this.importController = new jscape.keystore.ImportHostKeyController(resources);
        this.exportPublicKeyController = new jscape.keystore.ExportHostPublicKeyController(resources);
        this.exportCertificateController = new jscape.keystore.ExportHostCertificateController(resources);
        this.deleteController = new jscape.keystore.DeleteHostKeyController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, keys) {
        this.managementService = managementService;
        this.publicKeys = keys.getHostKeys();
        this.certificates = keys.getHostCertificates();

        this.view.setPublicKeys(this.publicKeys);
        this.view.setCertificates(this.certificates);
        this.view.show(this);
    },

    onImport: function() {
        this.importController.start(this.managementService)
                .done($.proxy(this._keyImported, this));
    },

    onExportCertificate: function(certificate) {
        this.exportCertificateController.start(this.managementService, certificate.alias);
    },

    onExportCertificatePublicKey: function(certificate) {
        this.exportPublicKeyController.start(this.managementService, certificate.alias, true);
    },

    onExportPublicKey: function(key) {
        this.exportPublicKeyController.start(this.managementService, key.alias, false);
    },

    onDeletePublicKey: function(key) {
        this.deleteController.start(this.managementService, key.alias, false)
                .done($.proxy(this._publicKeyDeleted, this, key));
    },

    onDeleteCertificate: function(certificate) {
        this.deleteController.start(this.managementService, certificate.alias, true)
                .done($.proxy(this._certificateDeleted, this, certificate));
    },

    _keyImported: function(key) {
        if (key.serial) {
            this.certificates.push(key);
            this.view.setCertificates(this.certificates);
            this.view.selectItem(key);
        } else {
            this.publicKeys.push(key);
            this.view.setPublicKeys(this.publicKeys);
            this.view.selectItem(key);
        }
    },

    _publicKeyDeleted: function(key) {
        var index = $.inArray(key, this.publicKeys);
        if (index != -1) {
            this.publicKeys.splice(index, 1);
            this.view.setPublicKeys(this.publicKeys);

            if (this.publicKeys.length) {
                index = Math.max(0, Math.min(index, this.publicKeys.length - 1));
                this.view.selectItem(this.publicKeys[index]);
            }
        }
    },

    _certificateDeleted: function(certificate) {
        var index = $.inArray(certificate, this.certificates);
        if (index != -1) {
            this.certificates.splice(index, 1);
            this.view.setCertificates(this.certificates);

            if (this.certificates.length) {
                index = Math.max(0,Math.min(index, this.certificates.length - 1));
                this.view.selectItem(this.certificates[index - 1]);
            }
        }
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("hostkey_nonempty", resources.getString("KEY_MANAGER_HOST_KEYS_ERROR_BAD_ALIAS"));
        v.rule("hostkey_notexists", $.proxy(this._keyAliasNotExists, this), resources.getString("KEY_MANAGER_HOST_KEYS_ERROR_ALIAS_EXISTS"));
    },

    _keyAliasNotExists: function(value, excludes) {
        var keys = [].concat(this.publicKeys).concat(this.certificates);
        for (var i = 0; i < keys.length; i++) {
            var alias = keys[i].alias;
            if (excludes && $.inArray(alias, excludes) != -1) {
                continue;
            }
            if (alias == value) {
                return false;
            }
        }
        return true;
    }
});

/** @class */
jscape.keystore.HostKeysView = Class.extend(
    /** @lends {jscape.keystore.HostKeysView.prototype} */
{
    CERTIFICATES_ROOT_ID: "certificates",
    PUBLIC_KEYS_ROOT_ID: "public-keys",

    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=hostkeysfields]");

        this.keyInfo = new jscape.keystore.HostKeyInfoView(resources);

        this.tree = $("ul[role=tree]", this.fieldset);
        this.tree.tree({lines: true, onSelect: $.proxy(this._adjustSelection, this)});
        this.certificatesRoot = this.tree.tree("find", this.CERTIFICATES_ROOT_ID);
        this.publicKeysRoot = this.tree.tree("find", this.PUBLIC_KEYS_ROOT_ID);

        this.importButton = $("a[role=button][name=import]", this.fieldset);
        this.importButton.linkbutton({onClick: $.proxy(this._onClick$ImportButton, this)});

        this.exportButton = $("a[role=button][name=export]", this.fieldset);
        this.exportButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ExportButton, this)});
        this.exportMenu = $("div[role=menu][aria-label=export-menu]", this.fieldset);
        this.exportMenu.menu({onClick: $.proxy(this._onClick$ExportMenu, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.keystore.HostKeysView.LISTENER;
        this._adjustSelection();
    },

    getCertificates: function() {
        var nodes = this.tree.tree("getChildren", this.certificatesRoot.target);
        return $.map(nodes || [], $.proxy(this._node2Key, this));
    },

    setCertificates: function (values) {
        this._updateChildren(this.certificatesRoot, values || []);
    },

    getPublicKeys: function() {
        var nodes = this.tree.tree("getChildren", this.publicKeysRoot.target);
        return $.map(nodes || [], $.proxy(this._node2Key, this));
    },

    setPublicKeys: function (values) {
        this._updateChildren(this.publicKeysRoot, values || []);
    },

    selectItem: function(value) {
        var node = this.tree.tree("find", this._key2Node(value).id);
        if (node) {
            this.tree.tree("select", node.target);
        }
    },

    _onClick$ImportButton: function() {
        this.listener.onImport();
    },

    _onClick$ExportButton: function () {
        if (this._publicKeySelected()) {
            this.listener.onExportPublicKey(this.selection);
        } else if (this._certificateSelected()) {
            this.exportMenu.menu("show", {alignTo: this.exportButton});
            this.exportButton.blur();
        }
    },

    _onClick$ExportMenu: function(item) {
        var type = $(item.target).attr("aria-label");
        if ("menu-crt" == type) {
            this.listener.onExportCertificate(this.selection);
        } else if ("menu-pubkey" == type) {
            this.listener.onExportCertificatePublicKey(this.selection);
        }
    },

    _onClick$DeleteButton: function() {
        if (this._publicKeySelected()) {
            this.listener.onDeletePublicKey(this.selection);
        } else if (this._certificateSelected()) {
            this.listener.onDeleteCertificate(this.selection);
        }
    },

    _key2Node: function (key) {
        return {id: key.alias, text: key.alias, iconCls: "icon-key", attributes: {key: key}};
    },

    _node2Key: function (node) {
        return node && node.attributes ? node.attributes.key : null;
    },

    _updateChildren: function(root, values) {
        if (!this.tree.tree("isLeaf", root.target)) {
            var children = this.tree.tree("getChildren", root.target);
            for (var i = 0; i < children.length; i++) {
                this.tree.tree("remove", children[i].target);
            }
        }
        this.tree.tree("append", {parent: root.target, data: $.map(values, $.proxy(this._key2Node, this))});
    },

    _adjustSelection: function() {
        var node = this.tree.tree("getSelected");
        this.selection = $.inArray(node, this.tree.tree("getRoots")) == -1 ? this._node2Key(node) : null;
        this._adjustKeyInfo();
        this._adjustButtonsState();
    },

    _adjustKeyInfo: function () {
        if (this.selection != null) {
            this.keyInfo.show(this.selection);
        } else {
            this.keyInfo.hide();
        }
    },

    _adjustButtonsState: function() {
        var enabled = this.selection != null;
        this.exportButton.linkbutton(enabled ? "enable" : "disable");
        this.deleteButton.linkbutton(enabled ? "enable" : "disable");
    },

    _certificateSelected: function() {
        var node = this.tree.tree("getSelected");
        return node != null && this.tree.tree("getParent", node.target) == this.certificatesRoot;
    },

    _publicKeySelected: function() {
        var node = this.tree.tree("getSelected");
        return node != null && this.tree.tree("getParent", node.target) == this.publicKeysRoot;
    }
});
jscape.keystore.HostKeysView.LISTENER = {
    onImport: $.noop,
    onExportPublicKey: $.noop,
    onExportCertificate: $.noop,
    onExportCertificatePublicKey: $.noop,
    onDeletePublicKey: $.noop,
    onDeleteCertificate: $.noop
};

/** @class */
jscape.keystore.HostKeyInfoView = jscape.keystore.BaseKeyInfoView.extend(
    /** @lends {jscape.keystore.HostKeyInfoView.prototype} */
{
    /** @constructor */
    initialize: function() {
        this._super($("fieldset[name=hostkeyinfofields]"));
    }
});

/** @class */
jscape.keystore.ImportHostKeyController = Class.extend(
    /** @lends {jscape.keystore.ImportHostKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService) {
        this.managementService = managementService;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function (dialog) {
        this.managementService.importHostKey(dialog.getAlias(), dialog.getFile())
                .done($.proxy(function (key) { this.deferred.resolve(key); }, this))
                .fail($.proxy(function () { this.deferred.reject(); }, this))
                .always($.proxy(function () {
                            dialog.destroy();
                            this.deferred = null;
                        }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function() {
        return new jscape.keystore.ImportHostKeyDialog(this.resources);
    }
});

/** @class */
jscape.keystore.ImportHostKeyDialog = jscape.keystore.ImportPublicKeyDialog.extend(
    /** @lends {jscape.keystore.ImportHostKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.alias.validatebox({required: true, validType: ["hostkey_nonempty", "hostkey_notexists"], missingMessage: resources.getString("KEY_MANAGER_HOST_KEYS_ERROR_BAD_ALIAS")});
        this.file.validatebox({required: true, missingMessage: resources.getString("KEY_MANAGER_HOST_KEYS_ERROR_BAD_FILE")});
    }
});

/** @class */
jscape.keystore.ExportHostPublicKeyController = Class.extend(
    /** @lends {jscape.keystore.ExportHostPublicKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, alias, isCertificate) {
        this.managementService = managementService;
        this.alias = alias;
        this.isCertificate = !!isCertificate;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var format = dialog.getFormat();
        var filename = dialog.getFilename();
        dialog.destroy();

        var promise = this.isCertificate
                ? this.managementService.exportHostCertificatePublicKey(this.alias, format, filename)
                : this.managementService.exportHostPublicKey(this.alias, format, filename);
        promise.done($.proxy(function () { this.deferred.resolve(); }, this))
                .fail($.proxy(function () { this.deferred.reject(); }, this))
                .always($.proxy(function () { this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function() {
        var dialog = new jscape.keystore.ExportPublicKeyDialog(this.resources);
        dialog.setFilename(this.alias + ".pub");
        return dialog;
    }
});

/** @class */
jscape.keystore.ExportHostCertificateController = Class.extend(
    /** @lends {jscape.keystore.ExportHostCertificateController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, alias) {
        this.managementService = managementService;
        this.alias = alias;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var format = dialog.getFormat();
        var filename = dialog.getFilename();
        dialog.destroy();

        this.managementService.exportHostCertificate(this.alias, format, filename)
                .done($.proxy(function() { this.deferred.resolve(); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this))
                .always($.proxy(function() { this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function() {
        var dialog = new jscape.keystore.ExportCertificateDialog(this.resources);
        dialog.setFilename(this.alias + ".crt");
        return dialog;
    }
});

/** @class */
jscape.keystore.DeleteHostKeyController = Class.extend(
    /** @lends {jscape.keystore.DeleteHostKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, alias, isCertificate) {
        this.managementService = managementService;
        this.deferred = $.Deferred();

        this._confirmDelete(alias, isCertificate)
                .done($.proxy(this._deleteConfirmed, this, alias, isCertificate))
                .fail($.proxy(function() { this.deferred.reject(); }, this));

        return this.deferred.promise();
    },

    _confirmDelete: function(alias, isCertificate) {
        if (!!isCertificate) {
            return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_HOST_KEYS_DELETE_CERTIFICATE_TITLE"),
                                                this.resources.getString("KEY_MANAGER_HOST_KEYS_DELETE_CERTIFICATE_MESSAGE", {0: alias}));
        } else {
            return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_HOST_KEYS_DELETE_PUBLIC_KEY_TITLE"),
                                                this.resources.getString("KEY_MANAGER_HOST_KEYS_DELETE_PUBLIC_KEY_MESSAGE", {0: alias}));
        }
    },

    _deleteConfirmed: function (alias, isCertificate) {
        var promise = !!isCertificate
                      ? this.managementService.deleteHostCertificate(alias)
                      : this.managementService.deleteHostPublicKey(alias);
        promise.done($.proxy(function () { this.deferred.resolve(); }, this))
                .fail($.proxy(function () { this.deferred.reject(); }, this));
    }
});