var jscape = jscape || {};
jscape.keystore = jscape.keystore || {};

/** @class */
jscape.keystore.KeystoreModule = jscape.ApplicationModule.extend(
    /** @lends {jscape.keystore.KeystoreModule.prototype} */
{
    keys: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    onCreate: function(managementService, keys) {
        this._super.apply(this, arguments);

        this.keys = keys;
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.keystore.KeyManager = jscape.Application.extend(
    /** @lends {jscape.keystore.KeyManager.prototype} */
{
    keys: null,

    /** @constructor */
    initialize: function(view, resources, modules) {
        this._super(modules);

        this.view = view;
        this.resources = resources;

        this._initValidators(resources);
    },

    /** @override */
    start: function(managementService) {
        this.managementService = managementService;
        var _super = this._super;

        this.managementService.getKeys()
                .done($.proxy(function(keys) { this.keys = keys; }, this))
                .always($.proxy(function() { _super.call(this, this.managementService, this.keys); }, this));
    },

    /** @override */
    onMenuItemSelected: function (item) {
        if (item && item.id) {
            switch (item.id) {
                case jscape.Menu.ITEM.Home:
                case jscape.Menu.ITEM.KeyManager:
                case jscape.Menu.ITEM.ServerSettings:
                    setTimeout($.proxy(function() { this.onRedirect(item.url || "/"); }, this), 10);
                    return true;
                case jscape.Menu.ITEM.Logout:
                    this.onLogout();
                    return true;
            }
        }
        return this._super(item);
    },

    /** @override */
    onReady: function() {
        this._super();
        this.view.show(this);
    },

    onViewStateChanged: function (id, index, visible) {
        this._super.call(this, index, visible);
    },

    onLogout: function() {
        this.managementService.logout();
    },

    onRedirect: function (url) {
        window.location = url;
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.rule("keystore_confirmpwd", $.proxy(this._passwordConfirmed, this), resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_CONFIRM_PASSWORD"));
    },

    _passwordConfirmed: function (value, params) {
        return value == $(params[0]).val();
    }
});

/** @class */
jscape.keystore.KeyManagerView = Class.extend(
    /** @lends {jscape.keystore.KeyManagerView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.throbber = $("#throbber");
        this.panel = $("#viewpane");
        this.panel.layout({fit: true, border: false}).css({visibility: "hidden"});

        this.modules = $("#key-manager[role=tablist]");
        this.modules.tabs({
            fit: true,
            border: true,
            tabPosition: "top",
            headerWidth: 150,
            onSelect: $.proxy(this._onSelect$Tab, this),
            onUnselect: $.proxy(this._onUnselect$Tab, this)
        });
        this.modules.find("div[role=tablist]")
                .filter(function() { return $(this).parents("div.dialog").length == 0; })
                .each(function(_, tablist) { $(tablist).tabs({border:false}); });
    },

    show: function(listener) {
        this.listener = listener || jscape.keystore.KeyManagerView.LISTENER;
        this.throbber.remove();
        this.throbber = null;
        this.panel.css({visibility: "visible"});
        this._adjustSelection();
    },

    valid: function() {
        return new jscape.Validator().validateTabs(this.panel);
    },

    _onSelect$Tab: function(title, index) {
        if (this.listener) {
            this.listener.onViewStateChanged(title, index, true);
        }
    },

    _onUnselect$Tab: function(title, index) {
        if (this.listener) {
            this.listener.onViewStateChanged(title, index, false);
        }
    },

    _adjustSelection: function() {
        var tab = this.modules.tabs("getSelected");
        var index = this.modules.tabs("getTabIndex", tab);
        this._onSelect$Tab(tab.title, index);
    }
});
jscape.keystore.KeyManagerView.LISTENER = {
    onViewStateChanged: $.noop
};

/** @class */
jscape.keystore.BaseKeyInfoView = Class.extend(
    /** @lends {jscape.keystore.BaseKeyInfoView.prototype} */
{
    /** @constructor */
    initialize: function(fieldset) {
        this.dateFormat = new jscape.DateFormat("{MM}-{dd}-{yy}");

        this.fieldset = fieldset.hide();

        this.algorithm = $("code[aria-label=algorithm]", this.fieldset);
        this.certificate = $("code[aria-label=certificate]", this.fieldset);
        this.version = $("code[aria-label=version]", this.fieldset);
        this.serial = $("code[aria-label=serialnum]", this.fieldset);
        this.issuer = $("code[aria-label=issuer]", this.fieldset);
        this.subject = $("code[aria-label=subject]", this.fieldset);
        this.validFromDate = $("code[aria-label=validfrom]", this.fieldset);
        this.validToDate = $("code[aria-label=validto]", this.fieldset);
        this.signature = $("code[aria-label=signature]", this.fieldset);
    },

    show: function(data) {
        if (data && data.certificateType) {
            this._showCertificate(data);
        } else {
            this._showPublicKey(data);
        }
        this.fieldset.show();
    },

    hide: function() {
        this.fieldset.hide();
    },

    _showCertificate: function (data) {
        this.algorithm.text(data.algorithm);
        this.certificate.text(data.certificateType);
        this.version.text(data.version);
        this.serial.text(data.serial);
        this.issuer.text(data.issuer);
        this.subject.text(data.subject);
        this.validFromDate.text(this._formatDate(data.validNotBefore));
        this.validToDate.text(this._formatDate(data.validNotAfter));
        this.signature.text(data.signature);
        this._adjustVisibility(true);
    },

    _showPublicKey: function(data) {
        this.algorithm.text(data.algorithm).show();
        this._adjustVisibility(false);
    },

    _adjustVisibility: function(visible) {
        $.each([this.certificate, this.version, this.serial, this.issuer, this.subject, this.validFromDate, this.validToDate, this.signature],
               function (_, item) {
                   if (visible) {
                       item.parent("dd").show().prev("dt").css("visibility", "visible");
                   } else {
                       item.parent("dd").hide().prev("dt").css("visibility", "hidden");
                   }
               }
        );
    },

    _formatDate: function (value) {
        return value ? this.dateFormat.format($.isNumeric(value) ? new Date(value) : value) : null;
    }
});

/** @class */
jscape.keystore.ImportPublicKeyDialog = Class.extend(
    /** @lends {jscape.keystore.ImportPublicKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-pubkey-import").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=importpubkeyfields]", this.dialog);

        this.alias = $("input[name=alias]", this.fieldset);
        this.alias.validatebox({required: true});

        this.file = $("input[name=file]", this.fieldset);
        this.file.validatebox({required: true}).change(function() { $(this).validatebox("validate"); });
    },

    getAlias: function() {
        return this.alias.val();
    },

    getFile: function() {
        return this.file[0].files[0];
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.alias.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.keystore.ExportDialog = Class.extend(
    /** @lends {jscape.keystore.ExportDialog.prototype} */
{
    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=exportkeyfields]", this.dialog);

        this.filename = $("input[name=filename]", this.fieldset);
        this.filename.validatebox({required: true, missingMessage: resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_BAD_FILENAME")});

        this.format = $("select[name=format]", this.fieldset);
        this.format.combobox({required: true, editable: false, panelHeight: "auto"});
    },

    getFilename: function() {
        return this.filename.val();
    },

    setFilename: function(value) {
        this.filename.val(value);
    },

    getFormat: function() {
        return this.format.combobox("getValue");
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.keystore.ExportPublicKeyDialog = jscape.keystore.ExportDialog.extend(
    /** @lends {jscape.keystore.ExportPublicKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-publickey-export"), resources);
    }
});

/** @class */
jscape.keystore.ExportPrivateKeyDialog = jscape.keystore.ExportDialog.extend(
    /** @lends {jscape.keystore.ExportPrivateKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-privatekey-export"), resources);

        var passwordId = "pwd_" + Math.random().toString(36).substr(2);
        this.password = $("input[name=password]", this.fieldset).attr("id", passwordId);
        this.password.on("input", $.proxy(function() {
            var enabled = this.getPassword().length != 0;
            this.confirmPassword.validatebox(enabled ? "enableValidation" : "disableValidation").validatebox("validate");
        }, this));

        this.confirmPassword = $("input[name=confirmpwd]", this.fieldset);
        this.confirmPassword.validatebox({required: true, validType: "keystore_confirmpwd['#" + passwordId + "']", missingMessage: resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_CONFIRM_PASSWORD")});
    },

    getPassword: function() {
        return this.password.val();
    },

    /** @0verride */
    show: function() {
        this._super.apply(this, arguments);
        this.password.triggerHandler("input");
    }
});

/** @class */
jscape.keystore.ExportCertificateDialog = jscape.keystore.ExportDialog.extend(
    /** @lends {jscape.keystore.ExportCertificateDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-certificate-export"), resources);
    }
});



/** @class */
jscape.keystore.KeyExpirationModule = jscape.keystore.KeystoreModule.extend(
    /** @lends {jscape.keystore.KeyExpirationModule.prototype} */
{
    keys: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.keystore.KeyExpirationView(resources);
    },

    onStart: function() {
        this.view.show(this);
        this._setupView();
    },

    onResume: function() {
        this._setupView();
    },

    _setupView: function() {
        var serverKeys = this.keys.getServerKeys();
        var clientKeys = this.keys.getClientKeys().concat(this.keys.getClientCertificates());
        var pgpKeys = this.keys.getPgpPublicKeys().concat(this.keys.getPgpSecretKeys());

        this.view.setKeys(serverKeys, clientKeys, pgpKeys);
    }
});

/** @class */
jscape.keystore.KeyExpirationView = Class.extend(
    /** @lends {jscape.keystore.KeyExpirationView.prototype} */
{
    /** @constructor */
    initialize: function (resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=keyexpiryreportfields]");

        this.report = $("table[role=grid][aria-label=keyexpiryreport]", this.fieldset);
        this.report.propertygrid({
            height: "auto",
            showGroup: true,
            columns: [[
                {field: "alias", title: $("thead th:nth-child(1)", this.report).text(), sortable: true, width: 100},
                {field: "expires", title: $("thead th:nth-child(2)", this.report).text(), width: 100, resizable: false, align: "right", formatter: $.proxy(function(_, row) { return this._formatExpirationtionDate(row); }, this), editor: null}
            ]]
        });
    },

    show: function(listener) {
        this.listener = listener || jscape.keystore.KeyExpirationView.LISTENER;
    },

    setKeys: function(serverKeys, clientKeys, pgpKeys) {
        var resources = this.resources;
        var data = $.map(serverKeys, function(key) { return $.extend({group: resources.getString("KEY_MANAGER_REPORT_GROUP_SERVER_KEYS")}, key); });
        data = data.concat($.map(clientKeys, function(key) { return $.extend({group: resources.getString("KEY_MANAGER_REPORT_GROUP_CLIENT_KEYS")}, key); }));
        data = data.concat($.map(pgpKeys, function(key) { return $.extend({group: resources.getString("KEY_MANAGER_REPORT_GROUP_PGP_KEYS")}, key); }));

        this.report.propertygrid("loading");
        this.report.propertygrid("loadData", data);
        this.report.propertygrid("loaded");
    },

    _formatExpirationtionDate: function (key) {
        return key.validNotAfter
                ? new Date(key.validNotAfter).toLocaleString()
                : this.resources.getString("KEY_MANAGER_REPORT_DATE_NON_EXPIRING");
    }
});