jscape.keystore = jscape.keystore || {};

/** @class */
jscape.keystore.PgpKeysModule = jscape.keystore.KeystoreModule.extend(
    /** @lends {jscape.keystore.PgpKeysModule.prototype} */
{

    onCreate: function() {
        this._super.apply(this, arguments);

        this.controller = new jscape.keystore.PgpKeysController(this.resources);
    },

    onStart: function() {
        this.controller.start(this.managementService, this.keys);
    }
});

/** @class */
jscape.keystore.PgpKeysController = Class.extend(
    /** @lends {jscape.keystore.PgpKeysController.prototype} */
{
    secretKeys: null,
    publicKeys: null,

    /** @constructor */
    initialize: function(resources) {
        this.view = new jscape.keystore.PgpKeysView(resources);
        this.importController = new jscape.keystore.ImportPgpKeyController(resources);
        this.exportPublicKeyController = new jscape.keystore.ExportPgpPublicKeyController(resources);
        this.exportSecretKeyController = new jscape.keystore.ExportPgpSecretKeyController(resources);
        this.generateController = new jscape.keystore.GeneratePgpKeyController(resources);
        this.signController = new jscape.keystore.SignPgpKeyController(resources);
        this.viewController = new jscape.keystore.ViewPgpKeySignaturesController(resources);
        this.deleteController = new jscape.keystore.DeletePgpKeyController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, keys) {
        this.managementService = managementService;
        this.secretKeys = keys.getPgpSecretKeys();
        this.publicKeys = keys.getPgpPublicKeys();

        this.view.setSecretKeys(this.secretKeys);
        this.view.setPublicKeys(this.publicKeys);
        this.view.show(this);
    },

    onImport: function() {
        this.importController.start(this.managementService)
                .done($.proxy(this._keyImported, this));
    },

    onGenerate: function() {
        this.generateController.start(this.managementService)
                .done($.proxy(this._keyGenerated, this));
    },

    onExportPublicKey: function(key) {
        this.exportPublicKeyController.start(this.managementService, key, false);
    },

    onExportSecretKey: function(key) {
        this.exportSecretKeyController.start(this.managementService, key);
    },

    onExportSecretPublicKey: function(key) {
        this.exportPublicKeyController.start(this.managementService, key, true);
    },

    onDeletePublicKey: function(key) {
        this.deleteController.start(this.managementService, key, false)
                .done($.proxy(this._keyDeleted, this, key, false));
    },

    onDeleteSecretKey: function(key) {
        this.deleteController.start(this.managementService, key, true)
                .done($.proxy(this._keyDeleted, this, key, true));
    },

    onSign: function(key) {
        this.signController.start(this.managementService, key, this.secretKeys);
    },

    onViewSecretKeySignatures: function(key) {
        this.viewController.start(this.managementService, key.alias, true);
    },

    onViewPublicKeySignatures: function(key) {
        this.viewController.start(this.managementService, key.alias, false);
    },

    _keyImported: function(key, isSecretKey) {
        if (!!isSecretKey) {
            this.secretKeys.push(key);
            this.view.setSecretKeys(this.secretKeys);
            this.view.selectItem(key);
        } else {
            this.publicKeys.push(key);
            this.view.setPublicKeys(this.publicKeys);
            this.view.selectItem(key);
        }
    },

    _keyGenerated: function(key) {
        this.secretKeys.push(key);
        this.view.setSecretKeys(this.secretKeys);
        this.view.selectItem(key);
    },

    _keyDeleted: function(key, isSecretKey) {
        var items = isSecretKey ? this.secretKeys : this.publicKeys;
        var index = $.inArray(key, items);
        if (index != -1) {
            items.splice(index, 1);
            if (isSecretKey) {
                this.view.setSecretKeys(items);
            } else {
                this.view.setPublicKeys(items);
            }
        }
        if (items.length) {
            index = Math.max(0, Math.min(index, items.length - 1));
            this.view.selectItem(items[index]);
        }
    },

    _initValidators: function (resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("pgp_nonempty_name", resources.getString("KEY_MANAGER_PGP_KEYS_ERROR_BAD_NAME"));
        v.rule("pgp_unique_key", $.proxy(this._keyAliasNotExists, this), resources.getString("KEY_MANAGER_PGP_KEYS_ERROR_ALIAS_EXISTS"));
    },

    _keyAliasNotExists: function(_, params) {
        var realName = $(params[0]).val();
        var email = $(params[1]).val();
        var alias = email.trim().length != 0 ? "{0} <{1}>".supplant({0: realName, 1: email}) : realName;
        var keys = [].concat(this.publicKeys).concat(this.secretKeys);

        for (var i = 0; i < keys.length; i++) {
            if (alias == keys[i].alias) {
                return false;
            }
        }
        return true;
    }
});

/** @class */
jscape.keystore.PgpKeysView = Class.extend(
    /** @lends {jscape.keystore.PgpKeysView.prototype} */
{
    SECRET_KEYS_ROOT_ID: "secret-keys",
    PUBLIC_KEYS_ROOT_ID: "public-keys",

    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=pgpkeysfields]");

        this.tree = $("ul[role=tree]", this.fieldset);
        this.tree.tree({lines: true, onSelect: $.proxy(this._adjustSelection, this)});
        this.secretKeysRoot = this.tree.tree("find", this.SECRET_KEYS_ROOT_ID);
        this.publicKeysRoot = this.tree.tree("find", this.PUBLIC_KEYS_ROOT_ID);

        this.keyInfo = new jscape.keystore.PgpKeyInfoView(resources);

        this.importButton = $("a[role=button][name=import]", this.fieldset);
        this.importButton.linkbutton({onClick: $.proxy(this._onClick$ImportButton, this)});

        this.exportButton = $("a[role=button][name=export]", this.fieldset);
        this.exportButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ExportButton, this)});
        this.exportMenu = $("div[role=menu][aria-label=export-menu]", this.fieldset);
        this.exportMenu.menu({onClick: $.proxy(this._onClick$ExportMenu, this)});

        this.signButton = $("a[role=button][name=sign]", this.fieldset);
        this.signButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$SignButton, this)});

        this.signaturesButton = $("a[role=button][name=signatures]", this.fieldset);
        this.signaturesButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$SignaturesButton, this)});

        this.generateButton = $("a[role=button][name=generate]", this.fieldset);
        this.generateButton.linkbutton({onClick: $.proxy(this._onClick$GenerateButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.selection = null;
        this.listener = listener || jscape.keystore.PgpKeysView.LISTENER;
        this._adjustSelection();
    },

    getPublicKeys: function() {
        var nodes = this.tree.tree("getChildren", this.publicKeysRoot.target);
        return $.map(nodes || [], $.proxy(this._node2Key, this));
    },

    setPublicKeys: function (values) {
        this._updateChildren(this.publicKeysRoot, values || []);
    },

    getSecretKeys: function() {
        var nodes = this.tree.tree("getChildren", this.secretKeysRoot.target);
        return $.map(nodes || [], $.proxy(this._node2Key, this));
    },

    setSecretKeys: function (values) {
        this._updateChildren(this.secretKeysRoot, values || []);
    },

    selectItem: function(value) {
        var node = this.tree.tree("find", this._key2Node(value).id);
        if (node) {
            this.tree.tree("select", node.target);
        }
    },

    _onClick$GenerateButton: function() {
        this.listener.onGenerate();
    },

    _onClick$ImportButton: function() {
        this.listener.onImport();
    },

    _onClick$ExportButton: function() {
        if (this._publicKeySelected()) {
            this.listener.onExportPublicKey(this.selection);
        } else if (this._secretKeySelected()) {
            this.exportMenu.menu("show", {alignTo: this.exportButton});
            this.exportButton.blur();
        }
    },

    _onClick$ExportMenu: function(item) {
        var type = $(item.target).attr("aria-label");
        if ("secret" == type) {
            this.listener.onExportSecretKey(this.selection);
        } else if ("pubkey" == type) {
            this.listener.onExportSecretPublicKey(this.selection);
        }
    },

    _onClick$SignButton: function() {
        if (this._publicKeySelected()) {
            this.listener.onSign(this.selection);
        }
    },

    _onClick$DeleteButton: function() {
        if (this._secretKeySelected()) {
            this.listener.onDeleteSecretKey(this.selection);
        } else if (this._publicKeySelected()) {
            this.listener.onDeletePublicKey(this.selection);
        }
    },

    _onClick$SignaturesButton: function() {
        if (this._secretKeySelected()) {
            this.listener.onViewSecretKeySignatures(this.selection);
        } else if (this._publicKeySelected()) {
            this.listener.onViewPublicKeySignatures(this.selection);
        }
    },

    _key2Node: function (key) {
        return {id: key.alias, text: key.alias, iconCls: "icon-key", attributes: {key: key}};
    },

    _node2Key: function (node) {
        return node && node.attributes ? node.attributes.key : null;
    },

    _updateChildren: function(root, values) {
        if (!this.tree.tree("isLeaf", root.target)) {
            var children = this.tree.tree("getChildren", root.target);
            for (var i = 0; i < children.length; i++) {
                this.tree.tree("remove", children[i].target);
            }
        }
        this.tree.tree("append", {parent: root.target, data: $.map(values, $.proxy(this._key2Node, this))});
    },

    _adjustSelection: function() {
        var node = this.tree.tree("getSelected");
        this.selection = $.inArray(node, this.tree.tree("getRoots")) == -1 ? this._node2Key(node) : null;
        this._adjustKeyInfo();
        this._adjustButtonsState();
    },

    _adjustKeyInfo: function () {
        if (this.selection != null) {
            this.keyInfo.show(this.selection);
        } else {
            this.keyInfo.hide();
        }
    },

    _adjustButtonsState: function() {
        this.exportButton.linkbutton(this.selection != null ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection != null ? "enable" : "disable");
        this.signButton.linkbutton(this._publicKeySelected() && this.getSecretKeys().length != 0 ? "enable" : "disable");
        this.signaturesButton.linkbutton(this.selection != null ? "enable" : "disable");
    },

    _secretKeySelected: function() {
        var node = this.tree.tree("getSelected");
        return node != null && this.tree.tree("getParent", node.target) == this.secretKeysRoot;
    },

    _publicKeySelected: function() {
        var node = this.tree.tree("getSelected");
        return node != null && this.tree.tree("getParent", node.target) == this.publicKeysRoot;
    }
});
jscape.keystore.PgpKeysView.LISTENER = {
    onImport: $.noop,
    onGenerate: $.noop,
    onExportSecretKey: $.noop,
    onExportPublicKey: $.noop,
    onDeleteSecretKey: $.noop,
    onDeletePublicKey: $.noop,
    onSign: $.noop,
    onViewSecretKeySignatures: $.noop,
    onViewPublicKeySignatures: $.noop
};

/** @class */
jscape.keystore.PgpKeyInfoView = Class.extend(
    /** @lends {jscape.keystore.PgpKeyInfoView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dateFormat = new jscape.DateFormat("{MM}-{dd}-{yy}");

        this.fieldset = $("fieldset[name=pgpkeyinfofields]").hide();

        this.algorithm = $("code[aria-label=algorithm]", this.fieldset);
        this.canEncrypt = $("code[aria-label=encrypt]", this.fieldset);
        this.canDecrypt = $("code[aria-label=decrypt]", this.fieldset);
        this.canSign = $("code[aria-label=sign]", this.fieldset);
        this.canVerify = $("code[aria-label=verify]", this.fieldset);
        this.fingerprint = $("code[aria-label=fingerprint]", this.fieldset);
    },

    show: function(data) {
        this.fieldset.show();
        this._setData(data || {});
    },

    hide: function() {
        this.fieldset.hide();
    },

    _setData: function (data) {
        this.algorithm.text(data.algorithm || "");
        this.canEncrypt.text(this._formatBoolean(data.canEncrypt));
        this.canDecrypt.text(this._formatBoolean(data.canDecrypt));
        this.canSign.text(this._formatBoolean(data.canSign));
        this.canVerify.text(this._formatBoolean(data.canVerify));
        this.fingerprint.text(data.fingerprint || "");
    },

    _formatDate: function (value) {
        return value ? this.dateFormat.format(typeof value === "number" ? new Date(value) : value) : null;
    },

    _formatBoolean: function (value) {
        return !!value ? "yes" : "no"
    }
});

/** @class */
jscape.keystore.ImportPgpKeyController = Class.extend(
    /** @lends {jscape.keystore.ImportPgpKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService) {
        this.managementService = managementService;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        if (dialog.isSecretKey()) {
            this.managementService.importPgpSecretKey(dialog.getFileSource(), dialog.getFilePassword())
                    .done($.proxy(function(key) { this.deferred.resolve(key, true); }, this))
                    .always(function() { dialog.destroy(); });
        } else if (dialog.isPublicKey()) {
            this.managementService.importPgpPublicKey(dialog.getFileSource(), dialog.getFilePassword())
                    .done($.proxy(function(key) { this.deferred.resolve(key, false); }, this))
                    .always(function() { dialog.destroy(); });
        } else {
            dialog.destroy();
            this.deferred.reject();
        }
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _setupDialog: function() {
        return new jscape.keystore.ImportPgpKeyDialog(this.resources);
    }
});

/** @class */
jscape.keystore.ImportPgpKeyDialog = Class.extend(
    /** @lends {jscape.keystore.ImportPgpKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-pgpkey-import").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=importpgpkeyfields]", this.dialog);

        this.type = $("input[name=type]:radio", this.fieldset);
        this.type.change($.proxy(this._adjustFieldState, this));

        this.file = $("input[name=filesource]", this.fieldset);
        this.file.validatebox({required: true, missingMessage: resources.getString("KEY_MANAGER_PGP_KEYS_ERROR_BAD_FILE")});

        this.password = $("input[name=filepwd]", this.fieldset);
        this.password.validatebox({required: true, missingMessage: resources.getString("KEY_MANAGER_PGP_KEYS_ERROR_BAD_FILE_PASSWORD")});
    },

    isPublicKey: function() {
        return this.type.filter("[value=public]").is(":checked");
    },

    isSecretKey: function() {
        return this.type.filter("[value=secret]").is(":checked");
    },

    getFileSource: function() {
        return this.file[0].files[0];
    },

    getFilePassword: function() {
        return this.password.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.type.filter(":first").prop("checked", true).change().focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _adjustFieldState: function() {
        var enabled = this.isSecretKey();
        this.password.prop("disabled", !enabled)
                .validatebox(enabled ? "enableValidation" : "disableValidation")
                .validatebox("validate");
    }
});

/** @class */
jscape.keystore.ExportPgpPublicKeyController = Class.extend(
    /** @lends {jscape.keystore.ExportPgpPublicKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, key, isSecretKey) {
        this.managementService = managementService;
        this.keyAlias = key.alias;
        this.secretKey = !!isSecretKey;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var filename = dialog.getFilename();
        dialog.destroy();

        (this.secretKey
            ? this.managementService.exportPgpSecretPublicKey(this.keyAlias, filename)
            : this.managementService.exportPgpPublicKey(this.keyAlias, filename))
                .done($.proxy(function() { this.deferred.resolve(); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _setupDialog: function() {
        var dialog = new jscape.keystore.ExportPgpPublicKeyDialog(this.resources);
        dialog.setFilename("pgp-key.pub");
        return dialog;

    }
});

/** @class */
jscape.keystore.ExportPgpPublicKeyDialog = Class.extend(
    /** @lends {jscape.keystore.ExportPgpPublicKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-pgpkey-exportpublic").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=exportpgpkeyfields]", this.dialog);

        this.filename = $("input[name=filename]", this.fieldset);
        this.filename.validatebox({required: true, missingMessage: resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_BAD_FILENAME")});
    },

    getFilename: function() {
        return this.filename.val();
    },

    setFilename: function(value) {
        this.filename.val(value);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.keystore.ExportPgpSecretKeyController = Class.extend(
    /** @lends {jscape.keystore.ExportPgpSecretKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, key) {
        this.managementService = managementService;
        this.keyAlias = key.alias;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var filename = dialog.getFilename();
        var password = dialog.getPassword();
        dialog.destroy();

        this.managementService.exportPgpSecretKey(this.keyAlias, filename, password)
                .done($.proxy(function() { this.deferred.resolve(); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _setupDialog: function() {
        var dialog = new jscape.keystore.ExportPgpSecretKeyDialog(this.resources);
        dialog.setFilename("pgp-key.sec");
        return dialog;
    }
});

/** @class */
jscape.keystore.ExportPgpSecretKeyDialog = Class.extend(
    /** @lends {jscape.keystore.ExportPgpSecretKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-pgpkey-exportsecret").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=exportpgpkeyfields]", this.dialog);

        this.filename = $("input[name=filename]", this.fieldset);
        this.filename.validatebox({required: true, missingMessage: resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_BAD_FILENAME")});

        this.password = $("input[name=filepwd]", this.fieldset);
        this.password.validatebox({required: true, missingMessage: resources.getString("KEY_MANAGER_EXPORT_KEYS_ERROR_BAD_FILE_PASSWORD")});
    },

    getFilename: function() {
        return this.filename.val();
    },

    setFilename: function(value) {
        this.filename.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.filename.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.keystore.GeneratePgpKeyController = Class.extend(
    /** @lends {jscape.keystore.GeneratePgpKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService) {
        this.managementService = managementService;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var realName = dialog.getRealName();
        var email = dialog.getEmail();
        var keyType = dialog.getAlgorithm();
        var keyLength = dialog.getKeyLength();
        dialog.destroy();

        this.managementService.generatePgpKey(realName, email, keyType, keyLength)
                .done($.proxy(function(key) { this.deferred.resolve(key); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _setupDialog: function() {
        var dialog = new jscape.keystore.GeneratePgpKeyDialog(this.resources);
        dialog.setAlgorithms(jscape.KeyPairAlgorithm.values());
        return dialog;
    }
});

/** @class */
jscape.keystore.GeneratePgpKeyDialog = Class.extend(
    /** @lends {jscape.keystore.GeneratePgpKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-pgpkey-generate").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=generatepgpkeyfields]", this.dialog);

        var realNameId = "realname_" + $.now().toString(36);
        var emailId = "email_" + $.now().toString(36);
        this.realName = $("input[name=realname]", this.fieldset).attr("id", realNameId);
        this.email = $("input[name=email]", this.fieldset).attr("id", emailId);
        this.realName.validatebox({
            required: true,
            validType: ["pgp_nonempty_name", "pgp_unique_key['#{0}','#{1}']".supplant({0: realNameId, 1: emailId})],
            missingMessage: resources.getString("KEY_MANAGER_PGP_KEYS_ERROR_BAD_NAME")
        });
        this.email.validatebox({validType: ["email", "pgp_unique_key['#{0}','#{1}']".supplant({0: realNameId, 1: emailId})]});
        this.realName.on("input", $.proxy(function() { this.email.validatebox("validate");}, this));
        this.email.on("input", $.proxy(function() { this.realName.validatebox("validate");}, this));

        this.keyAlgorithm = $("select[name=keytype]", this.fieldset);
        this.keyAlgorithm.combobox({required: true, editable: false, width: 80, panelHeight: "auto", onChange: $.proxy(this._adjustKeyLength, this)});

        this.keyLength = $("select[name=keylength]", this.fieldset);
        this.keyLength.combobox({required: true, editable: false, width: 80, panelHeight: "auto"});
    },

    getRealName: function() {
        return this.realName.val();
    },

    getEmail: function() {
        return this.email.val();
    },

    getAlgorithm: function() {
        return this.keyAlgorithm.combobox("getValue");
    },

    setAlgorithms: function(values) {
        var data = $.map(values || [], function(item) { return {text: item.name, value: item.name}; });
        this.keyAlgorithm.combobox("loadData", data);
    },

    getKeyLength: function() {
        return this.keyLength.combobox("getValue");
    },

    setKeyLengths: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.keyLength.combobox("loadData", data);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.realName.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _adjustKeyLength: function() {
        this.setKeyLengths(jscape.KeyPairAlgorithm.lengthsOf(this.getAlgorithm()));
    }
});

/** @class */
jscape.keystore.ViewPgpKeySignaturesController = Class.extend(
    /** @lends {jscape.keystore.ViewPgpKeySignaturesController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, keyAlias, isSecretKey) {
        this.managementService = managementService;
        this.keyAlias = keyAlias;
        this.secretKey = !!isSecretKey;
        this.deferred = $.Deferred();

        this._loadData()
                .done($.proxy(function(signatures) { this._setupDialog(signatures).show(this); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this));

        return this.deferred.promise();
    },

    onClose: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _loadData: function() {
        if (this.secretKey) {
            return this.managementService.getPgpSecretKeySignatures(this.keyAlias);
        } else {
            return this.managementService.getPgpPublicKeySignatures(this.keyAlias);
        }
    },

    _setupDialog: function(signatures) {
        var dialog = new jscape.keystore.PgpKeySignaturesDialog(this.resources);
        dialog.setSignatures(signatures);
        return dialog;
    }
});

/** @class */
jscape.keystore.PgpKeySignaturesDialog = Class.extend(
    /** @lends {jscape.keystore.PgpKeySignaturesDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.dateFormat = new jscape.DateFormat("{MM}-{dd}-{yy}");

        this.dialog = $("#d-pgpkey-signatures").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [{text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onClose, this)}],
            onClose: $.proxy(function () {if (!this.closing) { this._onClose(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=pgpkeysignaturesfields]", this.dialog);
        this.template = $("fieldset[name=pgpkeysignatureentryfields]", this.dialog);
    },

    setSignatures: function(values) {
        for (var i = 0, size = values.length; i < size; i++) {
            var signature = values[i];
            this.template.find("code[aria-label=number]").text(i + 1);
            this.template.find("code[aria-label=signingkey]").text(signature.signingKey);
            this.template.find("code[aria-label=creationtime]").text(this.dateFormat.format(new Date(signature.creationTime)));
            this.template.find("code[aria-label=type]").text(this.resources.getString("KEY_MANAGER_PGP_KEYS_SIGNATURE_TYPE_" + signature.type));
            this.template.find("code[aria-label=version]").text(signature.version);
            if (i < size - 1) {
                this.template = this.template.clone().appendTo(this.fieldset);
            }
        }
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClose: function () {
        this.hide();
        this.listener.onClose(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.keystore.SignPgpKeyController = Class.extend(
    /** @lends {jscape.keystore.SignPgpKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, key, signingKeys) {
        this.managementService = managementService;
        this.keyAlias = key.alias;
        this.deferred = $.Deferred();
        this._setupDialog(signingKeys).show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var signingKey = dialog.getSigningKey();
        dialog.destroy();

        this.managementService.signPgpPublicKey(this.keyAlias, signingKey)
                .done($.proxy(function(key) { this.deferred.resolve(key); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _setupDialog: function(signingKeys) {
        var dialog = new jscape.keystore.SignPgpKeyDialog(this.resources);
        dialog.setSigningKeys(signingKeys);
        return dialog;
    }
});

/** @class */
jscape.keystore.SignPgpKeyDialog = Class.extend(
    /** @lends {jscape.keystore.SignPgpKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-pgpkey-sign").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=signpgpkeyfields]", this.dialog);

        this.signingKey = $("select[name=signingkey]", this.fieldset);
        this.signingKey.combobox({required: true, editable: false, width: 180, panelHeight: "auto"});
    },

    getSigningKey: function () {
        return this.signingKey.combobox("getValue");
    },

    setSigningKeys: function (values) {
        var data = $.map(values || [], function(key) { return {text: key.alias, value: key.alias}; });
        this.signingKey.combobox("loadData", data);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.signingKey.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.keystore.DeletePgpKeyController = Class.extend(
    /** @lends {jscape.keystore.DeletePgpKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, key, isSecretKey) {
        this.managementService = managementService;
        this.alias = key.alias;
        this.secretKey = !!isSecretKey;
        this.deferred = $.Deferred();

        this._confirmDelete()
                .done($.proxy(this._deleteConfirmed, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this));

        return this.deferred.promise();
    },

    _confirmDelete: function() {
        return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_PGP_KEYS_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("KEY_MANAGER_PGP_KEYS_CONFIRM_DELETE_MESSAGE", {0: this.alias}));
    },

    _deleteConfirmed: function() {
        var promise = this.secretKey
                      ? this.managementService.deletePgpSecretKey(this.alias)
                      : this.managementService.deletePgpPublicKey(this.alias);
        $.when.call(jQuery, promise)
                .done($.proxy(function() { this.deferred.resolve(); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this));
    }
});