jscape.keystore = jscape.keystore || {};

/** @class */
jscape.keystore.ServerKeysModule = jscape.keystore.KeystoreModule.extend(
    /** @lends {jscape.keystore.ServerKeysModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.controller = new jscape.keystore.ServerKeysController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.keys);
    }
});

/** @class */
jscape.keystore.ServerKeysController = Class.extend(
    /** @lends {jscape.keystore.ServerKeysController.prototype} */
{
    serverKeys: null,

    /** @constructor */
    initialize: function(resources) {
        this.view = new jscape.keystore.ServerKeysView(resources);
        this.importKeyController = new jscape.keystore.ImportServerKeyController(resources);
        this.importCertificateController = new jscape.keystore.ImportServerCertificateController(resources);
        this.exportPublicKeyController = new jscape.keystore.ExportServerPublicKeyController(resources);
        this.exportPrivateKeyController = new jscape.keystore.ExportServerPrivateKeyController(resources);
        this.exportCertificateController = new jscape.keystore.ExportServerCertificateController(resources);
        this.generateKeyController = new jscape.keystore.GenerateServerKeyController(resources);
        this.generateCsrController = new jscape.keystore.GenerateServerKeyCsrController(resources);
        this.deleteController = new jscape.keystore.DeleteServerKeyController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, keys) {
        this.managementService = managementService;
        this.serverKeys = keys.getServerKeys();

        this.view.setKeys(this.serverKeys);
        this.view.show(this);
    },

    onImport: function() {
        this.importKeyController.start(this.managementService)
                .done($.proxy(this._keyImported, this));
    },

    onImportCertificates: function(key) {
        this.importCertificateController.start(this.managementService, key.alias);
    },

    onExportPrivateKey: function(key) {
        this.exportPrivateKeyController.start(this.managementService, key.alias);
    },

    onExportPublicKey: function(key) {
        this.exportPublicKeyController.start(this.managementService, key.alias);
    },

    onExportCertificate: function(certificate) {
        this.exportCertificateController.start(this.managementService, certificate.alias);
    },

    onGenerate: function() {
        this.generateKeyController.start(this.managementService)
                .done($.proxy(this._keyGenerated, this));
    },

    onGenerateCsr: function(key) {
        this.generateCsrController.start(this.managementService, key.alias);
    },

    onDelete: function(key) {
        this.deleteController.start(this.managementService, key.alias)
                .done($.proxy(this._keyDeleted, this, key));
    },

    _keyImported: function(key) {
        this.serverKeys.push(key);
        this.view.setKeys(this.serverKeys);
        this.view.selectKey(key);
    },

    _keyGenerated: function(key) {
        this.serverKeys.push(key);
        this.view.setKeys(this.serverKeys);
        this.view.selectKey(key);
    },

    _keyDeleted: function(key) {
        var index = $.inArray(key, this.serverKeys);
        if (index != -1) {
            this.serverKeys.splice(index, 1);
            this.view.setKeys(this.serverKeys);
        }
        if (this.serverKeys.length) {
            index = Math.max(0, Math.min(index, this.serverKeys.length - 1));
            this.view.selectKey(this.serverKeys[index]);
        }
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("serverkeys_filename_nonempty", resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_FILENAME"));
        v.nonEmptyRule("serverkeys_file_nonempty", resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_FILE"));
        v.nonEmptyRule("serverkeys_alias_nonempty", resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_ALIAS"));
        v.rule("serverkeys_notexists", $.proxy(this._keyAliasNotExists, this), resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_ALIAS_EXISTS"));
    },

    _keyAliasNotExists: function(value, excludes) {
        var keys = this.serverKeys;
        for (var i = 0; i < keys.length; i++) {
            var alias = keys[i].alias;
            if (excludes && $.inArray(alias, excludes) != -1) {
                continue;
            }
            if (alias == value) {
                return false;
            }
        }
        return true;
    }
});

/** @class */
jscape.keystore.ServerKeysView = Class.extend(
    /** @lends {jscape.keystore.ServerKeysView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=serverkeysfields]");

        this.tree = $("ul[role=tree]", this.fieldset);
        this.tree.tree({lines: true, onSelect: $.proxy(this._adjustSelection, this), onLoadSuccess: $.proxy(this._adjustSelection, this)});

        this.keyInfo = new jscape.keystore.ServerKeyInfo(resources);

        this.importButton = $("a[role=button][name=import]", this.fieldset);
        this.importButton.linkbutton({onClick: $.proxy(this._onClick$ImportButton, this)});

        this.importCertificatesButton = $("a[role=button][name=importcrt]", this.fieldset);
        this.importCertificatesButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ImportCertificatesButton, this)});

        this.exportButton = $("a[role=button][name=export]", this.fieldset);
        this.exportButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ExportButton, this)});
        this.exportMenu = $("div[role=menu][aria-label=export-menu]", this.fieldset);
        this.exportMenu.menu({onClick: $.proxy(this._onMenu$Export, this)});

        this.generateButton = $("a[role=button][name=generate]", this.fieldset);
        this.generateButton.linkbutton({onClick: $.proxy(this._onClick$GenerateButton, this)});

        this.generateCsrButton = $("a[role=button][name=generatecsr]", this.fieldset);
        this.generateCsrButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$GenerateCsrButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.keystore.ServerKeysView.LISTENER;
        this._adjustSelection();
    },

    getKeys: function() {
        return $.map(this.tree.tree("getChildren", null), $.proxy(this._node2Key, this));
    },

    setKeys: function(values) {
        this.tree.tree("loadData", $.map(values || [], $.proxy(this._key2Node, this)));
    },

    selectKey: function (value) {
        var node = this.tree.tree("find", this._key2Node(value).id);
        if (node) {
            this.tree.tree("select", node.target);
        }
    },

    _onClick$ExportButton: function() {
        this.exportMenu.menu("show", {alignTo: this.exportButton});
        this.exportButton.blur();
    },

    _onClick$ImportButton: function () {
        this.listener.onImport();
    },

    _onClick$ImportCertificatesButton: function () {
        this.listener.onImportCertificates(this.selection);
    },

    _onClick$GenerateButton: function() {
        this.listener.onGenerate();
    },

    _onClick$GenerateCsrButton: function() {
        this.listener.onGenerateCsr(this.selection);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDelete(this.selection);
    },

    _onMenu$Export: function(item) {
        var type = $(item.target).attr("aria-label");
        if ("menu-crt" == type) {
            this.listener.onExportCertificate(this.selection);
        } else if ("menu-pubkey" == type) {
            this.listener.onExportPublicKey(this.selection);
        } else if ("menu-prvkey" == type) {
            this.listener.onExportPrivateKey(this.selection);
        }
    },

    _key2Node: function (key) {
        return {id: key.alias, text: key.alias, iconCls: "icon-key", attributes: {key: key}};
    },

    _node2Key: function (node) {
        return node && node.attributes ? node.attributes.key : null;
    },

    _adjustSelection: function () {
        var node = this.tree.tree("getSelected");
        this.selection = node != null ? this._node2Key(node) : null;
        this._adjustKeyInfo();
        this._adjustButtonsState();
    },

    _adjustKeyInfo: function() {
        if (this.selection != null) {
            this.keyInfo.show(this.selection);
        } else {
            this.keyInfo.hide();
        }
    },

    _adjustButtonsState: function() {
        var enabled = this.selection != null;
        this.importCertificatesButton.linkbutton(enabled ? "enable" : "disable");
        this.exportButton.linkbutton(enabled ? "enable" : "disable");
        this.generateCsrButton.linkbutton(enabled ? "enable" : "disable");
        this.deleteButton.linkbutton(enabled ? "enable" : "disable");
    }
});
jscape.keystore.ServerKeysView.LISTENER = {
    onImport: $.noop,
    onImportCertificates: $.noop,
    onExportCertificate: $.noop,
    onExportPublicKey: $.noop,
    onExportPrivateKey: $.noop,
    onGenerate: $.noop,
    onGenerateCsr: $.noop,
    onDelete: $.noop
};

/** @class */
jscape.keystore.ServerKeyInfo = Class.extend(
    /** @lends {jscape.keystore.ServerKeyInfo.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dateFormat = new jscape.DateFormat("{MM}-{dd}-{yy}");

        this.fieldset = $("fieldset[name=serverkeyinfofields]").hide();

        this.algorithm = $("code[aria-label=algorithm]", this.fieldset);
        this.certificate = $("code[aria-label=certificate]", this.fieldset);
        this.version = $("code[aria-label=version]", this.fieldset);
        this.serial = $("code[aria-label=serialnum]", this.fieldset);
        this.issuer = $("code[aria-label=issuer]", this.fieldset);
        this.subject = $("code[aria-label=subject]", this.fieldset);
        this.validFromDate = $("code[aria-label=validfrom]", this.fieldset);
        this.validToDate = $("code[aria-label=validto]", this.fieldset);
        this.signature = $("code[aria-label=signature]", this.fieldset);
        this.domains = $("code[aria-label=domains]", this.fieldset);
    },

    show: function(data) {
        this.fieldset.show();
        this._setData(data || {});
    },

    hide: function() {
        this.fieldset.hide();
    },

    _setData: function (data) {
        this.algorithm.text(data.algorithm || "");
        this.certificate.text(data.certificateType || "");
        this.version.text(data.version || "");
        this.serial.text(data.serial || "");
        this.issuer.text(data.issuer || "");
        this.subject.text(data.subject || "");
        this.validFromDate.text(this._formatDate(data.validNotBefore));
        this.validToDate.text(this._formatDate(data.validNotAfter));
        this.signature.text(data.signature || "");
        this.domains.text((data.domains || []).join(", "));
    },

    _formatDate: function (value) {
        return value ? this.dateFormat.format(typeof value === "number" ? new Date(value) : value) : null;
    }
});

/** @class */
jscape.keystore.ImportServerKeyController = Class.extend(
    /** @lends {jscape.keystore.ImportServerKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService) {
        this.managementService = managementService;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        this.managementService.importServerKey(dialog.getAlias(),
                                               dialog.getFile(),
                                               dialog.getFilePassword(),
                                               dialog.getKeyAlias(),
                                               dialog.getKeyPassword(),
                                               dialog.getCertificateFile(),
                                               dialog.getCertificateFilePassword(),
                                               dialog.getCerfiticateAlias())
                .done($.proxy(function (key) { this.deferred.resolve(key); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this))
                .always($.proxy(function() { dialog.destroy(); this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function() {
        return new jscape.keystore.ImportServerKeyDialog(this.resources);
    }
});

/** @class */
jscape.keystore.ImportServerKeyDialog = Class.extend(
    /** @lends {jscape.keystore.ImportServerKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-serverkey-import").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=importserverkeyfields]", this.dialog);

        this.alias = $("input[name=alias]", this.fieldset);
        this.alias.validatebox({required: true, validType: "serverkeys_alias_nonempty", missingMessage: resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_ALIAS")});

        this.file = $("input[name=file]", this.fieldset);
        this.file.validatebox({required: true, validType: "serverkeys_file_nonempty", missingMessage: resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_FILE")});
        this.file.change(function() { $(this).validatebox("validate"); });

        this.filePassword = $("input[name=filepwd]", this.fieldset);
        this.keyAlias = $("input[name=keyalias]", this.fieldset);
        this.keyPassword = $("input[name=keypwd]", this.fieldset);

        this.certificateFile = $("input[name=crtfile]", this.fieldset);
        this.certificateFilePassword = $("input[name=crtfilepwd]", this.fieldset);
        this.certificateAlias = $("input[name=crtalias]", this.fieldset);
    },

    getAlias: function() {
        return this.alias.val();
    },

    setAlias: function(value) {
        this.alias.val(value);
        if (value) {
            this.alias.validatebox("options").validType = ["serverkeys_alias_nonempty", "serverkeys_notexists['" + value + "']"];
        }
    },

    getFile: function () {
        var files = this.file[0].files;
        return files && files.length ? files[0] : null;
    },

    getFilePassword: function () {
        return this.filePassword.val();
    },

    getKeyAlias: function () {
        return this.keyAlias.val();
    },

    getKeyPassword: function () {
        return this.keyPassword.val();
    },

    getCertificateFile: function() {
        var files = this.certificateFile[0].files;
        return files && files.length ? files[0] : null;
    },

    getCertificateFilePassword: function() {
        return this.certificateFilePassword.val();
    },

    getCerfiticateAlias: function() {
        return this.certificateAlias.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.alias.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.keystore.ImportServerCertificateController = Class.extend(
    /** @lends {jscape.keystore.ImportServerCertificateController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, keyAlias) {
        this.managementService = managementService;
        this.keyAlias = keyAlias;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        this.managementService.importServerKeyCertificates(this.keyAlias,
                                                           dialog.getFile(),
                                                           dialog.getFilePassword(),
                                                           dialog.getAlias())
                .done($.proxy(function () { this.deferred.resolve(); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this))
                .always($.proxy(function() { dialog.destroy(); this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function() {
        return new jscape.keystore.ImportServerCertificatesDialog(this.resources);
    }
});

/** @class */
jscape.keystore.ImportServerCertificatesDialog = Class.extend(
    /** @lends {jscape.keystore.ImportServerCertificatesDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-serverkey-importcrt").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=importserverkeycrtfields]", this.dialog);

        this.file = $("input[name=file]", this.fieldset);
        this.file.validatebox({required: true, validType: "serverkeys_file_nonempty", missingMessage: resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_FILE")});

        this.filePassword = $("input[name=filepwd]", this.fieldset);
        this.alias = $("input[name=alias]", this.fieldset);
    },

    getFile: function() {
        var files = this.file[0].files;
        return files && files[0];
    },

    getFilePassword: function() {
        return this.filePassword.val();
    },

    getAlias: function() {
        return this.alias.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.file.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.keystore.ExportServerPublicKeyController = Class.extend(
    /** @lends {jscape.keystore.ExportServerPublicKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, keyAlias) {
        this.managementService = managementService;
        this.keyAlias = keyAlias;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        this.managementService.exportServerPublicKey(this.keyAlias,
                                                     dialog.getFormat(),
                                                     dialog.getFilename())
                .done($.proxy(function () { this.deferred.resolve(); }, this))
                .fail($.proxy(function () { this.deferred.reject(); }, this))
                .always($.proxy(function () { dialog.destroy(); this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function () {
        var dialog = new jscape.keystore.ExportPublicKeyDialog(this.resources);
        dialog.setFilename(this.keyAlias + ".pub");
        return dialog;
    }
});

/** @class */
jscape.keystore.ExportServerPrivateKeyController = Class.extend(
    /** @lends {jscape.keystore.ExportServerPrivateKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, keyAlias) {
        this.managementService = managementService;
        this.keyAlias = keyAlias;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        this.managementService.exportServerPrivateKey(this.keyAlias,
                                                      dialog.getFormat(),
                                                      dialog.getPassword(),
                                                      dialog.getFilename())
                .done($.proxy(function () { this.deferred.resolve(); }, this))
                .fail($.proxy(function () { this.deferred.reject(); }, this))
                .always($.proxy(function () { dialog.destroy(); this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function () {
        var dialog = new jscape.keystore.ExportPrivateKeyDialog(this.resources);
        dialog.setFilename(this.keyAlias + ".prv");
        return dialog;
    }
});


/** @class */
jscape.keystore.ExportServerCertificateController = Class.extend(
    /** @lends {jscape.keystore.ExportServerCertificateController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, alias) {
        this.managementService = managementService;
        this.alias = alias;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        this.managementService.exportServerCertificate(this.alias,
                                                       dialog.getFormat(),
                                                       dialog.getFilename())
                .done($.proxy(function () { this.deferred.resolve(); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this))
                .always($.proxy(function() { dialog.destroy(); this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function() {
        var dialog = new jscape.keystore.ExportCertificateDialog(this.resources);
        dialog.setFilename(this.alias + ".crt");
        return dialog;
    }
});

/** @class */
jscape.keystore.GenerateServerKeyController = Class.extend(
    /** @lends {jscape.keystore.GenerateServerKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService) {
        this.managementService = managementService;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        this.managementService.generateServerKey(dialog.getAlias(),
                                                 dialog.getAlgorithm(),
                                                 dialog.getKeyLength(),
                                                 dialog.getPeriod(),
                                                 dialog.getCommonName(),
                                                 dialog.getOrganizationUnit(),
                                                 dialog.getOrganization(),
                                                 dialog.getLocality(),
                                                 dialog.getState(),
                                                 dialog.getCountry())
                .done($.proxy(function(key) { this.deferred.resolve(key); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this))
                .always($.proxy(function() { dialog.destroy(); this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function () {
        var dialog = new jscape.keystore.GenerateServerKeyDialog(this.resources);
        dialog.setAlgorithms(jscape.KeyPairAlgorithm.values());
        return dialog;
    }
});

/** @class */
jscape.keystore.GenerateServerKeyDialog = Class.extend(
    /** @lends {jscape.keystore.GenerateServerKeyDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-serverkey-generate").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=generateserverkeyfields]", this.dialog);

        this.alias = $("input[name=alias]", this.fieldset);
        this.alias.validatebox({required: true, validType: ["serverkeys_alias_nonempty", "serverkeys_notexists"], missingMessage: resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_ALIAS")});

        this.keyAlgorithm = $("select[name=algorithm]", this.fieldset);
        this.keyAlgorithm.combobox({editable: false, width: 80, panelHeight: "auto", onChange: $.proxy(this._adjustKeyLength, this)});

        this.keyLength = $("select[name=length]", this.fieldset);
        this.keyLength.combobox({editable: false, width: 80, panelHeight: "auto"});

        this.period = $("input[name=period]", this.fieldset);
        this.period.numberspinner({min: 1, max: 10000, value: 365, increment: 10, width: 80});

        this.commonName = $("input[name=commonname]", this.fieldset);
        this.organizationUnit = $("input[name=organizationunit]", this.fieldset);
        this.organization = $("input[name=organization]", this.fieldset);
        this.locality = $("input[name=locality]", this.fieldset);
        this.state = $("input[name=state]", this.fieldset);
        this.country = $("input[name=country]", this.fieldset);
    },

    getAlias: function() {
        return this.alias.val();
    },

    getAlgorithm: function() {
        return this.keyAlgorithm.combobox("getValue");
    },

    setAlgorithms: function(values) {
        var data = $.map(values || [], function(item) { return {text: item.name, value: item.name}; });
        this.keyAlgorithm.combobox("loadData", data);
    },

    getKeyLength: function() {
        return this.keyLength.combobox("getValue");
    },

    setKeyLengths: function (values) {
        var data = $.map(values || [], function (value) { return {text: value, value: value}; });
        this.keyLength.combobox("loadData", data);
    },

    getPeriod: function() {
        return this.period.numberspinner("getValue");
    },

    getCommonName: function() {
        return this.commonName.val();
    },

    getOrganizationUnit: function() {
        return this.organizationUnit.val();
    },

    getOrganization: function() {
        return this.organization.val();
    },

    getLocality: function() {
        return this.locality.val();
    },

    getState: function() {
        return this.state.val();
    },

    getCountry: function() {
        return this.country.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.alias.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _adjustKeyLength: function () {
        this.setKeyLengths(jscape.KeyPairAlgorithm.lengthsOf(this.getAlgorithm()));
    }
});

/** @class */
jscape.keystore.GenerateServerKeyCsrController = Class.extend(
    /** @lends {jscape.keystore.GenerateServerKeyCsrController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, keyAlias) {
        this.managementService = managementService;
        this.keyAlias = keyAlias;
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        this.managementService.generateServerKeyCSR(this.keyAlias, dialog.getFilename())
                .done($.proxy(function() { this.deferred.resolve(); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this))
                .always($.proxy(function() { dialog.destroy(); this.deferred = null; }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _setupDialog: function () {
        return new jscape.keystore.GenerateCsrDialog(this.resources);
    }
});

/** @class */
jscape.keystore.GenerateCsrDialog = Class.extend(
    /** @lends {jscape.keystore.GenerateCsrDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-serverkey-generatecsr").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=generateserverkeycsrfields]", this.dialog);

        this.filename = $("input[name=filename]", this.fieldset);
        this.filename.validatebox({required: true, validType: "serverkeys_filename_nonempty", missingMessage: resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_BAD_FILENAME")});
    },

    getFilename: function() {
        return this.filename.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.filename.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.keystore.DeleteServerKeyController = Class.extend(
    /** @lends {jscape.keystore.DeleteServerKeyController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function (managementService, keyAlias) {
        var deferred = $.Deferred();

        managementService.keyUsage(keyAlias)
                .done($.proxy(function (used) {
                    if (used) {
                        this._keyUsedError(keyAlias);
                        deferred.reject();
                    } else {
                        this._confirmDelete(keyAlias)
                                .done(function () {
                                          managementService.deleteServerKey(keyAlias)
                                                  .done(deferred.resolve)
                                                  .fail(deferred.reject);
                                      })
                                .fail(deferred.reject);

                    }
                }, this))
                .fail(deferred.reject);

        return deferred.promise();
    },

    _confirmDelete: function (keyAlias) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("KEY_MANAGER_SERVER_KEYS_DELETE_TITLE"),
                                            this.resources.getString("KEY_MANAGER_SERVER_KEYS_DELETE_MESSAGE", {0: keyAlias}));
    },

    _keyUsedError: function (keyAlias) {
        $.messager.alert(this.resources.getString("DIALOG_TITLE_ERROR"),
                         this.resources.getString("KEY_MANAGER_SERVER_KEYS_ERROR_KEY_USED", {0: keyAlias}),
                         "error");
    }
});