/** @class */
jscape.ServerParameters = Class.extend(
    /** @lends {jscape.ServerParameters.prototype} */
{
    /** @const */
    ALL_HOSTS_ADDRESS: "0.0.0.0",

    /** @constructor */
    initialize: function(a, b, c, d, e, f) {
        this.appVersion = a;
        this.allowedDescriptors = b || [];
        this.allowedServices = c || [];
        this.interfaces = d || [];
        this.sslCiphers = e || [];
        this.license = f || {};
    },

    getInetAddresses: function() {
        var result = [this.ALL_HOSTS_ADDRESS];
        for (var i = 0; i < this.interfaces.length; i++) {
            var addresses = this.interfaces[i].addresses;
            for (var j = 0; j < addresses.length; j++) {
                result.push(addresses[j]);
            }
        }
        return result;
    },

    getSslCiphers: function() {
        return this.sslCiphers;
    },

    allowsDescriptor: function(descriptor) {
        return descriptor && $.inArray(descriptor, this.allowedDescriptors) != -1;
    },

    allowsService: function(service) {
        return service && $.inArray(service, this.allowedServices) != -1;
    }
});
jscape.ServerParameters.fromJSON = function(data) {
    return new jscape.ServerParameters(data.applicationVersion,
                                       data.allowedDescriptors,
                                       data.allowedServices,
                                       data.interfaces,
                                       data.sslCiphers,
                                       data.license);
};

/** @class */
jscape.ServerStatistics = Class.extend(
    /** @lends {jscape.ServerStatistics.prototype} */
{
    /** @constructor */
    initialize: function (a, b, c, d, e, f) {
        this.uptime = a;
        this.maxMemory = b;
        this.usedMemory = c;
        this.threads = d;
        this.managementConnections = e;
        this.domainStatistics = f || []
    }
});
jscape.ServerStatistics.fromJSON = function(data) {
    var domainStatistics = $.map(data.domainStatistics || [], jscape.DomainStatistics.fromJSON);
    return new jscape.ServerStatistics(data.uptime,
                                       data.maxMemory,
                                       data.usedMemory,
                                       data.threads,
                                       data.managementConnections,
                                       domainStatistics);
};

/** @class */
jscape.DomainStatistics = Class.extend(
    /** @lends {jscape.DomainStatistics.prototype} */
{
    /** @constructor */
    initialize: function () {
        this.domainName = null;
        this.startDate = null;
        this.stopDate = null;
        this.paused = false;
        this.totalSessions = 0;
        this.currentSessions = 0;
        this.currentTransfers = 0;
        this.filesUploaded = 0;
        this.filesDownloaded = 0;
        this.filesDeleted = 0;
        this.bytesUploaded = 0;
        this.bytesDownloaded = 0;
        this.uploadsQuota = null;
        this.currentUploadedBytes = null;
        this.downloadsQuota = null;
        this.currentDownloadedBytes = null;
        this.transfersQuota = null;
        this.currentTransferredBytes = null;
        this.connectionStatistics = [];
        this.log = null;
    },

    isRunning: function() {
        return this.startDate != null && this.stopDate == null;
    },

    toJSON: function() {
        return JSON.stringify(this);
    }
});
jscape.DomainStatistics.fromJSON = function (data) {
    return $.extend(new jscape.DomainStatistics() , data);
};

jscape.DomainState = {RUNNING: "RUNNING", PAUSED: "PAUSED", STOPPED: "STOPPED"};
jscape.DomainState.stateFor = function (statistics) {
    return statistics.isRunning() && statistics.paused
            ? jscape.DomainState.PAUSED
            : statistics.isRunning() ? jscape.DomainState.RUNNING : jscape.DomainState.STOPPED;
};

/** @class */
jscape.ServerSettings = Class.extend(
    /** @lends {jscape.ServerSettings.prototype} */
{
    getServerParameters: function() {
        return this.serverParameters;
    },

    setServerParameters: function(value) {
        this.serverParameters = value;
    },

    getServerManagementServiceConfiguration: function() {
        return this.serverManagementServiceConfiguration;
    },

    setServerManagementServiceConfiguration: function(value) {
        this.serverManagementServiceConfiguration = value;
    },

    getManagementAccessServiceConfiguration: function() {
        return this.managementAccessServiceConfiguration;
    },

    setManagementAccessServiceConfiguration: function(value) {
        this.managementAccessServiceConfiguration = value;
    },

    getAdministrators: function() {
        return this.administrators;
    },

    setAdministrators: function(value) {
        this.administrators = value;
    },

    getDomainsAutoStart: function() {
        return this.domainsAutoStart;
    },

    setDomainsAutoStart: function(value) {
        this.domainsAutoStart = value;
    },

    getDbDrivers: function() {
        return this.dbDrivers;
    },

    setDbDrivers: function(value) {
        this.dbDrivers = value;
    },

    getWebServiceDescriptor: function() {
        return this.webServiceDescriptor;
    },

    setWebServiceDescriptor: function(value) {
        this.webServiceDescriptor = value;
    },

    getManagementWebServerConfiguration: function() {
        return this.managementWebServerConfiguration;
    },

    setManagementWebServerConfiguration: function(value) {
        this.managementWebServerConfiguration = value;
    },

    getDocumentViewerConfiguration: function() {
        return this.documentViewerConfiguration;
    },

    setDocumentViewerConfiguration: function(value) {
        this.documentViewerConfiguration = value;
    },

    getEmailServiceDescriptor: function() {
        return this.emailServiceDescriptor;
    },

    setEmailServiceDescriptor: function(value) {
        this.emailServiceDescriptor = value;
    },

    getFailoverServiceConfiguration: function() {
        return this.failoverServiceConfiguration;
    },

    setFailoverServiceConfiguration: function(value) {
        this.failoverServiceConfiguration = value;
    },

    getSearchingServiceConfiguration: function() {
        return this.searchingServiceConfiguration;
    },

    setSearchingServiceConfiguration: function(value) {
        this.searchingServiceConfiguration = value;
    },

    getJmxServiceConfiguration: function() {
        return this.jmxServiceConfiguration;
    },

    setJmxServiceConfiguration: function(value) {
        this.jmxServiceConfiguration = value;
    },

    getSelfRegistrationServiceDescriptor: function() {
        return this.selfRegistrationServiceDescriptor;
    },

    setSelfRegistrationServiceDescriptor: function(value) {
        this.selfRegistrationServiceDescriptor = value;
    },

    getKeys: function() {
        return this.keys || new jscape.Keys();
    },

    setKeys: function(keys) {
        this.keys = keys;
    }
});

/** @class */
jscape.KeyPairAlgorithm = Class.extend(
    /** @lends {jscape.KeyPairAlgorithm.prototype} */
{
    /** @constructor */
    initialize: function(name, lengths) {
        this.name = name;
        this.lengths = [].concat(lengths);
    }
});
jscape.KeyPairAlgorithm.RSA = new jscape.KeyPairAlgorithm("RSA", [1024, 2048]);
jscape.KeyPairAlgorithm.DSA = new jscape.KeyPairAlgorithm("DSA", [1024]);
jscape.KeyPairAlgorithm.values = function() { return [jscape.KeyPairAlgorithm.RSA, jscape.KeyPairAlgorithm.DSA]; };
jscape.KeyPairAlgorithm.lengthsOf = function(algorithm) { return $.map($.grep(jscape.KeyPairAlgorithm.values(), function(item) { return item.name == algorithm; }), function(item) { return item.lengths; }); };

/** @class */
jscape.SignatureAlgorithm = Class.extend(
    /** @lends {jscape.SignatureAlgorithm.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.hashAlgorithm = a;
        this.asymmetricalAlgorithm = b;
        this.name = (this.hashAlgorithm + "WITH" + this.asymmetricalAlgorithm).toUpperCase();
    },

    toJSON: function() {
        return this.name;
    }
});
jscape.SignatureAlgorithm.SHA1_WITH_RSA = new jscape.SignatureAlgorithm("SHA1", "RSA");
jscape.SignatureAlgorithm.SHA1_WITH_DSA = new jscape.SignatureAlgorithm("SHA1", "DSA");
jscape.SignatureAlgorithm.MD5_WITH_RSA = new jscape.SignatureAlgorithm("MD5", "RSA");
jscape.SignatureAlgorithm.values = function() { return [jscape.SignatureAlgorithm.SHA1_WITH_RSA, jscape.SignatureAlgorithm.SHA1_WITH_DSA, jscape.SignatureAlgorithm.MD5_WITH_RSA]; };
jscape.SignatureAlgorithm.algorithmsOf = function(asymmetricalAlgorithm) { return $.grep(jscape.SignatureAlgorithm.values(), function(item) { return item.asymmetricalAlgorithm == asymmetricalAlgorithm; }); };
jscape.SignatureAlgorithm.forName = function(data) {
    var values = jscape.SignatureAlgorithm.values();
    for (var i = 0; i < values.length; i++) {
        if (values[i].name === data) {
            return values[i];
        }
    }
    return null;
};

/** @class */
jscape.Keys = Class.extend(
    /** @lends {jscape.Keys.prototype} */
{
    /** @constructor */
    initialize: function() {
        this.serverKeys = [];
        this.clientKeys = {pub: [], crt: []};
        this.hostKeys = {pub: [], crt: []};
        this.pgpKeys = {pub: [], secret: []};
    },

    getServerKeys: function() {
        return this.serverKeys;
    },

    setServerKeys: function(value) {
        this.serverKeys = value || [];
    },

    getClientKeys: function() {
        return this.clientKeys.pub;
    },

    setClientKeys: function(value) {
        this.clientKeys.pub = value || [];
    },

    getClientCertificates: function() {
        return this.clientKeys.crt;
    },

    setClientCertificates: function(value) {
        this.clientKeys.crt = value || [];
    },

    getHostKeys: function() {
        return this.hostKeys.pub;
    },

    setHostKeys: function(value) {
        this.hostKeys.pub = value || [];
    },

    getHostCertificates: function() {
        return this.hostKeys.crt;
    },

    setHostCertificates: function(value) {
        this.hostKeys.crt = value || [];
    },

    getPgpPublicKeys: function() {
        return this.pgpKeys.pub;
    },

    setPgpPublicKeys: function(value) {
        this.pgpKeys.pub = value || [];
    },

    getPgpSecretKeys: function() {
        return this.pgpKeys.secret;
    },

    setPgpSecretKeys: function(value) {
        this.pgpKeys.secret = value || [];
    }
});

/** @class */
jscape.FileKeyProvider = Class.extend(
    /** @lends {jscape.FileKeyProvider.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.file = a || "";
        this.filePassword = b || "";
    }
});
jscape.FileKeyProvider.fromJSON = function(data) {
    return data ? new jscape.FileKeyProvider(data.file, data.filePassword) : null;
};

/** @class */
jscape.ServerKeyProvider = Class.extend(
    /** @lends {jscape.ServerKeyProvider.prototype} */
{
    /** @constructor */
    initialize: function(alias) {
        this.keyAlias = alias || "";
    }
});
jscape.ServerKeyProvider.fromJSON = function(data) {
    return data ? new jscape.ServerKeyProvider(data.keyAlias) : null;
};

/** @class */
jscape.NetworkInterfaceInfo = Class.extend(
    /** @lends {jscape.NetworkInterfaceInfo.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.name = a;
        this.displayName = b || a;
        this.addresses = c || [];
    }
});

/** @class */
jscape.KeyEntryDescriptor = Class.extend(
    /** @lends {jscape.KeyEntryDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e, f, g, h, i, j, k) {
        this.alias = a;
        this.algorithm = b;
        this.certificateType = c;
        this.version = d;
        this.serial = e;
        this.issuer = f;
        this.subject = g;
        this.validNotBefore = h;
        this.validNotAfter = i;
        this.signature = j;
        this.domains = k || [];
    }
});
jscape.KeyEntryDescriptor.fromJSON = function(data) {
    return new jscape.KeyEntryDescriptor(data.alias,
                                         data.algorithm,
                                         data.certificateType,
                                         data.version,
                                         data.serial,
                                         data.issuer,
                                         data.subject,
                                         data.validNotBefore,
                                         data.validNotAfter,
                                         data.signature,
                                         data.domains);
};

/** @class */
jscape.PgpKeyEntryDescriptor = Class.extend(
    /** @lends {jscape.PgpKeyEntryDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e, f, g) {
        this.alias = a;
        this.algorithm = b;
        this.canEncrypt = c !== undefined ? c : false;
        this.canDecrypt = d !== undefined ? d : false;
        this.canSign = e != undefined ? e : false;
        this.canVerify = f !== undefined ? f : false;
        this.fingerprint = g;
    }
});
jscape.PgpKeyEntryDescriptor.fromJSON = function(data) {
    return new jscape.PgpKeyEntryDescriptor(data.alias,
                                            data.algorithm,
                                            data.canEncrypt,
                                            data.canDecrypt,
                                            data.canSign,
                                            data.canVerify,
                                            data.fingerprint);
};

/** @class */
jscape.PgpSignatureDescriptor = Class.extend(
    /** @lends {jscape.PgpSignatureDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.signingKey = a;
        this.type = b;
        this.creationTime = c;
        this.version = d;
    }
});
jscape.PgpSignatureDescriptor.fromJSON = function(data) {
    return new jscape.PgpSignatureDescriptor(data.signingKey,
                                             data.type,
                                             data.creationTime,
                                             data.version);
};

/** @class */
jscape.WebResources = Class.extend(
    /** @lends {jscape.WebResources.prototype} */
{
    /** @constructor */
    initialize: function(lang, displayName, properties) {
        this.lang = lang || "";
        this.displayName = displayName || lang;
        this.properties = properties || {};
    },

    isDefault: function() {
        return this.lang === "";
    },

    clone: function() {
        return new jscape.WebResources(this.lang, this.displayName, $.extend({}, this.properties));
    }
});
jscape.WebResources.fromJSON = function(data) {
    return new jscape.WebResources(data.lang, data.displayName, data.properties);
};