jscape.management = jscape.management || {};

/** @class */
jscape.RestErrorDialog = Class.extend(
    /** @lends {jscape.RestErrorDialog.prototype} */
{
    TEMPLATE: "<div class='dialog'><div class='messager-body'><div class='messager-icon messager-error'></div><div class='messager-message'></div><div class='dialog-details'></div></div></div>",

    deferred: null,

    /** @constructor */
    initialize: function() {
        this.dialog = $(this.TEMPLATE).appendTo(document.body);
        this.dialog.dialog({
            width: 470, height: "auto",
            modal: true, resizable: true, closable: true, closed: true,
            buttons: [
                {text: "Ok", handler: $.proxy(this.hide, this)},
                {text: "Details", handler: $.proxy(function() { this.details.toggle(); this.dialog.dialog("resize").dialog("center"); }, this)}
            ],
            onClose: $.proxy(function () { this.hide(); }, this)
        }).show();

        this.message = $("div.messager-message", this.dialog);
        this.details = $("div.dialog-details", this.dialog).hide();
    },

    setTitle: function(value) {
        this.dialog.dialog("setTitle", value || "");
    },

    setMessage: function(value) {
        this.message.text(value);
    },

    setDetails: function(details) {
        this.details.html("<pre>" + details + "</pre>");
    },

    show: function () {
        this.deferred = $.Deferred();
        this.dialog.dialog("center").dialog("open");
        return this.deferred.promise();
    },

    hide: function () {
        this.dialog.dialog("destroy");
        this.deferred.resolve();
        this.deferred = null;
    }
});


/** @class */
jscape.management.ManagementService = Class.extend(
    /** @lends {jscape.management.ManagementService.prototype} */
{
    /** @const */
    DEFAULT_TIMEOUT: 180000,

    config: null,
    timeout: null,
    resources: null,

    /** @constructor */
    initialize: function(config, resources) {
        this.config = config;
        this.timeout = config.timeout || this.DEFAULT_TIMEOUT;
        this.resources = resources;
    },

    logout: function() {
        return this._submitForm("POST", "/logout");
    },

    editDomain: function(domainName) {
        return this._submitForm("GET", "/domains/{domain}".supplant({domain: domainName}));
    },

    getKeys: function () {
        return $.when(this.getServerKeys(),
                      this.getClientKeys(),
                      this.getClientCertificates(),
                      this.getHostKeys(),
                      this.getHostCertificates(),
                      this.getPgpPublicKeys(),
                      this.getPgpSecretKeys())
                .then(function (serverKeys, clientKeys, clientCertificates, hostKeys, hostCertificates, pgpPublicKeys, pgpSecretKeys) {
                        var keys = new jscape.Keys();
                        keys.setServerKeys(serverKeys);
                        keys.setClientKeys(clientKeys);
                        keys.setClientCertificates(clientCertificates);
                        keys.setHostKeys(hostKeys);
                        keys.setHostCertificates(hostCertificates);
                        keys.setPgpPublicKeys(pgpPublicKeys);
                        keys.setPgpSecretKeys(pgpSecretKeys);

                        return keys;
                });
    },

    getServerParameters: function() {
        return this._request("GET", "/server/parameters").send()
                .then(function(data) { return jscape.ServerParameters.fromJSON(data); });
    },

    getServerStatistics: function() {
        return this._request("GET", "/server/statistics").send()
                .then(function(data) { return jscape.ServerStatistics.fromJSON(data); });
    },

    getManagementServiceConfiguration: function() {
        return this._request("GET", "/server/management-service/configuration").send()
                .then(function(data) { return jscape.ServerManagementServiceConfiguration.fromJSON(data); });
    },

    setManagementServiceConfiguration: function(config) {
        return this._request("PUT", "/server/management-service/configuration").send(config);
    },

    getManagementAccessServiceConfiguration: function() {
        return this._request("GET", "/management-access-service/configuration").send()
                .then(function(data) { return jscape.ManagementAccessConfiguration.fromJSON(data); });
    },

    setManagementAccessServiceConfiguration: function(config) {
        return this._request("PUT", "/management-access-service/configuration").send(config);
    },

    getAdministrators: function() {
        return this._request("GET", "/administrators").send()
                .then(function(data) { return $.map(data || [], jscape.Administrator.fromJSON); });
    },

    setAdministrators: function(administrators) {
        return this._request("PUT", "/administrators").send(administrators);
    },

    getDomainsAutoStart: function () {
        return this._request("GET", "/domains-auto-start").send()
                .then(function (data) { return jscape.DomainsAutoStart.fromJSON(data); });
    },

    setDomainsAutoStart: function (domainAutoStart) {
        return this._request("PUT", "/domains-auto-start").send(domainAutoStart);
    },

    getWebServiceConfiguration: function () {
        return this._request("GET", "/web-service/configuration").send()
                .then(function (data) { return jscape.WebServiceDescriptor.fromJSON(data); });
    },

    setWebServiceConfiguration: function (config) {
        return this._request("PUT", "/web-service/configuration").send(config);
    },

    getDbDrivers: function () {
        return this._request("GET", "/db-drivers").send()
                .then(function (data) { return jscape.DbDrivers.fromJSON(data); });
    },

    setDbDrivers: function (drivers) {
        return this._request("PUT", "/db-drivers").send(drivers);
    },

    getManagementWebServerConfiguration: function () {
        return this._request("GET", "/web-management/configuration").send()
                .then(function (data) { return jscape.ManagementWebServerConfiguration.fromJSON(data); });
    },

    setManagementWebServerConfiguration: function (config) {
        return this._request("PUT", "/web-management/configuration").send(config);
    },

    testManagementWebServerConfiguration: function() {
        return this._request("GET", "/web-management/configuration").options({global: false}).send()
                .then(function (data) { return jscape.ManagementWebServerConfiguration.fromJSON(data); })
    },

    getRegistrationServiceConfiguration: function () {
        return this._request("GET", "/registration-service/configuration").send()
                .then(function (data) { return jscape.SelfRegistrationServiceDescriptor.fromJSON(data); });
    },

    setRegistrationServiceConfiguration: function (config) {
        return this._request("PUT", "/registration-service/configuration").send(config);
    },

    getDocumentViewerConfiguration: function () {
        return this._request("GET", "/document-viewer/configuration").send()
                .then(function (data) { return jscape.DocumentViewerConfiguration.fromJSON(data); });
    },

    setDocumentViewerConfiguration: function (config) {
        return this._request("PUT", "/document-viewer/configuration").send(config);
    },

    testDocumentViewerAllowed: function() {
        return this._request("GET", "/document-viewer/allowed").sendSync();
    },

    getEmailServiceConfiguration: function () {
        return this._request("GET", "/email-service/configuration").send()
                .then(function(data) { return jscape.EmailServiceDescriptor.fromJSON(data); });
    },

    setEmailServiceConfiguration: function (config) {
        return this._request("PUT", "/email-service/configuration").send(config);
    },

    testEmailServer: function (config, recipient) {
        return this._request("POST", "/email-service/test").headers({"x-js-recipient": recipient}).send(config);
    },

    testAdHocStore: function(config) {
        return this._request("POST", "/email-service/ad-hoc/test").send(config);
    },

    getFailoverServiceConfiguration: function () {
        return this._request("GET", "/failover-service/configuration").send()
                .then(function (data) { return jscape.FailoverConfiguration.fromJSON(data); });
    },

    setFailoverServiceConfiguration: function (config) {
        return this._request("PUT", "/failover-service/configuration").send(config);
    },

    syncFailoverServerState: function(config) {
        return this._request("POST", "/failover-service/sync").send(config);
    },

    failoverSyncComplete: function() {
        return this._request("GET", "/failover-service/sync").send();
    },

    getSearchingServiceConfiguration: function () {
        return this._request("GET", "/searching-service/configuration").send()
                .then(function (data) { return jscape.SearchingServiceConfiguration.fromJSON(data); });
    },

    setSearchingServiceConfiguration: function (config) {
        return this._request("PUT", "/searching-service/configuration").send(config);
    },

    getSearchFileIndexingStats: function (config) {
        return this._request("GET", "/searching-service/index").send(config);
    },

    startSearchFileIndexing: function (config) {
        return this._request("POST", "/searching-service/index").send(config);
    },

    getJmxServiceConfiguration: function () {
        return this._request("GET", "/jmx-service/configuration").send()
                .then(function (data) { return jscape.JmxServiceConfiguration.fromJSON(data); });
    },

    setJmxServiceConfiguration: function (config) {
        return this._request("PUT", "/jmx-service/configuration").send(config);
    },

    getDomainNames: function() {
        return this._request("GET", "/domains").send();
    },

    getDomain: function(domainName) {
        return this._request("GET", "/domains/{domain}").uri({domain: domainName}).send()
                .then(function(data) { return jscape.DomainDescriptor.fromJSON(data); });
    },

    storeDomain: function(descriptor) {
        return this._request("PUT", "/domains").send(descriptor);
    },

    deleteDomain: function(domainName) {
        return this._request("DELETE", "/domains/{domain}").uri({domain: domainName}).send();
    },

    getDomainDescription: function(domainName) {
        return this._request("GET", "/domains/{domain}/desc", false).uri({domain: domainName}).send();
    },

    setDomainDescription: function(domainName, description) {
        return this._request("POST", "/domains/{domain}/desc").uri({domain: domainName}).send(description);
    },

    startDomain: function(domainName) {
        return this._request("POST", "/domains/{domain}/start").uri({domain: domainName}).send();
    },

    stopDomain: function(domainName) {
        return this._request("POST", "/domains/{domain}/stop").uri({domain: domainName}).send();
    },

    pauseDomain: function(domainName) {
        return this._request("POST", "/domains/{domain}/pause").uri({domain: domainName}).send();
    },

    resumeDomain: function(domainName) {
        return this._request("POST", "/domains/{domain}/resume").uri({domain: domainName}).send();
    },

    listFileSystem: function(path) {
        return this._request("GET", "/files/vfs?path={path}").uri({path: path || ""}).send();
    },

    createFileSystemDir: function(path, name) {
        return this._request("PUT", "/files/vfs?path={path}").uri({path: path || ""}).send(name);
    },

    listMonitorsDir: function(path) {
        return this._request("GET", "/files/dir-monitors?path={path}").uri({path: path || ""}).send();
    },

    createMonitorsDir: function(path, name) {
        return this._request("PUT", "/files/dir-monitors?path={path}").uri({path: path || ""}).send(name);
    },

    listUsersRestrictPath: function(path) {
        return this._request("GET", "/files/users?path={path}").uri({path: path || ""}).send();
    },

    createUsersRestrictPath: function(path, name) {
        return this._request("PUT", "/files/users?path={path}").uri({path: path || ""}).send(name);
    },

    listKeystorePath: function(path) {
        return this._request("GET", "/files/keystore?path={path}").uri({path: path || ""}).send();
    },

    createKeystoreDir: function(path, name) {
        return this._request("PUT", "/files/keystore?path={path}").uri({path: path || ""}).send(name);
    },

    listDebugDir: function(path) {
        return this._request("GET", "/files/debug?path={path}").uri({path: path || ""}).send();
    },

    createDebugDir: function(path, name) {
        return this._request("PUT", "/files/debug?path={path}").uri({path: path || ""}).send(name);
    },

    listLogDir: function(path) {
        return this._request("GET", "/files/log?path={path}").uri({path: path || ""}).send();
    },

    createLogDir: function(path, name) {
        return this._request("PUT", "/files/log?path={path}").uri({path: path || ""}).send(name);
    },

    listDatastoreDir: function(path) {
        return this._request("GET", "/files/datastore?path={path}").uri({path: path || ""}).send();
    },

    createDatastoreDir: function(path, name) {
        return this._request("PUT", "/files/datastore?path={path}").uri({path: path || ""}).send(name);
    },



    getDomainStatistics: function(domainName) {
        return this._request("GET", "/domains/{domain}/statistics").uri({domain: domainName}).send()
                .then(function(data) { return jscape.DomainStatistics.fromJSON(data); });
    },

    closeConnection: function(domainName, stats, scope) {
        return this._request("POST", "/domains/{domain}/connections/close;scope={scope}").uri({domain: domainName, scope: scope}).send(stats);
    },

    getLogs: function(domainName, maxRecordCount) {
        return this._request("GET", "/domains/{domain}/logging/logs;entries={count}", false).uri({domain: domainName, count: maxRecordCount}).send();
    },

    getLogSearches: function(domainName) {
        return this._request("GET", "/domains/{domain}/logging/searches").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.LogService.fromJSON); });
    },

    searchLog: function(domainName, criteria) {
        return this._request("PUT", "/domains/{domain}/logging/searches").uri({domain: domainName}).send(criteria);
    },

    getLogSearchResult: function(domainName, searchName) {
        return this._request("GET", "/domains/{domain}/logging/searches/{search}", false).uri({domain: domainName, search: searchName}).send();
    },

    exportLogSearchResult: function(domainName, searchName, filename) {
        return this._request("GET", "/domains/{domain}/logging/searches/{search}/export?filename={file}").uri({domain: domainName, search: searchName, file: filename}).sendDownload();
    },

    deleteLogSearch: function(domainName, searchName) {
        return this._request("DELETE", "/domains/{domain}/logging/searches/{search}").uri({domain: domainName, search: searchName}).send();
    },

    startLogSearch: function(domainName, searchName) {
        return this._request("POST", "/domains/{domain}/logging/searches/{search}/start").uri({domain: domainName, search: searchName}).send();
    },

    getLogService: function(domainName) {
        return this._request("GET", "/domains/{domain}/logging").uri({domain: domainName}).send()
                .then(function(data) { return jscape.LogService.fromJSON(data); });
    },

    setLogService: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/logging").uri({domain: domainName}).send(descriptor);
    },

    testLogService: function (domainName, descriptor) {
        return typeof domainName === "string"
               ? this._request("POST", "/domains/{domain}/logging/test").uri({domain: domainName}).send(descriptor)
               : this._request("POST", "/services/log/test").send(descriptor);
    },

    getLogConfiguration: function(domainName) {
        return this._request("GET", "/domains/{domain}/logging/config").uri({domain: domainName}).send()
                .then(function(data) { return jscape.LogConfiguration.fromJSON(data); });
    },

    setLogConfiguration: function(domainName, config) {
        return this._request("PUT", "/domains/{domain}/logging/config").uri({domain: domainName}).send(config);
    },

    getSyslogService: function(domainName) {
        return this._request("GET", "/domains/{domain}/logging/syslog").uri({domain: domainName}).send()
                .then(function(data) { return jscape.SyslogService.fromJSON(data); });
    },

    setSyslogService: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/logging/syslog").uri({domain: domainName}).send(descriptor);
    },

    getReports: function(domainName) {
        return this._request("GET", "/domains/{domain}/reports").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.Report.fromJSON); });
    },

    storeReport: function(domainName, report) {
        return this._request("PUT", "/domains/{domain}/reports").uri({domain: domainName}).send(report);
    },

    deleteReport: function(domainName, reportName) {
        return this._request("DELETE", "/domains/{domain}/reports/{report}").uri({domain: domainName, report: reportName}).send();
    },

    reRunReport: function(domainName, reportName) {
        return this._request("POST", "/domains/{domain}/reports/{report}/rerun").uri({domain: domainName, report: reportName}).send()
                .then(function(data) { return jscape.Report.fromJSON(data); });
    },

    getAs2Messages: function(domainName) {
        return this._request("GET", "/domains/{domain}/as2").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.As2Message.fromJSON); });
    },

    getAs2MessageRecordView: function(domainName, messageId) {
        return this._request("GET", "/domains/{domain}/as2/{message}").uri({domain: domainName, message: messageId}).send();
    },

    getAs2MessageRecordRawView: function(domainName, messageId) {
        return this._request("GET", "/domains/{domain}/as2/{message}?out=raw", false).uri({domain: domainName, message: messageId}).send();
    },

    resendAs2Message: function(domainName, messageId) {
        return this._request("POST", "/domains/{domain}/as2/{message}/resend").uri({domain: domainName, message: messageId}).send();
    },

    deleteAs2Message: function(domainName, messageId) {
        return this._request("DELETE", "/domains/{domain}/as2/{message}").uri({domain: domainName, message: messageId}).send();
    },

    getDatastoreService: function(domainName) {
        return this._request("GET", "/domains/{domain}/datastore").uri({domain: domainName}).send()
                .then(function(data) { return jscape.DatastoreService.fromJSON(data); });
    },

    setDatastoreService: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/datastore").uri({domain: domainName}).send(descriptor);
    },

    testDatastoreService: function (domainName, descriptor) {
        return typeof domainName === "string"
               ? this._request("POST", "/domains/{domain}/datastore/test").uri({domain: domainName}).send(descriptor)
               : this._request("POST", "/services/datastore/test").send(descriptor);
    },

    getIpAccessRules: function(domainName) {
        return this._request("GET", "/domains/{domain}/access-rules").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.IpAccessRule.fromJSON); });
    },

    setIpAccessRules: function(domainName, rules) {
        return this._request("PUT", "/domains/{domain}/access-rules").uri({domain: domainName}).send(rules);
    },

    getTimeAccess: function(domainName) {
        return this._request("GET", "/domains/{domain}/time-access").uri({domain: domainName}).send()
                .then(function(data) { return jscape.TimeAccess.fromJSON(data); });
    },

    setTimeAccess: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/time-access").uri({domain: domainName}).send(descriptor);
    },

    getBannedFiles: function(domainName) {
        return this._request("GET", "/domains/{domain}/banned-files").uri({domain: domainName}).send()
                .then(function(data) { return jscape.BannedFiles.fromJSON(data); });
    },

    setBannedFiles: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/banned-files").uri({domain: domainName}).send(descriptor);
    },

    testBannedFilesPattern: function(domainName, pattern) {
        return this._request("POST", "/domains/{domain}/banned-files/test-pattern").uri({domain: domainName}).sendSync(pattern);
    },

    getPasswordCompliance: function(domainName) {
        return this._request("GET", "/domains/{domain}/compliance").uri({domain: domainName}).send()
                .then(function(data) { return jscape.PasswordCompliance.fromJSON(data); });
    },

    setPasswordCompliance: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/compliance").uri({domain: domainName}).send(descriptor);
    },

    testPasswordCompliance: function(domainName, accountName, password) {
        return this._request("POST", "/domains/{domain}/compliance/test").uri({domain: domainName}).sendSync({account: accountName, password: password});
    },

    getDlpRules: function(domainName) {
        return this._request("GET", "/domains/{domain}/dlp-rules").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.DlpRule.fromJSON); });
    },

    storeDlpRule: function(domainName, rule) {
        return this._request("PUT", "/domains/{domain}/dlp-rules").uri({domain: domainName}).send(rule)
                .then(function(data) { return jscape.DlpRule.fromJSON(data); });
    },

    deleteDlpRule: function(domainName, ruleName) {
        return this._request("DELETE", "/domains/{domain}/dlp-rules/{rule}").uri({domain: domainName, rule: ruleName}).send();
    },

    getDomainRestrictions: function(domainName) {
        return this._request("GET", "/domains/{domain}/restrictions").uri({domain: domainName}).send()
                .then(function(data) { return jscape.DomainRestrictions.fromJSON(data); });
    },

    setDomainRestrictions: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/restrictions").uri({domain: domainName}).send(descriptor);
    },

    getTriggers: function(domainName) {
        return this._request("GET", "/domains/{domain}/triggers").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.Trigger.fromJSON); });
    },

    setTriggers: function(domainName, triggers) {
        return this._request("PUT", "/domains/{domain}/triggers").uri({domain: domainName}).send(triggers);
    },

    getTriggerStates: function(domainName) {
        return this._request("GET", "/domains/{domain}/trigger-service/states").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.TriggerState.fromJSON); });
    },

    getTriggerEvents: function(domainName) {
        return this._request("GET", "/domains/{domain}/trigger-service/events").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.TriggerEvent.fromJSON); });
    },

    getTriggerEvent: function(domainName, eventType) {
        return this._request("GET", "/domains/{domain}/trigger-service/events/{event}").uri({domain: domainName, event: eventType}).send();
    },

    getTriggerActions: function(domainName) {
        return this._request("GET", "/domains/{domain}/trigger-service/actions").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.TriggerAction.fromJSON); });
    },

    getTriggerAction: function(domainName, actionType) {
        return this._request("GET", "/domains/{domain}/trigger-service/actions/{action}").uri({domain: domainName, action: actionType}).send();
    },

    getTriggerFunctions: function(domainName) {
        return this._request("GET", "/domains/{domain}/trigger-service/functions").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.TriggerFunction.fromJSON); });
    },

    validateTriggerExpression: function(domainName, expression, eventType) {
        return this._request("POST", "/domains/{domain}/trigger-service/validate-expression").uri({domain: domainName}).sendSync({expression: expression, event: eventType});
    },

    testTriggerExpression: function(domainName, expression, parameters) {
        return this._request("POST", "/domains/{domain}/trigger-service/test-expression?query={query}", false).uri({domain: domainName, query: expression}).send(parameters);
    },

    getTriggerServiceConfiguration: function(domainName) {
        return this._request("GET", "/domains/{domain}/trigger-service/config").uri({domain: domainName}).send()
                .then(function(data) { return jscape.TriggerServiceConfiguration.fromJSON(data); });
    },

    setTriggerServiceConfiguration: function(domainName, config) {
        return this._request("PUT", "/domains/{domain}/trigger-service/config").uri({domain: domainName}).send(config);
    },

    copyTrigger: function(domainName, sourceTriggerName, targetTriggerName) {
        return this._request("PUT", "/domains/{domain}/triggers/{trigger}").uri({domain: domainName, trigger: targetTriggerName}).headers({"x-js-copy-source": sourceTriggerName}).send()
                .then(function(data) { return jscape.Trigger.fromJSON(data); });
    },

    runTrigger: function(domainName, triggerName) {
        return this._request("POST", "/domains/{domain}/triggers/{trigger}/run").uri({domain: domainName, trigger: triggerName}).send();
    },

    pauseTrigger: function(domainName, id) {
        return this._request("POST", "/domains/{domain}/triggers/{trigger}/pause").uri({domain: domainName, trigger: id}).send();
    },

    resumeTrigger: function(domainName, id) {
        return this._request("POST", "/domains/{domain}/triggers/{trigger}/resume").uri({domain: domainName, trigger: id}).send();
    },

    killTrigger: function(domainName, id) {
        return this._request("POST", "/domains/{domain}/triggers/{trigger}/kill").uri({domain: domainName, trigger: id}).send();
    },

    getAuthenticationService: function(domainName) {
        return this._request("GET", "/domains/{domain}/authentication").uri({domain: domainName}).send()
                .then(function(data) { return jscape.AuthService.fromJSON(data); });
    },

    setAuthenticationService: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/authentication").uri({domain: domainName}).send(descriptor);
    },

    testAuthenticationService: function (domainName, descriptor, username, password, clientIp) {
        return this._request("POST", "/domains/{domain}/authentication/test").uri({domain: domainName}).sendFormMultipart({descriptor: descriptor, login: username, password: password, ip: clientIp}, true);
    },

    getPhoneFactorService: function(domainName) {
        return this._request("GET", "/domains/{domain}/authentication/phone-factor").uri({domain: domainName}).send()
                .then(function(data) { return jscape.PhoneFactorService.fromJSON(data); });
    },

    setPhoneFactorService: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/authentication/phone-factor").uri({domain: domainName}).send(descriptor);
    },

    getWebSsoService: function(domainName) {
        return this._request("GET", "/domains/{domain}/authentication/web-sso").uri({domain: domainName}).send()
                .then(function(data) { return jscape.WebSsoService.fromJSON(data); });
    },

    setWebSsoService: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/authentication/web-sso").uri({domain: domainName}).send(descriptor);
    },

    getAccounts: function(domainName) {
        return this._request("GET", "/domains/{domain}/accounts").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.AccountInfo.fromJSON); });
    },

    getAccount: function(domainName, accountName) {
        return this._request("GET", "/domains/{domain}/accounts/{account}").uri({domain: domainName, account: accountName}).send()
                .then(function(data) { return jscape.Account.fromJSON(data); });
    },

    createAccount: function(domainName, account) {
        return this._request("POST", "/domains/{domain}/accounts").uri({domain: domainName}).send(account)
                .then(function(data) { return jscape.AccountInfo.fromJSON(data); });
    },

    copyAccount: function(domainName, sourceAccountName, targetAccountName) {
        return this._request("PUT", "/domains/{domain}/accounts/{account}").uri({domain: domainName, account: targetAccountName}).headers({"x-js-copy-source": sourceAccountName}).send()
                .then(function(data) { return jscape.AccountInfo.fromJSON(data); });
    },

    updateAccount: function(domainName, account) {
        return this._request("PUT", "/domains/{domain}/accounts").uri({domain: domainName}).send(account)
                .then(function(data) { return jscape.AccountInfo.fromJSON(data); });
    },

    changeAccountPassword: function(domainName, accountName, password) {
        return this._request("POST", "/domains/{domain}/accounts/{account}/changepassword").uri({domain: domainName, account: accountName}).send(password);
    },

    deleteAccount: function(domainName, accountName) {
        return this._request("DELETE", "/domains/{domain}/accounts/{account}").uri({domain: domainName, account: accountName}).send();
    },

    getAccountQuotas: function(domainName, accountName) {
        return this._request("GET", "/domains/{domain}/accounts/{account}/quotas").uri({domain: domainName, account: accountName}).send()
                .then(function(data) { return jscape.AccountTransferQuotas.fromJSON(data); });
    },

    getTemplates: function(domainName) {
        return this._request("GET", "/domains/{domain}/account-templates").uri({domain: domainName}).send();
    },

    getTemplate: function(domainName, templateName) {
        return this._request("GET", "/domains/{domain}/account-templates/{template}").uri({domain: domainName, template: templateName}).send()
                .then(function(data) { return jscape.AccountTemplate.fromJSON(data); });
    },

    storeTemplate: function(domainName, template) {
        return this._request("PUT", "/domains/{domain}/account-templates").uri({domain: domainName}).send(template);
    },

    deleteTemplate: function(domainName, templateName) {
        return this._request("DELETE", "/domains/{domain}/account-templates/{template}").uri({domain: domainName, template: templateName}).send();
    },

    getGroups: function (domainName) {
        return this._request("GET", "/domains/{domain}/groups").uri({domain: domainName}).send()
                .then(function (data) { return $.map(data || [], jscape.Group.fromJSON); });
    },

    storeGroup: function (domainName, group) {
        return this._request("PUT", "/domains/{domain}/groups").uri({domain: domainName}).send(group)
                .then(function (data) { return jscape.Group.fromJSON(data); });
    },

    copyGroup: function (domainName, sourceGroupName, targetGroupName) {
        return this._request("PUT", "/domains/{domain}/groups/{group}").uri({domain: domainName, group: targetGroupName}).headers({"x-js-copy-source": sourceGroupName}).send()
                .then(function (data) { return jscape.Group.fromJSON(data); });
    },

    deleteGroup: function (domainName, groupName) {
        return this._request("DELETE", "/domains/{domain}/groups/{group}").uri({domain: domainName, group: groupName}).send();
    },

    getGroupUsers: function (domainName, groupName) {
        return this._request("GET", "/domains/{domain}/groups/{group}/accounts").uri({domain: domainName, group: groupName}).send();
    },

    setGroupUsers: function (domainName, groupName, accountNames) {
        return this._request("PUT", "/domains/{domain}/groups/{group}/accounts").uri({domain: domainName, group: groupName}).send(accountNames);
    },

    getReverseProxies: function(domainName) {
        return this._request("GET", "/domains/{domain}/reverse-proxies").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.ResourceDescriptor.fromJSON); });
    },

    testReverseProxy: function(domainName, descriptor) {
        return this._request("POST", "/domains/{domain}/reverse-proxies/{proxy}/test").uri({domain: domainName, proxy: descriptor.name}).send(descriptor);
    },

    storeReverseProxy: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/reverse-proxies").uri({domain: domainName}).send(descriptor);
    },

    deleteReverseProxy: function(domainName, proxyName) {
        return this._request("DELETE", "/domains/{domain}/reverse-proxies/{proxy}").uri({domain: domainName, proxy: proxyName}).send();
    },

    getDirectoryMonitors: function(domainName) {
        return this._request("GET", "/domains/{domain}/dir-monitors").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.DirectoryMonitor.fromJSON); });
    },

    storeDirectoryMonitor: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/dir-monitors").uri({domain: domainName}).send(descriptor);
    },

    deleteDirectoryMonitor: function(domainName, monitorName) {
        this._request("DELETE", "/domains/{domain}/dir-monitors/{monitor}").uri({domain: domainName, monitor: monitorName}).send();
    },

    runDirectoryMonitor: function(domainName, monitorName) {
        this._request("POST", "/domains/{domain}/dir-monitors/{monitor}/run").uri({domain: domainName, monitor: monitorName}).send();
    },

    getDirectoryMonitorsConfiguration: function(domainName) {
        return this._request("GET", "/domains/{domain}/dir-monitors-config").uri({domain: domainName}).send()
                .then(function(data) { return jscape.DirectoryMonitorsConfiguration.fromJSON(data); });
    },

    setDirectoryMonitorsConfiguration: function(domainName, config) {
        return this._request("PUT", "/domains/{domain}/dir-monitors-config").uri({domain: domainName}).send(config);
    },

    getDropZones: function (domainName) {
        return this._request("GET", "/domains/{domain}/drop-zones").uri({domain: domainName}).send()
                .then(function (data) { return $.map(data || [], jscape.DropZoneDescriptor.fromJSON); });
    },

    getDropZoneURI: function(domainName, zoneName) {
        return this._request("POST", "/domains/{domain}/drop-zones/{zone}/uri").uri({domain: domainName, zone: zoneName}).sendSync();
    },

    storeDropZone: function (domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/drop-zones").uri({domain: domainName}).send(descriptor);
    },

    deleteDropZone: function (domainName, zoneName) {
        return this._request("DELETE", "/domains/{domain}/drop-zones/{zone}").uri({domain: domainName, zone: zoneName}).send();
    },

    purgeDropZone: function (domainName, zoneName) {
        return this._request("POST", "/domains/{domain}/drop-zones/{zone}/purge").uri({domain: domainName, zone: zoneName}).send();
    },

    getWebThemes: function (domainName) {
        return this._request("GET", "/domains/{domain}/web-themes").uri({domain: domainName}).send()
                .then(function (data) { return $.map(data || [], jscape.WebTheme.fromJSON); });
    },

    getWebThemeURI: function(domainName, themeName) {
        return this._request("POST", "/domains/{domain}/web-themes/{theme}/uri").uri({domain: domainName, theme: themeName}).sendSync();
    },

    storeWebTheme: function (domainName, theme) {
        return this._request("PUT", "/domains/{domain}/web-themes").uri({domain: domainName}).send(theme);
    },

    deleteWebTheme: function (domainName, themeName) {
        return this._request("DELETE", "/domains/{domain}/web-themes/{theme}").uri({domain: domainName, theme: themeName}).send();
    },

    getTradingPartners: function(domainName) {
        return this._request("GET", "/domains/{domain}/trading-partners").uri({domain: domainName}).send()
                .then(function (data) { return $.map(data || [], jscape.TradingPartner.fromJSON); });
    },

    storeTradingPartner: function(domainName, partner) {
        return this._request("PUT", "/domains/{domain}/trading-partners").uri({domain: domainName}).send(partner);
    },

    copyTradingPartner: function(domainName, sourcePartnerName, targetPartnerName) {
        return this._request("PUT", "/domains/{domain}/trading-partners/{name}").uri({domain: domainName, name: targetPartnerName}).headers({"x-js-copy-source": sourcePartnerName}).send()
                .then(function(data) { return jscape.TradingPartner.fromJSON(data); });
    },

    deleteTradingPartner: function(domainName, partnerName) {
        return this._request("DELETE", "/domains/{domain}/trading-partners/{partner}").uri({domain: domainName, partner: partnerName}).send();
    },

    testTradingPartner: function(domainName, partner) {
        return this._request("POST", "/domains/{domain}/trading-partners/{partner}/test").uri({domain: domainName, partner: partner.name}).send(partner);
    },

    sendTradingPartnerFile: function(domainName, partnerName, path) {
        return this._request("POST", "/domains/{domain}/trading-partners/{partner}/send").uri({domain: domainName, partner: partnerName}).send(path);
    },

    getContacts: function(domainName) {
        return this._request("GET", "/domains/{domain}/contacts").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.Contact.fromJSON); });
    },

    storeContact: function(domainName, contact) {
        return this._request("PUT", "/domains/{domain}/contacts").uri({domain: domainName}).send(contact);
    },

    deleteContact: function(domainName, contact) {
        return this._request("DELETE", "/domains/{domain}/contacts").uri({domain: domainName}).send(contact);
    },

    getServices: function(domainName) {
        return this._request("GET", "/domains/{domain}/services").uri({domain: domainName}).send()
                .then(function(data) { return $.map(data || [], jscape.Service.fromJSON); });
    },

    addService: function(domainName, descriptor) {
        return this._request("PUT", "/domains/{domain}/services").uri({domain: domainName}).send(descriptor);
    },

    deleteService: function(domainName, descriptor) {
        return this._request("DELETE", "/domains/{domain}/services").uri({domain: domainName}).send(descriptor);
    },

    startService: function(domainName, descriptor) {
        return this._request("POST", "/domains/{domain}/services/start").uri({domain: domainName}).send(descriptor);
    },

    getServiceConfiguration: function(domainName, protocol) {
        return this._request("GET", "/domains/{domain}/services/{protocol}/config").uri({domain: domainName, protocol: protocol.toLowerCase()}).send()
                .then(function(data) { return jscape.ServiceConfiguration.fromJSON(data); });
    },

    setServiceConfiguration: function(domainName, config) {
        return this._request("PUT", "/domains/{domain}/services/config").uri({domain: domainName}).send(config);
    },

    getServerKeys: function() {
        return this._request("GET", "/keys/server").send()
                .then(function(data) { return $.map(data || [], jscape.KeyEntryDescriptor.fromJSON); });
    },

    getClientKeys: function() {
        return this._request("GET", "/keys/client").send()
                .then(function (data) { return $.map(data || [], jscape.KeyEntryDescriptor.fromJSON); });
    },

    getClientCertificates: function() {
        return this._request("GET", "/certificates/client").send()
                .then(function (data) { return $.map(data || [], jscape.KeyEntryDescriptor.fromJSON); });
    },

    getHostKeys: function() {
        return this._request("GET", "/keys/host").send()
                .then(function (data) { return $.map(data || [], jscape.KeyEntryDescriptor.fromJSON); });
    },

    getHostCertificates: function() {
        return this._request("GET", "/certificates/host").send()
                .then(function (data) { return $.map(data || [], jscape.KeyEntryDescriptor.fromJSON); });
    },

    getPgpPublicKeys: function() {
        return this._request("GET", "/keys/pgp/public").send()
                .then(function (data) { return $.map(data || [], jscape.PgpKeyEntryDescriptor.fromJSON); });
    },

    getPgpSecretKeys: function() {
        return this._request("GET", "/keys/pgp/secret").send()
                .then(function (data) { return $.map(data || [], jscape.PgpKeyEntryDescriptor.fromJSON); });
    },

    getClientPublicKey: function(alias) {
        return this._request("GET", "/keys/client/{alias}").uri({alias: alias}).send()
                .then(function(data) { return jscape.KeyEntryDescriptor.fromJSON(data); });
    },

    getClientCertificate: function(alias) {
        return this._request("GET", "/certificates/client/{alias}").uri({alias: alias}).send()
                .then(function(data) { return jscape.KeyEntryDescriptor.fromJSON(data); });
    },

    importServerKey: function(alias, file, filePassword, keyAlias, keyPassword, certificateFile, certificatePassword, certificateAlias) {
        return this._request("POST", "/keys/server/{alias}/import").uri({alias: alias}).sendFormMultipart({
            file: file, filePassword: filePassword, keyAlias: keyAlias, keyPassword: keyPassword, certificateFile: certificateFile, certificateFilePassword: certificatePassword, certificateAlias: certificateAlias
        }).then(function(data) { return jscape.KeyEntryDescriptor.fromJSON(data); });
    },

    importServerKeyCertificates: function(alias, file, filePassword, keyAlias) {
        return this._request("POST", "/keys/server/{alias}/import-crt").uri({alias: alias}).sendFormMultipart({
            file: file, filePassword: filePassword, keyAlias: keyAlias
        }).then(function(data) { return jscape.KeyEntryDescriptor.fromJSON(data); });
    },

    importHostKey: function(alias, file) {
        return this._request("POST", "/keys/host/{alias}/import").uri({alias: alias}).sendFormMultipart({file: file})
                .then(function(data) { return jscape.KeyEntryDescriptor.fromJSON(data); });
    },

    importClientKey: function(alias, file) {
        return this._request("POST", "/keys/client/{alias}/import").uri({alias: alias}).sendFormMultipart({file: file})
                .then(function(data) { return jscape.KeyEntryDescriptor.fromJSON(data); });
    },

    importPgpPublicKey: function(file) {
        return this._request("POST", "/keys/pgp/public/import").sendFormMultipart({file: file})
                .then(function(data) { return jscape.PgpKeyEntryDescriptor.fromJSON(data); });
    },

    importPgpSecretKey: function(file, password) {
        return this._request("POST", "/keys/pgp/secret/import").sendFormMultipart({file: file, filePassword: password})
                .then(function(data) { return jscape.PgpKeyEntryDescriptor.fromJSON(data); });
    },

    generateServerKey: function(alias, algorithm, keyLength, period, commonName, organizationUnit, organization, locality, state, country) {
        var data = {keyType: algorithm, keyLength: keyLength, period: period, commonName: commonName, organizationUnit: organizationUnit, organization: organization, locality: locality, state: state, country: country};
        return this._request("POST", "/keys/server/{alias}/generate?" + $.param(data)).uri({alias: alias}).send()
                .then(function(data) { return jscape.KeyEntryDescriptor.fromJSON(data); });
    },

    generateServerKeyCSR: function(alias, filename) {
        return this._request("GET", "/keys/server/{alias}/generate-csr").uri({alias: alias}).sendDownload({filename: filename});
    },

    generateClientKey: function(alias, algorithm, keyLength, period, commonName, organizationUnit, organization, locality, state, country, privateKeyFilename, privateKeyFormat, privateKeyPassword) {
        return this._request("GET", "/keys/client/{alias}/generate").uri({alias: alias}).sendDownload({
            keyType: algorithm, keyLength: keyLength, period: period, commonName: commonName,
            organizationUnit: organizationUnit, organization: organization, locality: locality, state: state, country: country,
            privateKeyFilename: privateKeyFilename, privateKeyFormat: privateKeyFormat, privateKeyPassword: privateKeyPassword
        });
    },

    generatePgpKey: function(realName, email, keyType, keyLength) {
        var data = {realName: realName, email: email, keyType: keyType, keyLength: keyLength};
        return this._request("POST", "/keys/pgp/generate?" + $.param(data)).send()
                .then(function(data) { return jscape.PgpKeyEntryDescriptor.fromJSON(data); });
    },

    exportServerCertificate: function(alias, format, filename) {
        return this._request("GET", "/keys/server/{alias}/export-crt").uri({alias: alias}).sendDownload({format: format, filename: filename});
    },

    exportServerPublicKey: function(alias, format, filename) {
        return this._request("GET", "/keys/server/{alias}/export-pub").uri({alias: alias}).sendDownload({format: format, filename: filename});
    },

    exportServerPrivateKey: function(alias, format, password, filename) {
        return this._request("GET", "/keys/server/{alias}/export-prv").uri({alias: alias}).sendDownload({format: format, filename: filename, filePassword: password});
    },

    exportHostCertificate: function(alias, format, filename) {
        return this._request("GET", "/certificates/host/{alias}/export").uri({alias: alias}).sendDownload({format: format, filename: filename});
    },

    exportHostCertificatePublicKey: function(alias, format, filename) {
        return this._request("GET", "/certificates/host/{alias}/export-pub").uri({alias: alias}).sendDownload({format: format, filename: filename});
    },

    exportHostPublicKey: function(alias, format, filename) {
        return this._request("GET", "/keys/host/{alias}/export").uri({alias: alias}).sendDownload({format: format, filename: filename});
    },

    exportClientCertificate: function(alias, format, filename) {
        return this._request("GET", "/certificates/client/{alias}/export").uri({alias: alias}).sendDownload({format: format, filename: filename});
    },

    exportClientCertificatePublicKey: function(alias, format, filename) {
        return this._request("GET", "/certificates/client/{alias}/export-pub").uri({alias: alias}).sendDownload({format: format, filename: filename});
    },

    exportClientPublicKey: function(alias, format, filename) {
        return this._request("GET", "/keys/client/{alias}/export").uri({alias: alias}).sendDownload({format: format, filename: filename});
    },

    exportPgpPublicKey: function(alias, filename) {
        return this._request("GET", "/keys/pgp/public/{alias}/export").uri({alias: alias}).sendDownload({filename: filename});
    },

    exportPgpSecretKey: function(alias, filename, password) {
        return this._request("GET", "/keys/pgp/secret/{alias}/export").uri({alias: alias}).sendDownload({filename: filename, password: password});
    },

    exportPgpSecretPublicKey: function(alias, filename) {
        return this._request("GET", "/keys/pgp/secret/{alias}/export-pub").uri({alias: alias}).sendDownload({filename: filename});
    },

    deleteServerKey: function(alias) {
        return this._request("DELETE", "/keys/server/{alias}").uri({alias: alias}).send();
    },

    deleteHostPublicKey: function(alias) {
        return this._request("DELETE", "/keys/host/{alias}").uri({alias: alias}).send();
    },

    deleteHostCertificate: function(alias) {
        return this._request("DELETE", "/certificates/host/{alias}").uri({alias: alias}).send();
    },

    deleteClientPublicKey: function(alias) {
        return this._request("DELETE", "/keys/client/{alias}").uri({alias: alias}).send();
    },

    deleteClientCertificate: function(alias) {
        return this._request("DELETE", "/certificates/client/{alias}").uri({alias: alias}).send();
    },

    deletePgpPublicKey: function(alias) {
        return this._request("DELETE", "/keys/pgp/public/{alias}").uri({alias: alias}).send();
    },

    deletePgpSecretKey: function(alias) {
        return this._request("DELETE", "/keys/pgp/secret/{alias}").uri({alias: alias}).send();
    },

    getPgpPublicKeySignatures: function(alias) {
        return this._request("GET", "/keys/pgp/public/{alias}/signatures").uri({alias: alias}).send()
                .then(function(data) { return $.map(data || [], jscape.PgpSignatureDescriptor.fromJSON); });
    },

    getPgpSecretKeySignatures: function(alias) {
        return this._request("GET", "/keys/pgp/secret/{alias}/signatures").uri({alias: alias}).send()
                .then(function(data) { return $.map(data || [], jscape.PgpSignatureDescriptor.fromJSON); });
    },

    signPgpPublicKey: function(alias, secretKeyAlias) {
        return this._request("PUT", "/keys/pgp/public/{alias}/sign?secretKeyAlias={ska}").uri({alias: alias, ska: secretKeyAlias}).send()
                .then(function(data) { return jscape.PgpKeyEntryDescriptor.fromJSON(data); });
    },

    keyUsage: function(alias) {
        return this._request("GET", "/key-usage/{alias}").uri({alias: alias}).send();
    },

    _request: function (method, url, parse) {
        var req = new jscape.Request(method, "/2/rest/management" + url, this.timeout, parse, this.resources);
        return $.isPlainObject(this.config.csrf) ? req.headers(this.config.csrf) : req;
    },

    _submitForm: function(method, url, data) {
        var html = ['<form action="' + encodeURI("/settings" + url) + '" method="' + (method || "post") + '" enctype="application/x-www-form-urlencoded" accept-charset="utf-8">'];
        html.concat($.map(data || {}, function(name, value) {
            return '<input type="hidden" name="' + encodeURIComponent(name) + '" value="' + encodeURIComponent + '" />';
        }));
        html.push("</form>");
        $(html.join("")).appendTo(document.body).submit();
        return $.Deferred().resolve().promise();
    }
});

/** @class */
jscape.Request = Class.extend(
    /** @lends {jscape.Request.prototype} */
{
    url: null,
    settings: {},

    /** @constructor */
    initialize: function(method, url, timeout, parseResponse, resources) {
        this.url = url;
        this.settings = {type: method, cache: false, timeout: timeout};
        this.parseResponse = parseResponse != undefined ? !!parseResponse : true;
        this.resources = resources;
    },

    uri: function(params) {
        params = $.extend({}, params);
        this.url = this.url.supplant($.each(params, function(key, value) {
            params[key] = encodeURIComponent(value);
        }));
        return this;
    },

    headers: function(headers) {
        this.options({headers: $.extend(this.settings["headers"] || {}, headers)});
        return this;
    },

    options: function(options) {
        $.extend(true, this.settings, options || {});
        return this;
    },

    send: function (data) {
        if (!(window.FormData && data instanceof FormData) && typeof data !== "string" && !$.isNumeric(data)) {
            data = JSON.stringify(data);
        }
        var parse = this.parseResponse;
        var _handleError = $.proxy(this._handleError, this);
        return $.ajax(this.url, $.extend({
            contentType: data ? "application/json" : "",
            data: data || "",
            dataType: "text",
            processData: false
        }, this.settings))
        .then(function(data, status, jqXhr) {
             if (!parse) { return jqXhr.responseText; }
             try { return data ? $.parseJSON(data) : null; } catch (e) { return null; }
        })
        .fail(function(jqXhr, status, error) {
            if (this.global) {
                _handleError(jqXhr, status, error);
            }
        });
    },

    sendForm: function(data) {
        data = $.map(data || [], function(value, key) { return key + "=" + ($.type(value) === "object" ? JSON.stringify(value) : encodeURIComponent(value))}).join("&");
        this.settings["contentType"] = "application/x-www-form-urlencoded";
        return this.send(data);
    },

    sendFormMultipart: function (data, fallback) {
        if (window.FormData === undefined) {
            return this.sendForm(data);
        }
        try {
            var fd = new FormData();
            for (var key in data) {
                if (data.hasOwnProperty(key)) {
                    var value = data[key];
                    if ($.type(value) === "object" && !(value instanceof File || value instanceof Blob)) {
                        fd.append(key, this._toBlob(JSON.stringify(value), "application/json"));
                    } else {
                        fd.append(key, value);
                    }
                }
            }
            this.settings["contentType"] = false; // append boundary to multipart/form-data type
            return this.send(fd);
        } catch (e) {
            if (fallback === true) {
                return this.sendForm(data);
            } else {
                throw e;
            }
        }
    },

    sendDownload: function (data) {
        var token = {name: "x-js-dload", value: "_" + Math.random().toString(36).substr(2)};
        var qs = ($.isPlainObject(data) ? $.param(data) + "&" : "") + $.param([token]);
        this.url += (this.url.indexOf("?") != -1 ? "&" : "?") + qs;
        this.settings["loadCompleteToken"] = token;
        this.settings["dataType"] = "iframe";
        return this.send();
    },

    sendSync: function(data) {
        var _handleError = $.proxy(this._handleError, this);
        var response = $.ajax(this.url, $.extend(this.settings, {
            async: false,
            data: data,
            error: function(jqXhr, status, error) {
                if (this.global) {
                    _handleError(jqXhr, status, error);
                }
            }
        }));
        return response.status >= 200 && response.status < 300 ? response.responseText : null;
    },

    _handleError: function(jqXhr, status, error) {
        var message, details, response;
        try {
            response = $.parseJSON(jqXhr.responseText);
            message = this.resources.contains(response.code)
                      ? this.resources.getString(response.code, {0: response.message || ""})
                      : response.message;
            details = response.details || response.message;
        } catch (e) {
            message = jqXhr.statusText || jqXhr.responseText || "Internal server error!";
            details = status || error || "";
        }
        var dialog = new jscape.RestErrorDialog();
        dialog.setTitle((jqXhr.status ? jqXhr.status + " " : "") + jqXhr.statusText);
        dialog.setMessage(message);
        dialog.setDetails(details);
        dialog.show().done(function() {
            if (response && response.code === "SESSION_EXPIRED") { window.location = "/"; }
        });
    },

    _toBlob: function(data, type) {
        try {
            return new Blob([data], {type: type});
        } catch (e) {
            window.BlobBuilder = window.BlobBuilder || window.WebKitBlobBuilder || window.MozBlobBuilder || window.MSBlobBuilder;
            if (window.BlobBuilder) {
                var b = new BlobBuilder();
                b.append(data);
                return b.getBlob(type);
            } else {
                throw "Blob not supported.";
            }
        }
    }
});



(function($) {
    $(document).ajaxStart(function() { this.body.style.cursor = "wait"; });
    $(document).ajaxStop(function() { this.body.style.cursor = "default"; });

    $.ajaxTransport("iframe", function(s) {
        if ("GET" !== s.type || !s.async) {
            throw "GET method supported only."; //todo: POST
        }
        var iframe;
        var timerId;

        function clearCookie(name) {
            document.cookie = name + "=; expires=" + new Date(0).toUTCString() + "; path=/";
        }

        return {
            send: function(headers, callback) {
                iframe = document.createElement("iframe");
                $(iframe).css({width: 0, height: 0, border: "none", visibility: "hidden"});

                iframe.onload = function(_, abort) {
                    var success = document.cookie.indexOf(s.loadCompleteToken.name + "=" + s.loadCompleteToken.value) != -1;
                    var doc = iframe && (iframe.contentDocument || iframe.contentWindow.document);
                    var error = doc && doc.body && doc.body.innerHTML.length != 0;

                    if (abort || success || error) {
                        var responses = error ? {text: doc.body.innerHTML} : {};

                        clearTimeout(timerId);
                        timerId = undefined;
                        iframe.onload = null;
                        setTimeout(function() { // FF bug fix
                            iframe && iframe.parentNode && iframe.parentNode.removeChild(iframe);
                            iframe = undefined;
                        }, 50);

                        if (success) {
                            clearCookie(s.loadCompleteToken.name);
                            callback(200, "success", responses);
                        } else if (error) {
                            callback(500, "error", responses);
                        }
                    } else {
                        if (iframe) {
                            timerId = setTimeout(iframe.onload, 1000);
                        }
                    }
                };
                iframe.src = s.url;
                document.body.appendChild(iframe);
                timerId = setTimeout(iframe.onload, 100);
            },

            abort: function() {
                iframe && iframe.onload(0, 1);
            }
        };
    });
})(jQuery);