var jscape = jscape || {};

/** @class */
jscape.ServerInfoController = jscape.ApplicationModule.extend(
    /** @lends {jscape.ServerInfoController.prototype} */
{
    modules: null,
    moduleQueue: null,
    serverParameters: null,

    /** @constructor */
    initialize: function (resources, modules) {
        this.modules = modules || [];
        this.view = new jscape.ServerInfoView(resources);
        this.menu = new jscape.Menu();
        this.menu.bind(this, "server");
    },

    start: function (managementService) {
        this.managementService = managementService;

        this._initModuleQueue();

        this.managementService.getServerParameters().done($.proxy(function (parameters) {
            this.serverParameters = parameters;
            this._startNextModule();
        }, this));

        this.view.show(this);
    },

    onMenuItemSelected: function (item) {
        if (item && item.id) {
            switch (item.id) {
                case jscape.Menu.ITEM.Home:
                case jscape.Menu.ITEM.KeyManager:
                case jscape.Menu.ITEM.ServerSettings:
                    setTimeout($.proxy(function() { this.onRedirect(item.url || "/"); }, this), 10);
                    return true;
                case jscape.Menu.ITEM.Logout:
                    this.onLogout();
                    return true;
            }
        }
        return false;
    },

    onSubmit: function () {
        this._initModuleQueue();
        this._submitNextModule();
    },

    onCancel: function () {
        this._initModuleQueue();
        this._cancelNextModule();
    },

    onStarted: function () {
        if (this.moduleQueue.length == 0) {
            this.view.show(this);
        } else {
            this._startNextModule();
        }
    },

    onStartError: function () {},
    onSubmitError: function () {},

    onChangesSubmitted: function () {
        if (this.moduleQueue.length == 0) {
            this.onRedirect("/");
        } else {
            this._submitNextModule();
        }
    },

    onChangesCancelled: function () {
        if (this.moduleQueue.length == 0) {
            this.onRedirect("/");
        } else {
            this._cancelNextModule();
        }
    },

    onLogout: function() {
        this.managementService.logout();
    },

    _initModuleQueue: function () {
        this.moduleQueue = [].concat(this.modules);
    },

    _startNextModule: function () {
        var m = this.moduleQueue.pop();
        m.start(this, this.managementService, this.serverParameters);
        this.menu.bind(m, "server");
    },

    _submitNextModule: function () {
        var m = this.moduleQueue.pop();
        m.submitChanges();
        this.menu.unbind("server");
    },

    _cancelNextModule: function () {
        var m = this.moduleQueue.pop();
        m.cancelChanges();
        this.menu.unbind("server");
    }
});

/** @class */
jscape.ServerInfoView = Class.extend(
    /** @lends {jscape.ServerInfoView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.throbber = $("#throbber");
        this.panel = $("#viewpane");
        this.panel.layout({fit: true, border: false}).css({visibility: "hidden"});

        this.tabs = $("#modules");
        this.tabs.tabs({border: true, plain: true});
    },

    show: function(listener) {
        this.listener = listener || jscape.ServerInfoView.LISTENER;
        this.throbber.remove();
        this.throbber = null;
        this.panel.css({visibility: "visible"});
    }
});
jscape.ServerInfoView.LISTENER = {};


/** @class */
jscape.ServerStatisticsController = jscape.ApplicationModule.extend(
    /** @lends {jscape.ServerStatisticsController.prototype} */
{
    /** @const */
    UPDATE_PERIOD: 15000,

    serverParameters: null,
    statistics: null,
    timer: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.serverStatsView = new jscape.ServerStatisticsView(resources);
        this.domainsView = new jscape.ServerDomainsView(resources);
        this.newDomainController = new jscape.domain.NewDomainController(resources);
        this.domainStatusController = new jscape.domain.DomainStatusController(resources);
    },

    start: function(listener, managementService, serverParameters) {
        //this._super(listener, managementService);
        this.managementService = managementService;
        this.parameters = serverParameters;

        this._setupView();
    },

    onMenuItemSelected: function (item) {
        switch (item.id) {
            case jscape.Menu.ITEM.NewDomain:
                this.onAddDomain();
                return true;
            case jscape.Menu.ITEM.StartDomains:
                this._startAllDomains();
                return true;
            case jscape.Menu.ITEM.StopDomains:
                this._stopAllDomains();
                return true;
            case jscape.Menu.ITEM.PauseDomains:
                this._pauseAllDomains();
                return true;
            case jscape.Menu.ITEM.ResumeDomains:
                this._resumeAllDomains();
                return true;
            default:
                return false;
        }
    },

    onStartDomain: function(domainName) {
        this.managementService.startDomain(domainName)
                .done($.proxy(this._domainStateChanged, this, domainName));
    },

    onStopDomain: function(domainName) {
        this.managementService.stopDomain(domainName)
                .done($.proxy(this._domainStateChanged, this, domainName));
    },

    onPauseDomain: function(domainName) {
        this.managementService.pauseDomain(domainName)
                .done($.proxy(this._domainStateChanged, this, domainName));
    },

    onResumeDomain: function(domainName) {
        this.managementService.resumeDomain(domainName)
                .done($.proxy(this._domainStateChanged, this, domainName));
    },

    onAddDomain: function() {
        this.newDomainController.start(this.managementService)
                .done($.proxy(this._refreshStatistics, this));
    },

    onEditDomain: function(domainName) {
        this.managementService.editDomain(domainName);
    },

    onDeleteDomain: function (domainName) {
        this.domainsView.showConfirmDeleteDialog(domainName)
                .done($.proxy(function () {
                    this.managementService.deleteDomain(domainName)
                            .done($.proxy(this._refreshStatistics, this));
                }, this));
    },

    onStatusDomain: function(domainName) {
        this.domainStatusController.start(this.managementService, domainName);
    },

    _domainStateChanged: function(domainName) {
        this.managementService.getDomainStatistics(domainName)
                .done($.proxy(function(statistics) {
                    this.domainsView.updateDomain(statistics);
                }, this));
    },

    _startAllDomains: function() {
        this._domainBatchOperation(this.managementService.startDomain);
    },

    _stopAllDomains: function() {
        this._domainBatchOperation(this.managementService.stopDomain);
    },

    _pauseAllDomains: function() {
        this._domainBatchOperation(this.managementService.pauseDomain);
    },

    _resumeAllDomains: function() {
        this._domainBatchOperation(this.managementService.resumeDomain);
    },

    _domainBatchOperation: function (operation) {
        this.managementService.getDomainNames()
                .done($.proxy(function (names) {
                    var promises = $.map(names, $.proxy(function(name) {
                        return operation.call(this.managementService, name);
                    }, this));

                    $.when(promises).done($.proxy(this._refreshStatistics, this));
                }, this));
    },

    _update: function() {
        clearTimeout(this.timer);
        this.timer = setTimeout($.proxy(function() {
            this._refreshStatistics().done($.proxy(this._update, this));
        }, this), this.UPDATE_PERIOD);
    },

    _refreshStatistics: function () {
        return this.managementService.getServerStatistics()
                .done($.proxy(function (statistics) {
                    this.statistics = statistics;
                    this.serverStatsView.setStatistics(this.parameters, this.statistics);
                    this.domainsView.setDomains(this.statistics.domainStatistics);
                }, this));
    },

    _setupView: function() {
        this.serverStatsView.show(this);
        this.domainsView.show(this);
        this._refreshStatistics();
        this._update();
    }
});

/** @class */
jscape.ServerStatisticsView = Class.extend(
    /** @lends {jscape.ServerStatisticsView.prototype} */
{
    /** @const */
    DATE_FORMAT: new jscape.DateFormat("{MM}-{dd}-{yy}"),

    parameters: null,
    statistics: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=serverstatisticsfields]");

        this.data = $("table[role=grid][aria-label=statistics]", this.fieldset);
        this.data.propertygrid({
            showGroup: false,
            loadFilter: $.proxy(this._loadFilter, this)
        });
    },

    show: function(listener) {
        this.listener = listener || jscape.ServerStatisticsView.LISTENER;
    },

    setStatistics: function(serverParameters, serverStatistics) {
        this.parameters = serverParameters;
        this.statistics = serverStatistics;
        this._refresh();
    },

    _refresh: function() {
        this.data.propertygrid("loadData", this);
    },

    _loadFilter: function() {
        var params = this.parameters;
        var stats = this.statistics;
        var licenseType, licenseOwner, licenseHost, licenseExpires = "";
        if (params && params.license) {
            licenseType = params.license.type;
            licenseOwner = params.license.owner;
            licenseHost = this._formatBindingHost(params.license.host);
            licenseExpires = this._formatExpirationDate(params.license.expirationDate);
        }
        var domains = stats ? stats.domainStatistics.length : "";
        var uptime = stats ? this._formatUptime(stats.uptime) : "";
        var connections = stats ? this._countCurrentConnections(stats.domainStatistics) : "";
        var memory = stats ? this._formatMemoryUsage(stats.usedMemory, stats.maxMemory) : "";
        var threads = stats ? stats.threads : "";
        var rows = [
            {name: this.resources.getString("SERVER_VERSION"), value: params && params.appVersion},
            {name: this.resources.getString("SERVER_LICENSE_TYPE"), value: licenseType},
            {name: this.resources.getString("SERVER_LICENSE_OWNER"), value: licenseOwner},
            {name: this.resources.getString("SERVER_LICENSE_HOST"), value: licenseHost},
            {name: this.resources.getString("SERVER_LICENSE_EXPIRES"), value: licenseExpires},
            {name: this.resources.getString("SERVER_DOMAINS"), value: domains},
            {name: this.resources.getString("SERVER_UPTIME"), value: uptime},
            {name: this.resources.getString("SERVER_MANAGEMENT_CONNECTIONS"), value: stats && stats.managementConnections},
            {name: this.resources.getString("SERVER_CONNECTIONS"), value: connections},
            {name: this.resources.getString("SERVER_MEMORY"), value: memory},
            {name: this.resources.getString("SERVER_THREADS"), value: threads}
        ];
        return {total: rows.length, rows: rows};
    },

    _countCurrentConnections: function(domainStatistics) {
        var connections = 0;
        for (var i = 0; i < domainStatistics.length; ++i) {
            connections += domainStatistics[i].currentSessions;
        }
        return connections;
    },

    _formatBindingHost: function(host) {
        return host ? host : this.resources.getString("SERVER_LICENSE_HOST_NON_BINDING");
    },

    _formatExpirationDate: function(date) {
        return date ? this.DATE_FORMAT.format(new Date(date)) : this.resources.getString("SERVER_LICENSE_EXPIRES_NON_EXPIRING");
    },

    _formatMemoryUsage: function(usedMemory, maxMemory) {
        return this._formatSize(usedMemory) + " of " + this._formatSize(maxMemory);
    },

    _formatUptime: function(value) {
        var uptime = new jscape.Hours(value);
        var pattern = [];
        if (uptime.hours) {
            pattern.push("{hours} hrs.");
        }
        if (uptime.minutes) {
            pattern.push("{minutes} min.");
        }
        if (uptime.seconds) {
            pattern.push("{seconds} sec.");
        }
        return pattern.join(" ").supplant(uptime);
    },

    _formatSize: function(value) {
        return new jscape.SizeFormat().format(value);
    }
});
jscape.ServerStatisticsView.LISTENER = {};

/** @class */
jscape.ServerDomainsView = Class.extend(
    /** @lends {jscape.ServerDomainsView.prototype} */
{
    /** @const */
    DOMAIN_STATE: {running: 0, paused: 1, stopped: 2},

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=serverdomainsfields]");

        this.domains = $("table[role=grid][aria-label=serverdomains]", this.fieldset);
        this.domains.datagrid({
            fitColumns: true,
            singleSelect:true,
            remoteSort: false,
            onSelect: $.proxy(this._adjustButtons, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this),
            idField: "name",
            columns: [[
                {field: "name", rowspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_NAME"), sortable: true, width: 175},
                {field: "state", rowspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_STATE"), sortable: true, width: 90, formatter: $.proxy(this._formatState, this), styler: $.proxy(this._styleState, this)},
                {field: "current_sessions", rowspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_CURRENT_SESSIONS"), sortable: true, width: 110, align: "right"},
                {field: "current_transfers", rowspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_CURRENT_TRANSFERS"), sortable: true, width: 110, align: "right"},
                {field: "total_sessions", rowspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_TOTAL_SESSIONS"), sortable: true, width: 110, align: "right"},
                {colspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_UPLOADED")},
                {colspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_DOWNLOADED")},
                {colspan: 3, title: resources.getString("DOMAINS_TABLE_COLUMN_QUOTA")},
                {field: "date_start", rowspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_DATE_START"), sortable: true, width: 150, align: "right", formatter: $.proxy(this._formatDate, this)},
                {field: "date_stop", rowspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_DATE_STOP"), sortable: true, width: 150, align: "right", formatter: $.proxy(this._formatDate, this)}
            ],[
                {field: "uploaded_bytes", rowspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_UPLOADED_BYTES"), sortable: true, width: 110, align: "right", formatter: $.proxy(this._formatSize, this)},
                {field: "uploaded_files", rowspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_UPLOADED_FILES"), sortable: true, width: 100, align: "right"},
                {field: "downloaded_bytes", rowspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_DOWNLOADED_BYTES"), sortable: true, width: 110, align: "right", formatter: $.proxy(this._formatSize, this)},
                {field: "downloaded_files", rowspan: 2, title: resources.getString("DOMAINS_TABLE_COLUMN_DOWNLOADED_FILES"), sortable: true, width: 100, align: "right"},
                {field: "quota_uploads", title: resources.getString("DOMAINS_TABLE_COLUMN_QUOTA_UPLOADS"), sortable: true, width: 140, align: "right", formatter: $.proxy(this._formatQuota, this)},
                {field: "quota_downloads", title: resources.getString("DOMAINS_TABLE_COLUMN_QUOTA_DOWNLOADS"), sortable: true, width: 150, align: "right", formatter: $.proxy(this._formatQuota, this)},
                {field: "quota_transfers", title: resources.getString("DOMAINS_TABLE_COLUMN_QUOTA_TRANSFERS"), sortable: true, width: 150, align: "right", formatter: $.proxy(this._formatQuota, this)}
            ]]
        });

        this.startButton = $("a[role=button][name=start]", this.fieldset);
        this.startButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$StartButton, this)});

        this.stopButton = $("a[role=button][name=stop]", this.fieldset);
        this.stopButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$StopButton, this)});

        this.pauseButton = $("a[role=button][name=pause]", this.fieldset);
        this.pauseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$PauseButton, this)});

        this.resumeButton = $("a[role=button][name=resume]", this.fieldset);
        this.resumeButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ResumeButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.statusButton = $("a[role=button][name=status]", this.fieldset);
        this.statusButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$StatusButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || new jscape.ServerDomainsView.LISTENER;
        this._adjustButtons();
    },

    showConfirmDeleteDialog: function (domainName) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("DOMAINS_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("DOMAINS_CONFIRM_DELETE_MESSAGE", {0: domainName}));
    },

    addDomain: function(domainStatistics) {
        this.domains.datagrid("appendRow", this._statistics2Row(domainStatistics));
        this._adjustButtons();
    },

    updateDomain: function(statistics) {
        var index = this.domains.datagrid("getRowIndex", statistics.domainName);
        if (index != -1) {
            this.domains.datagrid("updateRow", {index: index, row: this._statistics2Row(statistics)});
            this._adjustButtons();
        }
    },

    removeDomain: function(domainName) {
        var index = this.domains.datagrid("getRowIndex", domainName);
        if (index != -1) {
            this.domains.datagrid("deleteRow", index);
            this._adjustButtons();
        }
    },

    setDomains: function(values) {
        var data = $.map(values || [], $.proxy(this._statistics2Row, this));
        this.domains.datagrid("loading");
        this.domains.datagrid("loadData", data);
        this.domains.datagrid("loaded");
        this._adjustButtons();
    },

    _statistics2Row: function (stats) {
        return {
            "name": stats.domainName,
            "state": stats.isRunning() ? (stats.paused ? this.DOMAIN_STATE.paused : this.DOMAIN_STATE.running) : this.DOMAIN_STATE.stopped,
            "current_sessions": stats.currentSessions,
            "current_transfers": stats.currentTransfers,
            "total_sessions": stats.totalSessions,
            "uploaded_bytes": stats.bytesUploaded,
            "uploaded_files": stats.filesUploaded,
            "downloaded_bytes": stats.bytesDownloaded,
            "downloaded_files": stats.filesDownloaded,
            "quota_uploads": stats.uploadsQuota ? {used: stats.currentUploadedBytes, total: stats.uploadsQuota} : null,
            "quota_downloads": stats.downloadsQuota ? {used: stats.currentDownloadedBytes, total: stats.downloadsQuota} : null,
            "quota_transfers": stats.transfersQuota ? {used: stats.currentUploadedBytes + stats.currentDownloadedBytes, total: stats.transfersQuota} : null,
            "date_start": stats.startDate ? new Date(stats.startDate) : null,
            "date_stop": stats.stopDate ? new Date(stats.stopDate) : null
        };
    },

    _formatState: function(value) {
        switch (value) {
            case this.DOMAIN_STATE.running:
                return this.resources.getString("DOMAIN_STATE_RUNNING");
            case this.DOMAIN_STATE.paused:
                return this.resources.getString("DOMAIN_STATE_PAUSED");
            case this.DOMAIN_STATE.stopped:
                return this.resources.getString("DOMAIN_STATE_STOPPED");
            default:
                return "";
        }
    },

    _styleState: function(value) {
        switch (value) {
            case this.DOMAIN_STATE.running:
                return "color:green;font-weight:normal";
            case this.DOMAIN_STATE.paused:
                return "color:lightgreen;font-weight:bold";
            case this.DOMAIN_STATE.stopped:
                return "color:red;font-weight:normal";
            default:
                return "color:black;font-weight:normal";
        }
    },

    _formatDate: function(value) {
        return $.type(value) === "date" ? value.toLocaleString() : "";
    },

    _formatQuota: function(quota) {
        return quota != null ? this.resources.getString("DOMAINS_TABLE_QUOTA_PATTERN", {0: this._formatSize(quota.used), 1: this._formatSize(quota.total)}) : "";
    },

    _formatSize: function(value) {
        return new jscape.SizeFormat().format(value);
    },

    getSelectedDomain: function() {
        var domain = this.domains.datagrid("getSelected");
        return domain != null ? domain.name : null;
    },

    _onClick$StartButton: function() {
        this.listener.onStartDomain(this.getSelectedDomain());
    },

    _onClick$StopButton: function() {
        this.listener.onStopDomain(this.getSelectedDomain());
    },

    _onClick$PauseButton: function() {
        this.listener.onPauseDomain(this.getSelectedDomain());
    },

    _onClick$ResumeButton: function() {
        this.listener.onResumeDomain(this.getSelectedDomain());
    },

    _onClick$AddButton: function() {
        this.listener.onAddDomain();
    },

    _onClick$EditButton: function() {
        this.listener.onEditDomain(this.getSelectedDomain());
    },

    _onClick$StatusButton: function() {
        this.listener.onStatusDomain(this.getSelectedDomain());
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteDomain(this.getSelectedDomain());
    },

    _adjustButtons: function() {
        var domain = this.domains.datagrid("getSelected");
        this.startButton.linkbutton(domain != null && domain.state == this.DOMAIN_STATE.stopped ? "enable" : "disable");
        this.stopButton.linkbutton(domain != null && domain.state != this.DOMAIN_STATE.stopped ? "enable" : "disable");
        this.pauseButton.linkbutton(domain != null && domain.state == this.DOMAIN_STATE.running ? "enable" : "disable");
        this.resumeButton.linkbutton(domain != null && domain.state == this.DOMAIN_STATE.paused ? "enable" : "disable");
        this.editButton.linkbutton(domain != null ? "enable" : "disable");
        this.deleteButton.linkbutton(domain != null ? "enable" : "disable");
        this.statusButton.linkbutton(domain != null ? "enable" : "disable");
    }
});
jscape.ServerDomainsView.LISTENER = {
    onStartDomain: $.noop,
    onStopDomain: $.noop,
    onPauseDomain: $.noop,
    onResumeDomain: $.noop,
    onAddDomain: $.noop,
    onEditDomain: $.noop,
    onDeleteDomain: $.noop,
    onStatusDomain: $.noop
};

/** @class */
jscape.ServerInterfacesController = jscape.ApplicationModule.extend(
    /** @lends {jscape.ServerInterfacesController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.ServerInterfacesView(resources);
    },

    start: function(listener, managementService, serverParameters) {
        this.managementService = managementService;
        this.view.setInterfaces(serverParameters.interfaces);
        this.view.show(this);

        listener.onStarted();
    }
});
/** @class */
jscape.ServerInterfacesView = Class.extend(
    /** @lends {jscape.ServerInterfacesView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=serverinterfacesfields]");

        this.data = $("table[role=grid][aria-label=interfaces]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            height: 310,
            remoteSort: false,
            singleSelect: true,
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true},
                {field: "displayName", title: $("thead th:nth-child(2)", this.data).text(), sortable: true},
                {field: "addresses", title: $("thead th:nth-child(3)", this.data).text(), sortable: true, width: 200, formatter: function(rowData) { return rowData.join(", "); }}
            ]]
        });
    },

    show: function(listener) {
        this.listener = listener || jscape.ServerInterfacesView.LISTENER;
    },

    setInterfaces: function(values) {
        this.data.datagrid("loading", values);
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded", values);
    }
});
jscape.ServerInterfacesView.LISTENER = {};

/** @class */
jscape.domain.DomainStatusController = Class.extend(
    /** @lends {jscape.domain.DomainStatusController.prototype} */
{
    DELAY: 10 * 1000,

    timerId: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._setupDialog().show(this);
        this._loadDomainState();

        return this.deferred.promise();
    },

    update: function(statistics) {
        this.dialog.setDomainState(this.domainName, jscape.DomainState.stateFor(statistics));

        this.statisticsController.setStatistics(statistics);
        this.sessionsController.setSessions(statistics.connectionStatistics);
        this.eventLogController.update(statistics);
    },

    onStartDomain: function() {
        this.managementService.startDomain(this.domainName).done($.proxy(this._loadDomainState, this));
    },

    onStopDomain: function() {
        this.managementService.stopDomain(this.domainName).done($.proxy(this._loadDomainState, this));
    },

    onPauseDomain: function() {
        this.managementService.pauseDomain(this.domainName).done($.proxy(this._loadDomainState, this));
    },

    onResumeDomain: function() {
        this.managementService.resumeDomain(this.domainName).done($.proxy(this._loadDomainState, this));
    },

    onClose: function (dialog) {
        dialog.destroy();
        this._stopHeartbeat();
        this.deferred.resolve();

        this.deferred = null;
        this.statisticsController = null;
        this.sessionsController = null;
        this.eventLogController = null;
    },

    _loadDomainState: function () {
        return this.managementService.getDomainStatistics(this.domainName)
                .done($.proxy(function (statistics) {
                    this.update(statistics);
                    this._heartbeat(this.DELAY);
                }, this))
                .fail($.proxy(this._stopHeartbeat, this));
    },

    _heartbeat: function(delay) {
        this._stopHeartbeat();
        this.timerId = setTimeout($.proxy(function() {
            this.timerId = null;
            this._loadDomainState()
                    .fail($.proxy(function() { this.timerId = null; }, this));
        }, this), delay);
    },

    _stopHeartbeat: function() {
        if (this.timerId) {
            clearInterval(this.timerId);
            this.timerId = null;
        }
    },

    _setupDialog: function() {
        var dialog = new jscape.domain.DomainStatusDialog(this.resources);

        try {
            this.statisticsController = new jscape.domain.StatisticsController(this.resources, dialog.container());
            this.sessionsController = new jscape.domain.SessionsController(this.resources, dialog.container());
            this.eventLogController = new jscape.domain.EventLogController(this.resources, dialog.container());

            this.statisticsController.start(this.managementService, this.domainName);
            this.sessionsController.start(this.managementService, this.domainName);
            this.eventLogController.start(this.managementService, this.domainName);
            this.dialog = dialog;
            return dialog;
        } catch (e) {
            dialog.destroy();
            throw e;
        }
    }
});

/** @class */
jscape.domain.DomainStatusDialog = Class.extend(
    /** @lends {jscape.domain.DomainStatusDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.dialog = $("#d-domain-status").clone().attr("id", "");
        this.dialog.dialog({
            width: 980, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_CLOSE") || "Close", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.title = this.dialog.dialog("header").text() || "";

        this.fieldset = $("fieldset[name=domainstatusfields]", this.dialog);
        this.tabs = $("div[role=tablist]", this.fieldset);
        this.tabs.tabs({tabPosition: "top", plain: true, width: 920, height: 470});

        this.startButton = $("a[role=button][name=startdomain]", this.fieldset);
        this.startButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$StartButton, this)});

        this.stopButton = $("a[role=button][name=stopdomain]", this.fieldset);
        this.stopButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$StopButton, this)});

        this.pauseButton = $("a[role=button][name=pausedomain]", this.fieldset);
        this.pauseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$PauseButton, this)});

        this.resumeButton = $("a[role=button][name=resumedomain]", this.fieldset);
        this.resumeButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ResumeButton, this)});
    },

    container: function() {
        return this.dialog;
    },

    setDomainState: function(domainName, state) {
        this._adjustDomainState(domainName, state);
        this._adjustButtonsState(state);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.find("textarea[name=eventlogs]").css("width", "63em");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onCancel: function () {
        this.hide();
        this.listener.onClose(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    },

    _onClick$StartButton: function() {
        this.listener.onStartDomain();
    },

    _onClick$StopButton: function() {
        this.listener.onStopDomain();
    },

    _onClick$PauseButton: function() {
        this.listener.onPauseDomain();
    },

    _onClick$ResumeButton: function() {
        this.listener.onResumeDomain();
    },

    _adjustDomainState: function (domainName, domainState) {
        var title = domainState
                    ? this.title.supplant({0: domainName, 1: this.resources.getString("DOMAIN_STATE_" + domainState.toUpperCase())})
                    : this.resources.getString("DOMAIN_MESSAGE_LOADING");
        this.dialog.dialog("setTitle", title);
    },

    _adjustButtonsState: function (domainState) {
        this.startButton.linkbutton(jscape.DomainState.STOPPED == domainState ? "enable" : "disable");
        this.stopButton.linkbutton(jscape.DomainState.STOPPED != domainState ? "enable" : "disable");
        this.pauseButton.linkbutton(jscape.DomainState.RUNNING == domainState ? "enable" : "disable");
        this.resumeButton.linkbutton(jscape.DomainState.PAUSED == domainState ? "enable" : "disable");
    }
});
jscape.domain.DomainStatusDialog.LISTENER = {
    onStartDomain: $.noop,
    onStopDomain: $.noop,
    onPauseDomain: $.noop,
    onResumeDomain: $.noop,
    onClose: $.noop
};