var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.DbDrivers = Class.extend(
    /** @lends {jscape.DbDrivers.prototype} */
{
    /** @constructor */
    initialize: function(drivers) {
        this.entries = drivers || [];
    },

    toJSON: function() {
        var items = [];
        for (var i = 0; i < this.entries.length; i++) {
            items.push(this.entries[i].className);
        }
        return items;
    }
});
jscape.DbDrivers.fromJSON = function(data) {
    var entries = [];
    for (var i = 0; i < data.length; i++) {
        entries.push(new jscape.DbDrivers.Entry(data[i]));
    }
    return new jscape.DbDrivers(entries);
};
/** @class */
jscape.DbDrivers.Entry = Class.extend(
    /** @lends {jscape.DbDrivers.Entry.prototype} */
{
    /** @constructor */
    initialize: function(className) {
        this.className = className || "";
    }
});

/** @class */
jscape.settings.DbDriversController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.DbDriversController.prototype} */
{
    view: null,
    config: null,
    drivers: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.DbDriversView(resources);

        this._initValidators(resources);
    },

    /** @override */
    hasChanged: function () {
        return !Object._equals(this.config, this._createConfiguration());
    },

    /** @override */
    submitChanges: function() {
        var drivers = this._createConfiguration();
        this.serverSettings.setDbDrivers(drivers);
        this.listener.onChangesSubmitted(this);
    },

    onAddDriver: function() {
        this.view.showAddDialog({
            onSubmit: $.proxy(this._onSubmit$AddDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onEditDriver: function(driver) {
        var index = $.inArray(driver, this.drivers);
        this.view.showEditDialog({
            onSubmit: $.proxy(this._onSubmit$EditDialog, this, index),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, driver.className);
    },

    onDeleteDriver: function(driver) {
        this.view.showConfirmDeleteDialog(driver.className).done($.proxy(this._onConfirm$DeleteDialog, this, driver));
    },

    onDriverUp: function(driver) {
        var index = $.inArray(driver, this.drivers);
        this.drivers.splice(index - 1, 2, this.drivers[index], this.drivers[index - 1]);
        this.view.setDrivers(this.drivers, index - 1);
    },

    onDriverDown: function(driver) {
        var index = $.inArray(driver, this.drivers);
        this.drivers.splice(index, 2, this.drivers[index + 1], this.drivers[index]);
        this.view.setDrivers(this.drivers, index + 1);
    },

    _onSubmit$AddDialog: function(dialog) {
        var driver = new jscape.DbDrivers.Entry(dialog.getClassName());
        dialog.hide();

        this.drivers.push(driver);
        this.view.setDrivers(this.drivers, $.inArray(driver, this.drivers));
    },

    _onSubmit$EditDialog: function(index, dialog) {
        var driver = new jscape.DbDrivers.Entry(dialog.getClassName());
        dialog.hide();

        this.drivers[index] = driver;
        this.view.setDrivers(this.drivers, index);
    },

    _onConfirm$DeleteDialog: function(driver) {
        var index = $.inArray(driver, this.drivers);
        if (index != -1) {
            this.drivers.splice(index, 1);
            this.view.setDrivers(this.drivers, index);
        }
    },

    _onCancel$Dialog: function(dialog) {
        dialog.hide();
    },

    _initConfiguration: function() {
        this.config = this.serverSettings.getDbDrivers();
        this.drivers = $.extend(true, [], this.config.entries);
    },

    _createConfiguration: function() {
        return new jscape.DbDrivers(this.view.getDrivers());
    },

    _setupView: function() {
        this.view.setDrivers(this.drivers);
        this.view.show(this);
    },

    _initValidators: function(resources) {
        $.extend($.fn.validatebox.defaults.rules, {
            'dbdriver_nonempty': {
                validator: function(value) { return value && value.trim().length != 0; },
                message: resources.getString("DB_DRIVERS_BAD_CLASS")
            },
            'dbdriver_notexists': {
                validator: $.proxy(this._validatorDriverExists, this),
                message: resources.getString("DB_DRIVERS_ERROR_CLASS_EXISTS")
            }
        });
    },

    _validatorDriverExists: function(value, excludes) {
        for (var i = 0; i < this.drivers.length; i++) {
            var driver = this.drivers[i];
            if (excludes && $.inArray(driver.className, excludes) != -1) {
                continue;
            }
            if (driver.className == value) {
                return false;
            }
        }
        return true;
    }
});

/** @class */
jscape.settings.DbDriverDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.settings.DbDriverDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#ssjdbc-dialog-dbdriver"), resources);

        this.className = $("#ssjdbc-driverclass");
        this._initClassValidator();
    },

    reset: function() {
        this.setClassName("");
    },

    getClassName: function() {
        return this.className.val();
    },

    setClassName: function(value) {
        this.className.val(value);
        this._initClassValidator(value);
    },

    /** @override */
    _valid: function() {
        return this.className.validatebox("isValid");
    },

    _initClassValidator: function(value) {
        this.className.validatebox({
            required: true,
            validType: ["dbdriver_nonempty", "dbdriver_notexists" + (value ? "['" + value + "']" : "")],
            missingMessage: this.resources.getString("DB_DRIVERS_BAD_CLASS")
        });
    }
});

/** @class */
jscape.settings.DbDriversView = Class.extend(
    /** @lends {jscape.settings.DbDriversView.prototype} */
{
    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.table = $("#ssjdbc-data");
        this.moveUpButton = $("#ssjdbc-up");
        this.moveDownButton = $("#ssjdbc-down");
        this.addButton = $("#ssjdbc-add");
        this.editButton = $("#ssjdbc-edit");
        this.deleteButton = $("#ssjdbc-delete");

        this._init();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.DbDriversView.LISTENER;
        this._initState();
    },

    showAddDialog: function(listener) {
        var dialog = new jscape.settings.DbDriverDialog(this.resources);
        dialog.setTitle(this.resources.getString("DB_DRIVERS_ADD_TITLE"));
        dialog.reset();
        dialog.show(listener);
    },

    showEditDialog: function(listener, className) {
        var dialog = new jscape.settings.DbDriverDialog(this.resources);
        dialog.setTitle(this.resources.getString("DB_DRIVERS_EDIT_TITLE"));
        dialog.setClassName(className);
        dialog.show(listener);
    },

    showConfirmDeleteDialog: function(className) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("DB_DRIVERS_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("DB_DRIVERS_CONFIRM_DELETE_MESSAGE", {0: className}));
    },

    getDrivers: function() {
        return this.table.datagrid("getRows");
    },

    setDrivers: function(drivers, selectIndex) {
        this.table.datagrid("loading");
        this.table.datagrid("loadData", drivers);
        this.table.datagrid("loaded");

        if (selectIndex !== undefined) {
            var index = Math.max(0, Math.min(selectIndex, drivers.length - 1));
            this.table.datagrid("selectRow", index);
        }
    },

    _init: function() {
        this._initComponents();
    },

    _initComponents: function() {
        this.table.datagrid({
            singleSelect: true,
            remoteSort: false,
            fitColumns: true,
            onSelect: $.proxy(this._onSelect$Row, this),
            onSortColumn: $.proxy(this._onSort$Column, this),
            columns: [[
                {field: "className", title: this.resources.getString("TABLE_COLUMN_DB_DRIVER_CLASS"), sortable: true, width: 100}
            ]]
        });

        this.moveUpButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveUp, this)});
        this.moveDownButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveDown, this)});
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$Add, this)});
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$Edit, this)});
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$Delete, this)});
    },

    _initState: function() {
        this._adjustButtonsState();
    },

    _onSelect$Row: function() {
        this._adjustButtonsState();
    },

    _onSort$Column: function() {
        this._adjustButtonsState();
    },

    _onClick$MoveUp: function() {
        if (this._buttonEnabled(this.moveUpButton)) {
            var driver = this.table.datagrid("getSelected");
            this.listener.onDriverUp(driver);
        }
    },

    _onClick$MoveDown: function() {
        if (this._buttonEnabled(this.moveDownButton)) {
            var driver = this.table.datagrid("getSelected");
            this.listener.onDriverDown(driver);
        }
    },

    _onClick$Add: function() {
        this.listener.onAddDriver();
    },

    _onClick$Edit: function() {
        if (this._buttonEnabled(this.editButton)) {
            var driver = this.table.datagrid("getSelected");
            this.listener.onEditDriver(driver);
        }
    },

    _onClick$Delete: function() {
        if (this._buttonEnabled(this.deleteButton)) {
            var driver = this.table.datagrid("getSelected");
            this.listener.onDeleteDriver(driver);
        }
    },

    _adjustButtonsState: function() {
        var driver = this.table.datagrid("getSelected");
        var index = this.table.datagrid("getRowIndex", driver);
        var count = this.table.datagrid("getRows").length;

        this._setButtonEnabled(this.moveUpButton, index > 0 && index <= count - 1);
        this._setButtonEnabled(this.moveDownButton, index >= 0 && index < count - 1);
        this._setButtonEnabled(this.editButton, driver != null);
        this._setButtonEnabled(this.deleteButton, driver != null);
    },

    _buttonEnabled: function (button) {
        return !button.linkbutton("options").disabled;
    },

    _setButtonEnabled: function(button, enable) {
        button.linkbutton(enable ? "enable" : "disable");
    }

});
/** @const */
jscape.settings.DbDriversView.LISTENER = {
    onAddDriver: $.noop,
    onEditDriver: $.noop,
    onDeleteDriver: $.noop,
    onDriverUp: $.noop,
    onDriverDown: $.noop
};