var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.DomainsAutoStart = Class.extend(
    /** @lends {jscape.DomainsAutoStart.prototype} */
{
    /** @constructor */
    initialize: function(autostart) {
        this.entries = autostart || [];
    },

    addAutoStart: function(domainName, autoStart) {
        this.entries.push({domain: String(domainName), startup: !!autoStart});
    },

    toJSON: function() {
        var domains = {};
        for (var i = 0; i < this.entries.length; i++) {
            var entry = this.entries[i];
            domains[entry.domain] = entry.startup;
        }
        return {
            domains: domains
        }
    }
});
jscape.DomainsAutoStart.fromJSON = function(data) {
    var domains = data.domains;
    var obj = new jscape.DomainsAutoStart();
    for (var key in domains) {
        if (domains.hasOwnProperty(key)) {
            if ("@class" !== key) {
                obj.addAutoStart(key, domains[key]);
            }
        }
    }
    return obj;
};

/** @class */
jscape.settings.DomainsAutoStartController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.DomainsAutoStartController.prototype} */
{
    view: null,
    config: null,
    domains: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.DomainsAutoStartView(resources);
    },

    /** @override */
    hasChanged: function () {
        return !Object._equals(this.config, this._createConfiguration());
    },

    /** @override */
    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setDomainsAutoStart(config);
        this.listener.onChangesSubmitted(this);
    },

    _initConfiguration: function() {
        this.config = this.serverSettings.getDomainsAutoStart();
        this.domains = $.extend(true, [], this.config.entries);
    },

    _createConfiguration: function() {
        return new jscape.DomainsAutoStart(this.view.getAutoStart());
    },

    _setupView: function() {
        this.view.setAutoStart(this.domains);
        this.view.show(this);
    }
});

/** @class */
jscape.settings.DomainsAutoStartView = Class.extend(
    /** @lends {jscape.settings.DomainsAutoStartView.prototype} */
{
    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=domainstartupfields]");

        this.data = $("table[role=grid][aria-label=domains]", this.fieldset);
        this.data.datagrid({
            width: 800,
            remoteSort: false,
            singleSelect: true,
            fitColumns: true,
            onSelect: $.proxy(this._onSelect$Row, this),
            onClickRow: $.proxy(this._onClick$Row, this),
            onSortColumn: $.proxy(this._onSort$Column, this),
            columns: [[
                {field: "domain", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 80},
                {field: "startup", title: $("thead th:nth-child(2)", this.data).text(), sortable: true, width: 20, align: "center",
                    formatter: function(value) {return "<input type=\"checkbox\"{0}/>".supplant({0: value ? " checked" : ""}); },
                    editor: {type: "checkbox", options: {on: true, off: false}}}
            ]],
            editors: {
                checkbox: $.extend({}, $.fn.datagrid.defaults.editors.checkbox, {
                    getValue: function(target) {
                        var value = $(target).is(":checked") ? $(target).val() : $(target).attr("offval");
                        return (/^true$/i).test(value);
                    },
                    setValue:function(target, value) {
                        $(target)._propAttr("checked", (/^true$/i).test(value));
                    }
                })
            }
        });

        this.setAutoStartButton = $("a[role=button][name=set]", this.fieldset);
        this.setAutoStartButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$SetAutoStartButton, this)});

        this.clearAutoStartButton = $("a[role=button][name=clear]", this.fieldset);
        this.clearAutoStartButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ClearAutoStartButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.DomainsAutoStartView.LISTENER;
        this._adjustButtonsState();
    },

    getAutoStart: function() {
        this.data.datagrid("acceptChanges");
        return this.data.datagrid("getRows");
    },

    setAutoStart: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    _onSelect$Row: function() {
        this._adjustButtonsState();
    },

    _onSort$Column: function() {
        this._adjustButtonsState();
    },

    _onClick$SetAutoStartButton: function() {
        this._setAutoStart(true);
        this._endEdit();
    },

    _onClick$ClearAutoStartButton: function() {
        this._setAutoStart(false);
        this._endEdit();
    },

    _onClick$Row: function(index) {
        this._beginEdit(index);
    },

    _beginEdit: function(index) {
        this._endEdit();
        this.data.datagrid("beginEdit", index);
        this.editIndex = index;
    },

    _endEdit: function() {
        if (this.editIndex != null) {
            var index = this.editIndex;
            this.editIndex = null;
            if (this.data.datagrid("validateRow", index)) {
                this.data.datagrid("endEdit", index);
            } else {
                this.data.datagrid("cancelEdit", index);
            }
        }
    },

    _setAutoStart: function(value) {
        var rows = this.data.datagrid("getSelections");

        while (rows.length != 0) {
            var row = rows.pop();
            var index = this.data.datagrid("getRowIndex", row);
            row.startup = value;

            this.data.datagrid("refreshRow", index);
            this.data.datagrid("selectRow", index);
        }
    },

    _adjustButtonsState: function() {
        var enable = this.data.datagrid("getSelections").length != 0;
        this.setAutoStartButton.linkbutton(enable ? "enable" : "disable");
        this.clearAutoStartButton.linkbutton(enable ? "enable" : "disable");
    }
});
/** @const */
jscape.settings.DomainsAutoStartView.LISTENER = {};