jscape.settings = jscape.settings || {};

/** @class */
jscape.EmailServiceDescriptor = Class.extend(
    /** @lends {jscape.EmailServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.configuration = a || new jscape.EmailServiceConfiguration();
        this.properties = b || {};
    }
});
jscape.EmailServiceDescriptor.fromJSON = function(data) {
    return new jscape.EmailServiceDescriptor(jscape.EmailServiceConfiguration.fromJSON(data.configuration), data.properties);
};
/** @class */
jscape.EmailServiceConfiguration = Class.extend(
    /** @lends {jscape.EmailServiceConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e, f, g, h, i, j, k, l, m, o, p, q, r, s, t, u) {
        this.enabled = a || false;
        this.host = b || "";
        this.port = parseInt(c) || 0;
        this.connectionType = d || "";
        this.username = e || "";
        this.password = f || "";
        this.sender = g || "";
        this.replyTo = h || "";
        this.debugFile = i || "";
        this.encryptionKeyAlias = j || "";
        this.signingKeyAlias = k || "";
        this.enableAdHocTransfer = l || false;
        this.enablePasswordProtection = m || false;
        this.passwordProtectionType = parseInt(o) || null;
        this.linkExpiresRange = p || new jscape.LinkExpiresRange();
        this.maxDownloadsDefault = q;
        this.publicContactRecipientsAllowed = r || false;
        this.topLevelDomainRecipientsAllowed = s || false;
        this.allowedTopLevelDomains = t || [];
        this.adHocDbConfiguration = u || null;
    }
});
jscape.EmailServiceConfiguration.fromJSON = function(data) {
    return $.extend(new jscape.EmailServiceConfiguration(), data, {
        linkExpiresRange: jscape.LinkExpiresRange.fromJSON(data.linkExpiresRange),
        adHocDbConfiguration: jscape.AdHocDbConfiguration.fromJSON(data.adHocDbConfiguration)
    });
};

/** @class */
jscape.LinkExpiresRange = Class.extend(
    /** @lends {jscape.LinkExpiresRange.prototype} */
{
    /** @constructor */
    initialize: function(lowerBound, upperBound, step) {
        this.lowerBound = lowerBound;
        this.upperBound = upperBound;
        this.step = step;
    }
});
jscape.LinkExpiresRange.fromJSON = function(data) {
    return $.extend(new jscape.LinkExpiresRange(), data);
};

/** @class */
jscape.AdHocDbConfiguration = Class.extend(
    /** @lends {jscape.AdHocDbConfiguration.prototype} */
{
    /** @constructor */
    initialize: function (a, b, c, d, e) {
        this.url = a || "";
        this.username = b || "";
        this.password = c || "";
        this.poolSize = parseInt(d) || 0;
        this.timeout = parseInt(e) || 0;
    }
});
jscape.AdHocDbConfiguration.fromJSON = function(data) {
    return data ? $.extend(new jscape.AdHocDbConfiguration(), data) : null;
};

/** @class */
jscape.settings.EmailController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.EmailController.prototype} */
{
    VARIABLES: [
        "username", "URL", "date", "message", "password",
        "transferEntries", "entry.filename", "entry.filesize", "entry.downloadUrl", "entry.expirationDate", "entry.maxDownloads"
    ],

    descriptor: null,
    properties: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.EmailView(resources);
        this.miscView = new jscape.settings.EmailMiscView(resources);
        this.resourcesView = new jscape.settings.EmailResourcesView(resources);

        this.adHocController = new jscape.settings.AdHocTransferController(resources);
        this.testController = new jscape.settings.TestEmailServerController(resources);
        this.variableController = new jscape.FreemarkerVariableController();

        this._initValidators(resources);
    },

    /** @override */
    hasChanged: function () {
        return !Object._equals(this.descriptor, this._createDescriptor());
    },

    /** @override */
    submitChanges: function() {
        var descriptor = this._createDescriptor();
        this.serverSettings.setEmailServiceDescriptor(descriptor);
        this.listener.onChangesSubmitted(this);
    },

    onSelectDebugFile: function() {
        var loadFn = $.proxy(this.managementService.listDebugDir, this.managementService);
        var createFn = $.proxy(this.managementService.createDebugDir, this.managementService);
        new SelectPathController(loadFn, createFn).selectFile($.proxy(function(entry) { this.view.setDebugFile(entry.path); }, this));
    },

    onTestServer: function() {
        this.testController.start(this.managementService, this._createDescriptor());
    },

    onAddVariable: function(input) {
        this.variableController.start(input, this.VARIABLES);
    },

    _initConfiguration: function() {
        this.descriptor = this.serverSettings.getEmailServiceDescriptor();
        this.properties = $.extend(true, {}, this.descriptor.properties);
    },

    _createDescriptor: function() {
        return new jscape.EmailServiceDescriptor(this._createConfiguration(), this.resourcesView.getResources());
    },

    _createConfiguration: function() {
        var config = new jscape.EmailServiceConfiguration(this.view.getEnabled(),
                                                          this.view.getHost(),
                                                          this.view.getPort(),
                                                          this.view.getConnectionType(),
                                                          this.view.getUsername(),
                                                          this.view.getPassword(),
                                                          this.view.getFromAddress(),
                                                          this.view.getReplyToAddress(),
                                                          this.view.getDebugFile(),
                                                          this.miscView.getEncryptionKey(),
                                                          this.miscView.getSigningKey());
        this.adHocController.setupConfig(config);
        return  config;
    },

    _setupView: function() {
        var config = this.descriptor.configuration;

        this.view.setHost(config.host);
        this.view.setPort(config.port);
        this.view.setConnectionType(config.connectionType);
        this.view.setUsername(config.username);
        this.view.setPassword(config.password);
        this.view.setFromAddress(config.sender);
        this.view.setReplyToAddress(config.replyTo);
        this.view.setDebugFile(config.debugFile);
        this.view.setEnabled(config.enabled);
        this.view.show(this);

        this.miscView.setEncryptionKeys(this._getEncryptionKeys());
        this.miscView.setEncryptionKey(config.encryptionKeyAlias);
        this.miscView.setSigningKeys(this._getSigningKeys());
        this.miscView.setSigningKey(config.signingKeyAlias);
        this.miscView.show(this);

        this.adHocController.start(this.managementService, config);

        this.resourcesView.setResources(this.properties);
        this.resourcesView.show(this);
    },

    _getEncryptionKeys: function() {
        var keys = this.serverSettings.getKeys();
        return this._getKeysAliases(keys.getPgpPublicKeys().concat(keys.getPgpSecretKeys()));
    },

    _getSigningKeys: function() {
        return this._getKeysAliases(this.serverSettings.getKeys().getPgpSecretKeys());
    },

    _getKeysAliases: function(keys) {
        var aliases = [];
        for (var i = 0; i < keys.length; i++) {
            aliases.push(keys[i].alias)
        }
        return aliases;
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("smtp_host_nonempty", resources.getString("EMAIL_SERVICE_ERROR_BAD_HOST"));
    }
});

/** @class */
jscape.settings.EmailView = Class.extend(
    /** @lends {jscape.settings.EmailView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=emailsettingsfields]");

        this.enabled = $("input[name=enabled]:checkbox", this.fieldset);
        this.enabled.change($.proxy(this._adjustFieldState, this));

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, validType: "smtp_host_nonempty", missingMessage: resources.getString("EMAIL_SERVICE_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 100});

        this.connectionType = $("select[name=conntype]", this.fieldset);
        this.connectionType.combobox({required: true, editable: false, panelHeight: "auto"});

        this.username = $("input[name=username]", this.fieldset);
        this.password = $("input[name=password]", this.fieldset);

        this.debugFile = $("input[name=debugfile]", this.fieldset);
        this.browseButton = $("a[role=button][name=browsefile]", this.fieldset);
        this.browseButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$BrowseButton, this)});

        this.fromAddress = $("input[name=from]", this.fieldset);
        this.fromAddress.validatebox({required: true, validType: "email", missingMessage: resources.getString("EMAIL_SERVICE_ERROR_BAD_EMAIL")});

        this.replyToAddress = $("input[name=replyto]", this.fieldset);
        this.replyToAddress.validatebox({required: false, validType: "email", invalidMessage: resources.getString("EMAIL_SERVICE_ERROR_BAD_EMAIL")});

        this.testButton = $("a[role=button][name=test]", this.fieldset);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.EmailView.LISTENER;
        this._adjustFieldState();
    },

    getEnabled: function() {
        return this.enabled.is(":checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", !!value).change();
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", parseInt(value)).numberspinner("validate");
    },

    getConnectionType: function() {
        return this.connectionType.combobox("getValue");
    },

    setConnectionType: function(value) {
        this.connectionType.combobox("select", value);
    },

    getUsername: function() {
        return this.username.val();
    },

    setUsername: function(value) {
        this.username.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getDebugFile: function() {
        return this.debugFile.val();
    },

    setDebugFile: function(value) {
        this.debugFile.val(value);
    },

    getFromAddress: function() {
        return this.fromAddress.val();
    },

    setFromAddress: function(value) {
        this.fromAddress.val(value);
    },

    getReplyToAddress: function() {
        return this.replyToAddress.val();
    },

    setReplyToAddress: function(value) {
        this.replyToAddress.val(value);
    },

    _onClick$BrowseButton: function() {
        this.listener.onSelectDebugFile();
    },

    _onClick$TestButton: function() {
        if (this.fieldset.form("validate")) {
            this.listener.onTestServer();
        }
    },

    _adjustFieldState: function() {
        var enabled = this.getEnabled();
        this.host.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
        this.port.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        this.connectionType.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        this.username.prop("disabled", !enabled);
        this.password.prop("disabled", !enabled);
        this.debugFile.prop("disabled", !enabled);
        this.fromAddress.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
        this.replyToAddress.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
        this.browseButton.linkbutton(enabled ? "enable" : "disable");
        this.testButton.linkbutton(enabled ? "enable" : "disable");

        this.fieldset.form("validate");
    }
});
jscape.settings.EmailView.LISTENER = {
    onSelectDebugFile: $.noop,
    onTestServer: $.noop
};

/** @class */
jscape.settings.EmailMiscView = Class.extend(
    /** @lends {jscape.settings.EmailMiscView.prototype} */
{
    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.encrypt = $("#ssesmisc-encrypt");
        this.encrypt.change($.proxy(this._adjustEncryptKeyFieldState, this));

        this.encryptKey = $("#ssesmisc-encryptkey");
        this.encryptKey.combobox({
            editable: false,
            disabled: true,
            panelHeight: "auto",
            formatter: function(rowData) { var t = rowData[$(this).combobox("options").textField]; return t.escapeXml(); }
        }).combobox("autocomplete", true);

        this.sign = $("#ssesmisc-sign");
        this.sign.change($.proxy(this._adjustSignKeyFieldState, this));
        this.signKey = $("#ssesmisc-signkey");
        this.signKey.combobox({
            editable: false,
            disabled: true,
            panelHeight: "auto",
            formatter: function(rowData) { var t = rowData[$(this).combobox("options").textField]; return t.escapeXml(); }
        }).combobox("autocomplete", true);
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.EmailMiscView.LISTENER;
        this._adjustEncryptKeyFieldState();
        this._adjustSignKeyFieldState();
    },

    getEncryptionKey: function() {
        return this.encrypt.prop("checked") ? this.encryptKey.combobox("getValue") : "";
    },

    setEncryptionKey: function(value) {
        if (value) {
            this.encrypt.prop("checked", true);
            this.encryptKey.combobox("setValue" , value);
            this.encryptKey.combobox("select", value);
        } else {
            this.encrypt.prop("checked", false);
        }
    },

    setEncryptionKeys: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.encryptKey.combobox("loadData", data);
    },

    getSigningKey: function() {
        return this.sign.prop("checked") ? this.signKey.combobox("getValue") : "";
    },

    setSigningKey: function(value) {
        this.sign.prop("checked", !!value).change();
        if (value) {
            this.signKey.combobox("clear").combobox("select", value);
        }
    },

    setSigningKeys: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.signKey.combobox("loadData", data);
    },

    _adjustEncryptKeyFieldState: function() {
        this.encryptKey.combobox(this.encrypt.prop("checked") ? "enable" : "disable");
    },

    _adjustSignKeyFieldState: function() {
        this.signKey.combobox(this.sign.prop("checked") ? "enable" : "disable");
    }
});

/** @class */
jscape.settings.AdHocTopLevelDomainDialog = Class.extend(
    /** @lends {jscape.settings.AdHocTopLevelDomainDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#sseft-dialog-tld").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=adhoctldfields]", this.dialog);
        this.domain = $("input[name=domain]", this.fieldset);
        this.domain.validatebox({required: true, validType: ["adhoc_tld_nonempty", "adhoc_tld_unique"], missingMessage: resources.getString("ADHOC_TRANSFER_ERROR_BAD_DOMAIN")});
    },

    getDomain: function() {
        return this.domain.val();
    },

    setDomain: function(value) {
        this.domain.val(value || "");
        if (value && value.trim().length) {
            this.domain.validatebox("options").validType = ["adhoc_tld_nonempty", "adhoc_tld_unique['" + value + "']"];
        }
    },

    setTitle: function(value) {
        this.dialog.dialog("setTitle", value);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.domain.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.settings.AdHocTransferController = Class.extend(
    /** @lends {jscape.settings.AdHocTransferController.prototype} */
{
    domains: [],

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.view = new jscape.settings.AdHocTransferView(resources);
        this.addController = new jscape.settings.AddAdHocTLDController(resources);
        this.editController = new jscape.settings.EditAdHocTLDController(resources);
        this.dbConfigController = new jscape.settings.AdHocDbConfigurationController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, config) {
        this.managementService = managementService;
        this.config = config || {};
        this.domains = this.config.allowedTopLevelDomains || [];
        this.adHocDbConfig = this.config.adHocDbConfiguration || null;

        this._setupView();
    },

    setupConfig: function(config) {
        config.enableAdHocTransfer = this.view.getEnabled();
        config.enablePasswordProtection = this.view.getProtectionEnabled();
        config.passwordProtectionType = this.view.getProtectionType();
        config.linkExpiresRange = new jscape.LinkExpiresRange(this.view.getLinkExpiresRangeBegin(), this.view.getLinkExpiresRangeEnd(), this.view.getLinkExpiresRangeStep());
        config.maxDownloadsDefault = this.view.getMaxDownloadsDefault();
        config.publicContactRecipientsAllowed = this.view.getRecipientsPublicContactAllowed();
        config.topLevelDomainRecipientsAllowed = this.view.getRecipientsTopLevelDomainAllowed();
        config.allowedTopLevelDomains = this.view.getTopLevelDomains();
        config.adHocDbConfiguration = this.adHocDbConfig;
    },

    onChangeDatastore: function() {
        this.dbConfigController.start(this.managementService, this.adHocDbConfig)
                .done($.proxy(function(config) { this.adHocDbConfig = config; }, this));
    },

    onAddDomain: function() {
        this.addController.start().done($.proxy(this._tldAdded, this));
    },

    onEditDomain: function(domain) {
        var index = $.inArray(domain, this.domains);
        if (index != -1) {
            this.editController.start(domain).done($.proxy(this._tldEdited, this, index));
        }
    },

    onDeleteDomain: function(domain) {
        this.view.showConfirmDelete(domain)
                .done($.proxy(this._tldDeleted, this, domain));
    },

    _tldAdded: function(domain) {
        this.domains.push(domain);
        this.view.setTopLevelDomains(this.domains);
        this.view.selectItem(domain);
    },

    _tldEdited: function(index, domain) {
        this.domains[index] = domain;
        this.view.setTopLevelDomains(this.domains);
        this.view.selectItem(domain);
    },

    _tldDeleted: function (domain) {
        var index = $.inArray(domain, this.domains);
        if (index != -1) {
            this.domains.splice(index, 1);
            this.view.setTopLevelDomains(this.domains);

            if (this.domains.length) {
                index = Math.max(0, Math.min(index, this.domains.length - 1));
                this.view.selectItem(this.domains[index]);
            }
        }
    },

    _setupView: function() {
        this.view.setEnabled(this.config.enableAdHocTransfer);
        this.view.setProtectionEnabled(this.config.enablePasswordProtection);
        this.view.setProtectionType(this.config.passwordProtectionType);
        this.view.setLinkExpiresRangeBegin(this.config.linkExpiresRange.lowerBound);
        this.view.setLinkExpiresRangeEnd(this.config.linkExpiresRange.upperBound);
        this.view.setLinkExpiresRangeStep(this.config.linkExpiresRange.step);
        this.view.setMaxDownloadsDefault(this.config.maxDownloadsDefault);
        this.view.setRecipientsPublicContactAllowed(this.config.publicContactRecipientsAllowed);
        this.view.setRecipientsTopLevelDomainAllowed(this.config.topLevelDomainRecipientsAllowed);
        this.view.setTopLevelDomains(this.domains);
        this.view.show(this);
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("adhoc_tld_nonempty", resources.getString("ADHOC_TRANSFER_ERROR_BAD_DOMAIN"));
        v.rule("adhoc_tld_unique", $.proxy(this._tldNotExists, this), resources.getString("ADHOC_TRANSFER_ERROR_DOMAIN_EXISTS"));
    },

    _tldNotExists: function(value, excludes) {
        return $.inArray(value, this.domains) == -1 || (excludes && $.inArray(value, excludes) != -1);
    }
});

/** @class */
jscape.settings.AddAdHocTLDController = Class.extend(
    /** @lends {jscape.settings.AddAdHocTLDController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function() {
        this.deferred = $.Deferred();

        var dialog = new jscape.settings.AdHocTopLevelDomainDialog(this.resources);
        dialog.setTitle(this.resources.getString("ADHOC_TRANSFER_ADD_TITLE"));
        dialog.setDomain(null);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var domain = dialog.getDomain();
        dialog.destroy();
        this.deferred.resolve(domain);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    }
});

/** @class */
jscape.settings.EditAdHocTLDController = Class.extend(
    /** @lends {jscape.settings.EditAdHocTLDController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(domain) {
        this.deferred = $.Deferred();

        var dialog = new jscape.settings.AdHocTopLevelDomainDialog(this.resources);
        dialog.setTitle(this.resources.getString("ADHOC_TRANSFER_EDIT_TITLE"));
        dialog.setDomain(domain);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var domain = dialog.getDomain();
        dialog.destroy();
        this.deferred.resolve(domain);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    }
});

/** @class */
jscape.settings.AdHocTransferView = Class.extend(
    /** @lends {jscape.settings.AdHocTransferView.prototype} */
{
    /** @const */
    DEFAULT_EXPIRES_BEGIN: 5,
    DEFAULT_EXPIRES_END: 90,
    DEFAULT_EXPIRES_STEP: 5,
    DEFAULT_MAX_DOWNLOADS: 5,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=adhocservicefields]");
        this.enable = $("input[name=enable]:checkbox", this.fieldset);
        this.enable.click($.proxy(this._adjustFieldState, this));

        this.linkExpiresBegin = $("input[name=exprangebegin]", this.fieldset);
        this.linkExpiresBegin.numberspinner({min: 1, max: 90, width: 50, onChange: $.proxy(this._adjustLinkExpiresRange, this)});
        this.linkExpiresEnd = $("input[name=exprangeend]", this.fieldset);
        this.linkExpiresEnd.numberspinner({min: 1, max: 90, width: 50, onChange: $.proxy(this._adjustLinkExpiresRange, this)});
        this.linkExpiresStep = $("input[name=exprangestep]", this.fieldset);
        this.linkExpiresStep.numberspinner({min: 1, max: 90, width: 50, onChange: $.proxy(this._adjustLinkExpiresRange, this)});

        this.maxDownloads = $("input[name=maxdownloads]", this.fieldset);
        this.maxDownloads.numberspinner({disabled: true, min: 1, max: 10000, width: 70});

        this.protectEnabled = $("input[name=pwdprotectenabled]:checkbox", this.fieldset);
        this.protectEnabled.change($.proxy(function() {
            var enabled = this.protectEnabled.is(":checked") && !this.protectEnabled.is(":disabled");
            this.protectType.combobox(enabled ? "enable" : "disable");
        }, this));
        this.protectType = $("select[name=pwdprotect]", this.fieldset);
        this.protectType.combobox({editable: false, panelHeight: "auto"});

        this.recipientsPublicContactAllowed = $("input[name=recipientpubcontactallowed]:checkbox", this.fieldset);
        this.recipientsTopLevelDomainAllowed = $("input[name=recipienttldallowed]:checkbox", this.fieldset);
        this.tld = $("table[role=grid][aria-label=topleveldomains]", this.fieldset);
        this.tld.datagrid({
            height: 100,
            singleSelect: true,
            remoteSort: false,
            fitColumns: true,
            onSelect: $.proxy(this._adjustButtonsState, this),
            onSortColumn: $.proxy(this._adjustButtonsState, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this),
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.tld).text(), resizable: true, sortable: true, width: 100, formatter: function(_, row) { return (row || "").escapeXml(); }}
            ]]
        });

        this.datastoreButton = $("a[role=button][name=datastore]", this.fieldset);
        this.datastoreButton.linkbutton({onClick: $.proxy(this._onClick$DatastoreButton, this)});

        this.addButton = $("a[role=button][name=addtld]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edittld]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=deletetld]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.AdHocTransferView.LISTENER;
        this._adjustLinkExpiresRange();
        this._adjustFieldState();
        this._adjustButtonsState();
    },

    showConfirmDelete: function(domain) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("ADHOC_TRANSFER_DELETE_TITLE"),
                                            this.resources.getString("ADHOC_TRANSFER_DELETE_MESSAGE", {0: domain}));
    },

    getEnabled: function() {
        return this.enable.is(":checked");
    },

    setEnabled: function(value) {
        this.enable.prop("checked", !!value);
    },

    getLinkExpiresRangeBegin: function() {
        return parseInt(this.linkExpiresBegin.numberspinner("getValue"));
    },

    setLinkExpiresRangeBegin: function(value) {
        this.linkExpiresBegin.numberspinner("setValue", value || this.DEFAULT_EXPIRES_BEGIN);
    },

    getLinkExpiresRangeEnd: function() {
        return parseInt(this.linkExpiresEnd.numberspinner("getValue"));
    },

    setLinkExpiresRangeEnd: function(value) {
        this.linkExpiresEnd.numberspinner("setValue", value || this.DEFAULT_EXPIRES_END);
    },

    getLinkExpiresRangeStep: function() {
        return parseInt(this.linkExpiresStep.numberspinner("getValue"));
    },

    setLinkExpiresRangeStep: function(value) {
        this.linkExpiresStep.numberspinner("setValue", value || this.DEFAULT_EXPIRES_STEP);
    },

    getMaxDownloadsDefault: function() {
        return parseInt(this.maxDownloads.numberspinner("getValue"));
    },

    setMaxDownloadsDefault: function(value) {
        this.maxDownloads.numberspinner("setValue", value || this.DEFAULT_MAX_DOWNLOADS);
    },

    getProtectionEnabled: function() {
        return this.protectEnabled.is(":checked");
    },

    setProtectionEnabled: function(value) {
        this.protectEnabled.prop("checked", !!value).change();
    },

    getProtectionType: function() {
        return parseInt(this.protectType.combobox("getValue"), 10);
    },

    setProtectionType: function(value) {
        this.protectType.combobox("clear");
        this.protectType.combobox("setValue", value.toString()).combobox("select", value.toString());
    },

    getRecipientsPublicContactAllowed: function() {
        return this.recipientsPublicContactAllowed.is(":checked");
    },

    setRecipientsPublicContactAllowed: function(value) {
        this.recipientsPublicContactAllowed.prop("checked", !!value);
    },

    getRecipientsTopLevelDomainAllowed: function() {
        return this.recipientsTopLevelDomainAllowed.is(":checked");
    },

    setRecipientsTopLevelDomainAllowed: function(value) {
        this.recipientsTopLevelDomainAllowed.prop("checked", !!value);
    },

    getTopLevelDomains: function() {
        return this.tld.datagrid("getRows");
    },

    setTopLevelDomains: function(values) {
        this.tld.datagrid("loading");
        this.tld.datagrid("loadData", values);
        this.tld.datagrid("loaded");
    },

    selectItem: function (value) {
        this.tld.datagrid("selectRow", this.tld.datagrid("getRowIndex", value));
    },

    _onClick$DatastoreButton: function() {
        this.listener.onChangeDatastore(this);
    },

    _onClick$AddButton: function() {
        this.listener.onAddDomain();
    },

    _onClick$EditButton: function() {
        this.listener.onEditDomain(this.tld.datagrid("getSelected"));
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteDomain(this.tld.datagrid("getSelected"));
    },

    _adjustFieldState: function() {
        var enabled = this.getEnabled();
        this.linkExpiresBegin.numberspinner(enabled ? "enable" : "disable");
        this.linkExpiresEnd.numberspinner(enabled ? "enable" : "disable");
        this.linkExpiresStep.numberspinner(enabled ? "enable" : "disable");
        this.maxDownloads.numberspinner(enabled ? "enable" : "disable");
        this.protectEnabled.prop("disabled", !enabled).change();
        this.datastoreButton.linkbutton(enabled ? "enable" : "disable");
    },

    _adjustLinkExpiresRange: function() {
        var begin = this.getLinkExpiresRangeBegin();
        var end = this.getLinkExpiresRangeEnd();

        this.linkExpiresBegin.numberspinner("options").max = end;

        if (begin > end) {
            this.setLinkExpiresRangeBegin(end);
            this.linkExpiresStep.numberspinner("options").max = 1;
        } else {
            var stepMax = end - begin;
            var step = this.getLinkExpiresRangeStep();
            this.linkExpiresStep.numberspinner("options").max = stepMax;
            if (step > stepMax) {
                this.setLinkExpiresRangeStep(stepMax);
            }
        }
    },

    _adjustButtonsState: function() {
        var enabled = this.tld.datagrid("getSelected") != null;
        this._setButtonEnabled(this.editButton, enabled);
        this._setButtonEnabled(this.deleteButton, enabled);
    },

    _buttonEnabled: function(button) {
        return !button.linkbutton("options").disabled;
    },

    _setButtonEnabled: function(button, enable) {
        button.linkbutton(enable ? "enable" : "disable");
    }
});
/** @const */
jscape.settings.AdHocTransferView.LISTENER = {
    onAddDomain: $.noop,
    onEditDomain: $.noop,
    onDeleteDomain: $.noop
};

/** @class */
jscape.settings.AdHocDbConfigurationController = Class.extend(
    /** @lends {jscape.settings.AdHocDbConfigurationController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, config) {
        this.managementService = managementService;
        this.config = config;
        this.deferred = $.Deferred();

        this._setupDialog();

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var config = this._createConfiguration(dialog);
        dialog.destroy();
        this.deferred.resolve(config);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    onTestParameters: function(dialog) {
        var config = this._createConfiguration(dialog);
        this.managementService.testAdHocStore(config)
                .done($.proxy(this._testPassed, this));
    },

    _testPassed: function () {
        $.messager.alert(this.resources.getString("ADHOC_TRANSFER_TEST_TITLE"),
                         this.resources.getString("ADHOC_TRANSFER_TEST_PASSED_MESSAGE"),
                         "info");
    },

    _createConfiguration: function(dialog) {
        return dialog.getUseCustomConfiguration()
                ? new jscape.AdHocDbConfiguration(dialog.getUrl(), dialog.getUsername(), dialog.getPassword(), dialog.getPoolSize(), dialog.getPoolTimeout())
                : null;
    },

    _setupDialog: function () {
        var dialog = new jscape.settings.AdHocDbConfigurationDialog(this.resources);
        dialog.show(this);
        if (this.config) {
            dialog.setUrl(this.config.url);
            dialog.setUsername(this.config.username);
            dialog.setPassword(this.config.password);
            dialog.setPoolSize(this.config.poolSize);
            dialog.setPoolTimeout(this.config.timeout);
            dialog.setUseCustomConfiguration(true);
        } else {
            dialog.setUseDefaultConfiguration(true);
        }

    }
});

/** @class */
jscape.settings.AdHocDbConfigurationDialog = Class.extend(
    /** @lends {jscape.settings.AdHocDbConfigurationDialog.prototype} */
{
    /** @const */
    TIMEOUT_FACTOR: 60 * 1000,

    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#sseft-dialog-db").clone().attr("id", "");
        this.dialog.dialog({
            width: 630, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=adhocdbfields]", this.dialog);

        this.type = $("input[name=type]:radio", this.fieldset);
        this.type.change($.proxy(this._adjustFieldState, this));
        this.typeDefault = this.type.filter("[value=default]");
        this.typeCustom = this.type.filter("[value=custom]");

        this.url = $("input[name=url]", this.fieldset);
        this.url.validatebox({required: true, missingMessage: resources.getString("ADHOC_TRANSFER_ERROR_BAD_DB_URL")});

        this.username = $("input[name=username]", this.fieldset);
        this.username.validatebox({required: true, missingMessage: resources.getString("ADHOC_TRANSFER_ERROR_BAD_USERNAME")});

        this.password = $("input[name=password]", this.fieldset);

        this.poolSize = $("input[name=poolsize]", this.fieldset);
        this.poolSize.numberspinner({required: true, min: 1, max: 100, value: 5, width: 50});

        this.poolTimeout = $("input[name=poolttl]", this.fieldset);
        this.poolTimeout.numberspinner({required: true, min: 1, max: 100, value: 5, width: 50});

        this.testButton = $("a[role=button][name=test]", this.fieldset);
        this.testButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getUseDefaultConfiguration: function() {
        return this.typeDefault.is(":checked");
    },

    setUseDefaultConfiguration: function(value) {
        this.typeDefault.prop("checked", !!value);
        this.type.change();
    },

    getUseCustomConfiguration: function() {
        return this.typeCustom.is(":checked");
    },

    setUseCustomConfiguration: function(value) {
        this.typeCustom.prop("checked", !!value);
        this.type.change();
    },

    getUrl: function() {
        return this.url.val();
    },

    setUrl: function(value) {
        this.url.val(value);
    },

    getUsername: function() {
        return this.username.val();
    },

    setUsername: function(value) {
        this.username.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getPoolSize: function() {
        return parseInt(this.poolSize.numberspinner("getValue"));
    },

    setPoolSize: function(value) {
        this.poolSize.val(parseInt(value));
    },

    getPoolTimeout: function() {
        return parseInt(this.poolTimeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setPoolTimeout: function(value) {
        this.poolTimeout.numberspinner(parseInt(value) / this.TIMEOUT_FACTOR);
    },

    show: function (listener) {
        this.listener = listener || jscape.settings.AdHocDbConfigurationDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.fieldset.find("input:first").focus();
        this._adjustFieldState();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _adjustFieldState: function() {
        var enabled = this.getUseCustomConfiguration();
        this.url.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
        this.username.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
        this.password.prop("disabled", !enabled);
        this.poolSize.numberspinner(enabled ? "enable" : "disable").validatebox(enabled ? "enableValidation" : "disableValidation");
        this.poolTimeout.numberspinner(enabled ? "enable" : "disable").validatebox(enabled ? "enableValidation" : "disableValidation");
        this.testButton.linkbutton(enabled ? "enable" : "disable");
    },

    _onClick$TestButton: function() {
        if (this.fieldset.form("validate")) {
            this.listener.onTestParameters(this);
        }
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.settings.AdHocDbConfigurationDialog.LISTENER = {
    onTestParameters: $.noop
};


/** @class */
jscape.settings.EmailResourcesController = Class.extend(
    /** @lends {jscape.settings.EmailResourcesController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.view = new jscape.settings.EmailResourcesView(resources);
    },

    start: function(config) {
        this.config = config;
        this._setupView();
    },

    setupConfig: function(config) {

    },

    _setupView: function() {
        this.view.setResources();
    }
});

/** @class */
jscape.settings.EmailResourcesView = Class.extend(
    /** @lends {jscape.settings.EmailResourcesView.prototype} */
{
    rowIndex: -1,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.table = $("#sseres-data");
        this.table.propertygrid({
            showGroup: false,
            scrollbarSize: 0,
            sortName: "name",
            sortOrder: "asc",
            onClickRow: $.proxy(this._adjustButtonsState, this),
            onBeforeEdit: $.proxy(function(index) { this.rowIndex = index; }, this),
            onCancelEdit: $.proxy(function() { this.rowIndex = -1; }, this)
        });

        this.addVariableButton = $("#sseres-addvar");
        this.addVariableButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$AddVariableButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.EmailResourcesView.LISTENER;
        this._adjustButtonsState();
    },

    getResources: function() {
        var resources = {};
        var rows = this.table.propertygrid("getRows");
        for (var i = 0; i < rows.length; i++) {
            var row = rows[i];
            resources[row.name] = row.value;
        }
        return resources;
    },

    setResources: function(resources) {
        var data = [];
        for (var key in resources) {
            if (resources.hasOwnProperty(key)) {
                data.push({name: key, value: resources[key], editor: "text"});
            }
        }
        this.table.propertygrid("loadData", data);
    },

    _onClick$AddVariableButton: function() {
        var index = this.rowIndex;

        if (index != -1) {
            this.table.propertygrid("selectRow", index);
            this.table.propertygrid("beginEdit", index);
            this.table.propertygrid("options").editIndex = index;
            var editor = this.table.propertygrid("getEditor", {index: index, field: "value"});
            if (editor != null) {
                var input = $(editor.target);
                this.listener.onAddVariable(input);
            }
        }
    },

    _adjustButtonsState: function() {
        var enabled = this.table.propertygrid("getSelected") != null;
        this.addVariableButton.linkbutton(enabled ? "enable" : "disable");
    }
});
jscape.settings.EmailResourcesView.LISTENER = {
    onAddVariable: $.noop
};

/** @class */
jscape.settings.TestEmailServerController = Class.extend(
    /** @lends {jscape.settings.TestEmailServerController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, descriptor) {
        this.managementService = managementService;
        this.descriptor = descriptor;
        this.deferred = $.Deferred();

        this._setupDialog().show(this);

        return this.deferred.promise();
    },

    onSubmit: function (dialog) {
        var recipient = dialog.getRecipient();
        dialog.destroy();

        this.managementService.testEmailServer(this.descriptor, recipient)
                .done($.proxy(function () {
                    $.messager.alert(this.resources.getString("EMAIL_SERVICE_TEST_SUCCESS_TITLE"),
                                     this.resources.getString("EMAIL_SERVICE_TEST_SUCCESS_MESSAGE"),
                                     "info");
                    this.deferred.resolve();
                }, this))
                .fail($.proxy(function () { this.deferred.reject(); }, this));
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _setupDialog: function() {
        return new jscape.settings.TestEmailServerDialog(this.resources);
    }
});

/** @class */
jscape.settings.TestEmailServerDialog = Class.extend(
    /** @lends {jscape.settings.TestEmailServerDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#sses-dialog-test").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=emailsettingstestfields]", this.dialog);

        this.recipient = $("input[name=recipient]", this.fieldset);
        this.recipient.validatebox({
            required: true,
            validType: "email",
            missingMessage: resources.getString("EMAIL_SERVICE_ERROR_BAD_RECIPIENT"),
            invalidMessage: resources.getString("EMAIL_SERVICE_ERROR_BAD_RECIPIENT")
        });
    },

    getRecipient: function() {
        return this.recipient.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.recipient.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});