jscape.settings = jscape.settings || {};

/** @class */
jscape.FailoverConfiguration = Class.extend(
    /** @lends {jscape.FailoverConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.managerConfiguration = a;
        this.ipSubstitutor = b;
        this.enabled = !!c || false;
        this.forceStart = !!d || false;
    },

    clone: function() {
        return $.extend(true, new jscape.FailoverConfiguration(), this);
    }
});
jscape.FailoverConfiguration.fromJSON = function(data) {
    return new jscape.FailoverConfiguration(jscape.ManagerConfiguration.fromJSON(data.managerConfiguration),
                                            jscape.IpSubstitutor.fromJSON(data.ipSubstitutor),
                                            data.enabled,
                                            data.forceStart);
};

/** @class */
jscape.ManagerConfiguration = Class.extend(
    /** @lends {jscape.ManagerConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e) {
        this.host = a || "";
        this.port = parseInt(b) || 0;
        this.timeout = parseInt(c) || 0;
        this.username = d || "";
        this.password = e || "";
    }
});
jscape.ManagerConfiguration.fromJSON = function(data) {
    return data ? new jscape.ManagerConfiguration(data.host, data.port, data.timeout, data.username, data.password) : null;
};

/** @class */
jscape.IpSubstitutor = Class.extend(
    /** @lends {jscape.IpSubstitutor.prototype} */
{
    /** @constructor */
    initialize: function(rules) {
        this.rules = rules || [];
    }
});
jscape.IpSubstitutor.fromJSON = function(data) {
    return new jscape.IpSubstitutor($.map(data.rules || [], jscape.IpSubstitutor.Rule.fromJSON));
};

/** @class */
jscape.IpSubstitutor.Rule = Class.extend(
    /** @lends {jscape.IpSubstitutor.Rule.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.ip = a || "";
        this.substitution = b || "";
    }
});
jscape.IpSubstitutor.Rule.fromJSON = function(data) {
    return new jscape.IpSubstitutor.Rule(data.ip, data.substitution);
};

/** @class */
jscape.settings.FailoverController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.FailoverController.prototype} */
{
    SYNC_STATE_PERIOD: 3 * 1000,

    config: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.FailoverView(resources);
        this.substitutorController = new jscape.settings.IpSubstitutorController(resources);
    },

    /** @override */
    start: function(listener, managementService, serverSettings) {
        this._super(listener, managementService, serverSettings);

        var copy = $.extend(true, {}, this.config.ipSubstitutor);
        this.substitutorController.start(copy);
    },

    /** @override */
    hasChanged: function () {
        return !Object._equals(this.config, this._createConfiguration());
    },

    /** @override */
    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setFailoverServiceConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    onSyncServerState: function() {
        $(document).triggerHandler("failoversync", false);
        var config = this._createSyncConfiguration();
        this.managementService.syncFailoverServerState(config)
                .done($.proxy(this._loadFailoverSyncState, this))
    },

    _loadFailoverSyncState: function() {
        setTimeout($.proxy(function() {
            this.managementService.failoverSyncComplete()
                    .done($.proxy(function (complete) {
                        if (complete) {
                            $(document).triggerHandler("failoversync", true);
                        } else {
                            this._loadFailoverSyncState();
                        }
                    }, this))
                    .fail(function() { $(document).triggerHandler("failoversync", true); });
        }, this), this.SYNC_STATE_PERIOD);
    },

    _initConfiguration: function() {
        this.config = this.serverSettings.getFailoverServiceConfiguration();
    },

    _createConfiguration: function() {
        return new jscape.FailoverConfiguration(new jscape.ManagerConfiguration(this.view.getHost(),
                                                                                this.view.getPort(),
                                                                                this.view.getTimeout(),
                                                                                this.view.getUsername(),
                                                                                this.view.getPassword()),
                                                this.substitutorController.createSubstitutor(),
                                                this.view.getEnabled(),
                                                this.view.getForceStart());
    },

    _createSyncConfiguration: function() {
        return {
            host: this.view.getHost(),
            port: this.view.getPort(),
            timeoutMillis: this.view.getTimeout(),
            username: this.view.getUsername(),
            password: this.view.getPassword(),
            substitutor: this.substitutorController.createSubstitutor(),
            forceStart: this.view.getForceStart()
        };
    },

    _setupView: function() {
        var config = this.config.managerConfiguration;
        this.view.setHost(config.host);
        this.view.setPort(config.port);
        this.view.setTimeout(config.timeout);
        this.view.setUsername(config.username);
        this.view.setPassword(config.password);
        this.view.setEnabled(this.config.enabled);
        this.view.setForceStart(this.config.forceStart);
        this.view.show(this);
    }
});

/** @class */
jscape.settings.FailoverView = Class.extend(
    /** @lends {jscape.settings.FailoverView.prototype} */
{
    /** @const */
    DEFAULT_HOST: "localhost",
    DEFAULT_PORT: 10880,
    DEFAULT_TIMEOUT: 10000,
    TIMEOUT_FACTOR: 1000,

    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=failoversettingsfields]");

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, missingMessage: resources.getString("FAILOVER_ERROR_BAD_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 1, max: 65535, width: 100});

        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({min: 1, max: 999, width: 60});

        this.username = $("input[name=username]", this.fieldset);
        this.username.validatebox({required: true, missingMessage: resources.getString("FAILOVER_ERROR_BAD_USERNAME")});

        this.password = $("input[name=password]", this.fieldset);
        this.password.validatebox({required: true, missingMessage: resources.getString("FAILOVER_ERROR_BAD_PASSWORD")});

        this.enabled = $("input[name=enabled]:checkbox", this.fieldset);
        this.enabled.change($.proxy(this._adjustFieldState, this));
        this.forceStart = $("input[name=forcestart]:checkbox", this.fieldset);

        this.syncButton = $("a[role=button][name=synchronize]", this.fieldset);
        this.syncButton.linkbutton({onClick: $.proxy(this._onClick$SyncButton, this)});

        $(document).on("failoversync", $.proxy(function(e, complete) {
            this.syncButton.linkbutton(!!complete ? "enable" : "disable");
        }, this));
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.FailoverView.LISTENER;
        this._adjustFieldState();
    },

    getEnabled: function () {
        return this.enabled.is(":checked");
    },

    setEnabled: function (value) {
        this.enabled.prop("checked", !!value).change();
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value ? value : this.DEFAULT_HOST);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value ? value : this.DEFAULT_PORT);
    },

    getTimeout: function() {
        return parseInt(this.timeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", value ? value / this.TIMEOUT_FACTOR : this.DEFAULT_TIMEOUT);
    },

    getUsername: function() {
        return this.username.val();
    },

    setUsername: function(value) {
        this.username.val(value);
    },

    getPassword: function() {
        return this.password.val();
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getForceStart: function() {
        return this.forceStart.is(":checked");
    },

    setForceStart: function(value) {
        this.forceStart.prop("checked", !!value);
    },

    _onClick$SyncButton: function() {
        if (this.fieldset.form("validate")) {
            this.listener.onSyncServerState();
        }
    },

    _adjustFieldState: function () {
        var enabled = this.getEnabled();
        this.host.validatebox(enabled ? "enableValidation" : "disableValidation");
        this.port.numberspinner(enabled ? "enableValidation" : "disableValidation");
        this.username.validatebox(enabled ? "enableValidation" : "disableValidation");
        this.password.validatebox(enabled ? "enableValidation" : "disableValidation");

        this.fieldset.form("validate");
    }
});
jscape.settings.FailoverView.LISTENER = {
    onSyncServerState: $.noop
};

/** @class */
jscape.settings.IpSubstitutorController = Class.extend(
    /** @lends {jscape.settings.IpSubstitutorController.prototype} */
{
    rules: null,

    /** @constructor */
    initialize: function(resources) {
        this.view = new jscape.settings.IpSubstitutorView(resources);
        this.addController = new jscape.settings.AddIpSubstitutionController(resources);
        this.editController = new jscape.settings.EditIpSubstitutionController(resources);
        this.deleteController = new jscape.settings.DeleteIpSubstitutionController(resources);

        this._initValidators(resources);
    },

    start: function(substitutor) {
        this.rules = substitutor ? substitutor.rules : [];
        this.view.setRules(this.rules);
        this.view.show(this);
    },

    createSubstitutor: function() {
        return new jscape.IpSubstitutor(this.rules);
    },

    onAddRule: function() {
        this.addController.start()
                .done($.proxy(this._ruleAdded, this));
    },

    onEditRule: function(rule) {
        var index = $.inArray(rule, this.rules);
        if (index != -1) {
            this.editController.start(rule)
                    .done($.proxy(this._ruleEdited, this, index));
        }
    },

    onDeleteRule: function(rules) {
        this.deleteController.start(rules)
                .done($.proxy(this._ruleDeleted, this));
    },

    _ruleAdded: function (rule) {
        this.rules.push(rule);
        this.view.setRules(this.rules);
        this.view.selectItem(rule);
    },

    _ruleEdited: function (index, rule) {
        this.rules[index] = rule;
        this.view.setRules(this.rules);
        this.view.selectItem(rule);
    },

    _ruleDeleted: function (rules) {
        var index = -1;
        while (rules.length) {
            index = $.inArray(rules.pop(), this.rules);
            this.rules.splice(index, 1);
        }
        this.view.setRules(this.rules);

        if (this.rules.length) {
            index = Math.max(0, Math.min(index, this.rules.length - 1));
            this.view.selectItem(this.rules[index]);
        }
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("ip_nonempty", resources.getString("IP_SUBSTITUTION_ERROR_BAD_IP"));
        v.nonEmptyRule("substitution_nonempty", resources.getString("IP_SUBSTITUTION_ERROR_BAD_SUBSTITUTION"));
    }
});

/** @class */
jscape.settings.IpSubstitutorView = Class.extend(
    /** @lends {jscape.settings.IpSubstitutorView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.selection = [];
        this.fieldset = $("fieldset[name=failoveripsubstitutionfields]");

        this.data = $("table[role=grid][aria-label=ipsubstitution]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            remoteSort: false,
            ctrlSelect: true,
            onSelect: $.proxy(this._adjustRowSelection, this),
            onSortColumn: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$EditButton, this),
            columns: [[
                {field: "ip", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 100},
                {field: "substitution", title: $("thead th:nth-child(2)", this.data).text(), sortable: true, resizable: true, width: 100}
            ]]
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.IpSubstitutorView.LISTENER;
        this._adjustButtonsState();
    },

    setRules: function(values) {
        this.selection = [];
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectItem: function(value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value));
    },

    _onClick$AddButton: function() {
        this.listener.onAddRule();
    },

    _onClick$EditButton: function() {
        this.listener.onEditRule(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteRule(this.selection);
    },

    _adjustRowSelection: function () {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        this.editButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 ? "enable" : "disable");
    }
});
jscape.settings.IpSubstitutorView.LISTENER = {
    onAddRule: $.noop,
    onEditRule: $.noop,
    onDeleteRule: $.noop
};

/** @class */
jscape.settings.AddIpSubstitutionController = Class.extend(
    /** @lends {jscape.settings.AddIpSubstitutionController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function() {
        this.deferred = $.Deferred();
        this._setupDialog().show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var rule = this._createRule(dialog);
        dialog.destroy();
        this.deferred.resolve(rule);
        this.deferred = null;
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _createRule: function(dialog) {
        return new jscape.IpSubstitutor.Rule(dialog.getIp(), dialog.getSubstitution());
    },

    _setupDialog: function() {
        return new jscape.settings.AddIpSubstitutionDialog(this.resources);
    }
});

/** @class */
jscape.settings.EditIpSubstitutionController = Class.extend(
    /** @lends {jscape.settings.EditIpSubstitutionController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(rule) {
        this.deferred = $.Deferred();
        this._setupDialog(rule).show(this);
        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var rule = this._createRule(dialog);
        dialog.destroy();
        this.deferred.resolve(rule);
        this.deferred = null;
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
        this.deferred = null;
    },

    _createRule: function(dialog) {
        return new jscape.IpSubstitutor.Rule(dialog.getIp(), dialog.getSubstitution());
    },

    _setupDialog: function(rule) {
        var dialog = new jscape.settings.EditIpSubstitutionDialog(this.resources);
        dialog.setIp(rule.ip);
        dialog.setSubstitution(rule.substitution);
        return dialog;
    }
});

/** @class */
jscape.settings.IpSubstitutionDialog = Class.extend(
    /** @lends {jscape.settings.IpSubstitutionDialog.prototype} */
{
    /** @constructor */
    initialize: function(dialog, resources) {
        this.dialog = dialog.clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=failoveripsubstitutionrulefields]", this.dialog);

        this.ip = $("input[name=ip]", this.fieldset);
        this.ip.validatebox({required: true, validType: "ip_nonempty", missingMessage: resources.getString("IP_SUBSTITUTION_ERROR_BAD_IP")});

        this.substitution = $("input[name=substitution]", this.fieldset);
        this.substitution.validatebox({required: true, validType: "substitution_nonempty", missingMessage: resources.getString("IP_SUBSTITUTION_ERROR_BAD_SUBSTITUTION")});
    },

    getIp: function() {
        return this.ip.val();
    },

    setIp: function(value) {
        this.ip.val(value);
    },

    getSubstitution: function() {
        return this.substitution.val();
    },

    setSubstitution: function(value) {
        this.substitution.val(value);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.ip.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.settings.AddIpSubstitutionDialog = jscape.settings.IpSubstitutionDialog.extend(
    /** @lends {jscape.settings.AddIpSubstitutionDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-failover-addrule"), resources);
    }
});

/** @class */
jscape.settings.EditIpSubstitutionDialog = jscape.settings.IpSubstitutionDialog.extend(
    /** @lends {jscape.settings.EditIpSubstitutionDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#d-failover-editrule"), resources);
    }
});

/** @class */
jscape.settings.DeleteIpSubstitutionController = Class.extend(
    /** @lends {jscape.settings.DeleteIpSubstitutionController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(rules) {
        var deferred = $.Deferred();
        this._confirmDelete(rules)
                .done(function() { deferred.resolve(rules); })
                .fail(function() { deferred.reject(); });
        return deferred.promise();
    },

    _confirmDelete: function(rules) {
        var text = $.map(rules || [], function(rule) { return rule.ip; }).join(", ");
        return jscape.ConfirmDialog.confirm(this.resources.getString("IP_SUBSTITUTION_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("IP_SUBSTITUTION_CONFIRM_DELETE_MESSAGE", {0: text}));
    }
});