jscape.settings = jscape.settings || {};

/** @class */
jscape.JmxServiceConfiguration = Class.extend(
    /** @lends {jscape.JmxServiceConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.enabled = a || false;
        this.host = b || "";
        this.port = parseInt(c) || 0;
        this.serverPort = parseInt(d) || null;
    }
});
jscape.JmxServiceConfiguration.fromJSON = function(data) {
    return data ? new jscape.JmxServiceConfiguration(data.enabled, data.host, data.port, data.serverPort) : null;
};

/** @class */
jscape.settings.JmxController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.JmxController.prototype} */
{
    /** @const */
    URL_TEMPLATE: "service:jmx:rmi://{host}:{serverPort}/jndi/rmi://{host}:{registryPort}/server",

    config: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.JmxView(resources);

        this._initValidators(resources);
    },

    /** @override */
    hasChanged: function () {
        return !Object._equals(this.config, this._createConfiguration());
    },

    /** @override */
    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setJmxServiceConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    onChange: function() {
        this._adjustJmxUrl();
    },

    /** @override */
    _initConfiguration: function() {
        this.config = this.serverSettings.getJmxServiceConfiguration();
    },

    _createConfiguration: function() {
        return new jscape.JmxServiceConfiguration(this.view.getEnabled(),
                                                  this.view.getHost(),
                                                  this.view.getRegistryPort(),
                                                  this.view.getServerPort());
    },

    _adjustJmxUrl: function() {
        this.view.setUrl(this._createJmxUrl());
    },

    _createJmxUrl: function() {
        var config = this._createConfiguration();
        return this.URL_TEMPLATE.supplant({host: config.host, serverPort: config.serverPort != null ? config.serverPort : config.port + 1, registryPort: config.port});
    },

    _setupView: function() {
        this.view.show(this);
        this.view.setHosts(this.serverSettings.getServerParameters().getInetAddresses());
        this.view.setHost(this.config.host);
        this.view.setRegistryPort(this.config.port);
        this.view.setServerPort(this.config.serverPort != null ? this.config.serverPort : this.config.port);
        this.view.setEnabled(this.config.enabled);
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.rule("jmx_ports", $.proxy(this._portNotEquals, this), resources.getString("JMX_ERROR_BAD_PORTS"));
    },

    _portNotEquals: function() {
        return this.view.getServerPort() != this.view.getRegistryPort();
    }
});

/** @class */
jscape.settings.JmxView = Class.extend(
    /** @lends {jscape.settings.JmxView.prototype} */
{
    /** @const */
    DEFAULT_HOST: "0.0.0.0",
    /** @const */
    DEFAULT_SERVER_PORT: 30051,
    /** @const */
    DEFAULT_REGISTRY_PORT: 30050,

    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=jmxservicefields]");

        this.enabled = $("input[name=enabled]:checkbox", this.fieldset);
        this.enabled.change($.proxy(this._adjustFieldState, this)).change($.proxy(this._onForm$Change, this));

        this.host = $("select[name=host]", this.fieldset);
        this.host.combobox({required: true, editable: false, disabled: true, onChange: $.proxy(this._onForm$Change, this), missingMessage: resources.getString("JMX_ERROR_BAD_HOST"), width: 170, panelWidth: 255, panelHeight: "auto"});

        this.serverPort = $("input[name=serverport]", this.fieldset);
        this.serverPort.numberspinner({required: true, disabled: true, validType: "jmx_ports", min: 1, max: 65535, onChange: $.proxy(this._onForm$Change, this)});

        this.registryPort = $("input[name=registryport]", this.fieldset);
        this.registryPort.numberspinner({required: true, disabled: true, validType: "jmx_ports", min: 1, max: 65535, onChange: $.proxy(this._onForm$Change, this)});

        this.jmxUrl = $("#jmx-service-url", this.fieldset);
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.JmxView.LISTENER;
        this._adjustFieldState();
    },

    getEnabled: function() {
        return this.enabled.is(":checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", !!value).change();
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHost: function(value) {
        this.host.combobox("setValue", value || this.DEFAULT_HOST);
    },

    setHosts: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.host.combobox("loadData",data);
    },

    getServerPort: function() {
        return parseInt(this.serverPort.numberspinner("getValue"));
    },

    setServerPort: function(value) {
        this.serverPort.numberspinner("setValue", parseInt(value) || this.DEFAULT_SERVER_PORT);
    },

    getRegistryPort: function() {
        return parseInt(this.registryPort.numberspinner("getValue"));
    },

    setRegistryPort: function(value) {
        this.registryPort.numberspinner("setValue", parseInt(value) || this.DEFAULT_REGISTRY_PORT);
    },

    setUrl: function(url) {
        this.jmxUrl.text(url);
    },

    _onForm$Change: function() {
        this.listener.onChange(this);
    },

    _adjustFieldState: function() {
        var enabled = this.enabled.is(":checked");
        this.host.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
        this.serverPort.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        this.registryPort.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");

        this.fieldset.form("validate");
    }
});
jscape.settings.JmxView.LISTENER = {
    onChange: $.noop
};