jscape.settings = jscape.settings || {};

/** @class */
jscape.ManagementAccessConfiguration = Class.extend(
    /** @lends {jscape.ManagementAccessConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.rules = a || [];
        this.authTimeout = parseInt(b) || 0;
        this.maxAuthAttempts = parseInt(c) || 0;
        this.maxAuthTimeout = parseInt(d) || 0;
    }
});
jscape.ManagementAccessConfiguration.fromJSON = function (data) {
    return new jscape.ManagementAccessConfiguration($.map(data.rules || [], jscape.IPAccessRule.fromJSON),
                                                    data.authTimeout,
                                                    data.maxAuthAttempts,
                                                    data.maxAuthTimeout);
};

/** @class */
jscape.IPAccessRule = Class.extend(
    /** @lends {jscape.IPAccessRule.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.mask = a;
        this.reason = b || "";
        this.allowed = c || false;
    }
});
jscape.IPAccessRule.fromJSON = function(data) {
    return data ? new jscape.IPAccessRule(data.mask, data.reason, data.allowed) : null;
};

/** @class */
jscape.settings.ManagerAccessController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.ManagerAccessController.prototype} */
{
    view: null,
    config: null,
    rules: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.ManagerAccessView(resources);

        this._initValidators(resources);
    },

    /** @override */
    hasChanged: function () {
        return !Object._equals(this.config, this._createConfiguration());
    },

    /** @override */
    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setManagementAccessServiceConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    onAddRule: function() {
        this.view.showAddDialog({
            onSubmit: $.proxy(this._onSubmit$AddDialog, this),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        });
    },

    onEditRule: function(rule) {
        var index = $.inArray(rule, this.rules);
        this.view.showEditDialog({
            onSubmit: $.proxy(this._onSubmit$EditDialog, this, index),
            onCancel: $.proxy(this._onCancel$Dialog, this)
        }, rule.mask, rule.reason, rule.allowed);
    },

    onDeleteRule: function(rule) {
        this.view.showConfirmDeleteDialog(rule.mask).done($.proxy(this._onConfirm$DeleteDialog, this, rule));
    },

    onRuleUp: function(rule) {
        var index = $.inArray(rule, this.rules);
        this.rules.splice(index - 1, 2, this.rules[index], this.rules[index - 1]);
        this.view.setAccessRules(this.rules, index - 1);
    },

    onRuleDown: function(rule) {
        var index = $.inArray(rule, this.rules);
        this.rules.splice(index, 2, this.rules[index + 1], this.rules[index]);
        this.view.setAccessRules(this.rules, index + 1);
    },

    _onSubmit$AddDialog: function(dialog) {
        var rule = new jscape.IPAccessRule(dialog.getMask(), dialog.getReason(), dialog.getAllowed());
        dialog.hide();

        this.rules.push(rule);
        this.view.setAccessRules(this.rules, $.inArray(rule, this.rules));
    },

    _onSubmit$EditDialog: function(index, dialog) {
        var rule = new jscape.IPAccessRule(dialog.getMask(), dialog.getReason(), dialog.getAllowed());
        dialog.hide();

        this.rules[index] = rule;
        this.view.setAccessRules(this.rules, index);
    },

    _onConfirm$DeleteDialog: function(rule) {
        var index = $.inArray(rule, this.rules);
        if (index != -1) {
            this.rules.splice(index, 1);
            this.view.setAccessRules(this.rules, index);
        }
    },

    _onCancel$Dialog: function(dialog) {
        dialog.hide();
    },

    _initConfiguration: function() {
        this.config = this.serverSettings.getManagementAccessServiceConfiguration();
        this.rules = $.extend(true, [], this.config.rules);
    },

    _createConfiguration: function() {
        return new jscape.ManagementAccessConfiguration(this.view.getAccessRules(),
                                                        this.view.getTimeout(),
                                                        this.view.getAuthAttempts(),
                                                        this.view.getAuthTimeout(),
                                                        null);
    },

    _setupView: function() {
        this.view.setAccessRules(this.rules);
        this.view.setTimeout(this.config.authTimeout);
        this.view.setAuthAttempts(this.config.maxAuthAttempts);
        this.view.setAuthTimeout(this.config.maxAuthTimeout);
        this.view.show(this);
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("ipaccess_mask_nonempty", resources.getString("MANAGER_ACCESS_ERROR_BAD_MASK"));
        v.rule("ipaccess_unique", $.proxy(this._ruleNotExists, this), resources.getString("MANAGER_ACCESS_ERROR_RULE_EXISTS"));
    },

    _ruleNotExists: function(value, excludes) {
        for (var i = 0; i < this.rules.length; i++) {
            var mask = this.rules[i].mask;
            if (excludes && $.inArray(mask, excludes) != -1) {
                continue;
            }
            if (mask == value) {
                return false;
            }
        }
        return true;
    }
});

/** @class */
jscape.settings.IpAccessDialog = Class.extend(
    /** @lends {jscape.settings.IpAccessDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#ssma-dialog-ipaccess").clone().attr("id", Math.random().toString(36).substr(2));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=manageripaccessfields]", this.dialog);

        this.mask = $("input[name=mask]", this.fieldset);
        this.mask.validatebox({required: true, validType: ["ipaccess_mask_nonempty", "ipaccess_unique"], missingMessage: resources.getString("MANAGER_ACCESS_ERROR_BAD_MASK")});

        this.reason = $("input[name=reason]", this.fieldset);

        this.access = $("input:radio[name=access]", this.fieldset);
        this.accessAllowed = this.access.filter("[value=allowed]");
        this.accessDenied = this.access.filter("[value=denied]");
    },

    getMask: function() {
        return this.mask.val();
    },

    setMask: function(value) {
        this.mask.val(value);
        if (value && value.trim().length) {
            this.mask.validatebox("options").validType = ["ipaccess_mask_nonempty", "ipaccess_unique['" + value + "']"];
        }
    },

    getReason: function () {
        return this.reason.val();
    },

    setReason: function (value) {
        this.reason.val(value);
    },

    getAllowed: function () {
        return this.accessAllowed.is(":checked");
    },

    setAllowed: function (value) {
        if (value) {
            this.accessAllowed.prop("checked", true).change();
        } else {
            this.accessDenied.prop("checked", true);
        }
    },

    setTitle: function(value) {
        this.dialog.dialog("setTitle", value);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.mask.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});


/** @class */
jscape.settings.ManagerAccessView = Class.extend(
    /** @lends {jscape.settings.ManagerAccessView.prototype} */
{
    /** @const */
    DEFAULT_TIMEOUT: 10,
    DEFAULT_AUTH_ATTEMPTS: 3,
    DEFAULT_AUTH_TIMEOUT: 1,
    TIMEOUT_FACTOR: 1000,
    AUTH_TIMEOUT_FACTOR: 1000 * 60,

    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=manageraccessfields]");

        this.timeoutEnabled = $("input[name=timeoutenabled]:checkbox", this.fieldset);
        this.timeoutEnabled.click($.proxy(this._adjustFieldState, this));
        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({min: 1, max: 10, disabled: true});

        this.authEnabled = $("input[name=authenabled]:checkbox", this.fieldset);
        this.authEnabled.click($.proxy(this._adjustFieldState, this));
        this.authAttempts = $("input[name=authattempts]", this.fieldset);
        this.authAttempts.numberspinner({min: 1, max: 10, disabled: true});
        this.authTimeout = $("input[name=authtimeout]", this.fieldset);
        this.authTimeout.numberspinner({min: 1, max: 10, disabled: true});

        this.data = $("table[role=grid][aria-label=accessrules]", this.fieldset);
        this.data.datagrid({
            singleSelect: true,
            remoteSort: false,
            fitColumns: true,
            onSelect: $.proxy(this._onSelect$Row, this),
            onSortColumn: $.proxy(this._onSort$Column, this),
            columns: [[
                {field: "mask", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 60},
                {field: "allowed", title: $("thead th:nth-child(2)", this.data).text(), sortable: true, width: 15, formatter: function (value) {return value ? "allowed" : "denied"}, styler: function (value) {return value ? "color:green" : "color:red"}},
                {field: "reason", title: $("thead th:nth-child(3)", this.data).text(), sortable: true, width: 25}
            ]]
        });

        this.moveUpButton = $("a[role=button][name=moveup]", this.fieldset);
        this.moveUpButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveUpButton, this)});

        this.moveDownButton = $("a[role=button][name=movedown]", this.fieldset);
        this.moveDownButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveDownButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.ManagerAccessView.LISTENER;
        this._initState();
    },

    showAddDialog: function(listener) {
        var dialog = new jscape.settings.IpAccessDialog(this.resources);
        dialog.setTitle(this.resources.getString("MANAGER_ACCESS_ADD_TITLE"));
        dialog.setAllowed(true);
        dialog.show(listener);
    },

    showEditDialog: function(listener, mask, reason, allowed) {
        var dialog = new jscape.settings.IpAccessDialog(this.resources);
        dialog.setTitle(this.resources.getString("MANAGER_ACCESS_EDIT_TITLE"));
        dialog.setMask(mask);
        dialog.setReason(reason);
        dialog.setAllowed(allowed);
        dialog.show(listener);
    },

    showConfirmDeleteDialog: function (mask) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("MANAGER_ACCESS_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("MANAGER_ACCESS_CONFIRM_DELETE_MESSAGE", {0: mask}));
    },

    getAccessRules: function() {
        return this.data.datagrid("getRows");
    },

    setAccessRules: function(rules, selectIndex) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", rules);
        this.data.datagrid("loaded");

        if (selectIndex !== undefined) {
            var index = Math.max(0, Math.min(selectIndex, rules.length - 1));
            this.data.datagrid("selectRow", index);
        }
    },

    getTimeout: function() {
        return this.timeoutEnabled.prop("checked")
                ? parseInt(this.timeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR
                : null;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", value ? value / this.TIMEOUT_FACTOR : this.DEFAULT_TIMEOUT);
        this.timeoutEnabled.prop("checked", !!value);
        this._adjustFieldState();
    },

    getAuthAttempts: function() {
        return this.authEnabled.prop("checked")
                ? parseInt(this.authAttempts.numberspinner("getValue"))
                : null;
    },

    setAuthAttempts: function(value) {
        this.authAttempts.numberspinner("setValue", value ? value : this.DEFAULT_AUTH_ATTEMPTS);
        this.authEnabled.prop("checked", !!value);
        this._adjustFieldState();
    },

    getAuthTimeout: function() {
        return this.authEnabled.prop("checked")
                ? parseInt(this.authTimeout.numberspinner("getValue")) * this.AUTH_TIMEOUT_FACTOR
                : null;
    },

    setAuthTimeout: function(value) {
        value = value ? value / this.AUTH_TIMEOUT_FACTOR : this.DEFAULT_AUTH_TIMEOUT;
        this.authTimeout.numberspinner("setValue", value);
    },

    _initState: function() {
        this._adjustFieldState();
        this._adjustButtonsState();
    },

    _onSelect$Row: function() {
        this._adjustButtonsState();
    },

    _onSort$Column: function() {
        this._adjustButtonsState();
    },

    _onClick$MoveUpButton: function() {
        this.listener.onRuleUp(this.data.datagrid("getSelected"));
    },

    _onClick$MoveDownButton: function() {
        this.listener.onRuleDown(this.data.datagrid("getSelected"));
    },

    _onClick$AddButton: function() {
        this.listener.onAddRule();
    },

    _onClick$EditButton: function() {
        this.listener.onEditRule(this.data.datagrid("getSelected"));
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteRule(this.data.datagrid("getSelected"));
    },

    _adjustFieldState: function() {
        this.timeout.numberspinner(this.timeoutEnabled.prop("checked") ? "enable" : "disable");
        var enable = this.authEnabled.prop("checked") ? "enable" : "disable";
        this.authAttempts.numberspinner(enable);
        this.authTimeout.numberspinner(enable);
    },

    _adjustButtonsState: function() {
        var rule = this.data.datagrid("getSelected");
        var index = this.data.datagrid("getRowIndex", rule);
        var count = this.data.datagrid("getRows").length;

        this._setButtonEnabled(this.moveUpButton, index > 0 && index <= count - 1);
        this._setButtonEnabled(this.moveDownButton, index >= 0 && index < count - 1);
        this._setButtonEnabled(this.editButton, rule != null);
        this._setButtonEnabled(this.deleteButton, rule != null);
    },

    _buttonEnabled: function(button) {
        return !button.linkbutton("options").disabled;
    },

    _setButtonEnabled: function(button, enabled) {
        button.linkbutton(enabled ? "enable" : "disable");
    }
});
/** @const */
jscape.settings.ManagerAccessView.LISTENER = {
    onAddRule: $.noop,
    onEditRule: $.noop,
    onDeleteRule: $.noop,
    onRuleUp: $.noop,
    onRuleDown: $.noop
};