var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.Administrator = Class.extend(
    /** @lends {jscape.Administrator.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.name = a;
        this.username = b;
        this.enabled = c || false;
        this.passwordHash = d || undefined;
    }
});
jscape.Administrator.fromJSON = function(data) {
    return new jscape.Administrator(data.name, data.username, data.enabled, data.passwordHash);
};

/** @class */
jscape.settings.AdministratorsController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.AdministratorsController.prototype} */
{
    view: null,
    administrators: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.AdministratorsView(resources);
        this.addController = new jscape.settings.AddAdministratorController(resources);
        this.editController = new jscape.settings.EditAdministratorController(resources);

        this._initValidators(resources);
    },

    /** @override */
    hasChanged: function () {
        return !Object._equals(this.administrators, this._createConfiguration());
    },

    /** @override */
    submitChanges: function() {
        var admins = this._createConfiguration();
        this.serverSettings.setAdministrators(admins);
        this.listener.onChangesSubmitted(this);
    },

    onAdd: function() {
        this.addController.start().done($.proxy(this._adminAdded, this));
    },

    onEdit: function(admin) {
        var index = $.inArray(admin, this.administrators);
        if (index != -1) {
            var enableChangeAllowed = this.view._hasEnabledAdministrators(admin);
            this.editController.start(admin, enableChangeAllowed)
                    .done($.proxy(this._adminEdited, this, index))
        }
    },

    onDelete: function(admin) {
        this.view.showConfirmDeleteDialog(admin.name)
                .done($.proxy(this._onConfirm$DeleteDialog, this, admin));
    },

    _adminAdded: function(admin) {
        this.administrators.push(admin);
        this.view.setAdministrators(this.administrators, $.inArray(admin, this.administrators));
    },

    _adminEdited: function(index, admin) {
        this.administrators[index] = admin;
        this.view.setAdministrators(this.administrators, index);
    },

    _onConfirm$DeleteDialog: function (admin) {
        var index = $.inArray(admin, this.administrators);
        if (index != -1) {
            this.administrators.splice(index, 1);
            this.view.setAdministrators(this.administrators, index);
        }
    },

    _initConfiguration: function() {
        this.administrators = this.serverSettings.getAdministrators();
    },

    _createConfiguration: function() {
        return this.view.getAdministrators();
    },

    _setupView: function() {
        this.view.show(this);
        this.view.setAdministrators(this.administrators);
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("admin_login_nonempty", resources.getString("ADMINISTRATORS_ERROR_BAD_LOGIN"));
        v.rule("admin_unique", $.proxy(this._loginNotExists, this), resources.getString("ADMINISTRATORS_ERROR_LOGIN_EXISTS"));
        v.rule("admin_confirmpwd", $.proxy(this._passwordConfirmed, this), resources.getString("ADMINISTRATORS_ERROR_PASSWORDS_NOT_MATCH"));
    },

    _loginNotExists: function(value) {
        for (var i = 0; i < this.administrators.length; i++) {
            if (this.administrators[i].username == value) {
                return false;
            }
        }
        return true;
    },

    _passwordConfirmed: function(value, params) {
        return value == $(params[0]).val();
    }
});

/** @class */
jscape.settings.AddAdministratorController = Class.extend(
    /** @lends {jscape.settings.AddAdministratorDialog.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function() {
        this.deferred = $.Deferred();

        this._setupDialog().show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var admin = this._createAdministrator(dialog);
        dialog.destroy();
        this.deferred.resolve(admin);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    _createAdministrator: function(dialog) {
        return $.extend(new jscape.Administrator(dialog.getName(), dialog.getLogin(), dialog.getEnabled()), {password: dialog.getPassword()});
    },

    _setupDialog: function() {
        var dialog = new jscape.settings.AddAdministratorDialog(this.resources);
        dialog.setEnabled(true);
        return dialog;
    }
});

/** @class */
jscape.settings.AddAdministratorDialog = Class.extend(
    /** @lends {jscape.settings.AddAdministratorDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#ssmsadmin-dialog-add").clone().attr("id", Math.random().toString(36));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=administratorfileds]", this.dialog);

        this.name = $("input[name=name]", this.fieldset);
        this.enabled = $("input[name=enabled]:checkbox", this.fieldset);

        this.login = $("input[name=login]", this.fieldset);
        this.login.validatebox({required: true, validType: ["admin_login_nonempty", "admin_unique"], missingMessage: resources.getString("ADMINISTRATORS_ERROR_BAD_LOGIN")});

        var passwordId = "pwd_" + Math.random().toString(36).substr(2);
        this.password = $("input[name=password]", this.fieldset).attr("id", passwordId);
        this.password.validatebox({required: true, missingMessage: resources.getString("ADMINISTRATORS_ERROR_BAD_PASSWORD")});
        this.confirmPassword = $("input[name=confirmpwd]", this.fieldset);
        this.confirmPassword.validatebox({required: true, validType: "admin_confirmpwd['#" + passwordId + "']", missingMessage: resources.getString("ADMINISTRATORS_ERROR_BAD_PASSWORD")});
    },

    getName: function() {
        return this.name.val();
    },

    getLogin: function() {
        return this.login.val();
    },

    getPassword: function() {
        return this.password.val();
    },

    getEnabled: function() {
        return this.enabled.prop("checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", !!value);
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.name.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.settings.EditAdministratorController = Class.extend(
    /** @lends {jscape.settings.EditAdministratorController.prototype} */
{
    newPassword: null,
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(admin, enableChangeAllowed) {
        this.newPassword = null;
        this.deferred = $.Deferred();
        this.administrator = admin;

        this._setupDialog(admin, enableChangeAllowed).show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var admin = this._createAdministrator(dialog);
        dialog.destroy();
        this.deferred.resolve(admin);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    },

    onChangePassword: function() {
        var dialog = new jscape.settings.ChangePasswordDialog(this.resources);
        dialog.show({
            onSubmit: $.proxy(function(d) { this.newPassword = d.getPassword(); d.destroy(); }, this),
            onCancel: function(d) { d.destroy(); }
        });
    },

    _createAdministrator: function(dialog) {
        return this.newPassword == null
               ? new jscape.Administrator(dialog.getName(), this.administrator.username, dialog.getEnabled(), this.administrator.passwordHash)
               : $.extend(new jscape.Administrator(dialog.getName(), this.administrator.username, dialog.getEnabled()), {password: this.newPassword});
    },

    _setupDialog: function (administrator, enableChangeAllowed) {
        var dialog = new jscape.settings.EditAdministratorDialog(this.resources);
        dialog.setName(administrator.name);
        dialog.setLogin(administrator.username);
        dialog.setEnabled(administrator.enabled);
        dialog.setEnabledChangeAllowed(enableChangeAllowed);
        return dialog;
    }
});

/** @class */
jscape.settings.EditAdministratorDialog = Class.extend(
    /** @lends {jscape.settings.EditAdministratorDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#ssmsadmin-dialog-edit").clone().attr("id", Math.random().toString(36).substr(2));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=administratorfileds]", this.dialog);

        this.name = $("input[name=name]", this.fieldset);
        this.login = $("input[name=login]", this.fieldset).prop("disabled", true);
        this.enabled = $("input[name=enabled]:checkbox", this.fieldset);

        this.changePasswordButton = $("a[role=button][name=changepwd]", this.fieldset);
        this.changePasswordButton.linkbutton({onClick: $.proxy(this._onClick$ChangePasswordButton, this)});
    },

    getName: function() {
        return this.name.val();
    },

    setName: function(value) {
        this.name.val(value);
    },

    setLogin: function(value) {
        this.login.val(value);
    },

    getEnabled: function() {
        return this.enabled.prop("checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", value);
    },

    setEnabledChangeAllowed: function(value) {
        if (!value) {
            this.enabled.prop("readonly", true);
            this.enabled.click(function() { return false; });
        }
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.name.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$ChangePasswordButton: function() {
        this.listener.onChangePassword(this);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.settings.EditAdministratorDialog.LISTENER = {
    onChangePassword: $.noop
};


/** @class */
jscape.settings.ChangePasswordDialog = Class.extend(
    /** @lends {jscape.settings.ChangePasswordDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#ssmsadmin-dialog-changepwd").clone().attr("id", Math.random().toString(36).substr(2));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=administratorfileds]", this.dialog);

        var passwordId = "changepwd_" + Math.random().toString(36).substr(2);
        this.password = $("input[name=password]", this.fieldset).attr("id", passwordId);
        this.password.validatebox({required: true, missingMessage: resources.getString("ADMINISTRATORS_ERROR_BAD_PASSWORD")});
        this.confirmPassword = $("input[name=confirmpwd]", this.fieldset);
        this.confirmPassword.validatebox({required: true, validType: "admin_confirmpwd['#" + passwordId + "']", missingMessage: resources.getString("ADMINISTRATORS_ERROR_BAD_PASSWORD"), invalidMessage: resources.getString("ADMINISTRATORS_ERROR_PASSWORDS_NOT_MATCH")});
    },

    getPassword: function() {
        return this.password.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.password.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});


/** @class */
jscape.settings.AdministratorsView = Class.extend(
    /** @lends {jscape.settings.AdministratorsView.prototype} */
{
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=systemadministratorsfields]");

        this.data = $("table[role=grid][aria-label=systemadministrators]", this.fieldset);
        this.data.datagrid({
            singleSelect: true,
            remoteSort: false,
            fitColumns: true,
            onDblClickRow: $.proxy(this._onClick$EditButton, this),
            onSelect: $.proxy(this._adjustButtonsState, this),
            onSortColumn: $.proxy(this._adjustButtonsState, this),
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 200},
                {field: "username", title: $("thead th:nth-child(2)", this.data).text(), sortable: true, width: 200},
                {field: "enabled", title: $("thead th:nth-child(3)", this.data).text(), sortable: true, width: 80, formatter: function (value) { return value ? "enabled" : "disabled"; }, styler: function (value) { return value ? "color:green" : "color:grey"; }}
            ]]
        });

        this.enableButton = $("a[role=button][name=enable]", this.fieldset);
        this.enableButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EnableButton, this)});

        this.disableButton = $("a[role=button][name=disable]", this.fieldset);
        this.disableButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DisableButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.AdministratorsView.LISTENER;
        this._adjustButtonsState();
    },

    showConfirmDeleteDialog: function(name) {
        return jscape.ConfirmDialog.confirm(this.resources.getString("ADMINISTRATORS_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("ADMINISTRATORS_CONFIRM_DELETE_MESSAGE", {0: name}));
    },

    getAdministrators: function() {
        return this.data.datagrid("getRows");
    },

    setAdministrators: function(items, selectIndex) {
        this.data.datagrid("loading");
        this.data.datagrid("loadData", items);
        this.data.datagrid("loaded");

        if (selectIndex !== undefined) {
            var index = Math.max(0, Math.min(selectIndex, items.length - 1));
            this.data.datagrid("selectRow", index);
        }
    },

    _onClick$AddButton: function () {
        this.listener.onAdd();
    },

    _onClick$EditButton: function () {
        this.listener.onEdit(this.data.datagrid("getSelected"));
    },

    _onClick$DeleteButton: function() {
        this.listener.onDelete(this.data.datagrid("getSelected"));
    },

    _onClick$EnableButton: function () {
        var admin = this.data.datagrid("getSelected");
        var index = this.data.datagrid("getRowIndex", admin);
        admin.enabled = true;
        this.data.datagrid("refreshRow", index);
        this.data.datagrid("selectRow", index);
    },

    _onClick$DisableButton: function () {
        var admin = this.data.datagrid("getSelected");
        var index = this.data.datagrid("getRowIndex", admin);
        admin.enabled = false;
        this.data.datagrid("refreshRow", index);
        this.data.datagrid("selectRow", index);
    },

    _adjustButtonsState: function() {
        var admin = this.data.datagrid("getSelected");
        var count = this.data.datagrid("getRows").length;
        var enableChangeAllowed = this._hasEnabledAdministrators(admin);

        this._setButtonEnabled(this.enableButton, admin && !admin.enabled);
        this._setButtonEnabled(this.disableButton, admin && admin.enabled && enableChangeAllowed);
        this._setButtonEnabled(this.editButton, admin != null);
        this._setButtonEnabled(this.deleteButton, admin != null && count > 1 && enableChangeAllowed);
    },

    _hasEnabledAdministrators: function (excludeAdmin) {
        var rows = this.data.datagrid("getRows");
        for (var i = 0; i < rows.length; ++i) {
            var admin = rows[i];
            if (admin != excludeAdmin && admin.enabled) {
                return true;
            }
        }
        return false;
    },

    _setButtonEnabled: function(button, enabled) {
        button.linkbutton(enabled ? "enable" : "disable");
    }
});
/** @const */
jscape.settings.AdministratorsView.LISTENER ={
    onAdd: $.noop,
    onEdit: $.noop,
    onDelete: $.noop
};