jscape.settings = jscape.settings || {};

/** @class */
jscape.ServerManagementServiceConfiguration = Class.extend(
    /** @lends {jscape.ServerManagementServiceConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.host = a || "";
        this.port = parseInt(b) || 0;
        this.timeout = parseInt(c) || 0;
    },

    clone: function() {
        return new jscape.ServerManagementServiceConfiguration(this.host, this.port, this.timeout);
    }
});
jscape.ServerManagementServiceConfiguration.fromJSON = function(data) {
    return data ? $.extend(new jscape.ServerManagementServiceConfiguration, data) : null;
};

/** @class */
jscape.settings.ManagerServiceController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.ManagerServiceController.prototype} */
{
    config: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.ManagerServiceView(resources);
    },

    /** @override */
    hasChanged: function () {
        return !Object._equals(this.config, this._createConfiguration());
    },

    /** @override */
    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setServerManagementServiceConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    _initConfiguration: function() {
        this.config = this.serverSettings.getServerManagementServiceConfiguration();
    },

    _createConfiguration: function() {
        return new jscape.ServerManagementServiceConfiguration(this.view.getHost(),
                                                               this.view.getPort(),
                                                               this.view.getTimeout());
    },

    _setupView: function() {
        this.view.setHosts(this.serverSettings.getServerParameters().getInetAddresses());
        this.view.setHost(this.config.host);
        this.view.setPort(this.config.port);
        this.view.setTimeout(this.config.timeout);
        this.view.show(this);
    }
});

/** @class */
jscape.settings.ManagerServiceView = Class.extend(
    /** @lends {jscape.settings.ManagerServiceView.prototype} */
{
    /** @const */
    DEFAULT_PORT: 10880,
    DEFAULT_TIMEOUT: 600,
    TIMEOUT_FACTOR: 1000,

    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=managerservicefields]");

        this.host = $("select[name=host]", this.fieldset);
        this.host.combobox({
            valueField: "host",
            textField: "value",
            required: true,
            missingMessage: resources.getString("MANAGER_SERVICE_ERROR_BAD_HOST"),
            width: 170,
            panelWidth: 255,
            panelHeight: "auto"
        });

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({min: 1, max: 65535, required: true});

        this.timeout = $("input[name=timeout]", this.fieldset);
        this.timeout.numberspinner({min: 1, max: 1000, required: true});
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.ManagerServiceView.LISTENER;
        this.fieldset.form("validate");
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHost: function(value) {
        this.host.combobox("setValue", value).combobox("select", value)
    },

    setHosts: function(values) {
        var data = $.map(values, function(value) { return {host: value, value: value}; });
        this.host.combobox("loadData", data);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", value || this.DEFAULT_PORT);
    },

    getTimeout: function() {
        return parseInt(this.timeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setTimeout: function(value) {
        this.timeout.numberspinner("setValue", value ? value / this.TIMEOUT_FACTOR : this.DEFAULT_TIMEOUT);
    }
});
/** @const */
jscape.settings.ManagerServiceView.LISTENER = {};