jscape.settings = jscape.settings || {};

/** @class */
jscape.SearchingServiceConfiguration = Class.extend(
    /** @lends {jscape.SearchingServiceConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e) {
        this.indexingPeriod = parseInt(a) || null;
        this.updateFileIndexUponUpload = b || false;
        this.updateFileIndexUponRename = c || false;
        this.updateAccountIndexUponUpdating = d || false;
        this.updateGroupIndexUponUpdating = e || false;
    }
});
jscape.SearchingServiceConfiguration.fromJSON = function(data) {
    return data ? $.extend(new jscape.SearchingServiceConfiguration(), data) : null;
};

/** @class */
jscape.settings.SearchingController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.SearchingController.prototype} */
{
    DELAY: 5000,

    config: null,
    timerId: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.SearchingView(resources);
    },

    /** @override */
    hasChanged: function () {
        return !Object._equals(this.config, this._createConfiguration());
    },

    /** @override */
    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setSearchingServiceConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    onRebuildIndex: function() {
        var deferred = $.Deferred();
        this.managementService.startSearchFileIndexing()
                .done($.proxy(this._poll, this, deferred, 1000))
                .fail(function() { deferred.reject(); });
        return deferred.promise();
    },

    /** @override */
    _initConfiguration: function() {
        this.config = this.serverSettings.getSearchingServiceConfiguration();
    },

    _poll: function(deferred, delay) {
        if (this.timerId) {
            clearTimeout(this.timerId);
        }
        this.timerId = setTimeout($.proxy(function () {
            this.managementService.getSearchFileIndexingStats()
                    .done($.proxy(function (stats) {
                        var state = stats && stats.state ? stats.state : undefined;
                        if ("IN_PROGRESS" === state) {
                            this._poll(deferred, this.DELAY);
                        } else {
                            var success = "SUCCESS" === state;
                            deferred.resolve(success);
                            this._fireFileIndexingComplete(success);
                        }
                    }, this))
                    .fail(function () { deferred.reject(); });
        }, this), delay || 0);
    },

    _createConfiguration: function() {
        return new jscape.SearchingServiceConfiguration(this.view.getIndexingPeriod(),
                                                        this.view.getUpdateFileIndexUponUpload(),
                                                        this.view.getUpdateFileIndexUponRename(),
                                                        this.view.getUpdateAccountIndexUponUpdate(),
                                                        this.view.getUpdateGroupIndexUponUpdate());
    },

    _setupView: function() {
        this.view.setIndexingPeriod(this.config.indexingPeriod);
        this.view.setUpdateFileIndexUponUpload(this.config.updateFileIndexUponUpload);
        this.view.setUpdateFileIndexUponRename(this.config.updateFileIndexUponRename);
        this.view.setUpdateAccountIndexUponUpdate(this.config.updateAccountIndexUponUpdating);
        this.view.setUpdateGroupIndexUponUpdate(this.config.updateGroupIndexUponUpdating);
        this.view.show(this);
    },

    _fireFileIndexingComplete: function(success) {
        $(document).triggerHandler("broadcast", [success ? this.resources.getString("SETTINGS_SUCCESS_FILE_INDEX") : this.resources.getString("SETTINGS_ERROR_FILE_INDEX")]);
    }
});

/** @class */
jscape.settings.SearchingView = Class.extend(
    /** @lends {jscape.settings.SearchingView.prototype} */
{
    INDEXING_PERIOD_FACTOR: 1000 * 60 * 60,
    DEFAULT_INDEXING_PERIOD: 24,

    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=searchingservicefields]");

        this.rebuildIndexEnabled = $("input[name=reindex]:checkbox", this.fieldset);
        this.rebuildIndexEnabled.change($.proxy(function() {
            var enabled = this.rebuildIndexEnabled.is(":checked");
            this.rebuildIndexPeriod.numberspinner(enabled ? "enable" : "disable");
        }, this));
        this.rebuildIndexPeriod = $("input[name=indexperiod]", this.fieldset);
        this.rebuildIndexPeriod.numberspinner({disabled: true, min: 1, max: 1000, width: 60});

        this.updateFileIndexUponUpload = $("input[name=fileupload]:checkbox", this.fieldset);
        this.updateFileIndexUponRename = $("input[name=filerename]:checkbox", this.fieldset);
        this.updateAccountIndexUponUpdate = $("input[name=accountupdate]:checkbox", this.fieldset);
        this.updateGroupIndexUponUpdate = $("input[name=groupupdate]:checkbox", this.fieldset);

        this.rebuildButton = $("a[role=button][name=rebuild]", this.fieldset);
        this.rebuildButton.linkbutton({onClick: $.proxy(this._onClick$RebuildButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.SearchingView.LISTENER;
    },

    getIndexingPeriod: function() {
        return this.rebuildIndexEnabled.is(":checked") ? parseInt(this.rebuildIndexPeriod.numberspinner("getValue")) * this.INDEXING_PERIOD_FACTOR : null;
    },

    setIndexingPeriod: function(value) {
        this.rebuildIndexEnabled.prop("checked", value != null).change();
        this.rebuildIndexPeriod.numberspinner("setValue", value != null ? parseInt(value) / this.INDEXING_PERIOD_FACTOR : this.DEFAULT_INDEXING_PERIOD);
    },

    getUpdateFileIndexUponUpload: function() {
        return this.updateFileIndexUponUpload.is(":checked");
    },

    setUpdateFileIndexUponUpload: function(value) {
        this.updateFileIndexUponUpload.prop("checked", !!value);
    },

    getUpdateFileIndexUponRename: function() {
        return this.updateFileIndexUponRename.is(":checked");
    },

    setUpdateFileIndexUponRename: function(value) {
        this.updateFileIndexUponRename.prop("checked", !!value);
    },

    getUpdateAccountIndexUponUpdate: function() {
        return this.updateAccountIndexUponUpdate.is(":checked");
    },

    setUpdateAccountIndexUponUpdate: function(value) {
        this.updateAccountIndexUponUpdate.prop("checked", !!value);
    },

    getUpdateGroupIndexUponUpdate: function() {
        return this.updateGroupIndexUponUpdate.is(":checked");
    },

    setUpdateGroupIndexUponUpdate: function(value) {
        this.updateGroupIndexUponUpdate.prop("checked", !!value);
    },

    _onClick$RebuildButton: function() {
        this.rebuildButton.linkbutton("disable");
        this._toggleButtonText(this.rebuildButton);

        this.listener.onRebuildIndex(this)
                .always($.proxy(function() {
                    this.rebuildButton.linkbutton("enable");
                    this._toggleButtonText(this.rebuildButton);
                }, this));
    },

    _toggleButtonText: function(button) {
        var text = button.text();
        button.html(button.html().replace(text, button.data("toggle-text") || text));
        button.data("toggle-text", text);
    }
});
jscape.settings.SearchingView.LISTENER = {
    onRebuildIndex: $.noop
};
