var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.settings.ServerSettingsController = jscape.ApplicationModule.extend(
    /** @lends {jscape.settings.ServerSettingsController.prototype} */
{
    modules: null,
    moduleQueue: null,
    serverSettings: null,
    restConfigCopy: null,
    documentViewerAllowed: null,

    /** @constructor */
    initialize: function(view, resources, modules) {
        this.view = view;

        this.resources = resources;
        this.modules = modules || [];
        this.menu = new jscape.Menu();
        this.menu.bind(this, "settings");
        $(document).on("broadcast", $.proxy(this.onBroadcast, this))
                .on("done.settings", $.proxy(this.onDone, this));
    },

    start: function(managementService) {
        this.managementService = managementService;

        this._initModuleQueue();
        this._loadServerSettings().done($.proxy(function(settings) {
            this.serverSettings = settings;
            this.restConfigCopy = this._copyRestConfig();
            this._startNextModule();
        }, this));
    },

    onMenuItemSelected: function (item) {
        if (item && item.id) {
            switch (item.id) {
                case jscape.Menu.ITEM.Home:
                case jscape.Menu.ITEM.KeyManager:
                case jscape.Menu.ITEM.ServerSettings:
                    setTimeout($.proxy(function() { this.onRedirect(item.url || "/"); }, this), 10);
                    return true;
                case jscape.Menu.ITEM.Logout:
                    this.onLogout();
                    return true;
            }
        }
        return false;
    },

    onBroadcast: function (e, message) {
        if (message && message.trim().length) {
            var style = {top: document.body.scrollTop + document.documentElement.scrollTop + 20, right: "", bottom: ""};
            $.messager.show({msg: message, noheader: true, height: "auto", showType: "fade", showSpeed: 200, style: style});
        }
    },

    onDone: function(e, success) {
        if (success) {
            this._redirect();
        }
    },

    onSubmit: function() {
        this._initModuleQueue();
        this._submitNextModule();
    },

    onCancel: function() {
        this._initModuleQueue();
        this._cancelNextModule();
    },

    onLeave: function(data) {
        data.modified = false;
        var copy = [].concat(this.modules);
        while (copy.length) {
            var m = copy.pop();
            data.modified |= m.hasChanged();
        }
    },

    onLogout: function() {
        this.managementService.logout();
    },

    onStarted: function() {
        if (this.moduleQueue.length == 0) {
            this.view.show(this);
        } else {
            this._startNextModule();
        }
    },

    onStartError: function() {},

    onChangesSubmitted: function() {
        if (this.moduleQueue.length == 0) {
            this._submitServerSettings();
        } else {
            this._submitNextModule();
        }
    },

    onChangesCancelled: function() {
        if (this.moduleQueue.length == 0) {
            this._redirect();
        } else {
            this._cancelNextModule();
        }
    },

    _initModuleQueue: function() {
        this.moduleQueue = [].concat(this.modules);
    },

    _startNextModule: function() {
        var m = this.moduleQueue.pop();
        m.start(this, this.managementService, this.serverSettings);
        this.menu.bind(m, "settings");
    },

    _submitNextModule: function() {
        var m = this.moduleQueue.pop();
        m.submitChanges();
        this.menu.unbind("settings");
    },

    _cancelNextModule: function() {
        var m = this.moduleQueue.pop();
        m.cancelChanges();
        this.menu.unbind("settings");
    },

    _copyRestConfig: function() {
        return $.extend(new jscape.ManagementWebServerConfiguration(), this.serverSettings.getManagementWebServerConfiguration());
    },

    _submitServerSettings: function () {
        var webConfig = this.serverSettings.getWebServiceDescriptor();
        var restConfig = this.serverSettings.getManagementWebServerConfiguration();
        var restServiceModified = !Object._equals(this.restConfigCopy, restConfig);
        var deferred = $.Deferred();
        deferred.done(function() { $(document).triggerHandler("done.settings", [true]); })
                .fail(function() { $(document).triggerHandler("done.settings", [false]); });

        if (restServiceModified) {
            this._confirmServerRestart()
                    .done($.proxy(function() {
                        this._storeServerSettings()
                                .then($.proxy(function() {
                                    return $.when(this.managementService.setWebServiceConfiguration(webConfig),
                                                  this.managementService.setManagementWebServerConfiguration(restConfig))
                                }, this))
                                .done($.proxy(this._waitUntilServerRestart(restConfig, deferred, 1000), this))
                                .fail(function() { deferred.reject(); });
                    }, this))
                    .fail(function() { deferred.reject(); });
        } else {
            this._storeServerSettings()
                    .then($.proxy(function() {
                        return this.managementService.setWebServiceConfiguration(webConfig);
                    }, this))
                    .done($.proxy(function() { deferred.resolve(); }, this))
                    .fail(function() { deferred.reject(); });
        }
    },

    _waitUntilServerRestart: function (restConfig, deferred, delay) {
        setTimeout($.proxy(function () {
            this.managementService.testManagementWebServerConfiguration()
                    .done($.proxy(function (config) {
                        if (!Object._equals(restConfig, config)) {
                            this._waitUntilServerRestart(restConfig, deferred, delay * 2);
                        } else {
                            deferred.resolve();
                        }
                    }, this))
                    .fail($.proxy(function (jqXhr) {
                        if (jqXhr.status == 0) {
                            this._waitUntilServerRestart(restConfig, deferred, delay * 2);
                        } else {
                            deferred.resolve();
                        }
                    }, this));
        }, this), delay);
    },

    _loadServerSettings: function () {
        return $.when(this.managementService.getServerParameters(),
                      this.managementService.getManagementServiceConfiguration(),
                      this.managementService.getManagementAccessServiceConfiguration(),
                      this.managementService.getAdministrators(),
                      this.managementService.getDomainsAutoStart(),
                      this.managementService.getDbDrivers(),
                      this.managementService.getWebServiceConfiguration(),
                      this.managementService.getManagementWebServerConfiguration(),
                      this._documentViewerAllowed() ? this.managementService.getDocumentViewerConfiguration() : $.Deferred().resolve(null),
                      this.managementService.getEmailServiceConfiguration(),
                      this.managementService.getRegistrationServiceConfiguration(),
                      this.managementService.getFailoverServiceConfiguration(),
                      this.managementService.getSearchingServiceConfiguration(),
                      this.managementService.getJmxServiceConfiguration(),
                      this.managementService.getKeys())
                .then(function (serverParameters, managementServiceConfig, managementAccessServiceConfig,
                                administrators, domainsAutoStart, dbDrivers, webServiceDescriptor,
                                managementWebServerConfig, documentViewerConfig, emailServiceDescriptor,
                                selfRegistrationServiceDescriptor, failoverServiceConfig, searchingServiceConfig,
                                jmxServiceConfig, keys) {
                          var settings = new jscape.ServerSettings();
                          settings.setServerParameters(serverParameters);
                          settings.setServerManagementServiceConfiguration(managementServiceConfig);
                          settings.setManagementAccessServiceConfiguration(managementAccessServiceConfig);
                          settings.setAdministrators(administrators);
                          settings.setDomainsAutoStart(domainsAutoStart);
                          settings.setDbDrivers(dbDrivers);
                          settings.setWebServiceDescriptor(webServiceDescriptor);
                          settings.setManagementWebServerConfiguration(managementWebServerConfig);
                          settings.setDocumentViewerConfiguration(documentViewerConfig);
                          settings.setEmailServiceDescriptor(emailServiceDescriptor);
                          settings.setSelfRegistrationServiceDescriptor(selfRegistrationServiceDescriptor);
                          settings.setFailoverServiceConfiguration(failoverServiceConfig);
                          settings.setSearchingServiceConfiguration(searchingServiceConfig);
                          settings.setJmxServiceConfiguration(jmxServiceConfig);
                          settings.setKeys(keys);
                          return settings;
                      });
    },

    _storeServerSettings: function () {
        var settings = this.serverSettings;
        return $.when(this.managementService.setManagementServiceConfiguration(settings.getServerManagementServiceConfiguration()),
                      this.managementService.setManagementAccessServiceConfiguration(settings.getManagementAccessServiceConfiguration()),
                      this.managementService.setAdministrators(settings.getAdministrators()),
                      this.managementService.setDomainsAutoStart(settings.getDomainsAutoStart()),
                      this.managementService.setRegistrationServiceConfiguration(settings.getSelfRegistrationServiceDescriptor()),
                      this._documentViewerAllowed() ? this.managementService.setDocumentViewerConfiguration(settings.getDocumentViewerConfiguration()) : $.Deferred().resolve(),
                      this.managementService.setDbDrivers(settings.getDbDrivers()),
                      this.managementService.setEmailServiceConfiguration(settings.getEmailServiceDescriptor()),
                      this.managementService.setFailoverServiceConfiguration(settings.getFailoverServiceConfiguration()),
                      this.managementService.setSearchingServiceConfiguration(settings.getSearchingServiceConfiguration()),
                      this.managementService.setJmxServiceConfiguration(settings.getJmxServiceConfiguration()));
    },

    _documentViewerAllowed: function() {
        if (this.docViewerAllowed == null) {
            this.docViewerAllowed = this.managementService.testDocumentViewerAllowed() === "true";
        }
        return this.docViewerAllowed;
    },

    _confirmServerRestart: function() {
        return jscape.ConfirmDialog.confirm(this.resources.getString("WEB_SERVICE_CONFIRM_RESTART_TITLE"),
                                            this.resources.getString("WEB_SERVICE_CONFIRM_RESTART_MESSAGE"));
    },

    _redirect: function() {
        var config = this.serverSettings.getManagementWebServerConfiguration();
        var secure = /^https/i.test(window.location.protocol) && config.httpsEnabled;
        var host = secure ? config.httpsHost : config.httpHost;
        if ("0.0.0.0" === host) {
            window.location = "//" + window.location.host;
        } else {
            var port = secure ? config.httpsPort : config.httpPort;
            window.location = "//" + host + ((!secure && port == 80) || (secure && port == 443) ? "" : ":" + port);
        }
    }
});

/** @class */
jscape.settings.ServerSettingsView = Class.extend(
    /** @lends {jscape.settings.ServerSettingsView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.throbber = $("#throbber");
        this.panel = $("#viewpane");
        this.panel.layout({fit: true, border: false}).css({visibility: "hidden"});

        this.tabs = $("#server-settings[role=tablist]");
        this.tabs.tabs({tabPosition: "left", fit: true, headerWidth: 150, border: true});
        this.tabs.find("div[role=tablist]")
                .filter(function() { return $(this).parents("div.dialog").length == 0; })
                .each(function(_, tablist) { $(tablist).tabs({border:false}); });

        this.saveButton = $("a[role=button][name=savesettings]", this.panel);
        this.saveButton.linkbutton({onClick: $.proxy(this._onSubmit, this)});

        this.cancelButton = $("a[role=button][name=cancelsettings]", this.panel);
        this.cancelButton.linkbutton({onClick: $.proxy(this._onCancel, this)});

        $(window).on("beforeunload.settings", $.proxy(this._onLeave, this));
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.ServerSettingsView.LISTENER;
        this.throbber.remove();
        this.throbber = null;
        this.panel.css({visibility: "visible"});
    },

    _onSubmit: function() {
        if (this._valid()) {
            $(window).off("beforeunload.settings");
            this.listener.onSubmit();

            this.saveButton.linkbutton("disable");
            $(document).one("done.settings", $.proxy(function() { this.saveButton.linkbutton("enable"); }, this));
        }
    },

    _onCancel: function() {
        $(window).off("beforeunload.settings");
        this.listener.onCancel(this);
    },

    _onLeave: function(e) {
        var data = {modified: !this._valid()};
        this.listener.onLeave(data);

        var ret = undefined;
        if (data.modified) {
            ret = this.resources.getString("SETTINGS_NOT_SAVED");
            e.returnValue = ret;
        }
        return ret;
    },

    _valid: function() {
        return new jscape.Validator().validateTabs(this.panel);
    }
});
jscape.settings.ServerSettingsView.LISTENER = {
    onSubmit: $.noop,
    onCancel: $.noop,
    onLeave: $.noop
};

/** @class */
jscape.settings.SettingsModule = jscape.ApplicationModule.extend(
    /** @lends {jscape.settings.SettingsModule.prototype} */
{
    serverSettings: null,
    managementService: null,

    /** @override */
    start: function(listener, managementService, serverSettings) {
        this.listener = listener || jscape.settings.SettingsModule.LISTENER;
        this.managementService = managementService;
        this.serverSettings = serverSettings;

        this._initConfiguration();
        this._setupView();

        this.listener.onStarted(this);
    },

    cancelChanges: function() { this.listener.onChangesCancelled() },
    _initConfiguration: function() {},
    _setupView: function() {}
});
jscape.settings.SettingsModule.LISTENER = $.extend({
    onChangesCancelled: $.noop
}, jscape.ApplicationModule.LISTENER);