jscape.settings = jscape.settings || {};

/** @class */
jscape.WebServiceDescriptor = Class.extend(
    /** @lends {jscape.WebServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d) {
        this.configuration = a || null;
        this.resources = b || [];
        this.logoData = (c || "").replace(/[\r\n]/g, "");
        this.as2ServiceConfiguration = d || null;
    }
});
jscape.WebServiceDescriptor.fromJSON = function(data) {
    return new jscape.WebServiceDescriptor(jscape.WebConfiguration.fromJSON(data.configuration),
                                           $.map(data.resources || [], jscape.WebResources.fromJSON),
                                           data.logoData,
                                           jscape.AS2Configuration.fromJSON(data.as2ServiceConfiguration));
};
/** @class */
jscape.WebConfiguration = Class.extend(
    /** @lends {jscape.WebConfiguration.prototype} */
{
    /** @const */
    HTML_UI: 1,
    APPLET_UI: 2,
    DEFAULT_UI: this.HTML_UI | this.APPLET_UI,
    DEFAULT_UI_VALUE: this.HTML_UI,
    DEFAULT_PLAIN_ADDRESS: {host: "127.0.0.1", port: 80},
    DEFAULT_SECURE_ADDRESS: {host: "127.0.0.1", port: 443},
    DEFAULT_SESSION_TTL: 30 * 60 * 1000,

    /** @constructor */
    initialize: function(a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p, q, r, s, t, u) {
        this.httpEnabled = a || false;
        this.httpAddress = b || this.DEFAULT_PLAIN_ADDRESS.host;
        this.httpPort = parseInt(c) || this.DEFAULT_PLAIN_ADDRESS.port;
        this.httpsEnabled = d || false;
        this.httpsAddress = e|| this.DEFAULT_SECURE_ADDRESS.host;
        this.httpsPort = parseInt(f) || this.DEFAULT_SECURE_ADDRESS.port;
        this.keyAlias = g || "";
        this.sessionTimeout = h || this.DEFAULT_SESSION_TTL;
        this.excludedCiphers = i || null;
        this.lostPasswordEnabled = j != undefined ? j : true;
        this.redirectHttp = k || false;
        this.domainVisible = l != undefined ? l : true;
        this.domainDropdown = m || false;
        this.defaultDomain = n || "";
        this.clientCertificateRequired = o || false;
        this.serverName = p || "";
        this.includeServerPort = q != undefined ? q : true;
        this.userInterface = r || this.DEFAULT_UI;
        this.defaultUserInterface = s || this.DEFAULT_UI_VALUE;
        this.sslRenegotiationAllowed = t != undefined ? t : true;
        this.captchaRequired = u || false;
    }
});
jscape.WebConfiguration.fromJSON = function(data) {
    return data ? $.extend(new jscape.WebConfiguration(), data, {defaultUserInterface: data.defaultUserInterface || jscape.WebConfiguration.prototype.DEFAULT_UI_VALUE}) : null;
};

/** @class */
jscape.ManagementWebServerConfiguration = Class.extend(
    /** @lends {jscape.ManagementWebServerConfiguration.prototype} */
{
    /** @const */
    DEFAULT_SESSION_TTL: 30 * 60 * 1000,
    DEFAULT_PLAIN_ADDRESS: {host: "127.0.0.1", port: 11880},
    DEFAULT_SECURE_ADDRESS: {host: "127.0.0.1", port: 11443},

    /** @constructor */
    initialize: function(a, b, c, d, e, f, g, h, i, j, k) {
        this.httpHost = a || this.DEFAULT_PLAIN_ADDRESS.host;
        this.httpPort = parseInt(b) || this.DEFAULT_PLAIN_ADDRESS.port;
        this.httpEnabled = c || false;
        this.httpsHost = d || this.DEFAULT_SECURE_ADDRESS.host;
        this.httpsPort = parseInt(e) || this.DEFAULT_SECURE_ADDRESS.port;
        this.httpsEnabled = f || false;
        this.httpsKeyAlias = g || "";
        this.excludedCiphers = h || null;
        this.clientCertificateRequired = i || false;
        this.sslRenegotiationAllowed = j || false;
        this.sessionTimeout = parseInt(k) || this.DEFAULT_SESSION_TTL;
    }
});
jscape.ManagementWebServerConfiguration.fromJSON = function(data) {
    return data ? $.extend(new jscape.ManagementWebServerConfiguration(), data) : null;
};

/** @class */
jscape.WebUIType = Class.extend(
    /** @lends {jscape.WebUIType.prototype} */
{
    /** @constructor */
    initialize: function(code, defaultCode, value) {
        this.code = code;
        this.defaultCode = defaultCode;
        this.value = value;
    }
});
/** @const */
jscape.WebUIType.HTML = new jscape.WebUIType(1, null, "HTML");
jscape.WebUIType.APPLET = new jscape.WebUIType(2, null, "APPLET");
jscape.WebUIType.HTML_APPLET = new jscape.WebUIType(3, 1, "HTML_APPLET");
jscape.WebUIType.APPLET_HTML = new jscape.WebUIType(3, 2, "APPLET_HTML");
jscape.WebUIType.typeFor = function(code, defaultCode) {
    if (jscape.WebUIType.HTML.code == code) {
        return jscape.WebUIType.HTML;
    } else if (jscape.WebUIType.APPLET.code == code) {
        return jscape.WebUIType.APPLET;
    } else if (jscape.WebUIType.APPLET_HTML.defaultCode == defaultCode) {
        return jscape.WebUIType.APPLET_HTML;
    } else {
        return jscape.WebUIType.HTML_APPLET;
    }
};
jscape.WebUIType.valueOf = function(value) {
    if (jscape.WebUIType.HTML.value == value) {
        return jscape.WebUIType.HTML;
    } else if (jscape.WebUIType.APPLET.value == value) {
        return jscape.WebUIType.APPLET;
    } else if (jscape.WebUIType.APPLET_HTML.value == value) {
        return jscape.WebUIType.APPLET_HTML;
    } else {
        return jscape.WebUIType.HTML_APPLET;
    }
};
/** @class */
jscape.AS2Configuration = Class.extend(
    /** @lends {jscape.AS2Configuration.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e, f, g, h, i, j, k) {
        this.enabled = a || false;
        this.decryptionKeyProvider = b || null;
        this.receiptSigningKeyProvider = c || null;
        this.receiptSignatureAlgorithm = d || null;
        this.messageEncryptionRequired = e || false;
        this.messageSignatureRequired = f || false;
        this.messageWithoutFilenameAllowed = g || false;
        this.overwriteExistingFileAllowed = h || false;
        this.unauthenticatedTransferBinding = i || null;
        this.uploadDirectory = j || "";
        this.receiptText = k || ""
    }
});
jscape.AS2Configuration.fromJSON = function(data) {
    return new jscape.AS2Configuration(data.enabled,
                                       jscape.ServerKeyProvider.fromJSON(data.decryptionKeyProvider),
                                       jscape.ServerKeyProvider.fromJSON(data.receiptSigningKeyProvider),
                                       jscape.SignatureAlgorithm.forName(data.receiptSignatureAlgorithm),
                                       data.messageEncryptionRequired,
                                       data.messageSignatureRequired,
                                       data.messageWithoutFilenameAllowed,
                                       data.overwriteExistingFileAllowed,
                                       data.unauthenticatedTransferBinding || null,
                                       data.uploadDirectory,
                                       data.receiptText);
};
/** @class */
jscape.AS2Configuration.TransferBinding = Class.extend(
    /** @lends {jscape.AS2Configuration.TransferBinding.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.domain = a || "";
        this.username = b || "";
    }
});

/** @class */
jscape.SelfRegistrationServiceDescriptor = Class.extend(
    /** @lends {jscape.SelfRegistrationServiceDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(signupLinkVisible, useEmailAsLogin, captchaRequired) {
        this.signupLinkVisible = signupLinkVisible || false;
        this.useEmailAsLogin = useEmailAsLogin || false;
        this.captchaRequired = captchaRequired || false;
    },

    toJSON: function() {
        return {
            showRegistrationLink: this.signupLinkVisible,
            useEmailAddressAsLogin: this.useEmailAsLogin,
            captchaRequired: this.captchaRequired
        }
    }
});
jscape.SelfRegistrationServiceDescriptor.fromJSON = function(data) {
    return new jscape.SelfRegistrationServiceDescriptor(data.showRegistrationLink, data.useEmailAddressAsLogin, data.captchaRequired);
};

/** @class */
jscape.settings.WebConfigurationController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.WebConfigurationController.prototype} */
{
    descriptor: null,
    signupDescriptor: null,
    webResources: null,
    excludedSslCiphers: null,
    restExcludedSslCiphers: null,

    /** @constructor */
    initialize: function(resources) {
        this._super();
        this.webResources = [];
        this.resources = resources;

        this.webView = new jscape.settings.WebConfigurationView(resources);
        this.restView = new jscape.settings.WebRestConfigurationView(resources);
        this.as2View = new jscape.settings.AS2ConfigurationView(resources);
        this.miscView = new jscape.settings.WebMiscView(resources);
        this.resourcesView = new jscape.settings.WebResourcesView(resources);

        this._initValidators();
    },

    /** @override */
    hasChanged: function () {
        return !Object._equals(this.descriptor, this._createDescriptor())
               || !Object._equals(this.signupDescriptor, this._createSelfRegistrationDescriptor())
               || !Object._equals(this.restConfig, this._createRestConfiguration());
    },

    /** @override */
    submitChanges: function() {
        var descriptor = this._createDescriptor();
        this.serverSettings.setWebServiceDescriptor(descriptor);

        var restConfig = this._createRestConfiguration();
        this.serverSettings.setManagementWebServerConfiguration(restConfig);

        var selfRegistrationDescriptor = this._createSelfRegistrationDescriptor();
        this.serverSettings.setSelfRegistrationServiceDescriptor(selfRegistrationDescriptor);

        this.listener.onChangesSubmitted(this);
    },

    onReceiptSigningKeyChanged: function(keyAlias) {
        if (keyAlias != null) {
            var keys = this.serverSettings.getKeys().getServerKeys();
            for (var i = 0; i < keys.length; i++) {
                var key = keys[i];
                if (key.alias == keyAlias) {
                    var algorithms = jscape.SignatureAlgorithm.algorithmsOf(key.algorithm);
                    this.as2View.setReceiptSignatureAlgorithms(algorithms);
                    this.as2View.setReceiptSignatureAlgorithm(algorithms[0]);
                    break;
                }
            }
        }
    },

    onChangeLogo: function (file) {
        if (file && file.type.match("image.*") && window.FileReader) {
            var reader = new FileReader();
            reader.onload = $.proxy(function (e) { this.resourcesView.setLogoData(e.target.result); }, this);
            reader.readAsDataURL(file);
        } else {
            this.resourcesView.setLogoData(null);
        }
    },

    onAddLanguage: function() {
        var dialog = new jscape.settings.AddLanguageDialog(this.resources);
        dialog.disableLanguages($.map(this.webResources, function(resource) { return resource.lang; }));
        dialog.show({
            onSubmit: $.proxy(function(d) {
                var resources = this._createWebResources(d);
                d.destroy();

                this.webResources.push(resources);
                this.resourcesView.setItems(this.webResources);
                this.resourcesView.selectItem(resources);
            }, this),
            onCancel: function(d) { d.destroy(); }
        });
    },

    onResetLanguages: function() {
        this.webResources = this._copyWebResources();
        this.resourcesView.setItems(this.webResources);
    },

    onChangeWebSslCiphers: function() {
        this._changeSslCiphers(this.excludedSslCiphers).done($.proxy(function(ciphers) {
            this.excludedSslCiphers = ciphers.length != 0 ? ciphers : null;
        }, this));
    },

    onChangeRestSslCiphers: function() {
        this._changeSslCiphers(this.restExcludedSslCiphers).done($.proxy(function(ciphers) {
            this.restExcludedSslCiphers = ciphers.length != 0 ? ciphers : null;
            this.onChangeRestConfiguration();
        }, this));
    },

    onChangeRestConfiguration: function() {
        var modified = !Object._equals(this.restConfig, this._createRestConfiguration());
        this.restView.setNote(modified ? this.resources.getString("WEB_SERVICE_WARNING_RESTART_REQUIRED") : "");
    },

    _changeSslCiphers: function(disabledCiphers) {
        var deferred = $.Deferred();

        var dialog = new jscape.settings.SslCiphersDialog(this.resources);
        dialog.setCiphers(this.serverSettings.getServerParameters().getSslCiphers());
        dialog.setDisabledCiphers(disabledCiphers);
        dialog.show({
            onSubmit: $.proxy(function(d) { var ciphers = d.getDisabledCiphers(); d.destroy(); deferred.resolve(ciphers); }, this),
            onCancel: function(d) { d.destroy(); deferred.reject(); }
        });

        return deferred.promise();
    },

    _initConfiguration: function() {
        this.descriptor = this.serverSettings.getWebServiceDescriptor();
        this.signupDescriptor = this.serverSettings.getSelfRegistrationServiceDescriptor();
        this.restConfig = this.serverSettings.getManagementWebServerConfiguration();
        this.excludedSslCiphers = this.descriptor.configuration.excludedCiphers;
        this.webResources = this._copyWebResources();
    },

    _copyWebResources: function () {
        return $.map(this.descriptor.resources || [], function(resource) { return resource.clone(); });
    },

    _createDescriptor: function() {
        var webConfig = this._createConfiguration();
        var as2Config = this._createAS2Configuration();
        return new jscape.WebServiceDescriptor(webConfig,
                                               this.resourcesView.getItems(),
                                               this.resourcesView.getLogoData(),
                                               as2Config);
    },

    _createConfiguration: function() {
        var uiType = jscape.WebUIType.valueOf(this.miscView.getUserInterface());
        return new jscape.WebConfiguration(this.webView.isHttpEnabled(),
                                           this.webView.getHttpHost(),
                                           this.webView.getHttpPort(),
                                           this.webView.isHttpsEnabled(),
                                           this.webView.getHttpsHost(),
                                           this.webView.getHttpsPort(),
                                           this.webView.getHttpsKey(),
                                           this.miscView.getSessionTimeout(),
                                           this.excludedSslCiphers,
                                           this.miscView.isShowLostPasswordLink(),
                                           this.miscView.isRedirectHttp(),
                                           this.miscView.isDomainVisible(),
                                           this.miscView.isDomainDropdown(),
                                           this.miscView.getDomainDefault(),
                                           this.webView.isClientCertificateRequired(),
                                           this.miscView.getServerName(),
                                           this.miscView.isIncludeServerPort(),
                                           uiType.code,
                                           uiType.defaultCode,
                                           this.webView.isSslRenegotiationAllowed(),
                                           this.miscView.isLoginCaptchaRequired());
    },

    _createRestConfiguration: function() {
        return new jscape.ManagementWebServerConfiguration(this.restView.getHttpHost(),
                                                           this.restView.getHttpPort(),
                                                           this.restView.isHttpEnabled(),
                                                           this.restView.getHttpsHost(),
                                                           this.restView.getHttpsPort(),
                                                           this.restView.isHttpsEnabled(),
                                                           this.restView.getHttpsKey(),
                                                           this.restExcludedSslCiphers,
                                                           this.restView.isClientCertificateRequired(),
                                                           this.restView.isSslRenegotiationAllowed(),
                                                           this.miscView.getSessionTimeout());
    },

    _createSelfRegistrationDescriptor: function() {
        return new jscape.SelfRegistrationServiceDescriptor(this.miscView.isShowRegistrationLink(),
                                                            this.miscView.isUseEmailAsLogin(),
                                                            this.miscView.isSignupCaptchaRequired());
    },

    _createAS2Configuration: function() {
        return new jscape.AS2Configuration(this.as2View.getEnabled(),
                                           this.as2View.getDecryptionKey() != null ? new jscape.ServerKeyProvider(this.as2View.getDecryptionKey()) : null,
                                           this.as2View.getReceiptSigningKey() != null ? new jscape.ServerKeyProvider(this.as2View.getReceiptSigningKey()) : null,
                                           jscape.SignatureAlgorithm.forName(this.as2View.getReceiptSignatureAlgorithm()),
                                           this.as2View.getMessageEncryptionRequired(),
                                           this.as2View.getMessageSignatureRequired(),
                                           this.as2View.getMessageWithoutFilenameAllowed(),
                                           this.as2View.getOverwriteExistingFileAllowed(),
                                           this.as2View.isBindUnauthenticatedTransfers() ? new jscape.AS2Configuration.TransferBinding(this.as2View.getBindTransfersDomain(), this.as2View.getBindTransfersUser()) : null,
                                           this.as2View.getUploadDirectory(),
                                           this.as2View.getReceiptText());
    },

    _createWebResources: function(dialog) {
        var defaultResources = $.grep(this.descriptor.resources, function(item) { return item.isDefault(); });
        var properties = defaultResources.length ? $.extend({}, defaultResources[0].properties) : {};
        return new jscape.WebResources(dialog.getLanguage(), dialog.getDisplayName(), properties);
    },

    _setupView: function() {
        var serverAddresses = this.serverSettings.getServerParameters().getInetAddresses();
        var serverKeys = $.map(this.serverSettings.getKeys().getServerKeys(), function(key) { return key.alias; });

        var config = this.descriptor.configuration;
        this.webView.setHttpEnabled(config.httpEnabled);
        this.webView.setHttpHosts(serverAddresses);
        this.webView.setHttpsHosts(serverAddresses);
        this.webView.setHttpHost(config.httpAddress);
        this.webView.setHttpPort(config.httpPort);
        this.webView.setHttpsEnabled(config.httpsEnabled);
        this.webView.setHttpsHost(config.httpsAddress);
        this.webView.setHttpsPort(config.httpsPort);
        this.webView.setHttpsKeys(serverKeys);
        this.webView.setHttpsKey(config.keyAlias);
        this.webView.setClientCertificateRequired(config.clientCertificateRequired);
        this.webView.setSslRenegotiationAllowed(config.sslRenegotiationAllowed);
        this.webView.show({
            onChangeSslCiphers: $.proxy(this.onChangeWebSslCiphers, this)
        });

        var restConfig = this.restConfig;
        this.restView.setHttpHosts(serverAddresses);
        this.restView.setHttpsHosts(serverAddresses);
        this.restView.setHttpHost(restConfig.httpHost);
        this.restView.setHttpPort(restConfig.httpPort);
        this.restView.setHttpEnabled(restConfig.httpEnabled);
        this.restView.setHttpsHost(restConfig.httpsHost);
        this.restView.setHttpsPort(restConfig.httpsPort);
        this.restView.setHttpsEnabled(restConfig.httpsEnabled);
        this.restView.setHttpsKeys(serverKeys);
        this.restView.setHttpsKey(restConfig.httpsKeyAlias);
        this.restView.setClientCertificateRequired(restConfig.clientCertificateRequired);
        this.restView.setSslRenegotiationAllowed(restConfig.sslRenegotiationAllowed);
        this.restView.show({
            onChange: $.proxy(this.onChangeRestConfiguration, this),
            onChangeSslCiphers: $.proxy(this.onChangeRestSslCiphers, this)
        });

        var as2Config = this.descriptor.as2ServiceConfiguration;
        this.as2View.show(this);
        this.as2View.setDecryptionKeys(serverKeys);
        this.as2View.setReceiptSigningKeys(serverKeys);
        this.as2View.setReceiptSignatureAlgorithms(jscape.SignatureAlgorithm.values());

        this.as2View.setEnabled(as2Config.enabled);
        this.as2View.setDecryptionKey(as2Config.decryptionKeyProvider && as2Config.decryptionKeyProvider.keyAlias);
        this.as2View.setReceiptSigningKey(as2Config.receiptSigningKeyProvider && as2Config.receiptSigningKeyProvider.keyAlias);
        this.as2View.setReceiptSignatureAlgorithm(as2Config.receiptSignatureAlgorithm);
        this.as2View.setMessageEncryptionRequired(as2Config.messageEncryptionRequired);
        this.as2View.setMessageSignatureRequired(as2Config.messageSignatureRequired);
        this.as2View.setMessageWithoutFilenameAllowed(as2Config.messageWithoutFilenameAllowed);
        this.as2View.setOverwriteExistingFileAllowed(as2Config.overwriteExistingFileAllowed);
        this.as2View.setBindUnauthenticatedTransfers(as2Config.unauthenticatedTransferBinding != null);
        this.as2View.setBindTransfersDomain(as2Config.unauthenticatedTransferBinding != null ? as2Config.unauthenticatedTransferBinding.domain : "");
        this.as2View.setBindTransfersUser(as2Config.unauthenticatedTransferBinding != null ? as2Config.unauthenticatedTransferBinding.username : "");
        this.as2View.setUploadDirectory(as2Config.uploadDirectory);
        this.as2View.setReceiptText(as2Config.receiptText);

        this.miscView.show(this);
        this.miscView.setServerName(config.serverName);
        this.miscView.setSessionTimeout(config.sessionTimeout);
        this.miscView.setIncludeServerPort(config.includeServerPort);
        this.miscView.setRedirectHttp(config.redirectHttp);
        this.miscView.setUserInterface(jscape.WebUIType.typeFor(config.userInterface, config.defaultUserInterface).value);
        this.miscView.setDomainDefault(config.defaultDomain);
        this.miscView.setDomainVisible(config.domainVisible);
        this.miscView.setDomainDropdown(config.domainDropdown);
        this.miscView.setShowLostPasswordLink(config.lostPasswordEnabled);
        this.miscView.setLoginCaptchaRequired(config.captchaRequired);
        this.miscView.setShowRegistrationLink(this.signupDescriptor.signupLinkVisible);
        this.miscView.setUseEmailAsLogin(this.signupDescriptor.useEmailAsLogin);
        this.miscView.setSignupCaptchaRequired(this.signupDescriptor.captchaRequired);

        this.resourcesView.show(this);
        this.resourcesView.setLogoData(this.descriptor.logoData);
        this.resourcesView.setItems(this.webResources);
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.rule("astwo_uploaddir", $.proxy(this._validatorAS2UploadDir, this));
    },

    _validatorAS2UploadDir: function(value) {
        return value && value.length != 0 && value.charAt(0) == "/";
    }
});

/** @class */
jscape.settings.SslCiphersDialog = Class.extend(
    /** @lends {jscape.settings.SslCiphersDialog.prototype} */
{
    STRONG_CIPHERS: [
        "SSL_RSA_WITH_RC4_128_MD5",
        "SSL_RSA_WITH_RC4_128_SHA",
        "SSL_RSA_WITH_3DES_EDE_CBC_SHA",
        "SSL_RSA_WITH_AES_128_CBC_SHA",
        "SSL_RSA_FIPS_WITH_3DES_EDE_CBC_SHA",
        "SSL_DHE_RSA_WITH_AES_128_CBC_SHA",
        "SSL_DHE_DSS_WITH_AES_128_CBC_SHA",
        "SSL_DHE_RSA_WITH_3DES_EDE_CBC_SHA",
        "SSL_DHE_DSS_WITH_3DES_EDE_CBC_SHA",
        "SSL_DHE_DSS_WITH_RC4_128_SHA",
        "SSL_KRB5_WITH_RC4_128_SHA",
        "SSL_KRB5_WITH_RC4_128_MD5",
        "SSL_KRB5_WITH_3DES_EDE_CBC_SHA",
        "SSL_KRB5_WITH_3DES_EDE_CBC_MD5",
        "TLS_RSA_WITH_AES_128_CBC_SHA",
        "TLS_DHE_RSA_WITH_AES_128_CBC_SHA",
        "TLS_DHE_DSS_WITH_AES_128_CBC_SHA",
        "TLS_KRB5_WITH_RC4_128_SHA",
        "TLS_KRB5_WITH_RC4_128_MD5",
        "TLS_KRB5_WITH_3DES_EDE_CBC_SHA",
        "TLS_KRB5_WITH_3DES_EDE_CBC_MD5"
    ],
    MEDIUM_CIPHERS: [
        "SSL_DHE_RSA_WITH_DES_CBC_SHA",
        "SSL_DHE_DSS_WITH_DES_CBC_SHA",
        "SSL_RSA_WITH_DES_CBC_SHA",
        "SSL_RSA_FIPS_WITH_DES_CBC_SHA",
        "SSL_KRB5_WITH_DES_CBC_SHA",
        "SSL_KRB5_WITH_DES_CBC_MD5",
        "TLS_KRB5_WITH_DES_CBC_SHA",
        "TLS_KRB5_WITH_DES_CBC_MD5"
    ],

    /** @constructor */
    initialize: function(resources) {
        this.MEDIUM_CIPHERS = this.MEDIUM_CIPHERS.concat(this.STRONG_CIPHERS);

        this.dialog = $("#d-ssweb-ciphers").clone().attr("id", "");
        this.dialog.dialog({
            width: 550, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=ciphersfields]", this.dialog);
        this.ciphers = $("#ssweb-ciphers-list", this.fieldset).css({"width": "330px", "height": "150px"});

        this.deselectAllButton = $("a[role=button][name=deselectall]", this.fieldset);
        this.deselectAllButton.linkbutton({onClick: $.proxy(function() {
            this.ciphers.find("input:checked").prop("checked", false);
        }, this)});

        this.selectAllButton = $("a[role=button][name=selectall]", this.fieldset);
        this.selectAllButton.linkbutton({onClick: $.proxy(function() {
            this.ciphers.find("input:not(:checked)").prop("checked", true);
        }, this)});

        this.selectMediumButton = $("a[role=button][name=selectmedium]", this.fieldset);
        this.selectMediumButton.linkbutton({onClick: $.proxy(function() {
            this._applyFilter(this.MEDIUM_CIPHERS);
        }, this)});

        this.selectStrongButton = $("a[role=button][name=selectstrong]", this.fieldset);
        this.selectStrongButton.linkbutton({onClick: $.proxy(function() {
            this._applyFilter(this.STRONG_CIPHERS);
        }, this)});
    },

    setCiphers: function(values) {
        this.ciphers.empty();
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            $('<div><label>&nbsp;<input type="checkbox" value="' + value + '" checked="checked" />' + value + '</label></div>').appendTo(this.ciphers);
        }
    },

    getDisabledCiphers: function() {
        var values = [];
        this.ciphers.find("input:not(:checked)").each(function() {
            values.push($(this).val());
        });
        return values;
    },

    setDisabledCiphers: function(values) {
        var parent = this.ciphers;
        $.each(values || [], function(_, value) {
            parent.find("input[value=" + value + "]:checkbox").prop("checked", false);
        });
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _applyFilter: function(filter) {
        this.ciphers.find("input:checkbox").each(function() {
            $(this).prop("checked", $.inArray($(this).val(), filter) != -1);
        });
    },

    _onSubmit: function () {
        this.hide();
        this.listener.onSubmit(this);
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.settings.BaseWebConfigurationView = Class.extend(
    /** @lends {jscape.settings.BaseWebConfigurationView.prototype} */
{
    DEFAULT_PLAIN_ADDRESS: {host: "127.0.0.1", port: 11880},
    DEFAULT_SECURE_ADDRESS: {host: "127.0.0.1", port: 11443},

    listener: null,

    /** @constructor */
    initialize: function (fieldset, resources) {
        this.fieldset = fieldset;

        this.httpEnabled = $("input[name=bindplain]:checkbox", this.fieldset);
        this.httpEnabled.change($.proxy(function() {
            var enabled = this.httpEnabled.is(":checked");
            this.httpHost.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
            this.httpPort.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.httpHost = $("select[name=plainhost]", this.fieldset);
        this.httpHost.combobox({
            width: 170, panelWidth: 255, panelHeight: "auto",
            required: true, missingMessage: resources.getString("WEB_SERVICE_ERROR_BAD_HOST"),
            onLoadSuccess: function(data) { data.length && $(this).combobox("select", data[0].value); }
        });
        this.httpPort = $("input[name=plainport]", this.fieldset);
        this.httpPort.numberspinner({required: true, disabled: true, min: 1, max: 65535, width: 80});

        this.httpsEnabled = $("input[name=bindsecure]:checkbox", this.fieldset);
        this.httpsEnabled.change($.proxy(function() {
            var enabled = this.httpsEnabled.is(":checked");
            this.httpsHost.combobox(enabled ? "enable" : "disable").combobox(enabled ? "enableValidation" : "disableValidation");
            this.httpsPort.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.httpsHost = $("select[name=securehost]", this.fieldset);
        this.httpsHost.combobox({
            width: 170, panelWidth: 255, panelHeight: "auto",
            required: true, missingMessage: resources.getString("WEB_SERVICE_ERROR_BAD_HOST"),
            onLoadSuccess: function(data) { data.length && $(this).combobox("select", data[0].value); }
        });
        this.httpsPort = $("input[name=secureport]", this.fieldset);
        this.httpsPort.numberspinner({required: true, disabled: true, min: 1, max: 65535, width: 80});

        this.privateKey = $("select[name=privatekey]", this.fieldset);
        this.privateKey.combobox({required: true, editable: false, width: 120, panelHeight: "auto", onLoadSuccess: function(data) { data.length && $(this).combobox("select", data[0].value); }});

        this.clientCertificateRequired = $("input[name=certrequired]", this.fieldset);
        this.sslRenegotiation = $("input[name=renegotiationallowed]", this.fieldset);

        this.sslCiphersButton = $("a[role=button][name=sslciphers]", this.fieldset);
        this.sslCiphersButton.linkbutton({onClick: $.proxy(this._onClick$SslCiphersButton, this)});
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.BaseWebConfigurationView.LISTENER;
    },

    isHttpEnabled: function() {
        return this.httpEnabled.is(":checked");
    },

    setHttpEnabled: function(value) {
        this.httpEnabled.prop("checked", !!value).change();
    },

    getHttpHost: function() {
        return this.httpHost.combobox("getValue") || this.httpHost.combobox("getText");
    },

    setHttpHost: function(value) {
        this.httpHost.combobox("setValue", value || this.DEFAULT_PLAIN_ADDRESS.host);
    },

    setHttpHosts: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value} });
        this.httpHost.combobox("loadData", data);
    },

    getHttpPort: function() {
        return parseInt(this.httpPort.numberspinner("getValue"));
    },

    setHttpPort: function(value) {
        this.httpPort.numberspinner("setValue", parseInt(value) || this.DEFAULT_PLAIN_ADDRESS.port);
    },

    isHttpsEnabled: function() {
        return this.httpsEnabled.is(":checked");
    },

    setHttpsEnabled: function(value) {
        this.httpsEnabled.prop("checked", !!value).change();
    },

    getHttpsHost: function() {
        return this.httpsHost.combobox("getValue") || this.httpsHost.combobox("getText");
    },

    setHttpsHost: function(value) {
        this.httpsHost.combobox("setValue", value || this.DEFAULT_SECURE_ADDRESS.host);
    },

    setHttpsHosts: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value} });
        this.httpsHost.combobox("loadData", data);
    },

    getHttpsPort: function() {
        return parseInt(this.httpsPort.numberspinner("getValue"));
    },

    setHttpsPort: function(value) {
        this.httpsPort.numberspinner("setValue", parseInt(value) || this.DEFAULT_SECURE_ADDRESS.port);
    },

    getHttpsKey: function() {
        return this.privateKey.combobox("getValue");
    },

    setHttpsKey: function(value) {
        this.privateKey.combobox("setValue", value);
    },

    setHttpsKeys: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value}; });
        this.privateKey.combobox("loadData", data);
    },

    isClientCertificateRequired: function() {
        return this.clientCertificateRequired.is(":checked");
    },

    setClientCertificateRequired: function(value) {
        this.clientCertificateRequired.prop("checked", !!value);
    },

    isSslRenegotiationAllowed: function() {
        return this.sslRenegotiation.is(":checked");
    },

    setSslRenegotiationAllowed: function(value) {
        this.sslRenegotiation.prop("checked", !!value);
    },

    _onClick$SslCiphersButton: function() {
        this.listener.onChangeSslCiphers();
    }
});
jscape.settings.BaseWebConfigurationView.LISTENER = {
    onChangeSslCiphers: $.noop
};

/** @class */
jscape.settings.WebConfigurationView = jscape.settings.BaseWebConfigurationView.extend(
    /** @lends {jscape.settings.WebConfigurationView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("fieldset[name=webconfigfields]"), resources);
    }
});

/** @class */
jscape.settings.WebRestConfigurationView = jscape.settings.BaseWebConfigurationView.extend(
    /** @lends {jscape.settings.WebRestConfigurationView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("fieldset[name=restconfigfields]"), resources);

        this.note = $(".warning", this.fieldset).hide();

        var handler = $.proxy(this._onChange$Form, this);
        this.httpEnabled.change(handler);
        this.httpHost.combobox({onChange: handler});
        this.httpPort.numberspinner({onChange: handler});
        this.httpsEnabled.change(handler);
        this.httpsHost.combobox({onChange: handler});
        this.httpsPort.numberspinner({onChange: handler});
        this.privateKey.combobox({onChange: handler});
        this.clientCertificateRequired.change(handler);
        this.sslRenegotiation.change(handler);
    },

    setNote: function(text) {
        if (text && text.trim().length) {
            this.note.text(text).show();
        } else {
            this.note.text("").hide();
        }
    },

    _onChange$Form: function() {
        if (this.listener) {
            this.listener.onChange();
        }
    }
});
jscape.settings.WebRestConfigurationView.LISTENER = $.extend({
    onChange: $.noop
}, jscape.settings.BaseWebConfigurationView.LISTENER);

/** @class */
jscape.settings.AS2ConfigurationView = Class.extend(
    /** @lends {jscape.settings.AS2ConfigurationView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=webserveras2fields]");

        this.enabled = $("input[name=enabled]:checkbox", this.fieldset);
        this.enabled.change($.proxy(this._adjustFieldState, this));

        this.useDecryptKey = $("input[name=usedecryptkey]:checkbox", this.fieldset);
        this.useDecryptKey.change($.proxy(function() {
            var enabled = this.useDecryptKey.is(":checked:not(:disabled)");
            this.decryptKey.combobox(enabled ? "enable" : "disable");
        }, this));
        this.decryptKey = $("select[name=decryptkey]", this.fieldset);
        this.decryptKey.combobox({width: 120, editable: false, disabled: true, panelHeight: "auto", onLoadSuccess: function(data) { data.length && $(this).combobox("select", data[0].value); }});

        this.useSignKey = $("input[name=usesignkey]:checkbox", this.fieldset);
        this.useSignKey.change($.proxy(function() {
            var enabled = this.useSignKey.is(":checked:not(:disabled)");
            this.signKey.combobox(enabled ? "enable" : "disable");
            this.receiptSignature.combobox(enabled ? "enable" : "disable");
        }, this));
        this.signKey = $("select[name=receiptsingkey]", this.fieldset);
        this.signKey.combobox({width: 120, editable: false, disabled: true, panelHeight: "auto", onChange: $.proxy(this._onChange$SigningKey, this), onLoadSuccess: function(data) { data.length && $(this).combobox("select", data[0].value); }});
        this.receiptSignature = $("select[name=receiptsignature]", this.fieldset);
        this.receiptSignature.combobox({width: 120, editable: false, disabled: true, panelHeight: "auto", onLoadSuccess: function(data) { data.length && $(this).combobox("select", data[0].value); }});

        this.messageEncryptRequired = $("input[name=encryptrequired]:checkbox", this.fieldset);
        this.messageSignatureRequired = $("input[name=signaturerequired]:checkbox", this.fieldset);
        this.messageWithoutFilenameAllowed = $("input[name=allowmessagewofile]:checkbox", this.fieldset);
        this.overwriteExistingFileAllowed = $("input[name=overwrite]:checkbox", this.fieldset);

        this.bindTransfers = $("input[name=bindtransfers]:checkbox", this.fieldset);
        this.bindTransfers.change($.proxy(function() {
            var enabled = this.bindTransfers.is(":checked:not(:disabled)");
            $.each([this.transferUser, this.transferDomain], function(_, input) {
                input.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
            });
        }, this));
        this.transferUser = $("input[name=transferuser]", this.fieldset);
        this.transferUser.validatebox({required: true, validType: "length[1,50]", missingMessage: resources.getString("AS2_ERROR_BAD_TRANSFER_BINDING"), invalidMessage: resources.getString("AS_ERROR_BAD_TRANAFER_USER")});
        this.transferDomain = $("input[name=transfdomain]", this.fieldset);
        this.transferDomain.validatebox({required: true, validType: "length[1,50]", missingMessage: resources.getString("AS2_ERROR_BAD_TRANSFER_BINDING"), invalidMessage: resources.getString("AS_ERROR_BAD_TRANAFER_DOMAIN")});

        this.uploadDir = $("input[name=uploaddir]", this.fieldset);
        this.uploadDir.validatebox({required: true, validType: "astwo_uploaddir", missingMessage: resources.getString("AS2_ERROR_BAD_UPLOAD_DIR"), invalidMessage: resources.getString("AS2_ERROR_BAD_UPLOAD_DIR")});

        this.receiptText = $("textarea[name=receiptext]", this.fieldset).css({width: "50em", height: "5em"});
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.AS2ConfigurationView.LISTENER;
        this._adjustFieldState();
    },

    getEnabled: function() {
        return this.enabled.is(":checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", !!value).change();
    },

    getDecryptionKey: function() {
        return this.useDecryptKey.is(":checked:not(:disabled)") ? this.decryptKey.combobox("getValue") : null;
    },

    setDecryptionKey: function(value) {
        this.useDecryptKey.prop("checked", !!value).change();
        if (value) {
            this.decryptKey.combobox("setValue", value)
        }
    },

    setDecryptionKeys: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value} });
        this.decryptKey.combobox("loadData", data);
    },

    getReceiptSigningKey: function() {
        return this.useSignKey.is(":checked:not(:disabled)") ? this.signKey.combobox("getValue") : null;
    },

    setReceiptSigningKey: function(value) {
        this.useSignKey.prop("checked", !!value).change();
        if (value) {
            this.signKey.combobox("setValue", value)
        }
    },

    setReceiptSigningKeys: function(values) {
        var data = $.map(values || [], function(value) { return {text: value, value: value} });
        this.signKey.combobox("loadData", data);
    },

    getReceiptSignatureAlgorithm: function() {
        return this.useSignKey.is(":checked:not(:disabled)") ? this.receiptSignature.combobox("getValue") : null;
    },

    setReceiptSignatureAlgorithm: function(value) {
        if (value) {
            this.receiptSignature.combobox("setValue", value.name);
        }
    },

    setReceiptSignatureAlgorithms: function(values) {
        var data = $.map(values || [], $.proxy(function(entry) {
            return {
                text: this.resources.getString("AS2_SIGNATURE_ALGORITHM_TEMPLATE").supplant({0: entry.hashAlgorithm, 1: entry.asymmetricalAlgorithm}),
                value: entry.name,
                algorithm: entry
            };
        }, this));
        this.receiptSignature.combobox("loadData", data);
    },

    getMessageEncryptionRequired: function() {
        return this.messageEncryptRequired.is(":checked");
    },

    setMessageEncryptionRequired: function(value) {
        this.messageEncryptRequired.prop("checked", !!value);
    },

    getMessageSignatureRequired: function() {
        return this.messageSignatureRequired.is(":checked");
    },

    setMessageSignatureRequired: function(value) {
        this.messageSignatureRequired.prop("checked", !!value);
    },

    getMessageWithoutFilenameAllowed: function() {
        return this.messageWithoutFilenameAllowed.is(":checked");
    },

    setMessageWithoutFilenameAllowed: function(value) {
        this.messageWithoutFilenameAllowed.prop("checked", !!value);
    },

    getOverwriteExistingFileAllowed: function() {
        return this.overwriteExistingFileAllowed.is(":checked");
    },

    setOverwriteExistingFileAllowed: function(value) {
        this.overwriteExistingFileAllowed.prop("checked", !!value);
    },

    isBindUnauthenticatedTransfers: function() {
        return this.bindTransfers.is(":checked");
    },

    setBindUnauthenticatedTransfers: function(value) {
        this.bindTransfers.prop("checked", !!value).change();
    },

    getBindTransfersDomain: function() {
        return this.transferDomain.val();
    },

    setBindTransfersDomain: function(value) {
        this.transferDomain.val(value);
    },

    getBindTransfersUser: function() {
        return this.transferUser.val();
    },

    setBindTransfersUser: function(value) {
        this.transferUser.val(value);
    },

    getUploadDirectory: function() {
        return this.uploadDir.val();
    },

    setUploadDirectory: function(value) {
        this.uploadDir.val(value);
    },

    getReceiptText: function() {
        return this.receiptText.val();
    },

    setReceiptText: function(value) {
        this.receiptText.val(value);
    },

    _onChange$SigningKey: function() {
        this.listener.onReceiptSigningKeyChanged(this.signKey.combobox("getValue"));
    },

    _adjustFieldState: function() {
        var enabled = this.getEnabled();
        this.useDecryptKey.prop("disabled", !enabled).change();
        this.useSignKey.prop("disabled", !enabled).change();
        this.messageEncryptRequired.prop("disabled", !enabled);
        this.messageSignatureRequired.prop("disabled", !enabled);
        this.messageWithoutFilenameAllowed.prop("disabled", !enabled);
        this.overwriteExistingFileAllowed.prop("disabled", !enabled);
        this.bindTransfers.prop("disabled", !enabled).change();
        this.uploadDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
        this.receiptText.prop("disabled", !enabled);

        this.fieldset.form("validate");
    }
});
jscape.settings.AS2ConfigurationView.LISTENER = {
    onReceiptSigningKeyChanged: $.noop
};

/** @class */
jscape.settings.WebMiscView = Class.extend(
    /** @lends {jscape.settings.WebMiscView.prototype} */
{
    TIMEOUT_FACTOR: 60 * 1000,
    DEFAULT_SESSION_TIMEOUT: 30,

    /** @constructor */
    initialize: function(resources) {
        this.serverName = $("#sswebmisc-server");
        this.serverName.on("input", $.proxy(this._onChange$ServerName, this)).change($.proxy(this._onChange$ServerName, this));

        this.sessionTimeout = $("#sswebmisc-ttl");
        this.sessionTimeout.numberspinner({min: 1, max: 999});

        this.redirectHttp = $("#sswebmisc-redirecthttp");
        this.includeServerPort = $("#sswebmisc-inclserverport");

        this.userInterface = $("#sswebmisc-ui");
        this.userInterface.combobox({required: true, editable: false, panelHeight: "auto", onLoadSuccess: function(data) { data.length && $(this).combobox("select", data[0].value); }});

        this.domainDefault = $("#sswebmisc-defaultdomain");
        this.domainDropdown = $("#sswebmisc-domaindropdown");

        this.hideDomain = $("#sswebmisc-hidedomain");
        this.hideDomain.change($.proxy(this._onClick$HideDomain, this));

        this.lostPasswordLink = $("#sswebmisc-lostpwd");
        this.loginCaptchaRequired = $("#sswebmisc-logincaptcha");
        this.singupLink = $("#sswebmisc-signuplink");
        this.useEmailAsLogin = $("#sswebmisc-emailaslogin");
        this.signupCaptchaRequired = $("#sswebmisc-signupcaptcha");
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.WebMiscView.LISTENER;
        this._adjustIncludePortState();
        this._adjustDomainDropDownState();
    },

    getServerName: function() {
        return this.serverName.val().trim();
    },

    setServerName: function(value) {
        this.serverName.val(value).change();
    },

    getSessionTimeout: function() {
        return parseInt(this.sessionTimeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setSessionTimeout: function(value) {
        value = parseInt(value);
        this.sessionTimeout.numberspinner("setValue", !isNaN(value) ? value / this.TIMEOUT_FACTOR : this.DEFAULT_SESSION_TIMEOUT);
    },

    isRedirectHttp: function() {
        return this.redirectHttp.is(":checked");
    },

    setRedirectHttp: function(value) {
        this.redirectHttp.prop("checked", value);
    },

    isIncludeServerPort: function() {
        return this.includeServerPort.is(":checked");
    },

    setIncludeServerPort: function(value) {
        this.includeServerPort.prop("checked", value);
    },

    getUserInterface: function() {
        return this.userInterface.combobox("getValue");
    },

    setUserInterface: function(value) {
        this.userInterface.combobox("setValue", value);
    },

    getDomainDefault: function() {
        return this.domainDefault.val();
    },

    setDomainDefault: function(value) {
        this.domainDefault.val(value);
    },

    isDomainVisible: function() {
        return !this.hideDomain.is(":checked");
    },

    setDomainVisible: function(value) {
        this.hideDomain.prop("checked", !value).change();
    },

    isDomainDropdown: function() {
        return this.domainDropdown.is(":checked");
    },

    setDomainDropdown: function(value) {
        this.domainDropdown.prop("checked", !!value);
    },

    isShowLostPasswordLink: function() {
        return this.lostPasswordLink.is(":checked");
    },

    setShowLostPasswordLink: function(value) {
        this.lostPasswordLink.prop("checked", !!value);
    },

    isShowRegistrationLink: function() {
        return this.singupLink.is(":checked");
    },

    setShowRegistrationLink: function(value) {
        this.singupLink.prop("checked", !!value);
    },

    isUseEmailAsLogin: function() {
        return this.useEmailAsLogin.is(":checked");
    },

    setUseEmailAsLogin: function(value) {
        this.useEmailAsLogin.prop("checked", !!value);
    },

    isLoginCaptchaRequired: function() {
        return this.loginCaptchaRequired.is(":checked");
    },

    setLoginCaptchaRequired: function(value) {
        this.loginCaptchaRequired.prop("checked", !!value);
    },

    isSignupCaptchaRequired: function() {
        return this.signupCaptchaRequired.is(":checked");
    },

    setSignupCaptchaRequired: function(value) {
        this.signupCaptchaRequired.prop("checked", !!value);
    },

    _onChange$ServerName: function() {
        this._adjustIncludePortState();
    },

    _onClick$HideDomain: function() {
        this._adjustDomainDropDownState();
    },

    _adjustIncludePortState: function() {
        this.includeServerPort.prop("disabled", this.getServerName().length == 0);
    },

    _adjustDomainDropDownState: function() {
        this.domainDropdown.prop("disabled", !this.isDomainVisible());
    }
});
jscape.settings.WebMiscView.LISTENER = {};

/** @class */
jscape.settings.AddLanguageDialog = Class.extend(
    /** @lends {jscape.settings.AddLanguageDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-sswres-add").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.language = $("select[name=language]", this.dialog);
    },

    getLanguage: function() {
        return this.language.val();
    },

    getDisplayName: function() {
        return this.language.find("option:selected").html();
    },

    disableLanguages: function(values) {
        for (var i = 0; i < values.length; i++) {
            this.language.find("option[value=" + values[i] + "]").prop("disabled", true);
        }
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.language.val(this.language.find("option:not(:disabled):first").val()).focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        this.hide();
        this.listener.onSubmit(this);
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.settings.WebResourcesView = Class.extend(
    /** @lends {jscape.settings.WebResourcesView.prototype} */
{
    /** @const */
    DATA_URL: {pattern: /data:image\/.*;base64,(.+)/, prefix: "data:image/png;base64,"},

    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=webresourcesfields]");

        this.logo = $("div.logo-preview>img", this.fieldset);
        this.logoFile = $("input[name=logo]", this.fieldset).css({width: 0, height: 0, opacity: 0, position: "absolute", left: "-10000px"});
        this.logoFile.change($.proxy(this._onChange$Logo, this));
        this.logoButton = $("a[role=button][name=changelogo]", this.fieldset);
        this.logoButton.linkbutton({onClick: $.proxy(function() { this.logoFile.click(); }, this)});
        this.logo.parent().click($.proxy(function() { this.logoFile.click(); }, this));

        this.language = $("select[name=language]", this.fieldset);
        this.language.combobox({
            required: true, editable: false, width: 170, panelHeight: "auto",
            valueField: "lang", textField: "displayName", selectOnNavigation: false,
            onChange: $.proxy(this._onChange$Language, this),
            onLoadSuccess: function(data) { data.length && $(this).combobox("select", data[0].lang).combobox("options").onChange.call(this); }
        });

        this.data = $("table[role=grid][aria-label=resources]", this.fieldset);
        this.data.propertygrid({
            showGroup: false, scrollbarSize: 0, height: 270,
            sortName: "name", sortOrder: "asc",
            onAfterEdit: $.proxy(this._commitChanges, this)
        });

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.resetButton = $("a[role=button][name=reset]", this.fieldset);
        this.resetButton.linkbutton({onClick: $.proxy(this._onClick$ResetButton, this)});
    },

    getLogoData: function () {
        return (this.logo.attr("src") || "").replace(this.DATA_URL.pattern, "$1");
    },

    setLogoData: function (data) {
        this.logo[0].src = this.DATA_URL.pattern.test(data) ? data : this.DATA_URL.prefix + (data || "");
    },

    getItems: function() {
        var data = this.language.combobox("getData");
        return $.map(data, function(item) { if (item.isDefault()) { item.displayName = ""; } return item; });
    },

    setItems: function (values) {
        var data = $.map(values || [], function(item) { if (item.isDefault()) { item.displayName = "default"; } return item; });
        this.language.combobox("loadData", data);
    },

    selectItem: function (value) {
        this.language.combobox("select", value.lang);
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.WebResourcesView.LISTENER;
    },

    _onChange$Logo: function (e) {
        var files = e.target.files;
        if (files && files.length) {
            this.listener.onChangeLogo(files[0]);
        }
        return false;
    },

    _onChange$Language: function() {
        var language = this.language.combobox("getValue");
        var properties = this._propertiesFor(language);
        if (properties != null) {
            this._selectProperties(properties);
        }
        return false;
    },

    _onClick$AddButton: function() {
        this.listener.onAddLanguage();
    },

    _onClick$ResetButton: function() {
        this.listener.onResetLanguages();
    },

    _commitChanges: function(_, data) {
        var language = this.language.combobox("getValue");
        var properties = this._propertiesFor(language);
        if (properties != null) {
            properties[data.name] = data.value;
        }
    },

    _selectProperties: function(properties) {
        var data = $.map(properties, function(value, key) { return {name: key, value: value, editor: "text"}});
        this.data.propertygrid("loading");
        this.data.propertygrid("loadData", data);
        this.data.propertygrid("loaded");
    },

    _propertiesFor: function (language) {
        var data = this.language.combobox("getData");
        for (var i = 0; i < data.length; i++) {
            var resources = data[i];
            if (resources.lang == language) {
                return resources.properties;
            }
        }
        return null;
    }
});
jscape.settings.WebResourcesView.LISTENER = {
    onChangeLogo: $.noop,
    onAddLanguage: $.noop,
    onResetLanguages: $.noop
};
