jscape.settings = jscape.settings || {};

/** @class */
jscape.DocumentViewerConfiguration = Class.extend(
    /** @lends {jscape.DocumentViewerConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c, d, e) {
        this.enabled = a || false;
        this.officeDirectory = b || "";
        this.swfToolDirectory = c || "";
        this.outputDirectory = d || "";
        this.threads = parseInt(e) || 10;
    }
});
jscape.DocumentViewerConfiguration.fromJSON = function(data) {
    return data ? $.extend(new jscape.DocumentViewerConfiguration(), data) : null;
};

/** @class */
jscape.settings.DocumentViewerController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.DocumentViewerController.prototype} */
{
    config: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.DocumentViewerView(resources);

        this._initValidators(resources);
    },

    /** @override */
    hasChanged: function () {
        return !Object._equals(this.config, this._createConfiguration());
    },

    /** @override */
    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setDocumentViewerConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    onSelectDirectory: function() {
        var deferred = $.Deferred();
        try {
            new SelectPathController($.proxy(this.managementService.listFileSystem, this.managementService),
                                     $.proxy(this.managementService.createFileSystemDir, this.managementService))
                    .selectDirectory(function (entry) { deferred.resolve(entry.path); });
        } catch (e) {
            deferred.reject();
        }
        return deferred.promise();
    },

    _initConfiguration: function() {
        this.config = this.serverSettings.getDocumentViewerConfiguration();
    },

    _createConfiguration: function() {
        return new jscape.DocumentViewerConfiguration(this.view.getEnabled(),
                                                      this.view.getOfficeDir(),
                                                      this.view.getSwfToolDir(),
                                                      this.view.getOutputDir(),
                                                      this.view.getThreadCount());
    },

    _setupView: function() {
        if (this.config) {
            this.view.setEnabled(this.config.enabled);
            this.view.setOfficeDir(this.config.officeDirectory);
            this.view.setSwfToolDir(this.config.swfToolDirectory);
            this.view.setOutputDir(this.config.outputDirectory);
            this.view.setThreadCount(this.config.threads);
        }
        this.view.show(this);
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("docviewer_nonempty_office_dir", resources.getString("DOC_VIEWER_ERROR_BAD_OFFICE_DIR"));
        v.nonEmptyRule("docviewer_nonempty_output_dir", resources.getString("DOC_VIEWER_ERROR_BAD_OUTPUT_DIR"));
        v.rule("docviewer_allowed", $.proxy(this._documentViewerAllowed, this));
    },

    _documentViewerAllowed: function() {
        var ret = this.managementService.testDocumentViewerAllowed();
        return ret && "true" === ret;
    }
});

/** @class */
jscape.settings.DocumentViewerView = Class.extend(
    /** @lends {jscape.settings.DocumentViewerView.prototype} */
{
    DEFAULT_THREAD_COUNT: 10,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset#sswdv-fields");
        this.fieldset = $("fieldset[name=docviewersettingsfields]");

        this.enabled = $("input[name=enabled]:checkbox", this.fieldset);
        this.enabled.click($.proxy(this._onClick$Enabled, this));
        this.enabled.change($.proxy(this._adjustFieldState, this));

        this.officeDir = $("input[name=officedir]", this.fieldset);
        this.officeDir.validatebox({required: true, validType: "docviewer_nonempty_office_dir", tipPosition: "bottom", missingMessage: resources.getString("DOC_VIEWER_ERROR_BAD_OFFICE_DIR")});
        this.browseOfficeDir = $("a[role=button][name=browseofficedir]", this.fieldset);
        this.browseOfficeDir.linkbutton({onClick: $.proxy(this._onClick$BrowseOfficeDir, this)});

        this.swfToolDir = $("input[name=swftooldir]", this.fieldset);
        this.browseSwfToolDir = $("a[role=button][name=browseswftooldir]", this.fieldset);
        this.browseSwfToolDir.linkbutton({onClick: $.proxy(this._onClick$BrowseSwfToolDir, this)});

        this.outputDir = $("input[name=outdir]", this.fieldset);
        this.outputDir.validatebox({required: true, validType: "docviewer_nonempty_output_dir", tipPosition: "bottom", missingMessage: resources.getString("DOC_VIEWER_ERROR_BAD_OUTPUT_DIR")});
        this.browseOutputDir = $("a[role=button][name=browseoutdir]", this.fieldset);
        this.browseOutputDir.linkbutton({onClick: $.proxy(this._onClick$BrowseOutputDir, this)});

        this.threadCount = $("input[name=threads]", this.fieldset);
        this.threadCount.numberspinner({required: true, min: 1, max: 100, width: 80});
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.DocumentViewerView.LISTENER;
        this._adjustFieldState();
    },

    getEnabled: function() {
        return this.enabled.is(":checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", !!value).change();
    },

    getOfficeDir: function() {
        return this.officeDir.val();
    },

    setOfficeDir: function(value) {
        this.officeDir.val(value).validatebox("validate");
    },

    getSwfToolDir: function() {
        return this.swfToolDir.val();
    },

    setSwfToolDir: function(value) {
        this.swfToolDir.val(value);
    },

    getOutputDir: function() {
        return this.outputDir.val();
    },

    setOutputDir: function(value) {
        this.outputDir.val(value).validatebox("validate");
    },

    getThreadCount: function() {
        return parseInt(this.threadCount.numberspinner("getValue"));
    },

    setThreadCount: function(value) {
        this.threadCount.numberspinner("setValue", parseInt(value) || this.DEFAULT_THREAD_COUNT);
    },

    _onClick$Enabled: function(e) {
        if (this.getEnabled()) {
            var allowed = new jscape.Validator().isSatisfied("docviewer_allowed");
            if (!allowed) {
                e.preventDefault();
                e.stopImmediatePropagation();
                this.enabled.prop("checked", false).change();
                $.messager.alert(this.resources.getString("DIALOG_TITLE_ERROR"),
                                 this.resources.getString("DOC_VIEWER_ERROR_NOT_ALLOWED"),
                                 "error");
            }
        }
    },

    _onClick$BrowseOfficeDir: function() {
        this.listener.onSelectDirectory()
                .done($.proxy(function(path) { this.setOfficeDir(path); }, this));
    },

    _onClick$BrowseSwfToolDir: function() {
        this.listener.onSelectDirectory()
                .done($.proxy(function(path) { this.setSwfToolDir(path); }, this));
    },

    _onClick$BrowseOutputDir: function() {
        this.listener.onSelectDirectory()
                .done($.proxy(function(path) { this.setOutputDir(path); }, this));
    },

    _adjustFieldState: function() {
        var enabled = this.getEnabled();
        this.officeDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
        this.browseOfficeDir.linkbutton(enabled ? "enable" : "disable");
        this.swfToolDir.prop("disabled", !enabled);
        this.browseSwfToolDir.linkbutton(enabled ? "enable" : "disable");
        this.outputDir.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
        this.browseOutputDir.linkbutton(enabled ? "enable" : "disable");
        this.threadCount.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");

        this.fieldset.form("validate");
    }
});
jscape.settings.DocumentViewerView.LISTENER = {
    onSelectDirectory: $.noop
};